@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -117,7 +116,7 @@ static SHOW_VAR show_var_filter_rules_decompile[] = {
 /*
   STRING_WITH_LEN
 */
-#include <string_with_len.h>
+#include <m_string.h>
 
 static bool inited = false;
 static int opened = 0;
@@ -353,11 +352,10 @@ static int log_filter_xlate_by_opcode(uint opcode, uint flags) {
   @param str      NTBS to append to that buffer
 */
 static void log_filter_append(char *out_buf, size_t out_siz, const char *str) {
-  const size_t out_used = log_bs->length(out_buf);
-  const size_t out_left = out_siz - out_used;
+  size_t out_used = log_bs->length(out_buf);
+  size_t out_left = out_siz - out_used;
   char *out_writepos = out_buf + out_used;
-  const size_t out_needed =
-      log_bs->substitute(out_writepos, out_left, "%s", str);
+  size_t out_needed = log_bs->substitute(out_writepos, out_left, "%s", str);
 
   if (out_needed >= out_left)    /* buffer exhausted. '\0' terminate */
     out_buf[out_siz - 1] = '\0'; /* purecov: inspected */
@@ -374,7 +372,7 @@ static void log_filter_append(char *out_buf, size_t out_siz, const char *str) {
 static void log_filter_append_item_value(char *out_buf, size_t out_siz,
                                          log_item *li) {
   size_t len = log_bs->length(out_buf);  // used bytes
-  const size_t out_left = out_siz - len;
+  size_t out_left = out_siz - len;
   char *out_writepos = out_buf + len;
 
   if (li->item_class == LOG_FLOAT)
@@ -448,12 +446,12 @@ static void log_filter_append_item_value(char *out_buf, size_t out_siz,
 static log_filter_result log_filter_rule_dump(log_filter_rule *rule,
                                               log_filter_result state,
                                               char *out_buf, size_t out_size) {
-  const log_filter_result ret = LOG_FILTER_LANGUAGE_OK;
+  log_filter_result ret = LOG_FILTER_LANGUAGE_OK;
   int cond;
   int verb;
   const log_filter_xlate_key *token;
 
-  assert(out_buf != nullptr);
+  DBUG_ASSERT(out_buf != nullptr);
 
   out_buf[0] = '\0';
 
@@ -719,7 +717,7 @@ static int log_filter_get_token(const char **inp_readpos, const char **token,
 */
 static int log_filter_make_field(const char **name, const size_t *len,
                                  log_item *li) {
-  const int wellknown = log_bi->wellknown_by_name(*name, *len);
+  int wellknown = log_bi->wellknown_by_name(*name, *len);
   log_item_type item_type;
   char *key = nullptr;
 
@@ -780,7 +778,7 @@ static set_arg_result log_filter_set_arg(const char **token, const size_t *len,
   bool is_symbol = false;
 
   // sanity check
-  assert(!(li->alloc & LOG_ITEM_FREE_VALUE));
+  DBUG_ASSERT(!(li->alloc & LOG_ITEM_FREE_VALUE));
   if (li->alloc & LOG_ITEM_FREE_VALUE) {
     log_bs->free(const_cast<char *>(li->data.data_string.str));
     li->data.data_string.str = nullptr;
@@ -870,7 +868,7 @@ static set_arg_result log_filter_set_arg(const char **token, const size_t *len,
     if ((val = log_bs->strndup(*token + 1, val_len)) == nullptr)
       return SET_ARG_OOM; /* purecov: inspected */
 
-    assert(val_len > 0);
+    DBUG_ASSERT(val_len > 0);
     val[--val_len] = '\0';  // cut trailing quotation mark
 
     li->data.data_string.str = val;
@@ -1262,7 +1260,7 @@ static int log_filter_dragnet_set(log_filter_ruleset *ruleset,
       if ((log_filter_xlate_keys[c].item == LOG_FILTER_ITEM_DEL) &&
           (*inp_readpos == '.'))  // end of statement reached, unset had no arg.
       {
-        int local_c;
+        int c;
         char *n;
         size_t key_len;
 
@@ -1274,11 +1272,11 @@ static int log_filter_dragnet_set(log_filter_ruleset *ruleset,
         }
 
         // see whether it's a well-known type
-        local_c = log_bi->wellknown_by_name(
+        c = log_bi->wellknown_by_name(
             delete_item->key, (key_len = log_bs->length(rule->match.key)));
 
         // for ad-hoc names, copy the key; otherwise use well-known record.
-        if (local_c != LOG_ITEM_TYPE_NOT_FOUND)
+        if (c != LOG_ITEM_TYPE_NOT_FOUND)
           n = nullptr;
         else if ((n = log_bs->strndup(rule->match.key, key_len)) == nullptr) {
           rr = -2; /* purecov: inspected */
@@ -1323,7 +1321,7 @@ static int log_filter_dragnet_set(log_filter_ruleset *ruleset,
       }
 
       {
-        const int auxval_success =
+        int auxval_success =
             log_filter_set_arg(&token, &len, &rule->aux, state);
 
         if ((rule->verb == LOG_FILTER_THROTTLE) &&
@@ -1390,7 +1388,7 @@ static int log_filter_dragnet_set(log_filter_ruleset *ruleset,
       flow_new = LOG_FILTER_WORD_NONE;
 
       {  // add relative jumps to the end of all blocks in THEN/ELSEIF
-        const int flow_last = tmp_filter_rules->count;
+        int flow_last = tmp_filter_rules->count;
         for (c = flow_first; c <= flow_last; c++) {
           if (tmp_filter_rules->rule[c].jump != 0)
             tmp_filter_rules->rule[c].jump = flow_last - c + 1;
@@ -1440,7 +1438,8 @@ done:
   @retval false    value OK, go ahead and update system variable (from "save")
   @retval true     value rejected, do not update variable
 */
-static int check_var_filter_rules(MYSQL_THD thd, SYS_VAR *self [[maybe_unused]],
+static int check_var_filter_rules(MYSQL_THD thd,
+                                  SYS_VAR *self MY_ATTRIBUTE((unused)),
                                   void *save, struct st_mysql_value *value) {
   int ret;
   log_filter_ruleset *log_filter_temp_rules;
@@ -1456,7 +1455,7 @@ static int check_var_filter_rules(MYSQL_THD thd, SYS_VAR *self [[maybe_unused]],
 
   if (proposed_rules == nullptr) return true;
 
-  assert(proposed_rules[value_len] == '\0');
+  DBUG_ASSERT(proposed_rules[value_len] == '\0');
 
   log_filter_temp_rules = log_bf->filter_ruleset_new(&rule_tag_dragnet, 0);
 
@@ -1508,8 +1507,8 @@ static int check_var_filter_rules(MYSQL_THD thd, SYS_VAR *self [[maybe_unused]],
   @param  var_ptr  where to save the resulting (char *) value
   @param  save     pointer to the new value (from check function)
 */
-static void update_var_filter_rules(MYSQL_THD thd [[maybe_unused]],
-                                    SYS_VAR *self [[maybe_unused]],
+static void update_var_filter_rules(MYSQL_THD thd MY_ATTRIBUTE((unused)),
+                                    SYS_VAR *self MY_ATTRIBUTE((unused)),
                                     void *var_ptr, const void *save) {
   const char *state = nullptr;
   const char *new_val = *(static_cast<const char **>(const_cast<void *>(save)));
@@ -1543,7 +1542,7 @@ static void update_var_filter_rules(MYSQL_THD thd [[maybe_unused]],
   @returns          int                 Number of matched rules
 */
 DEFINE_METHOD(int, log_service_imp::run,
-              (void *instance [[maybe_unused]], log_line *ll)) {
+              (void *instance MY_ATTRIBUTE((unused)), log_line *ll)) {
   return log_bf->filter_run(log_filter_dragnet_rules, ll);
 }
 
@@ -1562,11 +1561,11 @@ DEFINE_METHOD(int, log_service_imp::run,
                      the server/logging framework. It must be released
                      on close.
 
-  @returns  LOG_SERVICE_SUCCESS        Success, returned handle is valid
+  @returns  LOG_SERVICE_SUCCESS        Success, returned hande is valid
   @returns  otherwise                  A new instance could not be created
 */
 DEFINE_METHOD(log_service_error, log_service_imp::open,
-              (log_line * ll [[maybe_unused]], void **instance)) {
+              (log_line * ll MY_ATTRIBUTE((unused)), void **instance)) {
   if (instance == nullptr) return LOG_SERVICE_INVALID_ARGUMENT;
 
   *instance = nullptr;
@@ -1611,7 +1610,7 @@ DEFINE_METHOD(log_service_error, log_service_imp::close, (void **instance)) {
   @returns  LOG_SERVICE_NOTHING_DONE       no work was done
 */
 DEFINE_METHOD(log_service_error, log_service_imp::flush,
-              (void **instance [[maybe_unused]])) {
+              (void **instance MY_ATTRIBUTE((unused)))) {
   return LOG_SERVICE_NOTHING_DONE;
 }
 
@@ -1682,8 +1681,7 @@ mysql_service_status_t log_filter_init() {
             log_bf->filter_ruleset_new(&rule_tag_dragnet, 0)) == nullptr) ||
       mysql_service_component_sys_variable_register->register_variable(
           LOG_FILTER_LANGUAGE_NAME, LOG_FILTER_SYSVAR_NAME,
-          PLUGIN_VAR_STR | PLUGIN_VAR_MEMALLOC |
-              PLUGIN_VAR_PERSIST_AS_READ_ONLY,
+          PLUGIN_VAR_STR | PLUGIN_VAR_MEMALLOC,
           "Error log filter rules (for the dragnet filter "
           "configuration language)",
           check_var_filter_rules, update_var_filter_rules,
@@ -1738,7 +1736,7 @@ mysql_service_status_t log_filter_init() {
   }
 
 success:
-  assert(var_value[var_len] == '\0');
+  DBUG_ASSERT(var_value[var_len] == '\0');
   delete[] var_value;
   return false;
 }
