@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -44,7 +44,7 @@ extern native_mutex_t LOCK_continent_records_array;
 
 /* A structure to denote a single row of the table. */
 struct {
-  char name[20];
+  char name[CONTINENT_NAME_LEN];
   unsigned int name_length;
 
   /* If there is a value in this row */
@@ -95,16 +95,17 @@ class Continent_index {
 class Continent_index_by_name : public Continent_index {
  public:
   PSI_plugin_key_string m_name;
-  char m_name_buffer[20];
+  /* Number of characters * max multibyte length of character set */
+  char m_name_buffer[CONTINENT_NAME_LEN];
 
-  bool match(Continent_record *record) {
+  bool match(Continent_record *record) override {
     return mysql_service_pfs_plugin_table->match_key_string(
         false, record->name, record->name_length, &m_name);
   }
 };
 
 /* A structure to define a handle for table in plugin/component code. */
-typedef struct {
+struct Continent_Table_Handle {
   /* Current position instance */
   Continent_POS m_pos;
   /* Next position instance */
@@ -118,7 +119,7 @@ typedef struct {
 
   /* Index indicator */
   unsigned int index_num;
-} Continent_Table_Handle;
+};
 
 PSI_table_handle *continent_open_table(PSI_pos **pos);
 void continent_close_table(PSI_table_handle *handle);
