@@ -1,16 +1,15 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +24,7 @@
 #define COMPONENTS_SERVICES_BITS_PSI_BITS_H
 
 /**
-  @file mysql/components/services/bits/psi_bits.h
+  @file include/mysql/components/services/bits/psi_bits.h
   Performance schema instrumentation interface.
 
   @defgroup instrumentation_interface Instrumentation Interface
@@ -44,15 +43,7 @@ static constexpr unsigned PSI_INSTRUMENT_ME = 0;
 
 #define PSI_DOCUMENT_ME ""
 
-// Remove 'constexpr' in order to pick the right overload for
-//    Prealloced_array CTOR.
-// See https://bugs.llvm.org/show_bug.cgi?id=51582
-// and TEST_F(PreallocedArrayTest, CorrectOverloadIsChosen)
-#if defined(__clang__) && defined(_WIN32)
-static unsigned PSI_NOT_INSTRUMENTED = 0;
-#else
 static constexpr unsigned PSI_NOT_INSTRUMENTED = 0;
-#endif
 
 /**
   Singleton flag.
@@ -123,33 +114,6 @@ static constexpr unsigned PSI_NOT_INSTRUMENTED = 0;
 */
 #define PSI_FLAG_THREAD_SYSTEM (1 << 9)
 
-/**
-  Automatic sequence number flag.
-  Generate thread instances names automatically.
-*/
-#define PSI_FLAG_AUTO_SEQNUM (1 << 10)
-
-/**
-  No sequence number flag.
-  Use thread instances names without sequence numbers.
-*/
-#define PSI_FLAG_NO_SEQNUM (1 << 11)
-
-/**
-  Enable collecting the memory consumed by threads.
-*/
-#define PSI_FLAG_MEM_COLLECT (1 << 12)
-
-/**
-  Instrument is disabled by default.
-*/
-#define PSI_FLAG_DISABLED (1 << 13)
-
-/**
-  Instrument is not timed by default.
-*/
-#define PSI_FLAG_UNTIMED (1 << 14)
-
 #define PSI_VOLATILITY_UNKNOWN 0
 #define PSI_VOLATILITY_PERMANENT 1
 #define PSI_VOLATILITY_PROVISIONING 2
