@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -64,7 +64,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
   Schema
 
    Following are the example implementations of a plugin and a component which
-   uses this pfs_plugin_table_v1 service.
+   uses this pfs_plugin_table service.
 
    @subpage EXAMPLE_PLUGIN
 
@@ -374,24 +374,6 @@ typedef int (*delete_row_values_t)(PSI_table_handle *handle);
 /**
   API to Open a table handle in plugin/component code and reset position
   pointer when a new table handle in opened in Performance Schema.
-
-  @note The pos argument is a pointer to a pointer which references a
-  buffer which identifies the current position (row) in the table.
-  The value assigned to *pos by the open_table_t function is stored in
-  table_plugin_table::m_pos and also the inherited
-  PFS_engine_table::m_pos_ptr. In PFS_engine_table::get_position(void *ref)
-  and PFS_engine_table::set_position(const void *ref),
-  PFS_engine_table_share::m_ref_length bytes are copied (using memcpy) to/from
-  this buffer to save/restore the current position in the table.
-
-  For this reason, any class/struct object used to hold the current table
-  position and whose address gets assigned to *pos needs to be trivially
-  copyable (std::trivially_copyable<T>::value must be true where T is the
-  type of the object pointed to by *pos), and the
-  PFS_engine_table_share_proxy::m_ref_length member variable (inherited from
-  PFS_engine_table_share) must be set to the correct size for the class/struct
-  used.
-
   @param pos pos pointer to be updated.
 
   @return initialized table handle.
@@ -410,21 +392,22 @@ typedef void (*close_table_t)(PSI_table_handle *handle);
  plugin/component.
 */
 struct PFS_engine_table_proxy {
-  rnd_next_t rnd_next{nullptr};
-  rnd_init_t rnd_init{nullptr};
-  rnd_pos_t rnd_pos{nullptr};
-  index_init_t index_init{nullptr};
-  index_read_t index_read{nullptr};
-  index_next_t index_next{nullptr};
-  read_column_value_t read_column_value{nullptr};
-  reset_position_t reset_position{nullptr};
-  write_column_value_t write_column_value{nullptr};
-  write_row_values_t write_row_values{nullptr};
-  update_column_value_t update_column_value{nullptr};
-  update_row_values_t update_row_values{nullptr};
-  delete_row_values_t delete_row_values{nullptr};
-  open_table_t open_table{nullptr};
-  close_table_t close_table{nullptr};
+  rnd_next_t rnd_next;
+  rnd_init_t rnd_init;
+  rnd_pos_t rnd_pos;
+  index_init_t index_init;
+  index_read_t index_read;
+  index_next_t index_next;
+  read_column_value_t read_column_value;
+  reset_position_t reset_position;
+  write_column_value_t write_column_value;
+  write_row_values_t write_row_values;
+  update_column_value_t update_column_value;
+  update_row_values_t update_row_values;
+  delete_row_values_t delete_row_values;
+  open_table_t open_table;
+  close_table_t close_table;
+  PFS_engine_table_proxy() = default;
 };
 typedef struct PFS_engine_table_proxy PFS_engine_table_proxy;
 
@@ -456,7 +439,7 @@ typedef unsigned long long (*get_row_count_t)(void);
 
 /**
   A share to be initialized by plugin/component code and to be provided
-  to add_table() service method of pfs_plugin_table_v1 service.
+  to add_table() service method of pfs_plugin_table service.
  */
 struct PFS_engine_table_share_proxy {
  public:
@@ -480,6 +463,137 @@ struct PFS_engine_table_share_proxy {
 };
 typedef struct PFS_engine_table_share_proxy PFS_engine_table_share_proxy;
 
+/**
+  Definition of pfs_plugin_table service and its methods.
+  @deprecated
+  This service is functional but incomplete,
+  as many apis are missing.
+  Please use pfs_plugin_table_v1
+  and pfs_plugin_column_*_v1 instead.
+*/
+BEGIN_SERVICE_DEFINITION(pfs_plugin_table)
+/* Methods to add tables in Performance Schema */
+DECLARE_METHOD(int, add_tables,
+               (PFS_engine_table_share_proxy * *st_share,
+                unsigned int share_count));
+
+/* Methods to delete tables in Performance Schema */
+DECLARE_METHOD(int, delete_tables,
+               (PFS_engine_table_share_proxy * *st_share,
+                unsigned int share_count));
+
+/* TINYINT */
+DECLARE_METHOD(void, set_field_tinyint, (PSI_field * f, PSI_tinyint value));
+DECLARE_METHOD(void, set_field_utinyint, (PSI_field * f, PSI_utinyint value));
+DECLARE_METHOD(void, get_field_tinyint, (PSI_field * f, PSI_tinyint *value));
+
+/* SMALLINT */
+DECLARE_METHOD(void, set_field_smallint, (PSI_field * f, PSI_smallint value));
+DECLARE_METHOD(void, set_field_usmallint, (PSI_field * f, PSI_usmallint value));
+DECLARE_METHOD(void, get_field_smallint, (PSI_field * f, PSI_smallint *value));
+
+/* MEDIUMINT */
+DECLARE_METHOD(void, set_field_mediumint, (PSI_field * f, PSI_mediumint value));
+DECLARE_METHOD(void, set_field_umediumint,
+               (PSI_field * f, PSI_umediumint value));
+DECLARE_METHOD(void, get_field_mediumint,
+               (PSI_field * f, PSI_mediumint *value));
+
+/* INTEGER(INT) */
+DECLARE_METHOD(void, set_field_integer, (PSI_field * f, PSI_int value));
+DECLARE_METHOD(void, set_field_uinteger, (PSI_field * f, PSI_uint value));
+DECLARE_METHOD(void, get_field_integer, (PSI_field * f, PSI_int *value));
+DECLARE_METHOD(void, read_key_integer,
+               (PSI_key_reader * reader, PSI_plugin_key_integer *key,
+                int find_flag));
+DECLARE_METHOD(bool, match_key_integer,
+               (bool record_null, long record_value,
+                PSI_plugin_key_integer *key));
+
+/* BIGINT */
+DECLARE_METHOD(void, set_field_bigint, (PSI_field * f, PSI_bigint value));
+DECLARE_METHOD(void, set_field_ubigint, (PSI_field * f, PSI_ubigint value));
+DECLARE_METHOD(void, get_field_bigint, (PSI_field * f, PSI_bigint *value));
+
+/* DECIMAL */
+DECLARE_METHOD(void, set_field_decimal, (PSI_field * f, PSI_double value));
+DECLARE_METHOD(void, get_field_decimal, (PSI_field * f, PSI_double *value));
+
+/* FLOAT */
+DECLARE_METHOD(void, set_field_float, (PSI_field * f, PSI_double value));
+DECLARE_METHOD(void, get_field_float, (PSI_field * f, PSI_double *value));
+
+/* DOUBLE */
+DECLARE_METHOD(void, set_field_double, (PSI_field * f, PSI_double value));
+DECLARE_METHOD(void, get_field_double, (PSI_field * f, PSI_double *value));
+
+/* CHAR */
+DECLARE_METHOD(void, set_field_char_utf8,
+               (PSI_field * f, const char *value, unsigned int length));
+DECLARE_METHOD(void, get_field_char_utf8,
+               (PSI_field * f, char *str, unsigned int *length));
+DECLARE_METHOD(void, read_key_string,
+               (PSI_key_reader * reader, PSI_plugin_key_string *key,
+                int find_flag));
+DECLARE_METHOD(bool, match_key_string,
+               (bool record_null, const char *record_string_value,
+                unsigned int record_string_length, PSI_plugin_key_string *key));
+
+/* VARCHAR */
+DECLARE_METHOD(void, set_field_varchar_utf8, (PSI_field * f, const char *str));
+DECLARE_METHOD(void, set_field_varchar_utf8_len,
+               (PSI_field * f, const char *str, unsigned int len));
+DECLARE_METHOD(void, get_field_varchar_utf8,
+               (PSI_field * f, char *str, unsigned int *length));
+
+DECLARE_METHOD(void, set_field_varchar_utf8mb4,
+               (PSI_field * f, const char *str));
+DECLARE_METHOD(void, set_field_varchar_utf8mb4_len,
+               (PSI_field * f, const char *str, unsigned int len));
+
+/* BLOB/TEXT */
+DECLARE_METHOD(void, set_field_blob,
+               (PSI_field * f, const char *val, unsigned int len));
+DECLARE_METHOD(void, get_field_blob,
+               (PSI_field * f, char *val, unsigned int *len));
+
+/* ENUM */
+DECLARE_METHOD(void, set_field_enum, (PSI_field * f, PSI_enum value));
+DECLARE_METHOD(void, get_field_enum, (PSI_field * f, PSI_enum *value));
+
+/* DATE */
+DECLARE_METHOD(void, set_field_date,
+               (PSI_field * f, const char *str, unsigned int length));
+DECLARE_METHOD(void, get_field_date,
+               (PSI_field * f, char *val, unsigned int *len));
+
+/* TIME */
+DECLARE_METHOD(void, set_field_time,
+               (PSI_field * f, const char *str, unsigned int length));
+DECLARE_METHOD(void, get_field_time,
+               (PSI_field * f, char *val, unsigned int *len));
+
+/* DATETIME */
+DECLARE_METHOD(void, set_field_datetime,
+               (PSI_field * f, const char *str, unsigned int length));
+DECLARE_METHOD(void, get_field_datetime,
+               (PSI_field * f, char *val, unsigned int *len));
+
+/* TIMESTAMP */
+DECLARE_METHOD(void, set_field_timestamp,
+               (PSI_field * f, const char *str, unsigned int length));
+DECLARE_METHOD(void, get_field_timestamp,
+               (PSI_field * f, char *val, unsigned int *len));
+
+/* YEAR */
+DECLARE_METHOD(void, set_field_year, (PSI_field * f, PSI_year value));
+DECLARE_METHOD(void, get_field_year, (PSI_field * f, PSI_year *value));
+
+/* NULL */
+DECLARE_METHOD(void, set_field_null, (PSI_field * f));
+
+END_SERVICE_DEFINITION(pfs_plugin_table)
+
 /**
   Definition of pfs_plugin_table_v1 service and its methods.
 */
@@ -608,11 +722,11 @@ DECLARE_METHOD(void, get, (PSI_field * f, PSI_double *value));
 /* No support for indexes. */
 END_SERVICE_DEFINITION(pfs_plugin_column_double_v1)
 
-BEGIN_SERVICE_DEFINITION(pfs_plugin_column_string_v2)
+BEGIN_SERVICE_DEFINITION(pfs_plugin_column_string_v1)
 /* CHAR */
-DECLARE_METHOD(void, set_char_utf8mb4,
+DECLARE_METHOD(void, set_char_utf8,
                (PSI_field * f, const char *value, unsigned int length));
-DECLARE_METHOD(void, get_char_utf8mb4,
+DECLARE_METHOD(void, get_char_utf8,
                (PSI_field * f, char *str, unsigned int *length));
 DECLARE_METHOD(void, read_key_string,
                (PSI_key_reader * reader, PSI_plugin_key_string *key,
@@ -621,12 +735,15 @@ DECLARE_METHOD(bool, match_key_string,
                (bool record_null, const char *record_string_value,
                 unsigned int record_string_length, PSI_plugin_key_string *key));
 /* VARCHAR */
-DECLARE_METHOD(void, get_varchar_utf8mb4,
+DECLARE_METHOD(void, set_varchar_utf8, (PSI_field * f, const char *str));
+DECLARE_METHOD(void, set_varchar_utf8_len,
+               (PSI_field * f, const char *str, unsigned int len));
+DECLARE_METHOD(void, get_varchar_utf8,
                (PSI_field * f, char *str, unsigned int *length));
 DECLARE_METHOD(void, set_varchar_utf8mb4, (PSI_field * f, const char *str));
 DECLARE_METHOD(void, set_varchar_utf8mb4_len,
                (PSI_field * f, const char *str, unsigned int len));
-END_SERVICE_DEFINITION(pfs_plugin_column_string_v2)
+END_SERVICE_DEFINITION(pfs_plugin_column_string_v1)
 
 BEGIN_SERVICE_DEFINITION(pfs_plugin_column_blob_v1)
 DECLARE_METHOD(void, set, (PSI_field * f, const char *val, unsigned int len));
@@ -694,15 +811,4 @@ DECLARE_METHOD(void, get, (PSI_field * f, PSI_year *value));
 /* No support for indexes. */
 END_SERVICE_DEFINITION(pfs_plugin_column_year_v1)
 
-/*
-  SERVICE_DEFINITION(pfs_plugin_column_text_v1)
-  Introduced in MySQL 8.0.34
-  Status: Active.
-*/
-BEGIN_SERVICE_DEFINITION(pfs_plugin_column_text_v1)
-DECLARE_METHOD(void, set,
-               (PSI_field * f, const char *str, unsigned int length));
-DECLARE_METHOD(void, get, (PSI_field * f, char *val, unsigned int *len));
-END_SERVICE_DEFINITION(pfs_plugin_column_text_v1)
-
 #endif
