@@ -1,16 +1,15 @@
-/* Copyright (c) 2005, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -46,7 +45,6 @@
   for functions.
 */
 #if defined(_MSC_VER)
-
 #if defined(MYSQL_DYNAMIC_PLUGIN)
 #ifdef __cplusplus
 #define MYSQL_PLUGIN_EXPORT extern "C" __declspec(dllexport)
@@ -60,17 +58,10 @@
 #define MYSQL_PLUGIN_EXPORT
 #endif
 #endif /*MYSQL_DYNAMIC_PLUGIN */
-
-#else /*_MSC_VER */
-
-#if defined(MYSQL_DYNAMIC_PLUGIN)
-#define MYSQL_PLUGIN_EXPORT MY_ATTRIBUTE((visibility("default")))
-#else
+#else  /*_MSC_VER */
 #define MYSQL_PLUGIN_EXPORT
 #endif
 
-#endif /*_MSC_VER */
-
 #ifdef __cplusplus
 class THD;
 class Item;
@@ -105,7 +96,7 @@ struct MYSQL_XID {
   Plugin API. Common for all plugin types.
 */
 
-#define MYSQL_PLUGIN_INTERFACE_VERSION 0x010B
+#define MYSQL_PLUGIN_INTERFACE_VERSION 0x010A
 
 /*
   The allowable types of plugins
@@ -160,11 +151,8 @@ struct MYSQL_XID {
                          builtin_##NAME##_sizeof_struct_st_plugin,        \
                          builtin_##NAME##_plugin)
 
-#define mysql_declare_plugin_end                                            \
-  , {                                                                       \
-    0, nullptr, nullptr, nullptr, nullptr, 0, nullptr, nullptr, nullptr, 0, \
-        nullptr, nullptr, nullptr, 0                                        \
-  }                                                                         \
+#define mysql_declare_plugin_end                 \
+  , { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } \
   }
 
 /*
@@ -174,27 +162,66 @@ struct MYSQL_XID {
 #define PLUGIN_OPT_NO_INSTALL 1UL   /* Not dynamically loadable */
 #define PLUGIN_OPT_NO_UNINSTALL 2UL /* Not dynamically unloadable */
 #define PLUGIN_OPT_ALLOW_EARLY 4UL  /* allow --early-plugin-load */
-#define PLUGIN_OPT_DEFAULT_OFF 8UL  /* Turned off by default */
-/*
-  All "extra" plugins declared together, same mysql_declare_plugin statement,
-  depends on the first "main" plugin.
-
-  This option is used to turn off the extra plugins if the main plugin is off,
-  even if extra option by default should be on or user specifies that some
-  extra plugin should be on.
-
-  Use it when it does not make sense to have the extra plugins on when the main
-  plugin is off.
- */
-#define PLUGIN_OPT_DEPENDENT_EXTRA_PLUGINS 16UL
 
 /*
   declarations for server variables and command line options
 */
 
-#include <mysql/components/services/bits/system_variables_bits.h>
+#define PLUGIN_VAR_BOOL 0x0001
+#define PLUGIN_VAR_INT 0x0002
+#define PLUGIN_VAR_LONG 0x0003
+#define PLUGIN_VAR_LONGLONG 0x0004
+#define PLUGIN_VAR_STR 0x0005
+#define PLUGIN_VAR_ENUM 0x0006
+#define PLUGIN_VAR_SET 0x0007
+#define PLUGIN_VAR_DOUBLE 0x0008
+#define PLUGIN_VAR_UNSIGNED 0x0080
+#define PLUGIN_VAR_THDLOCAL 0x0100 /* Variable is per-connection */
+#define PLUGIN_VAR_READONLY 0x0200 /* Server variable is read only */
+#define PLUGIN_VAR_NOSYSVAR 0x0400 /* Configurable only by cmd-line */
+
+/**
+  plugin variable CAN'T be used through command line at all
+  neither "--option", nor "--option=value" will work
+  @note you should probably set a default variable value if you use this flag
+*/
+#define PLUGIN_VAR_NOCMDOPT 0x0800
+
+/**
+  plugin variable *value* CAN'T be set via command line
+  you can invoke it with "--option" only, but "--option=value" will not work
+  @note you should probably set a default variable value if you use this flag
+*/
+#define PLUGIN_VAR_NOCMDARG 0x1000
+
+/**
+  plugin variable CAN'T be used through command line without a value
+  "--option=value" must be used, only "--option" won't work
+*/
+#define PLUGIN_VAR_RQCMDARG 0x0000
+
+/**
+  plugin variable can be set via command line, both with or without value
+  either "--option=value", or "--option" will work
+  @note you should probably set a default variable value if you use this flag
+*/
+#define PLUGIN_VAR_OPCMDARG 0x2000
+#define PLUGIN_VAR_NODEFAULT 0x4000 /* SET DEFAULT is prohibited */
+#define PLUGIN_VAR_MEMALLOC 0x8000  /* String needs memory allocated */
+#define PLUGIN_VAR_NOPERSIST                \
+  0x10000 /* SET PERSIST_ONLY is prohibited \
+             for read only variables */
+
+/**
+  There can be some variables which needs to be set before plugin is loaded but
+  not after plugin is loaded. ex: GR specific variables. Below flag must be set
+  for these kind of variables.
+*/
+#define PLUGIN_VAR_PERSIST_AS_READ_ONLY 0x20000
+#define PLUGIN_VAR_INVISIBLE 0x40000 /* Variable should not be shown */
 
 struct SYS_VAR;
+struct st_mysql_value;
 
 /*
   SYNOPSIS
@@ -241,8 +268,7 @@ typedef void (*mysql_var_update_func)(MYSQL_THD thd, SYS_VAR *var,
   (PLUGIN_VAR_READONLY | PLUGIN_VAR_NOSYSVAR | PLUGIN_VAR_NOCMDOPT |   \
    PLUGIN_VAR_NOCMDARG | PLUGIN_VAR_OPCMDARG | PLUGIN_VAR_RQCMDARG |   \
    PLUGIN_VAR_MEMALLOC | PLUGIN_VAR_NODEFAULT | PLUGIN_VAR_NOPERSIST | \
-   PLUGIN_VAR_PERSIST_AS_READ_ONLY | PLUGIN_VAR_INVISIBLE |            \
-   PLUGIN_VAR_SENSITIVE)
+   PLUGIN_VAR_PERSIST_AS_READ_ONLY | PLUGIN_VAR_INVISIBLE)
 
 #define MYSQL_PLUGIN_VAR_HEADER \
   int flags;                    \
@@ -727,6 +753,28 @@ struct Mysql_replication {
   int interface_version;
 };
 
+/*************************************************************************
+  st_mysql_value struct for reading values from mysqld.
+  Used by server variables framework to parse user-provided values.
+  Will be used for arguments when implementing UDFs.
+
+  Note that val_str() returns a string in temporary memory
+  that will be freed at the end of statement. Copy the string
+  if you need it to persist.
+*/
+
+#define MYSQL_VALUE_TYPE_STRING 0
+#define MYSQL_VALUE_TYPE_REAL 1
+#define MYSQL_VALUE_TYPE_INT 2
+
+struct st_mysql_value {
+  int (*value_type)(struct st_mysql_value *);
+  const char *(*val_str)(struct st_mysql_value *, char *buffer, int *length);
+  int (*val_real)(struct st_mysql_value *, double *realbuf);
+  int (*val_int)(struct st_mysql_value *, long long *intbuf);
+  int (*is_unsigned)(struct st_mysql_value *);
+};
+
 /*************************************************************************
   Miscellaneous functions for plugin implementors
 */
@@ -784,8 +832,9 @@ void thd_mark_transaction_to_rollback(MYSQL_THD thd, int all);
 int mysql_tmpfile(const char *prefix);
 
 /**
-  Check the killed state of a connection.
+  Check the killed state of a connection
 
+  @details
   In MySQL support for the KILL statement is cooperative. The KILL
   statement only sets a "killed" flag. This function returns the value
   of that flag.  A thread should check it often, especially inside
