@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -40,15 +40,14 @@
 #define MYSQL_FILE_H
 
 /* For strlen() */
-#include <assert.h>
 #include <string.h>
 
 /* HAVE_PSI_*_INTERFACE */
 #include "my_psi_config.h"  // IWYU pragma: keep
 
+#include "my_dbug.h"
 /* For MY_STAT */
 #include "my_dir.h"
-#include "my_inttypes.h"
 /* For my_chsize */
 #include "my_sys.h"
 #include "mysql/psi/psi_file.h"
@@ -216,7 +215,7 @@
   With the instrumentation, mysql_fclose(NULL, ...) will safely return 0,
   which is an extension compared to my_fclose and is therefore compliant.
   mysql_fclose is on purpose *not* implementing
-  @code assert(file != NULL) @endcode,
+  @code DBUG_ASSERT(file != NULL) @endcode,
   since doing so could introduce regressions.
 */
 #ifdef HAVE_PSI_FILE_INTERFACE
@@ -495,8 +494,8 @@ static inline void inline_mysql_file_register(
 #ifdef HAVE_PSI_FILE_INTERFACE
     const char *category, PSI_file_info *info, int count
 #else
-    const char *category [[maybe_unused]], void *info [[maybe_unused]],
-    int count [[maybe_unused]]
+    const char *category MY_ATTRIBUTE((unused)),
+    void *info MY_ATTRIBUTE((unused)), int count MY_ATTRIBUTE((unused))
 #endif
 ) {
 #ifdef HAVE_PSI_FILE_INTERFACE
