@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,6 +24,7 @@
   @file include/sslopt-case.h
 */
 
+#if defined(HAVE_OPENSSL)
 
 #if defined(MYSQL_SERVER) && !defined(XTRABACKUP)
 #error This header is supposed to be used only in the client
@@ -48,6 +49,4 @@ case OPT_SSL_CIPHER:
 case OPT_SSL_CRL:
 case OPT_SSL_CRLPATH:
 case OPT_TLS_VERSION:
-case OPT_SSL_SESSION_DATA:
-case OPT_TLS_SNI_SERVERNAME:
   break;
