@@ -1,4 +1,4 @@
--- Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
+-- Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License, version 2.0,
@@ -79,7 +79,8 @@ BEGIN
     WHERE variable_name NOT IN ('timestamp', 'server_uuid',
                                 'gtid_executed', 'gtid_purged',
                                 'group_replication_group_name',
-                                'keyring_file_data')
+                                'keyring_file_data',
+                                'innodb_thread_sleep_delay')
   ORDER BY VARIABLE_NAME;
 
   -- Dump all persisted variables, those that may change.
@@ -185,6 +186,8 @@ BEGIN
     mysql.help_topic,
     mysql.procs_priv,
     mysql.proxies_priv,
+    mysql.replication_asynchronous_connection_failover,
+    mysql.replication_asynchronous_connection_failover_managed,
     mysql.role_edges,
     mysql.tables_priv,
     mysql.time_zone,
@@ -194,6 +197,10 @@ BEGIN
     mysql.time_zone_transition_type,
     mysql.user;
 
+  -- Check that Replica IO Monitor thread state is the same before
+  -- and after the test run, which is not running.
+  SELECT * FROM performance_schema.threads WHERE NAME="thread/sql/replica_monitor";
+
 END$$
 
 -- Procedure used by test case used to force all
