@@ -1,5 +1,5 @@
 # -*- cperl -*-
-# Copyright (c) 2007, 2023, Oracle and/or its affiliates.
+# Copyright (c) 2007, 2020, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0,
@@ -119,23 +119,6 @@ sub fix_host {
   return $hosts[$host_no];
 }
 
-sub fix_cpubind {
-  my ($self, $config, $group_name, $group) = @_;
-  # Replace cpubind with #cpubind
-  my $value = $group->if_exist('cpubind');
-  if (defined $value) {
-    $group->remove('cpubind');
-  }
-  return $value;
-}
-
-sub fix_cluster_config_suffix {
-  my ($self, $config, $group_name, $group) = @_;
-
-  my ($process_type, $idx, $suffix) = split(/\./, $group_name);
-  return ".$suffix";
-}
-
 sub is_unique {
   my ($config, $name, $value) = @_;
 
@@ -219,26 +202,6 @@ sub fix_log {
   return "$dir/mysqld.log";
 }
 
-sub fix_group_setting {
-  my ($self, $config, $group_name, $group, $option_name) = @_;
-  # Use value from [mysqld.X], [mysqld](if group exists) or 1
-  return $group->if_exist($option_name) ||
-         ($config->group_exists('mysqld') &&
-          $config->group('mysqld')->if_exist($option_name)) || 1;
-}
-
-sub fix_slow_query_log {
-  my ($self, $config, $group_name, $group) = @_;
-  return fix_group_setting($self, $config, $group_name,
-                           $group, 'slow_query_log');
-}
-
-sub fix_general_log {
-  my ($self, $config, $group_name, $group) = @_;
-  return fix_group_setting($self, $config, $group_name,
-                           $group, 'general_log');
-}
-
 sub fix_log_slow_queries {
   my ($self, $config, $group_name, $group) = @_;
   my $dir = dirname($group->value('datadir'));
@@ -297,7 +260,6 @@ sub fix_rsa_public_key {
 #    in the order listed here
 my @mysqld_rules = (
   { '#abs_datadir'                                 => \&fix_abs_datadir },
-  { '#cpubind'                                     => \&fix_cpubind },
   { '#host'                                        => \&fix_host },
   { '#log-error'                                   => \&fix_log_error },
   { 'caching_sha2_password_private_key_path'       => \&fix_rsa_private_key },
@@ -308,7 +270,7 @@ my @mysqld_rules = (
   { 'datadir'                                      => \&fix_datadir },
   { 'port'                                         => \&fix_port },
   { 'admin-port'                                   => \&fix_admin_port },
-  { 'general_log'                                  => \&fix_general_log },
+  { 'general_log'                                  => 1 },
   { 'general_log_file'                             => \&fix_log },
   { 'loose-mysqlx-port'                            => \&fix_x_port },
   { 'loose-mysqlx-socket'                          => \&fix_x_socket },
@@ -318,7 +280,7 @@ my @mysqld_rules = (
   { 'loose-mysqlx-ssl-key'                         => "" },
   { 'pid-file'                                     => \&fix_pidfile },
   { 'server-id'                                    => \&fix_server_id, },
-  { 'slow_query_log'                               => \&fix_slow_query_log },
+  { 'slow_query_log'                               => 1 },
   { 'slow_query_log_file'                          => \&fix_log_slow_queries },
   { 'socket'                                       => \&fix_socket },
   { 'ssl-ca'                                       => \&fix_ssl_ca },
@@ -371,15 +333,13 @@ sub fix_cluster_backup_dir {
 #  - will be run in order listed here
 my @ndb_mgmd_rules =
   ({ 'DataDir' => \&fix_cluster_dir },
-   { 'PortNumber' => \&fix_ndb_mgmd_port },
-   { '#cpubind'   => \&fix_cpubind },);
+   { 'PortNumber' => \&fix_ndb_mgmd_port },);
 
 # Rules to run for each ndbd in the config
 #  - will be run in order listed here
 my @ndbd_rules = ({ 'BackupDataDir' => \&fix_cluster_backup_dir },
                   { 'DataDir'       => \&fix_cluster_dir },
-                  { 'HostName'      => \&fix_host },
-                  { '#cpubind'      => \&fix_cpubind },);
+                  { 'HostName'      => \&fix_host },);
 
 # Rules to run for each memcached in the config
 #  - will be run in order listed here
@@ -399,8 +359,7 @@ my @client_rules = ();
 # Rules to run for [mysqltest] section
 #  - will be run in order listed here
 my @mysqltest_rules = ({ 'server-public-key-path' => \&fix_rsa_public_key },
-                       { 'ssl-mode'               => \&fix_ssl_disabled },
-                       { '#cpubind'               => \&fix_cpubind },);
+                       { 'ssl-mode'               => \&fix_ssl_disabled },);
 
 # Rules to run for [mysqlbinlog] section
 #  - will be run in order listed here
@@ -562,16 +521,6 @@ sub post_fix_mysql_cluster_section {
       }
     }
 
-    # Add ndb_connectstring to each ndb_mgmd connected to this
-    # cluster.
-    foreach my $ndb_mgmd ($config->like('cluster_config.ndb_mgmd.')) {
-      if ($ndb_mgmd->suffix() eq $group->suffix()) {
-        my $after = $ndb_mgmd->after('cluster_config.ndb_mgmd');
-        $config->insert("ndb_mgmd$after",
-                        'ndb_connectstring', $ndb_connectstring);
-      }
-    }
-
     # Add ndb_connectstring to each mysqld connected to this
     # cluster.
     foreach my $mysqld ($config->like('cluster_config.mysqld.')) {
@@ -658,7 +607,6 @@ sub run_generate_sections_from_cluster_config {
       $group->insert($option_name, join(",", @hosts));
 
       # Generate sections for each host
-      my $instances = @hosts;
       foreach my $host (@hosts) {
         my $idx    = $idxes{$option_name}++;
         my $suffix = $group->suffix();
@@ -684,19 +632,8 @@ sub run_generate_sections_from_cluster_config {
         # If prediction of node id is wrong, and node id for another node type
         # is used, that will cause testcase to fail during setup.
         if ($option_name eq 'ndbd') {
-          if ($instances > 1) {
-            $config->insert("$option_name.$idx$suffix",
-                            'ndb-nodeid', $nodeid);
-          }
-        }
-
-        if ($option_name eq 'ndb_mgmd') {
-          if ($instances > 1) {
-            $config->insert("$option_name.$idx$suffix",
-                            'ndb-nodeid', $nodeid);
-          }
           $config->insert("$option_name.$idx$suffix",
-                          'cluster-config-suffix', $suffix);
+                          'ndb-nodeid', $nodeid);
         }
 
         if ($option_name eq 'mysqld') {
@@ -757,9 +694,6 @@ sub new_config {
   $self->run_section_rules($config, 'cluster_config.ndb_mgmd.',
                            @ndb_mgmd_rules);
 
-  $self->run_section_rules($config, 'ndb_mgmd.',
-    ({ 'cluster-config-suffix' => \&fix_cluster_config_suffix },));
-
   $self->run_section_rules($config, 'cluster_config.ndbd', @ndbd_rules);
 
   $self->run_section_rules($config, 'mysqld.', @mysqld_rules);
