@@ -1,17 +1,16 @@
 # -*- cperl -*-
-# Copyright (c) 2004, 2024, Oracle and/or its affiliates.
+# Copyright (c) 2004, 2020, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0,
 # as published by the Free Software Foundation.
 #
-# This program is designed to work with certain software (including
+# This program is also distributed with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
 # documentation.  The authors of MySQL hereby grant you an additional
 # permission to link the program and your derivative works with the
-# separately licensed software that they have either included with
-# the program or referenced in the documentation.
+# separately licensed software that they have included with MySQL.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -232,7 +231,7 @@ sub mtr_report_test ($) {
     } else {
       my $timeout = $tinfo->{'timeout'};
       if ($timeout) {
-        mtr_report($report, "[ $fail ]  timeout after $timeout seconds");
+        mtr_report($report, "[ $retry$fail ]  timeout after $timeout seconds");
         mtr_report("        Test ended at $timest");
         mtr_report("\n$tinfo->{'comment'}");
         return;
@@ -440,12 +439,6 @@ sub mtr_generate_xml_report($) {
       }
 
       $failure_comment = ": " . $1 if ($failure_log =~ /mysqltest: (.*)/);
-      #Replace invalid characters in xml attribute values
-      $failure_comment =~ s/&/&amp;/g;
-      $failure_comment =~ s/"/&quot;/g;
-      $failure_comment =~ s/'/&apos;/g;
-      $failure_comment =~ s/>/&lt;/g;
-      $failure_comment =~ s/</&gt;/g;
 
       # For large comments from mysql-test-run.pl, display a brief message in
       # the attribute 'message' and prepend details within the <failure> tag
@@ -500,12 +493,6 @@ sub mtr_generate_xml_report($) {
       my $not_run_status = $tinfo->{'disable'} ? "disabled" : "skipped";
 
       print $xml_report_file " " x 4;
-      # Replace invalid characters from test comments
-      $tinfo->{'comment'} =~ s/&/&amp;/g;
-      $tinfo->{'comment'} =~ s/"/&quot;/g;
-      $tinfo->{'comment'} =~ s/'/&apos;/g;
-      $tinfo->{'comment'} =~ s/>/&lt;/g;
-      $tinfo->{'comment'} =~ s/</&gt;/g;
       print $xml_report_file "<testcase name=\"$tname\"$combination " .
         "status=\"$not_run_status\" " . "time=\"$test_time\" " .
         "suitename=\"$tsuite\" " . "comment=\"$tinfo->{'comment'}\" />\n";
@@ -699,13 +686,13 @@ sub mtr_report_stats ($$;$) {
     # Print the list of tests that failed in a format that can be copy
     # pasted to rerun only failing tests.
     if (%seen) {
-      summary_print("Failing test(s): " . join(" ", sort keys %seen) . "\n\n");
+      summary_print("Failing test(s): " . join(" ", keys %seen) . "\n\n");
     }
 
     # Print unstable tests, if any
     if (%seen_unstable) {
       summary_print("Unstable test(s)(failures/attempts): " .
-                join(" ", map { $_ . $seen_unstable{$_} } sort keys %seen_unstable) .
+                join(" ", map { $_ . $seen_unstable{$_} } keys %seen_unstable) .
                 "\n\n");
     }
 
