@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -34,11 +34,12 @@
   Possibly use of asyncronic io.
   macros for read and writes for faster io.
   Used instead of FILE when reading or writing whole files.
-  This code makes mf_rec_cache obsolete (currently only used by ISAM).
-  One can change info->pos_in_file to a higher value to skip bytes in the file
-  if also info->read_pos is set to info->read_end. If called through
-  open_cached_file(), then the temporary file will only be created if a write
-  exceeds the file buffer or if one calls my_b_flush_io_cache().
+  This code makes mf_rec_cache obsolete (currently only used by ISAM)
+  One can change info->pos_in_file to a higher value to skip bytes in file if
+  also info->read_pos is set to info->read_end.
+  If called through open_cached_file(), then the temporary file will
+  only be created if a write exeeds the file buffer or if one calls
+  my_b_flush_io_cache().
 
   If one uses SEQ_READ_APPEND, then two buffers are allocated, one for
   reading and another for writing.  Reads are first done from disk and
@@ -70,6 +71,7 @@ TODO:
 #include <sys/types.h>
 #include <algorithm>
 
+#include "m_string.h"
 #include "my_byteorder.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
@@ -84,12 +86,11 @@ TODO:
 #include "mysql/psi/mysql_file.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_mysql_alloc.h"
-#include "mysql/strings/int2str.h"
 #include "mysys/mysys_priv.h"
 #include "thr_mutex.h"
 
 PSI_file_key key_file_io_cache;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 bool binlog_cache_temporary_file_is_encrypted = false;
 #endif
 
@@ -108,7 +109,7 @@ bool binlog_cache_temporary_file_is_encrypted = false;
     info		IO_CACHE handler
 
   NOTES
-    This is called on automatically on init or reinit of IO_CACHE
+    This is called on automaticly on init or reinit of IO_CACHE
     It must be called externally if one moves or copies an IO_CACHE
     object.
 */
@@ -125,7 +126,7 @@ void setup_io_cache(IO_CACHE *info) {
 }
 
 static void init_functions(IO_CACHE *info) {
-  const enum cache_type type = info->type;
+  enum cache_type type = info->type;
   switch (type) {
     case READ_NET:
       /*
@@ -161,7 +162,7 @@ static void init_functions(IO_CACHE *info) {
                        If == 0 then use my_default_record_cache_size
     type               Type of cache
     seek_offset        Where cache should start reading/writing
-    use_async_io       Set to 1 of we should use async_io (if available)
+    use_async_io       Set to 1 of we should use async_io (if avaiable)
     cache_myflags      Bitmap of different flags
                        MY_WME | MY_FAE | MY_NABP | MY_FNABP |
                        MY_DONT_CHECK_FILESIZE
@@ -208,7 +209,7 @@ int init_io_cache_ext(IO_CACHE *info, File file, size_t cachesize,
         the beginning of whatever this file is, then somebody made a bad
         assumption.
       */
-      assert(seek_offset == 0);
+      DBUG_ASSERT(seek_offset == 0);
     } else
       info->seek_not_done = (seek_offset != pos);
   }
@@ -325,15 +326,16 @@ int init_io_cache(IO_CACHE *info, File file, size_t cachesize,
 */
 
 bool reinit_io_cache(IO_CACHE *info, enum cache_type type, my_off_t seek_offset,
-                     bool use_async_io [[maybe_unused]], bool clear_cache) {
+                     bool use_async_io MY_ATTRIBUTE((unused)),
+                     bool clear_cache) {
   DBUG_TRACE;
   DBUG_PRINT("enter", ("cache: %p type: %d  seek_offset: %lu  clear_cache: %d",
                        info, type, (ulong)seek_offset, (int)clear_cache));
 
   /* One can't do reinit with the following types */
-  assert(type != READ_NET && info->type != READ_NET && type != WRITE_NET &&
-         info->type != WRITE_NET && type != SEQ_READ_APPEND &&
-         info->type != SEQ_READ_APPEND);
+  DBUG_ASSERT(type != READ_NET && info->type != READ_NET && type != WRITE_NET &&
+              info->type != WRITE_NET && type != SEQ_READ_APPEND &&
+              info->type != SEQ_READ_APPEND);
 
   /* If the whole file is in memory, avoid flushing to disk */
   if (!clear_cache && seek_offset >= info->pos_in_file &&
@@ -427,7 +429,7 @@ int _my_b_read(IO_CACHE *info, uchar *Buffer, size_t Count) {
 
   /* If the buffer is not empty yet, copy what is available. */
   if ((left_length = (size_t)(info->read_end - info->read_pos))) {
-    assert(Count >= left_length); /* User is not using my_b_read() */
+    DBUG_ASSERT(Count >= left_length); /* User is not using my_b_read() */
     memcpy(Buffer, info->read_pos, left_length);
     Buffer += left_length;
     Count -= left_length;
@@ -453,7 +455,7 @@ int _my_b_read(IO_CACHE *info, uchar *Buffer, size_t Count) {
         info->file is a pipe or socket or FIFO.  We never should have tried
         to seek on that.  See Bugs#25807 and #22828 for more info.
       */
-      assert(my_errno() != ESPIPE);
+      DBUG_ASSERT(my_errno() != ESPIPE);
       info->error = -1;
       return 1;
     }
@@ -623,9 +625,9 @@ void init_io_cache_share(IO_CACHE *read_cache, IO_CACHE_SHARE *cshare,
                                 "write_cache: %p  threads: %u",
                                 read_cache, cshare, write_cache, num_threads));
 
-  assert(num_threads > 1);
-  assert(read_cache->type == READ_CACHE);
-  assert(!write_cache || (write_cache->type == WRITE_CACHE));
+  DBUG_ASSERT(num_threads > 1);
+  DBUG_ASSERT(read_cache->type == READ_CACHE);
+  DBUG_ASSERT(!write_cache || (write_cache->type == WRITE_CACHE));
 
   mysql_mutex_init(key_IO_CACHE_SHARE_mutex, &cshare->mutex,
                    MY_MUTEX_INIT_FAST);
@@ -926,7 +928,7 @@ int _my_b_read_r(IO_CACHE *cache, uchar *Buffer, size_t Count) {
   DBUG_TRACE;
 
   if ((left_length = (size_t)(cache->read_end - cache->read_pos))) {
-    assert(Count >= left_length); /* User is not using my_b_read() */
+    DBUG_ASSERT(Count >= left_length); /* User is not using my_b_read() */
     memcpy(Buffer, cache->read_pos, left_length);
     Buffer += left_length;
     Count -= left_length;
@@ -949,7 +951,7 @@ int _my_b_read_r(IO_CACHE *cache, uchar *Buffer, size_t Count) {
     }
     if (lock_io_cache(cache, pos_in_file)) {
       /* With a synchronized write/read cache we won't come here... */
-      assert(!cshare->source_cache);
+      DBUG_ASSERT(!cshare->source_cache);
       /*
         ... unless the writer has gone before this thread entered the
         lock. Simulate EOF in this case. It can be distinguished by
@@ -1040,18 +1042,18 @@ static void copy_to_read_buffer(IO_CACHE *write_cache,
                                 size_t write_length) {
   IO_CACHE_SHARE *cshare = write_cache->share;
 
-  assert(cshare->source_cache == write_cache);
+  DBUG_ASSERT(cshare->source_cache == write_cache);
   /*
     write_length is usually less or equal to buffer_length.
     It can be bigger if _my_b_write() is called with a big length.
   */
   while (write_length) {
     size_t copy_length = std::min(write_length, write_cache->buffer_length);
-    [[maybe_unused]] int rc;
+    int MY_ATTRIBUTE((unused)) rc;
 
     rc = lock_io_cache(write_cache, write_cache->pos_in_file);
     /* The writing thread does always have the lock when it awakes. */
-    assert(rc);
+    DBUG_ASSERT(rc);
 
     memcpy(cshare->buffer, write_buffer, copy_length);
 
@@ -1087,7 +1089,7 @@ int _my_b_seq_read(IO_CACHE *info, uchar *Buffer, size_t Count) {
 
   /* first, read the regular buffer */
   if ((left_length = (size_t)(info->read_end - info->read_pos))) {
-    assert(Count > left_length); /* User is not using my_b_read() */
+    DBUG_ASSERT(Count > left_length); /* User is not using my_b_read() */
     memcpy(Buffer, info->read_pos, left_length);
     Buffer += left_length;
     Count -= left_length;
@@ -1162,7 +1164,7 @@ int _my_b_seq_read(IO_CACHE *info, uchar *Buffer, size_t Count) {
 
       /*
          added the line below to make
-         assert(pos_in_file==info->end_of_file) pass.
+         DBUG_ASSERT(pos_in_file==info->end_of_file) pass.
          otherwise this does not appear to be needed
       */
       pos_in_file += length;
@@ -1185,16 +1187,15 @@ read_append_buffer:
 
   {
     /* First copy the data to Count */
-    const size_t len_in_buff =
-        (size_t)(info->write_pos - info->append_read_pos);
+    size_t len_in_buff = (size_t)(info->write_pos - info->append_read_pos);
     size_t copy_len;
     size_t transfer_len;
 
-    assert(info->append_read_pos <= info->write_pos);
+    DBUG_ASSERT(info->append_read_pos <= info->write_pos);
     /*
       TODO: figure out if the assert below is needed or correct.
     */
-    assert(pos_in_file == info->end_of_file);
+    DBUG_ASSERT(pos_in_file == info->end_of_file);
     copy_len = std::min(Count, len_in_buff);
     memcpy(Buffer, info->append_read_pos, copy_len);
     info->append_read_pos += copy_len;
@@ -1277,9 +1278,9 @@ int _my_b_write(IO_CACHE *info, const uchar *Buffer, size_t Count) {
       MY_FNABP are not set.
     */
     DBUG_EXECUTE_IF("verify_mysql_encryption_file_write_bytes",
-                    const size_t write_bytes = mysql_encryption_file_write(
+                    size_t write_bytes = mysql_encryption_file_write(
                         info, Buffer, length, info->myflags);
-                    assert(write_bytes == length););
+                    DBUG_ASSERT(write_bytes == length););
 
     if (DBUG_EVALUATE_IF("verify_mysql_encryption_file_write_bytes", false,
                          true)) {
@@ -1323,7 +1324,7 @@ int my_b_append(IO_CACHE *info, const uchar *Buffer, size_t Count) {
     Assert that we cannot come here with a shared cache. If we do one
     day, we might need to add a call to copy_to_read_buffer().
   */
-  assert(!info->share);
+  DBUG_ASSERT(!info->share);
 
   lock_append_buffer(info);
   rest_length = (size_t)(info->write_end - info->write_pos);
@@ -1381,7 +1382,7 @@ int my_block_write(IO_CACHE *info, const uchar *Buffer, size_t Count,
     Assert that we cannot come here with a shared cache. If we do one
     day, we might need to add a call to copy_to_read_buffer().
   */
-  assert(!info->share);
+  DBUG_ASSERT(!info->share);
 
   if (pos < info->pos_in_file) {
     /* Of no overlap, write everything without buffering */
@@ -1404,7 +1405,7 @@ int my_block_write(IO_CACHE *info, const uchar *Buffer, size_t Count,
   /* Check if we want to write inside the used part of the buffer.*/
   length = (size_t)(info->write_end - info->buffer);
   if (pos < info->pos_in_file + length) {
-    const size_t offset = (size_t)(pos - info->pos_in_file);
+    size_t offset = (size_t)(pos - info->pos_in_file);
     length -= offset;
     if (length > Count) length = Count;
     memcpy(info->buffer + offset, Buffer, length);
@@ -1430,7 +1431,7 @@ int my_block_write(IO_CACHE *info, const uchar *Buffer, size_t Count,
 int my_b_flush_io_cache(IO_CACHE *info, int need_append_buffer_lock) {
   size_t length;
   my_off_t pos_in_file;
-  const bool append_cache = (info->type == SEQ_READ_APPEND);
+  bool append_cache = (info->type == SEQ_READ_APPEND);
   DBUG_TRACE;
   DBUG_PRINT("enter", ("cache: %p", info));
 
@@ -1479,7 +1480,7 @@ int my_b_flush_io_cache(IO_CACHE *info, int need_append_buffer_lock) {
         info->end_of_file = std::max(info->end_of_file, (pos_in_file + length));
       } else {
         info->end_of_file += (info->write_pos - info->append_read_pos);
-        assert(info->end_of_file == mysql_file_tell(info->file, MYF(0)));
+        DBUG_ASSERT(info->end_of_file == mysql_file_tell(info->file, MYF(0)));
       }
 
       info->append_read_pos = info->write_pos = info->write_buffer;
@@ -1526,7 +1527,7 @@ int end_io_cache(IO_CACHE *info) {
     Every thread must call remove_io_thread(). The last one destroys
     the share elements.
   */
-  assert(!info->share || !info->share->total_threads);
+  DBUG_ASSERT(!info->share || !info->share->total_threads);
 
   if ((pre_close = info->pre_close)) {
     (*pre_close)(info);
@@ -1646,7 +1647,7 @@ my_off_t mysql_encryption_file_seek(IO_CACHE *cache, my_off_t pos, int whence,
 
 size_t mysql_encryption_file_read(IO_CACHE *cache, uchar *buffer, size_t count,
                                   myf flags) {
-  const size_t ret = mysql_file_read(cache->file, buffer, count, flags);
+  size_t ret = mysql_file_read(cache->file, buffer, count, flags);
   if (ret != MY_FILE_ERROR && cache->m_decryptor != nullptr)
     cache->m_decryptor->decrypt(buffer, buffer, ret ? ret : count);
   return ret;
