@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -49,7 +48,6 @@
   --print-defaults	  ; Print the modified command line and exit
   --login-path=login-path-name ; Read options under login-path-name from
                                 the login file.
-  --no-login-paths ; login file is not read.
 */
 
 #include "my_config.h"
@@ -59,6 +57,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 
+#include "m_ctype.h"
 #include "m_string.h"
 #include "my_aes.h"
 #include "my_alloc.h"
@@ -70,18 +69,14 @@
 #include "my_getopt.h"
 #include "my_inttypes.h"
 #include "my_io.h"
+#include "my_loglevel.h"
 #include "my_macros.h"
 #include "my_psi_config.h"
-#include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_file.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysql_version.h"  // MYSQL_PERSIST_CONFIG_NAME
 #include "mysys/my_default_priv.h"
 #include "mysys/mysys_priv.h"
 #include "mysys_err.h"
-#include "nulls.h"
-#include "strmake.h"
-#include "strxmov.h"
 #include "typelib.h"
 #ifdef _WIN32
 #include <winbase.h>
@@ -155,7 +150,7 @@ PSI_memory_key key_memory_defaults;
    check the pointer, use "----args-separator----" here to ease debug
    if someone misused it.
 
-   The args separator will only be added when
+   The args seprator will only be added when
    my_getopt_use_args_seprator is set to TRUE before calling
    load_defaults();
 
@@ -163,7 +158,7 @@ PSI_memory_key key_memory_defaults;
 */
 static const char *args_separator = "----args-separator----";
 inline static void set_args_separator(const char **arg) {
-  assert(my_getopt_use_args_separator);
+  DBUG_ASSERT(my_getopt_use_args_separator);
   *arg = args_separator;
 }
 /*
@@ -214,7 +209,7 @@ bool no_defaults = false;
 static const char **default_directories = nullptr;
 
 #ifdef _WIN32
-static const char *f_extensions[] = {".ini", ".cnf", nullptr};
+static const char *f_extensions[] = {".ini", ".cnf", 0};
 #else
 static const char *f_extensions[] = {".cnf", nullptr};
 #endif
@@ -241,17 +236,11 @@ struct handle_option_ctx {
 static int search_default_file(Process_option_func func, void *func_ctx,
                                const char *dir, const char *config_file,
                                bool is_login_file);
-static int search_default_file_with_ext(Process_option_func func,
-                                        void *func_ctx, const char *dir,
-                                        const char *ext,
-                                        const char *config_file,
-                                        int recursion_level, bool is_login_file,
-                                        bool report_os_error_on_open);
-using mysql_file_getline_ret = std::unique_ptr<char, decltype(std::free) *>;
-static mysql_file_getline_ret mysql_file_getline(char *str, int size,
-                                                 MYSQL_FILE *file,
-                                                 bool is_login_file)
-    MY_ATTRIBUTE((nonnull));
+static int search_default_file_with_ext(
+    Process_option_func func, void *func_ctx, const char *dir, const char *ext,
+    const char *config_file, int recursion_level, bool is_login_file);
+static bool mysql_file_getline(char *str, int size, MYSQL_FILE *file,
+                               bool is_login_file);
 
 /**
   Create the list of default directories.
@@ -350,7 +339,6 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
   const char **dirs;
   char *forced_default_file, *forced_extra_defaults;
   DBUG_TRACE;
-  bool found_no_login_paths_dummy = false;
 
   /* Skip for login file. */
   if (!is_login_file) {
@@ -358,14 +346,13 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
     *args_used += get_defaults_options(
         *argc - *args_used, *argv + *args_used, &forced_default_file,
         &forced_extra_defaults, const_cast<char **>(&my_defaults_group_suffix),
-        const_cast<char **>(&my_login_path), found_no_defaults,
-        &found_no_login_paths_dummy);
+        const_cast<char **>(&my_login_path), found_no_defaults);
 
     if (!my_defaults_group_suffix)
       my_defaults_group_suffix = getenv("MYSQL_GROUP_SUFFIX");
 
     if (forced_extra_defaults && !defaults_already_read) {
-      const int error =
+      int error =
           fn_expand(forced_extra_defaults, my_defaults_extra_file_buffer);
       if (error) return error;
 
@@ -373,7 +360,7 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
     }
 
     if (forced_default_file && !defaults_already_read) {
-      const int error = fn_expand(forced_default_file, my_defaults_file_buffer);
+      int error = fn_expand(forced_default_file, my_defaults_file_buffer);
       if (error) return error;
       my_defaults_file = my_defaults_file_buffer;
     }
@@ -468,9 +455,8 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
   // If my defaults file is set (from a previous run), we read it
   else if (my_defaults_file) {
     int error;
-    if ((error = search_default_file_with_ext(func, func_ctx, "", "",
-                                              my_defaults_file, 0,
-                                              is_login_file, false)) < 0)
+    if ((error = search_default_file_with_ext(
+             func, func_ctx, "", "", my_defaults_file, 0, is_login_file)) < 0)
       goto err;
     if (error > 0) {
       my_message_local(ERROR_LEVEL, EE_FAILED_TO_OPEN_DEFAULTS_FILE,
@@ -487,7 +473,7 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
         int error;
         if ((error = search_default_file_with_ext(func, func_ctx, "", "",
                                                   my_defaults_extra_file, 0,
-                                                  is_login_file, false)) < 0)
+                                                  is_login_file)) < 0)
           goto err; /* Fatal error */
         if (error > 0) {
           my_message_local(ERROR_LEVEL, EE_FAILED_TO_OPEN_DEFAULTS_FILE,
@@ -548,8 +534,7 @@ static int handle_default_option(void *in_ctx, const char *group_name,
 /*
   Gets options from the command line, however if --no-defaults
   option is used, --defaults-file & --defaults-extra-file options
-  would be ignored and if --no-login-paths option is used,
-  --login-path option would be ignored.
+  would be ignored.
 
   SYNOPSIS
     get_defaults_options()
@@ -559,7 +544,6 @@ static int handle_default_option(void *in_ctx, const char *group_name,
     extra_defaults              --defaults-extra-file option
     group_suffix                --defaults-group-suffix option
     login_path                  --login-path option
-    found_no_login_paths        --no-login-paths option present or not
 
   RETURN
     # Number of arguments used from *argv
@@ -569,11 +553,9 @@ static int handle_default_option(void *in_ctx, const char *group_name,
 
 int get_defaults_options(int argc, char **argv, char **defaults,
                          char **extra_defaults, char **group_suffix,
-                         char **login_path, bool found_no_defaults,
-                         bool *found_no_login_paths) {
+                         char **login_path, bool found_no_defaults) {
   int org_argc = argc, prev_argc = 0, default_option_count = 0;
   *defaults = *extra_defaults = *group_suffix = *login_path = nullptr;
-  bool found_login_path = false;
 
   while (argc >= 2 && argc != prev_argc) {
     /* Skip program name or previously handled argument */
@@ -605,33 +587,13 @@ int get_defaults_options(int argc, char **argv, char **defaults,
       default_option_count++;
       continue;
     }
-    if (is_prefix(*argv, "--no-login-paths") && !found_login_path) {
-      argc--;
-      default_option_count++;
-      *found_no_login_paths = true;
-      continue;
-    }
-    if (!*login_path && is_prefix(*argv, "--login-path=") &&
-        !*found_no_login_paths) {
+    if (!*login_path && is_prefix(*argv, "--login-path=")) {
       *login_path = *argv + sizeof("--login-path=") - 1;
       argc--;
       default_option_count++;
-      found_login_path = true;
       continue;
     }
   }
-  /* Check if --no-login-paths specified before --no-defaults */
-  if (*found_no_login_paths && !found_no_defaults) {
-    int count = argc;
-    while (count >= 2) {
-      if (is_prefix(*argv, "--no-defaults")) {
-        *found_no_login_paths = false;
-        break;
-      }
-      count--;
-      argv++;
-    }
-  }
   return org_argc - argc;
 }
 
@@ -642,7 +604,7 @@ int get_defaults_options(int argc, char **argv, char **defaults,
     load_defaults()
     conf_file			Basename for configuration file to search for.
                                 If this is a path, then only this file is read.
-    groups			Which [group] entries to read.
+    groups			Which [group] entrys to read.
                                 Points to an null terminated array of pointers
     argc			Pointer to argc of original program
     argv			Pointer to argv of original program
@@ -672,7 +634,7 @@ bool my_defaults_read_login_file = true;
     my_load_defaults()
     conf_file			Basename for configuration file to search for.
                                 If this is a path, then only this file is read.
-    groups			Which [group] entries to read.
+    groups			Which [group] entrys to read.
                                 Points to an null terminated array of pointers
     argc			Pointer to argc of original program
     argv			Pointer to argv of original program
@@ -710,18 +672,14 @@ int my_load_defaults(const char *conf_file, const char **groups, int *argc,
   TYPELIB group;
   bool found_print_defaults = false;
   uint args_used = 0;
-  int error = 0, l_argc = *argc;
+  int error = 0;
   const char **ptr;
   const char **res;
-  char **l_argv = argv[0];
-  char *defaults_dummy, *extra_defaults_dummy, *group_suffix_dummy,
-      *login_path_dummy;
   struct handle_option_ctx ctx;
   const char **dirs;
   char my_login_file[FN_REFLEN];
   bool found_no_defaults = false;
-  bool found_no_login_paths = false;
-  const uint args_sep = my_getopt_use_args_separator ? 1 : 0;
+  uint args_sep = my_getopt_use_args_separator ? 1 : 0;
   DBUG_TRACE;
 
   if ((dirs = init_default_directories(alloc)) == nullptr) goto err;
@@ -732,11 +690,6 @@ int my_load_defaults(const char *conf_file, const char **groups, int *argc,
   if (*argc >= 2 && !strcmp(argv[0][1], "--no-defaults"))
     no_defaults = found_no_defaults = true;
 
-  /* Check if the user doesn't want login file processing */
-  get_defaults_options(l_argc, l_argv, &defaults_dummy, &extra_defaults_dummy,
-                       &group_suffix_dummy, &login_path_dummy,
-                       found_no_defaults, &found_no_login_paths);
-
   group.count = 0;
   group.name = "defaults";
   group.type_names = groups;
@@ -753,13 +706,13 @@ int my_load_defaults(const char *conf_file, const char **groups, int *argc,
     return error;
   }
 
-  if (my_defaults_read_login_file && !found_no_login_paths) {
+  if (my_defaults_read_login_file) {
     /* Read options from login group. */
     if (my_default_get_login_file(my_login_file, sizeof(my_login_file)) &&
         (error = my_search_option_files(my_login_file, argc, argv, &args_used,
                                         handle_default_option, (void *)&ctx,
                                         dirs, true, found_no_defaults))) {
-      alloc->Clear();
+      free_root(alloc, MYF(0));
       return error;
     }
   }
@@ -781,7 +734,7 @@ int my_load_defaults(const char *conf_file, const char **groups, int *argc,
   (*argv) += args_used;
 
   /*
-    Check if we want to see the new argument list
+    Check if we wan't to see the new argument list
     This options must always be the last of the default options
   */
   if (*argc >= 2 && !strcmp(argv[0][1], "--print-defaults")) {
@@ -838,14 +791,14 @@ static int search_default_file(Process_option_func opt_handler,
                                const char *config_file, bool is_login_file) {
   const char **ext;
   const char *empty_list[] = {"", nullptr};
-  const bool have_ext = fn_ext(config_file)[0] != 0;
+  bool have_ext = fn_ext(config_file)[0] != 0;
   const char **exts_to_use = have_ext ? empty_list : f_extensions;
 
   for (ext = exts_to_use; *ext; ext++) {
     int error;
-    if ((error = search_default_file_with_ext(opt_handler, handler_ctx, dir,
-                                              *ext, config_file, 0,
-                                              is_login_file, false)) < 0)
+    if ((error =
+             search_default_file_with_ext(opt_handler, handler_ctx, dir, *ext,
+                                          config_file, 0, is_login_file)) < 0)
       return error;
   }
   return 0;
@@ -858,7 +811,7 @@ static int search_default_file(Process_option_func opt_handler,
    get_argument()
    keyword		Include directive keyword
    kwlen		Length of keyword
-   ptr			Pointer to the keyword in the line under process
+   ptr			Pointer to the keword in the line under process
    line			line number
 
   RETURN
@@ -910,7 +863,6 @@ static char *get_argument(const char *keyword, size_t kwlen, char *ptr,
     recursion_level             the level of recursion, got while processing
                                 "!include" or "!includedir"
     is_login_file               TRUE, when login file is being processed.
-    report_os_error_on_open     true when OS error is to be reported
 
   RETURN
     0   Success
@@ -922,11 +874,11 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
                                         void *handler_ctx, const char *dir,
                                         const char *ext,
                                         const char *config_file,
-                                        int recursion_level, bool is_login_file,
-                                        bool report_os_error_on_open) {
+                                        int recursion_level,
+                                        bool is_login_file) {
   char name[FN_REFLEN + 10], buff[4096], curr_gr[4096], *ptr, *end;
   const char **tmp_ext;
-  char *value, tmp[FN_REFLEN];
+  char *value, option[4096 + 2], tmp[FN_REFLEN];
   static const char includedir_keyword[] = "includedir";
   static const char include_keyword[] = "include";
   const int max_recursion_level = 10;
@@ -936,7 +888,6 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
   uint i, rc;
   MY_DIR *search_dir;
   FILEINFO *search_file;
-  const myf flags = MYF(report_os_error_on_open ? MY_WME : 0);
 
   if ((dir ? strlen(dir) : 0) + strlen(config_file) >= FN_REFLEN - 3)
     return 0; /* Ignore wrong paths */
@@ -950,26 +901,21 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
   }
   fn_format(name, name, "", "", MY_UNPACK_FILENAME);
 
-  if ((rc = check_file_permissions(name, is_login_file, flags)) < 2)
-    return (int)rc;
+  if ((rc = check_file_permissions(name, is_login_file)) < 2) return (int)rc;
 
   if (is_login_file) {
     if (!(fp = mysql_file_fopen(key_file_cnf, name, O_RDONLY | MY_FOPEN_BINARY,
-                                flags)))
+                                MYF(0))))
       return 1; /* Ignore wrong files. */
   } else {
-    if (!(fp = mysql_file_fopen(key_file_cnf, name, O_RDONLY, flags)))
+    if (!(fp = mysql_file_fopen(key_file_cnf, name, O_RDONLY, MYF(0))))
       return 1; /* Ignore wrong files */
   }
 
-  while (true) {
-    auto fileline = mysql_file_getline(buff, sizeof(buff), fp, is_login_file);
-    char *linebuff = fileline.get();
-    if (linebuff == nullptr) break;
-
+  while (mysql_file_getline(buff, sizeof(buff) - 1, fp, is_login_file)) {
     line++;
     /* Ignore comment and empty lines */
-    for (ptr = linebuff; my_isspace(&my_charset_latin1, *ptr); ptr++) {
+    for (ptr = buff; my_isspace(&my_charset_latin1, *ptr); ptr++) {
     }
 
     if (*ptr == '#' || *ptr == ';' || !*ptr) continue;
@@ -997,11 +943,7 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
                                  ptr, name, line)))
           goto err;
 
-        if (!(search_dir = my_dir(ptr, MYF(MY_WME)))) {
-          my_message_local(ERROR_LEVEL, EE_FAILED_PROCESSING_DIRECTIVE,
-                           includedir_keyword, name, line);
-          goto err;
-        }
+        if (!(search_dir = my_dir(ptr, MYF(MY_WME)))) goto err;
 
         for (i = 0; i < search_dir->number_off_files; i++) {
           search_file = search_dir->dir_entry + i;
@@ -1028,11 +970,9 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
             */
             if (it != default_paths.end()) default_paths[tmp] = it->second;
 
-            if (search_default_file_with_ext(opt_handler, handler_ctx, nullptr,
-                                             nullptr, tmp, recursion_level + 1,
-                                             is_login_file, true))
-              my_message_local(ERROR_LEVEL, EE_FAILED_PROCESSING_DIRECTIVE,
-                               includedir_keyword, name, line);
+            search_default_file_with_ext(opt_handler, handler_ctx, nullptr,
+                                         nullptr, tmp, recursion_level + 1,
+                                         is_login_file);
           }
         }
 
@@ -1059,11 +999,8 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
             fn_format(tmp, ptr, "", "", MY_UNPACK_FILENAME | MY_SAFE_PATH))
           default_paths[tmp] = it->second;
 
-        if (search_default_file_with_ext(opt_handler, handler_ctx, nullptr,
-                                         nullptr, ptr, recursion_level + 1,
-                                         is_login_file, true))
-          my_message_local(ERROR_LEVEL, EE_FAILED_PROCESSING_DIRECTIVE,
-                           include_keyword, name, line);
+        search_default_file_with_ext(opt_handler, handler_ctx, nullptr, nullptr,
+                                     ptr, recursion_level + 1, is_login_file);
       }
 
       continue;
@@ -1108,10 +1045,6 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
     for (; my_isspace(&my_charset_latin1, end[-1]); end--) {
     }
 
-    /* Self freeing option buffer */
-    std::unique_ptr<char[]> optionBuffer{new char[strlen(linebuff) + 3]};
-    char *option = optionBuffer.get();
-
     if (!value) {
       strmake(my_stpcpy(option, "--"), ptr, (size_t)(end - ptr));
       if (opt_handler(handler_ctx, curr_gr, option, name)) goto err;
@@ -1212,110 +1145,61 @@ static char *remove_end_comment(char *ptr) {
 }
 
 /**
-  Read one line from the specified file.
+  Read one line from the specified file. In case
+  of scrambled login file, the line read is first
+  decrypted and then returned.
 
-  In case of scrambled login file, the line read is first decrypted and then
-  returned.
-
-  @param [out] buff          Static buffer to store the read line.
-                             If the buffer is not enough, this function will
-                             allocate a dynamic buffer from heap.
+  @param [out] str           Buffer to store the read text.
   @param [in] size           At max, size-1 bytes to be read.
   @param [in] file           Source file.
   @param [in] is_login_file  TRUE, when login file is being processed.
 
-  @return nullptr                                On Error
-          The next line from the supplied file   On Success
+  @return 1               Success
+          0               Error
 */
-static mysql_file_getline_ret mysql_file_getline(char *buff, int size,
-                                                 MYSQL_FILE *file,
-                                                 bool is_login_file) {
+
+static bool mysql_file_getline(char *str, int size, MYSQL_FILE *file,
+                               bool is_login_file) {
   uchar cipher[4096], len_buf[MAX_CIPHER_STORE_LEN];
   static unsigned char my_key[LOGIN_KEY_LEN];
   int length = 0, cipher_len = 0;
 
-  /*
-    If the supplied buff/size is enough to store the line, then we return the
-    buff itself. In this case, we use this noop deleter
-  */
-  static auto noop_free = [](void *) noexcept {};
-
   if (is_login_file) {
     if (mysql_file_ftell(file) == 0) {
       /* Move past unused bytes. */
       mysql_file_fseek(file, 4, SEEK_SET);
       if (mysql_file_fread(file, my_key, LOGIN_KEY_LEN, MYF(MY_WME)) !=
           LOGIN_KEY_LEN)
-        return {nullptr, noop_free};
+        return false;
     }
+
     if (mysql_file_fread(file, len_buf, MAX_CIPHER_STORE_LEN, MYF(MY_WME)) ==
         MAX_CIPHER_STORE_LEN) {
       cipher_len = sint4korr(len_buf);
-      /*
-        This is safe to do since mysql_config_editor never writes
-        lines longer than 4k
-      */
-      if (cipher_len > size) return {nullptr, noop_free};
+      if (cipher_len > size) return false;
     } else
-      return {nullptr, noop_free};
+      return false;
+
     mysql_file_fread(file, cipher, cipher_len, MYF(MY_WME));
     if ((length =
-             my_aes_decrypt(cipher, cipher_len, (unsigned char *)buff, my_key,
+             my_aes_decrypt(cipher, cipher_len, (unsigned char *)str, my_key,
                             LOGIN_KEY_LEN, my_aes_128_ecb, nullptr)) < 0) {
       /* Attempt to decrypt failed. */
-      return {nullptr, noop_free};
+      return false;
     }
-    buff[length] = 0;
-    return {buff, noop_free};
+    str[length] = 0;
+    return true;
   } else {
-    mysql_file_getline_ret line{nullptr, noop_free}; /* The output line */
-    size_t lineLen = 0;                              /* Cached length of line */
-
-    while (true) {
-      /* Read up to size bytes */
-      if (mysql_file_fgets(buff, size, file) == nullptr) {
-        /* End of file */
-        return line;
-      }
-
-      /* Calculate size of line, including null termination */
-      const size_t buffLen = strlen(buff);
-
-      /* Check if the provided buff is enough for the line */
-      if (lineLen == 0 && buff[buffLen - 1] == '\n') {
-        return {buff, noop_free};
-      }
-
-      if (buffLen == 0) return line;
-
-      lineLen += buffLen;
-
-      /* Allocate the line buffer */
-      char *l = static_cast<char *>(malloc(lineLen + 1));
-      if (l == nullptr) {
-        /* malloc failed */
-        return {nullptr, noop_free};
-      }
-
-      if (line.get() != nullptr) {
-        /* Append new output of fgets to existing line */
-        sprintf(l, "%s%s", line.get(), buff);
-      } else {
-        sprintf(l, "%s", buff);
-      }
-      line = {l, std::free};
-
-      /* Check if we reached the end of the line */
-      if (buff[buffLen - 1] == '\n') {
-        return line;
-      }
-    }
+    if (mysql_file_fgets(str, size, file))
+      return true;
+    else
+      return false;
   }
 }
 
 void my_print_default_files(const char *conf_file) {
   const char *empty_list[] = {"", nullptr};
-  const bool have_ext = fn_ext(conf_file)[0] != 0;
+  bool have_ext = fn_ext(conf_file)[0] != 0;
   const char **exts_to_use = have_ext ? empty_list : f_extensions;
   char name[FN_REFLEN];
   const char **ext;
@@ -1328,7 +1212,8 @@ void my_print_default_files(const char *conf_file) {
     fputs(conf_file, stdout);
   else {
     const char **dirs;
-    MEM_ROOT alloc(key_memory_defaults, 512);
+    MEM_ROOT alloc;
+    init_alloc_root(key_memory_defaults, &alloc, 512, 0);
 
     if ((dirs = init_default_directories(&alloc)) == nullptr) {
       fputs("Internal error initializing default directories list", stdout);
@@ -1356,7 +1241,7 @@ void my_print_default_files(const char *conf_file) {
       }
     }
 
-    alloc.Clear();
+    free_root(&alloc, MYF(0));
   }
   puts("");
 }
@@ -1388,8 +1273,7 @@ void print_defaults(const char *conf_file, const char **groups) {
 --defaults-extra-file=# Read this file after the global files are read.\n\
 --defaults-group-suffix=#\n\
                         Also read groups with concat(group, suffix)\n\
---login-path=#          Read this path from the login file.\n\
---no-login-paths        Don't read login paths from the login path file.");
+--login-path=#          Read this path from the login file.");
 }
 
 /**
@@ -1418,8 +1302,8 @@ void init_variable_default_paths() {
   }
   default_paths["C:\\my.ini"] = enum_variable_source::GLOBAL;
   default_paths["C:\\my.cnf"] = enum_variable_source::GLOBAL;
-  if (GetModuleFileName(nullptr, buffer, (DWORD)sizeof(buffer))) {
-    char *end = strend(buffer), *last = nullptr;
+  if (GetModuleFileName(NULL, buffer, (DWORD)sizeof(buffer))) {
+    char *end = strend(buffer), *last = NULL;
     for (; end > buffer; end--) {
       if (*end == FN_LIBCHAR) {
         if (last) {
@@ -1478,7 +1362,7 @@ void init_variable_default_paths() {
   /etc/my.cnf has max_connections
   /$datadir/mysqld.auto.cnf has max_user_connections
   ./mysqld --server-id=47
-  with this setup, variables_hash has 3 entries of the above options
+  with this setup, variables_hash has 3 entires of the above options
   along with path of config files and its enum value which is as below:
   max_connections -> (/etc/my.cnf , enum_variable_source::GLOBAL)
   max_user_connections -> ($datadir/mysqld.auto.cnf ,
@@ -1491,10 +1375,9 @@ void init_variable_default_paths() {
 */
 void update_variable_source(const char *opt_name, const char *value) {
   string var_name = string(opt_name);
-  const string path = (value ? value : string(""));
-  const string prefix[] = {"loose_", "disable_", "enable_", "maximum_",
-                           "skip_"};
-  const uint prefix_count = sizeof(prefix) / sizeof(prefix[0]);
+  string path = (value ? value : string(""));
+  string prefix[] = {"loose_", "disable_", "enable_", "maximum_", "skip_"};
+  uint prefix_count = sizeof(prefix) / sizeof(prefix[0]);
 
   /* opt_name must be of form --XXXXX which means min length must be 3 */
   if (var_name.length() < 3) return;
@@ -1522,9 +1405,9 @@ void update_variable_source(const char *opt_name, const char *value) {
       /* check if variables are prefixed with skip_ */
       if (id == 4) {
         bool skip_variable = false;
-        const string skip_variables[] = {
-            "skip_name_resolve",     "skip_networking",  "skip_show_database",
-            "skip_external_locking", "skip_slave_start", "skip_replica_start"};
+        string skip_variables[] = {"skip_name_resolve", "skip_networking",
+                                   "skip_show_database",
+                                   "skip_external_locking"};
         for (uint skip_index = 0;
              skip_index < sizeof(skip_variables) / sizeof(skip_variables[0]);
              ++skip_index) {
@@ -1598,13 +1481,13 @@ static int add_directory(MEM_ROOT *alloc, const char *dir, const char **dirs) {
   char buf[FN_REFLEN];
   size_t len;
   char *p;
-  bool err [[maybe_unused]];
+  bool err MY_ATTRIBUTE((unused));
 
   len = normalize_dirname(buf, dir);
   if (!(p = strmake_root(alloc, buf, len))) return 1; /* Failure */
   /* Should never fail if DEFAULT_DIRS_SIZE is correct size */
   err = array_append_string_unique(p, dirs, DEFAULT_DIRS_SIZE);
-  assert(err == false);
+  DBUG_ASSERT(err == false);
 
   return 0;
 }
@@ -1641,9 +1524,9 @@ static size_t my_get_system_windows_directory(char *buffer, size_t size) {
 }
 
 static const char *my_get_module_parent(char *buf, size_t size) {
-  char *last = nullptr;
+  char *last = NULL;
   char *end;
-  if (!GetModuleFileName(nullptr, buf, (DWORD)size)) return nullptr;
+  if (!GetModuleFileName(NULL, buf, (DWORD)size)) return NULL;
   end = strend(buf);
 
   /*
@@ -1687,7 +1570,7 @@ static const char **init_default_directories(MEM_ROOT *alloc) {
 
     errors += add_directory(alloc, "C:/", dirs);
 
-    if (my_get_module_parent(fname_buffer, sizeof(fname_buffer)) != nullptr)
+    if (my_get_module_parent(fname_buffer, sizeof(fname_buffer)) != NULL)
       errors += add_directory(alloc, fname_buffer, dirs);
   }
 
@@ -1754,18 +1637,16 @@ int my_default_get_login_file(char *file_name, size_t file_name_size) {
 
   @param [in] file_name        Name of the option file.
   @param [in] is_login_file    TRUE, when login file is being processed.
-  @param [in] flags            error handling flags
 
   @return  0 - Non-allowable file permissions.
            1 - Failed to stat.
            2 - Success.
 */
-int check_file_permissions(const char *file_name, bool is_login_file,
-                           myf flags) {
+int check_file_permissions(const char *file_name, bool is_login_file) {
 #if !defined(_WIN32)
   MY_STAT stat_info;
 
-  if (!my_stat(file_name, &stat_info, flags)) return 1;
+  if (!my_stat(file_name, &stat_info, MYF(0))) return 1;
   /*
     Ignore .mylogin.cnf file if not exclusively readable/writable
     by current user.
@@ -1788,10 +1669,6 @@ int check_file_permissions(const char *file_name, bool is_login_file,
                      file_name);
     return 0;
   }
-#else
-  (void)file_name;      // maybe_unused
-  (void)is_login_file;  // maybe_unused
-  (void)flags;          // maybe_unused
 #endif
   return 2; /* Success */
 }
