@@ -1,4 +1,4 @@
-/* Copyright (c) 2002, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -61,16 +61,17 @@ typedef void (*init_func_p)(const struct my_option *option, void *variable,
 my_error_reporter my_getopt_error_reporter = &my_message_local;
 
 static bool getopt_compare_strings(const char *, const char *, uint);
-static longlong getopt_ll(const char *, bool, const my_option *, int *);
-static ulonglong getopt_ull(const char *, bool, const my_option *, int *);
-static double getopt_double(const char *, bool, const my_option *, int *);
+static longlong getopt_ll(const char *arg, const struct my_option *optp,
+                          int *err);
+static ulonglong getopt_ull(const char *, const struct my_option *, int *);
+static double getopt_double(const char *arg, const struct my_option *optp,
+                            int *err);
 static void init_variables(const struct my_option *, init_func_p);
 static void init_one_value(const struct my_option *, void *, longlong);
 static void fini_one_value(const struct my_option *, void *, longlong);
-static int setval(const struct my_option *, void *, const char *, bool, bool);
+static int setval(const struct my_option *, void *, const char *, bool);
 static void setval_source(const struct my_option *, void *);
 static char *check_struct_option(char *cur_arg, char *key_name);
-static bool get_bool_int_argument(const char *argument, bool *error);
 
 /*
   The following three variables belong to same group and the number and
@@ -112,16 +113,13 @@ void my_getopt_register_get_addr(my_getopt_value func_addr) {
   getopt_get_addr = func_addr;
 }
 
-bool is_key_cache_variable_suffix(std::string_view suffix) {
-  constexpr static std::array<std::string_view, 4> key_cache_components = {
+bool is_key_cache_variable_suffix(const char *suffix) {
+  static std::array<const char *, 4> key_cache_components = {
       {"key_buffer_size", "key_cache_block_size", "key_cache_division_limit",
        "key_cache_age_threshold"}};
 
-  for (auto component : key_cache_components) {
-    if (suffix.size() == component.size() &&
-        !native_strncasecmp(suffix.data(), component.data(), suffix.size()))
-      return true;
-  }
+  for (auto component : key_cache_components)
+    if (!my_strcasecmp(&my_charset_latin1, component, suffix)) return true;
 
   return false;
 }
@@ -236,34 +234,16 @@ double getopt_ulonglong2double(ulonglong v) {
   @param [in] ignore_unknown_option When set to true, options are continued to
                                     be read even when unknown options are
                                     encountered.
-  @param [in] boolean_as_int        Parse boolean as integer value.
-                                    Mimic the logic of parsing booleans at
-  runtime: Instead of parsing
 
   @param [in] validate_only When set to true, options values are not assinged.
-  @code
-  bool_val = (str_val == '1' || str_val = 'ON') ?
-                true :
-                (str_val == '0' || str_val = 'OFF') ?
-                  false :
-                  false;
-  @endcode
-  do:
-  @code
-  bool_val == (str_val == 'OFF') ?
-                false :
-                ((str_val == 'ON') ?
-                  true :
-                  atoi(str_val) != 0)
-  @endcode
+
   @return error in case of ambiguous or unknown options,
           0 on success.
 */
-int my_handle_options2(int *argc, char ***argv,
-                       const struct my_option *longopts,
-                       my_get_one_option get_one_option,
-                       const char **command_list, bool ignore_unknown_option,
-                       bool boolean_as_int, bool validate_only) {
+int my_handle_options(int *argc, char ***argv, const struct my_option *longopts,
+                      my_get_one_option get_one_option,
+                      const char **command_list, bool ignore_unknown_option,
+                      bool validate_only) {
   uint argvpos = 0, length;
   bool end_of_options = false, must_be_var, set_maximum_value, option_is_loose;
   char **pos, **pos_end, *optend, *opt_str, key_name[FN_REFLEN];
@@ -274,8 +254,8 @@ int my_handle_options2(int *argc, char ***argv,
   int opt_found;
 
   /* handle_options() assumes arg0 (program name) always exists */
-  assert(argc && *argc >= 1);
-  assert(argv && *argv);
+  DBUG_ASSERT(argc && *argc >= 1);
+  DBUG_ASSERT(argv && *argv);
   (*argc)--; /* Skip the program name */
   (*argv)++; /*      --- || ----      */
   if (!validate_only) init_variables(longopts, init_one_value);
@@ -336,7 +316,7 @@ int my_handle_options2(int *argc, char ***argv,
     if (!is_cmdline_arg && (my_getopt_is_args_separator(cur_arg))) {
       is_cmdline_arg = true;
 
-      /* save the separator too if skip unknown options  */
+      /* save the separator too if skip unkown options  */
       if (my_getopt_skip_unknown)
         (*argv)[argvpos++] = cur_arg;
       else
@@ -388,10 +368,7 @@ int my_handle_options2(int *argc, char ***argv,
 
           if (!is_key_cache_variable_suffix(tmp_name.c_str())) {
             opt_str = cur_arg;
-            if (optend)
-              length = (uint)((optend - opt_str) - 1);
-            else
-              length = strlen(opt_str);
+            length = (uint)((optend - opt_str) - 1);
           }
         }
         /*
@@ -421,7 +398,7 @@ int my_handle_options2(int *argc, char ***argv,
                 if ((opt_found = findopt(opt_str, length, &optp))) {
                   switch (i) {
                     case OPT_SKIP:
-                    case OPT_DISABLE:
+                    case OPT_DISABLE: /* fall through */
                       /*
                         double negation is actually enable again,
                         for example: --skip-option=0 -> option = true
@@ -516,8 +493,7 @@ int my_handle_options2(int *argc, char ***argv,
             } else {
               bool ret = false;
               bool error = false;
-              ret = boolean_as_int ? get_bool_int_argument(optend, &error)
-                                   : get_bool_argument(optend, &error);
+              ret = get_bool_argument(optend, &error);
               if (error) {
                 my_getopt_error_reporter(WARNING_LEVEL,
                                          EE_OPTION_IGNORED_DUE_TO_INVALID_VALUE,
@@ -608,9 +584,8 @@ int my_handle_options2(int *argc, char ***argv,
                 }
               }
               int error;
-              if (!validate_only &&
-                  (error = setval(optp, optp->value, argument,
-                                  set_maximum_value, boolean_as_int)))
+              if (!validate_only && (error = setval(optp, optp->value, argument,
+                                                    set_maximum_value)))
                 return error;
               if (get_one_option && get_one_option(optp->id, optp, argument))
                 return EXIT_UNSPECIFIED_ERROR;
@@ -626,10 +601,10 @@ int my_handle_options2(int *argc, char ***argv,
                 Hack the string "-XYZ" to make a "-YZ" substring in it,
                 and push that to the output as an unrecognized parameter.
               */
-              assert(optend > *pos);
-              assert(optend >= cur_arg);
-              assert(optend <= *pos + strlen(*pos));
-              assert(*optend);
+              DBUG_ASSERT(optend > *pos);
+              DBUG_ASSERT(optend >= cur_arg);
+              DBUG_ASSERT(optend <= *pos + strlen(*pos));
+              DBUG_ASSERT(*optend);
               optend--;
               optend[0] = '-'; /* replace 'X' or '-' by '-' */
               (*argv)[argvpos++] = optend;
@@ -651,8 +626,8 @@ int my_handle_options2(int *argc, char ***argv,
         continue;
       }
       int error;
-      if (!validate_only && (error = setval(optp, value, argument,
-                                            set_maximum_value, boolean_as_int)))
+      if (!validate_only &&
+          (error = setval(optp, value, argument, set_maximum_value)))
         return error;
       if (get_one_option && get_one_option(optp->id, optp, argument))
         return EXIT_UNSPECIFIED_ERROR;
@@ -688,14 +663,6 @@ done:
   return 0;
 }
 
-int my_handle_options(int *argc, char ***argv, const struct my_option *longopts,
-                      my_get_one_option get_one_option,
-                      const char **command_list, bool ignore_unknown_option,
-                      bool validate_only) {
-  return my_handle_options2(argc, argv, longopts, get_one_option, command_list,
-                            ignore_unknown_option, false, validate_only);
-}
-
 /**
  * This function should be called to print a warning message
  * if password string is specified on the command line.
@@ -776,35 +743,6 @@ bool get_bool_argument(const char *argument, bool *error) {
   return false;
 }
 
-/**
-   Parse a boolean command line argument as the SQL interpreter does
-
-   "ON" and "TRUE" will return true,
-   "OFF" and FALSE" will return false;
-
-   Non-zero numeric values will return true, zero will return false.
-
-   @param argument The value argument
-   @param [out] error Error indicator
-   @return boolean value
-*/
-
-static bool get_bool_int_argument(const char *argument, bool *error) {
-  if (!my_strcasecmp(&my_charset_latin1, argument, "true") ||
-      !my_strcasecmp(&my_charset_latin1, argument, "on"))
-    return true;
-
-  if (!my_strcasecmp(&my_charset_latin1, argument, "false") ||
-      !my_strcasecmp(&my_charset_latin1, argument, "off"))
-    return false;
-
-  if (!strchr("0123456789+-", argument[0])) {
-    *error = true;
-    return false;
-  }
-  return atoi(argument) != 0;
-}
-
 /**
   Will set the source and file name from where this options is set in
   my_option struct.
@@ -820,8 +758,7 @@ static void setval_source(const struct my_option *opts, void *value) {
 */
 
 static int setval(const struct my_option *opts, void *value,
-                  const char *argument, bool set_maximum_value,
-                  bool boolean_as_int) {
+                  const char *argument, bool set_maximum_value) {
   int err = 0, res = 0;
   ulong var_type = opts->var_type & GET_TYPE_MASK;
 
@@ -859,41 +796,32 @@ static int setval(const struct my_option *opts, void *value,
     switch (var_type) {
       case GET_BOOL: /* If argument differs from 0, enable option, else disable
                       */
-        *((bool *)value) = boolean_as_int
-                               ? get_bool_int_argument(argument, &error)
-                               : get_bool_argument(argument, &error);
+        *((bool *)value) = get_bool_argument(argument, &error);
         if (error)
           my_getopt_error_reporter(WARNING_LEVEL,
                                    EE_INCORRECT_BOOLEAN_VALUE_FOR_OPTION,
                                    opts->name, argument);
         break;
       case GET_INT:
-        *((int *)value) =
-            (int)getopt_ll(argument, set_maximum_value, opts, &err);
+        *((int *)value) = (int)getopt_ll(argument, opts, &err);
         break;
       case GET_UINT:
-        *((uint *)value) =
-            (uint)getopt_ull(argument, set_maximum_value, opts, &err);
+        *((uint *)value) = (uint)getopt_ull(argument, opts, &err);
         break;
       case GET_LONG:
-        *((long *)value) =
-            (long)getopt_ll(argument, set_maximum_value, opts, &err);
+        *((long *)value) = (long)getopt_ll(argument, opts, &err);
         break;
       case GET_ULONG:
-        *((long *)value) =
-            (long)getopt_ull(argument, set_maximum_value, opts, &err);
+        *((long *)value) = (long)getopt_ull(argument, opts, &err);
         break;
       case GET_LL:
-        *((longlong *)value) =
-            getopt_ll(argument, set_maximum_value, opts, &err);
+        *((longlong *)value) = getopt_ll(argument, opts, &err);
         break;
       case GET_ULL:
-        *((ulonglong *)value) =
-            getopt_ull(argument, set_maximum_value, opts, &err);
+        *((ulonglong *)value) = getopt_ull(argument, opts, &err);
         break;
       case GET_DOUBLE:
-        *((double *)value) =
-            getopt_double(argument, set_maximum_value, opts, &err);
+        *((double *)value) = getopt_double(argument, opts, &err);
         break;
       case GET_STR:
       case GET_PASSWORD:
@@ -1146,13 +1074,9 @@ template ulonglong eval_num_suffix<ulonglong>(const char *, int *,
   In case of an error, set error value in *err.
 */
 
-static longlong getopt_ll(const char *arg, bool set_maximum_value,
-                          const my_option *optp, int *err) {
+static longlong getopt_ll(const char *arg, const struct my_option *optp,
+                          int *err) {
   longlong num = eval_num_suffix<longlong>(arg, err, optp->name);
-  if (set_maximum_value && *err == 0 &&
-      *static_cast<longlong *>(optp->value) > num) {
-    *static_cast<longlong *>(optp->value) = num;
-  }
   return getopt_ll_limit_value(num, optp, nullptr);
 }
 
@@ -1176,7 +1100,7 @@ ulonglong max_of_int_range(int var_type) {
     case GET_ULL:
       return ULLONG_MAX;
     default:
-      assert(0);
+      DBUG_ASSERT(0);
       return 0;
   }
 }
@@ -1238,8 +1162,8 @@ static inline bool is_negative_num(const char *num) {
   values.
 */
 
-static ulonglong getopt_ull(const char *arg, bool set_maximum_value,
-                            const my_option *optp, int *err) {
+static ulonglong getopt_ull(const char *arg, const struct my_option *optp,
+                            int *err) {
   char buf[255];
   ulonglong num;
 
@@ -1252,10 +1176,6 @@ static ulonglong getopt_ull(const char *arg, bool set_maximum_value,
   } else
     num = eval_num_suffix<ulonglong>(arg, err, optp->name);
 
-  if (set_maximum_value && *err == 0 &&
-      *static_cast<ulonglong *>(optp->value) > num) {
-    *static_cast<ulonglong *>(optp->value) = num;
-  }
   return getopt_ull_limit_value(num, optp, nullptr);
 }
 
@@ -1324,7 +1244,7 @@ double getopt_double_limit_value(double num, const struct my_option *optp,
 }
 
 /*
-  Get double value within ranges
+  Get double value withing ranges
 
   Evaluates and returns the value that user gave as an argument to a variable.
 
@@ -1332,11 +1252,11 @@ double getopt_double_limit_value(double num, const struct my_option *optp,
     decimal value of arg
 
     In case of an error, prints an error message and sets *err to
-    EXIT_ARGUMENT_INVALID.  Otherwise err is not touched.
+    EXIT_ARGUMENT_INVALID.  Otherwise err is not touched
 */
 
-static double getopt_double(const char *arg, bool set_maximum_value,
-                            const my_option *optp, int *err) {
+static double getopt_double(const char *arg, const struct my_option *optp,
+                            int *err) {
   double num;
   int error;
   const char *end = arg + 1000; /* Big enough as *arg is \0 terminated */
@@ -1347,10 +1267,6 @@ static double getopt_double(const char *arg, bool set_maximum_value,
     *err = EXIT_ARGUMENT_INVALID;
     return 0.0;
   }
-  if (set_maximum_value && *err == 0 &&
-      *static_cast<double *>(optp->value) > num) {
-    *static_cast<double *>(optp->value) = num;
-  }
   return getopt_double_limit_value(num, optp, nullptr);
 }
 
@@ -1442,7 +1358,7 @@ static void init_one_value(const struct my_option *option, void *variable,
 */
 
 static void fini_one_value(const struct my_option *option, void *variable,
-                           longlong value [[maybe_unused]]) {
+                           longlong value MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   switch ((option->var_type & GET_TYPE_MASK)) {
     case GET_STR_ALLOC:
@@ -1643,7 +1559,7 @@ void my_print_variables_ex(const struct my_option *options, FILE *file) {
           break;
         case GET_STR:
         case GET_PASSWORD:
-        case GET_STR_ALLOC:
+        case GET_STR_ALLOC: /* fall through */
           fprintf(file, "%s\n",
                   *((char **)value) ? *((char **)value) : "(No default value)");
           break;
