@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -249,40 +249,20 @@ Voluntary context switches %ld, Involuntary context switches %ld\n",
   my_init_done = false;
 } /* my_end */
 
-/**
-  Pointer to function that handles abort. It is the std's abort() by default.
-*/
-static void (*my_abort_func)() = abort;
-
-[[noreturn]] void my_abort() {
-  my_abort_func();
-  /*
-    We should not reach here, it is required only because my_abort_func() is
-    not [[noreturn]].
-  */
-  abort();
-}
-
-void set_my_abort(void (*new_my_abort_func)()) {
-  my_abort_func = new_my_abort_func;
-}
-
 #ifdef _WIN32
 /*
   my_parameter_handler
 
   Invalid parameter handler we will use instead of the one "baked"
   into the CRT for Visual Studio.
-  The assert will catch things typically *not* caught by sanitizers,
+  The DBUG_ASSERT will catch things typically *not* caught by sanitizers,
   e.g. iterator out-of-range, but pointing to valid memory.
 */
 
-void my_parameter_handler(const wchar_t *expression [[maybe_unused]],
-                          const wchar_t *function [[maybe_unused]],
-                          const wchar_t *file [[maybe_unused]],
-                          unsigned int line [[maybe_unused]],
-                          uintptr_t pReserved [[maybe_unused]]) {
-#ifndef NDEBUG
+void my_parameter_handler(const wchar_t *expression, const wchar_t *function,
+                          const wchar_t *file, unsigned int line,
+                          uintptr_t pReserved) {
+#ifndef DBUG_OFF
   fprintf(stderr,
           "my_parameter_handler errno %d "
           "expression: %ws  function: %ws  file: %ws, line: %d\n",
@@ -292,7 +272,7 @@ void my_parameter_handler(const wchar_t *expression [[maybe_unused]],
   //   DBUG_EXECUTE_IF("ib_export_io_write_failure_1", close(fileno(file)););
   // So ignore EBADF
   if (errno != EBADF) {
-    assert(false);
+    DBUG_ASSERT(false);
   }
 #endif
 }
@@ -569,14 +549,14 @@ static PSI_memory_info all_mysys_memory[] = {
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
 static PSI_thread_info all_mysys_thread[] = {
-    {&key_thread_timer_notifier, "thread_timer_notifier", "timer_notifier",
-     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME}};
+    {&key_thread_timer_notifier, "thread_timer_notifier", PSI_FLAG_SINGLETON, 0,
+     PSI_DOCUMENT_ME}};
 #endif /* HAVE_PSI_THREAD_INTERFACE */
 
 #ifdef HAVE_PSI_INTERFACE
 void my_init_mysys_psi_keys() {
-  const char *category [[maybe_unused]] = "mysys";
-  int count [[maybe_unused]];
+  const char *category MY_ATTRIBUTE((unused)) = "mysys";
+  int count MY_ATTRIBUTE((unused));
 
 #ifdef HAVE_PSI_MUTEX_INTERFACE
   count = sizeof(all_mysys_mutexes) / sizeof(all_mysys_mutexes[0]);
