@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2011, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -49,6 +49,7 @@
 #define HAVE_PSI_METADATA_INTERFACE
 #define HAVE_PSI_DATA_LOCK_INTERFACE
 #define HAVE_PSI_SYSTEM_INTERFACE
+#define HAVE_PSI_TLS_CHANNEL_INTERFACE
 
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
@@ -62,7 +63,7 @@
 #include "my_macros.h"
 #include "my_sys.h"  // IWYU pragma: keep
 #include "my_thread.h"
-#include "mysql/psi/psi_base.h"
+#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/psi/psi_cond.h"
 #include "mysql/psi/psi_data_lock.h"
 #include "mysql/psi/psi_error.h"
@@ -78,6 +79,7 @@
 #include "mysql/psi/psi_system.h"
 #include "mysql/psi/psi_table.h"
 #include "mysql/psi/psi_thread.h"
+#include "mysql/psi/psi_tls_channel.h"
 #include "mysql/psi/psi_transaction.h"
 
 class THD;
@@ -131,6 +133,8 @@ static void set_thread_info_noop(const char *, uint) { return; }
 
 static void set_thread_noop(PSI_thread *) { return; }
 
+static void set_thread_peer_port_noop(PSI_thread *, uint) { return; }
+
 static int set_thread_resource_group_noop(const char *, int, void *) {
   return 0;
 }
@@ -206,6 +210,7 @@ static PSI_thread_service_t psi_thread_noop = {
     set_thread_resource_group_noop,
     set_thread_resource_group_by_id_noop,
     set_thread_noop,
+    set_thread_peer_port_noop,
     aggregate_thread_status_noop,
     delete_current_thread_noop,
     delete_thread_noop,
@@ -894,7 +899,7 @@ static PSI_memory_key memory_realloc_noop(PSI_memory_key, size_t, size_t,
 }
 
 static PSI_memory_key memory_claim_noop(PSI_memory_key, size_t,
-                                        struct PSI_thread **owner) {
+                                        struct PSI_thread **owner, bool) {
   *owner = nullptr;
   return PSI_NOT_INSTRUMENTED;
 }
@@ -946,3 +951,22 @@ PSI_system_service_t *psi_system_service = &psi_system_noop;
 void set_psi_system_service(void *psi) {
   psi_system_service = (PSI_system_service_t *)psi;
 }
+
+// ===========================================================================
+
+static void register_tls_channel_noop(TLS_channel_property_iterator *) {
+  return;
+}
+
+static void unregister_tls_channel_noop(TLS_channel_property_iterator *) {
+  return;
+}
+static PSI_tls_channel_service_t psi_tls_channel_noop = {
+    register_tls_channel_noop, unregister_tls_channel_noop};
+
+struct PSI_tls_channel_bootstrap *psi_tls_channel_hook = nullptr;
+PSI_tls_channel_service_t *psi_tls_channel_service = &psi_tls_channel_noop;
+
+void set_psi_tls_channel_service(void *psi) {
+  psi_tls_channel_service = (PSI_tls_channel_service_t *)psi;
+}
