@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -88,7 +87,7 @@ int Local::clone_exec() {
   auto thd = m_clone_client.get_thd();
   auto dir_name = m_clone_client.get_data_dir();
   auto is_master = m_clone_client.is_master();
-  auto acquire_backup_lock = (is_master && clone_block_ddl);
+  auto acquire_backup_lock = (is_master && clone_ddl_timeout != 0);
   auto num_workers = m_clone_client.get_max_concurrency() - 1;
 
   auto &client_vector = m_clone_client.get_storage_vector();
@@ -191,7 +190,7 @@ int Local::clone_exec() {
 }
 
 int Local_Callback::file_cbk(Ha_clone_file from_file, uint len) {
-  assert(!m_apply_data);
+  DBUG_ASSERT(!m_apply_data);
 
   /* Set source file to external handle of "Clone Client". */
   auto ext_link = get_client_data_link();
@@ -223,7 +222,7 @@ int Local_Callback::buffer_cbk(uchar *from_buffer, uint buf_len) {
 }
 
 int Local_Callback::apply_ack() {
-  assert(m_apply_data);
+  DBUG_ASSERT(m_apply_data);
 
   auto client = get_clone_client();
 
@@ -273,11 +272,11 @@ int Local_Callback::apply_data() {
 
   auto &task_vector = client->get_task_vector();
 
-  assert(get_loc_index() < task_vector.size());
+  DBUG_ASSERT(get_loc_index() < task_vector.size());
   auto task_id = task_vector[get_loc_index()];
 
   /* Call storage engine to apply the data. */
-  assert(!m_apply_data);
+  DBUG_ASSERT(!m_apply_data);
   m_apply_data = true;
 
   auto error = hton->clone_interface.clone_apply(hton, thd, client_loc, loc_len,
@@ -305,7 +304,7 @@ int Local_Callback::apply_cbk(Ha_clone_file to_file, bool apply_file,
                               uchar *&to_buffer, uint &to_len) {
   int error;
 
-  assert(m_apply_data);
+  DBUG_ASSERT(m_apply_data);
 
   auto client = get_clone_client();
   auto server = get_clone_server();
@@ -327,8 +326,8 @@ int Local_Callback::apply_cbk(Ha_clone_file to_file, bool apply_file,
     auto from_buf = ext_link->get_buffer();
 
     /* Assert alignment to CLONE_OS_ALIGN for O_DIRECT */
-    assert(is_os_buffer_cache() ||
-           from_buf->m_buffer == clone_os_align(from_buf->m_buffer));
+    DBUG_ASSERT(is_os_buffer_cache() ||
+                from_buf->m_buffer == clone_os_align(from_buf->m_buffer));
 
     if (apply_file) {
       error = clone_os_copy_buf_to_file(from_buf->m_buffer, to_file,
@@ -342,7 +341,7 @@ int Local_Callback::apply_cbk(Ha_clone_file to_file, bool apply_file,
     info.update(from_buf->m_length, 0);
 
   } else {
-    assert(dest_type == CLONE_HANDLE_FILE);
+    DBUG_ASSERT(dest_type == CLONE_HANDLE_FILE);
     uchar *buf_ptr;
     uint buf_len;
 
