@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,9 +23,9 @@
 #ifndef PIPELINE_HANDLERS_INCLUDED
 #define PIPELINE_HANDLERS_INCLUDED
 
-#include <assert.h>
 #include <mysql/group_replication_priv.h>
 
+#include "my_dbug.h"
 #include "plugin/group_replication/include/member_info.h"
 #include "plugin/group_replication/include/pipeline_interfaces.h"
 #include "plugin/group_replication/libmysqlgcs/include/mysql/gcs/gcs_communication_interface.h"
@@ -82,7 +81,7 @@ class Handler_start_action : public Pipeline_action {
  public:
   Handler_start_action() : Pipeline_action(HANDLER_START_ACTION) {}
 
-  ~Handler_start_action() override = default;
+  ~Handler_start_action() override {}
 };
 
 /**
@@ -94,7 +93,7 @@ class Handler_stop_action : public Pipeline_action {
  public:
   Handler_stop_action() : Pipeline_action(HANDLER_STOP_ACTION) {}
 
-  ~Handler_stop_action() override = default;
+  ~Handler_stop_action() override {}
 };
 
 /**
@@ -121,7 +120,7 @@ class Handler_applier_configuration_action : public Pipeline_action {
         applier_shutdown_timeout(plugin_shutdown_timeout),
         group_sidno(group_sidno),
         initialization_conf(true) {
-    assert(applier_name != nullptr);
+    DBUG_ASSERT(applier_name != nullptr);
   }
 
   /**
@@ -137,7 +136,7 @@ class Handler_applier_configuration_action : public Pipeline_action {
         group_sidno(0),
         initialization_conf(false) {}
 
-  ~Handler_applier_configuration_action() override = default;
+  ~Handler_applier_configuration_action() override {}
 
   /**
     @return the applier's name
