@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,13 +34,8 @@
 #include "plugin/group_replication/include/plugin_server_include.h"
 #include "plugin/group_replication/libmysqlgcs/include/mysql/gcs/gcs_member_identifier.h"
 
-typedef std::list<Gcs_member_identifier,
-                  Malloc_allocator<Gcs_member_identifier>>
-    Members_list;
-
 // Define the data packet type
 #define DATA_PACKET_TYPE 1
-#define VIEW_CHANGE_PACKET_TYPE 3
 
 /**
   @class Packet
@@ -57,7 +51,7 @@ class Packet {
   */
   Packet(int type) : packet_type(type) {}
 
-  virtual ~Packet() = default;
+  virtual ~Packet() {}
 
   /**
    @return the packet type
@@ -80,12 +74,11 @@ class Data_packet : public Packet {
 
     @param[in]  data             the packet data
     @param[in]  len              the packet length
-    @param[in]  key              the memory instrument key
     @param[in]  consistency_level  the transaction consistency level
     @param[in]  online_members     the ONLINE members when the transaction
                                    message was delivered
   */
-  Data_packet(const uchar *data, ulong len, PSI_memory_key key,
+  Data_packet(const uchar *data, ulong len,
               enum_group_replication_consistency_level consistency_level =
                   GROUP_REPLICATION_CONSISTENCY_EVENTUAL,
               std::list<Gcs_member_identifier> *online_members = nullptr)
@@ -94,7 +87,7 @@ class Data_packet : public Packet {
         len(len),
         m_consistency_level(consistency_level),
         m_online_members(online_members) {
-    payload = (uchar *)my_malloc(key, len, MYF(0));
+    payload = (uchar *)my_malloc(PSI_NOT_INSTRUMENTED, len, MYF(0));
     memcpy(payload, data, len);
   }
 
@@ -109,44 +102,6 @@ class Data_packet : public Packet {
   std::list<Gcs_member_identifier> *m_online_members;
 };
 
-/**
-  @class View_change_packet
-  A packet to send view change related info to the applier
-*/
-class View_change_packet : public Packet {
- public:
-  /**
-    Create a new data packet with associated data.
-
-    @param view_id_arg The view id associated to this view
-    @param need_vcle   The flag determine if View_change_log_event is needed.
-  */
-  View_change_packet(std::string &view_id_arg, bool need_vcle)
-      : Packet(VIEW_CHANGE_PACKET_TYPE),
-        view_id(view_id_arg),
-        m_need_vcle(need_vcle) {}
-  View_change_packet(View_change_packet *packet)
-      : Packet(VIEW_CHANGE_PACKET_TYPE),
-        view_id(packet->view_id),
-        group_executed_set(packet->group_executed_set),
-        m_valid_sender_list(packet->m_valid_sender_list),
-        m_members_joining_in_view(packet->m_members_joining_in_view),
-        m_need_vcle(packet->m_need_vcle) {}
-
-  ~View_change_packet() override = default;
-
-  /* View ID of the view. */
-  const std::string view_id;
-  /* Group executed GTID-SET picked from group members. */
-  std::vector<std::string> group_executed_set;
-  /* Online members during view-change. */
-  std::vector<Gcs_member_identifier> m_valid_sender_list;
-  /* New members joining in the view. */
-  std::vector<Gcs_member_identifier> m_members_joining_in_view;
-  /* Does any members needs VCLE to be logged? */
-  const bool m_need_vcle;
-};
-
 // Define the data packet type
 #define UNDEFINED_EVENT_MODIFIER 0
 
@@ -165,19 +120,7 @@ class View_change_packet : public Packet {
         If not specified, this field has a default value of 0.
 */
 class Pipeline_event {
-  enum class Processing_state {
-    DEFAULT,
-    DELAYED_VIEW_CHANGE_WAITING_FOR_CONSISTENT_TRANSACTIONS,
-    DELAYED_VIEW_CHANGE_RESUMED
-  };
-
  public:
-  enum class Pipeline_event_type {
-    PEVENT_DATA_PACKET_TYPE_E = 1,
-    PEVENT_BINARY_LOG_EVENT_TYPE_E = 2,
-    PEVENT_APPLIER_ONLY_EVENT_E = 3,
-  };
-
   /**
     Create a new pipeline wrapper based on a packet.
 
@@ -195,16 +138,14 @@ class Pipeline_event {
                  int modifier = UNDEFINED_EVENT_MODIFIER,
                  enum_group_replication_consistency_level consistency_level =
                      GROUP_REPLICATION_CONSISTENCY_EVENTUAL,
-                 Members_list *online_members = nullptr)
+                 std::list<Gcs_member_identifier> *online_members = nullptr)
       : packet(base_packet),
         log_event(nullptr),
         event_context(modifier),
         format_descriptor(fde_event),
         m_consistency_level(consistency_level),
         m_online_members(online_members),
-        m_online_members_memory_ownership(true),
-        m_processing_event_type(
-            Pipeline_event_type::PEVENT_DATA_PACKET_TYPE_E) {}
+        m_online_members_memory_ownership(true) {}
 
   /**
     Create a new pipeline wrapper based on a log event.
@@ -222,31 +163,14 @@ class Pipeline_event {
                  int modifier = UNDEFINED_EVENT_MODIFIER,
                  enum_group_replication_consistency_level consistency_level =
                      GROUP_REPLICATION_CONSISTENCY_EVENTUAL,
-                 Members_list *online_members = nullptr)
+                 std::list<Gcs_member_identifier> *online_members = nullptr)
       : packet(nullptr),
         log_event(base_event),
         event_context(modifier),
         format_descriptor(fde_event),
         m_consistency_level(consistency_level),
         m_online_members(online_members),
-        m_online_members_memory_ownership(true),
-        m_processing_event_type(
-            Pipeline_event_type::PEVENT_BINARY_LOG_EVENT_TYPE_E) {}
-
-  Pipeline_event(Packet *packet, int modifier = UNDEFINED_EVENT_MODIFIER,
-                 enum_group_replication_consistency_level consistency_level =
-                     GROUP_REPLICATION_CONSISTENCY_EVENTUAL,
-                 Members_list *online_members = nullptr)
-      : packet(nullptr),
-        log_event(nullptr),
-        packet_event(packet),
-        event_context(modifier),
-        format_descriptor(nullptr),
-        m_consistency_level(consistency_level),
-        m_online_members(online_members),
-        m_online_members_memory_ownership(true),
-        m_processing_event_type(
-            Pipeline_event_type::PEVENT_APPLIER_ONLY_EVENT_E) {}
+        m_online_members_memory_ownership(true) {}
 
   ~Pipeline_event() {
     if (packet != nullptr) {
@@ -255,9 +179,6 @@ class Pipeline_event {
     if (log_event != nullptr) {
       delete log_event;
     }
-    if (packet_event != nullptr) {
-      delete packet_event;
-    }
     if (m_online_members_memory_ownership) {
       delete m_online_members;
     }
@@ -293,9 +214,6 @@ class Pipeline_event {
     *out_event = log_event;
     return 0;
   }
-  Pipeline_event_type get_pipeline_event_type() {
-    return m_processing_event_type;
-  }
 
   /**
     Sets the pipeline event's log event.
@@ -339,10 +257,9 @@ class Pipeline_event {
 
     @return the pipeline event type
   */
-  mysql::binlog::event::Log_event_type get_event_type() {
+  Log_event_type get_event_type() {
     if (packet != nullptr)
-      return (mysql::binlog::event::Log_event_type)
-          packet->payload[EVENT_TYPE_OFFSET];
+      return (Log_event_type)packet->payload[EVENT_TYPE_OFFSET];
     else
       return log_event->get_type_code();
   }
@@ -406,8 +323,9 @@ class Pipeline_event {
     @note the memory allocated for the list ownership belongs to the
           caller
   */
-  Members_list *get_online_members() { return m_online_members; }
-  Packet *get_applier_event_packet() { return packet_event; }
+  std::list<Gcs_member_identifier> *get_online_members() {
+    return m_online_members;
+  }
 
   /**
     Release memory ownership of m_online_members.
@@ -416,52 +334,6 @@ class Pipeline_event {
     m_online_members_memory_ownership = false;
   }
 
-  /**
-    Set view change cannot be processed now and should be delayed due to
-    consistent transaction.
-  */
-  void set_delayed_view_change_waiting_for_consistent_transactions() {
-    assert(m_packet_processing_state == Processing_state::DEFAULT);
-    m_packet_processing_state = Processing_state::
-        DELAYED_VIEW_CHANGE_WAITING_FOR_CONSISTENT_TRANSACTIONS;
-  }
-
-  /**
-    Check if current view change is delayed due to consistent transaction.
-
-    @return is event being queued for later processing
-      @retval true     event is being queued
-      @retval false    event is not being queued
-  */
-  bool is_delayed_view_change_waiting_for_consistent_transactions() {
-    return m_packet_processing_state ==
-           Processing_state::
-               DELAYED_VIEW_CHANGE_WAITING_FOR_CONSISTENT_TRANSACTIONS;
-  }
-
-  /**
-    Allow resume the log of delayed views that were waiting for consistent
-    transactions from previous view to complete.
-  */
-  void set_delayed_view_change_resumed() {
-    assert(m_packet_processing_state ==
-           Processing_state::
-               DELAYED_VIEW_CHANGE_WAITING_FOR_CONSISTENT_TRANSACTIONS);
-    m_packet_processing_state = Processing_state::DELAYED_VIEW_CHANGE_RESUMED;
-  }
-
-  /**
-    Check if old view change processing is resumed.
-
-    @return is event being processed from queue
-      @retval true     event is being processed from queue
-      @retval false    event is not being processed from queue
-  */
-  bool is_delayed_view_change_resumed() {
-    return m_packet_processing_state ==
-           Processing_state::DELAYED_VIEW_CHANGE_RESUMED;
-  }
-
  private:
   /**
     Converts the existing packet into a log event.
@@ -504,7 +376,7 @@ class Pipeline_event {
     }
 
     packet = new Data_packet(reinterpret_cast<const uchar *>(ostream.c_ptr()),
-                             ostream.length(), key_transaction_data);
+                             ostream.length());
 
     delete log_event;
     log_event = nullptr;
@@ -515,15 +387,12 @@ class Pipeline_event {
  private:
   Data_packet *packet;
   Log_event *log_event;
-  Packet *packet_event{nullptr};
   int event_context;
   /* Format description event used on conversions */
   Format_description_log_event *format_descriptor;
   enum_group_replication_consistency_level m_consistency_level;
-  Members_list *m_online_members;
+  std::list<Gcs_member_identifier> *m_online_members;
   bool m_online_members_memory_ownership;
-  Processing_state m_packet_processing_state{Processing_state::DEFAULT};
-  Pipeline_event_type m_processing_event_type;
 };
 
 /**
@@ -632,7 +501,7 @@ class Pipeline_action {
  public:
   Pipeline_action(int action_type) { type = action_type; }
 
-  virtual ~Pipeline_action() = default;
+  virtual ~Pipeline_action() {}
 
   /**
     Returns this action type.
@@ -662,7 +531,7 @@ class Event_handler {
  public:
   Event_handler() : next_in_pipeline(nullptr) {}
 
-  virtual ~Event_handler() = default;
+  virtual ~Event_handler() {}
 
   /**
     Initialization as defined in the handler implementation.
