@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -69,34 +68,34 @@ static pax_msg *init_pax_msg(pax_msg *p, int refcnt, synode_no synode,
   init_ballot(&p->proposal, -1, nodeno);
   p->synode = synode;
   p->msg_type = normal;
-  p->receivers = nullptr;
-  p->a = nullptr;
+  p->receivers = NULL;
+  p->a = NULL;
   p->force_delivery = 0;
   p->event_horizon = EVENT_HORIZON_MIN;
   return p;
 }
 
 pax_msg *pax_msg_new(synode_no synode, site_def const *site) {
-  pax_msg *p = (pax_msg *)xcom_calloc((size_t)1, sizeof(pax_msg));
+  pax_msg *p = (pax_msg *)calloc((size_t)1, sizeof(pax_msg));
   IFDBG(D_NONE, FN; PTREXP(p));
   return init_pax_msg(p, 0, synode, site);
 }
 
 pax_msg *pax_msg_new_0(synode_no synode) {
-  pax_msg *p = (pax_msg *)xcom_calloc((size_t)1, sizeof(pax_msg));
+  pax_msg *p = (pax_msg *)calloc((size_t)1, sizeof(pax_msg));
   IFDBG(D_NONE, FN; PTREXP(p));
-  return init_pax_msg(p, 0, synode, nullptr);
+  return init_pax_msg(p, 0, synode, 0);
 }
 
 pax_msg *clone_pax_msg_no_app(pax_msg *msg) {
-  pax_msg *p = (pax_msg *)xcom_calloc((size_t)1, sizeof(pax_msg));
+  pax_msg *p = (pax_msg *)calloc((size_t)1, sizeof(pax_msg));
   IFDBG(D_NONE, FN; STRLIT("clone_pax_msg"); PTREXP(p));
   *p = *msg;
   p->refcnt = 0;
   p->receivers = clone_bit_set(msg->receivers);
-  p->a = nullptr; /* Or copy_app_data will be confused */
-  p->snap = nullptr;
-  p->gcs_snap = nullptr;
+  p->a = NULL; /* Or copy_app_data will be confused */
+  p->snap = NULL;
+  p->gcs_snap = NULL;
   return p;
 }
 
@@ -132,7 +131,7 @@ int unref_msg(pax_msg **pp) {
     delete_pax_msg(p);
     return 0;
   }
-  *pp = nullptr;
+  *pp = NULL;
   return p->refcnt;
 }
 
