@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -44,7 +44,6 @@
 #include "xcom/xcom_cfg.h"
 #include "xcom/xcom_common.h"
 #include "xcom/xcom_detector.h"
-#include "xcom/xcom_memory.h"
 #include "xcom/xcom_profile.h"
 #include "xcom/xcom_transport.h"
 #include "xcom/xcom_vp_str.h"
@@ -102,7 +101,7 @@ static linkage probation_lru = {
 
 static void hash_init(stack_machine *hash_bucket) {
   size_t i;
-  hash_bucket->pax_hash = (linkage *)xcom_malloc(sizeof(linkage) * BUCKETS);
+  hash_bucket->pax_hash = (linkage *)malloc(sizeof(linkage) * BUCKETS);
   for (i = 0; i < BUCKETS; i++) {
     link_init(&hash_bucket->pax_hash[i], TYPE_HASH("pax_machine"));
   }
@@ -157,7 +156,7 @@ static pax_machine *hash_out(pax_machine *p) {
 
 pax_machine *hash_get(synode_no synode) {
   /* static pax_machine *cached_machine = NULL; */
-  stack_machine *hash_table = nullptr;
+  stack_machine *hash_table = NULL;
 
   /* if(cached_machine && synode_eq(synode, cached_machine->synode)) */
   /* return cached_machine; */
@@ -169,7 +168,7 @@ pax_machine *hash_get(synode_no synode) {
     }
   })
 
-  if (hash_table != nullptr) {
+  if (hash_table != NULL) {
     linkage *bucket = &hash_table->pax_hash[synode_hash(synode)];
 
     FWD_ITER(bucket, pax_machine, {
@@ -179,7 +178,7 @@ pax_machine *hash_get(synode_no synode) {
       }
     });
   }
-  return nullptr;
+  return NULL;
 }
 
 static int was_machine_executed(pax_machine *p) {
@@ -197,8 +196,8 @@ lest recently used order.
 */
 
 static lru_machine *lru_get(bool_t force) {
-  lru_machine *retval = nullptr;
-  lru_machine *force_retval = nullptr;
+  lru_machine *retval = NULL;
+  lru_machine *force_retval = NULL;
   if (!link_empty(&probation_lru)) {
     retval = (lru_machine *)link_first(&probation_lru);
   } else {
@@ -233,7 +232,7 @@ static void reset_cache() {
   link_init(&protected_lru, TYPE_HASH("lru_machine"));
   link_init(&probation_lru, TYPE_HASH("lru_machine"));
   link_init(&hash_stack, TYPE_HASH("stack_machine"));
-  init_cache_size(); /* After cache has been initialized, size is 0 */
+  init_cache_size(); /* After cache has been intialized, size is 0 */
   last_removed_cache = null_synode;
   highest_msgno = 0;
 }
@@ -255,16 +254,15 @@ void init_cache() {
 }
 
 static void deinit_pax_machine(pax_machine *p, lru_machine *l) {
-  init_pax_machine(&l->pax, nullptr, null_synode);
+  init_pax_machine(&l->pax, NULL, null_synode);
   if (p->proposer.prep_nodeset) {
     free_bit_set(p->proposer.prep_nodeset);
-    p->proposer.prep_nodeset = nullptr;
+    p->proposer.prep_nodeset = NULL;
   }
   if (p->proposer.prop_nodeset) {
     free_bit_set(p->proposer.prop_nodeset);
-    p->proposer.prop_nodeset = nullptr;
+    p->proposer.prop_nodeset = NULL;
   }
-  link_out(&p->watchdog);
 }
 
 static void free_lru_machine(lru_machine *link_iter) {
@@ -300,7 +298,7 @@ pax_machine *get_cache_no_touch(synode_no synode, bool_t force) {
   if (!retval) {
     lru_machine *l =
         lru_get(force); /* Need to know when it is safe to re-use... */
-    if (!l) return nullptr;
+    if (!l) return NULL;
     IFDBG(D_NONE, FN; PTREXP(l); COPY_AND_FREE_GOUT(dbg_pax_machine(&l->pax)););
     /* assert(l->pax.synode > log_tail); */
 
@@ -332,7 +330,7 @@ static inline int can_deallocate(lru_machine *link_iter) {
   site_def const *dealloc_site = find_site_def(link_iter->pax.synode);
 
   /* If we have no site, or site was just installed, refuse deallocation */
-  if (site == nullptr) return 0;
+  if (site == 0) return 0;
   /*
           With the patch that was put in to ensure that nodes always see  a
           global  view  message when it joins, the node that joins may need
@@ -347,8 +345,7 @@ static inline int can_deallocate(lru_machine *link_iter) {
           other nodes anyway, and expelled.
   */
   if ((site->install_time + DETECTOR_LIVE_TIMEOUT) > task_now()) return 0;
-  if (dealloc_site ==
-      nullptr) /* Synode does not match any site, OK to deallocate */
+  if (dealloc_site == 0) /* Synode does not match any site, OK to deallocate */
     return 1;
   delivered_msg = get_min_delivered_msg(site);
   if (synode_eq(delivered_msg,
@@ -395,11 +392,10 @@ pax_machine *init_pax_machine(pax_machine *p, lru_machine *lru,
   sub_cache_size(p);
   link_init(&p->hash_link, TYPE_HASH("pax_machine"));
   p->lru = lru;
-  p->stack_link = nullptr;
+  p->stack_link = NULL;
   p->synode = synode;
   p->last_modified = 0.0;
   link_init(&p->rv, TYPE_HASH("task_env"));
-  link_init(&p->watchdog, TYPE_HASH("time_queue"));
   init_ballot(&p->proposer.bal, -1, 0);
   init_ballot(&p->proposer.sent_prop, 0, 0);
   init_ballot(&p->proposer.sent_learn, -1, 0);
@@ -409,15 +405,14 @@ pax_machine *init_pax_machine(pax_machine *p, lru_machine *lru,
   if (!p->proposer.prop_nodeset)
     p->proposer.prop_nodeset = new_bit_set(NSERVERS);
   BIT_ZERO(p->proposer.prop_nodeset);
-  replace_pax_msg(&p->proposer.msg, nullptr);
+  replace_pax_msg(&p->proposer.msg, NULL);
   init_ballot(&p->acceptor.promise, 0, 0);
-  replace_pax_msg(&p->acceptor.msg, nullptr);
-  replace_pax_msg(&p->learner.msg, nullptr);
+  replace_pax_msg(&p->acceptor.msg, NULL);
+  replace_pax_msg(&p->learner.msg, NULL);
   p->lock = 0;
   p->op = initial_op;
   p->force_delivery = 0;
   p->enforcer = 0;
-  SET_PAXOS_FSM_STATE(p, paxos_fsm_idle);
   return p;
 }
 
@@ -500,7 +495,7 @@ size_t pax_machine_size(pax_machine const *p) {
 }
 /* }}} */
 
-/* The cache itself is statically allocated, set size of dynamically allocated
+/* The cache itself is statically allocated, set size of dynamically allocted
  * data to 0 */
 void init_cache_size() { cache_size = 0; }
 
@@ -555,7 +550,7 @@ uint64_t set_max_cache_size(uint64_t x) {
 static void expand_lru() {
   uint64_t i;
   for (i = 0; i < BUCKETS; i++) {
-    lru_machine *l = (lru_machine *)xcom_calloc(1, sizeof(lru_machine));
+    lru_machine *l = (lru_machine *)calloc(1, sizeof(lru_machine));
     link_init(&l->lru_link, TYPE_HASH("lru_machine"));
     link_into(&l->lru_link, &probation_lru);
     init_pax_machine(&l->pax, l, null_synode);
@@ -564,8 +559,7 @@ static void expand_lru() {
 }
 
 static void add_stack_machine(uint64_t start_msgno) {
-  stack_machine *hash_bucket =
-      (stack_machine *)xcom_malloc(sizeof(stack_machine));
+  stack_machine *hash_bucket = (stack_machine *)malloc(sizeof(stack_machine));
   link_init(&hash_bucket->stack_link, TYPE_HASH("stack_machine"));
   hash_bucket->occupation = 0;
   hash_bucket->start_msgno = start_msgno;
@@ -585,11 +579,9 @@ static void do_decrement_step() {
     if (++count == BUCKETS) break;
   })
 
-  linkage *tmp = link_last(&hash_stack);
-  free(((stack_machine *)tmp)->pax_hash);
-  link_out(tmp);
+  free(((stack_machine *)link_last(&hash_stack))->pax_hash);
+  free(link_out(link_last(&hash_stack)));
   ((stack_machine *)link_last(&hash_stack))->start_msgno = 0;
-  free(tmp);
 }
 
 /* Use vars instead of defines for unit testing */
@@ -630,11 +622,9 @@ void do_cache_maintenance() {
   }
 }
 
-int cache_manager_task(task_arg arg [[maybe_unused]]) {
+int cache_manager_task(task_arg arg MY_ATTRIBUTE((unused))) {
   DECL_ENV
   int dummy;
-  ENV_INIT
-  END_ENV_INIT
   END_ENV;
 
   TASK_BEGIN
@@ -658,11 +648,9 @@ void set_length_increment(size_t increment) { length_increment = increment; }
 
 void set_size_decrement(size_t decrement) { size_decrement = decrement; }
 
-#if 0
 void set_dec_threshold_length(uint64_t threshold) {
   dec_threshold_length = threshold;
 }
-#endif
 
 void set_min_target_occupation(float threshold) {
   min_target_occupation = threshold;
@@ -674,8 +662,3 @@ void set_min_length_threshold(float threshold) {
   min_length_threshold = threshold;
 }
 /* purecov: end */
-
-void paxos_timeout(pax_machine *p) {
-  (void)p;
-  IFDBG(D_BUG, FN; SYCEXP(p->synode));
-}
