@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,7 +28,7 @@
 #include "my_dbug.h"
 #include "plugin/group_replication/include/plugin.h"
 
-Applier_handler::Applier_handler() = default;
+Applier_handler::Applier_handler() {}
 
 int Applier_handler::initialize() {
   DBUG_TRACE;
@@ -106,27 +106,12 @@ int Applier_handler::stop_applier_thread() {
   return error;
 }
 
-int Applier_handler::handle_event(Pipeline_event *pevent, Continuation *cont) {
+int Applier_handler::handle_event(Pipeline_event *event, Continuation *cont) {
   DBUG_TRACE;
-  Pipeline_event::Pipeline_event_type event_type =
-      pevent->get_pipeline_event_type();
-  switch (event_type) {
-    case Pipeline_event::Pipeline_event_type::PEVENT_DATA_PACKET_TYPE_E:
-      return handle_binary_log_event(pevent, cont);
-    case Pipeline_event::Pipeline_event_type::PEVENT_BINARY_LOG_EVENT_TYPE_E:
-      return handle_binary_log_event(pevent, cont);
-    case Pipeline_event::Pipeline_event_type::PEVENT_APPLIER_ONLY_EVENT_E:
-      return handle_applier_event(pevent, cont);
-    default:
-      next(pevent, cont);
-      return 0;
-  }
-}
+  int error = 0;
 
-int Applier_handler::handle_binary_log_event(Pipeline_event *event,
-                                             Continuation *cont) {
   Data_packet *p = nullptr;
-  int error = event->get_Packet(&p);
+  error = event->get_Packet(&p);
   DBUG_EXECUTE_IF("applier_handler_force_error_on_pipeline", error = 1;);
   if (error || (p == nullptr)) {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_FETCH_TRANS_DATA_FAILED);
@@ -139,29 +124,24 @@ int Applier_handler::handle_binary_log_event(Pipeline_event *event,
     server applier, this event is only need for certification,
     performed on the previous handler.
   */
-  if (event->get_event_type() !=
-      mysql::binlog::event::TRANSACTION_CONTEXT_EVENT) {
+  if (event->get_event_type() != binary_log::TRANSACTION_CONTEXT_EVENT) {
     error = channel_interface.queue_packet((const char *)p->payload, p->len);
 
-    if (mysql::binlog::event::Log_event_type_helper::is_assigned_gtid_event(
-            event->get_event_type())) {
+    if (event->get_event_type() == binary_log::GTID_LOG_EVENT &&
+        local_member_info->get_recovery_status() ==
+            Group_member_info::MEMBER_ONLINE) {
       applier_module->get_pipeline_stats_member_collector()
           ->increment_transactions_waiting_apply();
     }
   }
 
 end:
-
   if (error)
     cont->signal(error);
   else
     next(event, cont);
-  return error;
-}
 
-int Applier_handler::handle_applier_event(Pipeline_event *event,
-                                          Continuation *cont) {
-  return next(event, cont);
+  return error;
 }
 
 int Applier_handler::handle_action(Pipeline_action *action) {
