@@ -24,6 +24,7 @@
 #include "plugin/group_replication/include/leave_group_on_failure.h"
 #include "plugin/group_replication/include/plugin.h"
 #include "plugin/group_replication/include/plugin_handlers/remote_clone_handler.h"
+#include "plugin/group_replication/include/plugin_variables/recovery_endpoints.h"
 
 [[noreturn]] void *Remote_clone_handler::launch_thread(void *arg) {
   Remote_clone_handler *thd = static_cast<Remote_clone_handler *>(arg);
@@ -486,6 +487,20 @@ int Remote_clone_handler::run_clone_query(
     std::string &port, std::string &username, std::string &password,
     bool use_ssl) {
   int error = 0;
+
+#ifndef DBUG_OFF
+  DBUG_EXECUTE_IF("gr_run_clone_query_fail_once", {
+    const char act[] =
+        "now signal signal.run_clone_query_waiting wait_for "
+        "signal.run_clone_query_continue";
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+
+    DBUG_SET("-d,gr_run_clone_query_fail_once");
+
+    return 1;
+  });
+#endif /* DBUG_OFF */
+
   mysql_mutex_lock(&m_clone_query_lock);
   m_clone_query_session_id =
       sql_command_interface->get_sql_service_interface()->get_session_id();
@@ -782,13 +797,15 @@ void Remote_clone_handler::gr_clone_debug_point() {
 
     std::string hostname("");
     std::string port("");
+    std::vector<std::pair<std::string, uint>> endpoints;
 
     mysql_mutex_lock(&m_donor_list_lock);
     empty_donor_list = m_suitable_donors.empty();
     if (!empty_donor_list) {
       Group_member_info *member = m_suitable_donors.front();
-      hostname.assign(member->get_hostname());
-      port.assign(std::to_string(member->get_port()));
+      Donor_recovery_endpoints donor_endpoints;
+      endpoints = donor_endpoints.get_endpoints(member);
+
       // define the current donor
       delete m_current_donor_address;
       m_current_donor_address =
@@ -801,45 +818,56 @@ void Remote_clone_handler::gr_clone_debug_point() {
     }
     mysql_mutex_unlock(&m_donor_list_lock);
 
-    // No valid donor remains in the list
-    if (hostname.empty()) continue;
-
-    /* Update the allowed donor list */
-    if ((error = update_donor_list(sql_command_interface, hostname, port))) {
-      break; /* purecov: inspected */
+    // No valid donor in the list
+    if (endpoints.size() == 0) {
+      error = 1;
+      continue;
     }
 
-    if (m_being_terminated) goto thd_end;
+    for (auto endpoint : endpoints) {
+      hostname.assign(endpoint.first);
+      port.assign(std::to_string(endpoint.second));
 
-    terminate_wait_on_start_process(true);
-
-    error = run_clone_query(sql_command_interface, hostname, port, username,
-                            password, use_ssl);
-
-    // Even on critical errors we continue as another clone can fix the issue
-    if (!critical_error) critical_error = evaluate_error_code(error);
-
-    // On ER_RESTART_SERVER_FAILED it makes no sense to retry
-    if (error == ER_RESTART_SERVER_FAILED) goto thd_end;
-
-    if (error && !m_being_terminated) {
-      if (evaluate_server_connection(sql_command_interface)) {
-        /* purecov: begin inspected */
-        // This is a bad sign, might as well quit
-        critical_error = true;
-        goto thd_end;
-        /* purecov: end */
+      /* Update the allowed donor list */
+      if ((error = update_donor_list(sql_command_interface, hostname, port))) {
+        continue; /* purecov: inspected */
       }
 
-      /*
-       If we reach this point and we are now alone it means no alternatives
-       remain for recovery. A fallback to distributed recovery would be
-       interpreted as a solo member recovery.
-      */
-      if (group_member_mgr->get_number_of_members() == 1) {
-        critical_error = true;
-        goto thd_end;
+      if (m_being_terminated) goto thd_end;
+
+      terminate_wait_on_start_process(true);
+
+      error = run_clone_query(sql_command_interface, hostname, port, username,
+                              password, use_ssl);
+
+      // Even on critical errors we continue as another clone can fix the issue
+      if (!critical_error) critical_error = evaluate_error_code(error);
+
+      // On ER_RESTART_SERVER_FAILED it makes no sense to retry
+      if (error == ER_RESTART_SERVER_FAILED) goto thd_end;
+
+      if (error && !m_being_terminated) {
+        if (evaluate_server_connection(sql_command_interface)) {
+          /* purecov: begin inspected */
+          // This is a bad sign, might as well quit
+          critical_error = true;
+          goto thd_end;
+          /* purecov: end */
+        }
+
+        /*
+          If we reach this point and we are now alone it means no alternatives
+          remain for recovery. A fallback to distributed recovery would be
+          interpreted as a solo member recovery.
+          */
+        if (group_member_mgr->get_number_of_members() == 1) {
+          critical_error = true;
+          goto thd_end;
+        }
       }
+
+      // try till there is a recovery endpoint that succeeds
+      if (!error) break;
     }
 
     // try till there is a clone command that succeeds
