@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,10 +29,10 @@
 
 namespace keyring {
 
-constexpr const char *AES = "AES";
-constexpr const char *RSA = "RSA";
-constexpr const char *DSA = "DSA";
-constexpr const char *SECRET = "SECRET";
+const std::string AES = "AES";
+const std::string RSA = "RSA";
+const std::string DSA = "DSA";
+const std::string SECRET = "SECRET";
 
 enum class Operation_type { fetch = 0, store, remove, generate };
 
@@ -62,10 +61,7 @@ struct IKey : public Keyring_alloc {
   virtual bool is_key_valid() = 0;
   virtual bool is_key_length_valid() = 0;
 
-  virtual ~IKey() = default;
-
- protected:
-  virtual void set_key_type_enum(const std::string *key_type) = 0;
+  virtual ~IKey() {}
 };
 
 }  // namespace keyring
