@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,70 +23,70 @@
 #ifndef KEYS_CONTAINER_INCLUDED
 #define KEYS_CONTAINER_INCLUDED
 
-#include <my_global.h>
-#include <hash.h>
-#include "keyring_key.h"
-#include "i_keyring_io.h"
-#include "logger.h"
-#include <sys_vars_shared.h> //For PolyLock, AutoWLock, AutoRLock
-#include "i_keys_container.h"
-#include "keyring_memory.h"
-#include "i_system_keys_container.h"
-
-#include <vector>
+#include <sys/types.h>
 
 #include <vector>
+#include "m_ctype.h"
+#include "map_helpers.h"
+#include "my_inttypes.h"
+#include "my_sharedlib.h"
+#include "plugin/keyring/common/i_keyring_io.h"
+#include "plugin/keyring/common/i_keys_container.h"
+#include "plugin/keyring/common/i_system_keys_container.h"
+#include "plugin/keyring/common/keyring_key.h"
+#include "plugin/keyring/common/keyring_memory.h"
+#include "plugin/keyring/common/logger.h"
+#include "sql/sys_vars_shared.h"  //For PolyLock, AutoWLock, AutoRLock
 
 namespace keyring {
 
-extern "C" MYSQL_PLUGIN_IMPORT CHARSET_INFO *system_charset_info;
-
-class Keys_container : public IKeys_container
-{
-private:
+class Keys_container : public IKeys_container {
+ private:
   bool remove_keys_metadata(IKey *key);
   void store_keys_metadata(IKey *key);
-public:
-  Keys_container(ILogger* logger);
-  my_bool init(IKeyring_io* keyring_io, std::string keyring_storage_url);
-  my_bool store_key(IKey *key);
-  IKey* fetch_key(IKey *key);
-  my_bool remove_key(IKey *key);
-  std::string get_keyring_storage_url();
-  std::vector<Key_metadata> get_keys_metadata()
-  {
+
+ public:
+  Keys_container(ILogger *logger);
+  bool init(IKeyring_io *keyring_io, std::string keyring_storage_url) override;
+  bool store_key(IKey *key) override;
+  IKey *fetch_key(IKey *key) override;
+  bool remove_key(IKey *key) override;
+  std::string get_keyring_storage_url() override;
+  void set_keyring_io(IKeyring_io *keyring_io) override;
+  std::vector<Key_metadata> get_keys_metadata() override {
     return keys_metadata;
   }
 
+  ~Keys_container() override;
 
-  ~Keys_container();
+  size_t get_number_of_keys() { return keys_hash->size(); }
 
-  ulong get_number_of_keys()
-  {
-    return keys_hash->records;
-  };
-protected:
+  /** Internal function, returns the key from the internal hash, no locks */
+  IKey *get_key_from_hash(IKey *key) override;
+
+ protected:
   Keys_container(const Keys_container &);
+
   virtual void allocate_and_set_data_for_key(IKey *key,
                                              std::string *source_key_type,
                                              uchar *source_key_data,
                                              size_t source_key_data_size);
-  my_bool load_keys_from_keyring_storage();
+  bool load_keys_from_keyring_storage();
   void free_keys_hash();
-  IKey *get_key_from_hash(IKey *key);
-  my_bool store_key_in_hash(IKey *key);
-  my_bool remove_key_from_hash(IKey *key);
-  virtual my_bool flush_to_backup();
-  virtual my_bool flush_to_storage(IKey *key, Key_operation operation);
+  bool store_key_in_hash(IKey *key);
+  bool remove_key_from_hash(IKey *key);
+  virtual bool flush_to_backup();
+  virtual bool flush_to_storage(IKey *key, Key_operation operation);
 
-  HASH *keys_hash;
+  using Key_hash = collation_unordered_map<std::string, std::unique_ptr<IKey>>;
+  std::unique_ptr<Key_hash> keys_hash;
   std::vector<Key_metadata> keys_metadata;
   ILogger *logger;
   IKeyring_io *keyring_io;
   std::string keyring_storage_url;
-  boost::movelib::unique_ptr<ISystem_keys_container> system_keys_container;
+  std::unique_ptr<ISystem_keys_container> system_keys_container;
 };
 
-} //namespace keyring
+}  // namespace keyring
 
-#endif //KEYS_CONTAINER_INCLUDED
+#endif  // KEYS_CONTAINER_INCLUDED
