@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,9 +48,7 @@
 #include "mysql/service_mysql_alloc.h"
 #include "mysql/service_security_context.h"
 #include "mysqld_error.h"
-#include "nulls.h"
 #include "sql/sql_error.h"
-#include "template_utils.h"
 #include "typelib.h"
 
 class THD;
@@ -113,7 +110,7 @@ static TYPELIB password_policy_typelib_t = {array_elements(policy_names) - 1,
 
 typedef std::string string_type;
 typedef std::set<string_type> set_type;
-static set_type *dictionary_words{nullptr};
+static set_type dictionary_words;
 
 static int validate_password_length;
 static int validate_password_number_count;
@@ -141,7 +138,7 @@ static void dictionary_activate(set_type *dict_words) {
   char *new_ts;
 
   /* fetch the start time */
-  start_time = time(nullptr);
+  start_time = my_time(MYF(0));
   localtime_r(&start_time, &tm);
   snprintf(timebuf, sizeof(timebuf), "%04d-%02d-%02d %02d:%02d:%02d",
            tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min,
@@ -149,8 +146,8 @@ static void dictionary_activate(set_type *dict_words) {
   new_ts = my_strdup(PSI_NOT_INSTRUMENTED, timebuf, MYF(0));
 
   mysql_rwlock_wrlock(&LOCK_dict_file);
-  std::swap(*dictionary_words, *dict_words);
-  validate_password_dictionary_file_words_count = dictionary_words->size();
+  std::swap(dictionary_words, *dict_words);
+  validate_password_dictionary_file_words_count = dictionary_words.size();
   std::swap(new_ts, validate_password_dictionary_file_last_parsed);
   mysql_rwlock_unlock(&LOCK_dict_file);
 
@@ -200,7 +197,7 @@ static void read_dictionary_file() {
 /* Clear words from std::set */
 static void free_dictionary_file() {
   mysql_rwlock_wrlock(&LOCK_dict_file);
-  if (!dictionary_words->empty()) dictionary_words->clear();
+  if (!dictionary_words.empty()) dictionary_words.clear();
   if (validate_password_dictionary_file_last_parsed) {
     my_free(validate_password_dictionary_file_last_parsed);
     validate_password_dictionary_file_last_parsed = nullptr;
@@ -217,19 +214,19 @@ static int validate_dictionary_check(mysql_string_handle password) {
   int error = 0;
   char *buffer;
 
-  if (dictionary_words->empty()) return (1);
+  if (dictionary_words.empty()) return (1);
 
   /* New String is allocated */
   mysql_string_handle lower_string_handle = mysql_string_to_lowercase(password);
   if (!(buffer = (char *)malloc(MAX_PASSWORD_LENGTH))) return (0);
 
-  length = mysql_string_convert_to_char_ptr(
-      lower_string_handle, "utf8mb3", buffer, MAX_PASSWORD_LENGTH, &error);
+  length = mysql_string_convert_to_char_ptr(lower_string_handle, "utf8", buffer,
+                                            MAX_PASSWORD_LENGTH, &error);
   /* Free the allocated string */
   mysql_string_free(lower_string_handle);
   int substr_pos = 0;
   int substr_length = length;
-  const string_type password_str = string_type((const char *)buffer, length);
+  string_type password_str = string_type((const char *)buffer, length);
   string_type password_substr;
   set_type::iterator itr;
   /*
@@ -241,8 +238,8 @@ static int validate_dictionary_check(mysql_string_handle password) {
     substr_pos = 0;
     while (substr_pos + substr_length <= length) {
       password_substr = password_str.substr(substr_pos, substr_length);
-      itr = dictionary_words->find(password_substr);
-      if (itr != dictionary_words->end()) {
+      itr = dictionary_words.find(password_substr);
+      if (itr != dictionary_words.end()) {
         mysql_rwlock_unlock(&LOCK_dict_file);
         free(buffer);
         return (0);
@@ -345,7 +342,7 @@ static bool is_valid_password_by_user_name(mysql_string_handle password) {
     return false;
   }
 
-  length = mysql_string_convert_to_char_ptr(password, "utf8mb3", buffer,
+  length = mysql_string_convert_to_char_ptr(password, "utf8", buffer,
                                             MAX_PASSWORD_LENGTH, &error);
 
   return is_valid_user(ctx, buffer, length, "user", "login user name") &&
@@ -476,7 +473,6 @@ static struct st_mysql_validate_password validate_password_descriptor = {
 static int validate_password_init(MYSQL_PLUGIN plugin_info) {
   push_deprecated_warn(thd_get_current_thd(), "validate password plugin",
                        "validate_password component");
-  dictionary_words = new set_type();
   // Initialize error logging service.
   if (init_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs)) return (1);
 
@@ -497,14 +493,12 @@ static int validate_password_init(MYSQL_PLUGIN plugin_info) {
   It empty the std::set and returns 0
 */
 
-static int validate_password_deinit(void *arg [[maybe_unused]]) {
+static int validate_password_deinit(void *arg MY_ATTRIBUTE((unused))) {
   push_deprecated_warn(thd_get_current_thd(), "validate password plugin",
                        "validate_password component");
   free_dictionary_file();
   mysql_rwlock_destroy(&LOCK_dict_file);
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
-  delete dictionary_words;
-  dictionary_words = nullptr;
   return (0);
 }
 
@@ -513,9 +507,9 @@ static int validate_password_deinit(void *arg [[maybe_unused]]) {
   If dictionary file is changed, this function will flush
   the cache and re-load the new dictionary file.
 */
-static void dictionary_update(MYSQL_THD thd [[maybe_unused]],
-                              SYS_VAR *var [[maybe_unused]], void *var_ptr,
-                              const void *save) {
+static void dictionary_update(MYSQL_THD thd MY_ATTRIBUTE((unused)),
+                              SYS_VAR *var MY_ATTRIBUTE((unused)),
+                              void *var_ptr, const void *save) {
   *static_cast<const char **>(var_ptr) =
       *static_cast<const char **>(const_cast<void *>(save));
   read_dictionary_file();
@@ -528,8 +522,8 @@ static void dictionary_update(MYSQL_THD thd [[maybe_unused]],
   3. validate_password_mixed_case_count
   4. validate_password_special_char_count
 */
-static void length_update(MYSQL_THD thd [[maybe_unused]],
-                          SYS_VAR *var [[maybe_unused]], void *var_ptr,
+static void length_update(MYSQL_THD thd MY_ATTRIBUTE((unused)),
+                          SYS_VAR *var MY_ATTRIBUTE((unused)), void *var_ptr,
                           const void *save) {
   /* check if there is an actual change */
   if (*static_cast<int *>(var_ptr) == *static_cast<const int *>(save)) return;
