@@ -489,6 +489,7 @@ const struct st_command_service_cbs sql_cbs = {
     sql_handle_ok,
     sql_handle_error,
     sql_shutdown,
+    nullptr,
 };
 
 static const char *fieldtype2str(enum enum_field_types type) {
@@ -636,6 +637,7 @@ static void exec_test_cmd(MYSQL_SESSION session, const char *test_cmd,
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   COM_DATA cmd;
   pctx->reset();
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = test_cmd;
   cmd.com_query.length = strlen(cmd.com_query.query);
   int fail = command_service_run_command(session, COM_QUERY, &cmd,
@@ -696,6 +698,7 @@ static void test_sql(void *p) {
   WRITE_STR("\nClose Session 1\n");
   if (srv_session_close(session_1))
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Close session_1 failed.");
+  session_1 = nullptr;
 
   /* Statement 4 */
   WRITE_SEP();
