@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -608,6 +608,7 @@ const struct st_command_service_cbs protocol_callbacks = {
     handle_ok,
     handle_error,
     handle_shutdown,
+    nullptr,
 };
 
 /****************************************************************************************/
@@ -625,10 +626,14 @@ static const char *fieldtype2str(enum enum_field_types type) {
       return "BIT";
     case MYSQL_TYPE_BLOB:
       return "BLOB";
+    case MYSQL_TYPE_BOOL:
+      return "BOOL";
     case MYSQL_TYPE_DATE:
       return "DATE";
     case MYSQL_TYPE_DATETIME:
       return "DATETIME";
+    case MYSQL_TYPE_INVALID:
+      return "?-invalid-?";
     case MYSQL_TYPE_NEWDECIMAL:
       return "NEWDECIMAL";
     case MYSQL_TYPE_DECIMAL:
@@ -714,6 +719,7 @@ static char *fieldflags2str(uint f) {
 }
 
 static void set_query_in_com_data(union COM_DATA *cmd, const char *query) {
+  memset(cmd, 0, sizeof(union COM_DATA));
   cmd->com_query.query = query;
   cmd->com_query.length = strlen(query);
 }
@@ -790,6 +796,7 @@ static void setup_test(MYSQL_SESSION session, void *p) {
   COM_DATA cmd;
 
   WRITE_STR("CHANGE DATABASE\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_init_db.db_name = "test";
   cmd.com_init_db.length = strlen("test");
   run_cmd(session, COM_INIT_DB, &cmd, &ctx, false, p);
@@ -874,6 +881,7 @@ static void test_1(MYSQL_SESSION session, void *p) {
   COM_DATA cmd;
 
   WRITE_STR("CREATE PREPARED STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_prepare.query = "SELECT * from t1 where a > ? and b < ?";
   cmd.com_stmt_prepare.length = strlen(cmd.com_stmt_prepare.query);
   run_cmd(session, COM_STMT_PREPARE, &cmd, &ctx, false, p);
@@ -881,6 +889,7 @@ static void test_1(MYSQL_SESSION session, void *p) {
   WRITE_STR("EXECUTE PREPARED STATEMENT WITH PARAMETERS AND CURSOR\n");
 
   PS_PARAM params[2];
+  memset(params, 0, sizeof(params));
   params[0].type = MYSQL_TYPE_STRING;
   params[0].unsigned_type = false;
   params[0].null_bit = false;
@@ -893,6 +902,7 @@ static void test_1(MYSQL_SESSION session, void *p) {
   params[1].value = (const unsigned char *)"20";
   params[1].length = 2;
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_execute.stmt_id = ctx.stmt_id;
   cmd.com_stmt_execute.open_cursor = true;
   cmd.com_stmt_execute.has_new_types = false;
@@ -907,6 +917,7 @@ static void test_1(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_STMT_EXECUTE, &cmd, &ctx, false, p);
 
   WRITE_STR("FETCH ONE ROW FROM THE CURSOR\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_fetch.stmt_id = ctx.stmt_id;
   cmd.com_stmt_fetch.num_rows = 1;
   run_cmd(session, COM_STMT_FETCH, &cmd, &ctx, false, p);
@@ -916,14 +927,18 @@ static void test_1(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_STMT_FETCH, &cmd, &ctx, false, p);
 
   WRITE_STR("CLOSE THE STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_close.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_CLOSE, &cmd, &ctx, false, p);
 
   WRITE_STR("CLOSE NON-EXISTING STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_close.stmt_id = 100001;
   run_cmd(session, COM_STMT_CLOSE, &cmd, &ctx, false, p);
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_fetch.stmt_id = ctx.stmt_id;
+  cmd.com_stmt_fetch.num_rows = 1;
   WRITE_STR("TRY TO FETCH ONE ROW FROM A DEALLOCATED(CLOSED) PS\n");
   run_cmd(session, COM_STMT_FETCH, &cmd, &ctx, false, p);
 }
@@ -936,11 +951,13 @@ static void test_2(MYSQL_SESSION session, void *p) {
   COM_DATA cmd;
 
   WRITE_STR("CREATE PREPARED STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_prepare.query = "SELECT * from t1 where a > ? and b < ?";
   cmd.com_stmt_prepare.length = strlen(cmd.com_stmt_prepare.query);
   run_cmd(session, COM_STMT_PREPARE, &cmd, &ctx, false, p);
 
   PS_PARAM params[2];
+  memset(params, 0, sizeof(params));
   params[0].type = MYSQL_TYPE_STRING;
   params[0].unsigned_type = false;
   params[0].null_bit = false;
@@ -953,6 +970,7 @@ static void test_2(MYSQL_SESSION session, void *p) {
   params[1].value = (const unsigned char *)"7";
   params[1].length = 1;
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_execute.stmt_id = ctx.stmt_id;
   cmd.com_stmt_execute.parameters = params;
   cmd.com_stmt_execute.parameter_count = 2;
@@ -963,23 +981,29 @@ static void test_2(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_STMT_EXECUTE, &cmd, &ctx, false, p);
 
   WRITE_STR("FETCH ONE ROW\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_fetch.stmt_id = ctx.stmt_id;
+  cmd.com_stmt_fetch.num_rows = 1;
   run_cmd(session, COM_STMT_FETCH, &cmd, &ctx, false, p);
 
   WRITE_STR("RESET THE STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_reset.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_RESET, &cmd, &ctx, false, p);
 
   WRITE_STR("RESET NON-EXISTING STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_reset.stmt_id = 199999;
   run_cmd(session, COM_STMT_RESET, &cmd, &ctx, false, p);
 
   WRITE_STR("TRY TO FETCH ONE ROW FROM THE PS WITH REMOVED CURSOR\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_fetch.num_rows = 1;
   cmd.com_stmt_fetch.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_FETCH, &cmd, &ctx, false, p);
 
   WRITE_STR("CLOSE THE STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_close.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_CLOSE, &cmd, &ctx, false, p);
 }
@@ -992,11 +1016,13 @@ static void test_3(MYSQL_SESSION session, void *p) {
   COM_DATA cmd;
 
   WRITE_STR("CREATE PREPARED STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_prepare.query = "SELECT * from t1 where a > ? and b > ?";
   cmd.com_stmt_prepare.length = strlen(cmd.com_stmt_prepare.query);
   run_cmd(session, COM_STMT_PREPARE, &cmd, &ctx, false, p);
 
   PS_PARAM params[2];
+  memset(params, 0, sizeof(params));
   params[0].type = MYSQL_TYPE_STRING;
   params[0].unsigned_type = false;
   params[0].null_bit = false;
@@ -1009,6 +1035,7 @@ static void test_3(MYSQL_SESSION session, void *p) {
   params[1].value = (const unsigned char *)"3";
   params[1].length = 1;
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_execute.stmt_id = ctx.stmt_id;
   cmd.com_stmt_execute.parameter_count = 2;
   cmd.com_stmt_execute.parameters = params;
@@ -1019,15 +1046,18 @@ static void test_3(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_STMT_EXECUTE, &cmd, &ctx, false, p);
 
   WRITE_STR("TRY TO FETCH ONE ROW FROM A PS WITHOUT CURSOR\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_fetch.num_rows = 1;
   cmd.com_stmt_fetch.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_FETCH, &cmd, &ctx, false, p);
 
   WRITE_STR("TRY TO RESET THE CURSOR FROM A PS WITHOUT CURSOR\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_reset.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_RESET, &cmd, &ctx, false, p);
 
   WRITE_STR("TRY TO CLOSE THE CURSOR FROM A PS WITHOUT CURSOR\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_close.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_CLOSE, &cmd, &ctx, false, p);
 }
@@ -1057,6 +1087,7 @@ static void test_4(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_QUERY, &cmd, &ctx, false, p);
 
   WRITE_STR("CREATE PREPARED STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_prepare.query =
       "INSERT INTO t2(c1, c2, c3, c4, c5, c6, c7, c8, c9) "
       "VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?)";
@@ -1065,6 +1096,8 @@ static void test_4(MYSQL_SESSION session, void *p) {
 
   WRITE_STR("EXECUTE PREPARED STATEMENT WITH PARAMETERS AND CURSOR\n");
   PS_PARAM multi_param[9];
+  memset(multi_param, 0, sizeof(multi_param));
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_execute.stmt_id = ctx.stmt_id;
   cmd.com_stmt_execute.open_cursor = false;
   cmd.com_stmt_execute.has_new_types = true;
@@ -1182,6 +1215,7 @@ static void test_4(MYSQL_SESSION session, void *p) {
   set_query_in_com_data(&cmd, "SELECT * FROM t2");
   run_cmd(session, COM_QUERY, &cmd, &ctx, false, p);
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_close.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_CLOSE, &cmd, &ctx, false, p);
 }
@@ -1200,11 +1234,13 @@ static void test_5(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_QUERY, &cmd, &ctx, false, p);
 
   WRITE_STR("CREATE PREPARED STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_prepare.query =
       "INSERT INTO test_long_data(col1, col2) VALUES(?, ?)";
   cmd.com_stmt_prepare.length = strlen(cmd.com_stmt_prepare.query);
   run_cmd(session, COM_STMT_PREPARE, &cmd, &ctx, false, p);
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_send_long_data.stmt_id = ctx.stmt_id;
   cmd.com_stmt_send_long_data.param_number = 1;
   cmd.com_stmt_send_long_data.length = 8;
@@ -1223,6 +1259,7 @@ static void test_5(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_STMT_SEND_LONG_DATA, &cmd, &ctx, false, p);
 
   PS_PARAM param[3];
+  memset(param, 0, sizeof(param));
   param[0].null_bit = false;
   param[0].length = sizeof(int32);
   param[0].type = MYSQL_TYPE_LONG;
@@ -1243,6 +1280,7 @@ static void test_5(MYSQL_SESSION session, void *p) {
   param[2].type = MYSQL_TYPE_STRING;
   param[2].unsigned_type = false;
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_execute.stmt_id = ctx.stmt_id;
   cmd.com_stmt_execute.open_cursor = false;
   cmd.com_stmt_execute.has_new_types = true;
@@ -1255,6 +1293,7 @@ static void test_5(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_QUERY, &cmd, &ctx, false, p);
 
   // Send long data to non existing prepared statement
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_send_long_data.stmt_id = 199999;
   cmd.com_stmt_send_long_data.param_number = 1;
   cmd.com_stmt_send_long_data.length = 8;
@@ -1277,6 +1316,7 @@ static void test_5(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_STMT_EXECUTE, &cmd, &ctx, false, p);
 
   WRITE_STR("TRY TO CLOSE THE CURSOR FROM A PS WITHOUT CURSOR\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_close.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_CLOSE, &cmd, &ctx, false, p);
 }
@@ -1342,6 +1382,7 @@ static void test_6(MYSQL_SESSION session, void *p) {
                         "END");
   run_cmd(session, COM_QUERY, &cmd, &ctx, false, p);
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_prepare.query = "CALL p1(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
   cmd.com_stmt_prepare.length = strlen(cmd.com_stmt_prepare.query);
   run_cmd(session, COM_STMT_PREPARE, &cmd, &ctx, false, p);
@@ -1440,6 +1481,7 @@ static void test_6(MYSQL_SESSION session, void *p) {
   ps_params[9].unsigned_type = false;
   ps_params[9].null_bit = false;
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_execute.parameters = ps_params;
   cmd.com_stmt_execute.open_cursor = false;
   cmd.com_stmt_execute.stmt_id = ctx.stmt_id;
@@ -1448,6 +1490,7 @@ static void test_6(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_STMT_EXECUTE, &cmd, &ctx, false, p);
 
   WRITE_STR("CLOSE PS\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_close.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_CLOSE, &cmd, &ctx, false, p);
 }
@@ -1460,6 +1503,7 @@ static void test_7(MYSQL_SESSION session, void *p) {
   COM_DATA cmd;
 
   WRITE_STR("CREATE PREPARED STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_prepare.query = "SELECT CONCAT(9< ?)";
   cmd.com_stmt_prepare.length = strlen(cmd.com_stmt_prepare.query);
   run_cmd(session, COM_STMT_PREPARE, &cmd, &ctx, false, p);
@@ -1467,12 +1511,14 @@ static void test_7(MYSQL_SESSION session, void *p) {
   WRITE_STR("EXECUTE PREPARED STATEMENT WITH PARAMETERS AND CURSOR\n");
 
   PS_PARAM params[1];
+  memset(params, 0, sizeof(params));
   params[0].type = MYSQL_TYPE_JSON;
   params[0].unsigned_type = false;
   params[0].null_bit = false;
   params[0].value = (const unsigned char *)"{}";
   params[0].length = 2;
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_execute.stmt_id = ctx.stmt_id;
   cmd.com_stmt_execute.open_cursor = true;
   cmd.com_stmt_execute.has_new_types = false;
@@ -1483,6 +1529,7 @@ static void test_7(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_STMT_EXECUTE, &cmd, &ctx, false, p);
 
   WRITE_STR("CLOSE PS\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_close.stmt_id = ctx.stmt_id;
   run_cmd(session, COM_STMT_CLOSE, &cmd, &ctx, false, p);
 }
@@ -1500,6 +1547,7 @@ static void test_8(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_QUERY, &cmd, &ctx, false, p);
 
   ctx.tables.clear();
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_prepare.query = "CALL proc_set_out_params(?, ?, ?, ?)";
   cmd.com_stmt_prepare.length = strlen(cmd.com_stmt_prepare.query);
   run_cmd(session, COM_STMT_PREPARE, &cmd, &ctx, false, p);
@@ -1507,6 +1555,7 @@ static void test_8(MYSQL_SESSION session, void *p) {
   WRITE_STR("EXECUTE PREPARED STATEMENT WITH PARAMETERS\n");
 
   PS_PARAM params[4];
+  memset(params, 0, sizeof(params));
   std::string values[4]{"@my_v1", "@my_v2", "@my_v3", "@my_v4"};
   params[0].type = MYSQL_TYPE_STRING;
   params[0].unsigned_type = false;
@@ -1530,6 +1579,7 @@ static void test_8(MYSQL_SESSION session, void *p) {
   params[3].length = values[3].length();
 
   ctx.tables.clear();
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_execute.stmt_id = ctx.stmt_id;
   cmd.com_stmt_execute.open_cursor = false;
   cmd.com_stmt_execute.has_new_types = false;
@@ -1562,6 +1612,7 @@ static void test_8(MYSQL_SESSION session, void *p) {
   }
 
   WRITE_STR("CLOSE PS\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_close.stmt_id = ctx.stmt_id;
   ctx.tables.clear();
   run_cmd(session, COM_STMT_CLOSE, &cmd, &ctx, false, p);
@@ -1580,6 +1631,7 @@ static void test_9(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_QUERY, &cmd, &ctx, false, p);
 
   ctx.tables.clear();
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_prepare.query =
       "CALL proc_set_out_params(@my_v1, @my_v2, @my_v3, @my_v4)";
   cmd.com_stmt_prepare.length = strlen(cmd.com_stmt_prepare.query);
@@ -1588,6 +1640,7 @@ static void test_9(MYSQL_SESSION session, void *p) {
   WRITE_STR("EXECUTE PREPARED STATEMENT WITHOUT PARAMETERS\n");
 
   ctx.tables.clear();
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_execute.stmt_id = ctx.stmt_id;
   cmd.com_stmt_execute.open_cursor = false;
   cmd.com_stmt_execute.has_new_types = false;
@@ -1620,6 +1673,7 @@ static void test_9(MYSQL_SESSION session, void *p) {
   }
 
   WRITE_STR("CLOSE PS\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_stmt_close.stmt_id = ctx.stmt_id;
   ctx.tables.clear();
   run_cmd(session, COM_STMT_CLOSE, &cmd, &ctx, false, p);
@@ -1678,6 +1732,49 @@ static void test_10(MYSQL_SESSION session, void *p) {
   run_cmd(session, COM_QUERY, &cmd, &ctx, false, p);
 }
 
+static void test_11(MYSQL_SESSION session, void *p) {
+  DBUG_ENTER("test_11");
+  char buffer[STRING_BUFFER_SIZE];
+
+  Server_context ctx;
+  COM_DATA cmd;
+
+  WRITE_STR("CREATE PREPARED STATEMENT\n");
+  memset(&cmd, 0, sizeof(cmd));
+  cmd.com_stmt_prepare.query = "SELECT * from t1 where a = ?";
+  cmd.com_stmt_prepare.length = strlen(cmd.com_stmt_prepare.query);
+  run_cmd(session, COM_STMT_PREPARE, &cmd, &ctx, false, p);
+
+  PS_PARAM params[1];
+  memset(params, 0, sizeof(params));
+  params[0].type = MYSQL_TYPE_INVALID;
+  params[0].unsigned_type = false;
+  params[0].null_bit = false;
+  params[0].value = (const unsigned char *)"invalid";
+  params[0].length = 1;
+
+  memset(&cmd, 0, sizeof(cmd));
+  cmd.com_stmt_execute.stmt_id = ctx.stmt_id;
+  cmd.com_stmt_execute.parameter_count = 1;
+  cmd.com_stmt_execute.parameters = params;
+  cmd.com_stmt_execute.open_cursor = false;
+  cmd.com_stmt_execute.has_new_types = true;
+
+  WRITE_STR("EXECUTE THE PS WITH INVALID PARAMETER TYPE\n");
+  run_cmd(session, COM_STMT_EXECUTE, &cmd, &ctx, false, p);
+
+  params[0].type = MYSQL_TYPE_BOOL;
+  params[0].unsigned_type = false;
+  params[0].null_bit = false;
+  params[0].value = (const unsigned char *)"bool";
+  params[0].length = 1;
+
+  WRITE_STR("EXECUTE THE PS WITH BOOL PARAMETER TYPE\n");
+  run_cmd(session, COM_STMT_EXECUTE, &cmd, &ctx, false, p);
+
+  DBUG_VOID_RETURN;
+}
+
 static void tear_down_test(MYSQL_SESSION session, void *p) {
   DBUG_TRACE;
 
@@ -1734,6 +1831,7 @@ static struct my_stmt_tests_st my_tests[] = {
     {"Test COM_STMT_EXECUTE with out-params as placeholders", test_8},
     {"Test COM_STMT_EXECUTE with out-params as variables", test_9},
     {"Test COM_QUERY with out-params as placeholders", test_10},
+    {"Test COM_STMT_EXECUTE with wrong parameters", test_11},
     {nullptr, nullptr}};
 
 static void test_sql(void *p) {
