@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -41,8 +41,6 @@
 #include "sql_string.h" /* STRING_PSI_MEMORY_KEY */
 #include "template_utils.h"
 
-struct CHARSET_INFO;
-
 static const char *log_filename = "test_sql_views_triggers";
 
 struct st_test_statement {
@@ -196,7 +194,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
-  DBUG_PRINT("info", ("resultcs->m_coll_name: %s", resultcs->m_coll_name));
+  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
   pctx->num_cols = num_cols;
   pctx->resultcs = resultcs;
   pctx->current_col = 0;
@@ -269,8 +267,8 @@ static ulong sql_get_client_capabilities(void *) {
 static int sql_get_null(void *ctx) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
   memcpy(pctx->sql_str_value[row][col], "[NULL]", sizeof("[NULL]"));
@@ -282,13 +280,12 @@ static int sql_get_null(void *ctx) {
 static int sql_get_integer(void *ctx, longlong value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len =
-      snprintf(pctx->sql_str_value[row][col],
-               sizeof(pctx->sql_str_value[row][col]), "%lld", value);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]), "%lld", value);
   pctx->sql_str_len[row][col] = len;
 
   return false;
@@ -297,13 +294,13 @@ static int sql_get_integer(void *ctx, longlong value) {
 static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(pctx->sql_str_value[row][col],
-                              sizeof(pctx->sql_str_value[row][col]),
-                              is_unsigned ? "%llu" : "%lld", value);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]),
+                        is_unsigned ? "%llu" : "%lld", value);
 
   pctx->sql_str_len[row][col] = len;
 
@@ -320,8 +317,8 @@ static const char *test_decimal_as_string(char *buff, const decimal_t *val,
 static int sql_get_decimal(void *ctx, const decimal_t *value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
   int len = SIZEOF_SQL_STR_VALUE;
@@ -334,13 +331,12 @@ static int sql_get_decimal(void *ctx, const decimal_t *value) {
 static int sql_get_double(void *ctx, double value, uint32) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len =
-      snprintf(pctx->sql_str_value[row][col],
-               sizeof(pctx->sql_str_value[row][col]), "%3.7g", value);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]), "%3.7g", value);
 
   pctx->sql_str_len[row][col] = len;
 
@@ -350,11 +346,11 @@ static int sql_get_double(void *ctx, double value, uint32) {
 static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len =
+  size_t len =
       snprintf(pctx->sql_str_value[row][col],
                sizeof(pctx->sql_str_value[row][col]), "%s%4d-%02d-%02d",
                value->neg ? "-" : "", value->year, value->month, value->day);
@@ -366,11 +362,11 @@ static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
 static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(
+  size_t len = snprintf(
       pctx->sql_str_value[row][col], sizeof(pctx->sql_str_value[row][col]),
       "%s%02d:%02d:%02d", value->neg ? "-" : "",
       value->day ? (value->day * 24 + value->hour) : value->hour, value->minute,
@@ -383,11 +379,11 @@ static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint) {
 static int sql_get_datetime(void *ctx, const MYSQL_TIME *value, uint) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(
+  size_t len = snprintf(
       pctx->sql_str_value[row][col], sizeof(pctx->sql_str_value[row][col]),
       "%s%4d-%02d-%02d %02d:%02d:%02d", value->neg ? "-" : "", value->year,
       value->month, value->day, value->hour, value->minute, value->second);
@@ -401,8 +397,8 @@ static int sql_get_string(void *ctx, const char *const value, size_t length,
                           const CHARSET_INFO *const) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
   strncpy(pctx->sql_str_value[row][col], value, length);
@@ -590,7 +586,7 @@ static void dump_cs_info(const CHARSET_INFO *cs) {
 
   WRITE_VAL("\t\t[meta][charset result] number: %d\n", cs->number);
   WRITE_VAL("\t\t[meta][charset result] name: %s\n", cs->csname);
-  WRITE_VAL("\t\t[meta][charset result] collation: %s\n", cs->m_coll_name);
+  WRITE_VAL("\t\t[meta][charset result] collation: %s\n", cs->name);
   WRITE_VAL("\t\t[meta][charset result] sort order: %s\n", cs->sort_order);
 }
 
@@ -600,7 +596,7 @@ static void dump_decoded_server_status(const char *prefix, uint server_status) {
   WRITE_VAL("%u\n", server_status);
   WRITE_STR(prefix);
   for (int i = 0; i < 30; i++) {
-    const uint flag = 1 << i;
+    uint flag = 1 << i;
     if (server_status & flag) {
 #define FLAG_DELIMITER " "
       switch (flag) {
@@ -716,7 +712,7 @@ static void set_query_in_com_data(const char *query, union COM_DATA *cmd) {
 
 static void run_statement(MYSQL_SESSION session, const char *query,
                           struct st_plugin_ctx *ctx, bool generates_result_set,
-                          void *p [[maybe_unused]]) {
+                          void *p MY_ATTRIBUTE((unused))) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
 
@@ -728,9 +724,9 @@ static void run_statement(MYSQL_SESSION session, const char *query,
   WRITE_STR("[CS_TEXT_REPRESENTATION]\n");
 again:
   ctx->reset();
-  const int fail = command_service_run_command(
-      session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci,
-      &protocol_callbacks, txt_or_bin, ctx);
+  int fail = command_service_run_command(session, COM_QUERY, &cmd,
+                                         &my_charset_utf8_general_ci,
+                                         &protocol_callbacks, txt_or_bin, ctx);
   if (fail) {
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "run_statement code: %d\n",
                     fail);
@@ -754,7 +750,7 @@ again:
 
 void static change_current_db(MYSQL_SESSION session, const char *db,
                               struct st_plugin_ctx *ctx,
-                              void *p [[maybe_unused]]) {
+                              void *p MY_ATTRIBUTE((unused))) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
   cmd.com_init_db.db_name = db;
@@ -763,8 +759,8 @@ void static change_current_db(MYSQL_SESSION session, const char *db,
   WRITE_DASHED_LINE();
   WRITE_VAL("EXECUTING:[COM_INIT_DB][%s]\n", db);
   ctx->reset();
-  const int fail = command_service_run_command(
-      session, COM_INIT_DB, &cmd, &my_charset_utf8mb3_general_ci,
+  int fail = command_service_run_command(
+      session, COM_INIT_DB, &cmd, &my_charset_utf8_general_ci,
       &protocol_callbacks, CS_TEXT_REPRESENTATION, ctx);
   if (fail)
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "change db code: %d\n",
@@ -778,8 +774,7 @@ static void test_selects(MYSQL_SESSION session, void *p) {
   struct st_plugin_ctx *plugin_ctx = new st_plugin_ctx();
 
   const char *last_db = nullptr;
-  const size_t stmt_count =
-      sizeof(test_query_plan) / sizeof(test_query_plan[0]);
+  size_t stmt_count = sizeof(test_query_plan) / sizeof(test_query_plan[0]);
   for (size_t i = 0; i < stmt_count; i++) {
     /* Change current DB if needed */
     if (last_db != test_query_plan[i].db) {
@@ -803,7 +798,7 @@ static void test_sql(void *p) {
   DBUG_TRACE;
 
   WRITE_STR("[srv_session_open]\n");
-  MYSQL_SESSION session = srv_session_open(nullptr, nullptr);
+  MYSQL_SESSION session = srv_session_open(NULL, NULL);
   if (!session) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "srv_session_open failed.");
     return;
@@ -895,7 +890,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
+static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
