@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -20,7 +20,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <assert.h>
 #include <mysql/components/my_service.h>
 #include <mysql/components/services/dynamic_privilege.h>
 #include <mysql/plugin_audit.h>
@@ -40,7 +39,7 @@
 #include "m_string.h"
 #include "map_helpers.h"
 #include "my_compiler.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_psi_config.h"
 #include "my_systime.h"  // TIMEOUT_INF, Timeout_type
@@ -50,7 +49,6 @@
 #include "sql/locking_service.h"
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
-#include "string_with_len.h"
 
 #ifdef WIN32
 #define PLUGIN_EXPORT extern "C" __declspec(dllexport)
@@ -106,8 +104,9 @@ class atomic_boolean {
     @retval false the atomic boolean value is different from the argument value
   */
   bool is_set(bool value = true) {
-    const int32 cmp = value ? m_true : m_false;
-    const int32 actual_value = m_value.load();
+    int32 cmp = value ? m_true : m_false, actual_value;
+
+    actual_value = m_value.load();
 
     return actual_value == cmp;
   }
@@ -118,7 +117,7 @@ class atomic_boolean {
     @param new_value value to set
   */
   void set(bool new_value) {
-    const int32 new_val = new_value ? m_true : m_false;
+    int32 new_val = new_value ? m_true : m_false;
     m_value.store(new_val);
   }
 };
@@ -194,7 +193,7 @@ static bool is_blank_string(char *input) {
   Check if user either has SUPER or VERSION_TOKEN_ADMIN privileges
   @param thd Thread handle
 
-  @return success state
+  @return succcess state
     @retval true User has the required privileges
     @retval false User has not the required privileges
 */
@@ -297,10 +296,10 @@ static int parse_vtokens(char *input, enum command type) {
   const char *separator = ";";
   int result = 0;
   THD *thd = current_thd;
-  const ulonglong thd_session_number = THDVAR(thd, session_number);
-  const ulonglong tmp_token_number = (ulonglong)session_number.load();
+  ulonglong thd_session_number = THDVAR(thd, session_number);
+  ulonglong tmp_token_number = (ulonglong)session_number.load();
 
-  const bool vtokens_unchanged = (thd_session_number == tmp_token_number);
+  bool vtokens_unchanged = (thd_session_number == tmp_token_number);
 
   token = my_strtok_r(input, separator, &lasts_token);
 
@@ -420,7 +419,7 @@ static int parse_vtokens(char *input, enum command type) {
 }
 
 /**
-  Audit API entry point for the version token plugin
+  Audit API entry point for the version token pluign
 
   Plugin audit function to compare session version tokens with
   the global ones.
@@ -431,23 +430,23 @@ static int parse_vtokens(char *input, enum command type) {
   compare their values with the ones found. Throws errors if not
   found or the version values do not match. See parse_vtokens().
   At query end (MYSQL_AUDIT_GENERAL_STATUS currently) it releases
-  the GET_LOCK shared locks it has acquired.
+  the GET_LOCK shared locks it has aquired.
 
   @param thd          The current thread
   @param event_class  audit API event class
   @param event        pointer to the audit API event data
 */
-static int version_token_check(MYSQL_THD thd,
-                               mysql_event_class_t event_class [[maybe_unused]],
-                               const void *event) {
+static int version_token_check(
+    MYSQL_THD thd, mysql_event_class_t event_class MY_ATTRIBUTE((unused)),
+    const void *event) {
   char *sess_var;
 
   const struct mysql_event_general *event_general =
       (const struct mysql_event_general *)event;
   const uchar *command = (const uchar *)event_general->general_command.str;
-  const size_t length = event_general->general_command.length;
+  size_t length = event_general->general_command.length;
 
-  assert(event_class == MYSQL_AUDIT_GENERAL_CLASS);
+  DBUG_ASSERT(event_class == MYSQL_AUDIT_GENERAL_CLASS);
 
   switch (event_general->event_subclass) {
     case MYSQL_AUDIT_GENERAL_LOG: {
@@ -512,7 +511,7 @@ class vtoken_lock_cleanup {
   atomic_boolean activated;
 
  public:
-  vtoken_lock_cleanup() = default;
+  vtoken_lock_cleanup() {}
   ~vtoken_lock_cleanup() {
     if (activated.is_set()) mysql_rwlock_destroy(&LOCK_vtoken_hash);
   }
@@ -537,7 +536,7 @@ static struct st_mysql_audit version_token_descriptor = {
 };
 
 /** Plugin init. */
-static int version_tokens_init(void *arg [[maybe_unused]]) {
+static int version_tokens_init(void *arg MY_ATTRIBUTE((unused))) {
 #ifdef HAVE_PSI_INTERFACE
   // Initialize psi keys.
   vtoken_init_psi_keys();
@@ -553,7 +552,7 @@ static int version_tokens_init(void *arg [[maybe_unused]]) {
     // Lock for version number.
     cleanup_lock.activate();
   }
-  mysql_service_status_t ret = 0;
+  bool ret = false;
   SERVICE_TYPE(registry) *r = mysql_plugin_registry_acquire();
   {
     my_service<SERVICE_TYPE(dynamic_privilege_register)> service(
@@ -568,7 +567,7 @@ static int version_tokens_init(void *arg [[maybe_unused]]) {
 }
 
 /** Plugin deinit. */
-static int version_tokens_deinit(void *arg [[maybe_unused]]) {
+static int version_tokens_deinit(void *arg MY_ATTRIBUTE((unused))) {
   SERVICE_TYPE(registry) *r = mysql_plugin_registry_acquire();
   {
     my_service<SERVICE_TYPE(dynamic_privilege_register)> service(
@@ -664,7 +663,7 @@ PLUGIN_EXPORT char *version_tokens_set(UDF_INIT *, UDF_ARGS *args, char *result,
                                        unsigned long *length, unsigned char *,
                                        unsigned char *error) {
   char *hash_str;
-  const int len = args->lengths[0];
+  int len = args->lengths[0];
   int vtokens_count = 0;
   std::stringstream ss;
 
@@ -744,7 +743,7 @@ PLUGIN_EXPORT char *version_tokens_edit(UDF_INIT *, UDF_ARGS *args,
                                         char *result, unsigned long *length,
                                         unsigned char *, unsigned char *error) {
   char *hash_str;
-  const int len = args->lengths[0];
+  int len = args->lengths[0];
   std::stringstream ss;
   int vtokens_count = 0;
 
@@ -1012,9 +1011,9 @@ PLUGIN_EXPORT bool version_tokens_lock_shared_init(UDF_INIT *initid,
 PLUGIN_EXPORT long long version_tokens_lock_shared(UDF_INIT *, UDF_ARGS *args,
                                                    unsigned char *,
                                                    unsigned char *error) {
-  const long long timeout = args->args[args->arg_count - 1] ?  // Null ?
-                                *((long long *)args->args[args->arg_count - 1])
-                                                            : -1;
+  long long timeout = args->args[args->arg_count - 1] ?  // Null ?
+                          *((long long *)args->args[args->arg_count - 1])
+                                                      : -1;
 
   if (timeout < 0) {
     my_error(ER_DATA_OUT_OF_RANGE, MYF(0), "timeout",
@@ -1040,9 +1039,9 @@ PLUGIN_EXPORT long long version_tokens_lock_exclusive(UDF_INIT *,
                                                       UDF_ARGS *args,
                                                       unsigned char *,
                                                       unsigned char *error) {
-  const long long timeout = args->args[args->arg_count - 1] ?  // Null ?
-                                *((long long *)args->args[args->arg_count - 1])
-                                                            : -1;
+  long long timeout = args->args[args->arg_count - 1] ?  // Null ?
+                          *((long long *)args->args[args->arg_count - 1])
+                                                      : -1;
 
   if (timeout < 0) {
     my_error(ER_DATA_OUT_OF_RANGE, MYF(0), "timeout",
