@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -321,8 +320,7 @@ void Client::handle_message(ngs::Message_request *request) {
           s->handle_message(*request);
         }
         break;
-      }
-      [[fallthrough]];
+      }  // Fall through.
 
     default:
       // invalid message at this time
@@ -422,7 +420,7 @@ void Client::on_client_addr() {
     } break;
 
     case xpl::Connection_namedpipe:
-    case xpl::Connection_unixsocket:
+    case xpl::Connection_unixsocket:  // fall through
       m_client_host = "localhost";
       return;
 
@@ -449,8 +447,8 @@ void Client::on_client_addr() {
 
 void Client::on_accept() {
   DBUG_TRACE;
-  log_debug("%s: Accepted client connection from %s (sock:" MY_SOCKET_FMT ")",
-            client_id(), client_address(), m_connection->get_fd());
+  log_debug("%s: Accepted client connection from %s (sock:%i)", client_id(),
+            client_address(), m_connection->get_fd());
 
   DBUG_EXECUTE_IF("client_accept_timeout", {
     int32_t i = 0;
@@ -513,7 +511,7 @@ void Client::on_session_auth_success(xpl::iface::Session *) {
   }
 }
 
-void Client::on_session_close(xpl::iface::Session *s [[maybe_unused]]) {
+void Client::on_session_close(xpl::iface::Session *s MY_ATTRIBUTE((unused))) {
   log_debug("%s: Session %i removed", client_id(), s->session_id());
 
   // no more open sessions, disconnect
@@ -521,7 +519,7 @@ void Client::on_session_close(xpl::iface::Session *s [[maybe_unused]]) {
   remove_client_from_server();
 }
 
-void Client::on_session_reset(xpl::iface::Session *s [[maybe_unused]]) {
+void Client::on_session_reset(xpl::iface::Session *s MY_ATTRIBUTE((unused))) {
   log_debug("%s: Resetting session %i", client_id(), s->session_id());
 
   if (!create_session()) {
