@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -48,9 +47,7 @@ static int audit_cache_clean_event_notify(MYSQL_THD thd,
                                           const void *event) {
   if (event_class == MYSQL_AUDIT_SERVER_STARTUP_CLASS) {
     auto server_obj_with_lock = modules::Module_mysqlx::get_instance_server();
-    if (server_obj_with_lock.container())
-      server_obj_with_lock->delayed_start_tasks();
-
+    if (server_obj_with_lock.container()) server_obj_with_lock->start_tasks();
     return 0;
   }
 
@@ -89,11 +86,11 @@ static int audit_cache_clean_event_notify(MYSQL_THD thd,
     if (subclass == MYSQL_AUDIT_AUTHENTICATION_CREDENTIAL_CHANGE ||
         subclass == MYSQL_AUDIT_AUTHENTICATION_AUTHID_RENAME ||
         subclass == MYSQL_AUDIT_AUTHENTICATION_AUTHID_DROP) {
-#ifndef NDEBUG
-      // "user" variable is going to be unused when the NDEBUG is defined
+#ifndef DBUG_OFF
+      // "user" variable is going to be unused when the DBUG_OFF is defined
       auto user = authentication_event->user;
-      assert(user.str[user.length] == '\0');
-#endif  // NDEBUG
+      DBUG_ASSERT(user.str[user.length] == '\0');
+#endif  // DBUG_OFF
       sha256_password_cache->remove(authentication_event->user.str,
                                     authentication_event->host.str);
     }
