@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,7 +42,7 @@ namespace detail {
 template <class T>
 class PFS_allocator : public std::allocator<T> {
  public:
-  PFS_allocator() = default;
+  PFS_allocator() {}
 
   template <class U>
   PFS_allocator(PFS_allocator<U> const &) {}
@@ -53,7 +52,7 @@ class PFS_allocator : public std::allocator<T> {
     typedef PFS_allocator<U> other;
   };
 
-  T *allocate(size_t n, const void *hint [[maybe_unused]] = nullptr) {
+  T *allocate(size_t n, const void *hint MY_ATTRIBUTE((unused)) = nullptr) {
     return reinterpret_cast<T *>(my_malloc(
         IS_PSI_AVAILABLE(KEY_memory_x_objects, 0), sizeof(T) * n, MYF(MY_WME)));
   }
