@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -46,8 +45,6 @@
 #include "plugin/x/src/notices.h"
 #include "plugin/x/src/xpl_log.h"
 
-struct CHARSET_INFO;
-
 namespace xpl {
 
 using google::protobuf::io::CodedOutputStream;
@@ -296,15 +293,15 @@ int Streaming_command_delegate::field_metadata(struct st_send_field *field,
 
     case MYSQL_TYPE_BOOL:
     case MYSQL_TYPE_INVALID:
-      assert(false);
+      DBUG_ASSERT(false);
       break;
 
     default:
-      assert(false);  // Shouldn't happen
+      DBUG_ASSERT(false);  // Shouldn't happen
   }
 
-  assert(column_info.get()->m_type !=
-         (Mysqlx::Resultset::ColumnMetaData::FieldType)0);
+  DBUG_ASSERT(column_info.get()->m_type !=
+              (Mysqlx::Resultset::ColumnMetaData::FieldType)0);
 
   if (!m_compact_metadata) {
     column_info.set_non_compact_data("def", field->col_name, field->table_name,
@@ -489,7 +486,7 @@ int Streaming_command_delegate::get_string(const char *const value,
         m_proto->row_builder()->field_set(conv.get_ptr(), conv.get_length());
         break;
       }
-      [[fallthrough]];
+      /* fall through */
     default: {
       Convert_if_necessary conv(m_resultcs, value, length, valuecs);
       m_proto->row_builder()->field_string(conv.get_ptr(), conv.get_length());
