@@ -29,7 +29,7 @@
 
 #include "my_inttypes.h"  // NOLINT(build/include_subdir)
 #include "my_sys.h"       // NOLINT(build/include_subdir)
-#include "mysql/psi/psi_base.h"
+#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/service_mysql_alloc.h"
 #include "plugin/x/generated/mysqlx_version.h"
 #include "plugin/x/src/interface/client.h"
