@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -24,8 +24,10 @@
 
 #include "plugin/x/tests/driver/connector/session_holder.h"
 
-#include "my_dbug.h"
-#include "template_utils.h"
+#include <array>
+
+#include "my_dbug.h"         // NOLINT(build/include_subdir)
+#include "template_utils.h"  // NOLINT(build/include_subdir)
 
 #include "plugin/x/protocol/stream/compression/compression_algorithm_lz4.h"
 #include "plugin/x/protocol/stream/compression/compression_algorithm_zlib.h"
@@ -359,17 +361,18 @@ xcl::Handler_result Session_holder::count_received_messages(
   if (msg_name != Mysqlx::Notice::Frame::descriptor()->full_name())
     return xcl::Handler_result::Continue;
 
-  static const std::string notice_type_id[] = {
+  static const std::array<std::string, 5> k_notice_type_id = {
       Mysqlx::Notice::Warning::descriptor()->full_name(),
       Mysqlx::Notice::SessionVariableChanged::descriptor()->full_name(),
       Mysqlx::Notice::SessionStateChanged::descriptor()->full_name(),
       Mysqlx::Notice::GroupReplicationStateChanged::descriptor()->full_name(),
-      Mysqlx::Notice::ServerHello::descriptor()->full_name()};
+      Mysqlx::Notice::ServerHello::descriptor()->full_name(),
+  };
 
   const auto notice_type =
       static_cast<const Mysqlx::Notice::Frame *>(&msg)->type() - 1u;
-  if (notice_type < array_elements(notice_type_id))
-    ++m_received_msg_counters[notice_type_id[notice_type]];
+  if (notice_type < k_notice_type_id.size())
+    ++m_received_msg_counters[k_notice_type_id[notice_type]];
 
   /** None of processed messages should be filtered out*/
   return xcl::Handler_result::Continue;
@@ -403,5 +406,9 @@ xcl::Handler_result Session_holder::dump_notices(const xcl::XProtocol *protocol,
 
 void Session_holder::print_message(const std::string &direction,
                                    const xcl::XProtocol::Message &msg) {
+#if (defined(GOOGLE_PROTOBUF_VERSION) && GOOGLE_PROTOBUF_VERSION > 3000000)
+  m_console.print(direction, msg.ByteSizeLong() + 1, " ", msg);
+#else
   m_console.print(direction, msg.ByteSize() + 1, " ", msg);
+#endif
 }
