@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,9 +30,7 @@
 #define USE_EVENTFD
 
 #ifdef HAVE_EPOLL
-#include <chrono>
 #include <mutex>
-#include <optional>
 #include <system_error>
 #include <unordered_map>
 
@@ -42,9 +39,7 @@
 #endif
 
 #include <iostream>
-#include <sstream>
 
-#include "my_compiler.h"  // MY_COMPILER_*
 #include "mysql/harness/net_ts/impl/io_service_base.h"
 #include "mysql/harness/net_ts/impl/linux_epoll.h"
 #include "mysql/harness/net_ts/impl/socket_error.h"
@@ -59,11 +54,6 @@ class linux_epoll_io_service : public IoServiceBase {
  public:
   using native_handle_type = impl::socket::native_handle_type;
 
-  static constexpr const int kSettableEvents = EPOLLIN | EPOLLOUT;
-  static constexpr const int kAlwaysEnabledEvents = EPOLLHUP | EPOLLERR;
-  static constexpr const int kAllEvents =
-      kSettableEvents | kAlwaysEnabledEvents;
-
   ~linux_epoll_io_service() override { close(); }
 
   bool is_open() const noexcept {
@@ -75,11 +65,12 @@ class linux_epoll_io_service : public IoServiceBase {
 
   stdx::expected<void, std::error_code> open() noexcept override {
     if (is_open()) {
-      return stdx::unexpected(make_error_code(net::socket_errc::already_open));
+      return stdx::make_unexpected(
+          make_error_code(net::socket_errc::already_open));
     }
 
     auto res = impl::epoll::create();
-    if (!res) return stdx::unexpected(res.error());
+    if (!res) return stdx::make_unexpected(res.error());
 
     epfd_ = *res;
 #if defined(USE_EVENTFD)
@@ -91,7 +82,7 @@ class linux_epoll_io_service : public IoServiceBase {
     }
 #endif
     auto pipe_res = impl::file::pipe(O_NONBLOCK);
-    if (!pipe_res) return stdx::unexpected(pipe_res.error());
+    if (!pipe_res) return stdx::make_unexpected(pipe_res.error());
 
     wakeup_fds_ = *pipe_res;
 
@@ -170,9 +161,9 @@ class linux_epoll_io_service : public IoServiceBase {
   }
 
   stdx::expected<void, std::error_code> close() {
-    if (wakeup_fds_.first != impl::file::kInvalidHandle) {
-      remove_fd(wakeup_fds_.first);
+    remove_fd(wakeup_fds_.first);
 
+    if (wakeup_fds_.first != impl::file::kInvalidHandle) {
       impl::file::close(wakeup_fds_.first);
       wakeup_fds_.first = impl::file::kInvalidHandle;
     }
@@ -200,55 +191,54 @@ class linux_epoll_io_service : public IoServiceBase {
     stdx::expected<void, std::error_code> merge(int epfd, native_handle_type fd,
                                                 impl::socket::wait_type wt,
                                                 bool oneshot) {
-      uint32_t new_events{};
+      epoll_event ev{};
       switch (wt) {
         case impl::socket::wait_type::wait_read:
-          new_events = EPOLLIN;
+          ev.events = EPOLLIN;
           break;
         case impl::socket::wait_type::wait_write:
-          new_events = EPOLLOUT;
+          ev.events = EPOLLOUT;
           break;
         case impl::socket::wait_type::wait_error:
-          new_events = EPOLLERR | EPOLLHUP;
+          ev.events = EPOLLERR;
           break;
       }
 
-      epoll_event ev{};
       ev.data.fd = fd;
-      new_events |= EPOLLET;
+      ev.events |= EPOLLET;
 
       if (oneshot) {
-        new_events |= EPOLLONESHOT;
+        ev.events |= EPOLLONESHOT;
       }
 
       auto &b = bucket(fd);
 
       std::lock_guard<std::mutex> lk(b.mtx_);
       const auto it = b.interest_.find(fd);
+      if (it != b.interest_.end()) {
+        // found, let's modify to handle the old and the new interest
 
-      auto old_events = (it == b.interest_.end()) ? 0 : it->second;
-      auto merged_events = new_events | old_events;
+        // std::cerr << __LINE__ << ": merge: " << fd << std::endl;
 
-      // the events passed to epoll should only contain IN|OUT
-      ev.events = merged_events & ~kAlwaysEnabledEvents;
-
-      if ((old_events & kAllEvents) == 0) {
-        // no events where registered before, add.
-        const auto ctl_res =
-            impl::epoll::ctl(epfd, impl::epoll::Cmd::add, fd, &ev);
-        if (!ctl_res) return ctl_res;
-      } else {
+        ev.events |= it->second;
         const auto ctl_res =
             impl::epoll::ctl(epfd, impl::epoll::Cmd::mod, fd, &ev);
         if (!ctl_res) return ctl_res;
-      }
 
-      // the tracked events should contain IN|OUT|ERR|HUP
-      if (it != b.interest_.end()) {
-        it->second = merged_events;
-      } else {
-        b.interest_.emplace(fd, merged_events);
+        it->second = ev.events;
+        return {};
       }
+      // std::cerr << __LINE__ << ": add: " << fd << " "
+      //           << std::bitset<32>(ev.events) << std::endl;
+
+      // not found, let's add it.
+
+      const auto ctl_res =
+          impl::epoll::ctl(epfd, impl::epoll::Cmd::add, fd, &ev);
+      if (!ctl_res) return ctl_res;
+
+      const uint32_t ev_events = ev.events;
+      b.interest_.emplace(fd, ev_events);
 
       return {};
     }
@@ -263,16 +253,14 @@ class linux_epoll_io_service : public IoServiceBase {
       // may be called from another thread through ->cancel()
       const auto it = b.interest_.find(fd);
       if (it != b.interest_.end()) {
-        if ((it->second & kAllEvents) != 0) {
-          auto epoll_ctl_res =
-              impl::epoll::ctl(epfd, impl::epoll::Cmd::del, fd, nullptr);
-          if (!epoll_ctl_res) return epoll_ctl_res;
-        }
+        auto epoll_ctl_res =
+            impl::epoll::ctl(epfd, impl::epoll::Cmd::del, fd, nullptr);
+        if (!epoll_ctl_res) return epoll_ctl_res;
 
         b.interest_.erase(it);
       } else {
         // return ENOENT as epoll_ctl() would do
-        return stdx::unexpected(
+        return stdx::make_unexpected(
             make_error_code(std::errc::no_such_file_or_directory));
       }
 
@@ -287,35 +275,32 @@ class linux_epoll_io_service : public IoServiceBase {
       std::lock_guard<std::mutex> lk(b.mtx_);
 
       const auto it = b.interest_.find(fd);
-      if (it == b.interest_.end()) {
-        // return ENOENT as epoll_ctl() would do
-        return stdx::unexpected(
-            make_error_code(std::errc::no_such_file_or_directory));
-      }
+      if (it != b.interest_.end()) {
+        // fd is found
 
-      // fd is found
-      auto &interest = *it;
+        // check if the register interest is aware of the interest we want to
+        // remove
+        auto const ev_mask = EPOLLIN | EPOLLOUT | EPOLLERR;
+        if (0 == ((it->second & ev_mask) & (revent & ev_mask))) {
+          return stdx::make_unexpected(
+              make_error_code(std::errc::argument_out_of_domain));
+        }
 
-      // one-shot-events which fired
-      const auto fd_events = revent & kAllEvents;
-      const auto updated_fd_events = interest.second & ~fd_events;
+        it->second &= ~revent;
 
-      if ((updated_fd_events & kSettableEvents) != 0) {
         epoll_event ev{};
         ev.data.fd = fd;
-        ev.events = updated_fd_events & ~kAlwaysEnabledEvents;
+        ev.events = it->second;
 
         const auto ctl_res =
             impl::epoll::ctl(epfd, impl::epoll::Cmd::mod, fd, &ev);
-        if (!ctl_res) return stdx::unexpected(ctl_res.error());
-      } else if ((updated_fd_events & kAllEvents) == 0) {
-        const auto ctl_res =
-            impl::epoll::ctl(epfd, impl::epoll::Cmd::del, fd, nullptr);
-        if (!ctl_res) return stdx::unexpected(ctl_res.error());
+        if (!ctl_res) return ctl_res;
+      } else {
+        // return ENOENT as epoll_ctl() would do
+        return stdx::make_unexpected(
+            make_error_code(std::errc::no_such_file_or_directory));
       }
 
-      interest.second = updated_fd_events;
-
       return {};
     }
 
@@ -331,85 +316,53 @@ class linux_epoll_io_service : public IoServiceBase {
       const auto it = b.interest_.find(fd);
       if (it == b.interest_.end()) {
         // return ENOENT as epoll_ctl() would do
-        return stdx::unexpected(
+        return stdx::make_unexpected(
             make_error_code(std::errc::no_such_file_or_directory));
       }
 
-      auto &interest = *it;
-
-      if (!(interest.second & EPOLLONESHOT)) {
+      if (!(it->second & EPOLLONESHOT)) {
         // not a oneshot event. The interest hasn't changed.
         return {};
       }
 
-      // check that the one-shot-events IN and OUT are expected and tracked.
-      //
-      // interest   | revent   | result
-      // -----------+----------+-------
-      // {}         | {IN}     | Fail
-      // {}         | {OUT}    | Fail
-      // {}         | {IN,OUT} | Fail
-      // {}         | {ERR}    | Ok({})
-      // {}         | {IN,ERR} | Fail
-      // {IN}       | {IN}     | Ok({})
-      // {IN}       | {OUT}    | Fail
-      // {IN}       | {IN,OUT} | Fail
-      // {IN}       | {ERR}    | Ok({IN})
-      // {IN}       | {IN,ERR} | Ok({})
-      // {IN,OUT}   | {IN}     | Ok({OUT})
-      // {IN,OUT}   | {OUT}    | Ok({IN})
-      // {IN,OUT}   | {IN,OUT} | Ok({})
-      // {IN,OUT}   | {ERR}    | Ok({IN,OUT})
-      // {IN,OUT}   | {IN,ERR} | Ok({OUT})
-
-      // events which fired
-      const auto fd_events = revent & kAllEvents;
-
-      // events that we are interested in.
-      const auto fd_interest = interest.second & kAllEvents;
-
-      if (fd_events != 0 &&  //
-          (fd_events & fd_interest) == 0) {
-        std::cerr << "after_event_fired(" << fd << ", "
-                  << std::bitset<32>(fd_events) << ") not in "
-                  << std::bitset<32>(fd_interest) << std::endl;
-        return stdx::unexpected(
+      // check the interest we remove is actually part of the requested set
+      auto const ev_mask = EPOLLIN | EPOLLOUT | EPOLLERR;
+      if (((it->second & ev_mask) & (revent & ev_mask)) == 0) {
+        std::cerr << "after_event_fired(" << fd << ",  "
+                  << std::bitset<32>(revent) << ") not in "
+                  << std::bitset<32>(it->second) << std::endl;
+        return stdx::make_unexpected(
             make_error_code(std::errc::argument_out_of_domain));
       }
 
       // update the fd-interest
-      const auto updated_fd_events = interest.second & ~fd_events;
+      it->second &= ~revent;
 
-      if ((updated_fd_events & kSettableEvents) != 0) {
+      if (it->second != 0) {
         // if a one shot event with multiple waiting events fired for one of the
         // events, it removes all interests for the fd.
         //
         // waiting for:      IN|OUT
         // fires:            IN
-        // epoll.interesting:0
+        // epoll.intersting: 0
         // not fired:        OUT
         //
-        // add back the events that have not fired yet.
+        // add back the events that havn't fired yet.
         epoll_event ev{};
         ev.data.fd = fd;
-        ev.events = updated_fd_events & ~kAlwaysEnabledEvents;
+        ev.events = it->second;
 
         const auto ctl_res =
             impl::epoll::ctl(epfd, impl::epoll::Cmd::mod, fd, &ev);
-        if (!ctl_res) return stdx::unexpected(ctl_res.error());
-      } else if ((updated_fd_events & kAllEvents) == 0) {
-        // no interest anymore.
-        const auto ctl_res =
-            impl::epoll::ctl(epfd, impl::epoll::Cmd::del, fd, nullptr);
-        if (!ctl_res) return stdx::unexpected(ctl_res.error());
+        if (!ctl_res) {
+          return ctl_res.get_unexpected();
+        }
       }
 
-      interest.second = updated_fd_events;
-
       return {};
     }
 
-    std::optional<int32_t> interest(native_handle_type fd) const {
+    stdx::expected<int32_t, void> interest(native_handle_type fd) const {
       auto &b = bucket(fd);
 
       std::lock_guard<std::mutex> lk(b.mtx_);
@@ -418,7 +371,7 @@ class linux_epoll_io_service : public IoServiceBase {
       if (it != b.interest_.end()) {
         return it->second;
       } else {
-        return std::nullopt;
+        return stdx::make_unexpected();
       }
     }
 
@@ -460,27 +413,7 @@ class linux_epoll_io_service : public IoServiceBase {
 
   stdx::expected<void, std::error_code> remove_fd(
       native_handle_type fd) override {
-    std::lock_guard lk(fd_events_mtx_);
-    auto res = registered_events_.erase(epfd_, fd);
-    if (res) {
-      // remove all events which are already fetched by poll_one()
-      for (size_t ndx = fd_events_processed_; ndx < fd_events_size_;) {
-        auto ev = fd_events_[ndx];
-
-        if (ev.data.fd == fd) {
-          // found one, move it to the end and throw away this one.
-          if (ndx != fd_events_size_ - 1) {
-            std::swap(fd_events_[ndx], fd_events_[fd_events_size_ - 1]);
-          }
-
-          --fd_events_size_;
-        } else {
-          ++ndx;
-        }
-      }
-    }
-
-    return res;
+    return registered_events_.erase(epfd_, fd);
   }
 
   stdx::expected<void, std::error_code> remove_fd_interest(
@@ -493,83 +426,10 @@ class linux_epoll_io_service : public IoServiceBase {
    *
    * @returns fd-interest as bitmask of raw EPOLL* flags
    */
-  std::optional<int32_t> interest(native_handle_type fd) const {
+  stdx::expected<int32_t, void> interest(native_handle_type fd) const {
     return registered_events_.interest(fd);
   }
 
-  stdx::expected<fd_event, std::error_code> pop_event() {
-    size_t ndx = fd_events_processed_;
-
-    auto ev = fd_events_[ndx];
-
-    // if there are multiple events:
-    // - OUT before IN.
-    // - IN before ERR|HUP.
-    // - ERR before HUP.
-    short revent{};
-    if (ev.events & EPOLLOUT) {
-      fd_events_[ndx].events &= ~EPOLLOUT;
-      revent = EPOLLOUT;
-    } else if (ev.events & EPOLLIN) {
-      fd_events_[ndx].events &= ~EPOLLIN;
-      revent = EPOLLIN;
-    } else if (ev.events & EPOLLERR) {
-      fd_events_[ndx].events &= ~EPOLLERR;
-      revent = EPOLLERR;
-    } else if (ev.events & EPOLLHUP) {
-      fd_events_[ndx].events &= ~EPOLLHUP;
-      revent = EPOLLHUP;
-    }
-
-    // all interesting events processed, go the next one.
-    if ((fd_events_[ndx].events & (EPOLLIN | EPOLLOUT | EPOLLERR | EPOLLHUP)) ==
-        0) {
-      fd_events_processed_++;
-    }
-
-    return fd_event{ev.data.fd, revent};
-  }
-
-  stdx::expected<fd_event, std::error_code> update_fd_events(
-      std::chrono::milliseconds timeout) {
-    MY_COMPILER_DIAGNOSTIC_PUSH();
-    MY_COMPILER_GCC_DIAGNOSTIC_IGNORE("-Wmaybe-uninitialized")
-
-    // not initialized as it is write-only to by epoll::wait().
-    decltype(fd_events_) evs;
-
-    auto res = impl::epoll::wait(epfd_, evs.data(), evs.size(), timeout);
-    MY_COMPILER_DIAGNOSTIC_POP();
-
-    if (!res) return stdx::unexpected(res.error());
-
-    std::lock_guard lk(fd_events_mtx_);
-    // copy the fd-events that were returned.
-    std::copy_n(evs.begin(), *res, fd_events_.begin());
-
-    fd_events_processed_ = 0;
-    fd_events_size_ = *res;
-
-    if (fd_events_size_ == 0) {
-      return stdx::unexpected(make_error_code(std::errc::timed_out));
-    }
-
-    for (size_t ndx{}; ndx < fd_events_size_; ++ndx) {
-      const ::epoll_event ev = fd_events_[ndx];
-
-      auto after_res = after_event_fired(epfd_, ev.data.fd, ev.events);
-      if (!after_res) {
-        std::ostringstream oss;
-        oss << "after_event_fired(" << ev.data.fd << ", "
-            << std::bitset<32>(ev.events) << ") " << after_res.error() << " "
-            << after_res.error().message() << std::endl;
-        std::cerr << oss.str();
-      }
-    }
-
-    return pop_event();
-  }
-
   /**
    * poll one event from the registered fd-interest.
    *
@@ -583,34 +443,47 @@ class linux_epoll_io_service : public IoServiceBase {
   stdx::expected<fd_event, std::error_code> poll_one(
       std::chrono::milliseconds timeout) override {
     if (!is_open()) {
-      return stdx::unexpected(make_error_code(std::errc::invalid_argument));
+      return stdx::make_unexpected(
+          make_error_code(std::errc::invalid_argument));
     }
 
-    auto ev_res = [this]() -> stdx::expected<fd_event, std::error_code> {
-      std::lock_guard lk(fd_events_mtx_);
+    if (fd_events_processed_ == fd_events_size_) {
+      auto res = impl::epoll::wait(epfd_, fd_events_.data(), fd_events_.size(),
+                                   timeout);
 
-      if (fd_events_processed_ == fd_events_size_) {
-        // no event.
-        return stdx::unexpected(
-            make_error_code(std::errc::no_such_file_or_directory));
+      if (!res) return stdx::make_unexpected(res.error());
+
+      fd_events_processed_ = 0;
+      fd_events_size_ = *res;
+
+      if (fd_events_size_ == 0) {
+        return stdx::make_unexpected(make_error_code(std::errc::timed_out));
       }
 
-      return pop_event();
-    }();
+      for (size_t ndx{}; ndx < fd_events_size_; ++ndx) {
+        const ::epoll_event ev = fd_events_[ndx];
 
-    if (!ev_res) {
-      if (ev_res.error() == std::errc::no_such_file_or_directory) {
-        ev_res = update_fd_events(timeout);
+        auto after_res = after_event_fired(epfd_, ev.data.fd, ev.events);
+        if (!after_res) {
+          std::cerr << "after_event_fired(" << ev.data.fd << ", "
+                    << std::bitset<32>(ev.events) << ") " << after_res.error()
+                    << " " << after_res.error().message() << std::endl;
+        }
       }
-
-      if (!ev_res) return stdx::unexpected(ev_res.error());
     }
 
-    auto ev = *ev_res;
+    // traverse the events in reverse order as the epoll::wait() returns the
+    // events in LIFO order which may result in higher average latency.
+
+    size_t ndx = fd_events_size_ - fd_events_processed_ - 1;
+
+    ::epoll_event ev = fd_events_[ndx];
+
+    fd_events_processed_++;
 
     if ((notify_fd_ != impl::file::kInvalidHandle)
-            ? (ev.fd == notify_fd_)
-            : (ev.fd == wakeup_fds_.first)) {
+            ? (ev.data.fd == notify_fd_)
+            : (ev.data.fd == wakeup_fds_.first)) {
       // wakeup fd fired
       //
       // - don't remove the interest for it
@@ -618,10 +491,10 @@ class linux_epoll_io_service : public IoServiceBase {
       // interrupted
       on_notify();
 
-      return stdx::unexpected(make_error_code(std::errc::interrupted));
+      return stdx::make_unexpected(make_error_code(std::errc::interrupted));
     }
 
-    return ev;
+    return fd_event{ev.data.fd, static_cast<short>(ev.events)};
   }
 
  private:
@@ -644,7 +517,6 @@ class linux_epoll_io_service : public IoServiceBase {
   // epoll_wait(.., 4, ...) = [ 1 2 3 4 ]
   //
   // ... and 5, 6 never get processed.
-  std::mutex fd_events_mtx_;
   std::array<epoll_event, 8192> fd_events_{};
   size_t fd_events_processed_{0};
   size_t fd_events_size_{0};
