@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -61,100 +60,22 @@ namespace net {
 // 18.3 [socket.err]
 // implemented in impl/socket_error.h
 
-// 18.5 [socket.opt]
-
-namespace socket_option {
-
-/**
- * base-class of socket options.
- *
- * can be used to implement type safe socket options.
- *
- * @see socket_option::integer
- * @see socket_option::boolean
- */
-template <int Level, int Name, class T, class V = T>
-class option_base {
- public:
-  using value_type = T;
-  using storage_type = V;
-
-  constexpr option_base() : value_{} {}
-
-  constexpr explicit option_base(value_type v)
-      : value_{static_cast<storage_type>(v)} {}
-
-  value_type value() const { return value_; }
-
-  template <typename Protocol>
-  constexpr int level(const Protocol & /* unused */) const noexcept {
-    return Level;
-  }
-
-  template <typename Protocol>
-  constexpr int name(const Protocol & /* unused */) const noexcept {
-    return Name;
-  }
-
-  template <typename Protocol>
-  const storage_type *data(const Protocol & /* unused */) const {
-    return std::addressof(value_);
-  }
-
-  template <typename Protocol>
-  storage_type *data(const Protocol & /* unused */) {
-    return std::addressof(value_);
-  }
-
-  template <typename Protocol>
-  constexpr size_t size(const Protocol & /* unused */) const {
-    return sizeof(value_);
-  }
-
-  template <class Protocol>
-  void resize(const Protocol & /* p */, size_t s) {
-    if (s != sizeof(value_)) {
-      throw std::length_error("size != sizeof(value_)");
-    }
-  }
-
- private:
-  storage_type value_;
-};
-
-/**
- * socket option that uses bool as value_type.
- */
-template <int Level, int Name>
-using boolean = option_base<Level, Name, bool, int>;
-
-/**
- * socket option that uses int as value_type.
- */
-template <int Level, int Name>
-using integer = option_base<Level, Name, int, int>;
-
-}  // namespace socket_option
-
 // 18.4 [socket.base]
 
 class socket_base {
  public:
-  using broadcast = socket_option::boolean<SOL_SOCKET, SO_BROADCAST>;
-  using debug = socket_option::boolean<SOL_SOCKET, SO_DEBUG>;
-  using do_not_route = socket_option::boolean<SOL_SOCKET, SO_DONTROUTE>;
-  using error =
-      socket_option::integer<SOL_SOCKET, SO_ERROR>;  // not part of std
-  using keep_alive = socket_option::boolean<SOL_SOCKET, SO_KEEPALIVE>;
-
+  class broadcast;
+  class debug;
+  class do_not_route;
+  class error;  // not part of std
+  class keep_alive;
   class linger;
-
-  using out_of_band_inline = socket_option::boolean<SOL_SOCKET, SO_OOBINLINE>;
-  using receive_buffer_size = socket_option::integer<SOL_SOCKET, SO_RCVBUF>;
-  using receive_low_watermark = socket_option::integer<SOL_SOCKET, SO_RCVLOWAT>;
-  using reuse_address = socket_option::boolean<SOL_SOCKET, SO_REUSEADDR>;
-  using send_buffer_size = socket_option::integer<SOL_SOCKET, SO_SNDBUF>;
-  using send_low_watermark = socket_option::integer<SOL_SOCKET, SO_SNDLOWAT>;
+  class out_of_band_inline;
+  class receive_buffer_size;
+  class receive_low_watermark;
+  class reuse_address;
+  class send_buffer_size;
+  class send_low_watermark;
 
   using message_flags = impl::socket::message_flags;
 
@@ -251,7 +172,7 @@ class socket_base {
     }
 
     /**
-     * set recipient of the message.
+     * set receipient of the message.
      */
     template <class endpoint_type>
     void set_recipient(const endpoint_type &ep) {
@@ -267,6 +188,109 @@ class socket_base {
   };
 };
 
+// 18.5 [socket.opt]
+
+namespace socket_option {
+
+/**
+ * base-class of socket options.
+ *
+ * can be used to implement type safe socket options.
+ *
+ * @see socket_option::integer
+ * @see socket_option::boolen
+ */
+template <int Level, int Name, class T, class V = T>
+class option_base {
+ public:
+  using value_type = T;
+  using storage_type = V;
+
+  constexpr option_base() : value_{} {}
+
+  constexpr explicit option_base(value_type v)
+      : value_{static_cast<storage_type>(v)} {}
+
+  value_type value() const { return value_; }
+
+  template <typename Protocol>
+  constexpr int level(const Protocol & /* unused */) const noexcept {
+    return Level;
+  }
+
+  template <typename Protocol>
+  constexpr int name(const Protocol & /* unused */) const noexcept {
+    return Name;
+  }
+
+  template <typename Protocol>
+  const storage_type *data(const Protocol & /* unused */) const {
+    return std::addressof(value_);
+  }
+
+  template <typename Protocol>
+  storage_type *data(const Protocol & /* unused */) {
+    return std::addressof(value_);
+  }
+
+  template <typename Protocol>
+  constexpr size_t size(const Protocol & /* unused */) const {
+    return sizeof(value_);
+  }
+
+  template <class Protocol>
+  void resize(const Protocol & /* p */, size_t s) {
+    if (s != sizeof(value_)) {
+      throw std::length_error("size != sizeof(value_)");
+    }
+  }
+
+ private:
+  storage_type value_;
+};
+
+/**
+ * socket option that uses bool as value_type.
+ */
+template <int Level, int Name>
+using boolean = option_base<Level, Name, bool, int>;
+
+/**
+ * socket option that uses int as value_type.
+ */
+template <int Level, int Name>
+using integer = option_base<Level, Name, int, int>;
+
+}  // namespace socket_option
+
+class socket_base::broadcast
+    : public socket_option::boolean<SOL_SOCKET, SO_BROADCAST> {
+  using __base = socket_option::boolean<SOL_SOCKET, SO_BROADCAST>;
+  using __base::__base;
+};
+
+class socket_base::debug : public socket_option::boolean<SOL_SOCKET, SO_DEBUG> {
+  using __base = socket_option::boolean<SOL_SOCKET, SO_DEBUG>;
+  using __base::__base;
+};
+
+class socket_base::do_not_route
+    : public socket_option::boolean<SOL_SOCKET, SO_DONTROUTE> {
+  using __base = socket_option::boolean<SOL_SOCKET, SO_DONTROUTE>;
+  using __base::__base;
+};
+
+class socket_base::error : public socket_option::integer<SOL_SOCKET, SO_ERROR> {
+  using __base = socket_option::integer<SOL_SOCKET, SO_ERROR>;
+  using __base::__base;
+};
+
+class socket_base::keep_alive
+    : public socket_option::boolean<SOL_SOCKET, SO_KEEPALIVE> {
+  using __base = socket_option::boolean<SOL_SOCKET, SO_KEEPALIVE>;
+  using __base::__base;
+};
+
 // 18.5.1
 
 /**
@@ -325,6 +349,42 @@ class socket_base::linger {
   ::linger value_;
 };
 
+class socket_base::out_of_band_inline
+    : public socket_option::boolean<SOL_SOCKET, SO_OOBINLINE> {
+  using __base = socket_option::boolean<SOL_SOCKET, SO_OOBINLINE>;
+  using __base::__base;
+};
+
+class socket_base::receive_buffer_size
+    : public socket_option::integer<SOL_SOCKET, SO_RCVBUF> {
+  using __base = socket_option::integer<SOL_SOCKET, SO_RCVBUF>;
+  using __base::__base;
+};
+
+class socket_base::receive_low_watermark
+    : public socket_option::integer<SOL_SOCKET, SO_RCVLOWAT> {
+  using __base = socket_option::integer<SOL_SOCKET, SO_RCVLOWAT>;
+  using __base::__base;
+};
+
+class socket_base::reuse_address
+    : public socket_option::boolean<SOL_SOCKET, SO_REUSEADDR> {
+  using __base = socket_option::boolean<SOL_SOCKET, SO_REUSEADDR>;
+  using __base::__base;
+};
+
+class socket_base::send_buffer_size
+    : public socket_option::integer<SOL_SOCKET, SO_SNDBUF> {
+  using __base = socket_option::integer<SOL_SOCKET, SO_SNDBUF>;
+  using __base::__base;
+};
+
+class socket_base::send_low_watermark
+    : public socket_option::integer<SOL_SOCKET, SO_SNDLOWAT> {
+  using __base = socket_option::integer<SOL_SOCKET, SO_SNDLOWAT>;
+  using __base::__base;
+};
+
 /**
  * template-less base-class of basic_socket_impl.
  *
@@ -373,7 +433,7 @@ class basic_socket_impl_base {
 
   stdx::expected<void, std::error_code> non_blocking(bool mode) {
     if (!is_open()) {
-      return stdx::unexpected(
+      return stdx::make_unexpected(
           std::make_error_code(std::errc::bad_file_descriptor));
     }
     non_blocking_ = mode;
@@ -393,12 +453,12 @@ class basic_socket_impl_base {
 
   stdx::expected<void, std::error_code> native_non_blocking(bool mode) {
     if (!is_open()) {
-      return stdx::unexpected(
+      return stdx::make_unexpected(
           std::make_error_code(std::errc::bad_file_descriptor));
     }
 
     if (!mode && non_blocking()) {
-      return stdx::unexpected(
+      return stdx::make_unexpected(
           std::make_error_code(std::errc::invalid_argument));
     }
 
@@ -418,10 +478,7 @@ class basic_socket_impl_base {
       cancel();
 
       auto res = io_ctx_->socket_service()->close(native_handle());
-
-      // after close() finished, the socket's state is undefined even if it
-      // failed. See "man close" on Linux.
-      native_handle_ = impl::socket::kInvalidSocket;
+      if (res) native_handle_ = impl::socket::kInvalidSocket;
 
       return res;
     }
@@ -431,7 +488,8 @@ class basic_socket_impl_base {
 
   stdx::expected<void, std::error_code> cancel() {
     if (!is_open()) {
-      return stdx::unexpected(make_error_code(std::errc::bad_file_descriptor));
+      return stdx::make_unexpected(
+          make_error_code(std::errc::bad_file_descriptor));
     }
 
     io_ctx_->cancel(native_handle());
@@ -502,11 +560,11 @@ class basic_socket_impl : public basic_socket_impl_base {
   stdx::expected<void, error_type> open(
       const protocol_type &protocol = protocol_type(), int flags = 0) {
     if (is_open()) {
-      return stdx::unexpected(make_error_code(socket_errc::already_open));
+      return stdx::make_unexpected(make_error_code(socket_errc::already_open));
     }
     auto res = io_ctx_->socket_service()->socket(
         protocol.family(), protocol.type() | flags, protocol.protocol());
-    if (!res) return stdx::unexpected(res.error());
+    if (!res) return stdx::make_unexpected(res.error());
 #ifdef SOCK_NONBLOCK
     if ((flags & SOCK_NONBLOCK) != 0) {
       native_non_blocking_ = 1;
@@ -519,7 +577,7 @@ class basic_socket_impl : public basic_socket_impl_base {
   stdx::expected<void, error_type> assign(
       const protocol_type &protocol, const native_handle_type &native_handle) {
     if (is_open()) {
-      return stdx::unexpected(make_error_code(socket_errc::already_open));
+      return stdx::make_unexpected(make_error_code(socket_errc::already_open));
     }
     protocol_ = protocol;
     native_handle_ = native_handle;
@@ -538,23 +596,22 @@ class basic_socket_impl : public basic_socket_impl_base {
                                                  struct sockaddr *endpoint_data,
                                                  socklen_t *endpoint_size,
                                                  int flags = 0) {
-    using ret_type = stdx::expected<socket_type, error_type>;
     if (flags != 0) {
       auto res = io_ctx_->socket_service()->accept4(
           native_handle(), endpoint_data, endpoint_size, flags);
       if (res) {
         return socket_type{io_ctx, protocol_, res.value()};
       } else if (res.error() != std::errc::operation_not_supported) {
-        return stdx::unexpected(res.error());
+        return stdx::make_unexpected(res.error());
       }
 
       // otherwise fall through to accept without flags
     }
     auto res = io_ctx_->socket_service()->accept(native_handle(), endpoint_data,
                                                  endpoint_size);
-    if (!res) return stdx::unexpected(res.error());
+    if (!res) return stdx::make_unexpected(res.error());
 
-    return ret_type{std::in_place, io_ctx, protocol_, std::move(res.value())};
+    return {stdx::in_place, io_ctx, protocol_, std::move(res.value())};
   }
 
   stdx::expected<socket_type, error_type> accept(io_context &io_ctx,
@@ -611,7 +668,7 @@ class basic_socket_impl : public basic_socket_impl_base {
     auto res = io_ctx_->socket_service()->getsockname(
         native_handle(), reinterpret_cast<struct sockaddr *>(ep.data()),
         &ep_len);
-    if (!res) return stdx::unexpected(res.error());
+    if (!res) return stdx::make_unexpected(res.error());
 
     ep.resize(ep_len);
 
@@ -625,7 +682,7 @@ class basic_socket_impl : public basic_socket_impl_base {
     auto res = io_ctx_->socket_service()->getpeername(
         native_handle(), reinterpret_cast<struct sockaddr *>(ep.data()),
         &ep_len);
-    if (!res) return stdx::unexpected(res.error());
+    if (!res) return stdx::make_unexpected(res.error());
 
     ep.resize(ep_len);
 
@@ -674,10 +731,10 @@ class basic_socket_impl : public basic_socket_impl_base {
     io_control_bytes_avail_recv ioc;
 
     auto res = io_control(ioc);
-    if (!res) return stdx::unexpected(res.error());
+    if (!res) return stdx::make_unexpected(res.error());
 
     if (ioc.value() < 0) {
-      return stdx::unexpected(make_error_code(std::errc::invalid_seek));
+      return stdx::make_unexpected(make_error_code(std::errc::invalid_seek));
     }
 
     // value is not negative, safe to cast to unsigned
@@ -688,7 +745,7 @@ class basic_socket_impl : public basic_socket_impl_base {
     io_control_at_mark ioc;
 
     auto res = io_control(ioc);
-    if (!res) return stdx::unexpected(res.error());
+    if (!res) return stdx::make_unexpected(res.error());
 
     return ioc.value();
   }
@@ -741,71 +798,6 @@ class basic_socket : public socket_base, private basic_socket_impl<Protocol> {
         endpoint.size());
   }
 
-  template <class CompletionToken>
-  auto async_connect(const endpoint_type &endpoint, CompletionToken &&token) {
-    async_completion<CompletionToken, void(std::error_code)> init{token};
-
-    if (!is_open()) {
-      auto res = open(endpoint.protocol());
-      if (!res) {
-        init.completion_handler(res.error());
-
-        return;
-      }
-    }
-
-    net::defer(get_executor(), [this, endpoint,
-                                __compl_handler =
-                                    std::move(init.completion_handler)]() {
-      // remember the non-blocking flag.
-      const auto old_non_blocking = native_non_blocking();
-      if (old_non_blocking == false) native_non_blocking(true);
-
-      auto res = connect(endpoint);
-
-      // restore the non-blocking flag if needed.
-      if (old_non_blocking == false) native_non_blocking(false);
-
-      if (res) {
-        __compl_handler({});
-      } else {
-        const auto ec = res.error();
-
-        if ((ec !=
-             make_error_condition(std::errc::operation_in_progress)) /* posix */
-            && (ec != make_error_condition(
-                          std::errc::operation_would_block)) /* windows */) {
-          __compl_handler(ec);
-        } else {
-          get_executor().context().async_wait(
-              native_handle(), net::socket_base::wait_write,
-              [this, __compl_handler = std::move(__compl_handler)](
-                  std::error_code error_code) mutable {
-                if (error_code) {
-                  __compl_handler(error_code);
-                  return;
-                }
-
-                // finish the non-blocking connect
-                net::socket_base::error so_error;
-
-                auto result = get_option(so_error);
-                if (!result) {
-                  __compl_handler(result.error());
-                  return;
-                }
-
-                // if so_error.value() is 0, the error_code will be 0 too
-                __compl_handler(
-                    impl::socket::make_error_code(so_error.value()));
-              });
-        }
-      }
-    });
-
-    return init.result.get();
-  }
-
   stdx::expected<void, error_type> bind(const endpoint_type &endpoint) {
     return __base::bind(endpoint);
   }
@@ -869,6 +861,24 @@ class basic_socket : public socket_base, private basic_socket_impl<Protocol> {
     return __base::shutdown(st);
   }
 
+#if defined(ALL_COMPILERS_FULLY_SUPPORT_CPP14)
+  // the define ALL_COMPILERS_FULLY_SUPPORT_CPP14 is a placeholder.
+  //
+  // The code below assumes the compiler supports generalized lambda
+  // capture from C++14, allowing use move the ownership into the lambda.
+  //
+  // The code works with:
+  //
+  // - gcc 5.x and later
+  // - clang 4 and later
+  // - MSVC ...
+  //
+  // but it fails for move-only completion handlers with:
+  //
+  // - sun pro 15.6
+  //
+  // As soon as sun pro is fixed or supported for it is dropped,
+  // C++11 implementation below and this comment can be removed.
   template <typename CompletionToken>
   auto async_wait(wait_type w, CompletionToken &&token) {
     async_completion<CompletionToken, void(std::error_code)> init{token};
@@ -880,6 +890,48 @@ class basic_socket : public socket_base, private basic_socket_impl<Protocol> {
 
     return init.result.get();
   }
+#else
+  // C++11 compatible implementation of async_wait() which allows move-only
+  // completion handlers
+  //
+  // to be removed, once all supported compilers fully support C++14 generalized
+  // lambda capture
+
+  template <typename CompletionToken>
+  typename async_result<std::decay_t<CompletionToken>,
+                        void(std::error_code)>::return_type
+  async_wait(wait_type w, CompletionToken &&token) {
+    async_completion<CompletionToken, void(std::error_code)> init{token};
+
+    using async_completion_handler_type =
+        typename decltype(init)::completion_handler_type;
+
+    // capture the completion handler
+    class ClosureType {
+     public:
+      explicit ClosureType(async_completion_handler_type &&compl_handler)
+          : compl_handler_{
+                std::forward<decltype(compl_handler)>(compl_handler)} {}
+
+      void operator()(std::error_code ec) { compl_handler_(ec); }
+
+     private:
+      async_completion_handler_type compl_handler_;
+    };
+
+    // get native-handle before std::move()ing the completion handler into the
+    // async_wait(). Otherwise the native_handle() may refer to the moved-from
+    // object
+    const auto handle = native_handle();
+
+    get_executor().context().async_wait(
+        handle, w,
+        ClosureType(std::forward<async_completion_handler_type>(
+            init.completion_handler)));
+
+    return init.result.get();
+  }
+#endif
 
  protected:
   explicit basic_socket(io_context &ctx) : __base{ctx} {}
@@ -1038,9 +1090,6 @@ class basic_datagram_socket : public basic_socket<Protocol> {
   auto async_receive(const MutableBufferSequence &buffers,
                      socket_base::message_flags flags,
                      CompletionToken &&token) {
-    static_assert(
-        net::is_mutable_buffer_sequence<MutableBufferSequence>::value);
-
     async_completion<CompletionToken, void(std::error_code, size_t)> init{
         token};
 
@@ -1053,22 +1102,22 @@ class basic_datagram_socket : public basic_socket<Protocol> {
 
     this->get_executor().context().async_wait(
         this->native_handle(), socket_base::wait_read,
-        [socket_service = this->get_executor().context().socket_service(),
-         compl_handler = std::move(init.completion_handler), &buffers,
-         native_handle = this->native_handle(),
-         flags](std::error_code ec) mutable {
+        [this, __compl_handler = std::move(init.completion_handler), &buffers,
+         __fd = this->native_handle(),
+         __flags = flags](std::error_code ec) mutable {
           if (ec) {
-            compl_handler(ec, 0);
+            __compl_handler(ec, 0);
             return;
           }
 
           socket_base::msg_hdr msgs(buffers);
 
-          auto res = socket_service->recvmsg(native_handle, msgs, flags);
+          auto res = this->get_executor().context().socket_service()->recvmsg(
+              __fd, msgs, __flags);
           if (!res) {
-            compl_handler(res.error(), 0);
+            __compl_handler(res.error(), 0);
           } else {
-            compl_handler({}, res.value());
+            __compl_handler({}, res.value());
           }
         });
 
@@ -1078,9 +1127,6 @@ class basic_datagram_socket : public basic_socket<Protocol> {
   template <class MutableBufferSequence, class CompletionToken>
   auto async_receive(const MutableBufferSequence &buffers,
                      CompletionToken &&token) {
-    static_assert(
-        net::is_mutable_buffer_sequence<MutableBufferSequence>::value);
-
     return async_receive(buffers, 0, std::forward<CompletionToken>(token));
   }
 };
@@ -1125,7 +1171,7 @@ class basic_stream_socket : public basic_socket<Protocol> {
 
     if (res && res.value() == 0) {
       // remote did a orderly shutdown
-      return stdx::unexpected(make_error_code(net::stream_errc::eof));
+      return stdx::make_unexpected(make_error_code(net::stream_errc::eof));
     }
 
     return res;
@@ -1157,7 +1203,7 @@ class basic_stream_socket : public basic_socket<Protocol> {
 
     if (res && res.value() == 0) {
       // remote did a orderly shutdown
-      return stdx::unexpected(make_error_code(net::stream_errc::eof));
+      return stdx::make_unexpected(make_error_code(net::stream_errc::eof));
     }
 
     return res;
@@ -1188,9 +1234,6 @@ class basic_stream_socket : public basic_socket<Protocol> {
   auto async_receive(const MutableBufferSequence &buffers,
                      socket_base::message_flags flags,
                      CompletionToken &&token) {
-    static_assert(
-        net::is_mutable_buffer_sequence<MutableBufferSequence>::value);
-
     async_completion<CompletionToken, void(std::error_code, size_t)> init{
         token};
     if ((flags & socket_base::message_peek).any()) {
@@ -1206,30 +1249,70 @@ class basic_stream_socket : public basic_socket<Protocol> {
       return init.result.get();
     }
 
+    /*
+     * as we use C++14, we could use a lambda here which supports movable
+     * arguments, but sun-cc 15.6 generates the wrong constructor:
+     *
+     *  "...::{lambda at socket.h,1203:9}::<constructor>(
+     *    net::basic_stream_socket<net::ip::tcp>*,
+     *    const server_mock::MySQLServerMockSessionClassic::StateMachine&,
+     *    const std::list<net::const_buffer>&, int, std::bitset<31>&)"
+     *
+     *  The "const StateMachine &" should have been "StateMachine &&".
+     */
+
+    using compl_handler_type = typename decltype(init)::completion_handler_type;
+
+    class Completor {
+     public:
+      Completor(net::impl::socket::SocketServiceBase *socket_service,
+                compl_handler_type &&compl_handler,
+                const MutableBufferSequence &buffers,
+                impl::socket::native_handle_type native_handle,
+                socket_base::message_flags flags)
+          : socket_service_{socket_service},
+            compl_handler_(std::move(compl_handler)),
+            buffers_{buffers},
+            native_handle_{native_handle},
+            flags_{flags} {}
+
+      Completor(const Completor &) = delete;
+      Completor(Completor &&) = default;
+
+      void operator()(std::error_code ec) {
+        if (ec) {
+          compl_handler_(ec, 0);
+          return;
+        }
+
+        socket_base::msg_hdr msgs(buffers_);
+
+        const auto res = socket_service_->recvmsg(native_handle_, msgs, flags_);
+        if (!res) {
+          compl_handler_(res.error(), 0);
+        } else if (res.value() == 0) {
+          // remote did a orderly shutdown
+          compl_handler_(make_error_code(stream_errc::eof), 0);
+        } else {
+          compl_handler_({}, res.value());
+        }
+
+        return;
+      }
+
+     private:
+      net::impl::socket::SocketServiceBase *socket_service_;
+      compl_handler_type compl_handler_;
+      const MutableBufferSequence &buffers_;
+      impl::socket::native_handle_type native_handle_;
+      socket_base::message_flags flags_;
+    };
+
     this->get_executor().context().async_wait(
         this->native_handle(), socket_base::wait_read,
-        [socket_service = this->get_executor().context().socket_service(),
-         compl_handler = std::move(init.completion_handler), buffers,
-         native_handle = this->native_handle(), flags](std::error_code ec) {
-          if (ec) {
-            compl_handler(ec, 0);
-            return;
-          }
-
-          socket_base::msg_hdr msgs(buffers);
-
-          const auto res = socket_service->recvmsg(native_handle, msgs, flags);
-          if (!res) {
-            compl_handler(res.error(), 0);
-          } else if (res.value() == 0) {
-            // remote did a orderly shutdown
-            compl_handler(make_error_code(stream_errc::eof), 0);
-          } else {
-            compl_handler({}, res.value());
-          }
-
-          return;
-        });
+        Completor(this->get_executor().context().socket_service(),
+                  std::move(init.completion_handler), buffers,
+                  this->native_handle(), flags));
 
     return init.result.get();
   }
@@ -1237,16 +1320,14 @@ class basic_stream_socket : public basic_socket<Protocol> {
   template <class MutableBufferSequence, class CompletionToken>
   auto async_receive(const MutableBufferSequence &buffers,
                      CompletionToken &&token) {
-    static_assert(
-        net::is_mutable_buffer_sequence<MutableBufferSequence>::value);
-
     return async_receive(buffers, 0, std::forward<CompletionToken>(token));
   }
 
   template <class ConstBufferSequence, class CompletionToken>
   auto async_send(const ConstBufferSequence &buffers,
                   socket_base::message_flags flags, CompletionToken &&token) {
-    static_assert(net::is_const_buffer_sequence<ConstBufferSequence>::value);
+    static_assert(net::is_const_buffer_sequence<ConstBufferSequence>::value,
+                  "");
     async_completion<CompletionToken, void(std::error_code, size_t)> init{
         token};
     if (buffer_size(buffers) == 0) {
@@ -1255,34 +1336,64 @@ class basic_stream_socket : public basic_socket<Protocol> {
       return init.result.get();
     }
 
+    // see async_receive() for why this isn't a lambda.
+    using compl_handler_type = typename decltype(init)::completion_handler_type;
+
+    class Completor {
+     public:
+      Completor(net::impl::socket::SocketServiceBase *socket_service,
+                compl_handler_type &&compl_handler,
+                const ConstBufferSequence &buffers,
+                impl::socket::native_handle_type native_handle,
+                socket_base::message_flags flags)
+          : socket_service_{socket_service},
+            compl_handler_(std::move(compl_handler)),
+            buffers_{buffers},
+            native_handle_{native_handle},
+            flags_{flags} {}
+
+      Completor(const Completor &) = delete;
+      Completor(Completor &&) = default;
+
+      void operator()(std::error_code ec) {
+        if (ec) {
+          compl_handler_(ec, 0);
+          return;
+        }
+
+        socket_base::msg_hdr msgs(buffers_);
+
+        const auto res = socket_service_->sendmsg(native_handle_, msgs, flags_);
+        if (!res) {
+          compl_handler_(res.error(), 0);
+        } else {
+          compl_handler_({}, res.value());
+        }
+
+        return;
+      }
+
+      net::impl::socket::SocketServiceBase *socket_service_;
+      compl_handler_type compl_handler_;
+      const ConstBufferSequence &buffers_;
+      impl::socket::native_handle_type native_handle_;
+      socket_base::message_flags flags_;
+    };
+
+    // do something
     this->get_executor().context().async_wait(
         this->native_handle(), socket_base::wait_write,
-        [socket_service = this->get_executor().context().socket_service(),
-         compl_handler = std::move(init.completion_handler), buffers,
-         native_handle = this->native_handle(), flags](std::error_code ec) {
-          if (ec) {
-            compl_handler(ec, 0);
-            return;
-          }
-
-          socket_base::msg_hdr msgs(buffers);
-
-          const auto res = socket_service->sendmsg(native_handle, msgs, flags);
-          if (!res) {
-            compl_handler(res.error(), 0);
-          } else {
-            compl_handler({}, res.value());
-          }
-
-          return;
-        });
+        Completor(this->get_executor().context().socket_service(),
+                  std::move(init.completion_handler), buffers,
+                  this->native_handle(), flags));
 
     return init.result.get();
   }
 
   template <class ConstBufferSequence, class CompletionToken>
   auto async_send(const ConstBufferSequence &buffers, CompletionToken &&token) {
-    static_assert(net::is_const_buffer_sequence<ConstBufferSequence>::value);
+    static_assert(net::is_const_buffer_sequence<ConstBufferSequence>::value,
+                  "");
     return async_send(buffers, 0, std::forward<CompletionToken>(token));
   }
 };
@@ -1445,68 +1556,6 @@ class basic_socket_acceptor : public socket_base,
     return init.result.get();
   }
 
-  template <class CompletionToken>
-  auto async_accept(endpoint_type &endpoint, CompletionToken &&token) {
-    return async_accept(get_executor().context(), endpoint,
-                        std::forward<CompletionToken>(token));
-  }
-
-  /**
-   * accept a connection with endpoint async'.
-   *
-   * - returns immediately
-   * - calls completiontoken when finished
-   *
-   * @param [in,out] io_ctx io-context to execute the waiting/execution in
-   * @param [out] endpoint remote endpoint of the accepted connection
-   * @param [in] token completion token of type 'void(std::error_code,
-   * socket_type)'
-   */
-  template <class CompletionToken>
-  auto async_accept(io_context &io_ctx, endpoint_type &endpoint,
-                    CompletionToken &&token) {
-    async_completion<CompletionToken, void(std::error_code, socket_type)> init{
-        token};
-
-    // - wait for acceptor to become readable
-    // - accept() it.
-    // - call completion with socket
-    io_ctx.get_executor().context().async_wait(
-        native_handle(), socket_base::wait_read,
-        [this, __compl_handler = std::move(init.completion_handler),
-         __protocol = protocol_, __fd = native_handle(), &__ep = endpoint,
-         &io_ctx](std::error_code ec) mutable {
-          if (ec) {
-            __compl_handler(ec, socket_type(io_ctx));
-            return;
-          }
-
-          while (true) {
-            socklen_t endpoint_len = __ep.capacity();
-
-            auto res = this->get_executor().context().socket_service()->accept(
-                __fd, static_cast<sockaddr *>(__ep.data()), &endpoint_len);
-
-            if (!res && !enable_connection_aborted() &&
-                res.error() ==
-                    make_error_condition(std::errc::connection_aborted)) {
-              continue;
-            }
-
-            if (!res) {
-              __compl_handler(res.error(), socket_type(io_ctx));
-            } else {
-              __ep.resize(endpoint_len);
-
-              __compl_handler({}, socket_type{io_ctx, __protocol, res.value()});
-            }
-            return;
-          }
-        });
-
-    return init.result.get();
-  }
-
   template <class CompletionToken>
   auto async_accept(CompletionToken &&token) {
     return async_accept(get_executor().context(),
@@ -1517,17 +1566,77 @@ class basic_socket_acceptor : public socket_base,
     return __base::wait(wt);
   }
 
+#if defined(ALL_COMPILERS_FULLY_SUPPORT_CPP14)
+  // the define ALL_COMPILERS_FULLY_SUPPORT_CPP14 is a placeholder.
+  //
+  // The code below assumes the compiler supports generalized lambda
+  // capture from C++14, allowing use move the ownership into the lambda.
+  //
+  // The code works with:
+  //
+  // - gcc 5.x and later
+  // - clang 4 and later
+  // - MSVC ...
+  //
+  // but it fails for move-only completion handlers with:
+  //
+  // - sun pro 15.6
+  //
+  // As soon as sun pro is fixed or supported for it is dropped,
+  // C++11 implementation below and this comment can be removed.
   template <typename CompletionToken>
   auto async_wait(wait_type w, CompletionToken &&token) {
     async_completion<CompletionToken, void(std::error_code)> init{token};
 
     get_executor().context().async_wait(
         native_handle(), w,
-        [compl_handler = std::move(init.completion_handler)](
-            std::error_code ec) mutable { compl_handler(ec); });
+        [__compl_handler = std::move(init.completion_handler)](
+            std::error_code ec) mutable { __compl_handler(ec); });
 
     return init.result.get();
   }
+#else
+  // C++11 compatible implementation of async_wait() which allows move-only
+  // completion handlers
+  //
+  // to be removed, once all supported compilers fully support C++14 generalized
+  // lambda capture
+
+  template <typename CompletionToken>
+  typename async_result<std::decay_t<CompletionToken>,
+                        void(std::error_code)>::return_type
+  async_wait(wait_type w, CompletionToken &&token) {
+    async_completion<CompletionToken, void(std::error_code)> init{token};
+
+    using async_completion_handler_type =
+        typename decltype(init)::completion_handler_type;
+
+    // capture the completion handler
+    class ClosureType {
+     public:
+      explicit ClosureType(async_completion_handler_type &&compl_handler)
+          : compl_handler_{
+                std::forward<decltype(compl_handler)>(compl_handler)} {}
+
+      void operator()(std::error_code ec) { compl_handler_(ec); }
+
+     private:
+      async_completion_handler_type compl_handler_;
+    };
+
+    // get native-handle before std::move()ing the completion handler into the
+    // async_wait(). Otherwise the native_handle() may refer to the moved-from
+    // object
+    const auto handle = native_handle();
+
+    get_executor().context().async_wait(
+        handle, w,
+        ClosureType(std::forward<async_completion_handler_type>(
+            init.completion_handler)));
+
+    return init.result.get();
+  }
+#endif
 
  private:
   protocol_type protocol_;
@@ -1613,9 +1722,8 @@ stdx::expected<InputIterator, std::error_code> connect(
     }
   }
 
-  return stdx::unexpected(make_error_code(socket_errc::not_found));
+  return stdx::make_unexpected(make_error_code(socket_errc::not_found));
 }
-
 template <class Protocol, class InputIterator, class ConnectCondition>
 stdx::expected<InputIterator, std::error_code> connect(
     basic_socket<Protocol> &s, InputIterator first, InputIterator last) {
