@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,8 +24,6 @@
 
 #include "syslog_plugin.h"
 
-#include "mysql/harness/logging/logger_plugin.h"
-#include "mysql/harness/logging/supported_logger_options.h"
 #include "mysql/harness/plugin.h"  // Plugin
 
 extern "C" {
@@ -44,8 +41,5 @@ mysql_harness::Plugin harness_plugin_syslog = {
     nullptr,  // start
     nullptr,  // stop
     false,    // declares_readiness
-    logger_sink_supported_options.size(),
-    logger_sink_supported_options.data(),
-    nullptr,  // expose_configuration
 };
 }
