@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,9 +40,11 @@
 
 ////////////////////////////////////////
 // Test system include files
-#include "router/tests/helpers/stdx_expected_no_error.h"
 #include "test/helpers.h"
-#include "test/temp_directory.h"
+
+using std::back_inserter;
+using std::cout;
+using std::endl;
 
 using mysql_harness::Directory;
 using mysql_harness::Path;
@@ -157,8 +158,9 @@ TEST(TestFilesystem, TestDirectory) {
 }
 
 TEST(TestFilesystem, list_recursive_empty) {
-  TempDirectory tmpdir("tmp");
-  const std::string dir_name = tmpdir.name();
+  const std::string dir_name = mysql_harness::get_tmp_dir("tmp");
+  std::shared_ptr<void> exit_guard(
+      nullptr, [&](void *) { mysql_harness::delete_dir_recursive(dir_name); });
 
   Directory test{dir_name};
   const auto &result = test.list_recursive();
@@ -166,8 +168,9 @@ TEST(TestFilesystem, list_recursive_empty) {
 }
 
 TEST(TestFilesystem, list_recursive_empty_directories) {
-  TempDirectory tmpdir("tmp");
-  const std::string dir_name = tmpdir.name();
+  const std::string dir_name = mysql_harness::get_tmp_dir("tmp");
+  std::shared_ptr<void> exit_guard(
+      nullptr, [&](void *) { mysql_harness::delete_dir_recursive(dir_name); });
 
   mysql_harness::mkdir(Path{dir_name}.join("x").c_str(), 0700);
   mysql_harness::mkdir(Path{dir_name}.join("y").c_str(), 0700);
@@ -179,8 +182,9 @@ TEST(TestFilesystem, list_recursive_empty_directories) {
 }
 
 TEST(TestFilesystem, list_recursive_only_files) {
-  TempDirectory tmpdir("tmp");
-  const std::string dir_name = tmpdir.name();
+  const std::string dir_name = mysql_harness::get_tmp_dir("tmp");
+  std::shared_ptr<void> exit_guard(
+      nullptr, [&](void *) { mysql_harness::delete_dir_recursive(dir_name); });
 
   Directory test{dir_name};
 
@@ -195,8 +199,9 @@ TEST(TestFilesystem, list_recursive_only_files) {
 }
 
 TEST(TestFilesystem, list_recursive_multiple_levels) {
-  TempDirectory tmpdir("tmp");
-  const std::string dir_name = tmpdir.name();
+  const std::string dir_name = mysql_harness::get_tmp_dir("tmp");
+  std::shared_ptr<void> exit_guard(
+      nullptr, [&](void *) { mysql_harness::delete_dir_recursive(dir_name); });
 
   Directory test{dir_name};
 
@@ -219,16 +224,18 @@ TEST(TestFilesystem, list_recursive_multiple_levels) {
 }
 
 TEST(TestFilesystem, is_empty_true) {
-  TempDirectory tmpdir("tmp");
-  const std::string dir_name = tmpdir.name();
+  const std::string dir_name = mysql_harness::get_tmp_dir("tmp");
+  std::shared_ptr<void> exit_guard(
+      nullptr, [&](void *) { mysql_harness::delete_dir_recursive(dir_name); });
 
   Directory test{dir_name};
   EXPECT_TRUE(test.is_empty());
 }
 
 TEST(TestFilesystem, is_empty_dir_with_empty_subdir) {
-  TempDirectory tmpdir("tmp");
-  const std::string dir_name = tmpdir.name();
+  const std::string dir_name = mysql_harness::get_tmp_dir("tmp");
+  std::shared_ptr<void> exit_guard(
+      nullptr, [&](void *) { mysql_harness::delete_dir_recursive(dir_name); });
 
   Directory test{dir_name};
   mysql_harness::mkdir(Path{dir_name}.join("foo").c_str(), 0700);
@@ -237,8 +244,9 @@ TEST(TestFilesystem, is_empty_dir_with_empty_subdir) {
 }
 
 TEST(TestFilesystem, is_empty_dir_with_file) {
-  TempDirectory tmpdir("tmp");
-  const std::string dir_name = tmpdir.name();
+  const std::string dir_name = mysql_harness::get_tmp_dir("tmp");
+  std::shared_ptr<void> exit_guard(
+      nullptr, [&](void *) { mysql_harness::delete_dir_recursive(dir_name); });
 
   Directory test{dir_name};
   std::ofstream file(Path{dir_name}.join("bar").str());
@@ -253,8 +261,9 @@ TEST(TestFilesystem, IsReadableIfFileCanBeRead) {
 #ifndef _WIN32
 
   // create temporary file
-  TempDirectory tmpdir("tmp");
-  const std::string directory = tmpdir.name();
+  const std::string directory = mysql_harness::get_tmp_dir("tmp");
+  std::shared_ptr<void> exit_guard(
+      nullptr, [&](void *) { mysql_harness::delete_dir_recursive(directory); });
 
   mysql_harness::Path path = mysql_harness::Path(directory).join("/tmp_file");
   std::ofstream file(path.str());
@@ -272,8 +281,9 @@ TEST(TestFilesystem, IsNotReadableIfFileCanNotBeRead) {
 #ifndef _WIN32
 
   // create temporary file
-  TempDirectory tmpdir("tmp");
-  const std::string directory = tmpdir.name();
+  const std::string directory = mysql_harness::get_tmp_dir("tmp");
+  std::shared_ptr<void> exit_guard(
+      nullptr, [&](void *) { mysql_harness::delete_dir_recursive(directory); });
 
   mysql_harness::Path path = mysql_harness::Path(directory).join("/tmp_file");
   std::ofstream file(path.str());
@@ -298,7 +308,7 @@ TEST(TestFilesystem, delete_dir_recursive) {
   std::ofstream().open("testdir/f2");
   std::ofstream().open("testdir/a/f");
   std::ofstream().open("testdir/a/b/f");
-  EXPECT_NO_ERROR(mysql_harness::delete_dir_recursive("testdir"));
+  EXPECT_EQ(0, mysql_harness::delete_dir_recursive("testdir"));
 }
 
 /*
@@ -308,8 +318,9 @@ TEST(TestFilesystem, delete_dir_recursive) {
 TEST(TestFilesystem, Mkdir) {
   constexpr auto kMode = 0700;
 
-  TempDirectory tmpdir("tmp");
-  const std::string tmp_dir = tmpdir.name();
+  auto tmp_dir = mysql_harness::get_tmp_dir("test");
+  std::shared_ptr<void> exit_guard(
+      nullptr, [&](void *) { mysql_harness::delete_dir_recursive(tmp_dir); });
 
   // non-recursive should fail
   EXPECT_NE(0, mysql_harness::mkdir(tmp_dir + "/a/b/c/d", kMode));
@@ -339,36 +350,6 @@ TEST(TestFilesystem, Mkdir) {
   EXPECT_THROW(mysql_harness::mkdir("", kMode, true), std::invalid_argument);
 }
 
-/*
- * Tests mysql_harness::mkdir()
- */
-TEST(TestFilesystem, get_tmp_dir_fail) {
-  try {
-    auto tmp_dir = mysql_harness::get_tmp_dir("/no/such/directory/test");
-
-    FAIL() << "expected get_tmp_dir() to fail";
-  } catch (const std::system_error &) {
-    SUCCEED();
-  } catch (const std::exception &e) {
-    FAIL() << "expected std::system_error, got " << e.what();
-  }
-}
-
-/*
- * Tests mysql_harness::mkdir()
- */
-TEST(TestFilesystem, TempDirectory_Constructor_fail) {
-  try {
-    auto tmp_dir = TempDirectory("/no/such/directory/test");
-
-    FAIL() << "expected TempDirectyory() to fail";
-  } catch (const std::system_error &) {
-    SUCCEED();
-  } catch (const std::exception &e) {
-    FAIL() << "expected std::system_error, got " << e.what();
-  }
-}
-
 int main(int argc, char *argv[]) {
   g_here = Path(argv[0]).dirname();
 
