@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2018, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -26,14 +26,18 @@
  * HTTP server plugin.
  */
 
+#include <array>
 #include <atomic>
 #include <chrono>
 #include <future>
+#include <memory>  // shared_ptr
 #include <mutex>
+#include <sstream>
 #include <stdexcept>
+#include <system_error>
 #include <thread>
 
-#include <sys/types.h>
+#include <sys/types.h>  // timeval
 
 #include <event2/buffer.h>
 #include <event2/bufferevent.h>
@@ -48,28 +52,27 @@
 #include "mysql/harness/config_parser.h"
 #include "mysql/harness/loader.h"
 #include "mysql/harness/logging/logging.h"
+#include "mysql/harness/net_ts/impl/socket_error.h"
+#include "mysql/harness/net_ts/internet.h"
+#include "mysql/harness/net_ts/io_context.h"
+#include "mysql/harness/net_ts/socket.h"
 #include "mysql/harness/plugin.h"
 #include "mysql/harness/utility/string.h"
 
 #include "http_auth.h"
 #include "http_server_plugin.h"
+#include "mysql/harness/tls_server_context.h"
 #include "mysqlrouter/http_auth_realm_component.h"
+#include "mysqlrouter/http_common.h"
 #include "mysqlrouter/http_server_component.h"
 #include "mysqlrouter/plugin_config.h"
 #include "posix_re.h"
-#include "socket_operations.h"
 #include "static_files.h"
-#include "tls_server_context.h"
 
 IMPORT_LOG_FUNCTIONS()
 
 static constexpr const char kSectionName[]{"http_server"};
 
-using mysql_harness::ARCHITECTURE_DESCRIPTOR;
-using mysql_harness::Plugin;
-using mysql_harness::PLUGIN_ABI_VERSION;
-using mysql_harness::PluginFuncEnv;
-
 std::promise<void> stopper;
 std::future<void> stopped = stopper.get_future();
 
@@ -134,6 +137,33 @@ void HttpRequestRouter::route(HttpRequest req) {
 
   auto uri = req.get_uri();
 
+  // CONNECT can't be routed to the request handlers as it doesn't have a "path"
+  // part.
+  //
+  // If the client Accepts "application/problem+json", send it a RFC7807 error
+  // otherwise a classic text/html one.
+  if (req.get_method() == HttpMethod::Connect) {
+    const char *hdr_accept = req.get_input_headers().get("Accept");
+    if (hdr_accept &&
+        std::string(hdr_accept).find("application/problem+json") !=
+            std::string::npos) {
+      req.get_output_headers().add("Content-Type", "application/problem+json");
+      auto buffers = req.get_output_buffer();
+      std::string json_problem(R"({
+  "title": "Method Not Allowed",
+  "status": 405
+})");
+      buffers.add(json_problem.data(), json_problem.size());
+      int status_code = HttpStatusCode::MethodNotAllowed;
+      req.send_reply(status_code,
+                     HttpStatusCode::get_default_status_text(status_code),
+                     buffers);
+    } else {
+      req.send_error(HttpStatusCode::MethodNotAllowed);
+    }
+    return;
+  }
+
   for (auto &request_handler : request_handlers_) {
     if (request_handler.url_regex.search(uri.get_path())) {
       request_handler.handler->handle_request(req);
@@ -182,95 +212,64 @@ void HttpRequestThread::wait_and_dispatch() {
 class HttpRequestMainThread : public HttpRequestThread {
  public:
   void bind(const std::string &address, uint16_t port) {
-    int err;
-    struct addrinfo hints, *ainfo;
-
-    auto *sock_ops = mysql_harness::SocketOperations::instance();
-
-    std::memset(&hints, 0, sizeof(hints));
-    hints.ai_family = AF_UNSPEC;
-    hints.ai_socktype = SOCK_STREAM;
-    hints.ai_flags = AI_PASSIVE;
-
-    err = getaddrinfo(address.c_str(), std::to_string(port).c_str(), &hints,
-                      &ainfo);
-    if (err != 0) {
-      throw std::runtime_error(std::string("getaddrinfo() failed: ") +
-                               gai_strerror(err));
+    net::io_context io_ctx;
+    net::ip::tcp::resolver resolver(io_ctx);
+    auto resolve_res = resolver.resolve(address, std::to_string(port));
+    if (!resolve_res) {
+      throw std::system_error(resolve_res.error(),
+                              "resolving " + address + " failed");
     }
 
-    std::shared_ptr<void> exit_guard(nullptr,
-                                     [&](void *) { freeaddrinfo(ainfo); });
+    for (auto const &resolved : resolve_res.value()) {
+      net::ip::tcp::acceptor sock(io_ctx);
 
-    accept_fd_ = sock_ops->socket(ainfo->ai_family, ainfo->ai_socktype,
-                                  ainfo->ai_protocol);
-    if (accept_fd_ == mysql_harness::kInvalidSocket) {
-      throw std::system_error(sock_ops->get_error_code(), "socket() failed");
+      auto open_res = sock.open(resolved.endpoint().protocol());
+      if (!open_res) {
+        throw std::system_error(open_res.error(), "socket() failed");
+      }
+
+      sock.native_non_blocking(true);
+      auto setop_res = sock.set_option(net::socket_base::reuse_address(true));
+      if (!setop_res) {
+        throw std::system_error(setop_res.error(),
+                                "setsockopt(SO_REUSEADDR) failed");
+      }
+      setop_res = sock.set_option(net::socket_base::keep_alive(true));
+      if (!setop_res) {
+        throw std::system_error(setop_res.error(),
+                                "setsockopt(SO_KEEPALIVE) failed");
+      }
+
+      auto bind_res = sock.bind(resolved.endpoint());
+      if (!bind_res) {
+        std::ostringstream ss;
+        ss << "bind(" << resolved.endpoint() << ") failed";
+
+        throw std::system_error(bind_res.error(), ss.str());
+      }
+      auto listen_res = sock.listen(128);
+      if (!listen_res) {
+        throw std::system_error(setop_res.error(), "listen(128) failed");
+      }
+
+      auto sock_release_res = sock.release();
+      if (!sock_release_res) {
+        throw std::system_error(sock_release_res.error(), "release() failed");
+      }
+
+      auto handle = evhttp_accept_socket_with_handle(ev_http.get(),
+                                                     sock_release_res.value());
+      if (nullptr == handle) {
+        const auto ec = net::impl::socket::last_error_code();
+
+        throw std::system_error(ec,
+                                "evhttp_accept_socket_with_handle() failed");
+      }
+
+      return;
     }
 
-    if (evutil_make_socket_nonblocking(accept_fd_) < 0) {
-      const auto ec = sock_ops->get_error_code();
-
-      sock_ops->close(accept_fd_);
-
-      throw std::system_error(ec, "evutil_make_socket_nonblocking() failed");
-    }
-
-    if (evutil_make_socket_closeonexec(accept_fd_) < 0) {
-      const auto ec = sock_ops->get_error_code();
-
-      sock_ops->close(accept_fd_);
-
-      throw std::system_error(ec, "evutil_make_socket_closeonexec() failed");
-    }
-
-    int option_value = 1;
-    if (sock_ops->setsockopt(accept_fd_, SOL_SOCKET, SO_REUSEADDR,
-                             reinterpret_cast<const char *>(&option_value),
-                             static_cast<socklen_t>(sizeof(int))) == -1) {
-      const auto ec = sock_ops->get_error_code();
-
-      sock_ops->close(accept_fd_);
-
-      throw std::system_error(ec, "setsockopt(SO_REUSEADDR) failed");
-    }
-    if (sock_ops->setsockopt(accept_fd_, SOL_SOCKET, SO_KEEPALIVE,
-                             reinterpret_cast<const char *>(&option_value),
-                             static_cast<socklen_t>(sizeof(int))) == -1) {
-      const auto ec = sock_ops->get_error_code();
-
-      sock_ops->close(accept_fd_);
-
-      throw std::system_error(ec, "setsockopt(SO_KEEPALIVE) failed");
-    }
-
-    err = sock_ops->bind(accept_fd_, ainfo->ai_addr, ainfo->ai_addrlen);
-    if (err < 0) {
-      auto ec = sock_ops->get_error_code();
-
-      sock_ops->close(accept_fd_);
-
-      throw std::system_error(
-          ec, "bind('0.0.0.0:" + std::to_string(port) + ") failed");
-    }
-
-    if (sock_ops->listen(accept_fd_, 128) == -1) {
-      auto ec = sock_ops->get_error_code();
-
-      sock_ops->close(accept_fd_);
-
-      throw std::system_error(ec, "listen() failed");
-    }
-
-    auto handle = evhttp_accept_socket_with_handle(ev_http.get(), accept_fd_);
-
-    if (nullptr == handle) {
-      auto ec = sock_ops->get_error_code();
-
-      sock_ops->close(accept_fd_);
-
-      throw std::system_error(ec, "evhttp_accept_socket_with_handle() failed");
-    }
+    throw std::invalid_argument("resolved to nothing?");
   }
 };
 
@@ -292,7 +291,7 @@ class HttpsRequestMainThread : public HttpRequestMainThread {
 
 class HttpRequestWorkerThread : public HttpRequestThread {
  public:
-  explicit HttpRequestWorkerThread(harness_socket_t accept_fd) {
+  explicit HttpRequestWorkerThread(native_handle_type accept_fd) {
     accept_fd_ = accept_fd;
   }
 };
@@ -300,7 +299,7 @@ class HttpRequestWorkerThread : public HttpRequestThread {
 #ifdef EVENT__HAVE_OPENSSL
 class HttpsRequestWorkerThread : public HttpRequestWorkerThread {
  public:
-  explicit HttpsRequestWorkerThread(harness_socket_t accept_fd,
+  explicit HttpsRequestWorkerThread(native_handle_type accept_fd,
                                     SSL_CTX *ssl_ctx)
       : HttpRequestWorkerThread(accept_fd) {
     evhttp_set_bevcb(
@@ -330,7 +329,8 @@ void HttpServer::start(size_t max_threads) {
     thread_contexts_.emplace_back(std::move(main_thread));
   }
 
-  const harness_socket_t accept_fd = thread_contexts_[0].get_socket_fd();
+  const net::impl::socket::native_handle_type accept_fd =
+      thread_contexts_[0].get_socket_fd();
   for (size_t ndx = 1; ndx < max_threads; ndx++) {
     thread_contexts_.emplace_back(HttpRequestWorkerThread(accept_fd));
   }
@@ -367,7 +367,8 @@ void HttpsServer::start(size_t max_threads) {
     thread_contexts_.emplace_back(std::move(main_thread));
   }
 
-  const harness_socket_t accept_fd = thread_contexts_[0].get_socket_fd();
+  const net::impl::socket::native_handle_type accept_fd =
+      thread_contexts_[0].get_socket_fd();
   for (size_t ndx = 1; ndx < max_threads; ndx++) {
     thread_contexts_.emplace_back(
         HttpsRequestWorkerThread(accept_fd, ssl_ctx_.get()));
@@ -406,7 +407,7 @@ void HttpServer::remove_route(const std::string &url_regex) {
   }
 }
 
-class PluginConfig : public mysqlrouter::BasePluginConfig {
+class HttpServerPluginConfig : public mysqlrouter::BasePluginConfig {
  public:
   std::string static_basedir;
   std::string srv_address;
@@ -419,7 +420,7 @@ class PluginConfig : public mysqlrouter::BasePluginConfig {
   bool with_ssl;
   uint16_t srv_port;
 
-  explicit PluginConfig(const mysql_harness::ConfigSection *section)
+  explicit HttpServerPluginConfig(const mysql_harness::ConfigSection *section)
       : mysqlrouter::BasePluginConfig(section),
         static_basedir(get_option_string(section, "static_folder")),
         srv_address(get_option_string(section, "bind_address")),
@@ -459,15 +460,29 @@ static std::map<std::string, std::shared_ptr<HttpServer>> http_servers;
 
 class HttpServerFactory {
  public:
-  static std::shared_ptr<HttpServer> create(const PluginConfig &config) {
+  static std::shared_ptr<HttpServer> create(
+      const HttpServerPluginConfig &config) {
     if (config.with_ssl) {
       // init the TLS Server context according to our config-values
       TlsServerContext tls_ctx;
-      tls_ctx.load_key_and_cert(config.ssl_cert, config.ssl_key);
+
+      {
+        const auto res =
+            tls_ctx.load_key_and_cert(config.ssl_key, config.ssl_cert);
+        if (!res) {
+          throw std::system_error(
+              res.error(), "using SSL private key file '" + config.ssl_key +
+                               "' or SSL certificate file '" + config.ssl_cert +
+                               "' failed");
+        }
+      }
 
       if (!config.ssl_curves.empty()) {
         if (tls_ctx.has_set_curves_list()) {
-          tls_ctx.curves_list(config.ssl_curves);
+          const auto res = tls_ctx.curves_list(config.ssl_curves);
+          if (!res) {
+            throw std::system_error(res.error(), "using ssl-curves failed");
+          }
         } else {
           throw std::invalid_argument(
               "setting ssl-curves is not supported by the ssl library, it "
@@ -475,9 +490,19 @@ class HttpServerFactory {
         }
       }
 
-      tls_ctx.init_tmp_dh(config.ssl_dh_params);
+      {
+        const auto res = tls_ctx.init_tmp_dh(config.ssl_dh_params);
+        if (!res) {
+          throw std::system_error(res.error(), "setting ssl_dh_params failed");
+        }
+      }
 
-      if (!config.ssl_cipher.empty()) tls_ctx.cipher_list(config.ssl_cipher);
+      if (!config.ssl_cipher.empty()) {
+        const auto res = tls_ctx.cipher_list(config.ssl_cipher);
+        if (!res) {
+          throw std::system_error(res.error(), "using ssl-cipher list failed");
+        }
+      }
 
 #ifdef EVENT__HAVE_OPENSSL
       // tls-context is owned by the HttpsServer
@@ -493,7 +518,7 @@ class HttpServerFactory {
   }
 };
 
-static void init(PluginFuncEnv *env) {
+static void init(mysql_harness::PluginFuncEnv *env) {
   const mysql_harness::AppInfo *info = get_app_info(env);
   bool has_started = false;
 
@@ -501,9 +526,6 @@ static void init(PluginFuncEnv *env) {
     return;
   }
 
-  // calls the openssl library initialize code
-  TlsLibraryContext tls_lib_ctx;
-
   // assume there is only one section for us
   try {
     std::set<std::string> known_realms;
@@ -526,7 +548,7 @@ static void init(PluginFuncEnv *env) {
 
       has_started = true;
 
-      PluginConfig config{section};
+      HttpServerPluginConfig config{section};
 
       if (config.with_ssl &&
           (config.ssl_cert.empty() || config.ssl_key.empty())) {
@@ -569,7 +591,7 @@ static void init(PluginFuncEnv *env) {
   }
 }
 
-static void start(PluginFuncEnv *env) {
+static void start(mysql_harness::PluginFuncEnv *env) {
   // - version string
   // - hostname
   // - active-connections vs. max-connections
@@ -598,6 +620,7 @@ static void start(PluginFuncEnv *env) {
     // add routes
 
     srv->start(8);
+    mysql_harness::on_service_ready(env);
 
     // wait until we got asked to shutdown.
     //
@@ -615,19 +638,28 @@ static void start(PluginFuncEnv *env) {
   }
 }
 
+const static std::array<const char *, 3> required = {{
+    "logger",
+    "router_openssl",
+    // as long as this plugin links against http_auth_backend_lib which links
+    // against metadata_cache there is a need to cleanup protobuf
+    "router_protobuf",
+}};
+
 extern "C" {
-Plugin HTTP_SERVER_EXPORT harness_plugin_http_server = {
-    PLUGIN_ABI_VERSION,
-    ARCHITECTURE_DESCRIPTOR,
-    "HTTP_SERVER",
+mysql_harness::Plugin HTTP_SERVER_EXPORT harness_plugin_http_server = {
+    mysql_harness::PLUGIN_ABI_VERSION,       // abi-version
+    mysql_harness::ARCHITECTURE_DESCRIPTOR,  // arch
+    "HTTP_SERVER",                           // name
     VERSION_NUMBER(0, 0, 1),
-    0,
-    nullptr,  // requires
-    0,
-    nullptr,  // conflicts
+    // requires
+    required.size(), required.data(),
+    // conflicts
+    0, nullptr,
     init,     // init
     nullptr,  // deinit
     start,    // start
     nullptr,  // stop
+    true,     // declares_readiness
 };
 }
