@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2016, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -32,8 +32,12 @@
 
 #include "common.h"
 #include "mysql/harness/logging/logging.h"
+#include "mysql/harness/plugin.h"
 #include "mysqlrouter/mysql_client_thread_token.h"
 
+using namespace std::chrono_literals;
+using namespace std::string_literals;
+
 IMPORT_LOG_FUNCTIONS()
 
 MetadataCache::MetadataCache(
@@ -101,7 +105,14 @@ void MetadataCache::refresh_thread() {
   while (!terminated_) {
     bool refresh_ok{false};
     try {
+      // Component tests are using this log message as a indicator of metadata
+      // refresh start
+      log_debug("Started refreshing the cluster metadata");
       refresh_ok = refresh();
+      // Component tests are using this log message as a indicator of metadata
+      // refresh finish
+      log_debug("Finished refreshing the cluster metadata");
+      on_refresh_completed();
     } catch (const mysqlrouter::MetadataUpgradeInProgressException &) {
       log_info(
           "Cluster metadata upgrade in progress, aborting the metada refresh");
@@ -111,6 +122,12 @@ void MetadataCache::refresh_thread() {
     }
 
     if (refresh_ok) {
+      if (!ready_announced_) {
+        ready_announced_ = true;
+        mysql_harness::on_service_ready(
+            "metadata_cache:" +
+            metadata_cache::MetadataCacheAPI::instance()->instance_name());
+      }
       // we want to update the router version in the routers table once
       // when we start
       if (!version_updated_) {
@@ -157,12 +174,21 @@ void MetadataCache::refresh_thread() {
     // wait for up to TTL until next refresh, unless some replicaset loses an
     // online (primary or secondary) server - in that case, "emergency mode" is
     // enabled and we refresh every 1s until "emergency mode" is called off.
-    while (ttl_left > std::chrono::milliseconds(0)) {
+    while (ttl_left > 0ms) {
       auto sleep_for =
           std::min(ttl_left, kTerminateOrForcedRefreshCheckInterval);
 
       {
         std::unique_lock<std::mutex> lock(refresh_wait_mtx_);
+        // frist check if we were not told to leave or refresh again while we
+        // were outside of the wait_for
+        if (terminated_) return;
+        if (refresh_requested_) {
+          auth_cache_force_update = true;
+          refresh_requested_ = false;
+          break;  // go to the refresh() in the outer loop
+        }
+
         if (sleep_for >= auth_cache_ttl_left) {
           refresh_wait_.wait_for(lock, auth_cache_ttl_left);
           ttl_left -= auth_cache_ttl_left;
@@ -219,18 +245,16 @@ void MetadataCache::start() {
  */
 void MetadataCache::stop() noexcept {
   {
-    std::unique_lock<std::mutex> lk(refresh_wait_mtx_);
+    std::unique_lock<std::mutex> lk(refresh_wait_mtx_, std::defer_lock);
+    std::unique_lock<std::mutex> lk2(refresh_completed_mtx_, std::defer_lock);
+    std::lock(lk, lk2);
     terminated_ = true;
   }
   refresh_wait_.notify_one();
+  refresh_completed_.notify_one();
   refresh_thread_.join();
 }
 
-/**
- * Return a list of servers that are part of a replicaset.
- *
- * @param replicaset_name The replicaset that is being looked up.
- */
 MetadataCache::metadata_servers_list_t MetadataCache::replicaset_lookup(
     const std::string &replicaset_name) {
   std::lock_guard<std::mutex> lock(cache_refreshing_mutex_);
@@ -249,34 +273,27 @@ MetadataCache::metadata_servers_list_t MetadataCache::replicaset_lookup(
 bool metadata_cache::ManagedInstance::operator==(
     const ManagedInstance &other) const {
   return mysql_server_uuid == other.mysql_server_uuid &&
-         replicaset_name == other.replicaset_name && role == other.role &&
-         mode == other.mode &&
-         std::fabs(weight - other.weight) <
-             0.001 &&  // 0.001 = reasonable guess, change if needed
-         host == other.host &&
-         port == other.port && version_token == other.version_token &&
-         xport == other.xport;
+         replicaset_name == other.replicaset_name && mode == other.mode &&
+         host == other.host && port == other.port && xport == other.xport &&
+         hidden == other.hidden &&
+         disconnect_existing_sessions_when_hidden ==
+             other.disconnect_existing_sessions_when_hidden;
 }
 
 metadata_cache::ManagedInstance::ManagedInstance(
     const std::string &p_replicaset_name,
-    const std::string &p_mysql_server_uuid, const std::string &p_role,
-    const ServerMode p_mode, const float p_weight,
-    const unsigned int p_version_token, const std::string &p_host,
-    const uint16_t p_port, const uint16_t p_xport)
+    const std::string &p_mysql_server_uuid, const ServerMode p_mode,
+    const std::string &p_host, const uint16_t p_port, const uint16_t p_xport)
     : replicaset_name(p_replicaset_name),
       mysql_server_uuid(p_mysql_server_uuid),
-      role(p_role),
       mode(p_mode),
-      weight(p_weight),
-      version_token(p_version_token),
       host(p_host),
       port(p_port),
       xport(p_xport) {}
 
 metadata_cache::ManagedInstance::ManagedInstance(const TCPAddress &addr) {
-  host = addr.addr == "localhost" ? "127.0.0.1" : addr.addr;
-  port = addr.port;
+  host = addr.address() == "localhost" ? "127.0.0.1" : addr.address();
+  port = addr.port();
 }
 
 metadata_cache::ManagedInstance::operator TCPAddress() const {
@@ -326,6 +343,19 @@ std::string to_string(metadata_cache::ServerMode mode) {
   }
 }
 
+std::string get_hidden_info(const metadata_cache::ManagedInstance &instance) {
+  std::string result;
+  // if both values are default return empty string
+  if (instance.hidden || !instance.disconnect_existing_sessions_when_hidden) {
+    result =
+        "hidden=" + (instance.hidden ? "yes"s : "no"s) +
+        " disconnect_when_hidden=" +
+        (instance.disconnect_existing_sessions_when_hidden ? "yes"s : "no"s);
+  }
+
+  return result;
+}
+
 void MetadataCache::on_refresh_failed(bool terminated) {
   refresh_failed_++;
   last_refresh_failed_ = std::chrono::system_clock::now();
@@ -388,6 +418,8 @@ void MetadataCache::on_refresh_requested() {
   refresh_wait_.notify_one();
 }
 
+void MetadataCache::on_refresh_completed() { refresh_completed_.notify_one(); }
+
 void MetadataCache::mark_instance_reachability(
     const std::string &instance_id, metadata_cache::InstanceStatus status) {
   // If the status is that the primary or secondary instance is physically
@@ -442,20 +474,31 @@ void MetadataCache::mark_instance_reachability(
 }
 
 bool MetadataCache::wait_primary_failover(const std::string &replicaset_name,
-                                          int timeout) {
-  log_debug("Waiting for failover to happen in '%s' for %is",
-            replicaset_name.c_str(), timeout);
-  time_t stime = std::time(nullptr);
-  while (std::time(nullptr) - stime <= timeout) {
-    {
-      std::lock_guard<std::mutex> lock(cache_refreshing_mutex_);
-      if (replicasets_with_unreachable_nodes_.count(replicaset_name) == 0) {
-        return true;
-      }
+                                          const std::chrono::seconds &timeout) {
+  log_debug("Waiting for failover to happen in '%s' for %lds",
+            replicaset_name.c_str(), static_cast<long>(timeout.count()));
+
+  const auto start = std::chrono::steady_clock::now();
+  std::chrono::milliseconds timeout_left = timeout;
+  do {
+    if (terminated_) {
+      return false;
     }
-    std::this_thread::sleep_for(metadata_cache::kDefaultMetadataTTL);
-  }
-  return false;
+    if (replicasets_with_unreachable_nodes_.count(replicaset_name) == 0) {
+      return true;
+    }
+    std::unique_lock<std::mutex> lock(refresh_completed_mtx_);
+    const auto wait_res = refresh_completed_.wait_for(lock, timeout_left);
+    if (wait_res == std::cv_status::timeout) {
+      return false;
+    }
+    const auto time_passed =
+        std::chrono::duration_cast<std::chrono::milliseconds>(
+            start - std::chrono::steady_clock::now());
+    timeout_left -= time_passed;
+  } while (timeout_left > 0ms);
+
+  return replicasets_with_unreachable_nodes_.count(replicaset_name) == 0;
 }
 
 void MetadataCache::add_listener(
@@ -473,8 +516,7 @@ void MetadataCache::remove_listener(
 }
 
 void MetadataCache::check_auth_metadata_timers() const {
-  if (auth_cache_ttl_ > std::chrono::milliseconds(0) &&
-      auth_cache_ttl_ < ttl_) {
+  if (auth_cache_ttl_ > 0ms && auth_cache_ttl_ < ttl_) {
     throw std::invalid_argument(
         "'auth_cache_ttl' option value '" +
         std::to_string(static_cast<float>(auth_cache_ttl_.count()) / 1000) +
@@ -489,8 +531,7 @@ void MetadataCache::check_auth_metadata_timers() const {
         "' cannot be less than the 'ttl' value which is '" +
         std::to_string(static_cast<float>(ttl_.count()) / 1000) + "'");
   }
-  if (auth_cache_ttl_ > std::chrono::milliseconds(0) &&
-      auth_cache_refresh_interval_ > auth_cache_ttl_) {
+  if (auth_cache_ttl_ > 0ms && auth_cache_refresh_interval_ > auth_cache_ttl_) {
     throw std::invalid_argument(
         "'auth_cache_ttl' option value '" +
         std::to_string(static_cast<float>(auth_cache_ttl_.count()) / 1000) +
