@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -64,14 +63,15 @@ class Codec<frame::Header> : public impl::EncodeBase<Codec<frame::Header>> {
 
   static constexpr size_t max_size() noexcept { return 4; }
 
+  template <class ConstBufferSequence>
   static stdx::expected<std::pair<size_t, value_type>, std::error_code> decode(
-      const net::const_buffer &buffer, capabilities::value_type caps) {
-    impl::DecodeBufferAccumulator accu(buffer, caps);
+      const ConstBufferSequence &buffers, capabilities::value_type caps) {
+    impl::DecodeBufferAccumulator<ConstBufferSequence> accu(buffers, caps);
 
     auto payload_size_res = accu.template step<wire::FixedInt<3>>();
     auto seq_id_res = accu.template step<wire::FixedInt<1>>();
 
-    if (!accu.result()) return stdx::unexpected(accu.result().error());
+    if (!accu.result()) return stdx::make_unexpected(accu.result().error());
 
     return std::make_pair(
         accu.result().value(),
@@ -107,15 +107,16 @@ class Codec<frame::CompressedHeader>
 
   static size_t max_size() noexcept { return 7; }
 
+  template <class ConstBufferSequence>
   static stdx::expected<std::pair<size_t, value_type>, std::error_code> decode(
-      const net::const_buffer &buffer, capabilities::value_type caps) {
-    impl::DecodeBufferAccumulator accu(buffer, caps);
+      const ConstBufferSequence &buffers, capabilities::value_type caps) {
+    impl::DecodeBufferAccumulator<ConstBufferSequence> accu(buffers, caps);
 
     auto payload_size_res = accu.template step<wire::FixedInt<3>>();
     auto seq_id_res = accu.template step<wire::FixedInt<1>>();
     auto uncompressed_size_res = accu.template step<wire::FixedInt<3>>();
 
-    if (!accu.result()) return stdx::unexpected(accu.result().error());
+    if (!accu.result()) return stdx::make_unexpected(accu.result().error());
 
     return std::make_pair(
         accu.result().value(),
@@ -156,25 +157,26 @@ class Codec<frame::Frame<PayloadType>>
   constexpr Codec(value_type v, capabilities::value_type caps)
       : __base(caps), v_{std::move(v)} {}
 
+  template <class ConstBufferSequence>
   static stdx::expected<std::pair<size_t, value_type>, std::error_code> decode(
-      net::const_buffer buffer, capabilities::value_type caps) {
-    impl::DecodeBufferAccumulator accu(buffer, caps);
+      const ConstBufferSequence &buffers, capabilities::value_type caps) {
+    impl::DecodeBufferAccumulator<ConstBufferSequence> accu(buffers, caps);
 
     auto header_res = accu.template step<frame::Header>();
-    if (!accu.result()) return stdx::unexpected(accu.result().error());
+    if (!accu.result()) return stdx::make_unexpected(accu.result().error());
 
     constexpr const size_t header_size{Codec<frame::Header>::max_size()};
 
     // check the payload is at least what we expect.
-    if (buffer.size() < header_size + header_res->payload_size()) {
-      return stdx::unexpected(
+    if (net::buffer_size(buffers) < header_size + header_res->payload_size()) {
+      return stdx::make_unexpected(
           make_error_code(classic_protocol::codec_errc::not_enough_input));
     }
 
     auto payload_res =
         accu.template step<PayloadType>(header_res->payload_size());
 
-    if (!accu.result()) return stdx::unexpected(accu.result().error());
+    if (!accu.result()) return stdx::make_unexpected(accu.result().error());
 
     return std::make_pair(
         accu.result().value(),
