@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -25,18 +25,19 @@
 #ifndef MYSQLROUTER_REST_API_UTILS_INCLUDED
 #define MYSQLROUTER_REST_API_UTILS_INCLUDED
 
-#include <chrono>
 #include <map>
 #include <string>
 
 #ifdef RAPIDJSON_NO_SIZETYPEDEFINE
+// if we build within the server, it will set RAPIDJSON_NO_SIZETYPEDEFINE
+// globally and require to include my_rapidjson_size_t.h
 #include "my_rapidjson_size_t.h"
 #endif
 
 #include <rapidjson/document.h>
 
-#include "mysql/harness/utility/string.h"  // string_format()
-#include "mysqlrouter/http_request.h"
+#include "mysqlrouter/http_common.h"
+#include "mysqlrouter/utils.h"  // string_format()
 
 /**
  * send a JsonProblem HTTP response.
@@ -70,7 +71,7 @@ bool ensure_http_method(HttpRequest &req, HttpMethod::Bitset allowed_methods);
 /**
  * ensure request is authenticated.
  *
- * sends HTTP-response with status 401 if authentication was not successful.
+ * sends HTTP-response with status 401 if authentication was not succesful.
  *
  * @returns success
  * @retval true if request is authenticaticated
@@ -139,7 +140,7 @@ rapidjson::GenericValue<Encoding, AllocatorType> json_value_from_timepoint(
   auto usec = std::chrono::duration_cast<std::chrono::microseconds>(
       tp - std::chrono::system_clock::from_time_t(cur));
 
-  std::string iso8601_datetime{mysql_harness::utility::string_format(
+  std::string iso8601_datetime{mysqlrouter::string_format(
       "%04d-%02d-%02dT%02d:%02d:%02d.%06ldZ", cur_gmtime.tm_year + 1900,
       cur_gmtime.tm_mon + 1, cur_gmtime.tm_mday, cur_gmtime.tm_hour,
       cur_gmtime.tm_min, cur_gmtime.tm_sec,
