@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2016, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,8 +27,6 @@
 #include <algorithm>
 #include <chrono>
 #include <functional>
-#include <map>
-#include <queue>
 #include <sstream>
 #include <string>
 
@@ -52,75 +49,10 @@ using namespace std::string_literals;
 /*static*/ const char MySQLSession::kSslModeVerifyIdentity[] =
     "VERIFY_IDENTITY";
 
-namespace {
+/*static*/ const std::function<void(unsigned, MYSQL_FIELD *)>
+    MySQLSession::null_field_validator = [](unsigned, MYSQL_FIELD *) {};
 
-class SSLSessionsCache {
- public:
-  using EndpointId = std::string;
-  using CachedType = std::string;
-
-  static SSLSessionsCache &instance() {
-    static SSLSessionsCache i;
-    return i;
-  }
-
-  bool was_session_reused(MYSQL *con) {
-    return mysql_get_ssl_session_reused(con);
-  }
-
-  void store_ssl_session(MYSQL *con, const EndpointId &endpoint_id) {
-    unsigned int len = 0;
-    std::lock_guard lk(mtx_);
-
-    static_assert(kMaxEntriesPerEndpoint > 0);
-
-    void *data = mysql_get_ssl_session_data(con, 0, &len);
-    if (len == 0) {
-      // we failed to get the ssl session data, nothing to store
-      return;
-    }
-
-    if (cache_.count(endpoint_id) > 0 &&
-        cache_[endpoint_id].size() >= kMaxEntriesPerEndpoint) {
-      // cache is full, remove the oldest entry to make room for the new one
-      cache_[endpoint_id].pop();
-    }
-
-    cache_[endpoint_id].emplace(reinterpret_cast<char *>(data), len);
-
-    mysql_free_ssl_session_data(con, data);
-  }
-
-  void try_reuse_session(MYSQL *con, const EndpointId &endpoint_id) {
-    std::lock_guard lk(mtx_);
-
-    if (cache_.count(endpoint_id) == 0 || cache_[endpoint_id].empty()) {
-      return;
-    }
-
-    const auto &sess_data = cache_[endpoint_id].front();
-    mysql_options(con, MYSQL_OPT_SSL_SESSION_DATA, sess_data.c_str());
-
-    // once the session data was reused remove it from the cache
-    cache_[endpoint_id].pop();
-  }
-
- private:
-  SSLSessionsCache() = default;
-
-  // disable copying
-  SSLSessionsCache(SSLSessionsCache &) = delete;
-  SSLSessionsCache *operator=(SSLSessionsCache &) = delete;
-
-  std::map<EndpointId, std::queue<CachedType>> cache_;
-  std::mutex mtx_;
-
-  static constexpr size_t kMaxEntriesPerEndpoint{2};
-};
-
-}  // namespace
-
-MySQLSession::MySQLSession(std::unique_ptr<LoggingStrategy> logging_strategy)
+MySQLSession::MySQLSession(std::unique_ptr<LoggingStrategy> &&logging_strategy)
     : logging_strategy_(std::move(logging_strategy)) {
   MySQLClientThreadToken api_token;
 
@@ -185,158 +117,79 @@ void MySQLSession::set_ssl_options(mysql_ssl_mode ssl_mode,
                                    const std::string &capath,
                                    const std::string &crl,
                                    const std::string &crlpath) {
-  if (!ssl_cipher.empty() && !set_option(SslCipher(ssl_cipher.c_str()))) {
+  if (!ssl_cipher.empty() && mysql_options(connection_, MYSQL_OPT_SSL_CIPHER,
+                                           ssl_cipher.c_str()) != 0) {
     throw Error(("Error setting SSL_CIPHER option for MySQL connection: " +
-                 std::string(mysql_error(connection_))),
+                 std::string(mysql_error(connection_)))
+                    .c_str(),
                 mysql_errno(connection_));
   }
 
-  if (!tls_version.empty() && !set_option(TlsVersion(tls_version.c_str()))) {
+  if (!tls_version.empty() && mysql_options(connection_, MYSQL_OPT_TLS_VERSION,
+                                            tls_version.c_str()) != 0) {
     throw Error("Error setting TLS_VERSION option for MySQL connection",
                 mysql_errno(connection_));
   }
 
-  if (!ca.empty() && !set_option(SslCa(ca.c_str()))) {
+  if (!ca.empty() &&
+      mysql_options(connection_, MYSQL_OPT_SSL_CA, ca.c_str()) != 0) {
     throw Error(("Error setting SSL_CA option for MySQL connection: " +
-                 std::string(mysql_error(connection_))),
+                 std::string(mysql_error(connection_)))
+                    .c_str(),
                 mysql_errno(connection_));
   }
 
-  if (!capath.empty() && !set_option(SslCaPath(capath.c_str()))) {
+  if (!capath.empty() &&
+      mysql_options(connection_, MYSQL_OPT_SSL_CAPATH, capath.c_str()) != 0) {
     throw Error(("Error setting SSL_CAPATH option for MySQL connection: " +
-                 std::string(mysql_error(connection_))),
+                 std::string(mysql_error(connection_)))
+                    .c_str(),
                 mysql_errno(connection_));
   }
 
-  if (!crl.empty() && !set_option(SslCrl(crl.c_str()))) {
+  if (!crl.empty() &&
+      mysql_options(connection_, MYSQL_OPT_SSL_CRL, crl.c_str()) != 0) {
     throw Error(("Error setting SSL_CRL option for MySQL connection: " +
-                 std::string(mysql_error(connection_))),
+                 std::string(mysql_error(connection_)))
+                    .c_str(),
                 mysql_errno(connection_));
   }
 
-  if (!crlpath.empty() && !set_option(SslCrlPath(crlpath.c_str()))) {
+  if (!crlpath.empty() &&
+      mysql_options(connection_, MYSQL_OPT_SSL_CRLPATH, crlpath.c_str()) != 0) {
     throw Error(("Error setting SSL_CRLPATH option for MySQL connection: " +
-                 std::string(mysql_error(connection_))),
+                 std::string(mysql_error(connection_)))
+                    .c_str(),
                 mysql_errno(connection_));
   }
 
   // this has to be the last option that gets set due to what appears to be a
   // bug in libmysql causing ssl_mode downgrade from REQUIRED if other options
   // (like tls_version) are also specified
-  if (!set_option(SslMode(ssl_mode))) {
+  if (mysql_options(connection_, MYSQL_OPT_SSL_MODE, &ssl_mode) != 0) {
     const char *text = ssl_mode_to_string(ssl_mode);
     std::string msg = std::string("Setting SSL mode to '") + text +
                       "' on connection failed: " + mysql_error(connection_);
-    throw Error(msg, mysql_errno(connection_));
-  }
-}
-
-mysql_ssl_mode MySQLSession::ssl_mode() const {
-  SslMode ssl_mode;
-  if (!get_option(ssl_mode)) {
-    assert(0 && "get_option<SslMode>() failed unexpectedly");
+    throw Error(msg.c_str(), mysql_errno(connection_));
   }
 
-  return mysql_ssl_mode(ssl_mode.value());
-}
-
-std::string MySQLSession::tls_version() const {
-  TlsVersion tls_version;
-  if (!get_option(tls_version)) {
-    assert(0 && "get_option<TlsVersion>() failed unexpectedly");
-  }
-
-  return tls_version.value() ? tls_version.value() : "";
-}
-
-std::string MySQLSession::ssl_cipher() const {
-  SslCipher ssl_cipher;
-  if (!get_option(ssl_cipher)) {
-    assert(0 && "get_option<SslCipher>() failed unexpectedly");
-  }
-
-  return ssl_cipher.value() ? ssl_cipher.value() : "";
-}
-
-std::string MySQLSession::ssl_ca() const {
-  SslCa ssl_ca;
-  if (!get_option(ssl_ca)) {
-    assert(0 && "get_option<SslCa>() failed unexpectedly");
-  }
-
-  return ssl_ca.value() ? ssl_ca.value() : "";
-}
-
-std::string MySQLSession::ssl_capath() const {
-  SslCaPath ssl_capath;
-  if (!get_option(ssl_capath)) {
-    assert(0 && "get_option<SslCaPath>() failed unexpectedly");
-  }
-
-  return ssl_capath.value() ? ssl_capath.value() : "";
-}
-
-std::string MySQLSession::ssl_crl() const {
-  SslCrl ssl_crl;
-  if (!get_option(ssl_crl)) {
-    assert(0 && "get_option<SslCrl>() failed unexpectedly");
-  }
-
-  return ssl_crl.value() ? ssl_crl.value() : "";
-}
-
-std::string MySQLSession::ssl_crlpath() const {
-  SslCrlPath ssl_crlpath;
-  if (!get_option(ssl_crlpath)) {
-    assert(0 && "get_option<SslCrlPath>() failed unexpectedly");
-  }
-
-  return ssl_crlpath.value() ? ssl_crlpath.value() : "";
+  // archive options for future connection templating
+  conn_params_.ssl_opts = {ssl_mode, tls_version, ssl_cipher, ca,
+                           capath,   crl,         crlpath};
 }
 
 void MySQLSession::set_ssl_cert(const std::string &cert,
                                 const std::string &key) {
-  if (!set_option(SslCert(cert.c_str())) || !set_option(SslKey(key.c_str()))) {
-    throw Error("Error setting client SSL certificate for connection: " +
-                    std::string(mysql_error(connection_)),
+  if (mysql_options(connection_, MYSQL_OPT_SSL_CERT, cert.c_str()) != 0 ||
+      mysql_options(connection_, MYSQL_OPT_SSL_KEY, key.c_str()) != 0) {
+    throw Error(("Error setting client SSL certificate for connection: " +
+                 std::string(mysql_error(connection_)))
+                    .c_str(),
                 mysql_errno(connection_));
   }
-}
 
-std::string MySQLSession::ssl_cert() const {
-  SslCert ssl_cert;
-  if (!get_option(ssl_cert)) {
-    assert(0 && "get_option<SslCert>() failed unexpectedly");
-  }
-
-  return ssl_cert.value() ? ssl_cert.value() : "";
-}
-
-std::string MySQLSession::ssl_key() const {
-  SslKey ssl_key;
-  if (!get_option(ssl_key)) {
-    assert(0 && "get_option<SslKey>() failed unexpectedly");
-  }
-
-  return ssl_key.value() ? ssl_key.value() : "";
-}
-
-int MySQLSession::connect_timeout() const {
-  ConnectTimeout connect_timeout;
-  if (!get_option(connect_timeout)) {
-    assert(0 && "get_option<ConnectTimeout>() failed unexpectedly");
-  }
-
-  return connect_timeout.value();
-}
-
-int MySQLSession::read_timeout() const {
-  ReadTimeout read_timeout;
-  if (!get_option(read_timeout)) {
-    assert(0 && "get_option<ReadTimeout>() failed unexpectedly");
-  }
-
-  return read_timeout.value();
+  // archive options for future connection templating
+  conn_params_.ssl_cert = {cert, key};
 }
 
 void MySQLSession::connect(const std::string &host, unsigned int port,
@@ -350,72 +203,67 @@ void MySQLSession::connect(const std::string &host, unsigned int port,
 
   // Following would fail only when invalid values are given. It is not possible
   // for the user to change these values.
-  set_option(ConnectTimeout(connect_timeout));
-  set_option(ReadTimeout(read_timeout));
+  mysql_options(connection_, MYSQL_OPT_CONNECT_TIMEOUT, &connect_timeout);
+  mysql_options(connection_, MYSQL_OPT_READ_TIMEOUT, &read_timeout);
 
-  if (!unix_socket.empty()) {
+  if (unix_socket.length() > 0) {
 #ifdef _WIN32
     protocol = MYSQL_PROTOCOL_PIPE;
 #else
     protocol = MYSQL_PROTOCOL_SOCKET;
 #endif
   }
-  set_option(Protocol(protocol));
+  mysql_options(connection_, MYSQL_OPT_PROTOCOL,
+                reinterpret_cast<char *>(&protocol));
 
   const unsigned long client_flags =
       (CLIENT_LONG_PASSWORD | CLIENT_LONG_FLAG | CLIENT_PROTOCOL_41 |
        CLIENT_MULTI_RESULTS);
-  std::string endpoint_str =
-      !unix_socket.empty() ? unix_socket : host + ":" + std::to_string(port);
-
-  const bool ssl_disabled = ssl_mode() == SSL_MODE_DISABLED;
-  auto &ssl_sessions_cache = SSLSessionsCache::instance();
-
-  if (!ssl_disabled) {
-    ssl_sessions_cache.try_reuse_session(connection_, endpoint_str);
-  }
-
-  if (!mysql_real_connect(
-          connection_, !unix_socket.empty() ? nullptr : host.c_str(),
-          username.c_str(), password.c_str(), default_schema.c_str(), port,
-          unix_socket.c_str(), client_flags)) {
+  std::string tmp_conn_addr = unix_socket.length() > 0
+                                  ? unix_socket
+                                  : host + ":" + std::to_string(port);
+  if (!mysql_real_connect(connection_, host.c_str(), username.c_str(),
+                          password.c_str(), default_schema.c_str(), port,
+                          unix_socket.c_str(), client_flags)) {
     std::stringstream ss;
-    ss << "Error connecting to MySQL server at " << endpoint_str;
+    ss << "Error connecting to MySQL server at " << tmp_conn_addr;
     ss << ": " << mysql_error(connection_) << " (" << mysql_errno(connection_)
        << ")";
-    throw Error(ss.str(), mysql_errno(connection_));
+    throw Error(ss.str().c_str(), mysql_errno(connection_));
   }
-
-  if (!ssl_disabled) {
-    ssl_sessions_cache.store_ssl_session(connection_, endpoint_str);
-  }
-
   connected_ = true;
-  connection_address_ = endpoint_str;
+  connection_address_ = tmp_conn_addr;
 
-  // save the information about the endpoint we connected to
-  connect_params_ = {host, port, unix_socket, default_schema};
+  // archive options for future connection templating
+  conn_params_.conn_opts = {
+      host,        port,           username,        password,
+      unix_socket, default_schema, connect_timeout, read_timeout};
 }
 
-void MySQLSession::connect(const MySQLSession &other,
-                           const std::string &username,
-                           const std::string &password) {
+void MySQLSession::connect_and_set_opts(
+    const ConnectionParameters &conn_params) {
+  // should only be used on fresh objects
+  // assert(!connected_);
+
   // below methods can throw:
   //   MySQLSession::Error (std::runtime_error)
   //   std::invalid_argument (std::logic_error)
 
-  set_ssl_options(other.ssl_mode(), other.tls_version(), other.ssl_cipher(),
-                  other.ssl_ca(), other.ssl_capath(), other.ssl_crl(),
-                  other.ssl_crlpath());
+  set_ssl_options(conn_params.ssl_opts.ssl_mode,
+                  conn_params.ssl_opts.tls_version,
+                  conn_params.ssl_opts.ssl_cipher, conn_params.ssl_opts.ca,
+                  conn_params.ssl_opts.capath, conn_params.ssl_opts.crl,
+                  conn_params.ssl_opts.crlpath);
 
-  if (!other.ssl_cert().empty() || !other.ssl_key().empty()) {
-    set_ssl_cert(other.ssl_cert(), other.ssl_key());
-  }
+  if (!conn_params.ssl_cert.cert.empty() || !conn_params.ssl_cert.key.empty())
+    set_ssl_cert(conn_params.ssl_cert.cert, conn_params.ssl_cert.key);
 
-  connect(other.connect_params_.host, other.connect_params_.port, username,
-          password, other.connect_params_.unix_socket,
-          other.connect_params_.unix_socket, other.connect_timeout(),
-          other.read_timeout());
+  connect(conn_params.conn_opts.host, conn_params.conn_opts.port,
+          conn_params.conn_opts.username, conn_params.conn_opts.password,
+          conn_params.conn_opts.unix_socket,
+          conn_params.conn_opts.default_schema,
+          conn_params.conn_opts.connect_timeout,
+          conn_params.conn_opts.read_timeout);
 }
 
 void MySQLSession::disconnect() {
@@ -452,13 +300,14 @@ static MysqlError make_mysql_error_code(MYSQL *m) {
 stdx::expected<MySQLSession::mysql_result_type, MysqlError>
 MySQLSession::real_query(const std::string &q) {
   if (!connected_) {
-    return stdx::unexpected(make_mysql_error_code(CR_COMMANDS_OUT_OF_SYNC));
+    return stdx::make_unexpected(
+        make_mysql_error_code(CR_COMMANDS_OUT_OF_SYNC));
   }
 
   auto query_res = mysql_real_query(connection_, q.data(), q.size());
 
   if (query_res != 0) {
-    return stdx::unexpected(make_mysql_error_code(connection_));
+    return stdx::make_unexpected(make_mysql_error_code(connection_));
   }
 
   mysql_result_type res{mysql_store_result(connection_)};
@@ -466,37 +315,23 @@ MySQLSession::real_query(const std::string &q) {
     // no error, but also no resultset
     if (mysql_errno(connection_) == 0) return {};
 
-    return stdx::unexpected(make_mysql_error_code(connection_));
+    return stdx::make_unexpected(make_mysql_error_code(connection_));
   }
 
+#if defined(__SUNPRO_CC)
+  // ensure sun-cc doesn't try to the copy-constructor on a move-only type
+  return std::move(res);
+#else
   return res;
+#endif
 }
 
 stdx::expected<MySQLSession::mysql_result_type, MysqlError>
 MySQLSession::logged_real_query(const std::string &q) {
-  using clock_type = std::chrono::steady_clock;
-
-  auto start = clock_type::now();
+  logging_strategy_->log("Executing query: "s + log_filter_.filter(q));
   auto query_res = real_query(q);
-  auto dur = clock_type::now() - start;
-  auto msg =
-      get_address() + " (" +
-      std::to_string(
-          std::chrono::duration_cast<std::chrono::microseconds>(dur).count()) +
-      " us)> " + log_filter_.filter(q);
-  if (query_res) {
-    auto const *res = query_res.value().get();
 
-    msg += " // OK";
-    if (res) {
-      msg += " " + std::to_string(res->row_count) + " row" +
-             (res->row_count != 1 ? "s" : "");
-    }
-  } else {
-    auto err = query_res.error();
-    msg += " // ERROR: " + std::to_string(err.value()) + " " + err.message();
-  }
-  logging_strategy_->log(msg);
+  logging_strategy_->log("Done executing query");
 
   return query_res;
 }
@@ -517,10 +352,9 @@ void MySQLSession::execute(const std::string &q) {
 }
 
 /*
-  Execute query on the session and iterate the results with the given
-  callback.
+  Execute query on the session and iterate the results with the given callback.
 
-  The processor callback is called with a vector of strings, which contain the
+  The processor callback is called with a vector of strings, which conain the
   values of each field of a row. It is called once per row.
   If the processor returns false, the result row iteration stops.
  */
@@ -621,7 +455,7 @@ unsigned MySQLSession::warning_count() noexcept {
   return mysql_warning_count(connection_);
 }
 
-std::string MySQLSession::quote(const std::string &s, char qchar) const {
+std::string MySQLSession::quote(const std::string &s, char qchar) noexcept {
   std::string r;
   r.resize(s.length() * 2 + 3);
   r[0] = qchar;
@@ -644,14 +478,6 @@ const char *MySQLSession::ssl_cipher() {
   return connection_ ? mysql_get_ssl_cipher(connection_) : nullptr;
 }
 
-bool MySQLSession::is_ssl_session_reused() {
-  return connection_ ? mysql_get_ssl_session_reused(connection_) : false;
-}
-
-unsigned long MySQLSession::server_version() {
-  return connection_ ? mysql_get_server_version(connection_) : 0;
-}
-
 void MySQLSession::LoggingStrategyDebugLogger::log(const std::string &msg) {
   log_debug("%s", msg.c_str());
 }
