@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,26 +33,24 @@
 #include "mysql_routing.h"
 #include "mysqlrouter/datatypes.h"
 #include "mysqlrouter/metadata_cache.h"
-#include "mysqlrouter/routing_export.h"
 #include "mysqlrouter/uri.h"
 #include "tcp_address.h"
 
 class DestMetadataCacheGroup final
     : public RouteDestination,
-      public metadata_cache::ClusterStateListenerInterface,
-      public metadata_cache::MetadataRefreshListenerInterface,
-      public metadata_cache::AcceptorUpdateHandlerInterface {
+      public metadata_cache::ReplicasetStateListenerInterface {
  public:
   enum ServerRole { Primary, Secondary, PrimaryAndSecondary };
 
   /** @brief Constructor */
-  DestMetadataCacheGroup(net::io_context &io_ctx_,
-                         const std::string &metadata_cache,
-                         const routing::RoutingStrategy routing_strategy,
-                         const mysqlrouter::URIQuery &query,
-                         const Protocol::Type protocol,
-                         metadata_cache::MetadataCacheAPIBase *cache_api =
-                             metadata_cache::MetadataCacheAPI::instance());
+  DestMetadataCacheGroup(
+      net::io_context &io_ctx_, const std::string &metadata_cache,
+      const std::string &replicaset,
+      const routing::RoutingStrategy routing_strategy,
+      const mysqlrouter::URIQuery &query, const Protocol::Type protocol,
+      const routing::AccessMode access_mode = routing::AccessMode::kUndefined,
+      metadata_cache::MetadataCacheAPIBase *cache_api =
+          metadata_cache::MetadataCacheAPI::instance());
 
   /** @brief Copy constructor */
   DestMetadataCacheGroup(const DestMetadataCacheGroup &other) = delete;
@@ -101,21 +98,12 @@ class DestMetadataCacheGroup final
   // get cache-api
   metadata_cache::MetadataCacheAPIBase *cache_api() { return cache_api_; }
 
-  std::optional<Destinations> refresh_destinations(
+  stdx::expected<Destinations, void> refresh_destinations(
       const Destinations &dests) override;
 
   Destinations primary_destinations();
 
-  /**
-   * advance the current position in the destination by n.
-   */
-  void advance(size_t n);
-
-  void handle_sockets_acceptors() override {
-    cache_api()->handle_sockets_acceptors_on_md_refresh();
-  }
-
-  routing::RoutingStrategy get_strategy() override { return routing_strategy_; }
+  void advance(size_t n) { start_pos_ += n; }
 
  private:
   /** @brief The Metadata Cache to use
@@ -131,6 +119,9 @@ class DestMetadataCacheGroup final
    */
   const std::string cache_name_;
 
+  /** @brief The HA Group which will be used for looking up managed servers */
+  const std::string ha_replicaset_;
+
   /** @brief Query part of the URI given as destination in the configuration
    *
    * For example, given following Metadata Cache configuration:
@@ -138,9 +129,9 @@ class DestMetadataCacheGroup final
    *     [routing:metadata_read_only]
    *     ..
    *     destination =
-   * metadata_cache:///cluster_name/replicaset_name?role=PRIMARY_AND_SECONDARY
+   * metadata_cache:///cluster_name/replicaset_name?allow_primary_reads=yes
    *
-   * The 'role' is part of uri_query_.
+   * The 'allow_primary_reads' is part of uri_query_.
    */
   const mysqlrouter::URIQuery uri_query_;
 
@@ -151,29 +142,39 @@ class DestMetadataCacheGroup final
    */
   void init();
 
+  struct AvailableDestination {
+    AvailableDestination(mysql_harness::TCPAddress a, std::string i)
+        : address{std::move(a)}, id{std::move(i)} {}
+
+    mysql_harness::TCPAddress address;
+    std::string id;
+  };
+
+  using AvailableDestinations = std::vector<AvailableDestination>;
+
   /** @brief Gets available destinations from Metadata Cache
    *
    * This method gets the destinations using Metadata Cache information. It uses
-   * the `metadata_cache::get_cluster_nodes()` function to get a list of current
+   * the `metadata_cache::lookup_replicaset()` function to get a list of current
    * managed servers. Bool in the returned pair indicates if (in case of the
    * round-robin-with-fallback routing strategy) the returned nodes are the
-   * primaries after the fallback (true), regular primaries (false) or
-   * secondaries (false).
+   * primaries after the fallback (true) or secondaries (false).
    *
    */
-  std::pair<metadata_cache::cluster_nodes_list_t, bool> get_available(
-      const metadata_cache::cluster_nodes_list_t &instances,
+  std::pair<AvailableDestinations, bool> get_available(
+      const metadata_cache::LookupResult &managed_servers,
       bool for_new_connections = true) const;
 
-  metadata_cache::cluster_nodes_list_t get_available_primaries(
-      const metadata_cache::cluster_nodes_list_t &managed_servers) const;
+  AvailableDestinations get_available_primaries(
+      const metadata_cache::LookupResult &managed_servers) const;
 
-  Destinations balance(
-      const metadata_cache::cluster_nodes_list_t &all_replicaset_nodes,
-      bool primary_fallback);
+  Destinations balance(const AvailableDestinations &all_replicaset_nodes,
+                       bool primary_fallback);
 
   routing::RoutingStrategy routing_strategy_;
 
+  routing::AccessMode access_mode_;
+
   ServerRole server_role_;
 
   metadata_cache::MetadataCacheAPIBase *cache_api_;
@@ -183,32 +184,16 @@ class DestMetadataCacheGroup final
   bool disconnect_on_promoted_to_primary_{false};
   bool disconnect_on_metadata_unavailable_{false};
 
-  void on_instances_change(
-      const metadata_cache::ClusterTopology &cluster_topology,
-      const bool md_servers_reachable);
+  void on_instances_change(const metadata_cache::LookupResult &instances,
+                           const bool md_servers_reachable);
   void subscribe_for_metadata_cache_changes();
-  void subscribe_for_acceptor_handler();
-  void subscribe_for_md_refresh_handler();
 
-  void notify_instances_changed(
-      const metadata_cache::ClusterTopology &cluster_topology,
-      const bool md_servers_reachable,
-      const uint64_t /*view_id*/) noexcept override;
-
-  bool update_socket_acceptor_state(
-      const metadata_cache::cluster_nodes_list_t &instances) noexcept override;
-
-  void on_md_refresh(
-      const bool instances_changed,
-      const metadata_cache::ClusterTopology &cluster_topology) override;
+  void notify(const metadata_cache::LookupResult &instances,
+              const bool md_servers_reachable,
+              const unsigned /*view_id*/) noexcept override;
 
   // MUST take the RouteDestination Mutex
   size_t start_pos_{};
-  size_t ro_start_pos_{};
-  size_t rw_start_pos_{};
 };
 
-ROUTING_EXPORT DestMetadataCacheGroup::ServerRole get_server_role_from_uri(
-    const mysqlrouter::URIQuery &uri);
-
 #endif  // ROUTING_DEST_METADATA_CACHE_INCLUDED
