@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,9 +32,10 @@
 #endif
 
 #include <gmock/gmock.h>
-#include <gtest/gtest-matchers.h>
 #include <gtest/gtest.h>
 
+#include "common.h"
+
 #include "mysql/harness/config_parser.h"
 #include "mysql/harness/filesystem.h"  // Path
 #include "mysql/harness/logging/logging.h"
@@ -86,6 +86,7 @@ class RoutingPluginTests : public ConsoleOutputTest {
         "127.0.0.1:" + std::to_string(tcp_port_pool_.get_next_available());
     socket = rundir + "/unix_socket";
     routing_strategy = "round-robin";
+    mode = "read-only";
     connect_timeout = "1";
     client_connect_timeout = "9";
     max_connect_errors = "100";
@@ -116,6 +117,7 @@ class RoutingPluginTests : public ConsoleOutputTest {
           {"socket", std::ref(socket)},
           {"destinations", std::ref(destinations)},
           {"routing_strategy", std::ref(routing_strategy)},
+          {"mode", std::ref(mode)},
           {"connect_timeout", std::ref(connect_timeout)},
           {"client_connect_timeout", std::ref(client_connect_timeout)},
           {"max_connect_errors", std::ref(max_connect_errors)},
@@ -159,6 +161,7 @@ class RoutingPluginTests : public ConsoleOutputTest {
   string destinations;
   string socket;
   string routing_strategy;
+  string mode;
   string connect_timeout;
   string client_connect_timeout;
   string max_connect_errors;
@@ -202,7 +205,7 @@ TEST_F(RoutingPluginTests, ListeningTcpSocket) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section = cfg.add("routing", "test_route");
   section.add("destinations", "localhost:1234");
-  section.add("routing_strategy", "round-robin");
+  section.add("mode", "read-only");
   section.add("bind_address", "127.0.0.1:15508");
 
   EXPECT_NO_THROW({
@@ -216,7 +219,7 @@ TEST_F(RoutingPluginTests, ListeningUnixSocket) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section = cfg.add("routing", "test_route");
   section.add("destinations", "localhost:1234");
-  section.add("routing_strategy", "round-robin");
+  section.add("mode", "read-only");
   section.add("socket", "./socket");  // if this test fails, check if you don't
                                       // have this file hanging around
 
@@ -230,7 +233,7 @@ TEST_F(RoutingPluginTests, ListeningBothSockets) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section = cfg.add("routing", "test_route");
   section.add("destinations", "localhost:1234");
-  section.add("routing_strategy", "round-robin");
+  section.add("mode", "read-only");
   section.add("bind_address", "127.0.0.1:15508");
   section.add("socket", "./socket");  // if this test fails, check if you don't
                                       // have this file hanging around
@@ -245,11 +248,11 @@ TEST_F(RoutingPluginTests, TwoUnixSocketsWithoutTcp) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section1 = cfg.add("routing", "test_route1");
   section1.add("destinations", "localhost:1234");
-  section1.add("routing_strategy", "round-robin");
+  section1.add("mode", "read-only");
   section1.add("socket", "./socket1");
   mysql_harness::ConfigSection &section2 = cfg.add("routing", "test_route2");
   section2.add("destinations", "localhost:1234");
-  section2.add("routing_strategy", "round-robin");
+  section2.add("mode", "read-only");
   section2.add("socket", "./socket2");
 
   mysql_harness::AppInfo info;
@@ -265,12 +268,12 @@ TEST_F(RoutingPluginTests, TwoUnixSocketsWithTcp) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section1 = cfg.add("routing", "test_route1");
   section1.add("destinations", "localhost:1234");
-  section1.add("routing_strategy", "round-robin");
+  section1.add("mode", "read-only");
   section1.add("bind_address", "127.0.0.1:15501");
   section1.add("socket", "./socket1");
   mysql_harness::ConfigSection &section2 = cfg.add("routing", "test_route2");
   section2.add("destinations", "localhost:1234");
-  section2.add("routing_strategy", "round-robin");
+  section2.add("mode", "read-only");
   section2.add("bind_address", "127.0.0.1:15502");
   section2.add("socket", "./socket2");
 
@@ -296,7 +299,7 @@ static void test_socket_length(const std::string &socket_name, size_t max_len) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section = cfg.add("routing", "test_route");
   section.add("destinations", "localhost:1234");
-  section.add("routing_strategy", "round-robin");
+  section.add("mode", "read-only");
   section.add("socket", socket_name);
 
   if (socket_name.length() <= max_len) {
@@ -341,11 +344,11 @@ TEST_F(RoutingPluginTests, TwoNonuniqueTcpSockets) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section1 = cfg.add("routing", "test_route1");
   section1.add("destinations", "localhost:1234");
-  section1.add("routing_strategy", "round-robin");
+  section1.add("mode", "read-only");
   section1.add("bind_address", "127.0.0.1:15508");
   mysql_harness::ConfigSection &section2 = cfg.add("routing", "test_route2");
   section2.add("destinations", "localhost:1234");
-  section2.add("routing_strategy", "round-robin");
+  section2.add("mode", "read-only");
   section2.add("bind_address", "127.0.0.1:15508");
 
   mysql_harness::AppInfo info;
@@ -378,7 +381,7 @@ TEST_F(RoutingPluginTests, EmptyUnixSocket) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section = cfg.add("routing", "test_route");
   section.add("destinations", "localhost:1234");
-  section.add("routing_strategy", "round-robin");
+  section.add("mode", "read-only");
   section.add("socket", "");
 
   // If this not provided, RoutingPluginConfig() will throw with its own error,
@@ -402,7 +405,7 @@ TEST_F(RoutingPluginTests, ListeningHostIsInvalid) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section = cfg.add("routing", "test_route");
   section.add("destinations", "localhost:1234");
-  section.add("routing_strategy", "round-robin");
+  section.add("mode", "read-only");
   section.add("bind_address", "host.that.does.not..exist:15508");
 
   try {
@@ -426,7 +429,7 @@ TEST_F(RoutingPluginTests, Ipv6LinkLocal) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section = cfg.add("routing", "test_route");
   section.add("destinations", "[fe80::3617:ebff:fecb:587e%3]:3306");
-  section.add("routing_strategy", "round-robin");
+  section.add("mode", "read-only");
   section.add("bind_port", "6446");
 
   try {
@@ -441,7 +444,7 @@ TEST_F(RoutingPluginTests, InvalidIpv6) {
   mysql_harness::Config cfg;
   mysql_harness::ConfigSection &section = cfg.add("routing", "test_route");
   section.add("destinations", "[fe80::3617:ebff:fecb:587e@3]:3306");
-  section.add("routing_strategy", "round-robin");
+  section.add("mode", "read-only");
   section.add("bind_port", "6446");
 
   try {
@@ -476,7 +479,7 @@ class RoutingConfigTest
 
     mysql_harness::ConfigSection &section = cfg_.add("routing", "test_route");
     section.add("destinations", "127.0.0.1:3306");
-    section.add("routing_strategy", "round-robin");
+    section.add("mode", "read-only");
     section.add("bind_port", "6446");
   }
   mysql_harness::Config cfg_{mysql_harness::Config::allow_keys};
@@ -1130,7 +1133,7 @@ class RoutingConfigFailTest
 
     mysql_harness::ConfigSection &section = cfg_.add("routing", "test_route");
     section.add("destinations", "127.0.0.1:3306");
-    section.add("routing_strategy", "round-robin");
+    section.add("mode", "read-only");
     section.add("bind_port", "6446");
   }
 
@@ -1168,7 +1171,7 @@ TEST_P(RoutingConfigFailTest, section_option) {
   }
 }
 
-const RoutingConfigFailParam routing_config_fail_params[] = {
+RoutingConfigFailParam routing_config_fail_params[] = {
     // server-ssl-mode
     //
     {"server_ssl_mode_unknown",  // RT1_ARGS_BAD_04
@@ -1176,33 +1179,27 @@ const RoutingConfigFailParam routing_config_fail_params[] = {
          {"server_ssl_mode", "unknown"},
      },
      [](const std::exception &e) {
-       ASSERT_THAT(e.what(),
-                   ::testing::ContainsRegex(
-                       "invalid value 'unknown' for option server_ssl_mode in "
-                       "\\[.+\\]\\. Allowed are: "
-                       "DISABLED,PREFERRED,REQUIRED,AS_CLIENT\\."));
+       ASSERT_STREQ(e.what(),
+                    "invalid value 'unknown' for server_ssl_mode. Allowed are: "
+                    "DISABLED,PREFERRED,REQUIRED,AS_CLIENT.");
      }},
     {"server_ssl_mode_quotes",  // RT1_ARGS_BAD_05
      {
          {"server_ssl_mode", "''"},
      },
      [](const std::exception &e) {
-       ASSERT_THAT(e.what(),
-                   ::testing::ContainsRegex(
-                       "invalid value '''' for option server_ssl_mode in "
-                       "\\[.+\\]\\. Allowed are: "
-                       "DISABLED,PREFERRED,REQUIRED,AS_CLIENT\\."));
+       ASSERT_STREQ(e.what(),
+                    "invalid value '''' for server_ssl_mode. Allowed are: "
+                    "DISABLED,PREFERRED,REQUIRED,AS_CLIENT.");
      }},
     {"server_ssl_mode_quotes_space",  // RT1_ARGS_BAD_06
      {
          {"server_ssl_mode", "' '"},
      },
      [](const std::exception &e) {
-       ASSERT_THAT(e.what(),
-                   ::testing::ContainsRegex(
-                       "invalid value '' '' for option server_ssl_mode in "
-                       "\\[.+\\]\\. Allowed are: "
-                       "DISABLED,PREFERRED,REQUIRED,AS_CLIENT\\."));
+       ASSERT_STREQ(e.what(),
+                    "invalid value '' '' for server_ssl_mode. Allowed are: "
+                    "DISABLED,PREFERRED,REQUIRED,AS_CLIENT.");
      }},
     {"server_ssl_mode_disabled",  // RT1_MODES_03, RT1_MODES_CERT_KEY_05
      {
@@ -1340,33 +1337,27 @@ const RoutingConfigFailParam routing_config_fail_params[] = {
          {"client_ssl_mode", "unknown"},
      },
      [](const std::exception &e) {
-       ASSERT_THAT(e.what(),
-                   ::testing::ContainsRegex(
-                       "invalid value 'unknown' for option client_ssl_mode in "
-                       "\\[.+\\]\\. Allowed are: "
-                       "DISABLED,PREFERRED,REQUIRED,PASSTHROUGH\\."));
+       ASSERT_STREQ(e.what(),
+                    "invalid value 'unknown' for client_ssl_mode. Allowed are: "
+                    "DISABLED,PREFERRED,REQUIRED,PASSTHROUGH.");
      }},
     {"client_ssl_mode_quotes",  // RT1_ARGS_BAD_02
      {
          {"client_ssl_mode", "''"},
      },
      [](const std::exception &e) {
-       ASSERT_THAT(e.what(),
-                   ::testing::ContainsRegex(
-                       "invalid value '''' for option client_ssl_mode in "
-                       "\\[.+\\]\\. Allowed are: "
-                       "DISABLED,PREFERRED,REQUIRED,PASSTHROUGH\\."));
+       ASSERT_STREQ(e.what(),
+                    "invalid value '''' for client_ssl_mode. Allowed are: "
+                    "DISABLED,PREFERRED,REQUIRED,PASSTHROUGH.");
      }},
     {"client_ssl_mode_quotes_space",  // RT1_ARGS_BAD_03
      {
          {"client_ssl_mode", "' '"},
      },
      [](const std::exception &e) {
-       ASSERT_THAT(e.what(),
-                   ::testing::ContainsRegex(
-                       "invalid value '' '' for option client_ssl_mode in "
-                       "\\[.+\\]\\. Allowed are: "
-                       "DISABLED,PREFERRED,REQUIRED,PASSTHROUGH\\."));
+       ASSERT_STREQ(e.what(),
+                    "invalid value '' '' for client_ssl_mode. Allowed are: "
+                    "DISABLED,PREFERRED,REQUIRED,PASSTHROUGH.");
      }},
     {"client_ssl_mode_preferred_missing_cert",
      {
@@ -1413,34 +1404,28 @@ const RoutingConfigFailParam routing_config_fail_params[] = {
          {"server_ssl_verify", "unknown"},
      },
      [](const std::exception &e) {
-       ASSERT_THAT(
+       ASSERT_STREQ(
            e.what(),
-           ::testing::ContainsRegex(
-               "invalid value 'unknown' for option server_ssl_verify in "
-               "\\[.+\\]\\. Allowed are: "
-               "DISABLED,VERIFY_CA,VERIFY_IDENTITY\\."));
+           "invalid value 'unknown' for server_ssl_verify. Allowed are: "
+           "DISABLED,VERIFY_CA,VERIFY_IDENTITY.");
      }},
     {"server_ssl_verify_quotes",  // RT1_ARGS_BAD_08
      {
          {"server_ssl_verify", "''"},
      },
      [](const std::exception &e) {
-       ASSERT_THAT(e.what(),
-                   ::testing::ContainsRegex(
-                       "invalid value '''' for option server_ssl_verify in "
-                       "\\[.+\\]\\. Allowed are: "
-                       "DISABLED,VERIFY_CA,VERIFY_IDENTITY\\."));
+       ASSERT_STREQ(e.what(),
+                    "invalid value '''' for server_ssl_verify. Allowed are: "
+                    "DISABLED,VERIFY_CA,VERIFY_IDENTITY.");
      }},
     {"server_ssl_verify_quotes_space",  // RT1_ARGS_BAD_09
      {
          {"server_ssl_verify", "' '"},
      },
      [](const std::exception &e) {
-       ASSERT_THAT(e.what(),
-                   ::testing::ContainsRegex(
-                       "invalid value '' '' for option server_ssl_verify in "
-                       "\\[.+\\]\\. Allowed are: "
-                       "DISABLED,VERIFY_CA,VERIFY_IDENTITY\\."));
+       ASSERT_STREQ(e.what(),
+                    "invalid value '' '' for server_ssl_verify. Allowed are: "
+                    "DISABLED,VERIFY_CA,VERIFY_IDENTITY.");
      }},
     {"server_ssl_verify_verify_ca_missing_ca",
      {
