@@ -7,6 +7,7 @@
  * via HTTP interface
  *
  * - MD_asked
+ * - primary_asked
  * - health_asked
  */
 
@@ -15,17 +16,18 @@ var gr_memberships = require("gr_memberships");
 
 var gr_node_host = "127.0.0.1";
 
-var group_replication_members_online =
-    gr_memberships.gr_members(gr_node_host, mysqld.global.gr_nodes);
+var group_replication_membership_online =
+    gr_memberships.nodes(gr_node_host, mysqld.global.gr_nodes);
 
 var options = {
-  group_replication_members: group_replication_members_online,
-  innodb_cluster_instances: gr_memberships.cluster_nodes(
-      mysqld.global.gr_node_host, mysqld.global.cluster_nodes),
+  group_replication_membership: group_replication_membership_online,
   cluster_type: "gr",
-  gr_id: mysqld.global.gr_id,
 };
 
+// first node is PRIMARY
+options.group_replication_primary_member =
+    options.group_replication_membership[0][0];
+
 // prepare the responses for common statements
 var common_responses = common_stmts.prepare_statement_responses(
     [
@@ -36,10 +38,6 @@ var common_responses = common_stmts.prepare_statement_responses(
       "router_commit",
       "router_select_schema_version",
       "router_select_cluster_type_v2",
-      "router_clusterset_present",
-      "router_check_member_state",
-      "router_select_members_count",
-      "router_select_router_options_view",
     ],
     options);
 
@@ -47,14 +45,20 @@ var common_responses = common_stmts.prepare_statement_responses(
 // if they have been executed ("asked")
 var router_select_metadata =
     common_stmts.get("router_select_metadata_v2_gr", options);
-var router_select_group_membership =
-    common_stmts.get("router_select_group_membership", options);
+var router_select_group_replication_primary_member =
+    common_stmts.get("router_select_group_replication_primary_member", options);
+var router_select_group_membership_with_primary_mode = common_stmts.get(
+    "router_select_group_membership_with_primary_mode", options);
 
 
 if (mysqld.global.MD_asked === undefined) {
   mysqld.global.MD_asked = false;
 }
 
+if (mysqld.global.primary_asked === undefined) {
+  mysqld.global.primary_asked = false;
+}
+
 if (mysqld.global.health_asked === undefined) {
   mysqld.global.health_asked = false;
 }
@@ -66,9 +70,12 @@ if (mysqld.global.health_asked === undefined) {
     } else if (stmt === router_select_metadata.stmt) {
       mysqld.global.MD_asked = true;
       return router_select_metadata;
-    } else if (stmt === router_select_group_membership.stmt) {
+    } else if (stmt === router_select_group_replication_primary_member.stmt) {
+      mysqld.global.primary_asked = true;
+      return router_select_group_replication_primary_member;
+    } else if (stmt === router_select_group_membership_with_primary_mode.stmt) {
       mysqld.global.health_asked = true;
-      return router_select_group_membership;
+      return router_select_group_membership_with_primary_mode;
     } else {
       return common_stmts.unknown_statement_response(stmt);
     }
