@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -30,10 +30,10 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
+#include "cluster_metadata.h"
 #include "config_builder.h"
 #include "mysql/harness/filesystem.h"
 #include "mysql/harness/string_utils.h"  // join
-#include "mysqlrouter/cluster_metadata.h"
 #include "router_component_test.h"
 #include "router_test_helpers.h"  // init_windows_socket()
 
@@ -68,7 +68,7 @@ TEST_P(BootstrapTlsEndpointFail, check) {
   EXPECT_NO_THROW(router.wait_for_exit());
 
   if (!GetParam().expected_result) {
-    // check if the bootstrapping was successful
+    // check if the bootstraping was successful
     EXPECT_THAT(router.get_full_output(),
                 ::testing::HasSubstr(GetParam().expected_result.error()));
   }
@@ -244,7 +244,7 @@ TEST_P(BootstrapTlsEndpointWithoutBootstrapFail, check) {
   EXPECT_NO_THROW(router.wait_for_exit());
 
   if (!GetParam().expected_result) {
-    // check if the bootstrapping was successful
+    // check if the bootstraping was successful
     EXPECT_THAT(router.get_full_output(),
                 ::testing::HasSubstr(GetParam().expected_result.error()));
   }
@@ -351,7 +351,7 @@ TEST_P(BootstrapTlsEndpointWithoutCertGeneration, succeeds) {
                  get_data_dir().join("bootstrap_gr.js").str()},
       },
       mysqlrouter::ClusterType::GR_V2, {}, EXIT_SUCCESS,
-      {"# MySQL Router configured"}, 5s, {2, 0, 3}, cmdline_args));
+      {"# MySQL Router configured"}, 1000ms, {2, 0, 3}, cmdline_args));
 
   ASSERT_NE(config_file.size(), 0);
 
@@ -376,7 +376,7 @@ const BootstrapTlsEndpointParams
                config_file_lines,
                ::testing::IsSupersetOf({
                    "client_ssl_mode=PREFERRED",
-                   "server_ssl_mode=PREFERRED",
+                   "server_ssl_mode=AS_CLIENT",
                    "client_ssl_cert=" SSL_TEST_DATA_DIR "server-cert.pem",
                    "client_ssl_key=" SSL_TEST_DATA_DIR "server-key.pem",
                    "server_ssl_verify=DISABLED",
@@ -401,7 +401,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({"client_ssl_mode=DISABLED",
-                                                "server_ssl_mode=PREFERRED"}));
+                                                "server_ssl_mode=AS_CLIENT"}));
          }},
 
         {"client_ssl_mode_disabled_disabled",  // BS_MODES_07
@@ -438,7 +438,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({"client_ssl_mode=PREFERRED",
-                                                "server_ssl_mode=PREFERRED"}));
+                                                "server_ssl_mode=AS_CLIENT"}));
          }},
         {"client_ssl_mode_preferred_disabled",  // BS_MODES_12
          {"--client-ssl-mode", "Preferred", "--server-ssl-mode", "Disabled"},
@@ -473,7 +473,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({"client_ssl_mode=REQUIRED",
-                                                "server_ssl_mode=PREFERRED"}));
+                                                "server_ssl_mode=AS_CLIENT"}));
          }},
         {"client_ssl_mode_required_disabled",  // BS_MODES_17
          {"--client-ssl-mode", "ReQuired", "--server-ssl-mode", "Disabled"},
@@ -552,7 +552,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines, ::testing::IsSupersetOf({
                                               "client_ssl_mode=PREFERRED",
-                                              "server_ssl_mode=PREFERRED",
+                                              "server_ssl_mode=AS_CLIENT",
                                               "server_ssl_verify=DISABLED",
                                           }));
          }},
@@ -562,7 +562,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=PREFERRED",
+                           "server_ssl_mode=AS_CLIENT",
                            "server_ssl_verify=VERIFY_IDENTITY",
                        }));
          }},
@@ -571,7 +571,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines, ::testing::IsSupersetOf({
                                               "client_ssl_mode=PREFERRED",
-                                              "server_ssl_mode=PREFERRED",
+                                              "server_ssl_mode=AS_CLIENT",
                                               "server_ssl_verify=VERIFY_CA",
                                           }));
          }},
@@ -581,7 +581,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=PREFERRED",
+                           "server_ssl_mode=AS_CLIENT",
                            "client_ssl_cipher=some-cipher-string",
                        }));
          }},
@@ -591,7 +591,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=PREFERRED",
+                           "server_ssl_mode=AS_CLIENT",
                            "client_ssl_curves=some-curves-string",
                        }));
          }},
@@ -601,7 +601,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=PREFERRED",
+                           "server_ssl_mode=AS_CLIENT",
                            "client_ssl_dh_params=some-dh-param-string",
                        }));
          }},
@@ -611,7 +611,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=PREFERRED",
+                           "server_ssl_mode=AS_CLIENT",
                            "server_ssl_cipher=some-cipher-string",
                        }));
          }},
@@ -621,7 +621,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=PREFERRED",
+                           "server_ssl_mode=AS_CLIENT",
                            "server_ssl_curves=some-curves-string",
                        }));
          }},
@@ -630,7 +630,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines, ::testing::IsSupersetOf({
                                               "client_ssl_mode=PREFERRED",
-                                              "server_ssl_mode=PREFERRED",
+                                              "server_ssl_mode=AS_CLIENT",
                                               "server_ssl_ca=some-ca-string",
                                           }));
          }},
@@ -640,7 +640,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=PREFERRED",
+                           "server_ssl_mode=AS_CLIENT",
                            "server_ssl_capath=some-capath-string",
                        }));
          }},
@@ -649,7 +649,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines, ::testing::IsSupersetOf({
                                               "client_ssl_mode=PREFERRED",
-                                              "server_ssl_mode=PREFERRED",
+                                              "server_ssl_mode=AS_CLIENT",
                                               "server_ssl_crl=some-crl-string",
                                           }));
          }},
@@ -659,7 +659,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=PREFERRED",
+                           "server_ssl_mode=AS_CLIENT",
                            "server_ssl_crlpath=some-crlpath-string",
                        }));
          }},
@@ -720,7 +720,7 @@ TEST_P(BootstrapTlsEndpoint, existing_config) {
               {"destinations", "127.0.0.1:" + std::to_string(server_port)},
               {"routing_strategy", "round-robin"},
           })},
-      "");
+      "\n");
   SCOPED_TRACE("starting router with config:\n" + config);
   auto conf_file = create_config_file(bootstrap_dir.name(), config);
 
@@ -763,7 +763,7 @@ TEST_P(BootstrapTlsEndpoint, existing_config_with_client_ssl_cert) {
               {"destinations", "127.0.0.1:" + std::to_string(server_port)},
               {"routing_strategy", "round-robin"},
           })},
-      "");
+      "\n");
   SCOPED_TRACE("starting router with config:\n" + config);
   auto conf_file = create_config_file(
       bootstrap_dir.name(), config, nullptr, "mysqlrouter.conf",
@@ -772,7 +772,7 @@ TEST_P(BootstrapTlsEndpoint, existing_config_with_client_ssl_cert) {
               mysql_harness::ConfigBuilder::build_pair(
                   std::make_pair("client_ssl_cert", "foo")),
           },
-          ""));
+          "\n"));
 
   ASSERT_NO_FATAL_FAILURE(bootstrap_failover(
       {
@@ -863,7 +863,7 @@ TEST_P(BootstrapTlsEndpoint, existing_config_with_client_ssl_cert_and_key) {
               {"destinations", "127.0.0.1:" + std::to_string(server_port)},
               {"routing_strategy", "round-robin"},
           })},
-      "");
+      "\n");
   SCOPED_TRACE("starting router with config:\n" + config);
   auto conf_file = create_config_file(
       bootstrap_dir.name(), config, nullptr, "mysqlrouter.conf",
@@ -1055,12 +1055,12 @@ const BootstrapTlsEndpointParams bootstrap_tls_endpoint_params[] = {
                       // BS_CERT_KEY_CONFIG_PERSISTS_10
      {},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_generated(config_file_lines, "PREFERRED", "PREFERRED");
+       check_cert_generated(config_file_lines, "PREFERRED", "AS_CLIENT");
      }},
     {"client_ssl_mode_preferred_cert_gen",  // BS_MODES_02, BS_CERT_KEY_MODE_02
      {"--client-ssl-mode", "PREFERRED"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_generated(config_file_lines, "PREFERRED", "PREFERRED");
+       check_cert_generated(config_file_lines, "PREFERRED", "AS_CLIENT");
      }},
     {"client_ssl_mode_required_cert_gen",  // BS_MODES_03, BS_CERT_KEY_MODE_03,
                                            // BS_CERT_KEY_CONFIG_PERSISTS_02
@@ -1069,7 +1069,7 @@ const BootstrapTlsEndpointParams bootstrap_tls_endpoint_params[] = {
                                            // BS_CERT_KEY_CONFIG_PERSISTS_11
      {"--client-ssl-mode", "REQUIRED"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_generated(config_file_lines, "REQUIRED", "PREFERRED");
+       check_cert_generated(config_file_lines, "REQUIRED", "AS_CLIENT");
      }},
     {"client_ssl_mode_passthrough_no_cert_gen",  // BS_MODES_04,
                                                  // BS_CERT_KEY_MODE_04
@@ -1091,7 +1091,7 @@ const BootstrapTlsEndpointParams bootstrap_tls_endpoint_params[] = {
                                               // BS_CERT_KEY_MODE_05
      {"--client-ssl-mode", "DISABLED"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_no_cert_generated(config_file_lines, "DISABLED", "PREFERRED");
+       check_no_cert_generated(config_file_lines, "DISABLED", "AS_CLIENT");
 
        // not specified at command-line, no set in config
        ASSERT_THAT(config_file_lines,
@@ -1111,28 +1111,28 @@ const BootstrapTlsEndpointParams bootstrap_tls_endpoint_params[] = {
                                  // BS_CERT_KEY_ARGS_01
      {"--client-ssl-cert", "some-ssl-cert", "--client-ssl-key", "some-ssl-key"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_specified(config_file_lines, "PREFERRED", "PREFERRED",
+       check_cert_specified(config_file_lines, "PREFERRED", "AS_CLIENT",
                             "some-ssl-cert", "some-ssl-key");
      }},
     {"client_ssl_cert_key_and_mode_disabled",  // BS_CERT_KEY_ARGS_??
      {"--client-ssl-cert", "some-ssl-cert", "--client-ssl-key", "some-ssl-key",
       "--client-ssl-mode", "disabled"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_specified(config_file_lines, "DISABLED", "PREFERRED",
+       check_cert_specified(config_file_lines, "DISABLED", "AS_CLIENT",
                             "some-ssl-cert", "some-ssl-key");
      }},
     {"client_ssl_cert_key_and_mode_preferred",  // BS_CERT_KEY_ARGS_??
      {"--client-ssl-cert", "some-ssl-cert", "--client-ssl-key", "some-ssl-key",
       "--client-ssl-mode", "preferred"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_specified(config_file_lines, "PREFERRED", "PREFERRED",
+       check_cert_specified(config_file_lines, "PREFERRED", "AS_CLIENT",
                             "some-ssl-cert", "some-ssl-key");
      }},
     {"client_ssl_cert_key_and_mode_required",  // BS_CERT_KEY_ARGS_??
      {"--client-ssl-cert", "some-ssl-cert", "--client-ssl-key", "some-ssl-key",
       "--client-ssl-mode", "REQUIRED"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_specified(config_file_lines, "REQUIRED", "PREFERRED",
+       check_cert_specified(config_file_lines, "REQUIRED", "AS_CLIENT",
                             "some-ssl-cert", "some-ssl-key");
      }},
 
@@ -1165,7 +1165,8 @@ TEST_P(BootstrapTlsEndpointFailMock, fails) {
                  get_data_dir().join("bootstrap_gr.js").str()},
       },
       mysqlrouter::ClusterType::GR_V2, {}, EXIT_FAILURE,
-      GetParam().expected_output_lines, 1s, {2, 0, 3}, GetParam().cmdline_args);
+      GetParam().expected_output_lines, 1000ms, {2, 0, 3},
+      GetParam().cmdline_args);
 }
 
 const BootstrapTlsEndpointFailMockParams
