@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2018, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -258,15 +258,12 @@ TEST_P(HttpServerPlainTest, ensure) {
   std::string conf_file{create_config_file(
       conf_dir_.name(),
       mysql_harness::join(
-          std::vector<std::string>{
-              ConfigBuilder::build_section("http_server", http_section),
-              ConfigBuilder::build_section("logger",
-                                           {
-                                               {"level", "DEBUG"},
-                                           })},
+          std::vector<std::string>{mysql_harness::ConfigBuilder::build_section(
+              "http_server", http_section)},
           "\n"))};
   ProcessWrapper &http_server{launch_router(
-      {"-c", conf_file}, GetParam().expected_success ? 0 : EXIT_FAILURE)};
+      {"-c", conf_file}, GetParam().expected_success ? 0 : EXIT_FAILURE, true,
+      false, GetParam().expected_success ? 5s : -1s)};
 
   if (GetParam().expected_success) {
     std::string rel_uri = GetParam().raw_uri_path;
@@ -866,7 +863,7 @@ const HttpServerPlainParams http_server_static_files_unusable_params[]{
 
 };
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     Spec, HttpServerPlainTest,
     ::testing::ValuesIn(http_server_static_files_params),
     [](const ::testing::TestParamInfo<HttpServerPlainParams> &info) {
@@ -934,7 +931,7 @@ class HttpClientSecureTest
         ssl_cert_data_dir_{SSL_TEST_DATA_DIR},
         conf_file_{create_config_file(
             conf_dir_.name(),
-            ConfigBuilder::build_section(
+            mysql_harness::ConfigBuilder::build_section(
                 "http_server",
                 {
                     {"port", std::to_string(http_port_)},  // port to listen on
@@ -1147,7 +1144,7 @@ static const HttpClientSecureParams http_client_secure_params[]{
      "DES-CBC-SHA", "invalid cipher"},
 #endif
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     Spec, HttpClientSecureTest, ::testing::ValuesIn(http_client_secure_params),
     [](const ::testing::TestParamInfo<HttpClientSecureParams> &info) {
       return gtest_sanitize_param_name(
@@ -1201,8 +1198,7 @@ class HttpServerSecureTest
   mysql_harness::Path ssl_cert_data_dir_;
 };
 
-constexpr const char kErrmsgRegexWeakSslKey[]{
-    "keylength of RSA public-key of certificate"};
+constexpr const char kErrmsgRegexWeakSslKey[]{"key-size too small"};
 
 TEST_P(HttpServerSecureTest, ensure) {
   // const size_t placeholder_length = strlen(kPlaceholder);
@@ -1230,15 +1226,17 @@ TEST_P(HttpServerSecureTest, ensure) {
                     .str();
       }
     }
-    http_section.push_back({e.first, value});
+    http_section.emplace_back(e.first, value);
   }
 
   std::string conf_file{create_config_file(
       conf_dir_.name(),
-      ConfigBuilder::build_section("http_server", http_section))};
+      mysql_harness::ConfigBuilder::build_section("http_server", http_section),
+      nullptr, "mysqlrouter.conf", "", false)};
   ProcessWrapper &http_server{
       launch_router({"-c", conf_file},
-                    GetParam().expected_success ? EXIT_SUCCESS : EXIT_FAILURE)};
+                    GetParam().expected_success ? EXIT_SUCCESS : EXIT_FAILURE,
+                    true, false, GetParam().expected_success ? 5s : -1s)};
 
   if (GetParam().expected_success) {
     HttpUri u;
@@ -1323,7 +1321,7 @@ const HttpServerSecureParams http_server_secure_params[] {
            {"ssl_cert", "does-not-exist"},
        },
        false,
-       "using SSL certificate file 'does-not-exist' failed"},
+       "SSL certificate file 'does-not-exist' failed"},
 // This fails with OpenSSL 1.1.1 that added TLS1.3 default ciphers that we can't
 // disable
 #if (OPENSSL_VERSION_NUMBER < 0x10101000L)
@@ -1386,7 +1384,7 @@ const HttpServerSecureParams http_server_secure_params[] {
            {"ssl_dh_param", "does-not-exist"},
        },
        false,
-       "failed to open dh-param"},
+       "setting ssl_dh_params failed"},
       {"dh_param file is no PEM",
        "WL12524::TS_CR_08",
        {
@@ -1400,7 +1398,7 @@ const HttpServerSecureParams http_server_secure_params[] {
             kPlaceholderDatadir + std::string("/") + "my_port.js"},
        },
        false,
-       "failed to parse dh-param file"},
+       "setting ssl_dh_params failed"},
       {"dh ciphers, default dh-params",
        "WL12524::TS_CR_09",
        {
@@ -1470,7 +1468,7 @@ const HttpServerSecureParams http_server_secure_params[] {
        "key size of DH param"},
 };
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     Spec, HttpServerSecureTest, ::testing::ValuesIn(http_server_secure_params),
     [](const ::testing::TestParamInfo<HttpServerSecureParams> &info) {
       return gtest_sanitize_param_name(
@@ -1513,7 +1511,7 @@ const HttpServerSecureParams http_server_secure_openssl102_plus_params[]{
      "no-error"},
 };
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     Openssl102_plus, HttpServerSecureTest,
     ::testing::ValuesIn(http_server_secure_openssl102_plus_params),
     [](const ::testing::TestParamInfo<HttpServerSecureParams> &info) {
@@ -1562,22 +1560,24 @@ class HttpServerAuthTest
             conf_dir_.name(),
             mysql_harness::join(
                 std::vector<std::string>{
-                    ConfigBuilder::build_section(
+                    mysql_harness::ConfigBuilder::build_section(
                         "http_server", {{"port", std::to_string(http_port_)},
                                         {"require_realm", "secure"}}),
-                    ConfigBuilder::build_section(
+                    mysql_harness::ConfigBuilder::build_section(
                         "http_auth_backend:local",
                         {{"backend", "file"},
                          {"filename", mysql_harness::Path(conf_dir_.name())
                                           .join(passwd_filename_)
                                           .str()}}),
-                    ConfigBuilder::build_section("http_auth_realm:secure",
-                                                 {{"backend", "local"},
-                                                  {"method", "basic"},
-                                                  {"name", "API"},
-                                                  {"require", "valid-user"}})},
+                    mysql_harness::ConfigBuilder::build_section(
+                        "http_auth_realm:secure", {{"backend", "local"},
+                                                   {"method", "basic"},
+                                                   {"name", "API"},
+                                                   {"require", "valid-user"}})},
                 "\n"))},
-        http_server_{launch_router({"-c", conf_file_})} {
+        http_server_{launch_router({"-c", conf_file_}, EXIT_SUCCESS,
+                                   /*catch_stderr*/ true, /*with_sudo*/ false,
+                                   /*wait_for_notify_ready*/ -1s)} {
     std::string pwf_name(
         mysql_harness::Path(conf_dir_.name()).join(passwd_filename_).str());
     std::fstream pwf{pwf_name, pwf.out};
@@ -1631,7 +1631,7 @@ const HttpServerAuthParams http_server_auth_params[]{
     {"wrong password", "WL12503::TS_2_2", "/", 401, "other", "test"},
 };
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     Spec, HttpServerAuthTest, ::testing::ValuesIn(http_server_auth_params),
     [](const ::testing::TestParamInfo<HttpServerAuthParams> &info) {
       return gtest_sanitize_param_name(
@@ -1685,11 +1685,12 @@ class HttpServerAuthFailTest
  */
 TEST_P(HttpServerAuthFailTest, ensure) {
   std::vector<std::string> config_sections{
-      ConfigBuilder::build_section("http_server",
-                                   {
-                                       {"port", std::to_string(http_port_)},
-                                       {"require_realm", "secure"},
-                                   }),
+      mysql_harness::ConfigBuilder::build_section(
+          "http_server",
+          {
+              {"port", std::to_string(http_port_)},
+              {"require_realm", "secure"},
+          }),
   };
   std::string passwd_filename =
       Path(conf_dir_.name()).join(passwd_filename_).str();
@@ -1709,7 +1710,8 @@ TEST_P(HttpServerAuthFailTest, ensure) {
 
   ProcessWrapper &http_server{
       launch_router({"-c", conf_file},
-                    GetParam().check_at_runtime ? EXIT_SUCCESS : EXIT_FAILURE)};
+                    GetParam().check_at_runtime ? EXIT_SUCCESS : EXIT_FAILURE,
+                    true, false, -1s)};
 
   std::fstream pwf{passwd_filename, std::ios::out};
 
@@ -1747,7 +1749,7 @@ const HttpServerAuthFailParams http_server_auth_fail_params[]{
     {"backend_no_section",
      "",
      {
-         ConfigBuilder::build_section(
+         mysql_harness::ConfigBuilder::build_section(
              "http_auth_backend",
              {{"backend", "file"}, {"filename", "does-not-exists"}}),
      },
@@ -1756,37 +1758,38 @@ const HttpServerAuthFailParams http_server_auth_fail_params[]{
      "[http_auth_backend:example]"},
     {"backend_file_filename_not_exists",
      "WL12503::TS_FR6_1",
-     {ConfigBuilder::build_section("http_auth_backend:local",
-                                   {
-                                       {"backend", "file"},
-                                       {"filename", "does-not-exists"},
-                                   }),
-      ConfigBuilder::build_section("http_auth_realm:secure",
-                                   {{"backend", "doesnotexist"},
-                                    {"method", "basic"},
-                                    {"name", "API"},
-                                    {"require", "valid-user"}})},
+     {mysql_harness::ConfigBuilder::build_section(
+          "http_auth_backend:local",
+          {
+              {"backend", "file"},
+              {"filename", "does-not-exists"},
+          }),
+      mysql_harness::ConfigBuilder::build_section("http_auth_realm:secure",
+                                                  {{"backend", "doesnotexist"},
+                                                   {"method", "basic"},
+                                                   {"name", "API"},
+                                                   {"require", "valid-user"}})},
      false,
      "parsing does-not-exists "},
     {"backend_method_unknown",
      "WL12503::TS_FR6_2",
      {
-         ConfigBuilder::build_section("http_auth_realm:secure",
-                                      {{"backend", "doesnotexist"},
-                                       {"method", "unknown"},
-                                       {"name", "API"},
-                                       {"require", "valid-user"}}),
+         mysql_harness::ConfigBuilder::build_section(
+             "http_auth_realm:secure", {{"backend", "doesnotexist"},
+                                        {"method", "unknown"},
+                                        {"name", "API"},
+                                        {"require", "valid-user"}}),
      },
      false,
      "unsupported authentication method for "},
     {"backend_does_not_exist",
      "WL12503::TS_FR6_1",
      {
-         ConfigBuilder::build_section("http_auth_realm:secure",
-                                      {{"backend", "doesnotexist"},
-                                       {"method", "basic"},
-                                       {"name", "API"},
-                                       {"require", "valid-user"}}),
+         mysql_harness::ConfigBuilder::build_section(
+             "http_auth_realm:secure", {{"backend", "doesnotexist"},
+                                        {"method", "basic"},
+                                        {"name", "API"},
+                                        {"require", "valid-user"}}),
      },
      false,
      "The option 'backend=doesnotexist' in [http_auth_realm:secure] does not "
@@ -1795,11 +1798,11 @@ const HttpServerAuthFailParams http_server_auth_fail_params[]{
     {"realm_no_section",
      "WL12503::TS_FR6_1",
      {
-         ConfigBuilder::build_section("http_auth_realm",
-                                      {{"backend", "doesnotexist"},
-                                       {"method", "basic"},
-                                       {"name", "API"},
-                                       {"require", "valid-user"}}),
+         mysql_harness::ConfigBuilder::build_section(
+             "http_auth_realm", {{"backend", "doesnotexist"},
+                                 {"method", "basic"},
+                                 {"name", "API"},
+                                 {"require", "valid-user"}}),
      },
      false,
      "The config section [http_auth_realm] requires a name, like "
@@ -1807,42 +1810,45 @@ const HttpServerAuthFailParams http_server_auth_fail_params[]{
     {"multiple_backends",
      "WL12503::TS_2_7",
      {
-         ConfigBuilder::build_section("http_auth_realm:secure",
-                                      {{"backend", "local"},
-                                       {"method", "basic"},
-                                       {"name", "API"},
-                                       {"require", "valid-user"}}),
-         ConfigBuilder::build_section("http_auth_backend:local",
-                                      {
-                                          {"backend", "file"},
-                                          {"filename", "@placeholder@"},
-                                      }),
-         ConfigBuilder::build_section("http_auth_backend:someother",
-                                      {
-                                          {"backend", "file"},
-                                          {"filename", "@placeholder@"},
-                                      }),
+         mysql_harness::ConfigBuilder::build_section(
+             "http_auth_realm:secure", {{"backend", "local"},
+                                        {"method", "basic"},
+                                        {"name", "API"},
+                                        {"require", "valid-user"}}),
+         mysql_harness::ConfigBuilder::build_section(
+             "http_auth_backend:local",
+             {
+                 {"backend", "file"},
+                 {"filename", "@placeholder@"},
+             }),
+         mysql_harness::ConfigBuilder::build_section(
+             "http_auth_backend:someother",
+             {
+                 {"backend", "file"},
+                 {"filename", "@placeholder@"},
+             }),
      },
      true,
      ""},
     {"multiple_realms",
      "",
      {
-         ConfigBuilder::build_section("http_auth_realm:secure",
-                                      {{"backend", "local"},
-                                       {"method", "basic"},
-                                       {"name", "API"},
-                                       {"require", "valid-user"}}),
-         ConfigBuilder::build_section("http_auth_realm:someother",
-                                      {{"backend", "local"},
-                                       {"method", "basic"},
-                                       {"name", "SomeOtherApi"},
-                                       {"require", "valid-user"}}),
-         ConfigBuilder::build_section("http_auth_backend:local",
-                                      {
-                                          {"backend", "file"},
-                                          {"filename", "@placeholder@"},
-                                      }),
+         mysql_harness::ConfigBuilder::build_section(
+             "http_auth_realm:secure", {{"backend", "local"},
+                                        {"method", "basic"},
+                                        {"name", "API"},
+                                        {"require", "valid-user"}}),
+         mysql_harness::ConfigBuilder::build_section(
+             "http_auth_realm:someother", {{"backend", "local"},
+                                           {"method", "basic"},
+                                           {"name", "SomeOtherApi"},
+                                           {"require", "valid-user"}}),
+         mysql_harness::ConfigBuilder::build_section(
+             "http_auth_backend:local",
+             {
+                 {"backend", "file"},
+                 {"filename", "@placeholder@"},
+             }),
      },
      true,
      ""},
@@ -1850,13 +1856,13 @@ const HttpServerAuthFailParams http_server_auth_fail_params[]{
     {"wrong_backend_type_doesnot_exist",
      "",
      {
-         ConfigBuilder::build_section("http_auth_backend:local",
-                                      {{"backend", "doesnotexist"}}),
+         mysql_harness::ConfigBuilder::build_section(
+             "http_auth_backend:local", {{"backend", "doesnotexist"}}),
      },
      false,
      "unknown backend=doesnotexist in section: http_auth_backend"}};
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     Spec, HttpServerAuthFailTest,
     ::testing::ValuesIn(http_server_auth_fail_params),
     [](const ::testing::TestParamInfo<HttpServerAuthFailParams> &info) {
