@@ -1,17 +1,16 @@
 /*
- Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+ Copyright (c) 2020, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is designed to work with certain software (including
+ This program is also distributed with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have either included with
- the program or referenced in the documentation.
+ separately licensed software that they have included with MySQL.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,13 +22,11 @@
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <gtest/gtest-matchers.h>
 #include <algorithm>
 #include <chrono>
 #include <cstdint>
 #include <cstdlib>
 #include <fstream>
-#include <initializer_list>
 #include <iterator>
 #include <map>
 #include <memory>
@@ -42,30 +39,26 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
+#include "filesystem_utils.h"
 #include "keyring/keyring_manager.h"
 #include "mock_server_rest_client.h"
 #include "mock_server_testutils.h"
 #include "mysql/harness/filesystem.h"
-#include "mysql/harness/string_utils.h"  // join
 #include "mysql/harness/tls_client_context.h"
 #include "mysql/harness/tls_context.h"
-#include "mysql/harness/utility/string.h"  // string_format
 #include "mysqlrouter/http_client.h"
-#include "mysqlrouter/utils.h"
 #include "process_wrapper.h"
 #include "rest_api_testutils.h"
 #include "router_component_test.h"
-#include "router_test_helpers.h"
 #include "tcp_port_pool.h"
+#include "utils.h"
 
 using namespace std::chrono_literals;
 
-using mysql_harness::utility::string_format;
-
-class TestRestApiEnable : public RouterComponentBootstrapTest {
+class TestRestApiEnable : public RouterComponentTest {
  public:
   void SetUp() override {
-    RouterComponentBootstrapTest::SetUp();
+    RouterComponentTest::SetUp();
 
     cluster_node_port = port_pool_.get_next_available();
     cluster_http_port = port_pool_.get_next_available();
@@ -77,149 +70,75 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
         trace_file, cluster_node_port, EXIT_SUCCESS, false, cluster_http_port);
 
     set_globals();
-    set_router_accepting_ports();
 
     custom_port = port_pool_.get_next_available();
+    router_port = port_pool_.get_next_available();
 
     setup_paths();
   }
 
-  ProcessWrapper &do_bootstrap(std::vector<std::string> additional_config,
-                               bool will_run_with_created_config = true) {
+  ProcessWrapper &do_bootstrap(std::vector<std::string> additional_config) {
     std::vector<std::string> cmdline = {
         "--bootstrap=" + gr_member_ip + ":" + std::to_string(cluster_node_port),
-        "-d",
-        temp_test_dir.name(),
-        "--conf-set-option=logger.level=DEBUG",
-    };
-
-    if (will_run_with_created_config) {
-      // since we are launching the Router after the bootstrap we
-      // can't allow default ports to be used
-      cmdline.insert(cmdline.end(),
-                     {"--conf-set-option=routing:bootstrap_rw.bind_port=" +
-                          std::to_string(router_port_rw),
-                      "--conf-set-option=routing:bootstrap_ro.bind_port=" +
-                          std::to_string(router_port_ro),
-                      "--conf-set-option=routing:bootstrap_x_rw.bind_port=" +
-                          std::to_string(router_port_x_rw),
-                      "--conf-set-option=routing:bootstrap_x_ro.bind_port=" +
-                          std::to_string(router_port_x_ro)});
-
-      // Let's overwrite the default bind_address to prevent the MacOS firewall
-      // from complaining
-      cmdline.insert(
-          cmdline.end(),
-          {"--conf-set-option=DEFAULT.bind_address=127.0.0.1",
-           "--conf-set-option=routing:bootstrap_rw.bind_address=127.0.0.1",
-           "--conf-set-option=routing:bootstrap_ro.bind_address=127.0.0.1",
-           "--conf-set-option=routing:bootstrap_x_rw.bind_address=127.0.0.1",
-           "--conf-set-option=routing:bootstrap_x_ro.bind_address=127.0.0.1"});
-
-      if (std::find(additional_config.begin(), additional_config.end(),
-                    "--disable-rw-split") == additional_config.end()) {
-        // if --disable-rw-split isn't set, set the bind-port and bind-address
-        cmdline.insert(
-            cmdline.end(),
-            {"--conf-set-option=routing:bootstrap_rw_split.bind_port=" +
-                 std::to_string(router_port_rw_split),
-             "--conf-set-option=routing:bootstrap_rw_split.bind_address="
-             "127.0.0.1"});
-      }
-    }
-
+        "-d", temp_test_dir.name(), "--conf-base-port",
+        std::to_string(router_port)};
     std::move(std::begin(additional_config), std::end(additional_config),
               std::back_inserter(cmdline));
-    auto &router_bootstrap = launch_router_for_bootstrap(
-        cmdline, EXIT_SUCCESS, /*disable_rest=*/false);
+    auto &router_bootstrap = launch_router(cmdline, EXIT_SUCCESS);
 
+    router_bootstrap.register_response("Please enter MySQL password for root: ",
+                                       k_root_password + "\n");
     check_exit_code(router_bootstrap, EXIT_SUCCESS);
 
     EXPECT_TRUE(router_bootstrap.expect_output(
         "MySQL Router configured for the InnoDB Cluster 'mycluster'"));
 
+    auto plugin_dir = mysql_harness::get_plugin_dir(get_origin().str());
+    EXPECT_TRUE(add_line_to_config_file(config_path.str(), "DEFAULT",
+                                        "plugin_folder", plugin_dir));
+
     return router_bootstrap;
   }
 
   void assert_rest_config(const mysql_harness::Path &config_path,
                           const bool is_enabled) const {
-    auto content = get_file_output(config_path.str());
+    const static char *nl = R"(((.|\r\n)*\s*))";
 
-    std::string rest_api_section = mysql_harness::join(
-        std::initializer_list<const char *>{
-            "[rest_api]",
-        },
-        "\n");
+    EXPECT_EQ(is_enabled, file_contains_regex(config_path, "\\[rest_api\\]"));
 
-    std::string http_server_section = mysql_harness::join(
-        std::initializer_list<const char *>{
-            R"(\[http_server\])",
-            R"(port=.+)",
-            R"(ssl=1)",
-            R"(ssl_cert=.*)",
-            R"(ssl_key=.*)",
-        },
-        "\n");
+    EXPECT_EQ(is_enabled,
+              file_contains_regex(config_path,
+                                  std::string{"\\[http_server\\]"} + nl +
+                                      "port=.*" + nl + "ssl=1" + nl +
+                                      "ssl_cert=.*" + nl + "ssl_key=.*"));
 
-    std::string http_auth_backend_section = mysql_harness::join(
-        std::initializer_list<const char *>{
-            "[http_auth_backend:default_auth_backend]",
-            "backend=metadata_cache",
-        },
-        "\n");
+    EXPECT_EQ(is_enabled,
+              file_contains_regex(
+                  config_path,
+                  std::string{"\\[http_auth_backend:default_auth_backend\\]"} +
+                      nl + "backend=metadata_cache"));
 
-    std::string http_auth_realm_section = mysql_harness::join(
-        std::initializer_list<const char *>{
-            "[http_auth_realm:default_auth_realm]",
-            "backend=default_auth_backend",
-            "method=basic",
-            "name=default_realm",
-        },
-        "\n");
+    EXPECT_EQ(is_enabled,
+              file_contains_regex(
+                  config_path,
+                  std::string{"\\[http_auth_realm:default_auth_realm\\]"} + nl +
+                      "backend=default_auth_backend" + nl + "method=basic" +
+                      nl + "name=default_realm"));
 
-    std::string rest_router_section = mysql_harness::join(
-        std::initializer_list<const char *>{
-            "[rest_router]",
-            "require_realm=default_auth_realm",
-        },
-        "\n");
+    EXPECT_EQ(is_enabled,
+              file_contains_regex(config_path,
+                                  std::string{"\\[rest_router\\]"} + nl +
+                                      "require_realm=default_auth_realm"));
 
-    std::string rest_routing_section = mysql_harness::join(
-        std::initializer_list<const char *>{
-            "[rest_routing]",
-            "require_realm=default_auth_realm",
-        },
-        "\n");
+    EXPECT_EQ(is_enabled,
+              file_contains_regex(config_path,
+                                  std::string{"\\[rest_routing\\]"} + nl +
+                                      "require_realm=default_auth_realm"));
 
-    std::string rest_metadata_cache_section = mysql_harness::join(
-        std::initializer_list<const char *>{
-            "[rest_metadata_cache]",
-            "require_realm=default_auth_realm",
-        },
-        "\n");
-
-    if (is_enabled) {
-      EXPECT_THAT(content, ::testing::AllOf(
-                               ::testing::HasSubstr(rest_api_section),
-                               ::testing::ContainsRegex(http_server_section),
-                               ::testing::HasSubstr(http_auth_backend_section),
-                               ::testing::HasSubstr(http_auth_realm_section),
-                               ::testing::HasSubstr(rest_router_section),
-                               ::testing::HasSubstr(rest_routing_section),
-                               ::testing::HasSubstr(rest_metadata_cache_section)
-
-                                   ));
-    } else {
-      EXPECT_THAT(content,
-                  ::testing::Not(::testing::AnyOf(
-                      ::testing::HasSubstr(rest_api_section),
-                      ::testing::ContainsRegex(http_server_section),
-                      ::testing::HasSubstr(http_auth_backend_section),
-                      ::testing::HasSubstr(http_auth_realm_section),
-                      ::testing::HasSubstr(rest_router_section),
-                      ::testing::HasSubstr(rest_routing_section),
-                      ::testing::HasSubstr(rest_metadata_cache_section))));
-    }
+    EXPECT_EQ(is_enabled,
+              file_contains_regex(
+                  config_path, std::string{"\\[rest_metadata_cache\\]"} + nl +
+                                   "require_realm=default_auth_realm"));
   }
 
   enum class CertFile { k_ca_key, k_ca_cert, k_router_key, k_router_cert };
@@ -236,7 +155,7 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
   }
 
   std::string read_cert(CertFile cert) const {
-    return get_file_output(datadir_path.join(cert_filenames.at(cert)).str());
+    return read_file(datadir_path.join(cert_filenames.at(cert)).str());
   }
 
   bool certificate_files_not_modified(
@@ -275,8 +194,8 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
   }
 
   void assert_rest_works(const uint16_t port) {
-    const auto uri = "https://" + gr_member_ip + ":" + std::to_string(port) +
-                     rest_api_basepath + "/router/status";
+    const auto uri = std::string(rest_api_basepath) + "/router/status";
+    wait_for_rest_endpoint_ready(uri, port);
 
     const auto ca_file =
         datadir_path.join(cert_filenames.at(CertFile::k_ca_cert));
@@ -296,7 +215,9 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
     }
 
     IOContext io_ctx;
-    RestClient rest_client(io_ctx, std::move(tls_ctx));
+    auto https_client = std::make_unique<HttpsClient>(
+        io_ctx, std::move(tls_ctx), gr_member_ip, port);
+    RestClient rest_client(std::move(https_client));
 
     JsonDocument json_doc;
     // We do not care to authenticate, just check if we got a response
@@ -309,9 +230,10 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
     std::vector<std::string> cmdline = {
         "--bootstrap=" + gr_member_ip + ":" + std::to_string(cluster_node_port),
         "-d", path.str()};
-    auto &router_bootstrap = launch_router_for_bootstrap(
-        cmdline, EXIT_SUCCESS, /*disable_rest=*/false);
+    auto &router_bootstrap = launch_router(cmdline, EXIT_SUCCESS);
 
+    router_bootstrap.register_response("Please enter MySQL password for root: ",
+                                       k_root_password + "\n");
     check_exit_code(router_bootstrap, EXIT_SUCCESS);
 
     auto custom_config_path = path.join("mysqlrouter.conf");
@@ -361,32 +283,28 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
 
   ProcessWrapper &launch_router(
       const std::vector<std::string> &params, int expected_exit_code /*= 0*/,
-      std::chrono::milliseconds wait_for_notify_ready = -1s,
-      ProcessWrapper::OutputResponder output_responder =
-          RouterComponentBootstrapTest::kBootstrapOutputResponder) {
+      std::chrono::milliseconds wait_for_notify_ready = -1s) {
     return ProcessManager::launch_router(
         params, expected_exit_code,
-        /*catch_stderr*/ true, /*with_sudo*/ false, wait_for_notify_ready,
-        output_responder);
+        /*catch_stderr*/ true, /*with_sudo*/ false, wait_for_notify_ready);
   }
 
   TlsLibraryContext m_tls_lib_ctx;
   const std::string gr_member_ip{"127.0.0.1"};
   const std::string cluster_id{"3a0be5af-0022-11e8-9655-0800279e6a88"};
-
+  const std::string k_root_password{"fake-pass"};
   uint16_t cluster_node_port;
   uint16_t cluster_http_port;
   uint16_t custom_port;
-  uint16_t router_port_rw;
-  uint16_t router_port_ro;
-  uint16_t router_port_rw_split;
-  uint16_t router_port_x_rw;
-  uint16_t router_port_x_ro;
+  uint16_t router_port;
+  uint16_t default_rest_port{8443};
   ProcessWrapper *cluster_node;
+  TcpPortPool port_pool_;
 
   TempDirectory temp_test_dir;
   mysql_harness::Path config_path;
   mysql_harness::Path datadir_path;
+  mysql_harness::Path logdir_path;
 
   static const std::string predefined_ca_key;
   static const std::string predefined_ca_cert;
@@ -406,25 +324,32 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
       {CertFile::k_router_cert, "router-cert.pem"}};
 
  protected:
-  void set_globals(std::string cluster_id = "") {
-    set_mock_metadata(cluster_http_port, cluster_id,
-                      classic_ports_to_gr_nodes({cluster_node_port}), 0,
-                      {cluster_node_port}, 0 /*view_id*/,
-                      false /*error_on_md_query*/);
+  void set_globals() {
+    auto json_doc = mock_GR_metadata_as_json(cluster_id, {cluster_node_port},
+                                             0 /*primary_id*/, 0 /*view_id*/,
+                                             false /*error_on_md_query*/);
+    JsonAllocator allocator;
+    JsonValue gr_members_json(rapidjson::kArrayType);
+    JsonValue member(rapidjson::kArrayType);
+    member.PushBack(
+        JsonValue(gr_member_ip.c_str(), gr_member_ip.length(), allocator),
+        allocator);
+    member.PushBack(cluster_node_port, allocator);
+    gr_members_json.PushBack(member, allocator);
+
+    json_doc.AddMember("innodb_cluster_instances", gr_members_json, allocator);
+
+    const auto json_str = json_to_string(json_doc);
+
+    EXPECT_NO_THROW(
+        MockServerRestClient(cluster_http_port).set_globals(json_str));
   }
 
   void setup_paths() {
     config_path =
         mysql_harness::Path{temp_test_dir.name()}.join("mysqlrouter.conf");
     datadir_path = mysql_harness::Path{temp_test_dir.name()}.join("data");
-  }
-
-  void set_router_accepting_ports() {
-    router_port_rw = port_pool_.get_next_available();
-    router_port_ro = port_pool_.get_next_available();
-    router_port_rw_split = port_pool_.get_next_available();
-    router_port_x_rw = port_pool_.get_next_available();
-    router_port_x_ro = port_pool_.get_next_available();
+    logdir_path = mysql_harness::Path{temp_test_dir.name()}.join("log");
   }
 };
 
@@ -535,42 +460,46 @@ VeZwMK4Cb8EO7PzsnX2tD6AA5Ums6GhNgYsbJgdq4MdKb3x6YWZ8DpksSIX2
  * WL13906:TS_FR06_01
  */
 TEST_F(TestRestApiEnable, ensure_rest_is_disabled) {
-  do_bootstrap({
-      "--disable-rest",                    //
-      "--client-ssl-mode", "PASSTHROUGH",  //
-      "--disable-rw-split",                //
-  });
+  do_bootstrap({"--disable-rest", "--client-ssl-mode", "PASSTHROUGH"});
 
   EXPECT_FALSE(certificate_files_exists(
       {cert_file_t::k_ca_key, cert_file_t::k_ca_cert, cert_file_t::k_router_key,
        cert_file_t::k_router_cert}));
   assert_rest_config(config_path, false);
 
-  auto &router = ProcessManager::launch_router({"-c", config_path.str()});
+  launch_router({"-c", config_path.str()}, EXIT_SUCCESS);
+  ASSERT_TRUE(wait_for_port_ready(router_port));
 
-  EXPECT_EQ(std::error_code{}, router.send_clean_shutdown_event());
-  EXPECT_EQ(0, router.wait_for_exit());
+  IOContext io_ctx;
+  auto http_client =
+      std::make_unique<HttpClient>(io_ctx, gr_member_ip, default_rest_port);
+  RestClient rest_client(std::move(http_client));
 
-  EXPECT_THAT(router.get_logfile_content(),
-              ::testing::Not(::testing::HasSubstr("rest_routing")));
+  const auto uri = std::string(rest_api_basepath) + "/router/status";
+  wait_endpoint_404(rest_client, uri, std::chrono::milliseconds(1000));
 }
 
 /**
  * @test
  * Verify that bootstrap enables REST API by default. 'mysqlrouter.conf' should
- * contain lines required to enable REST API.
+ * contain lines required to enable REST API and connecting to REST API
+ * should work.
  *
  * WL13906:TS_FR03_01
  * WL13906:TS_FR05_01
  */
-TEST_F(TestRestApiEnable, ensure_rest_is_configured_by_default) {
-  ASSERT_NO_FATAL_FAILURE(
-      do_bootstrap({/*default command line arguments*/}, false));
+TEST_F(TestRestApiEnable, ensure_rest_works) {
+  do_bootstrap({/*default command line arguments*/});
 
   EXPECT_TRUE(certificate_files_exists(
       {cert_file_t::k_ca_key, cert_file_t::k_ca_cert, cert_file_t::k_router_key,
        cert_file_t::k_router_cert}));
   assert_rest_config(config_path, true);
+
+  launch_router({"-c", config_path.str()}, EXIT_SUCCESS);
+  ASSERT_TRUE(wait_for_port_ready(router_port));
+
+  assert_rest_works(default_rest_port);
 }
 
 /**
@@ -588,7 +517,8 @@ TEST_F(TestRestApiEnable, ensure_rest_works_on_custom_port) {
        cert_file_t::k_router_cert}));
   assert_rest_config(config_path, true);
 
-  ProcessManager::launch_router({"-c", config_path.str()});
+  launch_router({"-c", config_path.str()}, EXIT_SUCCESS);
+  ASSERT_TRUE(wait_for_port_ready(router_port));
 
   assert_rest_works(custom_port);
 }
@@ -607,7 +537,7 @@ class UseEdgeHttpsPortValues : public TestRestApiEnable,
  */
 TEST_P(UseEdgeHttpsPortValues,
        ensure_bootstrap_works_for_edge_https_port_values) {
-  do_bootstrap({"--https-port", std::to_string(GetParam())}, false);
+  do_bootstrap({"--https-port", std::to_string(GetParam())});
 
   EXPECT_TRUE(certificate_files_exists(
       {cert_file_t::k_ca_key, cert_file_t::k_ca_cert, cert_file_t::k_router_key,
@@ -633,7 +563,7 @@ TEST_P(EnableWrongHttpsPort, ensure_bootstrap_fails_for_invalid_https_port) {
   std::vector<std::string> cmdline = {
       "--bootstrap=" + gr_member_ip + ":" + std::to_string(cluster_node_port),
       "-d", temp_test_dir.name(), "--https-port", std::to_string(GetParam())};
-  auto &router_bootstrap = launch_router_for_bootstrap(cmdline, EXIT_FAILURE);
+  auto &router_bootstrap = launch_router(cmdline, EXIT_FAILURE);
 
   check_exit_code(router_bootstrap, EXIT_FAILURE);
 
@@ -662,7 +592,7 @@ class OverlappingHttpsPort
  */
 TEST_P(OverlappingHttpsPort,
        ensure_bootstrap_works_for_overlapping_https_port) {
-  do_bootstrap({"--https-port", std::to_string(this->*GetParam())}, false);
+  do_bootstrap({"--https-port", std::to_string(this->*GetParam())});
 
   EXPECT_TRUE(certificate_files_exists(
       {cert_file_t::k_ca_key, cert_file_t::k_ca_cert, cert_file_t::k_router_key,
@@ -672,7 +602,7 @@ TEST_P(OverlappingHttpsPort,
 
 INSTANTIATE_TEST_SUITE_P(
     CheckOverlappingHttpsPort, OverlappingHttpsPort,
-    ::testing::Values(&TestRestApiEnable::router_port_rw,
+    ::testing::Values(&TestRestApiEnable::router_port,
                       &TestRestApiEnable::cluster_node_port));
 
 /**
@@ -690,7 +620,7 @@ TEST_F(TestRestApiEnable, bootstrap_conflicting_options) {
       "--https-port",
       std::to_string(custom_port),
       "--disable-rest"};
-  auto &router_bootstrap = launch_router_for_bootstrap(cmdline, EXIT_FAILURE);
+  auto &router_bootstrap = launch_router(cmdline, EXIT_FAILURE);
 
   check_exit_code(router_bootstrap, EXIT_FAILURE);
 
@@ -717,11 +647,10 @@ class RestApiEnableUserCertificates
  */
 TEST_P(RestApiEnableUserCertificates, ensure_rest_works_with_user_certs) {
   create_cert_files(GetParam());
-  auto &router_bootstrap =
-      do_bootstrap({"--https-port", std::to_string(custom_port)});
-  const auto expected_message =
-      string_format("- Using existing certificates from the '%s' directory",
-                    datadir_path.real_path().c_str());
+  auto &router_bootstrap = do_bootstrap({/*default command line arguments*/});
+  const auto expected_message = mysqlrouter::string_format(
+      "- Using existing certificates from the '%s' directory",
+      datadir_path.real_path().c_str());
   EXPECT_THAT(router_bootstrap.get_full_output(),
               ::testing::HasSubstr(expected_message));
 
@@ -730,9 +659,10 @@ TEST_P(RestApiEnableUserCertificates, ensure_rest_works_with_user_certs) {
   assert_rest_config(config_path, true);
   EXPECT_TRUE(certificate_files_not_changed(GetParam()));
 
-  ProcessManager::launch_router({"-c", config_path.str()});
+  launch_router({"-c", config_path.str()}, EXIT_SUCCESS);
+  ASSERT_TRUE(wait_for_port_ready(router_port));
 
-  assert_rest_works(custom_port);
+  assert_rest_works(default_rest_port);
 }
 
 INSTANTIATE_TEST_SUITE_P(
@@ -767,7 +697,10 @@ TEST_P(RestApiEnableNotEnoughFiles, ensure_rest_fail) {
   std::vector<std::string> cmdline = {
       "--bootstrap=" + gr_member_ip + ":" + std::to_string(cluster_node_port),
       "-d", temp_test_dir.name()};
-  auto &router_bootstrap = launch_router_for_bootstrap(cmdline, EXIT_FAILURE);
+  auto &router_bootstrap = launch_router(cmdline, EXIT_FAILURE);
+
+  router_bootstrap.register_response("Please enter MySQL password for root: ",
+                                     k_root_password + "\n");
   check_exit_code(router_bootstrap, EXIT_FAILURE);
 
   const auto &files = GetParam();
@@ -787,7 +720,7 @@ TEST_P(RestApiEnableNotEnoughFiles, ensure_rest_fail) {
   if (!missing_files.empty()) missing_files += ", ";
   if (!has_file(cert_file_t::k_router_cert))
     missing_files += router_cert_filename;
-  const std::string output = string_format(
+  const std::string output = mysqlrouter::string_format(
       "Error: Missing certificate files in %s: '%s'. Please provide them or "
       "erase the existing certificate files and re-run bootstrap.",
       datadir_path.real_path().c_str(), missing_files.c_str());
@@ -823,8 +756,7 @@ class RestApiInvalidUserCerts
 
 /**
  * @test
- * Verify that bootstrap does not check if user provided certs and keys are
- * valid.
+ * Verify that bootstrap do not check if user provided certs and keys are valid.
  * Verify that bootstrap succeeds and files are not changed.
  *
  * WL13906:TS_NFR01_01
@@ -854,24 +786,21 @@ TEST_P(RestApiInvalidUserCerts,
     router_cert_stream << GetParam();
   }
 
-  auto &router_bootstrap =
-      do_bootstrap({"--https-port", std::to_string(custom_port)});
-  const auto expected_message =
-      string_format("- Using existing certificates from the '%s' directory",
-                    datadir_path.real_path().c_str());
+  auto &router_bootstrap = do_bootstrap({/*default command line arguments*/});
+  const auto expected_message = mysqlrouter::string_format(
+      "- Using existing certificates from the '%s' directory",
+      datadir_path.real_path().c_str());
   EXPECT_THAT(router_bootstrap.get_full_output(),
               ::testing::HasSubstr(expected_message));
 
   EXPECT_TRUE(certificate_files_exists(
       {cert_file_t::k_ca_key, cert_file_t::k_ca_cert, cert_file_t::k_router_key,
        cert_file_t::k_router_cert}));
-  EXPECT_EQ(get_file_output(datadir_path.join(ca_key_filename).str()),
+  EXPECT_EQ(read_file(datadir_path.join(ca_key_filename).str()), GetParam());
+  EXPECT_EQ(read_file(datadir_path.join(ca_cert_filename).str()), GetParam());
+  EXPECT_EQ(read_file(datadir_path.join(router_key_filename).str()),
             GetParam());
-  EXPECT_EQ(get_file_output(datadir_path.join(ca_cert_filename).str()),
-            GetParam());
-  EXPECT_EQ(get_file_output(datadir_path.join(router_key_filename).str()),
-            GetParam());
-  EXPECT_EQ(get_file_output(datadir_path.join(router_cert_filename).str()),
+  EXPECT_EQ(read_file(datadir_path.join(router_cert_filename).str()),
             GetParam());
   assert_rest_config(config_path, true);
 
@@ -883,9 +812,16 @@ TEST_P(RestApiInvalidUserCerts,
       datadir_path.real_path().join(router_key_filename).str() +
       "' or SSL certificate file '" +
       datadir_path.real_path().join(router_cert_filename).str() + "' failed";
-  EXPECT_THAT(router.get_logfile_content("mysqlrouter.log",
-                                         temp_test_dir.name() + "/log"),
+  EXPECT_THAT(router.get_full_logfile("mysqlrouter.log", logdir_path.str()),
               ::testing::HasSubstr(log_error));
+
+  IOContext io_ctx;
+  auto http_client =
+      std::make_unique<HttpClient>(io_ctx, gr_member_ip, default_rest_port);
+  RestClient rest_client(std::move(http_client));
+
+  const auto uri = std::string(rest_api_basepath) + "/router/status";
+  wait_endpoint_404(rest_client, uri, std::chrono::milliseconds(1000));
 }
 
 INSTANTIATE_TEST_SUITE_P(CheckRestApiInvalidUserCerts, RestApiInvalidUserCerts,
@@ -920,12 +856,10 @@ TEST_F(TestRestApiEnable, use_custom_datadir_absolute_path) {
 }
 
 /**
- * @test Verify certificates and keys are cleaned up on error.
- *
- * Verify that
- *
- * 1) bootstrap fails and exits with a meaningful error,
- * 2) any key/certificate files created during bootstrap are erased.
+ * @test
+ * Verify certificates and keys are cleaned up on error. Verify that 1)
+ * bootstrap fails and exits with a meaningful error, 2) any key/certificate
+ * files created during bootstrap are erased.
  *
  * WL13906:TS_Extra_03
  */
@@ -934,17 +868,15 @@ TEST_F(TestRestApiEnable, ensure_certificate_files_cleanup) {
       "--bootstrap=" + gr_member_ip + ":" + std::to_string(cluster_node_port),
       "-d", temp_test_dir.name(), "--strict"};
 
-  // to fail account verification, use a cluster-id which leads to a failed
-  // query at bootstrap.
-  set_globals("some-garbage");
-
   // Account verification is done after the certificates are created, therefore
   // we expect the following order of events:
   // 1. Certificates are created
   // 2. Account verification fails due to the '--strict' option and missing
   //    queries in the rest_api_enable.js file.
   // 3. Certificates are cleaned up.
-  auto &router_bootstrap = launch_router_for_bootstrap(cmdline, EXIT_FAILURE);
+  auto &router_bootstrap = launch_router(cmdline, EXIT_FAILURE);
+  router_bootstrap.register_response("Please enter MySQL password for root: ",
+                                     k_root_password + "\n");
 
   check_exit_code(router_bootstrap, EXIT_FAILURE);
   EXPECT_THAT(router_bootstrap.get_full_output(),
@@ -960,21 +892,16 @@ class TestRestApiEnableBootstrapFailover : public TestRestApiEnable {
   void SetUp() override {
     RouterComponentTest::SetUp();
     setup_paths();
-    set_router_accepting_ports();
   }
 
   void setup_mocks(const bool failover_successful) {
-    std::vector<uint16_t> classic_ports;
     for (auto i = 0; i < k_node_count; ++i) {
-      classic_ports.emplace_back(port_pool_.get_next_available());
+      gr_members.emplace_back(gr_member_ip, port_pool_.get_next_available());
     }
 
-    gr_nodes = classic_ports_to_gr_nodes(classic_ports);
-    cluster_nodes = classic_ports_to_cluster_nodes(classic_ports);
-
     for (auto i = 0; i < k_node_count; ++i) {
       cluster_http_port = port_pool_.get_next_available();
-      auto port = gr_nodes[i].classic_port;
+      auto port = gr_members[i].second;
 
       std::string trace_file;
       if (i == 0 || !failover_successful) {
@@ -994,24 +921,19 @@ class TestRestApiEnableBootstrapFailover : public TestRestApiEnable {
       ASSERT_TRUE(MockServerRestClient(cluster_http_port)
                       .wait_for_rest_endpoint_ready());
 
-      set_mock_metadata(cluster_http_port, cluster_id, gr_nodes, 0,
-                        cluster_nodes, 0, false, gr_member_ip, "",
-                        metadata_version, cluster_name);
+      set_mock_bootstrap_data(cluster_http_port, cluster_name, gr_members,
+                              metadata_version, cluster_id);
     }
 
-    cluster_node_port = gr_nodes[0].classic_port;
-    router_port_rw = port_pool_.get_next_available();
-    router_port_ro = port_pool_.get_next_available();
-    router_port_x_rw = port_pool_.get_next_available();
-    router_port_x_ro = port_pool_.get_next_available();
+    cluster_node_port = gr_members[0].second;
+    router_port = port_pool_.get_next_available();
   }
 
  private:
-  const mysqlrouter::MetadataSchemaVersion metadata_version{2, 2, 0};
+  const mysqlrouter::MetadataSchemaVersion metadata_version{2, 0, 3};
   const std::string cluster_name{"mycluster"};
   std::vector<std::pair<uint16_t, ProcessWrapper &>> mock_servers;
-  std::vector<GRNode> gr_nodes;
-  std::vector<ClusterNode> cluster_nodes;
+  std::vector<std::pair<std::string, unsigned>> gr_members;
   static const uint8_t k_node_count{3};
 };
 
@@ -1027,20 +949,19 @@ TEST_F(TestRestApiEnableBootstrapFailover,
        ensure_rest_works_after_node_failover) {
   const bool successful_failover = true;
   setup_mocks(successful_failover);
-  const auto rest_port = port_pool_.get_next_available();
-  auto &router_bootstrap = do_bootstrap(
-      {"--conf-set-option=http_server.port=" + std::to_string(rest_port)});
+  auto &router_bootstrap = do_bootstrap({/*default command line arguments*/});
   EXPECT_THAT(router_bootstrap.get_full_output(),
-              ::testing::HasSubstr("trying to connect to"));
+              ::testing::HasSubstr("trying to connect to another node"));
 
   EXPECT_TRUE(certificate_files_exists(
       {cert_file_t::k_ca_key, cert_file_t::k_ca_cert, cert_file_t::k_router_key,
        cert_file_t::k_router_cert}));
   assert_rest_config(config_path, true);
 
-  ProcessManager::launch_router({"-c", config_path.str()});
+  launch_router({"-c", config_path.str()}, EXIT_SUCCESS);
+  ASSERT_TRUE(wait_for_port_ready(router_port));
 
-  assert_rest_works(rest_port);
+  assert_rest_works(default_rest_port);
 }
 
 /**
@@ -1059,11 +980,13 @@ TEST_F(TestRestApiEnableBootstrapFailover,
   std::vector<std::string> cmdline = {
       "--bootstrap=" + gr_member_ip + ":" + std::to_string(cluster_node_port),
       "-d", temp_test_dir.name(), "--strict"};
-  auto &router_bootstrap = launch_router_for_bootstrap(cmdline, EXIT_FAILURE);
+  auto &router_bootstrap = launch_router(cmdline, EXIT_FAILURE);
+  router_bootstrap.register_response("Please enter MySQL password for root: ",
+                                     k_root_password + "\n");
 
   check_exit_code(router_bootstrap, EXIT_FAILURE);
   EXPECT_THAT(router_bootstrap.get_full_output(),
-              ::testing::HasSubstr("trying to connect to"));
+              ::testing::HasSubstr("trying to connect to another node"));
   EXPECT_THAT(router_bootstrap.get_full_output(),
               ::testing::HasSubstr("Account verification failed"));
 
@@ -1087,7 +1010,9 @@ TEST_F(TestRestApiEnableBootstrapFailover,
   std::vector<std::string> cmdline = {
       "--bootstrap=" + gr_member_ip + ":" + std::to_string(cluster_node_port),
       "-d", temp_test_dir.name()};
-  auto &router_bootstrap = launch_router_for_bootstrap(cmdline, EXIT_FAILURE);
+  auto &router_bootstrap = launch_router(cmdline, EXIT_FAILURE);
+  router_bootstrap.register_response("Please enter MySQL password for root: ",
+                                     k_root_password + "\n");
 
   check_exit_code(router_bootstrap, EXIT_FAILURE);
   EXPECT_THAT(
