@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2017, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -26,6 +26,7 @@
 #include <thread>
 
 #include <gmock/gmock.h>
+#include <gtest/gtest.h>
 
 #include "mock_server_rest_client.h"
 #include "mock_server_testutils.h"
@@ -41,24 +42,25 @@
   ASSERT_THAT(expr, ::testing::Eq(std::error_code{})) << expr.message()
 
 using mysqlrouter::MySQLSession;
+using namespace std::chrono_literals;
 
 static const std::string kRestApiUsername("someuser");
 static const std::string kRestApiPassword("somepass");
 
 class RouterRoutingStrategyTest : public RouterComponentTest {
  protected:
-  virtual void SetUp() {
+  void SetUp() override {
     RouterComponentTest::SetUp();
 
     // Valgrind needs way more time
     if (getenv("WITH_VALGRIND")) {
-      wait_for_cache_ready_timeout = 5000;
-      wait_for_process_exit_timeout = 20000;
-      wait_for_static_ready_timeout = 1000;
+      wait_for_cache_ready_timeout = 5000ms;
+      wait_for_process_exit_timeout = 20000ms;
+      wait_for_static_ready_timeout = 1000ms;
     }
   }
 
-  std::string get_metadata_cache_section(unsigned metadata_server_port) {
+  std::string get_metadata_cache_section(unsigned metadata_server_port) const {
     return "[metadata_cache:test]\n"
            "router_id=1\n"
            "bootstrap_server_addresses=mysql://localhost:" +
@@ -71,7 +73,7 @@ class RouterRoutingStrategyTest : public RouterComponentTest {
 
   std::string get_static_routing_section(
       unsigned router_port, const std::vector<uint16_t> &destinations,
-      const std::string &strategy, const std::string &mode = "") {
+      const std::string &strategy, const std::string &mode = "") const {
     std::string result =
         "[routing:test_default]\n"
         "bind_port=" +
@@ -96,7 +98,7 @@ class RouterRoutingStrategyTest : public RouterComponentTest {
   // for error scenarios allow empty values
   std::string get_static_routing_section_error(
       unsigned router_port, const std::vector<unsigned> &destinations,
-      const std::string &strategy, const std::string &mode) {
+      const std::string &strategy, const std::string &mode) const {
     std::string result =
         "[routing:test_default]\n"
         "bind_port=" +
@@ -116,10 +118,9 @@ class RouterRoutingStrategyTest : public RouterComponentTest {
     return result;
   }
 
-  std::string get_metadata_cache_routing_section(unsigned router_port,
-                                                 const std::string &role,
-                                                 const std::string &strategy,
-                                                 const std::string &mode = "") {
+  std::string get_metadata_cache_routing_section(
+      unsigned router_port, const std::string &role,
+      const std::string &strategy, const std::string &mode = "") const {
     std::string result =
         "[routing:test_default]\n"
         "bind_port=" +
@@ -205,18 +206,16 @@ class RouterRoutingStrategyTest : public RouterComponentTest {
 
   ProcessWrapper &launch_router_static(const std::string &conf_dir,
                                        const std::string &routing_section,
-                                       bool expect_error = false,
-                                       bool log_to_console = true) {
+                                       bool expect_error = false) {
     auto def_section = get_DEFAULT_defaults();
-    if (log_to_console) {
-      def_section["logging_folder"] = "";
-    }
+
     // launch the router with the static routing configuration
     const std::string conf_file =
         create_config_file(conf_dir, routing_section, &def_section);
     const int expected_exit_code = expect_error ? EXIT_FAILURE : EXIT_SUCCESS;
     auto &router =
-        ProcessManager::launch_router({"-c", conf_file}, expected_exit_code);
+        ProcessManager::launch_router({"-c", conf_file}, expected_exit_code,
+                                      true, false, expect_error ? -1s : 5s);
 
     return router;
   }
@@ -240,9 +239,9 @@ class RouterRoutingStrategyTest : public RouterComponentTest {
   void kill_server(ProcessWrapper *server) { EXPECT_NO_THROW(server->kill()); }
 
   TcpPortPool port_pool_;
-  unsigned wait_for_cache_ready_timeout{1000};
-  unsigned wait_for_static_ready_timeout{100};
-  unsigned wait_for_process_exit_timeout{10000};
+  std::chrono::milliseconds wait_for_cache_ready_timeout{1000};
+  std::chrono::milliseconds wait_for_static_ready_timeout{100};
+  std::chrono::milliseconds wait_for_process_exit_timeout{10000};
 };
 
 struct MetadataCacheTestParams {
@@ -280,7 +279,7 @@ class RouterRoutingStrategyMetadataCache
     : public RouterRoutingStrategyTest,
       public ::testing::WithParamInterface<MetadataCacheTestParams> {
  protected:
-  virtual void SetUp() { RouterRoutingStrategyTest::SetUp(); }
+  void SetUp() override { RouterRoutingStrategyTest::SetUp(); }
 };
 
 ////////////////////////////////////////
@@ -342,15 +341,15 @@ TEST_P(RouterRoutingStrategyMetadataCache, MetadataCacheRoutingStrategy) {
 
   // give the router a chance to initialise metadata-cache module
   // there is currently now easy way to check that
-  SCOPED_TRACE("// waiting " + std::to_string(wait_for_cache_ready_timeout) +
+  SCOPED_TRACE("// waiting " +
+               std::to_string(wait_for_cache_ready_timeout.count()) +
                "ms until metadata is initialized");
   RestMetadataClient::MetadataStatus metadata_status;
   RestMetadataClient rest_metadata_client("127.0.0.1", monitoring_port,
                                           kRestApiUsername, kRestApiPassword);
 
   ASSERT_NO_ERROR(rest_metadata_client.wait_for_cache_ready(
-      std::chrono::milliseconds(wait_for_cache_ready_timeout),
-      metadata_status));
+      wait_for_cache_ready_timeout, metadata_status));
 
   if (!test_params.round_robin) {
     // check if the server nodes are being used in the expected order
@@ -398,7 +397,7 @@ TEST_P(RouterRoutingStrategyMetadataCache, MetadataCacheRoutingStrategy) {
   ASSERT_THAT(router.kill(), testing::Eq(0));
 }
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     MetadataCacheRoutingStrategy, RouterRoutingStrategyMetadataCache,
     // node_id=0 is PRIARY, node_id=1..3 are SECONDARY
     ::testing::Values(
@@ -493,7 +492,7 @@ class RouterRoutingStrategyTestRoundRobin
       public ::testing::WithParamInterface<
           std::pair<std::string, std::string>> {
  protected:
-  virtual void SetUp() { RouterRoutingStrategyTest::SetUp(); }
+  void SetUp() override { RouterRoutingStrategyTest::SetUp(); }
 };
 
 TEST_P(RouterRoutingStrategyTestRoundRobin, StaticRoutingStrategyRoundRobin) {
@@ -542,7 +541,7 @@ TEST_P(RouterRoutingStrategyTestRoundRobin, StaticRoutingStrategyRoundRobin) {
 
 // We expect round robin for routing-strategy=round-robin and as default for
 // read-only
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     StaticRoutingStrategyRoundRobin, RouterRoutingStrategyTestRoundRobin,
     ::testing::Values(
         std::make_pair(std::string("round-robin"), std::string("")),
@@ -556,7 +555,7 @@ class RouterRoutingStrategyTestFirstAvailable
       public ::testing::WithParamInterface<
           std::pair<std::string, std::string>> {
  protected:
-  virtual void SetUp() { RouterRoutingStrategyTest::SetUp(); }
+  void SetUp() override { RouterRoutingStrategyTest::SetUp(); }
 };
 
 TEST_P(RouterRoutingStrategyTestFirstAvailable,
@@ -588,7 +587,7 @@ TEST_P(RouterRoutingStrategyTestFirstAvailable,
   auto &router = launch_router_static(conf_dir.name(), routing_section);
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_port));
 
-  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+  std::this_thread::sleep_for(100ms);
 
   // expect consecutive connections to be done in first-available fashion
   std::string node_port;
@@ -597,32 +596,35 @@ TEST_P(RouterRoutingStrategyTestFirstAvailable,
   connect_client_and_query_port(router_port, node_port);
   EXPECT_EQ(std::to_string(server_ports[0]), node_port);
 
-  // "kill" server 1 and 2, expect moving to server 3
+  SCOPED_TRACE("// 'kill' server 1 and 2, expect moving to server 3");
   kill_server(server_instances[0]);
   kill_server(server_instances[1]);
-  // now we should connect to 3rd server
+  SCOPED_TRACE("// now we should connect to 3rd server");
   connect_client_and_query_port(router_port, node_port);
   EXPECT_EQ(std::to_string(server_ports[2]), node_port);
 
-  // kill also 3rd server
+  SCOPED_TRACE("// kill also 3rd server");
   kill_server(server_instances[2]);
-  // expect connection failure
+  SCOPED_TRACE("// expect connection failure");
   connect_client_and_query_port(router_port, node_port, /*should_fail=*/true);
   EXPECT_EQ("", node_port);
 
-  // bring back 1st server
+  SCOPED_TRACE("// bring back 1st server on port " +
+               std::to_string(server_ports[0]));
   server_instances.emplace_back(
       &launch_standalone_server(server_ports[0], get_data_dir().str()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(
       *server_instances[server_instances.size() - 1], server_ports[0]));
-  // we should now succesfully connect to this server
-  connect_client_and_query_port(router_port, node_port);
+  SCOPED_TRACE("// we should now succesfully connect to server on port " +
+               std::to_string(server_ports[0]));
+  ASSERT_NO_FATAL_FAILURE(
+      connect_client_and_query_port(router_port, node_port));
   EXPECT_EQ(std::to_string(server_ports[0]), node_port);
 }
 
 // We expect first-available for routing-strategy=first-available and as default
 // for read-write
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     StaticRoutingStrategyFirstAvailable,
     RouterRoutingStrategyTestFirstAvailable,
     ::testing::Values(
@@ -634,10 +636,7 @@ INSTANTIATE_TEST_CASE_P(
         std::make_pair(std::string(""), std::string("read-write"))));
 
 // for non-param tests
-class RouterRoutingStrategyStatic : public RouterRoutingStrategyTest {
- protected:
-  virtual void SetUp() { RouterRoutingStrategyTest::SetUp(); }
-};
+class RouterRoutingStrategyStatic : public RouterRoutingStrategyTest {};
 
 TEST_F(RouterRoutingStrategyStatic, StaticRoutingStrategyNextAvailable) {
   TempDirectory temp_test_dir;
@@ -663,7 +662,7 @@ TEST_F(RouterRoutingStrategyStatic, StaticRoutingStrategyNextAvailable) {
   auto &router = launch_router_static(conf_dir.name(), routing_section);
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_port));
 
-  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+  std::this_thread::sleep_for(100ms);
 
   // expect consecutive connections to be done in first-available fashion
   std::string node_port;
@@ -672,25 +671,28 @@ TEST_F(RouterRoutingStrategyStatic, StaticRoutingStrategyNextAvailable) {
   connect_client_and_query_port(router_port, node_port);
   EXPECT_EQ(std::to_string(server_ports[0]), node_port);
 
-  // "kill" server 1 and 2, expect connection to server 3 after that
+  SCOPED_TRACE(
+      "// 'kill' server 1 and 2, expect connection to server 3 after that");
   kill_server(server_instances[0]);
   kill_server(server_instances[1]);
-  // now we should connect to 3rd server
+  SCOPED_TRACE("// now we should connect to 3rd server");
   connect_client_and_query_port(router_port, node_port);
   EXPECT_EQ(std::to_string(server_ports[2]), node_port);
 
-  // kill also 3rd server
+  SCOPED_TRACE("// kill also 3rd server");
   kill_server(server_instances[2]);
-  // expect connection failure
+  SCOPED_TRACE("// expect connection failure");
   connect_client_and_query_port(router_port, node_port, /*should_fail=*/true);
   EXPECT_EQ("", node_port);
 
-  // bring back 1st server
+  SCOPED_TRACE("// bring back 1st server");
   server_instances.emplace_back(
       &launch_standalone_server(server_ports[0], get_data_dir().str()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(
       *server_instances[server_instances.size() - 1], server_ports[0]));
-  // we should NOT connect to this server (in next-available we NEVER go back)
+  SCOPED_TRACE(
+      "// we should NOT connect to this server (in next-available we NEVER go "
+      "back)");
   connect_client_and_query_port(router_port, node_port, /*should_fail=*/true);
   EXPECT_EQ("", node_port);
 }
@@ -710,10 +712,12 @@ TEST_F(RouterRoutingStrategyStatic, InvalidStrategyName) {
 
   check_exit_code(router, EXIT_FAILURE);
   EXPECT_TRUE(
-      router.expect_output("Configuration error: option routing_strategy in "
-                           "[routing:test_default] is invalid; "
-                           "valid are first-available, next-available, and "
-                           "round-robin (was 'round-robin-with-fallback'"));
+      wait_log_contains(router,
+                        "Configuration error: option routing_strategy in "
+                        "\\[routing:test_default\\] is invalid; "
+                        "valid are first-available, next-available, and "
+                        "round-robin \\(was 'round-robin-with-fallback'",
+                        500ms));
 }
 
 TEST_F(RouterRoutingStrategyStatic, InvalidMode) {
@@ -727,9 +731,12 @@ TEST_F(RouterRoutingStrategyStatic, InvalidMode) {
                                       /*expect_error=*/true);
 
   check_exit_code(router, EXIT_FAILURE);
-  EXPECT_TRUE(router.expect_output(
-      "option routing_strategy in [routing:test_default] is invalid; valid are "
-      "first-available, next-available, and round-robin (was 'invalid')"));
+  EXPECT_TRUE(wait_log_contains(
+      router,
+      "option routing_strategy in \\[routing:test_default\\] is invalid; valid "
+      "are "
+      "first-available, next-available, and round-robin \\(was 'invalid'\\)",
+      500ms));
 }
 
 TEST_F(RouterRoutingStrategyStatic, BothStrategyAndModeMissing) {
@@ -744,8 +751,10 @@ TEST_F(RouterRoutingStrategyStatic, BothStrategyAndModeMissing) {
 
   check_exit_code(router, EXIT_FAILURE);
   EXPECT_TRUE(
-      router.expect_output("Configuration error: option routing_strategy in "
-                           "[routing:test_default] is required"));
+      wait_log_contains(router,
+                        "Configuration error: option routing_strategy in "
+                        "\\[routing:test_default\\] is required",
+                        500ms));
 }
 
 TEST_F(RouterRoutingStrategyStatic, RoutingSrtategyEmptyValue) {
@@ -760,8 +769,10 @@ TEST_F(RouterRoutingStrategyStatic, RoutingSrtategyEmptyValue) {
 
   check_exit_code(router, EXIT_FAILURE);
   EXPECT_TRUE(
-      router.expect_output("Configuration error: option routing_strategy in "
-                           "[routing:test_default] needs a value"));
+      wait_log_contains(router,
+                        "Configuration error: option routing_strategy in "
+                        "\\[routing:test_default\\] needs a value",
+                        500ms));
 }
 
 TEST_F(RouterRoutingStrategyStatic, ModeEmptyValue) {
@@ -775,9 +786,10 @@ TEST_F(RouterRoutingStrategyStatic, ModeEmptyValue) {
                                       /*expect_error=*/true);
 
   check_exit_code(router, EXIT_FAILURE);
-  EXPECT_TRUE(
-      router.expect_output("Configuration error: option mode in "
-                           "[routing:test_default] needs a value"));
+  EXPECT_TRUE(wait_log_contains(router,
+                                "Configuration error: option mode in "
+                                "\\[routing:test_default\\] needs a value",
+                                500ms));
 }
 
 int main(int argc, char *argv[]) {
