@@ -1,6 +1,6 @@
 
 /*
-  Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -31,13 +31,15 @@
 
 #include <gmock/gmock.h>
 #ifdef RAPIDJSON_NO_SIZETYPEDEFINE
+// if we build within the server, it will set RAPIDJSON_NO_SIZETYPEDEFINE
+// globally and require to include my_rapidjson_size_t.h
 #include "my_rapidjson_size_t.h"
 #endif
 
 #include <rapidjson/error/en.h>
 
 #include "config_builder.h"
-#include "mysqlrouter/mock_server_rest_client.h"
+#include "mock_server_rest_client.h"
 #include "mysqlrouter/rest_client.h"
 #include "rest_api_testutils.h"
 
@@ -49,65 +51,48 @@ std::string json_to_string(const JsonValue &json_doc) {
   return out_buffer.GetString();
 }
 
-std::vector<GRNode> classic_ports_to_gr_nodes(
-    const std::vector<uint16_t> &classic_ports) {
-  std::vector<GRNode> result;
-  for (const auto &port : classic_ports) {
-    result.emplace_back(port);
-  }
-
-  return result;
-}
-
-std::vector<ClusterNode> classic_ports_to_cluster_nodes(
-    const std::vector<uint16_t> &classic_ports) {
-  std::vector<ClusterNode> result;
-  for (const auto &port : classic_ports) {
-    result.emplace_back(port);
-  }
-
-  return result;
-}
-
 JsonValue mock_GR_metadata_as_json(
-    const std::string &gr_id, const std::vector<GRNode> &gr_nodes,
-    unsigned gr_pos, const std::vector<ClusterNode> &cluster_nodes,
-    unsigned primary_id, uint64_t view_id, bool error_on_md_query,
-    const std::string &gr_node_host) {
+    const std::string &gr_id, const std::vector<uint16_t> &gr_node_ports,
+    unsigned primary_id, unsigned view_id, bool error_on_md_query,
+    const std::string &gr_node_host,
+    const std::vector<uint32_t> &gr_node_xports,
+    const std::vector<std::string> &node_attributes) {
   JsonValue json_doc(rapidjson::kObjectType);
   JsonAllocator allocator;
   json_doc.AddMember(
       "gr_id", JsonValue(gr_id.c_str(), gr_id.length(), allocator), allocator);
 
   JsonValue gr_nodes_json(rapidjson::kArrayType);
-  JsonValue cluster_nodes_json(rapidjson::kArrayType);
-  for (auto &gr_node : gr_nodes) {
+  if (!gr_node_xports.empty() &&
+      gr_node_xports.size() != gr_node_ports.size()) {
+    throw std::runtime_error("gr_node_xports.size() != gr_node_ports.size(), " +
+                             std::to_string(gr_node_xports.size()) +
+                             " != " + std::to_string(gr_node_ports.size()));
+  }
+  size_t i = 0;
+  for (auto &gr_node : gr_node_ports) {
     JsonValue node(rapidjson::kArrayType);
-    node.PushBack(static_cast<int>(gr_node.classic_port), allocator);
-    node.PushBack(JsonValue(gr_node.member_status.c_str(),
-                            gr_node.member_status.length(), allocator),
-                  allocator);
+    node.PushBack(static_cast<int>(gr_node), allocator);
+    node.PushBack(JsonValue("ONLINE", strlen("ONLINE"), allocator), allocator);
+    if (!gr_node_xports.empty())
+      node.PushBack(static_cast<int>(gr_node_xports[i]), allocator);
+    else
+      node.PushBack(0, allocator);
 
+    if (!node_attributes.empty())
+      node.PushBack(JsonValue(node_attributes[i].c_str(),
+                              node_attributes[i].length(), allocator),
+                    allocator);
+    else
+      node.PushBack(JsonValue("{}", 2, allocator), allocator);
+
+    ++i;
     gr_nodes_json.PushBack(node, allocator);
   }
-
-  for (auto &cluster_node : cluster_nodes) {
-    JsonValue node(rapidjson::kArrayType);
-    node.PushBack(static_cast<int>(cluster_node.classic_port), allocator);
-    node.PushBack(static_cast<int>(cluster_node.x_port), allocator);
-    node.PushBack(JsonValue(cluster_node.attributes.c_str(),
-                            cluster_node.attributes.length(), allocator),
-                  allocator);
-
-    cluster_nodes_json.PushBack(node, allocator);
-  }
-
   json_doc.AddMember("gr_nodes", gr_nodes_json, allocator);
-  json_doc.AddMember("gr_pos", gr_pos, allocator);
-  json_doc.AddMember("cluster_nodes", cluster_nodes_json, allocator);
   json_doc.AddMember("primary_id", static_cast<int>(primary_id), allocator);
   if (view_id > 0) {
-    json_doc.AddMember("view_id", view_id, allocator);
+    json_doc.AddMember("view_id", static_cast<int>(view_id), allocator);
   }
   json_doc.AddMember("error_on_md_query", error_on_md_query ? 1 : 0, allocator);
   json_doc.AddMember(
@@ -119,18 +104,18 @@ JsonValue mock_GR_metadata_as_json(
 }
 
 void set_mock_metadata(uint16_t http_port, const std::string &gr_id,
-                       const std::vector<GRNode> &gr_nodes, unsigned gr_pos,
-                       const std::vector<ClusterNode> &cluster_nodes,
-                       unsigned primary_id, uint64_t view_id,
-                       bool error_on_md_query,
-                       const std::string &gr_node_host) {
+                       const std::vector<uint16_t> &gr_node_ports,
+                       unsigned primary_id, unsigned view_id,
+                       bool error_on_md_query, const std::string &gr_node_host,
+                       const std::vector<uint32_t> &gr_node_xports,
+                       const std::vector<std::string> &node_attributes) {
   const auto json_doc = mock_GR_metadata_as_json(
-      gr_id, gr_nodes, gr_pos, cluster_nodes, primary_id, view_id,
-      error_on_md_query, gr_node_host);
+      gr_id, gr_node_ports, primary_id, view_id, error_on_md_query,
+      gr_node_host, gr_node_xports, node_attributes);
 
   const auto json_str = json_to_string(json_doc);
 
-  ASSERT_NO_THROW(MockServerRestClient(http_port).set_globals(json_str));
+  EXPECT_NO_THROW(MockServerRestClient(http_port).set_globals(json_str));
 }
 
 void set_mock_bootstrap_data(
@@ -146,17 +131,13 @@ void set_mock_bootstrap_data(
       allocator);
 
   JsonValue gr_members_json(rapidjson::kArrayType);
-  size_t i{1};
   for (auto &gr_member : gr_members_ports) {
     JsonValue member(rapidjson::kArrayType);
-    const std::string id = "uuid-" + std::to_string(i);
-    member.PushBack(JsonValue(id.c_str(), id.length(), allocator), allocator);
     member.PushBack(
         JsonValue(gr_member.first.c_str(), gr_member.first.length(), allocator),
         allocator);
     member.PushBack(static_cast<int>(gr_member.second), allocator);
     gr_members_json.PushBack(member, allocator);
-    i++;
   }
   JsonValue cluster_instances_json{gr_members_json, allocator};
   json_doc.AddMember("gr_members", gr_members_json, allocator);
