@@ -1,4 +1,4 @@
--- Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+-- Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -22,30 +22,29 @@
 -- 
 -- mysql> select * from x$statement_analysis limit 1\G
 -- *************************** 1. row ***************************
---                query: SELECT * FROM `schema_object_overview` SELECT `information_schema` . `routines`  -- truncated
---                   db: sys
---            full_scan: *
---           exec_count: 2
--- exec_secondary_count: 2
---            err_count: 0
---           warn_count: 0
---        total_latency: 16751388791000
---          max_latency: 16566171163000
---          avg_latency: 8375694395000
---         lock_latency: 16686483000000
---            rows_sent: 84
---        rows_sent_avg: 42
---        rows_examined: 20012
---    rows_examined_avg: 10006
---        rows_affected: 0
---    rows_affected_avg: 0
---           tmp_tables: 378
---      tmp_disk_tables: 66
---          rows_sorted: 168
---    sort_merge_passes: 0
---               digest: 54f9bd520f0bbf15db0c2ed93386bec9
---           first_seen: 2014-03-07 13:13:41
---            last_seen: 2014-03-07 13:13:48
+--             query: SELECT * FROM `schema_object_overview` SELECT `information_schema` . `routines`  -- truncated
+--                db: sys
+--         full_scan: *
+--        exec_count: 2
+--         err_count: 0
+--        warn_count: 0
+--     total_latency: 16751388791000
+--       max_latency: 16566171163000
+--       avg_latency: 8375694395000
+--      lock_latency: 16686483000000
+--         rows_sent: 84
+--     rows_sent_avg: 42
+--     rows_examined: 20012
+-- rows_examined_avg: 10006
+--     rows_affected: 0
+-- rows_affected_avg: 0
+--        tmp_tables: 378
+--   tmp_disk_tables: 66
+--       rows_sorted: 168
+-- sort_merge_passes: 0
+--            digest: 54f9bd520f0bbf15db0c2ed93386bec9
+--        first_seen: 2014-03-07 13:13:41
+--         last_seen: 2014-03-07 13:13:48
 --
 
 CREATE OR REPLACE
@@ -57,14 +56,12 @@ VIEW x$statement_analysis (
   db,
   full_scan,
   exec_count,
-  exec_secondary_count,
   err_count,
   warn_count,
   total_latency,
   max_latency,
   avg_latency,
   lock_latency,
-  cpu_latency,
   rows_sent,
   rows_sent_avg,
   rows_examined,
@@ -75,8 +72,6 @@ VIEW x$statement_analysis (
   tmp_disk_tables,
   rows_sorted,
   sort_merge_passes,
-  max_controlled_memory,
-  max_total_memory,
   digest,
   first_seen,
   last_seen
@@ -85,14 +80,12 @@ SELECT DIGEST_TEXT AS query,
        SCHEMA_NAME AS db,
        IF(SUM_NO_GOOD_INDEX_USED > 0 OR SUM_NO_INDEX_USED > 0, '*', '') AS full_scan,
        COUNT_STAR AS exec_count,
-       COUNT_SECONDARY AS exec_secondary_count,
        SUM_ERRORS AS err_count,
        SUM_WARNINGS AS warn_count,
        SUM_TIMER_WAIT AS total_latency,
        MAX_TIMER_WAIT AS max_latency,
        AVG_TIMER_WAIT AS avg_latency,
        SUM_LOCK_TIME AS lock_latency,
-       SUM_CPU_TIME AS cpu_latency,
        SUM_ROWS_SENT AS rows_sent,
        ROUND(IFNULL(SUM_ROWS_SENT / NULLIF(COUNT_STAR, 0), 0)) AS rows_sent_avg,
        SUM_ROWS_EXAMINED AS rows_examined,
@@ -103,8 +96,6 @@ SELECT DIGEST_TEXT AS query,
        SUM_CREATED_TMP_DISK_TABLES AS tmp_disk_tables,
        SUM_SORT_ROWS AS rows_sorted,
        SUM_SORT_MERGE_PASSES AS sort_merge_passes,
-       MAX_CONTROLLED_MEMORY AS max_controlled_memory,
-       MAX_TOTAL_MEMORY AS max_total_memory,
        DIGEST AS digest,
        FIRST_SEEN AS first_seen,
        LAST_SEEN as last_seen
