@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +28,6 @@
 #include "my_alloc.h"
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/mysql_lex_string.h"
-#include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_internal.h"
 #include "sql/auth/sql_security_ctx.h"
 #include "sql/field.h"
@@ -42,7 +40,6 @@
 namespace consts {
 const std::string mysql("mysql");
 const std::string system_user("SYSTEM_USER");
-const std::string connection_admin("CONNECTION_ADMIN");
 }  // namespace consts
 
 bool User_table_schema_factory::is_old_user_table_schema(TABLE *table) {
@@ -60,7 +57,7 @@ void Auth_id::create_key() {
   m_key.append(m_host.length() ? m_host : "");
 }
 
-Auth_id::Auth_id() = default;
+Auth_id::Auth_id() {}
 
 Auth_id::Auth_id(const char *user, size_t user_len, const char *host,
                  size_t host_len) {
@@ -96,7 +93,7 @@ Auth_id::Auth_id(const Auth_id &id) : m_user(id.m_user), m_host(id.m_host) {
   create_key();
 }
 
-Auth_id::~Auth_id() = default;
+Auth_id::~Auth_id() {}
 
 bool Auth_id::operator<(const Auth_id &id) const { return m_key < id.m_key; }
 
@@ -165,74 +162,10 @@ std::string get_one_priv(ulong &revoke_privs) {
 */
 void set_system_user_flag(THD *thd,
                           bool check_for_main_security_ctx /*= false*/) {
-  assert(thd);
+  DBUG_ASSERT(thd);
   Security_context *sctx = thd->security_context();
   if (check_for_main_security_ctx == false || sctx == &thd->m_main_security_ctx)
     thd->set_system_user(sctx->has_global_grant(consts::system_user.c_str(),
                                                 consts::system_user.length())
                              .first);
 }
-
-/**
-  Set the connection_admin flag in the THD. Probe the security context for the
-  CONNECTION_ADMIN or SUPER dynamic privilege only if it has not been changed
-  from original security context in the THD. If the original security context
-  does not have CONNECTION_ADMIN or SUPER privlege then reset the flag in the
-  THD, otherwise set it.
-
-  @param [in, out]  thd Thead handle
-  @param [in] check_for_main_security_ctx If this flag value is true then we
-                                          toggle value in THD only if current
-                                          security context is same as main
-                                          security context.
-*/
-void set_connection_admin_flag(THD *thd,
-                               bool check_for_main_security_ctx /*= false*/) {
-  assert(thd);
-  Security_context *sctx = thd->security_context();
-  if (check_for_main_security_ctx == false || sctx == &thd->m_main_security_ctx)
-    thd->set_connection_admin(
-        sctx->has_global_grant(consts::connection_admin.c_str(),
-                               consts::connection_admin.length())
-            .first ||
-        sctx->check_access(SUPER_ACL));
-}
-
-/**
-  Decrypt pkt data using RSA private key.
-  @param [in] pkt            Data to decrypt.
-  @param [in] cipher_length  Length of the data.
-  @param [in] plain_text     Buffer to store result.
-  @param [in] plain_text_len size of buffer
-  @param [in] private_key    Private key to use.
-  @return                    Error status.
-    @retval false            Success.
-    @retval true             Failure.
-*/
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-bool decrypt_RSA_private_key(uchar *pkt, int cipher_length,
-                             unsigned char *plain_text, size_t plain_text_len,
-                             EVP_PKEY *private_key) {
-  EVP_PKEY_CTX *key_ctx = EVP_PKEY_CTX_new(private_key, nullptr);
-  if (!key_ctx) return true;
-  if (EVP_PKEY_decrypt_init(key_ctx) <= 0 ||
-      EVP_PKEY_CTX_set_rsa_padding(key_ctx, RSA_PKCS1_OAEP_PADDING) <= 0 ||
-      EVP_PKEY_decrypt(key_ctx, plain_text, &plain_text_len, pkt,
-                       cipher_length) <= 0) {
-    EVP_PKEY_CTX_free(key_ctx);
-    return true;
-  }
-  EVP_PKEY_CTX_free(key_ctx);
-  return false;
-}
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-bool decrypt_RSA_private_key(uchar *pkt, int cipher_length,
-                             unsigned char *plain_text,
-                             size_t plain_text_len [[maybe_unused]],
-                             RSA *private_key) {
-  if (RSA_private_decrypt(cipher_length, pkt, plain_text, private_key,
-                          RSA_PKCS1_OAEP_PADDING) == -1)
-    return true;
-  return false;
-}
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
