@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -47,8 +46,8 @@
 #include "my_macros.h"
 #include "mysql/components/my_service.h"
 #include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/components/services/bits/psi_rwlock_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/components/services/psi_rwlock_bits.h"
 #include "mysql/mysql_lex_string.h"
 #include "mysql/plugin.h"
 #include "mysql/plugin_audit.h"
@@ -65,7 +64,6 @@
 #include "sql/auth/i_sha2_password_common.h"
 #include "sql/auth/sql_auth_cache.h" /* ACL_USER                    */
 #include "sql/auth/sql_authentication.h"
-#include "sql/debug_sync.h"       // DEBUG_SYNC
 #include "sql/protocol_classic.h" /* Protocol_classic            */
 #include "sql/sql_class.h"
 #include "sql/sql_const.h" /* MAX_FIELD_WIDTH             */
@@ -80,7 +78,6 @@ char *caching_sha2_rsa_private_key_path;
 char *caching_sha2_rsa_public_key_path;
 bool caching_sha2_auto_generate_rsa_keys = true;
 Rsa_authentication_keys *g_caching_sha2_rsa_keys = nullptr;
-int caching_sha2_digest_rounds = 0;
 
 namespace sha2_password {
 using std::min;
@@ -184,7 +181,8 @@ static PSI_rwlock_info all_rwlocks[] = {
 */
 
 Caching_sha2_password::Caching_sha2_password(
-    MYSQL_PLUGIN plugin_handle, size_t stored_digest_rounds,
+    MYSQL_PLUGIN plugin_handle,
+    size_t stored_digest_rounds,     /* = DEFAULT_STORED_DIGEST_ROUNDS */
     unsigned int fast_digest_rounds, /* = DEFAULT_FAST_DIGEST_ROUNDS */
     Digest_info digest_type)         /* = Digest_info::SHA256_DIGEST */
     : m_plugin_info(plugin_handle),
@@ -219,10 +217,10 @@ Caching_sha2_password::~Caching_sha2_password() {
     b. Hash iteration count
     c. Expected hash
   2. Use plaintext password, salt and hash iteration count to generate
-     hash.
-  3. Validate generated hash against expected hash.
+     hash
+  3. Validate generated hash against expected hash
 
-  In case of successful authentication, update password cache.
+  In case of successful authentication, update password cache
 
   @param [in] authorization_id   User information
   @param [in] serialized_string        Information retrieved from
@@ -238,11 +236,11 @@ std::pair<bool, bool> Caching_sha2_password::authenticate(
     const std::string &plaintext_password) {
   DBUG_TRACE;
 
-  /* Don't process the password if it is longer than maximum limit. */
+  /* Don't process the password if it is longer than maximum limit */
   if (plaintext_password.length() > CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH)
     return std::make_pair(true, false);
 
-  /* Empty authentication string. */
+  /* Empty authentication string */
   if (!serialized_string[0].length())
     return std::make_pair(plaintext_password.length() ? true : false, false);
 
@@ -274,7 +272,8 @@ std::pair<bool, bool> Caching_sha2_password::authenticate(
     */
 
     if (this->generate_sha2_multi_hash(plaintext_password, random,
-                                       &generated_digest, iterations)) {
+                                       &generated_digest,
+                                       m_stored_digest_rounds)) {
       if (m_plugin_info)
         LogPluginErr(ERROR_LEVEL,
                      ER_SHA_PWD_FAILED_TO_GENERATE_MULTI_ROUND_HASH,
@@ -304,18 +303,18 @@ std::pair<bool, bool> Caching_sha2_password::authenticate(
         return std::make_pair(false, second);
       }
 
-      const rwlock_scoped_lock wrlock(&m_cache_lock, true, __FILE__, __LINE__);
+      rwlock_scoped_lock wrlock(&m_cache_lock, true, __FILE__, __LINE__);
       if (m_cache.add(authorization_id, fast_digest)) {
         sha2_cache_entry stored_digest;
         m_cache.search(authorization_id, stored_digest);
 
-        /* Same digest is already added, so just return. */
+        /* Same digest is already added, so just return */
         if (memcmp(fast_digest.digest_buffer[i], stored_digest.digest_buffer[i],
                    sizeof(fast_digest.digest_buffer[i])) == 0)
           return std::make_pair(false, second);
 
-        /* Update the digest. */
-        const uint retain_index = i ? 0 : 1;
+        /* Update the digest */
+        uint retain_index = i ? 0 : 1;
         memcpy(fast_digest.digest_buffer[retain_index],
                stored_digest.digest_buffer[retain_index],
                sizeof(fast_digest.digest_buffer[retain_index]));
@@ -362,7 +361,7 @@ std::pair<bool, bool> Caching_sha2_password::fast_authenticate(
     return std::make_pair(true, false);
   }
 
-  const rwlock_scoped_lock rdlock(&m_cache_lock, false, __FILE__, __LINE__);
+  rwlock_scoped_lock rdlock(&m_cache_lock, false, __FILE__, __LINE__);
   sha2_cache_entry digest;
 
   if (m_cache.search(authorization_id, digest)) {
@@ -397,7 +396,7 @@ std::pair<bool, bool> Caching_sha2_password::fast_authenticate(
 
 void Caching_sha2_password::remove_cached_entry(
     const std::string authorization_id) {
-  const rwlock_scoped_lock wrlock(&m_cache_lock, true, __FILE__, __LINE__);
+  rwlock_scoped_lock wrlock(&m_cache_lock, true, __FILE__, __LINE__);
   /* It is possible that entry is not present at all, but we don't care */
   (void)m_cache.remove(authorization_id);
 }
@@ -449,7 +448,7 @@ bool Caching_sha2_password::deserialize(const std::string &serialized_string,
     DBUG_PRINT("info", ("Digest string is not in expected format."));
     return true;
   }
-  const std::string digest_type_info =
+  std::string digest_type_info =
       serialized_string.substr(delimiter + 1, DIGEST_INFO_LENGTH);
   if (digest_type_info == "A")
     digest_type = Digest_info::SHA256_DIGEST;
@@ -466,17 +465,17 @@ bool Caching_sha2_password::deserialize(const std::string &serialized_string,
                         "Missing iteration count information."));
     return true;
   }
-  const std::string::size_type delimiter_2 =
+  std::string::size_type delimiter_2 =
       serialized_string.find(DELIMITER, delimiter + 1);
   if (delimiter_2 == std::string::npos || delimiter_2 - delimiter != 4) {
     DBUG_PRINT("info", ("Digest string is not in expected format."
                         "Invalid iteration count information."));
     return true;
   }
-  const std::string iteration_info =
+  std::string iteration_info =
       serialized_string.substr(delimiter + 1, ITERATION_LENGTH);
   iterations =
-      std::min((std::stoul(iteration_info, nullptr, 16)) * ITERATION_MULTIPLIER,
+      std::min((std::stoul(iteration_info, nullptr)) * ITERATION_MULTIPLIER,
                MAX_ITERATIONS);
   if (!iterations) {
     DBUG_PRINT("info", ("Digest string is not in expected format."
@@ -555,13 +554,12 @@ bool Caching_sha2_password::serialize(std::string &serialized_string,
   }
 
   /* Iterations */
-  if (iterations < ITERATION_MULTIPLIER || iterations > MAX_ITERATIONS) {
+  unsigned int iteration_info = iterations / ITERATION_MULTIPLIER;
+  if (!iteration_info || iterations > MAX_ITERATIONS) {
     DBUG_PRINT("info", ("Invalid iteration count information."));
     return true;
   }
-  unsigned int iteration_info = iterations / ITERATION_MULTIPLIER;
-  ss << std::setfill('0') << std::setw(3) << std::uppercase << std::hex
-     << iteration_info << DELIMITER;
+  ss << std::setfill('0') << std::setw(3) << iteration_info << DELIMITER;
   serialized_string = ss.str();
 
   /* Salt */
@@ -602,10 +600,10 @@ bool Caching_sha2_password::generate_fast_digest(
     const std::string &plaintext_password, sha2_cache_entry &digest,
     unsigned int pos) {
   DBUG_TRACE;
-  assert(pos < MAX_PASSWORDS);
+  DBUG_ASSERT(pos < MAX_PASSWORDS);
   SHA256_digest sha256_digest;
   unsigned char digest_buffer[CACHING_SHA2_DIGEST_LENGTH];
-  assert(sizeof(digest.digest_buffer[pos]) == sizeof(digest_buffer));
+  DBUG_ASSERT(sizeof(digest.digest_buffer[pos]) == sizeof(digest_buffer));
 
   if (sha256_digest.update_digest(plaintext_password.c_str(),
                                   plaintext_password.length()) ||
@@ -660,7 +658,7 @@ bool Caching_sha2_password::generate_sha2_multi_hash(const std::string &source,
     case Digest_info::SHA256_DIGEST: {
       char buffer[CRYPT_MAX_PASSWORD_SIZE + 1];
       memset(buffer, 0, sizeof(buffer));
-      assert(source.length() <= CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH);
+      DBUG_ASSERT(source.length() <= CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH);
       my_crypt_genhash(buffer, CRYPT_MAX_PASSWORD_SIZE, source.c_str(),
                        source.length(), random.c_str(), nullptr, &iterations);
 
@@ -673,7 +671,7 @@ bool Caching_sha2_password::generate_sha2_multi_hash(const std::string &source,
       break;
     }
     default:
-      assert(false);
+      DBUG_ASSERT(false);
       return true;
   }
   return false;
@@ -687,14 +685,14 @@ bool Caching_sha2_password::generate_sha2_multi_hash(const std::string &source,
 
 size_t Caching_sha2_password::get_cache_count() {
   DBUG_TRACE;
-  const rwlock_scoped_lock rdlock(&m_cache_lock, false, __FILE__, __LINE__);
+  rwlock_scoped_lock rdlock(&m_cache_lock, false, __FILE__, __LINE__);
   return m_cache.size();
 }
 
 /** Clear the password cache */
 void Caching_sha2_password::clear_cache() {
   DBUG_TRACE;
-  const rwlock_scoped_lock wrlock(&m_cache_lock, true, __FILE__, __LINE__);
+  rwlock_scoped_lock wrlock(&m_cache_lock, true, __FILE__, __LINE__);
   m_cache.clear_cache();
 }
 
@@ -727,6 +725,10 @@ bool Caching_sha2_password::validate_hash(const std::string serialized_string) {
 /** Length of encrypted packet */
 const int MAX_CIPHER_LENGTH = 1024;
 
+/** Default iteration count */
+const int CACHING_SHA2_PASSWORD_ITERATIONS =
+    sha2_password::DEFAULT_STORED_DIGEST_ROUNDS;
+
 /** Caching_sha2_password handle */
 sha2_password::Caching_sha2_password *g_caching_sha2_password = nullptr;
 
@@ -744,7 +746,7 @@ static int my_vio_is_secure(MYSQL_PLUGIN_VIO *vio) {
 
   It is useful when we need to pass the scramble to another plugin.
   Especially in case when old 5.1 client with no CLIENT_PLUGIN_AUTH capability
-  tries to connect to server with default 1FA set to
+  tries to connect to server with default-authentication-plugin set to
   caching_sha2_password
 
   @param vio      Virtual Input-Output interface
@@ -878,7 +880,7 @@ static char perform_full_authentication = '\4';
        if (Client: Connection check) then
          -[#red]-> [Priority#1:\nTCP with TLS OR Socket Or\nShared Memory connection:\nSend password to server] "Server: received password"
        else
-         if (Priority#2:\nPublic key available) then
+         if (Priorty#2:\nPublic key available) then
            -[#yellow]-> [yes] "Client: Public key available"
          else
            if (No\nPriority#3: Should client get\nserver's public key) then
@@ -929,19 +931,14 @@ static int caching_sha2_password_authenticate(MYSQL_PLUGIN_VIO *vio,
   char scramble[SCRAMBLE_LENGTH + 1];
   int cipher_length = 0;
   unsigned char plain_text[MAX_CIPHER_LENGTH + 1];
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-  EVP_PKEY *private_key = nullptr;
-  EVP_PKEY *public_key = nullptr;
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
   RSA *private_key = nullptr;
   RSA *public_key = nullptr;
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
 
   generate_user_salt(scramble, SCRAMBLE_LENGTH + 1);
 
   /*
     Note: The nonce is split into 8 + 12 bytes according to
-    https://dev.mysql.com/doc/dev/mysql-server/latest/page_protocol_connection_phase_packets_protocol_handshake_v10.html
+    http://dev.mysql.com/doc/internals/en/connection-phase-packets.html#packet-Protocol::HandshakeV10
     Native authentication sent 20 bytes + '\0' character = 21 bytes.
     This plugin must do the same to stay consistent with historical behavior
     if it is set to operate as a default plugin.
@@ -1052,7 +1049,7 @@ static int caching_sha2_password_authenticate(MYSQL_PLUGIN_VIO *vio,
       encrypting the password.
     */
     if (pkt_len == 1 && *pkt == request_public_key) {
-      const uint pem_length = static_cast<uint>(
+      uint pem_length = static_cast<uint>(
           strlen(g_caching_sha2_rsa_keys->get_public_key_as_pem()));
       if (vio->write_packet(
               vio,
@@ -1071,9 +1068,8 @@ static int caching_sha2_password_authenticate(MYSQL_PLUGIN_VIO *vio,
     if (pkt_len != cipher_length) return CR_ERROR;
 
     /* Decrypt password */
-    if (decrypt_RSA_private_key(pkt, cipher_length, plain_text,
-                                sizeof(plain_text) - 1, private_key))
-      return CR_ERROR;
+    RSA_private_decrypt(cipher_length, pkt, plain_text, private_key,
+                        RSA_PKCS1_OAEP_PADDING);
 
     plain_text[cipher_length] = '\0';  // safety
     xor_string((char *)plain_text, cipher_length, (char *)scramble,
@@ -1087,13 +1083,13 @@ static int caching_sha2_password_authenticate(MYSQL_PLUGIN_VIO *vio,
   }  // if(!my_vio_is_encrypted())
 
   /* Fetch user authentication_string and extract the password salt */
-  const std::string serialized_string[] = {
+  std::string serialized_string[] = {
       std::string(info->auth_string, info->auth_string_length),
       std::string(info->additional_auth_string_length
                       ? info->additional_auth_string
                       : "",
                   info->additional_auth_string_length)};
-  const std::string plaintext_password((char *)pkt, pkt_len - 1);
+  std::string plaintext_password((char *)pkt, pkt_len - 1);
   std::pair<bool, bool> auth_success = g_caching_sha2_password->authenticate(
       authorization_id, serialized_string, plaintext_password);
   if (auth_success.first) return CR_AUTH_USER_CREDENTIALS;
@@ -1130,7 +1126,7 @@ static int caching_sha2_password_generate(char *outbuf, unsigned int *buflen,
                                           unsigned int inbuflen) {
   DBUG_TRACE;
   std::string digest;
-  const std::string source(inbuf, inbuflen);
+  std::string source(inbuf, inbuflen);
   std::string random;
   std::string serialized_string;
 
@@ -1152,8 +1148,7 @@ static int caching_sha2_password_generate(char *outbuf, unsigned int *buflen,
   random.assign(salt, sha2_password::SALT_LENGTH);
 
   if (g_caching_sha2_password->generate_sha2_multi_hash(
-          source, random, &digest,
-          g_caching_sha2_password->get_digest_rounds()))
+          source, random, &digest, CACHING_SHA2_PASSWORD_ITERATIONS))
     return 1;
 
   if (g_caching_sha2_password->serialize(
@@ -1186,7 +1181,7 @@ static int caching_sha2_password_generate(char *outbuf, unsigned int *buflen,
 static int caching_sha2_password_validate(char *const inbuf,
                                           unsigned int buflen) {
   DBUG_TRACE;
-  const std::string serialized_string(inbuf, buflen);
+  std::string serialized_string(inbuf, buflen);
   if (g_caching_sha2_password->validate_hash(serialized_string)) return 1;
   return 0;
 }
@@ -1202,32 +1197,30 @@ static int caching_sha2_password_validate(char *const inbuf,
   @returns Always returns success (0)
 */
 
-static int caching_sha2_password_salt(const char *password [[maybe_unused]],
-                                      unsigned int password_len
-                                      [[maybe_unused]],
-                                      unsigned char *salt [[maybe_unused]],
-                                      unsigned char *salt_len) {
+static int caching_sha2_password_salt(
+    const char *password MY_ATTRIBUTE((unused)),
+    unsigned int password_len MY_ATTRIBUTE((unused)),
+    unsigned char *salt MY_ATTRIBUTE((unused)), unsigned char *salt_len) {
   DBUG_TRACE;
   *salt_len = 0;
   return 0;
 }
 
 /*
-Initialize caching_sha2_password plugin
+  Initialize caching_sha2_password plugin
 
-@param [in] plugin_ref Plugin structure handle
+  @param [in] plugin_ref Plugin structure handle
 
-@returns Status of plugin initialization
-@retval 0 Success
-@retval 1 Error
+  @returns Status of plugin initialization
+    @retval 0 Success
+    @retval 1 Error
 */
 
 static int caching_sha2_authentication_init(MYSQL_PLUGIN plugin_ref) {
   DBUG_TRACE;
-
   caching_sha2_auth_plugin_ref = plugin_ref;
-  g_caching_sha2_password = new sha2_password::Caching_sha2_password(
-      caching_sha2_auth_plugin_ref, caching_sha2_digest_rounds);
+  g_caching_sha2_password =
+      new sha2_password::Caching_sha2_password(caching_sha2_auth_plugin_ref);
   if (!g_caching_sha2_password) return 1;
 
   return 0;
@@ -1241,7 +1234,8 @@ static int caching_sha2_authentication_init(MYSQL_PLUGIN plugin_ref) {
   @returns Always returns success
 */
 
-static int caching_sha2_authentication_deinit(void *arg [[maybe_unused]]) {
+static int caching_sha2_authentication_deinit(
+    void *arg MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   if (g_caching_sha2_password) {
     delete g_caching_sha2_password;
@@ -1272,16 +1266,16 @@ static int compare_caching_sha2_password_with_hash(
     unsigned long cleartext_length, int *is_error) {
   DBUG_TRACE;
 
-  const std::string serialized_string(hash, hash_length);
-  const std::string plaintext_password(cleartext, cleartext_length);
+  std::string serialized_string(hash, hash_length);
+  std::string plaintext_password(cleartext, cleartext_length);
   std::string random;
   std::string digest;
   std::string generated_digest;
   sha2_password::Digest_info digest_type;
   size_t iterations;
 
-  assert(cleartext_length <=
-         sha2_password::CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH);
+  DBUG_ASSERT(cleartext_length <=
+              sha2_password::CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH);
   if (cleartext_length >
       sha2_password::CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH)
     return -1;
@@ -1299,8 +1293,8 @@ static int compare_caching_sha2_password_with_hash(
   }
 
   *is_error = 0;
-  const int result = memcmp(digest.c_str(), generated_digest.c_str(),
-                            sha2_password::STORED_SHA256_DIGEST_LENGTH);
+  int result = memcmp(digest.c_str(), generated_digest.c_str(),
+                      sha2_password::STORED_SHA256_DIGEST_LENGTH);
 
   return result;
 }
@@ -1313,11 +1307,9 @@ static int compare_caching_sha2_password_with_hash(
   @param [out] var Status variable structure
   @param [in]  buff Value buffer
 */
-static int show_caching_sha2_password_rsa_public_key(MYSQL_THD thd
-                                                     [[maybe_unused]],
-                                                     SHOW_VAR *var,
-                                                     char *buff
-                                                     [[maybe_unused]]) {
+static int show_caching_sha2_password_rsa_public_key(
+    MYSQL_THD thd MY_ATTRIBUTE((unused)), SHOW_VAR *var,
+    char *buff MY_ATTRIBUTE((unused))) {
   var->type = SHOW_CHAR;
   var->value =
       const_cast<char *>(g_caching_sha2_rsa_keys->get_public_key_as_pem());
@@ -1354,23 +1346,10 @@ static MYSQL_SYSVAR_BOOL(
     "at the default location.",
     nullptr, nullptr, true);
 
-static MYSQL_SYSVAR_INT(
-    digest_rounds,               // Name.
-    caching_sha2_digest_rounds,  // Variable.
-    PLUGIN_VAR_READONLY,         // Argument optional for cmd line
-    "Number of SHA2 rounds to be done when storing a password hash onto disk.",
-    nullptr,                                      // Check function.
-    nullptr,                                      // Update function.
-    sha2_password::DEFAULT_STORED_DIGEST_ROUNDS,  // Default value.
-    sha2_password::MIN_STORED_DIGEST_ROUNDS,      // Min value.
-    sha2_password::MAX_STORED_DIGEST_ROUNDS,      // Max value.
-    1                                             // Block size.
-);
-
 /** Array of system variables. Used in plugin declaration. */
 static SYS_VAR *caching_sha2_password_sysvars[] = {
     MYSQL_SYSVAR(private_key_path), MYSQL_SYSVAR(public_key_path),
-    MYSQL_SYSVAR(auto_generate_rsa_keys), MYSQL_SYSVAR(digest_rounds), nullptr};
+    MYSQL_SYSVAR(auto_generate_rsa_keys), nullptr};
 
 /** Array of status variables. Used in plugin declaration. */
 static SHOW_VAR caching_sha2_password_status_variables[] = {
@@ -1395,7 +1374,7 @@ static int sha2_cache_cleaner_notify(MYSQL_THD, mysql_event_class_t event_class,
     const struct mysql_event_authentication *authentication_event =
         (const struct mysql_event_authentication *)event;
 
-    const mysql_event_authentication_subclass_t subclass =
+    mysql_event_authentication_subclass_t subclass =
         authentication_event->event_subclass;
 
     /*
@@ -1412,7 +1391,7 @@ static int sha2_cache_cleaner_notify(MYSQL_THD, mysql_event_class_t event_class,
     if (subclass == MYSQL_AUDIT_AUTHENTICATION_CREDENTIAL_CHANGE ||
         subclass == MYSQL_AUDIT_AUTHENTICATION_AUTHID_RENAME ||
         subclass == MYSQL_AUDIT_AUTHENTICATION_AUTHID_DROP) {
-      assert(
+      DBUG_ASSERT(
           authentication_event->user.str[authentication_event->user.length] ==
           '\0');
       std::string authorization_id;
@@ -1447,13 +1426,13 @@ struct st_mysql_audit sha2_cache_cleaner = {
     }};
 
 /** Init function for sha2_cache_cleaner */
-static int caching_sha2_cache_cleaner_init(MYSQL_PLUGIN plugin_info
-                                           [[maybe_unused]]) {
+static int caching_sha2_cache_cleaner_init(
+    MYSQL_PLUGIN plugin_info MY_ATTRIBUTE((unused))) {
   return 0;
 }
 
 /** Deinit function for sha2_cache_cleaner */
-static int caching_sha2_cache_cleaner_deinit(void *arg [[maybe_unused]]) {
+static int caching_sha2_cache_cleaner_deinit(void *arg MY_ATTRIBUTE((unused))) {
   return 0;
 }
 
@@ -1475,7 +1454,7 @@ mysql_declare_plugin(caching_sha2_password){
     0x0100,                                 /* version (1.0)                 */
     caching_sha2_password_status_variables, /* status variables              */
     caching_sha2_password_sysvars,          /* system variables              */
-    nullptr,                                /* reserved                      */
+    nullptr,                                /* reserverd                     */
     0,                                      /* flags                         */
 },
     {
@@ -1491,6 +1470,6 @@ mysql_declare_plugin(caching_sha2_password){
         0x0100,                            /* version (1.0)                 */
         nullptr,                           /* status variables              */
         nullptr,                           /* system variables              */
-        nullptr,                           /* reserved                      */
+        nullptr,                           /* reserverd                     */
         0                                  /* flags                         */
     } mysql_declare_plugin_end;
