@@ -1,7 +1,7 @@
 #ifndef SQL_CREATE_FIELD_INCLUDED
 #define SQL_CREATE_FIELD_INCLUDED
 
-/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -178,6 +178,9 @@ class Create_field {
   // Whether the field is actually an array of the field's type;
   bool is_array{false};
 
+  LEX_CSTRING m_engine_attribute = EMPTY_CSTR;
+  LEX_CSTRING m_secondary_engine_attribute = EMPTY_CSTR;
+
   Create_field()
       : after(nullptr),
         is_explicit_collation(false),
@@ -213,12 +216,13 @@ class Create_field {
 
   bool init(THD *thd, const char *field_name, enum_field_types type,
             const char *length, const char *decimals, uint type_modifier,
-            Item *default_value, Item *on_update_value, LEX_CSTRING *comment,
-            const char *change, List<String> *interval_list,
-            const CHARSET_INFO *cs, bool has_explicit_collation,
-            uint uint_geom_type, Value_generator *gcol_info,
-            Value_generator *default_val_expr, Nullable<gis::srid_t> srid,
-            dd::Column::enum_hidden_type hidden, bool is_array = false);
+            Item *default_value, Item *on_update_value,
+            const LEX_CSTRING *comment, const char *change,
+            List<String> *interval_list, const CHARSET_INFO *cs,
+            bool has_explicit_collation, uint uint_geom_type,
+            Value_generator *gcol_info, Value_generator *default_val_expr,
+            Nullable<gis::srid_t> srid, dd::Column::enum_hidden_type hidden,
+            bool is_array = false);
 
   ha_storage_media field_storage_type() const {
     return (ha_storage_media)((flags >> FIELD_FLAGS_STORAGE_MEDIA) & 3);
@@ -253,22 +257,23 @@ class Create_field {
 
   /// Whether or not the display width was given explicitly by the user.
   bool m_explicit_display_width{false};
-
-  /// The maximum number of bytes a TINYBLOB can hold.
-  static constexpr size_t TINYBLOB_MAX_SIZE_IN_BYTES{255};
-
-  /// The maximum number of bytes a BLOB can hold.
-  static constexpr size_t BLOB_MAX_SIZE_IN_BYTES{65535};
-
-  /// The maximum number of bytes a MEDIUMBLOB can hold.
-  static constexpr size_t MEDIUMBLOB_MAX_SIZE_IN_BYTES{16777215};
-
-  /// The maximum number of bytes a LONGBLOB can hold.
-  static constexpr size_t LONGBLOB_MAX_SIZE_IN_BYTES{4294967295};
 };
 
 /// @returns whether or not this field is a hidden column that represents a
 ///          functional index.
 bool is_field_for_functional_index(const Create_field *create_field);
 
+/**
+  @retval true  If this column is hidden either in the storage engine
+                or SQL layer. Either way, it is completely hidden from
+                the user.
+  @retval false Otherwise.
+*/
+bool is_hidden_by_system(const Create_field *create_field);
+
+/**
+  @retval true  If this column is hidden by the user.
+  @retval false otherwise.
+*/
+bool is_hidden_by_user(const Create_field *create_field);
 #endif
