@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -35,6 +35,7 @@
 #include "sql/dd/string_type.h"
 
 class THD;
+struct LEX_USER;
 
 namespace dd {
 class Schema;
@@ -494,6 +495,8 @@ class Dictionary_client {
   // Make sure all objects are released.
   ~Dictionary_client();
 
+  MY_COMPILER_DIAGNOSTIC_PUSH()
+  MY_COMPILER_CLANG_WORKAROUND_TPARAM_DOCBUG()
   /**
     Retrieve an object by its object id.
 
@@ -504,6 +507,7 @@ class Dictionary_client {
     @retval       false   No error.
     @retval       true    Error (from handling a cache miss).
   */
+  MY_COMPILER_DIAGNOSTIC_POP()
 
   template <typename T>
   bool acquire(Object_id id, const T **object)
@@ -963,6 +967,21 @@ class Dictionary_client {
   bool fetch_global_components(Const_ptr_vec<T> *coll)
       MY_ATTRIBUTE((warn_unused_result));
 
+  /**
+     Check if a user is referenced as definer by some object of the given type.
+
+     @tparam        T              Type of dictionary objects to check.
+     @param         user           User name, including @ and host.
+     @param   [out] is_definer     True if the user is referenced as definer
+                                   by some object.
+
+     @return      true   Failure (error is reported, is_definer is undefined).
+     @return      false  Success.
+   */
+  template <typename T>
+  bool is_user_definer(const LEX_USER &user, bool *is_definer) const
+      MY_ATTRIBUTE((warn_unused_result));
+
   /**
     Fetch Object ids of all the views referencing base table/ view/ stored
     function name specified in "schema"."name". The views are retrieved
