@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -163,7 +163,7 @@
 
   - Bug#29053560 Increases DD column mysql.tablespaces.name length to 268.
 
-  80017: Current
+  80017: Published in 8.0.17
   ----------------------------------------------------------------------------
   Changes from version 80016:
 
@@ -171,29 +171,82 @@
   - WL#12571 Support fully qualified hostnames longer than 60 characters
     Server metadata table columns size is increased to 255.
 
-  80018: Next DD version number after the previous is public.
+  80021: Published in 8.0.21
   ----------------------------------------------------------------------------
   Changes from version 80017:
+
+  - WL#13341 adds new columns
+      mysql.tables.engine_attribute
+      mysql.tables.secondary_engine_attribute
+      mysql.columns.engine_attribute
+      mysql.columns.secondary_engine_attribute
+      mysql.indexes.engine_attribute
+      mysql.indexes.secondary_engine_attribute
+      mysql.tablespaces.engine_attribute
+
+  80022: Published in 8.0.22
+  ----------------------------------------------------------------------------
+  Changes from version 80021:
+
+  - Bug#31587625: PERFORMANCE DEGRADATION AFTER WL14073: Adds definer index for
+    mysql.{events, routines, tables, triggers}.
+
+  80023: Current.
+  ----------------------------------------------------------------------------
+  Changes from version 80022:
+
+  - WL#10905 adds new hidden type 'USER' to mysql.columns.hidden column.
+  - Bug#31867653 changes the type of mysql.table_partition_values.list_num from
+    TINYINT to SMALLINT.
+
+  80024: Next DD version number after the previous is public.
+  ----------------------------------------------------------------------------
+  Changes from version 80023:
   - No changes, this version number is not active yet.
-
-
-  If a new DD version is published in a MRU, that version may or may not
-  be possible to downgrade to previous MRUs within the same GA. If
-  downgrade is supported, the constant DD_VERSION_MINOR_DOWNGRADE_THRESHOLD
-  should be set to the lowest DD_VERSION that we may downgrade to. If minor
-  downgrade is not supported at all, DD_VERSION_MINOR_DOWNGRADE_THRESHOLD
-  should be set to DD_VERSION.
-*/
+ */
 namespace dd {
 
-static const uint DD_VERSION = 80017;
+static const uint DD_VERSION = 80023;
 static_assert(DD_VERSION <= MYSQL_VERSION_ID,
               "This release can not use a version number from the future");
 
-static const uint DD_VERSION_MINOR_DOWNGRADE_THRESHOLD = 80017;
+/**
+  If a new DD version is published in a MRU, that version may or may not
+  be possible to downgrade to previous MRUs within the same GA. From a
+  technical perspective, we may support downgrade for some types of
+  changes to the DD tables, such as:
+
+  i)   Addition of new attributes to a predefined general purpose option-like
+       field.
+  ii)  Addition of a column at the end of the table definition.
+  iii) Addition of elements at the end of an enumeration column type.
+  iv)  Extension of a VARCHAR field.
+  v)   Addition of an index on a column.
+
+  This means we can support downgrade in terms of being able to open the DD
+  tables and read from them. However, additional considerations are relevant
+  in order to determine whether downgrade should be supported or not, e.g.:
+
+  - For changes like i) and iii): In the older version, will invalid entries
+    just be ignored, or will they lead to a failure?
+  - For changes like iv): In the older version, are there buffer sizes that
+    may be insufficient?
+
+  If downgrade is supported, the constant DD_VERSION_MINOR_DOWNGRADE_THRESHOLD
+  should be set to the lowest DD_VERSION that we may downgrade to. If downgrade
+  is not supported at all, then DD_VERSION_MINOR_DOWNGRADE_THRESHOLD should be
+  set to DD_VERSION.
+
+  It has been decided that the default policy for MySQL 8.0 is not to allow
+  downgrade for any minor release. One of the major reasons for this is that
+  this would lead to a huge amount of possible upgrade/downgrade paths with
+  correspondingly complicated and effort demanding QA. Thus, we set this
+  constant to be equal to DD_VERSION to prohibit downgrade attempts. This
+  decision may be relaxed for future releases.
+*/
+static const uint DD_VERSION_MINOR_DOWNGRADE_THRESHOLD = DD_VERSION;
 static_assert(DD_VERSION_MINOR_DOWNGRADE_THRESHOLD <= MYSQL_VERSION_ID,
               "This release can not use a version number from the future");
-
 }  // namespace dd
 
 #endif /* DD__DD_VERSION_INCLUDED */
