@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,8 +26,6 @@
 #include "sql/dd/impl/types/object_table_definition_impl.h"  // dd::Raw_record
 #include "sql/dd/impl/types/resource_group_impl.h"  // dd::Resource_group_impl
 
-struct CHARSET_INFO;
-
 namespace dd {
 namespace tables {
 
@@ -39,7 +36,7 @@ Resource_groups::Resource_groups() {
                          "id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT");
   m_target_def.add_field(FIELD_RESOURCE_GROUP_NAME, "FIELD_RESOURCE_GROUP_NAME",
                          "resource_group_name VARCHAR(64) NOT NULL COLLATE " +
-                             String_type(name_collation()->m_coll_name));
+                             String_type(name_collation()->name));
   m_target_def.add_field(FIELD_RESOURCE_GROUP_TYPE, "FIELD_RESOURCE_GROUP_TYPE",
                          "resource_group_type enum('SYSTEM', 'USER') NOT NULL");
   m_target_def.add_field(FIELD_RESOURCE_GROUP_ENABLED,
@@ -63,7 +60,7 @@ const Resource_groups &Resource_groups::instance() {
 }
 
 const CHARSET_INFO *Resource_groups::name_collation() {
-  return &my_charset_utf8mb3_general_ci;
+  return &my_charset_utf8_general_ci;
 }
 
 bool Resource_groups::update_object_key(Global_name_key *key,
