@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -144,9 +144,9 @@ bool Partition_index_impl::store_attributes(Raw_record *r) {
 }
 
 ///////////////////////////////////////////////////////////////////////////
-static_assert(
-    Index_partitions::FIELD_TABLESPACE_ID == 4,
-    "Index_partitions definition has changed, review (de)ser memfuns!");
+static_assert(Index_partitions::NUMBER_OF_FIELDS == 5,
+              "Index_partitions definition has changed, check if serialize() "
+              "and deserialize() need to be updated!");
 void Partition_index_impl::serialize(Sdi_wcontext *wctx, Sdi_writer *w) const {
   w->StartObject();
   write_properties(w, m_options, STRING_WITH_LEN("options"));
