@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,7 +23,6 @@
 #include "sql/dd/impl/types/tablespace_impl.h"
 #include "sql/dd/impl/bootstrap/bootstrap_ctx.h"  // DD_bootstrap_ctx
 
-#include <assert.h>
 #include <algorithm>
 #include <atomic>
 #include <functional>
@@ -66,11 +65,11 @@
 #include "sql/handler.h"    // handlerton
 #include "sql/sql_class.h"  // THD
 #include "sql/strfunc.h"    // casedn
-#include "string_with_len.h"
 
+#include "m_string.h"
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"  // ER_*
 
 using dd::tables::Index_partitions;
@@ -100,7 +99,7 @@ Tablespace_impl::Tablespace_impl()
       m_se_private_data(),
       m_files() {} /* purecov: tested */
 
-Tablespace_impl::~Tablespace_impl() = default;
+Tablespace_impl::~Tablespace_impl() {}
 
 ///////////////////////////////////////////////////////////////////////////
 
@@ -178,14 +177,14 @@ bool Tablespace_impl::restore_attributes(const Raw_record &r) {
 ///////////////////////////////////////////////////////////////////////////
 
 bool Tablespace_impl::store_attributes(Raw_record *r) {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   if (my_strcasecmp(system_charset_info, "InnoDB", m_engine.c_str()) == 0) {
     /* Innodb can request for space rename during upgrade when options are not
     upgraded yet. */
-    assert(m_options.exists("encryption") ||
-           bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade());
+    DBUG_ASSERT(m_options.exists("encryption") ||
+                bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade());
   } else {
-    assert(!m_options.exists("encryption"));
+    DBUG_ASSERT(!m_options.exists("encryption"));
   }
 #endif
 
@@ -262,15 +261,15 @@ bool Tablespace_impl::is_empty(THD *thd, bool *empty) const {
   Transaction_ro trx(thd, ISO_READ_COMMITTED);
   trx.otx.register_tables<Abstract_table>();
   Raw_table *table = trx.otx.get_table<Abstract_table>();
-  assert(table);
+  DBUG_ASSERT(table);
 
   std::unique_ptr<Raw_record_set> rs;
   if (trx.otx.open_tables() || table->open_record_set(object_key.get(), rs)) {
-    assert(thd->is_system_thread() || thd->killed || thd->is_error());
+    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
     return true;
   }
 
-  assert(empty);
+  DBUG_ASSERT(empty);
   *empty = (rs->current_record() == nullptr);
 
   return false;
@@ -356,7 +355,7 @@ namespace {
 /* purecov: begin inspected */
 template <typename PT>
 PT &ref(PT *tp) {
-  assert(tp != nullptr);
+  DBUG_ASSERT(tp != nullptr);
   return *tp;
 }
 /* purecov: end */
