@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,7 +23,6 @@
 #include "sql/dd/info_schema/table_stats.h"  // dd::info_schema::*
 
 #include "my_time.h"  // TIME_to_ulonglong_datetime
-#include "mysql/strings/m_ctype.h"
 #include "sql/dd/cache/dictionary_client.h"
 #include "sql/dd/dd.h"          // dd::create_object
 #include "sql/dd/impl/utils.h"  // dd::my_time_t_to_ull_datetime()
@@ -54,16 +53,12 @@ namespace {
     - None of the innodb_read_only, read_only, super_read_only
       or transactional_read_only is ON, OR
 
-    - Not in multi transaction mode.
-      i.e., not in START TRANSACTION or AUTOCOMMIT=0.
-
     - Table is not a partitioned table.
 
     - Table is not a performance schema table.
 
   @param thd           - Thread ID
   @param schema_name   - The schema name.
-  @param partition_name - The partition name
 
   @returns true if we can update the statistics, otherwise false.
 */
@@ -76,7 +71,6 @@ inline bool can_persist_I_S_dynamic_statistics(THD *thd,
   return (thd->variables.information_schema_stats_expiry &&
           !thd->variables.transaction_read_only && !super_read_only &&
           !thd->in_sub_stmt && !read_only && !partition_name &&
-          !thd->in_multi_stmt_transaction_mode() &&
           (strcmp(schema_name, "performance_schema") != 0));
 }
 
@@ -314,7 +308,7 @@ static bool report_error_except_ignore_dup(THD *thd, const char *object_type) {
 namespace dd {
 namespace info_schema {
 
-bool update_table_stats(THD *thd, Table_ref *table) {
+bool update_table_stats(THD *thd, TABLE_LIST *table) {
   // Update the object properties
   HA_CREATE_INFO create_info;
 
@@ -340,7 +334,7 @@ bool update_table_stats(THD *thd, Table_ref *table) {
          report_error_except_ignore_dup(thd, "table");
 }
 
-bool update_index_stats(THD *thd, Table_ref *table) {
+bool update_index_stats(THD *thd, TABLE_LIST *table) {
   // Update the object properties
   TABLE *analyze_table = table->table;
   KEY *key_info = analyze_table->s->key_info;
@@ -432,14 +426,14 @@ ulonglong Table_statistics::get_stat(ha_statistics &stat,
       return (stat.check_time);
 
     default:
-      assert(!"Should not hit here");
+      DBUG_ASSERT(!"Should not hit here");
   }
 
   return 0;
 }
 
 // Read dynamic table statistics from SE by opening the user table
-// provided OR by reading cached statistics from Query_block.
+// provided OR by reading cached statistics from SELECT_LEX.
 ulonglong Table_statistics::read_stat(
     THD *thd, const String &schema_name_ptr, const String &table_name_ptr,
     const String &index_name_ptr, const char *partition_name,
@@ -539,7 +533,7 @@ ulonglong Table_statistics::read_stat_from_SE(
   ulonglong return_value = 0;
 
   DBUG_EXECUTE_IF("information_schema_fetch_table_stats",
-                  assert(strncmp(table_name_ptr.ptr(), "fts", 3)););
+                  DBUG_ASSERT(strncmp(table_name_ptr.ptr(), "fts", 3)););
 
   // No engines implement these statistics retrieval. We always return zero.
   if (stype == enum_table_stats_type::CHECK_TIME ||
@@ -580,7 +574,7 @@ ulonglong Table_statistics::read_stat_from_SE(
         dd::Properties::parse_properties(
             tbl_se_private_data ? tbl_se_private_data : ""));
 
-    assert(tbl_se_private_data_obj.get() && ts_se_private_data_obj.get());
+    DBUG_ASSERT(tbl_se_private_data_obj.get() && ts_se_private_data_obj.get());
 
     //
     // Read statistics from SE
@@ -706,14 +700,14 @@ ulonglong Table_statistics::read_stat_by_open_table(
     goto end;
   }
 
-  if (make_table_list(thd, lex->query_block, db_name_lex_cstr,
+  if (make_table_list(thd, lex->select_lex, db_name_lex_cstr,
                       table_name_lex_cstr)) {
     error = -1;
     goto end;
   }
 
-  Table_ref *table_list;
-  table_list = lex->query_block->get_table_list();
+  TABLE_LIST *table_list;
+  table_list = lex->select_lex->table_list.first;
   table_list->required_type = dd::enum_table_type::BASE_TABLE;
 
   /*
@@ -754,7 +748,7 @@ ulonglong Table_statistics::read_stat_by_open_table(
   lex->sql_command = old_lex->sql_command;
 
   if (open_result) {
-    assert(thd->is_error() || thd->is_killed());
+    DBUG_ASSERT(thd->is_error() || thd->is_killed());
 
     if (thd->is_error()) {
       /*
@@ -791,7 +785,7 @@ ulonglong Table_statistics::read_stat_by_open_table(
         table_list->table->file->get_partition_handler();
     if (partition_name && part_handler) {
       partition_info *part_info = table_list->table->part_info;
-      assert(part_info);
+      DBUG_ASSERT(part_info);
 
       uint part_id;
       if (part_info->get_part_elem(partition_name, &part_id) &&
@@ -888,13 +882,13 @@ ulonglong Table_statistics::read_stat_by_open_table(
   }
 
 end:
-  lex->cleanup(true);
+  lex->cleanup(thd, true);
 
   /* Restore original LEX value, statement's arena and THD arena values. */
   lex_end(thd->lex);
 
   // Free items, before restoring backup_arena below.
-  assert(i_s_arena.item_list() == nullptr);
+  DBUG_ASSERT(i_s_arena.item_list() == nullptr);
   thd->free_items();
 
   /*
