@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -55,7 +55,7 @@ class Partition : virtual public Entity_object {
   typedef tables::Table_partitions DD_table;
 
  public:
-  virtual ~Partition() {}
+  ~Partition() override {}
 
   /////////////////////////////////////////////////////////////////////////
   // Table.
