@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2009, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -356,16 +356,9 @@
 #include <time.h>
 #include <algorithm>
 #include <atomic>
-#include <boost/algorithm/string/classification.hpp>
-#include <boost/algorithm/string/split.hpp>
-#include <boost/algorithm/string/trim.hpp>
-#include <boost/concept/usage.hpp>
-#include <boost/iterator/iterator_facade.hpp>
-#include <boost/type_index/type_index_facade.hpp>
 #include <memory>
 #include <vector>
 
-#include "boost/algorithm/string/detail/classification.hpp"
 #include "m_ctype.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
@@ -375,6 +368,7 @@
 #include "my_sys.h"
 #include "my_systime.h"
 #include "my_thread.h"
+#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/mysql_cond_bits.h"
 #include "mysql/components/services/mysql_mutex_bits.h"
@@ -384,12 +378,12 @@
 #include "mysql/plugin.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_mutex.h"
-#include "mysql/psi/psi_base.h"
 #include "mysql/service_mysql_alloc.h"
 #include "mysqld_error.h"
 #include "sql/sql_error.h"
 #include "sql/thr_malloc.h"
 #include "sql_string.h"
+#include "template_utils.h"
 #include "thr_mutex.h"
 
 #if defined(ENABLED_DEBUG_SYNC)
@@ -677,7 +671,7 @@ void debug_sync_init_thread(THD *thd) {
   }
 }
 
-void debug_sync_claim_memory_ownership(THD *thd) {
+void debug_sync_claim_memory_ownership(THD *thd, bool claim) {
   DBUG_TRACE;
   DBUG_ASSERT(thd);
 
@@ -688,14 +682,14 @@ void debug_sync_claim_memory_ownership(THD *thd) {
       st_debug_sync_action *action = ds_control->ds_action;
       st_debug_sync_action *action_end = action + ds_control->ds_allocated;
       for (; action < action_end; action++) {
-        action->signal.mem_claim();
-        action->wait_for.mem_claim();
-        action->sync_point.mem_claim();
+        action->signal.mem_claim(claim);
+        action->wait_for.mem_claim(claim);
+        action->sync_point.mem_claim(claim);
       }
-      my_claim(ds_control->ds_action);
+      my_claim(ds_control->ds_action, claim);
     }
 
-    my_claim(ds_control);
+    my_claim(ds_control, claim);
   }
 }
 
@@ -1797,15 +1791,16 @@ static void debug_sync_execute(THD *thd, st_debug_sync_action *action) {
 
     if (action->signal.length()) {
       std::string signal = action->signal.ptr();
-      std::vector<std::string> signals;
-      boost::split(signals, signal, boost::is_any_of(","));
-      for (std::vector<std::string>::const_iterator it = signals.begin();
-           it != signals.end(); ++it) {
-        /* Copy the signal to the global set. */
-        std::string s = *it;
-        boost::trim(s);
-        if (!s.empty()) add_signal_event(&s);
-      }
+      myu::Split(
+          signal.begin(), signal.end(), myu::IsComma,
+          [](std::string::const_iterator f, std::string::const_iterator l) {
+            auto tr = myu::FindTrimmedRange(f, l, myu::IsSpace);
+            if (tr.first != tr.second) {
+              std::string s{tr.first, tr.second};
+              add_signal_event(&s);
+            }
+          });
+
       /* Wake threads waiting in a sync point. */
       mysql_cond_broadcast(&debug_sync_global.ds_cond);
       DBUG_PRINT("debug_sync_exec",
