@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,6 @@
 
 #include "sql/default_values.h"
 
-#include <assert.h>
 #include <string.h>
 #include <sys/types.h>
 #include <algorithm>
@@ -31,11 +29,10 @@
 #include "my_alloc.h"
 #include "my_base.h"
 #include "my_compare.h"
-
+#include "my_dbug.h"
 #include "my_macros.h"
 #include "my_pointer_arithmetic.h"
 #include "my_sys.h"
-#include "mysql/strings/dtoa.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "sql/create_field.h"
@@ -46,10 +43,11 @@
 #include "sql/dd_table_share.h"   // dd_get_old_field_type
 #include "sql/field.h"            // calc_pack_length
 #include "sql/gis/srid.h"
-#include "sql/handler.h"    // handler
-#include "sql/item.h"       // Item
-#include "sql/sql_class.h"  // THD
-#include "sql/sql_list.h"   // List
+#include "sql/handler.h"     // handler
+#include "sql/item.h"        // Item
+#include "sql/my_decimal.h"  // DECIMAL_MAX_SCALE
+#include "sql/sql_class.h"   // THD
+#include "sql/sql_list.h"    // List
 #include "sql/table.h"
 
 /**
@@ -69,13 +67,13 @@
 
 size_t column_pack_length(const dd::Column &col_obj) {
   // Arrays always use JSON as storage
-  const dd::enum_column_types col_type =
+  dd::enum_column_types col_type =
       col_obj.is_array() ? dd::enum_column_types::JSON : col_obj.type();
   bool treat_bit_as_char = false;
 
   if (col_obj.type() == dd::enum_column_types::BIT) {
     if (col_obj.options().get("treat_bit_as_char", &treat_bit_as_char))
-      assert(false); /* purecov: deadcode */
+      DBUG_ASSERT(false); /* purecov: deadcode */
   }
   return calc_pack_length(col_type, col_obj.char_length(),
                           col_obj.elements_count(), treat_bit_as_char,
@@ -104,7 +102,7 @@ static bool find_record_length(const dd::Table &table, size_t min_length,
   bool pack_record;
   if (table.options().get("pack_record", &pack_record)) return true;
 
-  assert(share);
+  DBUG_ASSERT(share);
   share->fields = 0;
   share->null_fields = 0;
   share->reclength = 0;
@@ -165,7 +163,7 @@ static bool find_record_length(const dd::Table &table, size_t min_length,
 static void set_pack_record_and_unused_preamble_bits(bool pack_record,
                                                      ulong preamble_bits,
                                                      uchar *default_values) {
-  assert(default_values);
+  DBUG_ASSERT(default_values);
 
   // Set first bit if the HA_OPTION_PACK_RECORD is not set.
   if (!pack_record) *default_values |= 1;
@@ -194,6 +192,7 @@ bool prepare_default_value(THD *thd, uchar *buf, TABLE *table,
   bool retval = true;
   if (!regfield) goto err;
 
+  // save_in_field() will access regfield->table->in_use.
   regfield->init(table);
 
   // Set if the field may be NULL.
@@ -201,8 +200,8 @@ bool prepare_default_value(THD *thd, uchar *buf, TABLE *table,
 
   if (field.constant_default) {
     // Pointless to store the value of a function as it may not be constant.
-    assert(field.constant_default->type() != Item::FUNC_ITEM);
-    const type_conversion_status res =
+    DBUG_ASSERT(field.constant_default->type() != Item::FUNC_ITEM);
+    type_conversion_status res =
         field.constant_default->save_in_field(regfield, true);
     if (res != TYPE_OK && res != TYPE_NOTE_TIME_TRUNCATED &&
         res != TYPE_NOTE_TRUNCATED) {
@@ -212,12 +211,6 @@ bool prepare_default_value(THD *thd, uchar *buf, TABLE *table,
       my_error(ER_INVALID_DEFAULT, MYF(0), regfield->field_name);
       goto err;
     }
-    if (res != TYPE_OK && thd->is_error()) {
-      // Conversion errors may be reported as errors or warnings depending on
-      // user-configurable settings.
-      goto err;
-    }
-    assert(!thd->is_error());
   } else if (regfield->real_type() == MYSQL_TYPE_ENUM &&
              (field.flags & NOT_NULL_FLAG)) {
     regfield->set_notnull();
@@ -256,14 +249,14 @@ bool prepare_default_value(THD *thd, uchar *buf, TABLE *table,
 err:
   // Destroy the field, despite being MEM_ROOT allocated, to avoid memory
   // leak for fields that allocate extra memory (e.g Field_blob::value).
-  if (regfield != nullptr) ::destroy_at(regfield);
+  destroy(regfield);
   return retval;
 }
 
 bool prepare_default_value_buffer_and_table_share(THD *thd,
                                                   const dd::Table &table,
                                                   TABLE_SHARE *share) {
-  assert(share);
+  DBUG_ASSERT(share);
 
   // Get the handler temporarily, needed to get minimal record length as
   // well as extra record length.
@@ -278,10 +271,10 @@ bool prepare_default_value_buffer_and_table_share(THD *thd,
   }
 
   // Get the minimal and extra record buffer lengths from the handler.
-  const size_t extra_length = file->extra_rec_buf_length();
-  const size_t min_length =
+  size_t extra_length = file->extra_rec_buf_length();
+  size_t min_length =
       static_cast<size_t>(file->min_record_length(share->db_create_options));
-  ::destroy_at(file);
+  destroy(file);
 
   // Get the number of columns, record length etc.
   if (find_record_length(table, min_length, share)) return true;
