@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,14 +24,15 @@
 #ifndef ITEM_STRFUNC_INCLUDED
 #define ITEM_STRFUNC_INCLUDED
 
-#include <stddef.h>
 #include <sys/types.h>
-#include <algorithm>
+
+#include <cstdint>  // uint32_t
 
 #include "lex_string.h"
-#include "libbinlogevents/include/control_events.h"
+#include "libbinlogevents/include/uuid.h"  // Uuid
 #include "m_ctype.h"
 #include "my_dbug.h"
+#include "my_hostname.h"  // HOSTNAME_LENGTH
 #include "my_inttypes.h"
 #include "my_table_map.h"
 #include "my_time.h"
@@ -41,17 +42,19 @@
 #include "sql/enum_query_type.h"
 #include "sql/field.h"
 #include "sql/item.h"
-#include "sql/item_cmpfunc.h"  // Item_bool_func
-#include "sql/item_func.h"     // Item_func
-#include "sql/parse_tree_node_base.h"
+#include "sql/item_cmpfunc.h"    // Item_bool_func
+#include "sql/item_func.h"       // Item_func
+#include "sql/parse_location.h"  // POS
 #include "sql/sql_const.h"
-#include "sql/sql_digest.h"  // DIGEST_HASH_TO_STRING[_LENGTH]
 #include "sql_string.h"
+#include "template_utils.h"  // pointer_cast
 
 class MY_LOCALE;
 class PT_item_list;
 class THD;
 class my_decimal;
+struct Parse_context;
+
 template <class T>
 class List;
 
@@ -72,64 +75,41 @@ class Item_str_func : public Item_func {
   typedef Item_func super;
 
  public:
-  Item_str_func() : Item_func() { set_data_type_string_init(); }
+  Item_str_func() : Item_func() {}
 
-  explicit Item_str_func(const POS &pos) : super(pos) {
-    set_data_type_string_init();
-  }
+  explicit Item_str_func(const POS &pos) : super(pos) {}
 
-  Item_str_func(Item *a) : Item_func(a) { set_data_type_string_init(); }
+  Item_str_func(Item *a) : Item_func(a) {}
 
-  Item_str_func(const POS &pos, Item *a) : Item_func(pos, a) {
-    set_data_type_string_init();
-  }
+  Item_str_func(const POS &pos, Item *a) : Item_func(pos, a) {}
 
-  Item_str_func(Item *a, Item *b) : Item_func(a, b) {
-    set_data_type_string_init();
-  }
+  Item_str_func(Item *a, Item *b) : Item_func(a, b) {}
 
-  Item_str_func(const POS &pos, Item *a, Item *b) : Item_func(pos, a, b) {
-    set_data_type_string_init();
-  }
+  Item_str_func(const POS &pos, Item *a, Item *b) : Item_func(pos, a, b) {}
+
+  Item_str_func(Item *a, Item *b, Item *c) : Item_func(a, b, c) {}
 
-  Item_str_func(Item *a, Item *b, Item *c) : Item_func(a, b, c) {
-    set_data_type_string_init();
-  }
   Item_str_func(const POS &pos, Item *a, Item *b, Item *c)
-      : Item_func(pos, a, b, c) {
-    set_data_type_string_init();
-  }
+      : Item_func(pos, a, b, c) {}
 
-  Item_str_func(Item *a, Item *b, Item *c, Item *d) : Item_func(a, b, c, d) {
-    set_data_type_string_init();
-  }
+  Item_str_func(Item *a, Item *b, Item *c, Item *d) : Item_func(a, b, c, d) {}
 
   Item_str_func(const POS &pos, Item *a, Item *b, Item *c, Item *d)
-      : Item_func(pos, a, b, c, d) {
-    set_data_type_string_init();
-  }
+      : Item_func(pos, a, b, c, d) {}
 
   Item_str_func(Item *a, Item *b, Item *c, Item *d, Item *e)
-      : Item_func(a, b, c, d, e) {
-    set_data_type_string_init();
-  }
+      : Item_func(a, b, c, d, e) {}
 
   Item_str_func(const POS &pos, Item *a, Item *b, Item *c, Item *d, Item *e)
-      : Item_func(pos, a, b, c, d, e) {
-    set_data_type_string_init();
-  }
+      : Item_func(pos, a, b, c, d, e) {}
 
-  Item_str_func(List<Item> &list) : Item_func(list) {
-    set_data_type_string_init();
-  }
+  explicit Item_str_func(mem_root_deque<Item *> *list) : Item_func(list) {}
 
   Item_str_func(const POS &pos, PT_item_list *opt_list)
-      : Item_func(pos, opt_list) {
-    set_data_type_string_init();
-  }
+      : Item_func(pos, opt_list) {}
 
-  longlong val_int() override;
-  double val_real() override;
+  longlong val_int() override { return val_int_from_string(); }
+  double val_real() override { return val_real_from_string(); }
   my_decimal *val_decimal(my_decimal *) override;
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override {
     return get_date_from_string(ltime, fuzzydate);
@@ -140,6 +120,10 @@ class Item_str_func : public Item_func {
   enum Item_result result_type() const override { return STRING_RESULT; }
   void left_right_max_length();
   bool fix_fields(THD *thd, Item **ref) override;
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1)) return true;
+    return false;
+  }
   String *val_str_from_val_str_ascii(String *str, String *str2);
 
  protected:
@@ -187,7 +171,7 @@ class Item_str_ascii_func : public Item_str_func {
   String *val_str(String *str) override {
     return val_str_from_val_str_ascii(str, &ascii_buf);
   }
-  virtual String *val_str_ascii(String *) override = 0;
+  String *val_str_ascii(String *) override = 0;
 };
 
 class Item_func_md5 final : public Item_str_ascii_func {
@@ -349,7 +333,8 @@ class Item_func_concat : public Item_str_func {
 class Item_func_concat_ws : public Item_str_func {
   String tmp_value{"", 0, collation.collation};  // Initialize to empty
  public:
-  Item_func_concat_ws(List<Item> &list) : Item_str_func(list) {
+  explicit Item_func_concat_ws(mem_root_deque<Item *> *list)
+      : Item_str_func(list) {
     null_on_null = false;
   }
   Item_func_concat_ws(const POS &pos, PT_item_list *opt_list)
@@ -606,7 +591,7 @@ class Item_func_database : public Item_func_sysconst {
 
   String *val_str(String *) override;
   bool resolve_type(THD *) override {
-    set_data_type_string(uint32(MAX_FIELD_NAME));
+    set_data_type_string(uint32{MAX_FIELD_NAME});
     maybe_null = true;
     return false;
   }
@@ -620,10 +605,12 @@ class Item_func_user : public Item_func_sysconst {
   typedef Item_func_sysconst super;
 
  protected:
-  bool init(const char *user, const char *host);
-  type_conversion_status save_in_field_inner(Field *field, bool) override {
-    return save_str_value_in_field(field, &str_value);
-  }
+  /// True when function value is evaluated, set to false after each execution
+  bool m_evaluated = false;
+
+  /// Evaluate user name, must be called once per execution
+  bool evaluate(const char *user, const char *host);
+  type_conversion_status save_in_field_inner(Field *field, bool) override;
 
  public:
   Item_func_user() { str_value.set("", 0, system_charset_info); }
@@ -631,13 +618,12 @@ class Item_func_user : public Item_func_sysconst {
     str_value.set("", 0, system_charset_info);
   }
 
+  table_map get_initial_pseudo_tables() const override {
+    return INNER_TABLE_BIT;
+  }
+
   bool itemize(Parse_context *pc, Item **res) override;
 
-  String *val_str(String *) override {
-    DBUG_ASSERT(fixed == 1);
-    return (null_value ? nullptr : &str_value);
-  }
-  bool fix_fields(THD *thd, Item **ref) override;
   bool check_function_as_value_generator(uchar *checker_args) override {
     Check_function_as_value_generator_parameters *func_arg =
         pointer_cast<Check_function_as_value_generator_parameters *>(
@@ -646,13 +632,20 @@ class Item_func_user : public Item_func_sysconst {
     return true;
   }
   bool resolve_type(THD *) override {
-    set_data_type_string(uint32(USERNAME_CHAR_LENGTH + HOSTNAME_LENGTH + 1U));
+    set_data_type_string(uint32{USERNAME_CHAR_LENGTH + HOSTNAME_LENGTH + 1U});
     return false;
   }
+  void cleanup() override {
+    m_evaluated = false;
+    str_value.mem_free();
+    super::cleanup();
+  }
   const char *func_name() const override { return "user"; }
   const Name_string fully_qualified_func_name() const override {
     return NAME_STRING("user()");
   }
+
+  String *val_str(String *) override;
 };
 
 class Item_func_current_user : public Item_func_user {
@@ -660,16 +653,19 @@ class Item_func_current_user : public Item_func_user {
 
   Name_resolution_context *context;
 
+ protected:
+  type_conversion_status save_in_field_inner(Field *field, bool) override;
+
  public:
   explicit Item_func_current_user(const POS &pos) : super(pos) {}
 
   bool itemize(Parse_context *pc, Item **res) override;
-
-  bool fix_fields(THD *thd, Item **ref) override;
   const char *func_name() const override { return "current_user"; }
   const Name_string fully_qualified_func_name() const override {
     return NAME_STRING("current_user()");
   }
+
+  String *val_str(String *) override;
 };
 
 class Item_func_soundex : public Item_str_func {
@@ -714,7 +710,7 @@ class Item_func_make_set final : public Item_str_func {
     return res;
   }
   void split_sum_func(THD *thd, Ref_item_array ref_item_array,
-                      List<Item> &fields) override;
+                      mem_root_deque<Item *> *fields) override;
   bool resolve_type(THD *) override;
   void update_used_tables() override;
   const char *func_name() const override { return "make_set"; }
@@ -762,7 +758,8 @@ class Item_func_char final : public Item_str_func {
     collation.set(cs);
   }
   String *val_str(String *) override;
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_LONGLONG)) return true;
     set_data_type_string(arg_count * 4U);
     return false;
   }
@@ -870,7 +867,8 @@ class Item_func_hex : public Item_str_ascii_func {
   Item_func_hex(const POS &pos, Item *a) : Item_str_ascii_func(pos, a) {}
   const char *func_name() const override { return "hex"; }
   String *val_str_ascii(String *) override;
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1)) return true;
     set_data_type_string(args[0]->max_length * 2U, default_charset());
     return false;
   }
@@ -886,7 +884,8 @@ class Item_func_unhex final : public Item_str_func {
   }
   const char *func_name() const override { return "unhex"; }
   String *val_str(String *) override;
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1)) return true;
     set_data_type_string((1U + args[0]->max_length) / 2U, &my_charset_bin);
     return false;
   }
@@ -905,8 +904,10 @@ class Item_func_like_range : public Item_str_func {
     maybe_null = true;
   }
   String *val_str(String *) override;
-  bool resolve_type(THD *) override {
-    set_data_type_string(uint32(MAX_BLOB_WIDTH), args[0]->collation);
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1)) return true;
+    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_LONGLONG)) return true;
+    set_data_type_string(uint32{MAX_BLOB_WIDTH}, args[0]->collation);
     return false;
   }
 };
@@ -958,7 +959,11 @@ class Item_load_file final : public Item_str_func {
   bool itemize(Parse_context *pc, Item **res) override;
   String *val_str(String *) override;
   const char *func_name() const override { return "load_file"; }
-  bool resolve_type(THD *) override {
+  table_map get_initial_pseudo_tables() const override {
+    return INNER_TABLE_BIT;
+  }
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1)) return true;
     collation.set(&my_charset_bin, DERIVATION_COERCIBLE);
     set_data_type_blob(MAX_BLOB_WIDTH);
     maybe_null = true;
@@ -1079,10 +1084,10 @@ class Item_func_charset final : public Item_str_func {
   }
   String *val_str(String *) override;
   const char *func_name() const override { return "charset"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
     set_data_type_string(64U, system_charset_info);
     maybe_null = false;
-    return false;
+    return Item_str_func::resolve_type(thd);
   }
 };
 
@@ -1093,7 +1098,8 @@ class Item_func_collation : public Item_str_func {
   }
   String *val_str(String *) override;
   const char *func_name() const override { return "collation"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (Item_str_func::resolve_type(thd)) return true;
     set_data_type_string(64U, system_charset_info);
     maybe_null = false;
     return false;
@@ -1139,7 +1145,8 @@ class Item_func_crc32 final : public Item_int_func {
     unsigned_flag = true;
   }
   const char *func_name() const override { return "crc32"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1)) return true;
     max_length = 10;
     return false;
   }
@@ -1153,7 +1160,8 @@ class Item_func_uncompressed_length final : public Item_int_func {
   Item_func_uncompressed_length(const POS &pos, Item *a)
       : Item_int_func(pos, a) {}
   const char *func_name() const override { return "uncompressed_length"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1)) return true;
     max_length = 10;
     return false;
   }
@@ -1165,7 +1173,8 @@ class Item_func_compress final : public Item_str_func {
 
  public:
   Item_func_compress(const POS &pos, Item *a) : Item_str_func(pos, a) {}
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (Item_str_func::resolve_type(thd)) return true;
     set_data_type_string((args[0]->max_length * 120U) / 100U + 12U);
     return false;
   }
@@ -1178,9 +1187,10 @@ class Item_func_uncompress final : public Item_str_func {
 
  public:
   Item_func_uncompress(const POS &pos, Item *a) : Item_str_func(pos, a) {}
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (Item_str_func::resolve_type(thd)) return true;
     maybe_null = true;
-    set_data_type_string(uint32(MAX_BLOB_WIDTH));
+    set_data_type_string(uint32{MAX_BLOB_WIDTH});
     return false;
   }
   const char *func_name() const override { return "uncompress"; }
@@ -1302,7 +1312,7 @@ class Item_func_get_dd_column_privileges final : public Item_str_func {
       per privileges is 11 chars.
       So, setting max approximate to 200.
     */
-    set_data_type_string(14 * 11, system_charset_info);
+    set_data_type_string(14U * 11U, system_charset_info);
     maybe_null = true;
     null_on_null = false;
 
@@ -1323,7 +1333,7 @@ class Item_func_get_dd_create_options final : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256, system_charset_info);
+    set_data_type_string(256U, system_charset_info);
     maybe_null = false;
     null_on_null = false;
 
@@ -1335,6 +1345,27 @@ class Item_func_get_dd_create_options final : public Item_str_func {
   String *val_str(String *) override;
 };
 
+class Item_func_get_dd_schema_options final : public Item_str_func {
+ public:
+  Item_func_get_dd_schema_options(const POS &pos, Item *a)
+      : Item_str_func(pos, a) {}
+
+  enum Functype functype() const override { return DD_INTERNAL_FUNC; }
+  bool resolve_type(THD *) override {
+    // maximum string length of all options is expected
+    // to be less than 256 characters.
+    set_data_type_string(256, system_charset_info);
+    maybe_null = false;
+    null_on_null = false;
+
+    return false;
+  }
+
+  const char *func_name() const override { return "get_dd_schema_options"; }
+
+  String *val_str(String *) override;
+};
+
 class Item_func_internal_get_comment_or_error final : public Item_str_func {
  public:
   Item_func_internal_get_comment_or_error(const POS &pos, PT_item_list *list)
@@ -1347,7 +1378,7 @@ class Item_func_internal_get_comment_or_error final : public Item_str_func {
       which is same as size of column holding comments in dictionary,
       i.e., the mysql.tables.comment DD column.
     */
-    set_data_type_string(2048, system_charset_info);
+    set_data_type_string(2048U, system_charset_info);
     maybe_null = true;
     null_on_null = false;
 
@@ -1370,7 +1401,7 @@ class Item_func_get_dd_tablespace_private_data final : public Item_str_func {
   bool resolve_type(THD *) override {
     /* maximum string length of the property value is expected
     to be less than 256 characters. */
-    max_length = 256;
+    set_data_type_string(256U);
     maybe_null = false;
     null_on_null = false;
 
@@ -1393,7 +1424,7 @@ class Item_func_get_dd_index_private_data final : public Item_str_func {
   bool resolve_type(THD *) override {
     /* maximum string length of the property value is expected
     to be less than 256 characters. */
-    max_length = 256;
+    set_data_type_string(256U);
     maybe_null = false;
     null_on_null = false;
 
@@ -1414,7 +1445,7 @@ class Item_func_get_partition_nodegroup final : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256, system_charset_info);
+    set_data_type_string(256U, system_charset_info);
     maybe_null = true;
     null_on_null = false;
 
@@ -1438,7 +1469,7 @@ class Item_func_internal_tablespace_type : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256, system_charset_info);
+    set_data_type_string(256U, system_charset_info);
     maybe_null = true;
     null_on_null = false;
 
@@ -1460,7 +1491,7 @@ class Item_func_internal_tablespace_logfile_group_name : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256, system_charset_info);
+    set_data_type_string(256U, system_charset_info);
     maybe_null = true;
     null_on_null = false;
 
@@ -1484,7 +1515,7 @@ class Item_func_internal_tablespace_status : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256, system_charset_info);
+    set_data_type_string(256U, system_charset_info);
     maybe_null = true;
     null_on_null = false;
 
@@ -1507,7 +1538,7 @@ class Item_func_internal_tablespace_row_format : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256, system_charset_info);
+    set_data_type_string(256U, system_charset_info);
     maybe_null = true;
     null_on_null = false;
 
@@ -1531,7 +1562,7 @@ class Item_func_internal_tablespace_extra : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256, system_charset_info);
+    set_data_type_string(256U, system_charset_info);
     maybe_null = true;
     null_on_null = false;
 
@@ -1549,9 +1580,8 @@ class Item_func_convert_cpu_id_mask final : public Item_str_func {
       : Item_str_func(pos, list) {}
 
   bool resolve_type(THD *) override {
-    max_length = 1024;
     maybe_null = false;
-    set_data_type_string(1024, &my_charset_bin);
+    set_data_type_string(1024U, &my_charset_bin);
     return false;
   }
 
@@ -1607,7 +1637,7 @@ class Item_func_convert_interval_to_user_interval final : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256, system_charset_info);
+    set_data_type_string(256U, system_charset_info);
     maybe_null = true;
     null_on_null = false;
 
@@ -1710,7 +1740,7 @@ class Item_func_internal_get_dd_column_extra final : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256, system_charset_info);
+    set_data_type_string(256U, system_charset_info);
     maybe_null = false;
     null_on_null = false;
 
