@@ -1,7 +1,7 @@
 #ifndef ITEM_TIMEFUNC_INCLUDED
 #define ITEM_TIMEFUNC_INCLUDED
 
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,14 +25,15 @@
 
 /* Function items used by mysql */
 
-#include <stddef.h>
 #include <sys/types.h>
-#include <algorithm>
 
+#include <algorithm>
+#include <cstddef>
+
+#include "field_types.h"  // MYSQL_TYPE_DATETIME
 #include "m_ctype.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
-#include "my_macros.h"
 #include "my_table_map.h"
 #include "my_time.h"
 #include "mysql/udf_registration_types.h"
@@ -41,11 +42,12 @@
 #include "sql/field.h"
 #include "sql/item.h"
 #include "sql/item_func.h"
-#include "sql/item_strfunc.h"  // Item_str_func
-#include "sql/parse_tree_node_base.h"
+#include "sql/item_strfunc.h"    // Item_str_func
+#include "sql/parse_location.h"  // POS
 #include "sql/set_var.h"
 #include "sql/sql_const.h"
 #include "sql_string.h"
+#include "template_utils.h"  // pointer_cast
 
 class MY_LOCALE;
 class PT_item_list;
@@ -53,7 +55,7 @@ class THD;
 class Time_zone;
 class my_decimal;
 struct Date_time_format;
-struct Interval;
+struct Parse_context;
 struct TABLE;
 
 bool get_interval_value(Item *args, interval_type int_type, String *str_value,
@@ -65,7 +67,8 @@ class Item_func_period_add final : public Item_int_func {
       : Item_int_func(pos, a, b) {}
   longlong val_int() override;
   const char *func_name() const override { return "period_add"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_LONGLONG)) return true;
     fix_char_length(6); /* YYYYMM */
     return false;
   }
@@ -77,7 +80,8 @@ class Item_func_period_diff final : public Item_int_func {
       : Item_int_func(pos, a, b) {}
   longlong val_int() override;
   const char *func_name() const override { return "period_diff"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_LONGLONG)) return true;
     fix_char_length(6); /* YYYYMM */
     return false;
   }
@@ -89,7 +93,8 @@ class Item_func_to_days final : public Item_int_func {
   longlong val_int() override;
   const char *func_name() const override { return "to_days"; }
   enum Functype functype() const override { return TO_DAYS_FUNC; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(6);
     maybe_null = true;
     return false;
@@ -107,7 +112,8 @@ class Item_func_to_seconds final : public Item_int_func {
   Item_func_to_seconds(const POS &pos, Item *a) : Item_int_func(pos, a) {}
   longlong val_int() override;
   const char *func_name() const override { return "to_seconds"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(MY_INT64_NUM_DECIMAL_DIGITS);
     maybe_null = true;
     return false;
@@ -138,7 +144,8 @@ class Item_func_dayofmonth final : public Item_int_func {
   longlong val_int() override;
   const char *func_name() const override { return "dayofmonth"; }
   enum Functype functype() const override { return DAY_FUNC; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(2); /* 1..31 */
     maybe_null = true;
     return false;
@@ -176,7 +183,8 @@ class Item_func_month final : public Item_func {
   const char *func_name() const override { return "month"; }
   enum Functype functype() const override { return MONTH_FUNC; }
   enum Item_result result_type() const override { return INT_RESULT; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(2);
     maybe_null = true;
     return false;
@@ -193,6 +201,7 @@ class Item_func_monthname final : public Item_str_func {
  public:
   Item_func_monthname(const POS &pos, Item *a) : Item_str_func(pos, a) {}
   const char *func_name() const override { return "monthname"; }
+  enum Functype functype() const override { return MONTHNAME_FUNC; }
   String *val_str(String *str) override;
   bool resolve_type(THD *thd) override;
   bool check_partition_func_processor(uchar *) override { return true; }
@@ -206,7 +215,9 @@ class Item_func_dayofyear final : public Item_int_func {
   Item_func_dayofyear(const POS &pos, Item *a) : Item_int_func(pos, a) {}
   longlong val_int() override;
   const char *func_name() const override { return "dayofyear"; }
-  bool resolve_type(THD *) override {
+  enum Functype functype() const override { return DAYOFYEAR_FUNC; }
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(3);
     maybe_null = true;
     return false;
@@ -223,7 +234,8 @@ class Item_func_hour final : public Item_int_func {
   longlong val_int() override;
   const char *func_name() const override { return "hour"; }
   enum Functype functype() const override { return HOUR_FUNC; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(2); /* 0..23 */
     maybe_null = true;
     return false;
@@ -240,7 +252,8 @@ class Item_func_minute final : public Item_int_func {
   longlong val_int() override;
   const char *func_name() const override { return "minute"; }
   enum Functype functype() const override { return MINUTE_FUNC; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(2); /* 0..59 */
     maybe_null = true;
     return false;
@@ -256,7 +269,9 @@ class Item_func_quarter final : public Item_int_func {
   Item_func_quarter(const POS &pos, Item *a) : Item_int_func(pos, a) {}
   longlong val_int() override;
   const char *func_name() const override { return "quarter"; }
-  bool resolve_type(THD *) override {
+  enum Functype functype() const override { return QUARTER_FUNC; }
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(1); /* 1..4 */
     maybe_null = true;
     return false;
@@ -273,7 +288,8 @@ class Item_func_second final : public Item_int_func {
   longlong val_int() override;
   const char *func_name() const override { return "second"; }
   enum Functype functype() const override { return SECOND_FUNC; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(2); /* 0..59 */
     maybe_null = true;
     return false;
@@ -296,7 +312,9 @@ class Item_func_week final : public Item_int_func {
   longlong val_int() override;
   const char *func_name() const override { return "week"; }
   enum Functype functype() const override { return WEEK_FUNC; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
+    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_LONGLONG)) return true;
     fix_char_length(2); /* 0..54 */
     maybe_null = true;
     return false;
@@ -309,7 +327,9 @@ class Item_func_yearweek final : public Item_int_func {
       : Item_int_func(pos, a, b) {}
   longlong val_int() override;
   const char *func_name() const override { return "yearweek"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
+    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_LONGLONG)) return true;
     fix_char_length(6); /* YYYYWW */
     maybe_null = true;
     return false;
@@ -328,7 +348,8 @@ class Item_func_year final : public Item_int_func {
   enum Functype functype() const override { return YEAR_FUNC; }
   enum_monotonicity_info get_monotonicity_info() const override;
   longlong val_int_endpoint(bool left_endp, bool *incl_endp) override;
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(5); /* 9999 plus sign */
     maybe_null = true;
     return false;
@@ -339,6 +360,22 @@ class Item_func_year final : public Item_int_func {
   }
 };
 
+class Item_typecast_year final : public Item_int_func {
+ public:
+  Item_typecast_year(const POS &pos, Item *a) : Item_int_func(pos, a) {
+    set_data_type_year();
+  }
+  longlong val_int() override;
+  const char *func_name() const override { return "cast_as_year"; }
+  enum Functype functype() const override { return TYPECAST_FUNC; }
+  bool resolve_type(THD *) override {
+    if (reject_geometry_args(arg_count, args, this)) return true;
+    fix_char_length(4); /* 2155 */
+    maybe_null = true;
+    return false;
+  }
+};
+
 /**
   TS-TODO: This should probably have Item_int_func as parent class.
 */
@@ -370,7 +407,8 @@ class Item_func_weekday : public Item_func {
   }
   enum Functype functype() const override { return WEEKDAY_FUNC; }
   enum Item_result result_type() const override { return INT_RESULT; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
     fix_char_length(1);
     maybe_null = true;
     return false;
@@ -393,6 +431,7 @@ class Item_func_dayname final : public Item_func_weekday {
   Item_func_dayname(const POS &pos, Item *a)
       : Item_func_weekday(pos, a, false) {}
   const char *func_name() const override { return "dayname"; }
+  enum Functype functype() const override { return DAYNAME_FUNC; }
   String *val_str(String *str) override;
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override {
     return get_date_from_string(ltime, fuzzydate);
@@ -461,15 +500,14 @@ class Item_func_unix_timestamp final : public Item_timeval_func {
   bool check_valid_arguments_processor(uchar *) override {
     return !has_timestamp_args();
   }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
     collation.set_numeric();
     const uint8 dec = arg_count == 0 ? 0 : args[0]->datetime_precision();
     if (dec > 0) {
       set_data_type_decimal(11 + dec, dec);
     } else {
       set_data_type_longlong();
-      decimals = 0;
-      max_length = 11;
     }
     return false;
   }
@@ -494,7 +532,8 @@ class Item_func_time_to_sec final : public Item_int_func {
       : Item_int_func(pos, item) {}
   longlong val_int() override;
   const char *func_name() const override { return "time_to_sec"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_TIME)) return true;
     fix_char_length(10);
     maybe_null = true;
     return false;
@@ -531,7 +570,7 @@ class Item_temporal_func : public Item_func {
       : Item_func(pos, list) {}
 
   Item_result result_type() const override { return STRING_RESULT; }
-  const CHARSET_INFO *charset_for_protocol() const override {
+  const CHARSET_INFO *charset_for_protocol() override {
     return &my_charset_bin;
   }
   Field *tmp_table_field(TABLE *table) override {
@@ -575,7 +614,7 @@ class Item_temporal_hybrid_func : public Item_str_func {
       : Item_str_func(pos, a, b), sql_mode(0) {}
 
   Item_result result_type() const override { return STRING_RESULT; }
-  const CHARSET_INFO *charset_for_protocol() const override {
+  const CHARSET_INFO *charset_for_protocol() override {
     /*
       Can return TIME, DATE, DATETIME or VARCHAR depending on arguments.
       Send using "binary" when TIME, DATE or DATETIME,
@@ -649,8 +688,7 @@ class Item_date_func : public Item_temporal_func {
   // All date functions must implement get_date()
   // to avoid use of generic Item::get_date()
   // which converts to string and then parses the string as DATE.
-  virtual bool get_date(MYSQL_TIME *res,
-                        my_time_flags_t fuzzy_date) override = 0;
+  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override = 0;
 };
 
 /**
@@ -711,8 +749,7 @@ class Item_datetime_func : public Item_temporal_func {
   // All datetime functions must implement get_date()
   // to avoid use of generic Item::get_date()
   // which converts to string and then parses the string as DATETIME.
-  virtual bool get_date(MYSQL_TIME *res,
-                        my_time_flags_t fuzzy_date) override = 0;
+  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override = 0;
 };
 
 /**
@@ -757,7 +794,7 @@ class Item_time_func : public Item_temporal_func {
   // All time functions must implement get_time()
   // to avoid use of generic Item::get_time()
   // which converts to string and then parses the string as TIME.
-  virtual bool get_time(MYSQL_TIME *res) override = 0;
+  bool get_time(MYSQL_TIME *res) override = 0;
 };
 
 /**
@@ -902,10 +939,7 @@ class Item_date_literal final : public Item_date_func {
   bool basic_const_item() const override { return true; }
   table_map used_tables() const override { return 0; }
   table_map not_null_tables() const override { return used_tables(); }
-  void cleanup() override {
-    // See Item_basic_const::cleanup()
-    if (orig_name.is_set()) item_name = orig_name;
-  }
+  void cleanup() override { DBUG_ASSERT(marker == MARKER_NONE); }
   bool eq(const Item *item, bool binary_cmp) const override;
 };
 
@@ -946,10 +980,7 @@ class Item_time_literal final : public Item_time_func {
   bool basic_const_item() const override { return true; }
   table_map used_tables() const override { return 0; }
   table_map not_null_tables() const override { return used_tables(); }
-  void cleanup() override {
-    // See Item_basic_const::cleanup()
-    if (orig_name.is_set()) item_name = orig_name;
-  }
+  void cleanup() override { DBUG_ASSERT(marker == MARKER_NONE); }
   bool eq(const Item *item, bool binary_cmp) const override;
 };
 
@@ -993,18 +1024,66 @@ class Item_datetime_literal final : public Item_datetime_func {
   bool basic_const_item() const override { return true; }
   table_map used_tables() const override { return 0; }
   table_map not_null_tables() const override { return used_tables(); }
-  void cleanup() override {
-    // See Item_basic_const::cleanup()
-    if (orig_name.is_set()) item_name = orig_name;
-  }
+  void cleanup() override { DBUG_ASSERT(marker == MARKER_NONE); }
   bool eq(const Item *item, bool binary_cmp) const override;
 };
 
+/**
+  This function implements the `AT TIME ZONE` operator, which casts a temporal
+  value to a temporal with time zone.
+
+  This function is hidden from the user except when used in a cast() operation,
+  the reason being that it adds time zone information to a temporal value, and
+  we don't currently have a type that corresponds to such a value. Hence the
+  only way to evaluate this function is by a concomitant cast to a temporal
+  without time zone designation. However, the value is not converted according
+  to the current time zone as is normally the case. For `TIMESTAMP`, this means
+  that the value is converted to the time zone given as argument to this
+  function rather than the session's time zone. And as we currently only support
+  the UTC time zone or the equivalent `INTERVAL '+00:00'`, so in practice the
+  value is not converted at all. This is a bit similar to the unix_timestamp()
+  function, but that one converts any argument (DATETIME, TIME) to UTC from the
+  session's time zone. This operator only accepts `TIMESTAMP` values.
+*/
+class Item_func_at_time_zone final : public Item_datetime_func {
+ public:
+  Item_func_at_time_zone(const POS &pos, Item *datetime,
+                         const char *specifier_string, bool is_interval)
+      : Item_datetime_func(pos, datetime),
+        m_specifier_string(specifier_string),
+        m_is_interval(is_interval) {}
+
+  bool resolve_type(THD *) override;
+
+  const char *func_name() const override { return "time_zone"; }
+
+  bool set_time_zone(THD *thd);
+
+  bool get_date(MYSQL_TIME *res, my_time_flags_t) override;
+
+  const char *specifier_string() const { return m_specifier_string; }
+
+ protected:
+  bool check_type() const;
+
+ private:
+  /// The time zone that the specifier string argument resolves to.
+  const Time_zone *m_tz{nullptr};
+
+  /// The specifier string argument, not used after resolution.
+  const char *m_specifier_string;
+
+  /**
+    Whether the syntax used the `INTERVAL` construction. We have no interval
+    type.
+  */
+  bool m_is_interval;
+};
+
 /// Abstract CURTIME function. Children should define what time zone is used.
 class Item_func_curtime : public Item_time_func {
   typedef Item_time_func super;
 
-  MYSQL_TIME_cache cached_time;  // Initialized in resolve_type()
  protected:
   // Abstract method that defines which time zone is used for conversion.
   virtual Time_zone *time_zone() = 0;
@@ -1021,19 +1100,15 @@ class Item_func_curtime : public Item_time_func {
 
   bool itemize(Parse_context *pc, Item **res) override;
 
+  /// This function must assign a new value for each execution
+  table_map get_initial_pseudo_tables() const override {
+    return INNER_TABLE_BIT;
+  }
+
   bool resolve_type(THD *thd) override;
-  longlong val_time_temporal() override {
-    DBUG_ASSERT(fixed == 1);
-    return cached_time.val_packed();
-  }
-  bool get_time(MYSQL_TIME *ltime) override {
-    DBUG_ASSERT(fixed == 1);
-    return cached_time.get_time(ltime);
-  }
-  String *val_str(String *) override {
-    DBUG_ASSERT(fixed == 1);
-    return cached_time.val_str(&str_value);
-  }
+  longlong val_time_temporal() override;
+  bool get_time(MYSQL_TIME *ltime) override;
+  String *val_str(String *) override;
   bool check_function_as_value_generator(uchar *checker_args) override {
     Check_function_as_value_generator_parameters *func_arg =
         pointer_cast<Check_function_as_value_generator_parameters *>(
@@ -1072,7 +1147,6 @@ class Item_func_curtime_utc final : public Item_func_curtime {
 class Item_func_curdate : public Item_date_func {
   typedef Item_date_func super;
 
-  MYSQL_TIME_cache cached_time;  // Initialized in resolve_type()
  protected:
   virtual Time_zone *time_zone() = 0;
 
@@ -1081,19 +1155,15 @@ class Item_func_curdate : public Item_date_func {
 
   bool itemize(Parse_context *pc, Item **res) override;
 
+  /// This function must assign a new value for each execution
+  table_map get_initial_pseudo_tables() const override {
+    return INNER_TABLE_BIT;
+  }
+
   bool resolve_type(THD *) override;
-  longlong val_date_temporal() override {
-    DBUG_ASSERT(fixed == 1);
-    return cached_time.val_packed();
-  }
-  bool get_date(MYSQL_TIME *res, my_time_flags_t) override {
-    DBUG_ASSERT(fixed == 1);
-    return cached_time.get_time(res);
-  }
-  String *val_str(String *) override {
-    DBUG_ASSERT(fixed == 1);
-    return cached_time.val_str(&str_value);
-  }
+  longlong val_date_temporal() override;
+  bool get_date(MYSQL_TIME *res, my_time_flags_t) override;
+  String *val_str(String *) override;
   bool check_function_as_value_generator(uchar *checker_args) override {
     Check_function_as_value_generator_parameters *func_arg =
         pointer_cast<Check_function_as_value_generator_parameters *>(
@@ -1128,8 +1198,6 @@ class Item_func_curdate_utc final : public Item_func_curdate {
   @sa Item_func_curtime
 */
 class Item_func_now : public Item_datetime_func {
-  MYSQL_TIME_cache cached_time;
-
  protected:
   virtual Time_zone *time_zone() = 0;
   type_conversion_status save_in_field_inner(Field *to,
@@ -1145,19 +1213,15 @@ class Item_func_now : public Item_datetime_func {
     decimals = dec_arg;
   }
 
+  /// This function must assign a new value for each execution
+  table_map get_initial_pseudo_tables() const override {
+    return INNER_TABLE_BIT;
+  }
+
   bool resolve_type(THD *) override;
-  longlong val_date_temporal() override {
-    DBUG_ASSERT(fixed == 1);
-    return cached_time.val_packed();
-  }
-  bool get_date(MYSQL_TIME *res, my_time_flags_t) override {
-    DBUG_ASSERT(fixed == 1);
-    return cached_time.get_time(res);
-  }
-  String *val_str(String *) override {
-    DBUG_ASSERT(fixed == 1);
-    return cached_time.val_str(&str_value);
-  }
+  longlong val_date_temporal() override;
+  bool get_date(MYSQL_TIME *res, my_time_flags_t) override;
+  String *val_str(String *) override;
   bool check_function_as_value_generator(uchar *checker_args) override {
     Check_function_as_value_generator_parameters *func_arg =
         pointer_cast<Check_function_as_value_generator_parameters *>(
@@ -1236,6 +1300,10 @@ class Item_func_from_days final : public Item_date_func {
   bool check_valid_arguments_processor(uchar *) override {
     return has_date_args() || has_time_args();
   }
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_LONGLONG)) return true;
+    return Item_date_func::resolve_type(thd);
+  }
 };
 
 class Item_func_date_format final : public Item_str_func {
@@ -1298,7 +1366,8 @@ class Item_func_sec_to_time final : public Item_time_func {
  public:
   Item_func_sec_to_time(const POS &pos, Item *item)
       : Item_time_func(pos, item) {}
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_NEWDECIMAL)) return true;
     set_data_type_time(
         std::min(args[0]->decimals, uint8{DATETIME_MAX_DECIMALS}));
     maybe_null = true;
@@ -1395,22 +1464,33 @@ class Item_extract final : public Item_int_func {
 };
 
 class Item_typecast_date final : public Item_date_func {
+  bool m_explicit_cast{true};
+
  public:
-  Item_typecast_date(Item *a) : Item_date_func(a) { maybe_null = true; }
+  Item_typecast_date(Item *a, bool explicit_cast)
+      : Item_date_func(a), m_explicit_cast(explicit_cast) {
+    maybe_null = true;
+  }
   Item_typecast_date(const POS &pos, Item *a) : Item_date_func(pos, a) {
     maybe_null = true;
   }
 
+  bool resolve_type(THD *thd) override {
+    if (args[0]->propagate_type(thd, MYSQL_TYPE_DATE, false, true)) return true;
+    return Item_date_func::resolve_type(thd);
+  }
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
   const char *func_name() const override { return "cast_as_date"; }
   enum Functype functype() const override { return TYPECAST_FUNC; }
+  bool is_explicit_cast() const { return m_explicit_cast; }
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) override;
   const char *cast_type() const { return "date"; }
 };
 
 class Item_typecast_time final : public Item_time_func {
   bool detect_precision_from_arg;
+  bool m_explicit_cast{true};
 
  public:
   Item_typecast_time(Item *a) : Item_time_func(a) {
@@ -1419,19 +1499,25 @@ class Item_typecast_time final : public Item_time_func {
   Item_typecast_time(const POS &pos, Item *a) : Item_time_func(pos, a) {
     detect_precision_from_arg = true;
   }
-
   Item_typecast_time(const POS &pos, Item *a, uint8 dec_arg)
       : Item_time_func(pos, a) {
     detect_precision_from_arg = false;
     decimals = dec_arg;
   }
+  Item_typecast_time(Item *a, uint8 dec_arg) : Item_time_func(a) {
+    detect_precision_from_arg = false;
+    decimals = dec_arg;
+  }
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
   const char *func_name() const override { return "cast_as_time"; }
   enum Functype functype() const override { return TYPECAST_FUNC; }
+  bool is_explicit_cast() const { return m_explicit_cast; }
   bool get_time(MYSQL_TIME *ltime) override;
   const char *cast_type() const { return "time"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (args[0]->propagate_type(thd, MYSQL_TYPE_DATETIME, false, true))
+      return true;
     set_data_type_time(detect_precision_from_arg ? args[0]->time_precision()
                                                  : decimals);
     maybe_null = true;
@@ -1441,26 +1527,34 @@ class Item_typecast_time final : public Item_time_func {
 
 class Item_typecast_datetime final : public Item_datetime_func {
   bool detect_precision_from_arg;
+  bool m_explicit_cast{true};
 
  public:
-  Item_typecast_datetime(Item *a) : Item_datetime_func(a) {
+  Item_typecast_datetime(Item *a, bool explicit_cast)
+      : Item_datetime_func(a), m_explicit_cast(explicit_cast) {
     detect_precision_from_arg = true;
   }
   Item_typecast_datetime(const POS &pos, Item *a) : Item_datetime_func(pos, a) {
     detect_precision_from_arg = true;
   }
-
   Item_typecast_datetime(const POS &pos, Item *a, uint8 dec_arg)
       : Item_datetime_func(pos, a) {
     detect_precision_from_arg = false;
     decimals = dec_arg;
   }
+  Item_typecast_datetime(Item *a, uint8 dec_arg) : Item_datetime_func(a) {
+    detect_precision_from_arg = false;
+    decimals = dec_arg;
+  }
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
   const char *func_name() const override { return "cast_as_datetime"; }
   enum Functype functype() const override { return TYPECAST_FUNC; }
   const char *cast_type() const { return "datetime"; }
-  bool resolve_type(THD *) override {
+  bool is_explicit_cast() const { return m_explicit_cast; }
+  bool resolve_type(THD *thd) override {
+    if (args[0]->propagate_type(thd, MYSQL_TYPE_DATETIME, false, true))
+      return true;
     set_data_type_datetime(
         detect_precision_from_arg ? args[0]->datetime_precision() : decimals);
     maybe_null = true;
@@ -1477,6 +1571,10 @@ class Item_func_makedate final : public Item_date_func {
   }
   const char *func_name() const override { return "makedate"; }
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) override;
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_LONGLONG)) return true;
+    return Item_date_func::resolve_type(thd);
+  }
 };
 
 class Item_func_add_time final : public Item_temporal_hybrid_func {
@@ -1502,6 +1600,8 @@ class Item_func_add_time final : public Item_temporal_hybrid_func {
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
   const char *func_name() const override { return "add_time"; }
+  enum Functype functype() const override { return ADDTIME_FUNC; }
+  int get_sign() const { return sign; }
 };
 
 class Item_func_timediff final : public Item_time_func {
@@ -1509,7 +1609,15 @@ class Item_func_timediff final : public Item_time_func {
   Item_func_timediff(const POS &pos, Item *a, Item *b)
       : Item_time_func(pos, a, b) {}
   const char *func_name() const override { return "timediff"; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    /*
+      This function can operate on two TIME, or on two DATETIME (no mix).
+      We infer the type from the other argument. If both arguments are '?', we
+      choose VARCHAR; indeed, if we chose TIME and get DATETIME, we risk
+      cutting the date part, and if we chose DATETIME and get TIME, we risk
+      interpreting "01:01:01" as "2001:01:01 00:00:00".
+     */
+    if (param_type_uses_non_param(thd)) return true;
     set_data_type_time(
         std::max(args[0]->time_precision(), args[1]->time_precision()));
     maybe_null = true;
@@ -1524,7 +1632,9 @@ class Item_func_maketime final : public Item_time_func {
       : Item_time_func(pos, a, b, c) {
     maybe_null = true;
   }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 2, MYSQL_TYPE_LONGLONG)) return true;
+    if (param_type_is_default(thd, 2, 3, MYSQL_TYPE_NEWDECIMAL)) return true;
     set_data_type_time(
         std::min(args[2]->decimals, uint8{DATETIME_MAX_DECIMALS}));
     return false;
@@ -1539,7 +1649,8 @@ class Item_func_microsecond final : public Item_int_func {
   longlong val_int() override;
   const char *func_name() const override { return "microsecond"; }
   enum Functype functype() const override { return MICROSECOND_FUNC; }
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_DATETIME)) return true;
     maybe_null = true;
     return false;
   }
@@ -1560,7 +1671,8 @@ class Item_func_timestamp_diff final : public Item_int_func {
   enum Functype functype() const override { return TIMESTAMPDIFF_FUNC; }
   interval_type intervaltype() const { return int_type; }
   longlong val_int() override;
-  bool resolve_type(THD *) override {
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_DATETIME)) return true;
     maybe_null = true;
     return false;
   }
@@ -1614,11 +1726,13 @@ class Item_func_last_day final : public Item_date_func {
   }
   const char *func_name() const override { return "last_day"; }
   bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
+  bool resolve_type(THD *thd) override {
+    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
+    return Item_date_func::resolve_type(thd);
+  }
 };
 
 class Item_func_internal_update_time final : public Item_datetime_func {
-  THD *thd;
-
  public:
   Item_func_internal_update_time(const POS &pos, PT_item_list *list)
       : Item_datetime_func(pos, list) {}
@@ -1629,8 +1743,6 @@ class Item_func_internal_update_time final : public Item_datetime_func {
 };
 
 class Item_func_internal_check_time final : public Item_datetime_func {
-  THD *thd;
-
  public:
   Item_func_internal_check_time(const POS &pos, PT_item_list *list)
       : Item_datetime_func(pos, list) {}
