@@ -1,6 +1,6 @@
 #ifndef MDL_H
 #define MDL_H
-/* Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2009, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -53,7 +53,9 @@ class THD;
 struct LF_PINS;
 struct MDL_key;
 struct MEM_ROOT;
-
+namespace mdl_unittest {
+bool test_drive_fix_pins(MDL_context *);
+}
 /**
   @def ENTER_COND(C, M, S, O)
   Start a wait on a condition.
@@ -811,7 +813,7 @@ class MDL_request {
   }
 
   static void operator delete(void *, MEM_ROOT *,
-                              const std::nothrow_t &)noexcept {}
+                              const std::nothrow_t &) noexcept {}
 
   void init_with_source(MDL_key::enum_mdl_namespace namespace_arg,
                         const char *db_arg, const char *name_arg,
@@ -932,7 +934,8 @@ class MDL_wait_for_subgraph {
   */
   virtual bool accept_visitor(MDL_wait_for_graph_visitor *gvisitor) = 0;
 
-  static const uint DEADLOCK_WEIGHT_DML = 0;
+  static const uint DEADLOCK_WEIGHT_CO = 0;
+  static const uint DEADLOCK_WEIGHT_DML = 25;
   static const uint DEADLOCK_WEIGHT_ULL = 50;
   static const uint DEADLOCK_WEIGHT_DDL = 100;
 
@@ -995,8 +998,8 @@ class MDL_ticket : public MDL_wait_for_subgraph {
   bool is_incompatible_when_waiting(enum_mdl_type type) const;
 
   /** Implement MDL_wait_for_subgraph interface. */
-  virtual bool accept_visitor(MDL_wait_for_graph_visitor *dvisitor);
-  virtual uint get_deadlock_weight() const;
+  bool accept_visitor(MDL_wait_for_graph_visitor *dvisitor) override;
+  uint get_deadlock_weight() const override;
 
 #ifndef DBUG_OFF
   enum_mdl_duration get_duration() const { return m_duration; }
@@ -1034,7 +1037,7 @@ class MDL_ticket : public MDL_wait_for_subgraph {
         m_psi(nullptr) {
   }
 
-  virtual ~MDL_ticket() { DBUG_ASSERT(m_psi == nullptr); }
+  ~MDL_ticket() override { DBUG_ASSERT(m_psi == nullptr); }
 
   static MDL_ticket *create(MDL_context *ctx_arg, enum_mdl_type type_arg
 #ifndef DBUG_OFF
@@ -1646,7 +1649,9 @@ class MDL_context {
   void release_lock(enum_mdl_duration duration, MDL_ticket *ticket);
   bool try_acquire_lock_impl(MDL_request *mdl_request, MDL_ticket **out_ticket);
   void materialize_fast_path_locks();
-  inline bool fix_pins();
+
+  friend bool mdl_unittest::test_drive_fix_pins(MDL_context *);
+  bool fix_pins();
 
  public:
   void find_deadlock();
