@@ -1,4 +1,4 @@
-/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,22 +23,30 @@
 #ifndef MEMORY_UNIQUE_PTR_INCLUDED
 #define MEMORY_UNIQUE_PTR_INCLUDED
 
-#include <assert.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <algorithm>
 #include <cstddef>
 #include <iostream>
-#include <limits>
 #include <memory>
 #include <string>
 #include <tuple>
-
+#include "my_dbug.h"
 #include "my_sys.h"
 #include "mysql/service_mysql_alloc.h"  // my_malloc
 #include "sql/memory/aligned_atomic.h"  // memory::cache_line_size
 #include "sql/memory/ref_ptr.h"         // memory::Ref_ptr
 
+#if defined(__sun) && defined(__SVR4)
+namespace memory {
+template <typename T, typename A = std::nullptr_t>
+using Unique_ptr = std::unique_ptr<T>;
+template <typename T, typename... Args>
+Unique_ptr<T> make_unique(Args &&... args) {
+  return std::make_unique<T>(args...);
+}
+}  // namespace memory
+#else
 namespace memory {
 namespace traits {
 /**
@@ -54,7 +62,7 @@ std::false_type test_for_allocate(...);
 
 }  // namespace traits
 /**
- Struct that allows for checking if `T` fulfills the Allocator named
+ Struct that allows for checking if `T` fullfils the Allocator named
  requirements.
  */
 template <class T>
@@ -155,15 +163,15 @@ class Unique_ptr {
   using reference = type &;
 
   /**
-    Default class constructor, only to be used with no specific allocator.
+    Default class constuctor, only to be used with no specific allocator.
    */
   template <
       typename D = T, typename B = A,
       std::enable_if_t<std::is_same<B, std::nullptr_t>::value> * = nullptr>
   Unique_ptr();
   /**
-    Class constructor, to be used with specific allocators, passing the
-    allocator object to be used.
+    Class constuctor, to be used with specific allocators, passing the allocator
+    object to be used.
 
     @param alloc The allocator instance to be used.
    */
@@ -172,7 +180,7 @@ class Unique_ptr {
       std::enable_if_t<!std::is_same<B, std::nullptr_t>::value> * = nullptr>
   Unique_ptr(A &alloc);
   /**
-    Class constructor, to be used with specific allocators and when `T` is an
+    Class constuctor, to be used with specific allocators and when `T` is an
     array type, passing the allocator object to be used and the size of the
     array.
 
@@ -184,7 +192,7 @@ class Unique_ptr {
                              std::is_array<D>::value> * = nullptr>
   Unique_ptr(A &alloc, size_t size);
   /**
-    Class constructor, to be used with no specific allocators and when `T` is an
+    Class constuctor, to be used with no specific allocators and when `T` is an
     array type, passing the allocator object to be used and the size of the
     array.
 
@@ -195,9 +203,9 @@ class Unique_ptr {
                              std::is_array<D>::value> * = nullptr>
   Unique_ptr(size_t size);
   /**
-    Class constructor, to be used with specific allocators and when `T` is not
-    an array type, passing the allocator object to be used and the parameters to
-    be used with `T` object constructor.
+    Class constuctor, to be used with specific allocators and when `T` is not an
+    array type, passing the allocator object to be used and the parameters to be
+    used with `T` object contructor.
 
     @param alloc The allocator instance to be used.
     @param args The parameters to be used with `T` object constructor.
@@ -207,9 +215,8 @@ class Unique_ptr {
                              !std::is_array<D>::value> * = nullptr>
   Unique_ptr(A &alloc, Args &&... args);
   /**
-    Class constructor, to be used with no specific allocators and when `T` is
-    not an array type, passing the parameters to be used with `T` object
-    constructor.
+    Class constuctor, to be used with no specific allocators and when `T` is not
+    an array type, passing the parameters to be used with `T` object contructor.
 
     @param args The parameters to be used with `T` object constructor.
    */
@@ -217,7 +224,7 @@ class Unique_ptr {
             std::enable_if_t<std::is_same<B, std::nullptr_t>::value &&
                              !std::is_array<D>::value> * = nullptr>
   Unique_ptr(Args &&... args);
-  // Deleted copy constructor
+  // Deleted copy constuctor
   Unique_ptr(Unique_ptr<T, A> const &rhs) = delete;
   /**
     Move constructor.
@@ -226,7 +233,7 @@ class Unique_ptr {
    */
   Unique_ptr(Unique_ptr<T, A> &&rhs);
   /**
-    Destructor for the class.
+    Destructor fot the class.
    */
   virtual ~Unique_ptr();
   // Deleted copy operator
@@ -248,7 +255,7 @@ class Unique_ptr {
   /**
     Star operator to access the underlying object of type `T`.
 
-    @return A reference to the underlying object of type `T`.
+    @return A refernce to the underlying object of type `T`.
    */
   reference operator*() const;
   /**
@@ -304,7 +311,7 @@ class Unique_ptr {
   /**
     Will resize the allocated memory to `new_size`. If the configure allocator
     supports this operation, the allocator is used. If not, a new memory chunk
-    is allocated and the memory is copied.
+    is allocted and the memory is copied.
 
     @param new_size The new desired size for the memory.
 
@@ -318,7 +325,7 @@ class Unique_ptr {
   /**
     Will resize the allocated memory to `new_size`. If the configure allocator
     supports this operation, the allocator is used. If not, a new memory chunk
-    is allocated and the memory is copied.
+    is allocted and the memory is copied.
 
     @param new_size The new desired size for the memory.
 
@@ -408,7 +415,7 @@ Unique_ptr<T, std::nullptr_t> make_unique(size_t size);
   In-place constructs a new unique pointer with a specific allocator and with
   array type `T`.
 
-  @param alloc A reference to the allocator object to use.
+  @param alloc A referenc to the allocator object ot use.
   @param size The size of the array to allocate.
 
   @return A new instance of unique pointer.
@@ -420,7 +427,7 @@ Unique_ptr<T, A> make_unique(A &alloc, size_t size);
   In-place constructs a new unique pointer with a specific allocator and with
   non-array type `T`.
 
-  @param alloc A reference to the allocator object to use.
+  @param alloc A referenc to the allocator object ot use.
   @param args The parameters to be used in constructing the instance of `T`.
 
   @return A new instance of unique pointer.
@@ -517,21 +524,21 @@ void memory::PFS_allocator<T>::deallocate(T *p, std::size_t) noexcept {
 template <typename T>
 template <class U, class... Args>
 void memory::PFS_allocator<T>::construct(U *p, Args &&... args) {
-  assert(p != nullptr);
+  DBUG_ASSERT(p != nullptr);
   try {
     ::new ((void *)p) U(std::forward<Args>(args)...);
   } catch (...) {
-    assert(false);  // Constructor should not throw an exception.
+    DBUG_ASSERT(false);  // Constructor should not throw an exception.
   }
 }
 
 template <typename T>
 void memory::PFS_allocator<T>::destroy(T *p) {
-  assert(p != nullptr);
+  DBUG_ASSERT(p != nullptr);
   try {
     p->~T();
   } catch (...) {
-    assert(false);  // Destructor should not throw an exception
+    DBUG_ASSERT(false);  // Destructor should not throw an exception
   }
 }
 
@@ -778,10 +785,9 @@ typename memory::Unique_ptr<T, A>::pointer memory::Unique_ptr<T, A>::clone()
   return to_return;
 }
 
-#ifndef IN_DOXYGEN  // Doxygen doesn't understand this construction.
 template <typename T, std::enable_if_t<std::is_array<T>::value> *>
 memory::Unique_ptr<T, std::nullptr_t> memory::make_unique(size_t size) {
-  return memory::Unique_ptr<T, std::nullptr_t>{size};
+  return std::move(memory::Unique_ptr<T, std::nullptr_t>{size});
 }
 
 template <typename T, typename A, std::enable_if_t<std::is_array<T>::value> *>
@@ -800,8 +806,8 @@ memory::Unique_ptr<T, A> memory::make_unique(A &alloc, Args &&... args) {
 template <typename T, typename... Args,
           std::enable_if_t<!std::is_array<T>::value> *>
 memory::Unique_ptr<T, std::nullptr_t> memory::make_unique(Args &&... args) {
-  return memory::Unique_ptr<T, std::nullptr_t>{std::forward<Args>(args)...};
+  return std::move(
+      memory::Unique_ptr<T, std::nullptr_t>{std::forward<Args>(args)...});
 }
-#endif
-
+#endif  // Is solaris
 #endif  // MEMORY_UNIQUE_PTR_INCLUDED
