@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -45,6 +45,7 @@
 #include "my_macros.h"
 #include "my_sys.h"
 #include "my_thread.h"
+#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
 #include "mysql/components/services/psi_file_bits.h"
@@ -54,7 +55,6 @@
 #include "mysql/psi/mysql_file.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_mutex.h"
-#include "mysql/psi/psi_base.h"
 #include "mysql/status_var.h"
 #include "mysql_version.h"
 #include "mysqld_error.h"
@@ -708,7 +708,7 @@ bool Persisted_variables_cache::set_persist_options(bool plugin_options) {
         Grant_temporary_dynamic_privileges(thd, priv_list),
         Grant_temporary_static_privileges(thd, static_priv_list),
         Drop_temporary_dynamic_privileges(priv_list));
-    ctx = default_factory.create(thd->mem_root);
+    ctx = default_factory.create();
     /* attach this auth id to current security_context */
     thd->set_security_context(ctx.get());
     thd->real_id = my_thread_self();
@@ -817,7 +817,7 @@ bool Persisted_variables_cache::set_persist_options(bool plugin_options) {
       result = true;
       goto err;
     }
-    tmp_var_list.empty();
+    tmp_var_list.clear();
     /*
       Once persisted variables are SET in the server,
       update variables source/user/timestamp/host from m_persist_variables.
