@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,13 +22,10 @@
 
 #include "sql/regexp/regexp_facade.h"
 
-#include <optional>
 #include <string>
 #include <tuple>
 
 #include "my_pointer_arithmetic.h"
-#include "mysql/strings/m_ctype.h"  // CHARSET_INFO.
-#include "sql/item_func.h"
 #include "sql/mysqld.h"  // make_unique_destroy_only
 #include "sql/regexp/regexp_engine.h"
 #include "sql_string.h"
@@ -37,8 +33,6 @@
 
 namespace regexp {
 
-CHARSET_INFO *regexp_lib_charset = nullptr;
-
 /**
   When dealing with the binary character set, we tell ICU that we're using
   CP-1252. This way, comparisons will happen as the user expects; each octet
@@ -86,7 +80,7 @@ static bool EvalExprToCharset(Item *expr, std::u16string *out, int skip = 0) {
     */
     my_error(ER_REGEXP_INDEX_OUTOFBOUNDS_ERROR, MYF(0));
     out->clear();
-    return true;
+    return false;
   }
   if (expr->collation.collation != regexp_lib_charset) {
     // Character set conversion is called for.
@@ -103,12 +97,8 @@ static bool EvalExprToCharset(Item *expr, std::u16string *out, int skip = 0) {
     size_t converted_size = my_convert(to, to_size, regexp_lib_charset, start,
                                        length, source_charset, &errors);
 
-    if (errors > 0) {
-      report_conversion_error(regexp_lib_charset, start, length,
-                              source_charset);
-      return true;
-    }
-    assert(converted_size % sizeof(UChar) == 0);
+    if (errors > 0) return true;
+    DBUG_ASSERT(converted_size % sizeof(UChar) == 0);
     out->resize(converted_size / sizeof(UChar));
     return false;
   }
@@ -116,7 +106,7 @@ static bool EvalExprToCharset(Item *expr, std::u16string *out, int skip = 0) {
   // However: val_str() may ignore the input argument,
   // and return a pointer to some other buffer.
   if (!is_aligned_to(s->ptr(), alignof(UChar))) {
-    assert(s != &aligned_str);
+    DBUG_ASSERT(s != &aligned_str);
     aligned_str.copy(*s);
     s = &aligned_str;
   }
@@ -170,11 +160,11 @@ int Regexp_facade::ConvertLibPositionToCodePoint(int position) const {
   return cset->numchars(regexp_lib_charset, start, end);
 }
 
-std::optional<bool> Regexp_facade::Matches(Item *subject_expr, int start,
-                                           int occurrence) {
+Mysql::Nullable<bool> Regexp_facade::Matches(Item *subject_expr, int start,
+                                             int occurrence) {
   DBUG_TRACE;
 
-  if (Reset(subject_expr, start)) return std::optional<bool>();
+  if (Reset(subject_expr, start)) return Mysql::Nullable<bool>();
 
   /*
     As far as ICU is concerned, we always start on position 0, since we
@@ -183,10 +173,10 @@ std::optional<bool> Regexp_facade::Matches(Item *subject_expr, int start,
   return m_engine->Matches(0, occurrence);
 }
 
-std::optional<int> Regexp_facade::Find(Item *subject_expr, int start,
-                                       int occurrence, bool after_match) {
-  std::optional<bool> match_found = Matches(subject_expr, start, occurrence);
-  if (!match_found.has_value()) return std::optional<int>();
+Mysql::Nullable<int> Regexp_facade::Find(Item *subject_expr, int start,
+                                         int occurrence, bool after_match) {
+  Nullable<bool> match_found = Matches(subject_expr, start, occurrence);
+  if (!match_found.has_value()) return Mysql::Nullable<int>();
   if (!match_found.value()) return 0;
   int native_start =
       after_match ? m_engine->EndOfMatch() : m_engine->StartOfMatch();
