@@ -1,19 +1,18 @@
 #ifndef SQL_REGEXP_REGEXP_FACADE_H_
 #define SQL_REGEXP_REGEXP_FACADE_H_
 
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,16 +31,17 @@
 
 #include <stdint.h>
 
-#include <optional>
 #include <string>
 
+#include "nullable.h"
 #include "sql/item.h"
 #include "sql/regexp/regexp_engine.h"
 #include "sql_string.h"
 
-namespace regexp {
+extern int32_t opt_regexp_time_limit;
+extern int32_t opt_regexp_stack_limit;
 
-extern CHARSET_INFO *regexp_lib_charset;
+namespace regexp {
 
 /**
   This class handles
@@ -93,7 +93,7 @@ class Regexp_facade {
     doesn't have to make a special case for when the regular expression is
     NULL. Instead, the case is handled here in the facade.
   */
-  std::optional<bool> Matches(Item *subject_expr, int start, int occurrence);
+  Mysql::Nullable<bool> Matches(Item *subject_expr, int start, int occurrence);
 
   /**
     Searches the subject for a match of the compiled regular expression and
@@ -107,8 +107,8 @@ class Regexp_facade {
 
     @return The first character of the match, or a null value if not found.
   */
-  std::optional<int> Find(Item *subject_expr, int start, int occurrence,
-                          bool after_match);
+  Mysql::Nullable<int> Find(Item *subject_expr, int start, int occurrence,
+                            bool after_match);
 
   /**
     @param subject_expr The string to search.
