@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -37,11 +37,11 @@
 #include "my_loglevel.h"
 #include "my_sys.h"
 #include "my_thread.h"
+#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/psi_stage_bits.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_mutex.h"
-#include "mysql/psi/psi_base.h"
 #include "mysql/service_mysql_alloc.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
@@ -53,6 +53,8 @@
 #include "sql/mysqld.h"              // opt_mts_slave_parallel_workers
 #include "sql/mysqld_thd_manager.h"  // Global_THD_manager
 #include "sql/protocol_classic.h"
+#include "sql/raii/sentry.h"
+#include "sql/rpl_channel_credentials.h"
 #include "sql/rpl_gtid.h"
 #include "sql/rpl_info_factory.h"
 #include "sql/rpl_info_handler.h"
@@ -109,6 +111,10 @@ static void set_mi_settings(Master_info *mi,
 
   mi->rli->set_thd_tx_priority(channel_info->thd_tx_priority);
 
+  mi->rli->set_ignore_write_set_memory_limit(
+      channel_info->m_ignore_write_set_memory_limit);
+  mi->rli->set_allow_drop_write_set(channel_info->m_allow_drop_write_set);
+
   mi->rli->replicate_same_server_id =
       (channel_info->replicate_same_server_id == RPL_SERVICE_SERVER_DEFAULT)
           ? replicate_same_server_id
@@ -137,7 +143,23 @@ static void set_mi_settings(Master_info *mi,
           ? opt_mts_checkpoint_group
           : channel_info->channel_mts_checkpoint_group;
 
-  mi->set_mi_description_event(new Format_description_log_event());
+  Format_description_log_event *fde = new Format_description_log_event();
+  /*
+    Group replication applier channel shall not use checksum on its relay log
+    files.
+  */
+  if (channel_map.is_group_replication_channel_name(mi->get_channel(), true)) {
+    fde->footer()->checksum_alg = binary_log::BINLOG_CHECKSUM_ALG_OFF;
+    /*
+      When the receiver thread connects to the master, it gets its current
+      binlog checksum algorithm, but as GR applier channel has no receiver
+      thread (and also does not connect to a master), we will set the variable
+      here to BINLOG_CHECKSUM_ALG_OFF as events queued after certification have
+      no checksum information.
+    */
+    mi->checksum_alg_before_fd = binary_log::BINLOG_CHECKSUM_ALG_OFF;
+  }
+  mi->set_mi_description_event(fde);
 
   mysql_mutex_unlock(&mi->data_lock);
   mysql_mutex_unlock(mi->rli->relay_log.get_log_lock());
@@ -184,6 +206,8 @@ void initialize_channel_creation_info(Channel_creation_info *channel_info) {
   channel_info->get_public_key = 0;
   channel_info->compression_algorithm = nullptr;
   channel_info->zstd_compression_level = 0;
+  channel_info->m_ignore_write_set_memory_limit = false;
+  channel_info->m_allow_drop_write_set = false;
 }
 
 void initialize_channel_ssl_info(Channel_ssl_info *channel_ssl_info) {
@@ -339,6 +363,19 @@ int channel_create(const char *channel, Channel_creation_info *channel_info) {
     lex_mi->zstd_compression_level = channel_info->zstd_compression_level;
   }
 
+  lex_mi->m_source_connection_auto_failover = LEX_MASTER_INFO::LEX_MI_UNCHANGED;
+  if (channel_info->m_source_connection_auto_failover) {
+    if (mi && !mi->is_source_connection_auto_failover()) {
+      lex_mi->m_source_connection_auto_failover =
+          LEX_MASTER_INFO::LEX_MI_ENABLE;
+    }
+  } else {
+    if (mi && mi->is_source_connection_auto_failover()) {
+      lex_mi->m_source_connection_auto_failover =
+          LEX_MASTER_INFO::LEX_MI_DISABLE;
+    }
+  }
+
   if (channel_info->ssl_info != nullptr) {
     set_mi_ssl_options(lex_mi, channel_info->ssl_info);
   }
@@ -384,6 +421,7 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
   ulong thread_start_id = 0;
   bool thd_created = false;
   THD *thd = current_thd;
+  String_set user, pass, auth;
 
   /* Service channels are not supposed to use sql_slave_skip_counter */
   mysql_mutex_lock(&LOCK_sql_slave_skip_counter);
@@ -414,6 +452,16 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
   LEX_SLAVE_CONNECTION lex_connection;
   lex_connection.reset();
 
+  if (!Rpl_channel_credentials::get_instance().get_credentials(channel, user,
+                                                               pass, auth)) {
+    lex_connection.user =
+        (user.first ? const_cast<char *>(user.second.c_str()) : nullptr);
+    lex_connection.password =
+        (pass.first ? const_cast<char *>(pass.second.c_str()) : nullptr);
+    lex_connection.plugin_auth =
+        (auth.first ? const_cast<char *>(auth.second.c_str()) : nullptr);
+  }
+
   if (connection_info->until_condition != CHANNEL_NO_UNTIL_CONDITION) {
     switch (connection_info->until_condition) {
       case CHANNEL_UNTIL_APPLIER_AFTER_GTIDS:
@@ -502,6 +550,10 @@ int channel_stop(Master_info *mi, int threads_to_stop, long timeout) {
       (server_thd_mask & SLAVE_IO)) {
     thread_mask |= SLAVE_IO;
   }
+  if ((threads_to_stop & CHANNEL_RECEIVER_THREAD) &&
+      (server_thd_mask & SLAVE_MONITOR)) {
+    thread_mask |= SLAVE_MONITOR;
+  }
 
   if (thread_mask == 0) {
     goto end;
@@ -590,7 +642,7 @@ class Kill_binlog_dump : public Do_THD_Impl {
  public:
   Kill_binlog_dump() {}
 
-  virtual void operator()(THD *thd_to_kill) {
+  void operator()(THD *thd_to_kill) override {
     if (thd_to_kill->get_command() == COM_BINLOG_DUMP ||
         thd_to_kill->get_command() == COM_BINLOG_DUMP_GTID) {
       DBUG_ASSERT(thd_to_kill != current_thd);
@@ -1015,28 +1067,38 @@ int channel_get_retrieved_gtid_set(const char *channel, char **retrieved_set) {
   return error;
 }
 
-int channel_get_credentials(const char *channel, const char **user, char **pass,
-                            size_t *pass_size) {
-  DBUG_ENTER("channel_get_credentials(channel,user,password, pass_size)");
+int channel_get_credentials(const char *channel, std::string &username,
+                            std::string &password) {
+  DBUG_TRACE;
+  String_set user_store, pass_store, auth_store;
+
+  if (!Rpl_channel_credentials::get_instance().get_credentials(
+          channel, user_store, pass_store, auth_store)) {
+    if (user_store.first) username = user_store.second;
+    if (pass_store.first) password = pass_store.second;
+    return 0;
+  }
 
   channel_map.rdlock();
-
   Master_info *mi = channel_map.get_mi(channel);
 
   if (mi == nullptr) {
     channel_map.unlock();
-    DBUG_RETURN(RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR);
+    return RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR;
   }
 
   mi->inc_reference();
   channel_map.unlock();
 
-  *user = mi->get_user();
-  mi->get_password(*pass, pass_size);
+  char pass[MAX_PASSWORD_LENGTH + 1];
+  size_t pass_size;
+  mi->get_password(pass, &pass_size);
+  username.assign(mi->get_user());
+  password.assign(pass, pass_size);
 
   mi->dec_reference();
 
-  DBUG_RETURN(0);
+  return 0;
 }
 
 bool channel_is_stopping(const char *channel,
@@ -1082,6 +1144,29 @@ bool is_partial_transaction_on_channel_relay_log(const char *channel) {
   return ret;
 }
 
+bool channel_has_same_uuid_as_group_name(const char *group_name) {
+  DBUG_TRACE;
+  Master_info *mi = nullptr;
+  channel_map.rdlock();
+  raii::Sentry<> map_lock_sentry{[&]() -> void { channel_map.unlock(); }};
+
+  for (mi_map::iterator it = channel_map.begin(); it != channel_map.end();
+       it++) {
+    mi = it->second;
+    if (mi != nullptr &&
+        mi->rli->m_assign_gtids_to_anonymous_transactions_info.get_type() >
+            Assign_gtids_to_anonymous_transactions_info::enum_type::AGAT_OFF) {
+      if (!(strcmp((mi->rli->m_assign_gtids_to_anonymous_transactions_info
+                        .get_value())
+                       .data(),
+                   group_name))) {
+        return true;
+      }
+    }
+  }
+  return false;
+}
+
 bool is_any_slave_channel_running(int thread_mask) {
   DBUG_TRACE;
   Master_info *mi = nullptr;
@@ -1170,3 +1255,9 @@ has_any_slave_channel_open_temp_table_or_is_its_applier_running() {
 
   return SLAVE_CHANNEL_NO_APPLIER_RUNNING_AND_NO_OPEN_TEMPORARY_TABLE;
 }
+
+int channel_delete_credentials(const char *channel_name) {
+  DBUG_TRACE;
+  return Rpl_channel_credentials::get_instance().delete_credentials(
+      channel_name);
+}
