@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -45,18 +45,17 @@
 #include "mysqld_error.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/sql_security_ctx.h"
-#include "sql/changestreams/apply/replication_thread_status.h"
 #include "sql/current_thd.h"
 #include "sql/item.h"    // Item
 #include "sql/mysqld.h"  // table_alias_charset
 #include "sql/psi_memory_key.h"
-#include "sql/rpl_mi.h"       // Master_info
-#include "sql/rpl_msr.h"      // channel_map
-#include "sql/rpl_replica.h"  // SLAVE_SQL
-#include "sql/rpl_rli.h"      // Relay_log_info
+#include "sql/rpl_mi.h"     // Master_info
+#include "sql/rpl_msr.h"    // channel_map
+#include "sql/rpl_rli.h"    // Relay_log_info
+#include "sql/rpl_slave.h"  // SLAVE_SQL
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
-#include "sql/table.h"  // Table_ref
+#include "sql/table.h"  // TABLE_LIST
 #include "sql/thr_malloc.h"
 #include "sql_string.h"
 #include "template_utils.h"  // my_free_container_pointers
@@ -86,11 +85,11 @@ Rpl_pfs_filter::Rpl_pfs_filter(const Rpl_pfs_filter &other) {
   if (!other.m_filter_rule.is_empty()) m_filter_rule.copy(other.m_filter_rule);
 }
 
-Rpl_pfs_filter::~Rpl_pfs_filter() = default;
+Rpl_pfs_filter::~Rpl_pfs_filter() {}
 
 Rpl_filter_statistics::Rpl_filter_statistics() { reset(); }
 
-Rpl_filter_statistics::~Rpl_filter_statistics() = default;
+Rpl_filter_statistics::~Rpl_filter_statistics() {}
 
 void Rpl_filter_statistics::reset() {
   m_configured_by = CONFIGURED_BY_STARTUP_OPTIONS;
@@ -111,9 +110,9 @@ void Rpl_filter_statistics::set_all(enum_configured_by configured_by) {
       Calculate time stamp up to tenths of milliseconds elapsed
       from 1 Jan 1970 00:00:00.
     */
-    my_timeval stmt_start_time = thd->query_start_timeval_trunc(6);
-    m_active_since =
-        stmt_start_time.m_tv_sec * 1000000 + stmt_start_time.m_tv_usec;
+    struct timeval stmt_start_time = thd->query_start_timeval_trunc(6);
+    m_active_since = static_cast<ulonglong>(stmt_start_time.tv_sec) * 1000000 +
+                     stmt_start_time.tv_usec;
   }
 }
 
@@ -192,7 +191,7 @@ int Rpl_filter::copy_global_replication_filters() {
   bool need_unlock = false;
 
   /* Assert that it is not self copy. */
-  assert(this != &rpl_global_filter);
+  DBUG_ASSERT(this != &rpl_global_filter);
 
   /* Check if the source is empty. */
   if (rpl_global_filter.is_empty()) return 0;
@@ -276,7 +275,7 @@ int Rpl_filter::copy_global_replication_filters() {
                                         rpl_global_filter.wild_do_table_inited);
     if (res != 0) goto err;
 
-    assert(!wild_do_table.empty());
+    DBUG_ASSERT(!wild_do_table.empty());
 
     wild_do_table_inited = true;
     table_rules_on = true;
@@ -293,7 +292,7 @@ int Rpl_filter::copy_global_replication_filters() {
                     res = 1;);
     if (res != 0) goto err;
 
-    assert(!wild_ignore_table.empty());
+    DBUG_ASSERT(!wild_ignore_table.empty());
 
     wild_ignore_table_inited = true;
     table_rules_on = true;
@@ -358,7 +357,7 @@ err:
 
   SYNOPSIS
     tables_ok()
-    db              db to use if db in Table_ref is undefined for a table
+    db              db to use if db in TABLE_LIST is undefined for a table
     tables          list of tables to check
 
   NOTES
@@ -391,7 +390,7 @@ err:
     1           should be logged/replicated
 */
 
-bool Rpl_filter::tables_ok(const char *db, Table_ref *tables) {
+bool Rpl_filter::tables_ok(const char *db, TABLE_LIST *tables) {
   bool some_tables_updating = false;
   DBUG_TRACE;
 
@@ -525,7 +524,7 @@ bool Rpl_filter::db_ok(const char *db, bool need_increase_counter) {
     With replicate_wild_ignore_table, we only check tables. When
     one does 'DROP DATABASE db1', tables are not involved and the
     statement will be replicated, while users could expect it would not (as it
-    roughly means 'DROP db1.first_table, DROP db1.second_table...').
+    rougly means 'DROP db1.first_table, DROP db1.second_table...').
     In other words, we want to interpret 'db1.%' as "everything touching db1".
     That is why we want to match 'db1' against 'db1.%' wild table rules.
 
@@ -890,7 +889,7 @@ int Rpl_filter::set_db_rewrite(mem_root_deque<Item *> *rewrite_db_pair_list,
 
   auto it = rewrite_db_pair_list->begin();
 
-  /* Please note that grammar itself allows only even number of db values. So
+  /* Please note that grammer itself allows only even number of db values. So
    * it is ok to do it++ twice without checking anything. */
   while (status == 0 && it != rewrite_db_pair_list->end()) {
     Item *db_key = *it++;
@@ -1342,7 +1341,7 @@ void Sql_cmd_change_repl_filter::set_filter_value(
       break;
     default:
       /* purecov: begin deadcode */
-      assert(0);
+      DBUG_ASSERT(0);
       break;
       /* purecov: end */
   }
@@ -1377,7 +1376,7 @@ bool Sql_cmd_change_repl_filter::change_rpl_filter(THD *thd) {
 
   if (!lex->mi.for_channel) {
     if (channel_map.get_num_instances(true) == 0) {
-      my_error(ER_REPLICA_CONFIGURATION, MYF(0));
+      my_error(ER_SLAVE_CONFIGURATION, MYF(0));
       ret = true;
       goto err;
     }
@@ -1402,7 +1401,7 @@ bool Sql_cmd_change_repl_filter::change_rpl_filter(THD *thd) {
         init_thread_mask(&thread_mask, mi, false /*not inverse*/);
         if (thread_mask & SLAVE_SQL) {
           /* We refuse if any slave thread is running */
-          my_error(ER_REPLICA_CHANNEL_SQL_THREAD_MUST_STOP, MYF(0),
+          my_error(ER_SLAVE_CHANNEL_SQL_THREAD_MUST_STOP, MYF(0),
                    mi->get_channel());
           ret = true;
           /*
@@ -1491,7 +1490,7 @@ bool Sql_cmd_change_repl_filter::change_rpl_filter(THD *thd) {
         If an explicit FOR CHANNEL clause is provided, the statement
         is disallowed on group replication channels.
       */
-      my_error(ER_REPLICA_CHANNEL_OPERATION_NOT_ALLOWED, MYF(0),
+      my_error(ER_SLAVE_CHANNEL_OPERATION_NOT_ALLOWED, MYF(0),
                "CHANGE REPLICATION FILTER", lex->mi.channel);
       ret = true;
       goto err;
@@ -1508,7 +1507,7 @@ bool Sql_cmd_change_repl_filter::change_rpl_filter(THD *thd) {
     mi = channel_map.get_mi(lex->mi.channel);
 
     if (!Master_info::is_configured(mi)) {
-      my_error(ER_REPLICA_CONFIGURATION, MYF(0));
+      my_error(ER_SLAVE_CONFIGURATION, MYF(0));
       ret = true;
       goto err;
     }
@@ -1520,7 +1519,7 @@ bool Sql_cmd_change_repl_filter::change_rpl_filter(THD *thd) {
     init_thread_mask(&thread_mask, mi, false /*not inverse*/);
     /* We refuse if the slave thread is running */
     if (thread_mask & SLAVE_SQL) {
-      my_error(ER_REPLICA_CHANNEL_SQL_THREAD_MUST_STOP, MYF(0),
+      my_error(ER_SLAVE_CHANNEL_SQL_THREAD_MUST_STOP, MYF(0),
                mi->get_channel());
       ret = true;
     }
