@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2013, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,6 +24,7 @@
 
 #include <stdlib.h>
 #include <sys/types.h>
+#include <atomic>
 
 #include "m_string.h"
 #include "my_dbug.h"
@@ -42,18 +43,22 @@
 #include "sql/mysqld.h"              // mysqld_port
 #include "sql/mysqld_thd_manager.h"  // Global_THD_manager
 #include "sql/replication.h"         // Trans_context_info
+#include "sql/rpl_channel_credentials.h"
 #include "sql/rpl_channel_service_interface.h"
-#include "sql/rpl_gtid.h"    // gtid_mode_lock
-#include "sql/rpl_slave.h"   // report_host
+#include "sql/rpl_gtid.h"   // Gtid_mode::lock
+#include "sql/rpl_slave.h"  // report_host
+#include "sql/sql_class.h"  // THD
+#include "sql/sql_lex.h"
 #include "sql/sql_plugin.h"  // plugin_unlock
 #include "sql/sql_plugin_ref.h"
-#include "sql/ssl_acceptor_context.h"
+#include "sql/ssl_init_callback.h"
 #include "sql/system_variables.h"  // System_variables
 
 class THD;
 
 extern ulong opt_mi_repository_id;
 extern ulong opt_rli_repository_id;
+std::atomic_flag start_stop_executing = ATOMIC_FLAG_INIT;
 
 /*
   Struct to share server ssl variables
@@ -111,46 +116,110 @@ bool is_group_replication_plugin_loaded() {
   return result;
 }
 
-int group_replication_start(char **error_message) {
+int group_replication_start(char **error_message, THD *thd) {
+  LEX *lex = thd->lex;
   int result = 1;
+  plugin_ref plugin = nullptr;
 
-  plugin_ref plugin =
-      my_plugin_lock_by_name(nullptr, group_replication_plugin_name_str,
-                             MYSQL_GROUP_REPLICATION_PLUGIN);
+  if (start_stop_executing.test_and_set()) {
+    std::string msg;
+    msg.assign(
+        "Another instance of START/STOP GROUP_REPLICATION command is "
+        "executing.");
+    *error_message =
+        (char *)my_malloc(PSI_NOT_INSTRUMENTED, msg.size() + 1, MYF(0));
+    strcpy(*error_message, msg.c_str());
+    result = 8;
+    return result;
+  }
+  plugin = my_plugin_lock_by_name(nullptr, group_replication_plugin_name_str,
+                                  MYSQL_GROUP_REPLICATION_PLUGIN);
   if (plugin != nullptr) {
     /*
-      We need to take global_sid_lock because
+      We need to take Gtid_mode::lock because
       group_replication_handler->start function will (among other
       things) do the following:
 
-       1. Call get_server_startup_prerequirements, which calls get_gtid_mode.
+       1. Call global_gtid_mode.get (the call stack is: this function
+          calls plugin_handle->start, which is equal to
+          plugin_group_replication_init (as declared at the end of
+          plugin/group_replication/src/plugin.cc), which calls
+          plugin_group_replication_start, which calls
+          check_if_server_properly_configured, which calls
+          get_server_startup_prerequirements, which calls
+          global_gtid_mode.get).
        2. Set plugin-internal state that ensures that
           is_group_replication_running() returns true.
 
       In order to prevent a concurrent client from executing SET
       GTID_MODE=ON_PERMISSIVE between 1 and 2, we must hold
-      gtid_mode_lock.
+      Gtid_mode::lock.
     */
-    gtid_mode_lock->rdlock();
+    Checkable_rwlock::Guard g(Gtid_mode::lock, Checkable_rwlock::READ_LOCK);
     st_mysql_group_replication *plugin_handle =
         (st_mysql_group_replication *)plugin_decl(plugin)->info;
-    result = plugin_handle->start(error_message);
-    gtid_mode_lock->unlock();
+    /*
+      is_running check is required below before storing credentials.
+      Check makes sure runing instance of START GR is not impacted by
+      temporary storage of credentials or if storing credential failed
+      message is meaningful.
+      e.g. of credential conflict blocked by below check
+      1: START GROUP_REPLICATION;
+         START GROUP_REPLICATION USER="abc" , PASSWORD="xyz";
+      2: START GROUP_REPLICATION USER="abc" , PASSWORD="xyz";
+         START GROUP_REPLICATION USER="user" , PASSWORD="pass";
+     */
+    if (plugin_handle->is_running()) {
+      result = 2;
+      goto err;
+    }
+    if (lex != nullptr &&
+        (lex->slave_connection.user || lex->slave_connection.plugin_auth)) {
+      if (Rpl_channel_credentials::get_instance().store_credentials(
+              "group_replication_recovery", lex->slave_connection.user,
+              lex->slave_connection.password,
+              lex->slave_connection.plugin_auth)) {
+        // Parallel START/STOP GR is blocked.
+        // So by now START GR command should fail if running or stop should have
+        // cleared credentials. Post UNINSTALL we should not reach here.
+        /* purecov: begin inspected */
+        DBUG_ASSERT(false);
+        result = 2;
+        goto err;
+        /* purecov: end */
+      }
+    }
 
-    plugin_unlock(nullptr, plugin);
+    result = plugin_handle->start(error_message);
+    if (result)
+      Rpl_channel_credentials::get_instance().delete_credentials(
+          "group_replication_recovery");
   } else {
     LogErr(ERROR_LEVEL, ER_GROUP_REPLICATION_PLUGIN_NOT_INSTALLED);
   }
-
+err:
+  if (plugin != nullptr) plugin_unlock(nullptr, plugin);
+  start_stop_executing.clear();
   return result;
 }
 
 int group_replication_stop(char **error_message) {
   int result = 1;
+  plugin_ref plugin = nullptr;
 
-  plugin_ref plugin =
-      my_plugin_lock_by_name(nullptr, group_replication_plugin_name_str,
-                             MYSQL_GROUP_REPLICATION_PLUGIN);
+  if (start_stop_executing.test_and_set()) {
+    std::string msg;
+    msg.assign(
+        "Another instance of START/STOP GROUP_REPLICATION command is "
+        "executing.");
+    *error_message =
+        (char *)my_malloc(PSI_NOT_INSTRUMENTED, msg.size() + 1, MYF(0));
+    strcpy(*error_message, msg.c_str());
+    result = 8;
+    return result;
+  }
+  plugin = my_plugin_lock_by_name(nullptr, group_replication_plugin_name_str,
+                                  MYSQL_GROUP_REPLICATION_PLUGIN);
   if (plugin != nullptr) {
     st_mysql_group_replication *plugin_handle =
         (st_mysql_group_replication *)plugin_decl(plugin)->info;
@@ -159,7 +228,7 @@ int group_replication_stop(char **error_message) {
   } else {
     LogErr(ERROR_LEVEL, ER_GROUP_REPLICATION_PLUGIN_NOT_INSTALLED);
   }
-
+  start_stop_executing.clear();
   return result;
 }
 
@@ -293,7 +362,8 @@ inline char *my_strdup_nullable(OptionalString from) {
   include/mysql/group_replication_priv.h
 */
 void get_server_parameters(char **hostname, uint *port, char **uuid,
-                           unsigned int *out_server_version) {
+                           unsigned int *out_server_version,
+                           uint *out_admin_port) {
   /*
     use startup option report-host and report-port when provided,
     as value provided by glob_hostname, which used gethostname() function
@@ -338,15 +408,40 @@ void get_server_parameters(char **hostname, uint *port, char **uuid,
   *out_server_version =
       v0 + v1 * 16 + v2 * 256 + v3 * 4096 + v4 * 65536 + v5 * 1048576;
 
+  *out_admin_port = mysqld_admin_port;
+
   return;
 }
 
-void get_server_ssl_parameters(st_server_ssl_variables *server_ssl_variables) {
+void get_server_main_ssl_parameters(
+    st_server_ssl_variables *server_ssl_variables) {
   OptionalString ca, capath, cert, cipher, ciphersuites, key, crl, crlpath,
       version;
 
-  SslAcceptorContext::read_parameters(&ca, &capath, &version, &cert, &cipher,
-                                      &ciphersuites, &key, &crl, &crlpath);
+  server_main_callback.read_parameters(&ca, &capath, &version, &cert, &cipher,
+                                       &ciphersuites, &key, &crl, &crlpath);
+
+  server_ssl_variables->ssl_ca = my_strdup_nullable(ca);
+  server_ssl_variables->ssl_capath = my_strdup_nullable(capath);
+  server_ssl_variables->tls_version = my_strdup_nullable(version);
+  server_ssl_variables->tls_ciphersuites = my_strdup_nullable(ciphersuites);
+  server_ssl_variables->ssl_cert = my_strdup_nullable(cert);
+  server_ssl_variables->ssl_cipher = my_strdup_nullable(cipher);
+  server_ssl_variables->ssl_key = my_strdup_nullable(key);
+  server_ssl_variables->ssl_crl = my_strdup_nullable(crl);
+  server_ssl_variables->ssl_crlpath = my_strdup_nullable(crlpath);
+  server_ssl_variables->ssl_fips_mode = opt_ssl_fips_mode;
+
+  return;
+}
+
+void get_server_admin_ssl_parameters(
+    st_server_ssl_variables *server_ssl_variables) {
+  OptionalString ca, capath, cert, cipher, ciphersuites, key, crl, crlpath,
+      version;
+
+  server_admin_callback.read_parameters(&ca, &capath, &version, &cert, &cipher,
+                                        &ciphersuites, &key, &crl, &crlpath);
 
   server_ssl_variables->ssl_ca = my_strdup_nullable(ca);
   server_ssl_variables->ssl_capath = my_strdup_nullable(capath);
@@ -380,13 +475,11 @@ void set_auto_increment_offset(ulong auto_increment_offset) {
   global_system_variables.auto_increment_offset = auto_increment_offset;
 }
 
-void get_server_startup_prerequirements(Trans_context_info &requirements,
-                                        bool has_lock) {
+void get_server_startup_prerequirements(Trans_context_info &requirements) {
   requirements.binlog_enabled = opt_bin_log;
   requirements.binlog_format = global_system_variables.binlog_format;
   requirements.binlog_checksum_options = binlog_checksum_options;
-  requirements.gtid_mode =
-      get_gtid_mode(has_lock ? GTID_MODE_LOCK_GTID_MODE : GTID_MODE_LOCK_NONE);
+  requirements.gtid_mode = global_gtid_mode.get();
   requirements.log_slave_updates = opt_log_slave_updates;
   requirements.transaction_write_set_extraction =
       global_system_variables.transaction_write_set_extraction;
@@ -403,21 +496,17 @@ void get_server_startup_prerequirements(Trans_context_info &requirements,
 
 bool get_server_encoded_gtid_executed(uchar **encoded_gtid_executed,
                                       size_t *length) {
-  global_sid_lock->wrlock();
+  Checkable_rwlock::Guard g(*global_sid_lock, Checkable_rwlock::WRITE_LOCK);
 
-  DBUG_ASSERT(get_gtid_mode(GTID_MODE_LOCK_SID) > 0);
+  DBUG_ASSERT(global_gtid_mode.get() != Gtid_mode::OFF);
 
   const Gtid_set *executed_gtids = gtid_state->get_executed_gtids();
   *length = executed_gtids->get_encoded_length();
   *encoded_gtid_executed =
       (uchar *)my_malloc(key_memory_Gtid_set_to_string, *length, MYF(MY_WME));
-  if (*encoded_gtid_executed == nullptr) {
-    global_sid_lock->unlock();
-    return true;
-  }
+  if (*encoded_gtid_executed == nullptr) return true;
 
   executed_gtids->encode(*encoded_gtid_executed);
-  global_sid_lock->unlock();
   return false;
 }
 
@@ -445,14 +534,14 @@ void global_thd_manager_remove_thd(THD *thd) {
 }
 
 bool is_gtid_committed(const Gtid &gtid) {
-  bool result = false;
-  global_sid_lock->rdlock();
+  Checkable_rwlock::Guard g(*global_sid_lock, Checkable_rwlock::READ_LOCK);
 
-  DBUG_ASSERT(get_gtid_mode(GTID_MODE_LOCK_SID) > 0);
+  DBUG_ASSERT(global_gtid_mode.get() != Gtid_mode::OFF);
 
-  result = gtid_state->is_executed(gtid);
+  gtid_state->lock_sidno(gtid.sidno);
+  bool result = gtid_state->is_executed(gtid);
+  gtid_state->unlock_sidno(gtid.sidno);
 
-  global_sid_lock->unlock();
   return result;
 }
 
@@ -467,3 +556,31 @@ unsigned long get_max_slave_max_allowed_packet() {
 bool is_server_restarting_after_clone() { return clone_startup; }
 
 bool is_server_data_dropped() { return Clone_handler::is_data_dropped(); }
+
+std::string get_group_replication_group_name() {
+  std::string group_name{""};
+  auto set_channel_name_lambda = [](void *const, const char &, size_t) {};
+  auto set_source_uuid_lambda = [](void *const, const char &, size_t) {};
+  auto set_service_state_lambda = [](void *const, bool) {};
+  auto set_group_name_lambda = [](void *const context, const char &value,
+                                  size_t length) {
+    std::string *group_name_ptr = static_cast<std::string *>(context);
+    const size_t max = UUID_LENGTH;
+    length = std::min(length, max);
+    group_name_ptr->assign(&value, length);
+  };
+
+  const GROUP_REPLICATION_CONNECTION_STATUS_CALLBACKS callbacks = {
+      &group_name,
+      set_channel_name_lambda,
+      set_group_name_lambda,
+      set_source_uuid_lambda,
+      set_service_state_lambda,
+  };
+
+  // Query plugin and let callbacks do their job.
+  if (get_group_replication_connection_status_info(callbacks)) {
+    DBUG_PRINT("info", ("Group Replication stats not available!"));
+  }
+  return group_name;
+}
