@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,14 +24,14 @@
 #include <atomic>
 
 #include "lex_string.h"
+#include "libbinlogevents/include/control_events.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
 #include "my_systime.h"
-#include "mysql/binlog/event/control_events.h"
-#include "mysql/components/services/bits/psi_stage_bits.h"
 #include "mysql/components/services/log_builtins.h"  //LogErr
+#include "mysql/components/services/psi_stage_bits.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"
@@ -48,17 +48,15 @@
 #include "sql/system_variables.h"
 #include "sql/thr_malloc.h"
 
-class Table_ref;
+struct TABLE_LIST;
 
 PSI_memory_key key_memory_Gtid_state_group_commit_sidno;
 
-using Tsid = mysql::gtid::Tsid;
-
 int Gtid_state::clear(THD *thd) {
   DBUG_TRACE;
   int ret = 0;
   // the wrlock implies that no other thread can hold any of the mutexes
-  tsid_lock->assert_some_wrlock();
+  sid_lock->assert_some_wrlock();
   lost_gtids.clear();
   executed_gtids.clear();
   gtids_only_in_table.clear();
@@ -72,18 +70,18 @@ int Gtid_state::clear(THD *thd) {
     thd->clear_error();
     ret = 0;
   }
-  next_free_gno_map.clear();
+  next_free_gno = 1;
   return ret;
 }
 
 enum_return_status Gtid_state::acquire_ownership(THD *thd, const Gtid &gtid) {
   DBUG_TRACE;
-  // caller must take both global_tsid_lock and lock on the SIDNO.
-  global_tsid_lock->assert_some_lock();
+  // caller must take both global_sid_lock and lock on the SIDNO.
+  global_sid_lock->assert_some_lock();
   gtid_state->assert_sidno_lock_owner(gtid.sidno);
-  assert(!executed_gtids.contains_gtid(gtid));
-  DBUG_PRINT("info", ("gtid=%d:%" PRId64, gtid.sidno, gtid.gno));
-  assert(thd->owned_gtid.sidno == 0);
+  DBUG_ASSERT(!executed_gtids.contains_gtid(gtid));
+  DBUG_PRINT("info", ("gtid=%d:%lld", gtid.sidno, gtid.gno));
+  DBUG_ASSERT(thd->owned_gtid.sidno == 0);
   if (owned_gtids.add_gtid_owner(gtid, thd->thread_id()) != RETURN_STATUS_OK)
     goto err;
   if (thd->get_gtid_next_list() != nullptr) {
@@ -92,14 +90,13 @@ enum_return_status Gtid_state::acquire_ownership(THD *thd, const Gtid &gtid) {
     thd->owned_gtid.sidno = THD::OWNED_SIDNO_GTID_SET;
     thd->owned_sid.clear();
 #else
-    assert(0);
+    DBUG_ASSERT(0);
 #endif
   } else {
     thd->owned_gtid = gtid;
     thd->owned_gtid.dbug_print(nullptr, "set owned_gtid in acquire_ownership");
-    thd->owned_tsid = tsid_map->sidno_to_tsid(gtid.sidno);
-    thd->rpl_thd_ctx.last_used_gtid_tracker_ctx().set_last_used_gtid(
-        gtid, thd->owned_tsid);
+    thd->owned_sid = sid_map->sidno_to_sid(gtid.sidno);
+    thd->rpl_thd_ctx.last_used_gtid_tracker_ctx().set_last_used_gtid(gtid);
   }
   RETURN_OK;
 err:
@@ -112,7 +109,7 @@ err:
       g = git.get();
     }
 #else
-    assert(0);
+    DBUG_ASSERT(0);
 #endif
   }
   thd->clear_owned_gtids();
@@ -135,7 +132,7 @@ void Gtid_state::unlock_owned_sidnos(const THD *thd) {
 #ifdef HAVE_GTID_NEXT_LIST
     unlock_sidnos(&thd->owned_gtid_set);
 #else
-    assert(0);
+    DBUG_ASSERT(0);
 #endif
   } else if (thd->owned_gtid.sidno > 0) {
     unlock_sidno(thd->owned_gtid.sidno);
@@ -147,7 +144,7 @@ void Gtid_state::broadcast_owned_sidnos(const THD *thd) {
 #ifdef HAVE_GTID_NEXT_LIST
     broadcast_sidnos(&thd->owned_gtid_set);
 #else
-    assert(0);
+    DBUG_ASSERT(0);
 #endif
   } else if (thd->owned_gtid.sidno > 0) {
     broadcast_sidno(thd->owned_gtid.sidno);
@@ -160,12 +157,12 @@ void Gtid_state::update_commit_group(THD *first_thd) {
   bool gtid_threshold_breach = false;
   /*
     We are going to loop in all sessions of the group commit in order to avoid
-    being taking and releasing the global_tsid_lock and sidno_lock for each
+    being taking and releasing the global_sid_lock and sidno_lock for each
     session.
   */
-  DEBUG_SYNC(first_thd, "update_gtid_state_before_global_tsid_lock");
-  global_tsid_lock->rdlock();
-  DEBUG_SYNC(first_thd, "update_gtid_state_after_global_tsid_lock");
+  DEBUG_SYNC(first_thd, "update_gtid_state_before_global_sid_lock");
+  global_sid_lock->rdlock();
+  DEBUG_SYNC(first_thd, "update_gtid_state_after_global_sid_lock");
 
   update_gtids_impl_lock_sidnos(first_thd);
 
@@ -196,7 +193,7 @@ void Gtid_state::update_commit_group(THD *first_thd) {
 
   update_gtids_impl_broadcast_and_unlock_sidnos();
 
-  global_tsid_lock->unlock();
+  global_sid_lock->unlock();
 
   if (gtid_threshold_breach)
     LogErr(WARNING_LEVEL, ER_WARN_GTID_THRESHOLD_BREACH);
@@ -223,9 +220,9 @@ void Gtid_state::update_gtids_impl(THD *thd, bool is_commit) {
 
   bool more_trx_with_same_gtid_next = update_gtids_impl_begin(thd);
 
-  DEBUG_SYNC(thd, "update_gtid_state_before_global_tsid_lock");
-  global_tsid_lock->rdlock();
-  DEBUG_SYNC(thd, "update_gtid_state_after_global_tsid_lock");
+  DEBUG_SYNC(thd, "update_gtid_state_before_global_sid_lock");
+  global_sid_lock->rdlock();
+  DEBUG_SYNC(thd, "update_gtid_state_after_global_sid_lock");
   bool gtid_threshold_breach = (thd->owned_gtid.gno > GNO_WARNING_THRESHOLD);
 
   if (thd->owned_gtid.sidno == THD::OWNED_SIDNO_GTID_SET) {
@@ -241,7 +238,7 @@ void Gtid_state::update_gtids_impl(THD *thd, bool is_commit) {
     update_gtids_impl_own_nothing(thd);
   }
 
-  global_tsid_lock->unlock();
+  global_sid_lock->unlock();
 
   update_gtids_impl_end(thd, more_trx_with_same_gtid_next);
 
@@ -258,7 +255,7 @@ void Gtid_state::end_gtid_violating_transaction(THD *thd) {
     if (thd->variables.gtid_next.type == AUTOMATIC_GTID)
       end_automatic_gtid_violating_transaction();
     else {
-      assert(thd->variables.gtid_next.type == ANONYMOUS_GTID);
+      DBUG_ASSERT(thd->variables.gtid_next.type == ANONYMOUS_GTID);
       end_anonymous_gtid_violating_transaction();
     }
     thd->has_gtid_consistency_violation = false;
@@ -266,23 +263,15 @@ void Gtid_state::end_gtid_violating_transaction(THD *thd) {
 }
 
 bool Gtid_state::wait_for_sidno(THD *thd, rpl_sidno sidno,
-                                struct timespec *abstime,
-                                bool update_thd_status) {
+                                struct timespec *abstime) {
   DBUG_TRACE;
   PSI_stage_info old_stage;
-  PSI_stage_info stage = stage_waiting_for_gtid_to_be_committed;
-  tsid_lock->assert_some_lock();
-  tsid_locks.assert_owner(sidno);
-
-  if (!update_thd_status) {
-    // Keep the same stage info on the new condition.
-    stage.m_key = thd->get_current_stage_key();
-    stage.m_name = thd->proc_info();
-  }
-
-  tsid_locks.enter_cond(thd, sidno, &stage, &old_stage);
-  bool ret = tsid_locks.wait(thd, sidno, abstime);
-  // Can't call tsid_locks.unlock() as that requires global_tsid_lock.
+  sid_lock->assert_some_lock();
+  sid_locks.assert_owner(sidno);
+  sid_locks.enter_cond(thd, sidno, &stage_waiting_for_gtid_to_be_committed,
+                       &old_stage);
+  bool ret = sid_locks.wait(thd, sidno, abstime);
+  // Can't call sid_locks.unlock() as that requires global_sid_lock.
   mysql_mutex_unlock(thd->current_mutex);
   thd->EXIT_COND(&old_stage);
   return ret;
@@ -291,26 +280,26 @@ bool Gtid_state::wait_for_sidno(THD *thd, rpl_sidno sidno,
 bool Gtid_state::wait_for_gtid(THD *thd, const Gtid &gtid,
                                struct timespec *abstime) {
   DBUG_TRACE;
-  DBUG_PRINT("info", ("SIDNO=%d GNO=%" PRId64 " thread_id=%u", gtid.sidno,
-                      gtid.gno, thd->thread_id()));
-  assert(!owned_gtids.is_owned_by(gtid, thd->thread_id()));
-  assert(!owned_gtids.is_owned_by(gtid, 0));
+  DBUG_PRINT("info", ("SIDNO=%d GNO=%lld thread_id=%u", gtid.sidno, gtid.gno,
+                      thd->thread_id()));
+  DBUG_ASSERT(!owned_gtids.is_owned_by(gtid, thd->thread_id()));
+  DBUG_ASSERT(!owned_gtids.is_owned_by(gtid, 0));
 
   bool ret = wait_for_sidno(thd, gtid.sidno, abstime);
   return ret;
 }
 
-bool Gtid_state::wait_for_gtid_set(THD *thd, Gtid_set *wait_for, double timeout,
-                                   bool update_thd_status) {
+bool Gtid_state::wait_for_gtid_set(THD *thd, Gtid_set *wait_for,
+                                   double timeout) {
   struct timespec abstime;
   DBUG_TRACE;
   DEBUG_SYNC(thd, "begin_wait_for_executed_gtid_set");
   wait_for->dbug_print("Waiting for");
   DBUG_PRINT("info", ("Timeout %f", timeout));
 
-  global_tsid_lock->assert_some_rdlock();
+  global_sid_lock->assert_some_rdlock();
 
-  assert(wait_for->get_tsid_map() == global_tsid_map);
+  DBUG_ASSERT(wait_for->get_sid_map() == global_sid_map);
 
   if (timeout > 0) {
     set_timespec_nsec(&abstime,
@@ -331,26 +320,26 @@ bool Gtid_state::wait_for_gtid_set(THD *thd, Gtid_set *wait_for, double timeout,
 
     Once the loop over SIDNOs has completed, 'todo' is guaranteed to
     be empty.  However, it may still be the case that not all GTIDs of
-    wait_for are included in gtid_executed, since RESET BINARY LOGS AND GTIDS
-    may have been executed while we were waiting.
+    wait_for are included in gtid_executed, since RESET MASTER may
+    have been executed while we were waiting.
 
-    RESET BINARY LOGS AND GTIDS requires global_tsid_lock.wrlock.  We hold
-    global_tsid_lock.rdlock while removing GTIDs from 'todo', but the
-    wait operation releases global_tsid_lock.rdlock.  So if we
+    RESET MASTER requires global_sid_lock.wrlock.  We hold
+    global_sid_lock.rdlock while removing GTIDs from 'todo', but the
+    wait operation releases global_sid_lock.rdlock.  So if we
     completed the 'for' loop without waiting, we know for sure that
-    global_tsid_lock.rdlock was held while emptying 'todo', and thus
-    RESET BINARY LOGS AND GTIDS cannot have executed in the meantime.
-    But if we waited at some point during the execution of the 'for'
-    loop, RESET BINARY LOGS AND GTIDS may have been called.
-    Thus, we repeatedly run 'for' loop until it completes without waiting
-    (this is the outermost 'while' loop).
+    global_sid_lock.rdlock was held while emptying 'todo', and thus
+    RESET MASTER cannot have executed in the meantime.  But if we
+    waited at some point during the execution of the 'for' loop, RESET
+    MASTER may have been called.  Thus, we repeatedly run 'for' loop
+    until it completes without waiting (this is the outermost 'while'
+    loop).
   */
 
   // Will be true once the entire 'for' loop completes without waiting.
   bool verified = false;
 
   // The set of GTIDs that we are still waiting for.
-  Gtid_set todo(global_tsid_map, nullptr);
+  Gtid_set todo(global_sid_map, nullptr);
   // As an optimization, add 100 Intervals that do not need to be
   // allocated. This avoids allocation of these intervals.
   static const int preallocated_interval_count = 100;
@@ -376,12 +365,12 @@ bool Gtid_state::wait_for_gtid_set(THD *thd, Gtid_set *wait_for, double timeout,
         todo.remove_intervals_for_sidno(&executed_gtids, sidno);
 
         if (todo.contains_sidno(sidno)) {
-          bool ret = wait_for_sidno(
-              thd, sidno, timeout > 0 ? &abstime : nullptr, update_thd_status);
+          bool ret =
+              wait_for_sidno(thd, sidno, timeout > 0 ? &abstime : nullptr);
 
           // wait_for_gtid will release both the global lock and the
           // mutex.  Acquire the global lock again.
-          global_tsid_lock->rdlock();
+          global_sid_lock->rdlock();
           verified = false;
 
           if (thd->killed) {
@@ -439,18 +428,15 @@ rpl_gno Gtid_state::get_automatic_gno(rpl_sidno sidno) const {
     than the next_free_gno at Gtid_state::update_gtids_impl_own_gtid function
     to set next_free_gno with the "released" GNO in this case.
   */
-  auto next_free_gno_res = next_free_gno_map.find(sidno);
-
-  Gtid next_candidate = {sidno, next_free_gno_res == next_free_gno_map.end()
-                                    ? 1
-                                    : next_free_gno_res->second};
+  Gtid next_candidate = {sidno,
+                         sidno == get_server_sidno() ? next_free_gno : 1};
   while (true) {
     const Gtid_set::Interval *iv = ivit.get();
-    rpl_gno next_interval_start = iv != nullptr ? iv->start : GNO_END;
+    rpl_gno next_interval_start = iv != nullptr ? iv->start : MAX_GNO;
     while (next_candidate.gno < next_interval_start &&
            DBUG_EVALUATE_IF("simulate_gno_exhausted", false, true)) {
-      DBUG_PRINT("debug", ("Checking availability of gno= %" PRId64,
-                           next_candidate.gno));
+      DBUG_PRINT("debug",
+                 ("Checking availability of gno= %llu", next_candidate.gno));
       if (owned_gtids.is_owned_by(next_candidate, 0)) return next_candidate.gno;
       next_candidate.gno++;
     }
@@ -459,7 +445,7 @@ rpl_gno Gtid_state::get_automatic_gno(rpl_sidno sidno) const {
       my_error(ER_GNO_EXHAUSTED, MYF(0));
       return -1;
     }
-    if (next_candidate.gno < iv->end) next_candidate.gno = iv->end;
+    if (next_candidate.gno <= iv->end) next_candidate.gno = iv->end;
     ivit.next();
   }
 }
@@ -475,52 +461,64 @@ rpl_gno Gtid_state::get_last_executed_gno(rpl_sidno sidno) const {
   return gno;
 }
 
-rpl_sidno Gtid_state::specify_transaction_sidno(
-    THD *thd, Gtid_state::Locked_sidno_set &sidno_set) {
-  DBUG_TRACE;
-  const auto &gtid_next = thd->variables.gtid_next;
-  rpl_sidno sidno =
-      thd->get_transaction()->get_rpl_transaction_ctx()->get_sidno();
-  if (gtid_next.is_automatic() &&
-      global_gtid_mode.get() >= Gtid_mode::ON_PERMISSIVE) {
-    if (gtid_next.is_automatic_tagged() && sidno == 0) {
-      sidno = global_tsid_map->add_tsid(
-          Tsid(get_server_tsid().get_uuid(), gtid_next.generate_tag()));
-    }
-    if (sidno == 0) {
-      sidno = get_server_sidno();
-    }
-    sidno_set.add_lock_for_sidno(sidno);
-  }
-  return sidno;
-}
-
 enum_return_status Gtid_state::generate_automatic_gtid(
-    THD *thd, rpl_sidno specified_sidno, rpl_gno specified_gno) {
+    THD *thd, rpl_sidno specified_sidno, rpl_gno specified_gno,
+    rpl_sidno *locked_sidno) {
   DBUG_TRACE;
   enum_return_status ret = RETURN_STATUS_OK;
 
-  [[maybe_unused]] const auto &gtid_next = thd->variables.gtid_next;
+  DBUG_ASSERT(thd->variables.gtid_next.type == AUTOMATIC_GTID);
+  DBUG_ASSERT(specified_sidno >= 0);
+  DBUG_ASSERT(specified_gno >= 0);
+  DBUG_ASSERT(thd->owned_gtid.is_empty());
 
-  assert(gtid_next.is_automatic());
-  assert(specified_sidno >= 0);
-  assert(specified_gno >= 0);
-  assert(thd->owned_gtid.is_empty());
+  bool locked_sidno_was_passed_null = (locked_sidno == nullptr);
+
+  if (locked_sidno_was_passed_null)
+    sid_lock->rdlock();
+  else
+    /* The caller must lock the sid_lock when locked_sidno is passed */
+    sid_lock->assert_some_lock();
 
   // If GTID_MODE = ON_PERMISSIVE or ON, generate a new GTID
   if (global_gtid_mode.get() >= Gtid_mode::ON_PERMISSIVE) {
     Gtid automatic_gtid = {specified_sidno, specified_gno};
-    assert(specified_sidno > 0);  // assert that sidno has been already assigned
+
+    if (automatic_gtid.sidno == 0) automatic_gtid.sidno = get_server_sidno();
+
+    /*
+      We need to lock the sidno if locked_sidno wasn't passed as paramenter
+      or the already locked sidno doesn't match the one to generate the new
+      automatic GTID.
+    */
+    bool need_to_lock_sidno =
+        (locked_sidno_was_passed_null || *locked_sidno != automatic_gtid.sidno);
+    if (need_to_lock_sidno) {
+      /*
+        When locked_sidno contains a value greater than zero we must release
+        the current locked sidno. This should not happen with current code, as
+        the server only generates automatic GTIDs with server's UUID as sid.
+      */
+      if (!locked_sidno_was_passed_null && *locked_sidno != 0)
+        unlock_sidno(*locked_sidno);
+      lock_sidno(automatic_gtid.sidno);
+      /* Update the locked_sidno, so the caller would know what to unlock */
+      if (!locked_sidno_was_passed_null) *locked_sidno = automatic_gtid.sidno;
+    }
+
     if (automatic_gtid.gno == 0) {
       automatic_gtid.gno = get_automatic_gno(automatic_gtid.sidno);
-      if (automatic_gtid.gno != -1) {
-        // insert new element or update the map element
-        next_free_gno_map[automatic_gtid.sidno] = automatic_gtid.gno + 1;
-      }
+      if (automatic_gtid.sidno == get_server_sidno() &&
+          automatic_gtid.gno != -1)
+        next_free_gno = automatic_gtid.gno + 1;
     }
+
     if (automatic_gtid.gno == -1 || acquire_ownership(thd, automatic_gtid))
       ret = RETURN_STATUS_REPORTED_ERROR;
 
+    /* The caller will unlock the sidno_lock if locked_sidno was passed */
+    if (locked_sidno_was_passed_null) unlock_sidno(automatic_gtid.sidno);
+
   } else {
     // If GTID_MODE = OFF or OFF_PERMISSIVE, just mark this thread as
     // using an anonymous transaction.
@@ -531,27 +529,30 @@ enum_return_status Gtid_state::generate_automatic_gtid(
         nullptr, "set owned_gtid (anonymous) in generate_automatic_gtid");
   }
 
+  /* The caller will unlock the sid_lock if locked_sidno was passed */
+  if (locked_sidno_was_passed_null) sid_lock->unlock();
+
   gtid_set_performance_schema_values(thd);
 
   return ret;
 }
 
 void Gtid_state::lock_sidnos(const Gtid_set *gs) {
-  assert(gs);
+  DBUG_ASSERT(gs);
   rpl_sidno max_sidno = gs->get_max_sidno();
   for (rpl_sidno sidno = 1; sidno <= max_sidno; sidno++)
     if (gs->contains_sidno(sidno)) lock_sidno(sidno);
 }
 
 void Gtid_state::unlock_sidnos(const Gtid_set *gs) {
-  assert(gs);
+  DBUG_ASSERT(gs);
   rpl_sidno max_sidno = gs->get_max_sidno();
   for (rpl_sidno sidno = 1; sidno <= max_sidno; sidno++)
     if (gs->contains_sidno(sidno)) unlock_sidno(sidno);
 }
 
 void Gtid_state::broadcast_sidnos(const Gtid_set *gs) {
-  assert(gs);
+  DBUG_ASSERT(gs);
   rpl_sidno max_sidno = gs->get_max_sidno();
   for (rpl_sidno sidno = 1; sidno <= max_sidno; sidno++)
     if (gs->contains_sidno(sidno)) broadcast_sidno(sidno);
@@ -559,8 +560,8 @@ void Gtid_state::broadcast_sidnos(const Gtid_set *gs) {
 
 enum_return_status Gtid_state::ensure_sidno() {
   DBUG_TRACE;
-  tsid_lock->assert_some_wrlock();
-  rpl_sidno sidno = tsid_map->get_max_sidno();
+  sid_lock->assert_some_wrlock();
+  rpl_sidno sidno = sid_map->get_max_sidno();
   if (sidno > 0) {
     // The lock may be temporarily released during one of the calls to
     // ensure_sidno or ensure_index.  Hence, we must re-check the
@@ -570,16 +571,16 @@ enum_return_status Gtid_state::ensure_sidno() {
     PROPAGATE_REPORTED_ERROR(previous_gtids_logged.ensure_sidno(sidno));
     PROPAGATE_REPORTED_ERROR(lost_gtids.ensure_sidno(sidno));
     PROPAGATE_REPORTED_ERROR(owned_gtids.ensure_sidno(sidno));
-    PROPAGATE_REPORTED_ERROR(tsid_locks.ensure_index(sidno));
+    PROPAGATE_REPORTED_ERROR(sid_locks.ensure_index(sidno));
     PROPAGATE_REPORTED_ERROR(ensure_commit_group_sidnos(sidno));
-    sidno = tsid_map->get_max_sidno();
-    assert(executed_gtids.get_max_sidno() >= sidno);
-    assert(gtids_only_in_table.get_max_sidno() >= sidno);
-    assert(previous_gtids_logged.get_max_sidno() >= sidno);
-    assert(lost_gtids.get_max_sidno() >= sidno);
-    assert(owned_gtids.get_max_sidno() >= sidno);
-    assert(tsid_locks.get_max_index() >= sidno);
-    assert(commit_group_sidnos.size() >= (unsigned int)sidno);
+    sidno = sid_map->get_max_sidno();
+    DBUG_ASSERT(executed_gtids.get_max_sidno() >= sidno);
+    DBUG_ASSERT(gtids_only_in_table.get_max_sidno() >= sidno);
+    DBUG_ASSERT(previous_gtids_logged.get_max_sidno() >= sidno);
+    DBUG_ASSERT(lost_gtids.get_max_sidno() >= sidno);
+    DBUG_ASSERT(owned_gtids.get_max_sidno() >= sidno);
+    DBUG_ASSERT(sid_locks.get_max_index() >= sidno);
+    DBUG_ASSERT(commit_group_sidnos.size() >= (unsigned int)sidno);
   }
   RETURN_OK;
 }
@@ -591,7 +592,7 @@ void Gtid_state::update_prev_gtids(Gtid_set *write_gtid_set) {
   if (!opt_bin_log) {
     DBUG_VOID_RETURN;
   }
-  global_tsid_lock->wrlock();
+  global_sid_lock->wrlock();
 
   /* Remove from list if GTID is already written. */
   write_gtid_set->remove_gtid_set(&previous_gtids_logged);
@@ -599,14 +600,14 @@ void Gtid_state::update_prev_gtids(Gtid_set *write_gtid_set) {
   /* Add to the list so that it won't be written again later. */
   previous_gtids_logged.add_gtid_set(write_gtid_set);
 
-  global_tsid_lock->unlock();
+  global_sid_lock->unlock();
   DBUG_VOID_RETURN;
 }
 
 enum_return_status Gtid_state::add_lost_gtids(Gtid_set *gtid_set,
                                               bool starts_with_plus) {
   DBUG_TRACE;
-  tsid_lock->assert_some_wrlock();
+  sid_lock->assert_some_wrlock();
 
   gtid_set->dbug_print("add_lost_gtids");
 
@@ -628,7 +629,7 @@ enum_return_status Gtid_state::add_lost_gtids(Gtid_set *gtid_set,
              "the added gtid set must not overlap with @@GLOBAL.GTID_EXECUTED");
     RETURN_REPORTED_ERROR;
   }
-  assert(!lost_gtids.is_intersection_nonempty(gtid_set));
+  DBUG_ASSERT(!lost_gtids.is_intersection_nonempty(gtid_set));
 
   if (owned_gtids.is_intersection_nonempty(gtid_set)) {
     my_error(ER_CANT_SET_GTID_PURGED_DUE_SETS_CONSTRAINTS, MYF(0),
@@ -650,23 +651,22 @@ enum_return_status Gtid_state::add_lost_gtids(Gtid_set *gtid_set,
 int Gtid_state::init() {
   DBUG_TRACE;
 
-  global_tsid_lock->assert_some_wrlock();
+  global_sid_lock->assert_some_wrlock();
 
-  rpl_sid server_sid{};
-  if (server_sid.parse(server_uuid, mysql::gtid::Uuid::TEXT_LENGTH) != 0)
+  rpl_sid server_sid;
+  if (server_sid.parse(server_uuid, binary_log::Uuid::TEXT_LENGTH) != 0)
     return 1;
-  rpl_sidno sidno =
-      tsid_map->add_tsid(Tsid_map::Tsid(server_sid, Tsid_map::Tag()));
+  rpl_sidno sidno = sid_map->add_sid(server_sid);
   if (sidno <= 0) return 1;
   server_sidno = sidno;
-  next_free_gno_map.clear();
+  next_free_gno = 1;
   return 0;
 }
 
 int Gtid_state::save(THD *thd) {
   DBUG_TRACE;
-  assert(gtid_table_persistor != nullptr);
-  assert(thd->owned_gtid.sidno > 0);
+  DBUG_ASSERT(gtid_table_persistor != nullptr);
+  DBUG_ASSERT(thd->owned_gtid.sidno > 0);
   int error = 0;
 
   int ret = gtid_table_persistor->save(thd, &thd->owned_gtid);
@@ -701,11 +701,11 @@ int Gtid_state::save_gtids_of_last_binlog_into_table() {
   }
 
   /*
-    Use local Tsid_map, so that we don't need a lock while inserting
+    Use local Sid_map, so that we don't need a lock while inserting
     into the table.
   */
-  Tsid_map tsid_map(nullptr);
-  Gtid_set logged_gtids_last_binlog(&tsid_map, nullptr);
+  Sid_map sid_map(nullptr);
+  Gtid_set logged_gtids_last_binlog(&sid_map, nullptr);
   // Allocate some intervals on stack to reduce allocation.
   static const int PREALLOCATED_INTERVAL_COUNT = 64;
   Gtid_set::Interval iv[PREALLOCATED_INTERVAL_COUNT];
@@ -714,7 +714,7 @@ int Gtid_state::save_gtids_of_last_binlog_into_table() {
     logged_gtids_last_binlog= executed_gtids - previous_gtids_logged -
                               gtids_only_in_table
   */
-  global_tsid_lock->wrlock();
+  global_sid_lock->wrlock();
   ret = (logged_gtids_last_binlog.add_gtid_set(&executed_gtids) !=
          RETURN_STATUS_OK);
   if (!ret) {
@@ -727,16 +727,16 @@ int Gtid_state::save_gtids_of_last_binlog_into_table() {
       if (previous_gtids_logged.add_gtid_set(&logged_gtids_last_binlog))
         ret = ER_OOM_SAVE_GTIDS;
 
-      global_tsid_lock->unlock();
+      global_sid_lock->unlock();
       /* Save set of GTIDs of the last binlog into gtid_executed table */
       if (!ret) {
         if (save(&logged_gtids_last_binlog))
           ret = ER_RPL_GTID_TABLE_CANNOT_OPEN;
       }
     } else
-      global_tsid_lock->unlock();
+      global_sid_lock->unlock();
   } else
-    global_tsid_lock->unlock();
+    global_sid_lock->unlock();
 
   return ret;
 }
@@ -749,7 +749,7 @@ int Gtid_state::compress(THD *thd) {
   return gtid_table_persistor->compress(thd);
 }
 
-int Gtid_state::warn_or_err_on_modify_gtid_table(THD *thd, Table_ref *table) {
+int Gtid_state::warn_or_err_on_modify_gtid_table(THD *thd, TABLE_LIST *table) {
   DBUG_TRACE;
   int ret =
       gtid_table_persistor->warn_or_err_on_explicit_modification(thd, table);
@@ -779,7 +779,7 @@ bool Gtid_state::update_gtids_impl_do_nothing(THD *thd) {
 }
 
 bool Gtid_state::update_gtids_impl_begin(THD *thd) {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   if (current_thd != thd) mysql_mutex_lock(&thd->LOCK_thd_query);
   DBUG_PRINT("info", ("query='%s' thd->is_commit_in_middle_of_statement=%d",
                       thd->query().str, thd->is_commit_in_middle_of_statement));
@@ -788,15 +788,14 @@ bool Gtid_state::update_gtids_impl_begin(THD *thd) {
   return thd->is_commit_in_middle_of_statement;
 }
 
-void Gtid_state ::update_gtids_impl_own_gtid_set(THD *thd [[maybe_unused]],
-                                                 bool is_commit
-                                                 [[maybe_unused]]) {
+void Gtid_state ::update_gtids_impl_own_gtid_set(
+    THD *thd MY_ATTRIBUTE((unused)), bool is_commit MY_ATTRIBUTE((unused))) {
 #ifdef HAVE_GTID_NEXT_LIST
   rpl_sidno prev_sidno = 0;
   Gtid_set::Gtid_iterator git(&thd->owned_gtid_set);
   Gtid g = git.get();
   while (g.sidno != 0) {
-    if (g.sidno != prev_sidno) tsid_locks.lock(g.sidno);
+    if (g.sidno != prev_sidno) sid_locks.lock(g.sidno);
     owned_gtids.remove_gtid(g);
     git.next();
     g = git.get();
@@ -812,12 +811,12 @@ void Gtid_state ::update_gtids_impl_own_gtid_set(THD *thd [[maybe_unused]],
                              "in update_gtids_impl");
   thd->clear_owned_gtids();
 #else
-  assert(0);
+  DBUG_ASSERT(0);
 #endif
 }
 
 void Gtid_state::update_gtids_impl_lock_sidno(rpl_sidno sidno) {
-  assert(sidno > 0);
+  DBUG_ASSERT(sidno > 0);
   DBUG_PRINT("info", ("Locking sidno %d", sidno));
   lock_sidno(sidno);
 }
@@ -834,7 +833,7 @@ void Gtid_state::update_gtids_impl_lock_sidnos(THD *first_thd) {
       for (rpl_sidno i = 1; i < thd->owned_gtid_set.max_sidno; i++)
         if (owned_gtid_set.contains_sidno(i)) commit_group_sidnos[i] = true;
 #else
-      assert(0);
+      DBUG_ASSERT(0);
 #endif
   }
 
@@ -849,11 +848,11 @@ void Gtid_state::update_gtids_impl_own_gtid(THD *thd, bool is_commit) {
     In Group Replication the GTID may additionally be owned by another
     thread, and we won't remove that ownership (it will be rolled back later)
   */
-  assert(owned_gtids.is_owned_by(thd->owned_gtid, thd->thread_id()));
+  DBUG_ASSERT(owned_gtids.is_owned_by(thd->owned_gtid, thd->thread_id()));
   owned_gtids.remove_gtid(thd->owned_gtid, thd->thread_id());
 
   if (is_commit) {
-    assert(!executed_gtids.contains_gtid(thd->owned_gtid));
+    DBUG_ASSERT(!executed_gtids.contains_gtid(thd->owned_gtid));
     DBUG_EXECUTE_IF(
         "rpl_gtid_update_on_commit_simulate_out_of_memory",
         DBUG_SET("+d,rpl_gtid_get_free_interval_simulate_out_of_memory"););
@@ -865,30 +864,25 @@ void Gtid_state::update_gtids_impl_own_gtid(THD *thd, bool is_commit) {
       are always same, so we did not save gtid into lost_gtids for every
       transaction for improving performance.
 
-      If binlog is enabled and log_replica_updates is disabled, slave
+      If binlog is enabled and log_slave_updates is disabled, slave
       SQL thread or slave worker thread adds transaction owned GTID
       into global executed_gtids, lost_gtids and gtids_only_in_table.
     */
-    CONDITIONAL_SYNC_POINT_FOR_TIMESTAMP("before_gtid_externalization");
     executed_gtids._add_gtid(thd->owned_gtid);
     thd->rpl_thd_ctx.session_gtids_ctx().notify_after_gtid_executed_update(thd);
-    if (thd->slave_thread && opt_bin_log && !opt_log_replica_updates) {
+    if (thd->slave_thread && opt_bin_log && !opt_log_slave_updates) {
       lost_gtids._add_gtid(thd->owned_gtid);
       gtids_only_in_table._add_gtid(thd->owned_gtid);
     }
   } else {
-    auto iterator = next_free_gno_map.end();
-    std::tie(iterator, std::ignore) =
-        next_free_gno_map.try_emplace(thd->owned_gtid.sidno, 1);
-    rpl_gno &next_free_gno = iterator->second;
-    if (next_free_gno > thd->owned_gtid.gno) {
+    if (thd->owned_gtid.sidno == server_sidno &&
+        next_free_gno > thd->owned_gtid.gno)
       next_free_gno = thd->owned_gtid.gno;
-    }
   }
 
   thd->clear_owned_gtids();
   if (thd->variables.gtid_next.type == ASSIGNED_GTID) {
-    assert(!thd->is_commit_in_middle_of_statement);
+    DBUG_ASSERT(!thd->is_commit_in_middle_of_statement);
     thd->variables.gtid_next.set_undefined();
   } else {
     /*
@@ -896,8 +890,8 @@ void Gtid_state::update_gtids_impl_own_gtid(THD *thd, bool is_commit) {
       gtid_pre_statement_checks skips the test for undefined,
       e.g. ROLLBACK.
     */
-    assert(thd->variables.gtid_next.is_automatic() ||
-           thd->variables.gtid_next.is_undefined());
+    DBUG_ASSERT(thd->variables.gtid_next.type == AUTOMATIC_GTID ||
+                thd->variables.gtid_next.type == UNDEFINED_GTID);
   }
 }
 
@@ -916,8 +910,8 @@ void Gtid_state::update_gtids_impl_broadcast_and_unlock_sidnos() {
 }
 
 void Gtid_state::update_gtids_impl_own_anonymous(THD *thd, bool *more_trx) {
-  assert(thd->variables.gtid_next.type == ANONYMOUS_GTID ||
-         thd->variables.gtid_next.type == AUTOMATIC_GTID);
+  DBUG_ASSERT(thd->variables.gtid_next.type == ANONYMOUS_GTID ||
+              thd->variables.gtid_next.type == AUTOMATIC_GTID);
   /*
     If there is more in the transaction cache, set more_trx to indicate this.
 
@@ -939,10 +933,11 @@ void Gtid_state::update_gtids_impl_own_anonymous(THD *thd, bool *more_trx) {
   }
 }
 
-void Gtid_state::update_gtids_impl_own_nothing(THD *thd [[maybe_unused]]) {
-  assert(thd->commit_error != THD::CE_COMMIT_ERROR ||
-         thd->has_gtid_consistency_violation);
-  assert(thd->variables.gtid_next.type == AUTOMATIC_GTID);
+void Gtid_state::update_gtids_impl_own_nothing(
+    THD *thd MY_ATTRIBUTE((unused))) {
+  DBUG_ASSERT(thd->commit_error != THD::CE_COMMIT_ERROR ||
+              thd->has_gtid_consistency_violation);
+  DBUG_ASSERT(thd->variables.gtid_next.type == AUTOMATIC_GTID);
 }
 
 void Gtid_state::update_gtids_impl_end(THD *thd, bool more_trx) {
@@ -951,7 +946,7 @@ void Gtid_state::update_gtids_impl_end(THD *thd, bool more_trx) {
 
 enum_return_status Gtid_state::ensure_commit_group_sidnos(rpl_sidno sidno) {
   DBUG_TRACE;
-  tsid_lock->assert_some_wrlock();
+  sid_lock->assert_some_wrlock();
   /*
     As we use the sidno as index of commit_group_sidnos and there is no
     sidno=0, the array size must be at least sidno + 1.
