@@ -1,16 +1,15 @@
-/* Copyright (c) 2006, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,11 +28,12 @@
 #include <atomic>
 
 #include "compression.h"  // COMPRESSION_ALGORITHM_NAME_BUFFER_SIZE
+#include "libbinlogevents/include/binlog_event.h"  // enum_binlog_checksum_alg
+#include "m_string.h"
 #include "my_inttypes.h"
 #include "my_io.h"
 #include "my_psi_config.h"
-#include "mysql/binlog/event/binlog_event.h"  // enum_binlog_checksum_alg
-#include "mysql/components/services/bits/psi_mutex_bits.h"
+#include "mysql/components/services/psi_mutex_bits.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql_com.h"
 #include "sql/binlog.h"
@@ -43,7 +43,6 @@
 #include "sql/rpl_rli.h"                  // rli->get_log_lock()
 #include "sql/rpl_trx_boundary_parser.h"  // Transaction_boundary_parser
 #include "sql/sql_const.h"
-#include "strmake.h"
 
 class Rpl_info_handler;
 class Server_ids;
@@ -70,11 +69,11 @@ struct MYSQL;
 
   log_name
   log_pos
-  source_host
-  source_user
-  source_pass
-  source_port
-  source_connect_retry
+  master_host
+  master_user
+  master_pass
+  master_port
+  master_connect_retry
 
   To write out the contents of master.info to disk a call to flush_info()
   is required. Currently, it is needed every time we read and queue data
@@ -88,11 +87,6 @@ class Master_info : public Rpl_info {
   friend class Rpl_info_factory;
 
  public:
-  /// In case we start replication from the first binary log file and
-  /// source log name is empty, we use first_source_log_name instead of
-  /// 'master_log_name' in the error log
-  static constexpr auto first_source_log_name = "<First log>";
-
   /**
     Host name or ip address stored in the master.info.
   */
@@ -110,7 +104,7 @@ class Master_info : public Rpl_info {
 
  private:
   /**
-    If true, USER/PASSWORD was specified when running START REPLICA.
+    If true, USER/PASSWORD was specified when running START SLAVE.
   */
   bool start_user_configured;
   /**
@@ -122,20 +116,20 @@ class Master_info : public Rpl_info {
   */
   char password[MAX_PASSWORD_LENGTH + 1];
   /**
-    User specified when running START REPLICA.
+    User specified when running START SLAVE.
   */
   char start_user[USERNAME_LENGTH + 1];
   /**
-    Password specified when running START REPLICA.
+    Password specified when running START SLAVE.
   */
   char start_password[MAX_PASSWORD_LENGTH + 1];
   /**
-    Stores the authentication plugin specified when running START REPLICA.
+    Stores the autentication plugin specified when running START SLAVE.
   */
   char start_plugin_auth[FN_REFLEN + 1];
   /**
-    Stores the authentication plugin directory specified when running
-    START REPLICA.
+    Stores the autentication plugin directory specified when running
+    START SLAVE.
   */
   char start_plugin_dir[FN_REFLEN + 1];
 
@@ -172,13 +166,13 @@ class Master_info : public Rpl_info {
  public:
   /**
     Returns if USER/PASSWORD was specified when running
-    START REPLICA.
+    START SLAVE.
 
     @return true or false.
   */
   bool is_start_user_configured() const { return start_user_configured; }
   /**
-    Returns if DEFAULT_AUTH was specified when running START REPLICA.
+    Returns if DEFAULT_AUTH was specified when running START SLAVE.
 
     @return true or false.
   */
@@ -186,7 +180,7 @@ class Master_info : public Rpl_info {
     return (start_plugin_auth[0] != 0);
   }
   /**
-    Returns if PLUGIN_DIR was specified when running START REPLICA.
+    Returns if PLUGIN_DIR was specified when running START SLAVE.
 
     @return true or false.
   */
@@ -195,7 +189,7 @@ class Master_info : public Rpl_info {
   }
   /**
     Defines that USER/PASSWORD was specified or not when running
-    START REPLICA.
+    START SLAVE.
 
     @param config is true or false.
   */
@@ -204,7 +198,7 @@ class Master_info : public Rpl_info {
   }
   /**
     Sets either user's name in the master.info repository when CHANGE
-    MASTER is executed or user's name used in START REPLICA if USER is
+    MASTER is executed or user's name used in START SLAVE if USER is
     specified.
 
     @param user_arg is user's name.
@@ -225,7 +219,7 @@ class Master_info : public Rpl_info {
     return (start_user_configured ? sizeof(start_user) : sizeof(user));
   }
   /**
-    If an user was specified when running START REPLICA, this function returns
+    If an user was specified when running START SLAVE, this function returns
     such user. Otherwise, it returns the user stored in master.info.
 
     @return user's name.
@@ -235,7 +229,7 @@ class Master_info : public Rpl_info {
   }
   /**
     Stores either user's password in the master.info repository when CHANGE
-    MASTER is executed or user's password used in START REPLICA if PASSWORD
+    MASTER is executed or user's password used in START SLAVE if PASSWORD
     is specified.
 
     @param password_arg is user's password.
@@ -244,7 +238,7 @@ class Master_info : public Rpl_info {
   void set_password(const char *password_arg);
   /**
     Returns either user's password in the master.info repository or
-    user's password used in START REPLICA.
+    user's password used in START SLAVE.
 
     @param[out] password_arg is user's password.
     @param[out] password_arg_size is user's password size.
@@ -253,29 +247,29 @@ class Master_info : public Rpl_info {
   */
   bool get_password(char *password_arg, size_t *password_arg_size);
   /**
-    Cleans in-memory password defined by START REPLICA.
+    Cleans in-memory password defined by START SLAVE.
   */
   void reset_start_info();
   /**
-    Returns the DEFAULT_AUTH defined by START REPLICA.
+    Returns the DEFAULT_AUTH defined by START SLAVE.
 
     @return DEFAULT_AUTH.
   */
   const char *get_start_plugin_auth() { return start_plugin_auth; }
   /**
-    Returns the PLUGIN_DIR defined by START REPLICA.
+    Returns the PLUGIN_DIR defined by START SLAVE.
 
     @return PLUGIN_DIR.
   */
   const char *get_start_plugin_dir() { return start_plugin_dir; }
   /**
-    Stores the DEFAULT_AUTH defined by START REPLICA.
+    Stores the DEFAULT_AUTH defined by START SLAVE.
   */
   void set_plugin_auth(const char *src) {
     if (src) strmake(start_plugin_auth, src, sizeof(start_plugin_auth) - 1);
   }
   /**
-    Stores the DEFAULT_AUTH defined by START REPLICA.
+    Stores the DEFAULT_AUTH defined by START SLAVE.
   */
   void set_plugin_dir(const char *src) {
     if (src) strmake(start_plugin_dir, src, sizeof(start_plugin_dir) - 1);
@@ -315,8 +309,7 @@ class Master_info : public Rpl_info {
 
   */
   long clock_diff_with_master;
-  float heartbeat_period;         // interface with CHANGE REPLICATION SOURCE or
-                                  // master.info
+  float heartbeat_period;         // interface with CHANGE MASTER or master.info
   ulonglong received_heartbeats;  // counter of received heartbeat events
 
   ulonglong last_heartbeat;
@@ -329,7 +322,7 @@ class Master_info : public Rpl_info {
     Initialized to novalue, then set to the queried from master
     @@global.binlog_checksum and deactivated once FD has been received.
   */
-  mysql::binlog::event::enum_binlog_checksum_alg checksum_alg_before_fd;
+  binary_log::enum_binlog_checksum_alg checksum_alg_before_fd;
   ulong retry_count;
   char master_uuid[UUID_LENGTH + 1];
   char bind_addr[HOSTNAME_LENGTH + 1];
@@ -455,21 +448,12 @@ class Master_info : public Rpl_info {
   my_off_t master_log_pos;
 
  public:
-  inline const char *get_master_log_name() const { return master_log_name; }
-  inline const char *get_master_log_name_info() const {
-    if (m_is_receiver_position_info_invalid) return "INVALID";
-    return get_master_log_name();
-  }
-  inline ulonglong get_master_log_pos() const { return master_log_pos; }
-  inline ulonglong get_master_log_pos_info() const {
-    if (m_is_receiver_position_info_invalid) return 0;
-    return get_master_log_pos();
-  }
+  inline const char *get_master_log_name() { return master_log_name; }
+  inline ulonglong get_master_log_pos() { return master_log_pos; }
   inline void set_master_log_name(const char *log_file_name) {
     strmake(master_log_name, log_file_name, sizeof(master_log_name) - 1);
   }
   inline void set_master_log_pos(ulonglong log_pos) {
-    m_is_receiver_position_info_invalid = false;
     master_log_pos = log_pos;
   }
   inline const char *get_io_rpl_log_name() {
@@ -480,7 +464,7 @@ class Master_info : public Rpl_info {
   /**
      returns the column number of a channel in the TABLE repository.
      Mainly used during server startup to load the information required
-     from the slave repository tables. See rpl_info_factory.cc
+     from the slave repostiory tables. See rpl_info_factory.cc
   */
   static uint get_channel_field_num();
 
@@ -668,10 +652,10 @@ class Master_info : public Rpl_info {
     This is the channel lock. It is a rwlock used to serialize all replication
     administrative commands that cannot be performed concurrently for a given
     replication channel:
-    - START REPLICA;
-    - STOP REPLICA;
-    - CHANGE REPLICATION SOURCE;
-    - RESET REPLICA;
+    - START SLAVE;
+    - STOP SLAVE;
+    - CHANGE MASTER;
+    - RESET SLAVE;
     - end_slave() (when mysqld stops)).
     Any of these commands must hold the wrlock from the start till the end.
   */
@@ -737,7 +721,7 @@ class Master_info : public Rpl_info {
   */
   void wait_until_no_reference(THD *thd);
 
-  /* Set true when the Master_info object was cleared by a RESET REPLICA */
+  /* Set true when the Master_info object was cleared by a RESET SLAVE */
   bool reset;
 
   /**
@@ -766,67 +750,12 @@ class Master_info : public Rpl_info {
 
   void get_flushed_relay_log_info(LOG_INFO *linfo);
 
-  /**
-    Marks the receiver position information (master_log_name, master_log_pos)
-    as being invalid or not.
-
-    @param invalid The value to set the status to invalid or valid
-  */
-  void set_receiver_position_info_invalid(bool invalid);
-
-  /**
-    Returns if receiver position information is valid or invalid
-
-    @return true if receiver position information is not reliable,
-            false otherwise.
-  */
-  bool is_receiver_position_info_invalid() const;
-
-  /**
-    Enable or disable the gtid_only mode
-
-    @param gtid_only_mode value to set gtid_only (enable/disable it)
-  */
-  void set_gtid_only_mode(bool gtid_only_mode);
-
-  /**
-    Returns if gtid_only is enabled or not
-
-    @return true if gtid_only mode is active for a channel,
-            false otherwise.
-  */
-  bool is_gtid_only_mode() const;
-
  private:
   /*
     Holds the relay log coordinates (file name and position) of the last master
     coordinates flushed into Master_info repository.
   */
   LOG_INFO flushed_relay_log_info;
-
-  /**
-    Is the replica working in GTID only mode, meaning it does not
-    persist position related information when executing or queueing
-    transactions.
-  */
-  bool m_gtid_only_mode;
-
-  /**
-    Are positions invalid. When true this means the values for
-    receiver position related information might be outdated.
-  */
-  bool m_is_receiver_position_info_invalid;
-
- public:
-  /*
-    Hostname of the server where master_uuid was last read.
-  */
-  std::string m_uuid_from_host{};
-
-  /*
-    Port of the server where master_uuid was last read.
-  */
-  uint m_uuid_from_port{0};
 };
 
 #endif /* RPL_MI_H */
