@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -62,6 +62,7 @@
 #include "sql/mdl.h"
 #include "sql/mysqld.h"  // key_mutex_slave_parallel_worker
 #include "sql/psi_memory_key.h"
+#include "sql/raii/sentry.h"  // raii::Sentry<>
 #include "sql/rpl_info_handler.h"
 #include "sql/rpl_msr.h"  // For channel_map
 #include "sql/rpl_reporting.h"
@@ -326,6 +327,11 @@ int Slave_worker::init_worker(Relay_log_info *rli, ulong i) {
         rli->get_privilege_checks_hostname().c_str());
   }
 
+  if (this->m_assign_gtids_to_anonymous_transactions_info.set_info(
+          rli->m_assign_gtids_to_anonymous_transactions_info.get_type(),
+          (rli->m_assign_gtids_to_anonymous_transactions_info.get_value()
+               .c_str())))
+    return 1;
   id = i;
   curr_group_exec_parts.clear();
   relay_log_change_notified = false;  // the 1st group to contain relaylog name
@@ -380,7 +386,6 @@ int Slave_worker::init_worker(Relay_log_info *rli, ulong i) {
    minimum context for MTS recovery.
 
    @param is_gaps_collecting_phase
-
           clarifies what state the caller
           executes this method from. When it's @c true
           that is @c mts_recovery_groups() and Worker should
@@ -1839,17 +1844,6 @@ std::tuple<bool, bool, uint> Slave_worker::check_and_report_end_of_retries(
                             &silent)) {
       error = ER_LOCK_DEADLOCK;
     }
-#ifndef DBUG_OFF
-    else {
-      /*
-        The non-debug binary will not retry this transactions, stopping the
-        SQL thread because of the non-temporary error. But, as this situation
-        is not supposed to happen as described in the comment above, we will
-        fail an assert to ease the issue investigation when it happens.
-      */
-      if (DBUG_EVALUATE_IF("rpl_fake_cod_deadlock", 0, 1)) DBUG_ASSERT(false);
-    }
-#endif
   }
 
   if (!has_temporary_error(thd, error, &silent) ||
@@ -1878,12 +1872,28 @@ bool Slave_worker::retry_transaction(uint start_relay_number,
 
   DBUG_TRACE;
 
+  /* Flag to check for cleanup */
+  bool cleaned_up{false};
+
+  /* Resets the worker context for next transaction retry, if any */
+  auto clean_retry_context = [&]() -> void {
+    if (!cleaned_up) {
+      cleanup_context(thd, true);
+      reset_commit_order_deadlock();
+      cleaned_up = true;
+    }
+  };
+
+  /* Object of sentry class to perform cleanup */
+  raii::Sentry<> retry_context_guard{clean_retry_context};
+
   if (slave_trans_retries == 0) return true;
 
   do {
     /* Simulate a lock deadlock error */
     uint error = 0;
     bool ret;
+    cleaned_up = false;
 
     std::tie(ret, silent, error) = check_and_report_end_of_retries(thd);
     if (ret) return true;
@@ -1916,8 +1926,7 @@ bool Slave_worker::retry_transaction(uint start_relay_number,
     c_rli->retried_trans++;
     mysql_mutex_unlock(&c_rli->data_lock);
 
-    cleanup_context(thd, true);
-    reset_commit_order_deadlock();
+    clean_retry_context();
     worker_sleep(min<ulong>(trans_retries, MAX_SLAVE_RETRY_PAUSE));
 
   } while (read_and_apply_events(start_relay_number, start_relay_pos,
