@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -29,45 +29,39 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include <mysql/components/services/log_builtins.h>
 #include "component_sys_var_service_imp.h"
 #include "lex_string.h"
+#include "m_ctype.h"
+#include "m_string.h"
 #include "map_helpers.h"
 #include "my_compiler.h"
 #include "my_getopt.h"
 #include "my_inttypes.h"
+#include "my_loglevel.h"
 #include "my_macros.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
 #include "mysql/components/service_implementation.h"
 #include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/components/services/bits/psi_memory_bits.h"
 #include "mysql/components/services/component_sys_var_service.h"
 #include "mysql/components/services/log_shared.h"
+#include "mysql/components/services/psi_memory_bits.h"
 #include "mysql/components/services/system_variable_source_type.h"
-#include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/psi/mysql_rwlock.h"
 #include "mysql/service_mysql_alloc.h"
 #include "mysql/status_var.h"
-#include "mysql/strings/dtoa.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysql/udf_registration_types.h"
 #include "mysqld_error.h"
-#include "nulls.h"
 #include "sql/current_thd.h"
-#include "sql/error_handler.h"  // Internal_error_handler
 #include "sql/log.h"
 #include "sql/mysqld.h"
 #include "sql/persisted_variable.h"  // Persisted_variables_cache
 #include "sql/set_var.h"
-#include "sql/sql_class.h"  // THD
-#include "sql/sql_component.h"
-#include "sql/sql_lex.h"  // LEX
 #include "sql/sql_plugin_var.h"
 #include "sql/sql_show.h"
 #include "sql/sys_vars_shared.h"
 #include "sql/thr_malloc.h"
 #include "sql_string.h"
-#include "strxmov.h"
 
 #define FREE_RECORD(sysvar)                                              \
   my_free(const_cast<char *>(                                            \
@@ -98,21 +92,20 @@ void mysql_comp_sys_var_services_init() {
   return;
 }
 
-int mysql_add_sysvar(sys_var *var) {
-  assert(var->cast_pluginvar() != nullptr);
+int mysql_add_sysvar(sys_var *first) {
+  sys_var *var;
+
+  var = first;
   /* A write lock should be held on LOCK_system_variables_hash */
   /* this fails if there is a conflicting variable name. see HASH_UNIQUE */
-  mysql_mutex_assert_not_owner(&LOCK_plugin);
   mysql_rwlock_wrlock(&LOCK_system_variables_hash);
-  if (!get_dynamic_system_variable_hash()
-           ->emplace(to_string(var->name), var)
-           .second) {
+  if (!get_system_variable_hash()->emplace(to_string(var->name), var).second) {
     LogErr(ERROR_LEVEL, ER_DUPLICATE_SYS_VAR, var->name.str);
     mysql_rwlock_unlock(&LOCK_system_variables_hash);
     return 1;
   }
   /* Update system_variable_hash version. */
-  dynamic_system_variable_hash_version++;
+  system_variable_hash_version++;
   mysql_rwlock_unlock(&LOCK_system_variables_hash);
   return 0;
 }
@@ -124,8 +117,8 @@ DEFINE_BOOL_METHOD(mysql_component_sys_variable_imp::register_variable,
                     void *variable_value)) {
   try {
     struct sys_var_chain chain = {nullptr, nullptr};
-    sys_var *sysvar [[maybe_unused]];
-    char *com_sys_var_name, *optname, *com_sys_var_name_copy;
+    sys_var *sysvar MY_ATTRIBUTE((unused));
+    char *com_sys_var_name, *optname;
     int com_sys_var_len;
     SYS_VAR *opt = nullptr;
     my_option *opts = nullptr;
@@ -135,14 +128,13 @@ DEFINE_BOOL_METHOD(mysql_component_sys_variable_imp::register_variable,
     char ***argv;
     int argc_copy;
     char **argv_copy;
+    char **to_free = nullptr;
     void *mem;
     get_opt_arg_source *opts_arg_source;
-    THD *thd = current_thd;
-    bool option_value_found_in_install = false;
-    MEM_ROOT local_root{key_memory_comp_sys_var, 512};
 
     com_sys_var_len = strlen(component_name) + strlen(var_name) + 2;
-    com_sys_var_name = new (&local_root) char[com_sys_var_len];
+    com_sys_var_name =
+        (char *)my_malloc(key_memory_comp_sys_var, com_sys_var_len, MYF(0));
     strxmov(com_sys_var_name, component_name, ".", var_name, NullS);
     my_casedn_str(&my_charset_latin1, com_sys_var_name);
 
@@ -165,9 +157,8 @@ DEFINE_BOOL_METHOD(mysql_component_sys_variable_imp::register_variable,
     opts->arg_source = opts_arg_source;
     opts->arg_source->m_path_name[0] = 0;
     opts->arg_source->m_source = enum_variable_source::COMPILED;
-    std::unique_ptr<SYS_VAR, decltype(&my_free)> unique_opt(nullptr, &my_free);
 
-    switch (flags & PLUGIN_VAR_WITH_SIGN_TYPEMASK) {
+    switch (flags & PLUGIN_VAR_TYPEMASK) {
       case PLUGIN_VAR_BOOL:
         SYSVAR_BOOL_TYPE(bool) * sysvar_bool;
 
@@ -301,39 +292,10 @@ DEFINE_BOOL_METHOD(mysql_component_sys_variable_imp::register_variable,
                component_name);
         goto end;
     }
-    unique_opt.reset(opt);
 
     plugin_opt_set_limits(opts, opt);
     opts->value = opts->u_max_value = *(uchar ***)(opt + 1);
 
-    /*
-      If this is executed by a SQL executing thread that is executing
-      INSTALL COMPONENT
-    */
-    if (thd && thd->lex && thd->lex->m_sql_cmd &&
-        thd->lex->m_sql_cmd->sql_command_code() == SQLCOM_INSTALL_COMPONENT) {
-      Sql_cmd_install_component *c =
-          down_cast<Sql_cmd_install_component *>(thd->lex->m_sql_cmd);
-      /* and has a SET list */
-      if (c->m_arg_list && c->m_arg_list_size > 1) {
-        const int saved_opt_count = c->m_arg_list_size;
-        argv = &c->m_arg_list;
-        argc = &c->m_arg_list_size;
-        opt_error = my_handle_options2(argc, argv, opts, nullptr, nullptr,
-                                       false, true, false);
-        /* Add back the program name handle_options removes */
-        (*argc)++;
-        (*argv)--;
-        if (opt_error) {
-          LogErr(ERROR_LEVEL,
-                 ER_SYS_VAR_COMPONENT_FAILED_TO_PARSE_VARIABLE_OPTIONS,
-                 var_name);
-          if (opts) my_cleanup_options(opts);
-          goto end;
-        }
-        option_value_found_in_install = (saved_opt_count > *argc);
-      }
-    }
     /*
       This does what plugins do:
       before the server is officially "started" the options are read
@@ -348,55 +310,37 @@ DEFINE_BOOL_METHOD(mysql_component_sys_variable_imp::register_variable,
       This is approximately what mysql_install_plugin() does.
       TODO: clean up the options processing code so all this is not needed.
     */
-    if (!option_value_found_in_install) {
-      if (mysqld_server_started) {
-        Persisted_variables_cache *pv =
-            Persisted_variables_cache::get_instance();
-        argc_copy = orig_argc;
-        argv_copy = new (&local_root) char *[argc_copy + 1];
-        memcpy(argv_copy, orig_argv, argc_copy * sizeof(char *));
-        argv_copy[argc_copy] = nullptr;
-        argc = &argc_copy;
-        argv = &argv_copy;
-        if (pv && pv->append_read_only_variables(argc, argv, true, true,
-                                                 &local_root)) {
-          LogErr(ERROR_LEVEL,
-                 ER_SYS_VAR_COMPONENT_FAILED_TO_PARSE_VARIABLE_OPTIONS,
-                 var_name);
-          if (opts) my_cleanup_options(opts);
-          goto end;
-        }
-      } else {
-        argc = get_remaining_argc();
-        argv = get_remaining_argv();
-      }
-      opt_error = handle_options(argc, argv, opts, nullptr);
-      /* Add back the program name handle_options removes */
-      (*argc)++;
-      (*argv)--;
-
-      if (opt_error) {
-        LogErr(ERROR_LEVEL,
-               ER_SYS_VAR_COMPONENT_FAILED_TO_PARSE_VARIABLE_OPTIONS, var_name);
-        if (opts) my_cleanup_options(opts);
-        goto end;
-      }
+    if (mysqld_server_started) {
+      argc_copy = orig_argc;
+      to_free = argv_copy = (char **)my_malloc(
+          key_memory_comp_sys_var, (argc_copy + 1) * sizeof(char *), MYF(0));
+      memcpy(argv_copy, orig_argv, argc_copy * sizeof(char *));
+      argv_copy[argc_copy] = nullptr;
+      argc = &argc_copy;
+      argv = &argv_copy;
+    } else {
+      argc = get_remaining_argc();
+      argv = get_remaining_argv();
     }
+    opt_error = handle_options(argc, argv, opts, nullptr);
+    /* Add back the program name handle_options removes */
+    (*argc)++;
+    (*argv)--;
 
-    com_sys_var_name_copy =
-        my_strdup(key_memory_comp_sys_var, com_sys_var_name, MYF(0));
-    if (com_sys_var_name_copy == nullptr) {
-      LogErr(ERROR_LEVEL, ER_SYS_VAR_COMPONENT_OOM, var_name);
+    if (opt_error) {
+      LogErr(ERROR_LEVEL, ER_SYS_VAR_COMPONENT_FAILED_TO_PARSE_VARIABLE_OPTIONS,
+             var_name);
+      if (opts) my_cleanup_options(opts);
       goto end;
     }
+
     sysvar = reinterpret_cast<sys_var *>(
-        new sys_var_pluginvar(&chain, com_sys_var_name_copy, opt));
+        new sys_var_pluginvar(&chain, com_sys_var_name, opt));
 
     if (sysvar == nullptr) {
       LogErr(ERROR_LEVEL, ER_SYS_VAR_COMPONENT_OOM, var_name);
       goto end;
-    } else
-      unique_opt.release();
+    }
 
     sysvar->set_arg_source(opts->arg_source);
     sysvar->set_is_plugin(false);
@@ -408,43 +352,21 @@ DEFINE_BOOL_METHOD(mysql_component_sys_variable_imp::register_variable,
 
     /*
       Once server is started and if there are few persisted plugin variables
-      which needs to be handled, we do it here. But only if it wasn't set by
-      INSTALL COMPONENT
+      which needs to be handled, we do it here.
     */
-    if (mysqld_server_started && !option_value_found_in_install) {
+    if (mysqld_server_started) {
       Persisted_variables_cache *pv = Persisted_variables_cache::get_instance();
-      if (pv != nullptr) {
-        assert(thd != nullptr);
-
-        mysql_rwlock_wrlock(&LOCK_system_variables_hash);
-        mysql_mutex_lock(&LOCK_plugin);
-        // ignore the SET PERSIST errors, as they're reported into the log
-        class Error_to_warning_error_handler : public Internal_error_handler {
-         public:
-          bool handle_condition(THD *, uint, const char *,
-                                Sql_condition::enum_severity_level *level,
-                                const char *) override {
-            if (*level == Sql_condition::SL_ERROR)
-              *level = Sql_condition::SL_WARNING;
-            return false;
-          }
-        } err_to_warning;
-        thd->push_internal_handler(&err_to_warning);
-        const bool error =
-            pv->set_persisted_options(true, com_sys_var_name, com_sys_var_len);
-        thd->pop_internal_handler();
-        mysql_mutex_unlock(&LOCK_plugin);
-        mysql_rwlock_unlock(&LOCK_system_variables_hash);
-        if (error)
-          LogErr(ERROR_LEVEL,
-                 ER_SYS_VAR_COMPONENT_FAILED_TO_MAKE_VARIABLE_PERSISTENT,
-                 com_sys_var_name);
+      if (pv && pv->set_persist_options(true)) {
+        LogErr(ERROR_LEVEL,
+               ER_SYS_VAR_COMPONENT_FAILED_TO_MAKE_VARIABLE_PERSISTENT,
+               com_sys_var_name);
       }
     }
     ret = false;
 
   end:
     my_free(mem);
+    if (to_free) my_free(to_free);
 
     return ret;
   } catch (...) {
@@ -487,6 +409,7 @@ const char *get_variable_value(sys_var *system_var, char *val_buf,
   const char *variable_value = get_one_variable(
       current_thd, show, OPT_GLOBAL, show->type, nullptr, &fromcs,
       variable_data_buffer, &out_variable_data_length);
+  mysql_mutex_unlock(&LOCK_global_system_variables);
 
   /*
      Allocate a buffer that can hold "worst" case byte-length of the value
@@ -499,7 +422,6 @@ const char *get_variable_value(sys_var *system_var, char *val_buf,
   const size_t result_length =
       copy_and_convert(result.get(), new_len, tocs, variable_value,
                        out_variable_data_length, fromcs, &dummy_err);
-  mysql_mutex_unlock(&LOCK_global_system_variables);
 
   /*
      The length of the user supplied buffer is intentionally checked
@@ -511,13 +433,11 @@ const char *get_variable_value(sys_var *system_var, char *val_buf,
 
          (tocs->mbminlen * (len)) / fromcs->mbmaxlen
    */
-
-  if (*val_length < result_length + 1) {  // "+1" is for terminating '\0'
-    *val_length = result_length + 1;
-    return nullptr;
-  }
-
+  const bool is_user_buffer_too_small = *val_length < result_length;
   *val_length = result_length;
+
+  if (is_user_buffer_too_small) return nullptr;
+
   memcpy(val_buf, result.get(), result_length);
   val_buf[result_length] = '\0';
 
@@ -528,19 +448,28 @@ DEFINE_BOOL_METHOD(mysql_component_sys_variable_imp::get_variable,
                    (const char *component_name, const char *var_name,
                     void **val, size_t *out_length_of_val)) {
   try {
+    String com_sys_var_name;
+    sys_var *var;
+
     // all of the non-prefixed variables are treated as part of the server
     // component
-    const char *prefix =
-        strcmp(component_name, "mysql_server") == 0 ? "" : component_name;
-    auto f = [val, out_length_of_val](const System_variable_tracker &,
-                                      sys_var *var) -> bool {
-      return get_variable_value(var, (char *)*val, out_length_of_val) ==
-             nullptr;
-    };
-    return System_variable_tracker::make_tracker(prefix, var_name)
-        .access_system_variable<bool>(current_thd, f,
-                                      Suppress_not_found_error::YES)
-        .value_or(true);
+    if (!strcmp(component_name, "mysql_server"))
+      com_sys_var_name.set(var_name, strlen(var_name), &my_charset_latin1);
+    else if (com_sys_var_name.reserve(strlen(component_name) + 1 +
+                                      strlen(var_name) + 1) ||
+             com_sys_var_name.append(component_name) ||
+             com_sys_var_name.append(".") || com_sys_var_name.append(var_name))
+      return true;  // OOM
+    mysql_rwlock_rdlock(&LOCK_system_variables_hash);
+    var = intern_find_sys_var(com_sys_var_name.c_ptr(),
+                              com_sys_var_name.length());
+    mysql_rwlock_unlock(&LOCK_system_variables_hash);
+
+    if (!var) return true;
+
+    if (!get_variable_value(var, (char *)*val, out_length_of_val)) return true;
+
+    return false;
   } catch (...) {
     mysql_components_handle_std_exception(__func__);
   }
@@ -558,16 +487,11 @@ DEFINE_BOOL_METHOD(mysql_component_sys_variable_imp::unregister_variable,
         com_sys_var_name.append(component_name) ||
         com_sys_var_name.append(".") || com_sys_var_name.append(var_name))
       return true;  // OOM
-    if (current_thd != nullptr) {
-      // During shutdown we have no THD, and we have already done
-      // mysql_mutex_destroy(&LOCK_plugin);
-      mysql_mutex_assert_not_owner(&LOCK_plugin);
-    }
     mysql_rwlock_wrlock(&LOCK_system_variables_hash);
 
     sys_var *sysvar = nullptr;
-    if (get_dynamic_system_variable_hash() != nullptr) {
-      sysvar = find_or_nullptr(*get_dynamic_system_variable_hash(),
+    if (get_system_variable_hash() != nullptr) {
+      sysvar = find_or_nullptr(*get_system_variable_hash(),
                                to_string(com_sys_var_name));
     }
     if (sysvar == nullptr) {
@@ -576,29 +500,23 @@ DEFINE_BOOL_METHOD(mysql_component_sys_variable_imp::unregister_variable,
       return true;
     }
 
-    result =
-        !get_dynamic_system_variable_hash()->erase(to_string(sysvar->name));
+    result = !get_system_variable_hash()->erase(to_string(sysvar->name));
     /* Update system_variable_hash version. */
-    dynamic_system_variable_hash_version++;
+    system_variable_hash_version++;
     mysql_rwlock_unlock(&LOCK_system_variables_hash);
 
     /*
        Freeing the value of string variables if they have PLUGIN_VAR_MEMALLOC
        flag enabled while registering variables.
     */
-
-    sys_var_pluginvar *sv_pluginvar =
-        reinterpret_cast<sys_var_pluginvar *>(sysvar);
-
-    const int var_flags = sv_pluginvar->plugin_var->flags;
+    int var_flags =
+        reinterpret_cast<sys_var_pluginvar *>(sysvar)->plugin_var->flags;
     if (((var_flags & PLUGIN_VAR_TYPEMASK) == PLUGIN_VAR_STR) &&
         (var_flags & PLUGIN_VAR_MEMALLOC)) {
-      char **value_addr = *(char ***)(sv_pluginvar->plugin_var + 1);
-      char *var_value = *value_addr;
-      if (var_value != nullptr) {
-        my_free(var_value);
-        *value_addr = nullptr;
-      }
+      char *var_value = **(
+          char ***)(reinterpret_cast<sys_var_pluginvar *>(sysvar)->plugin_var +
+                    1);
+      if (var_value) my_free(var_value);
     }
 
     FREE_RECORD(sysvar)
