@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -35,6 +35,15 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 class String;
 
+/**
+  mysql_string_itrerator structure to provide service to components
+*/
+struct st_string_iterator {
+  String *iterator_str;
+  const char *iterator_ptr;
+  int ctype;
+};
+
 /**
   The string functions as a service to the mysql_server component.
   So, that by default this service is available to all the components
@@ -44,12 +53,6 @@ class String;
 */
 class mysql_string_imp {
  public: /* service implementations */
-  /* mysql_charset service. */
-
-  static DEFINE_METHOD(CHARSET_INFO_h, get_charset_utf8mb4, ());
-
-  static DEFINE_METHOD(CHARSET_INFO_h, get_charset_by_name, (const char *name));
-
   /**
     Creates a new instance of string object
 
@@ -64,6 +67,9 @@ class mysql_string_imp {
     Destroys specified string object and data contained by it.
 
     @param string String object handle to release.
+    @return Status of performed operation
+    @retval false success
+    @retval true failure
   */
   static DEFINE_METHOD(void, destroy, (my_h_string string));
 
@@ -93,8 +99,6 @@ class mysql_string_imp {
   static DEFINE_BOOL_METHOD(toupper,
                             (my_h_string * out_string, my_h_string in_string));
 
-  /* mysql_string_converter service. */
-
   /**
     Allocates a string object and converts the character buffer to string
     and just sets the specified charset_name in the string object. It does
@@ -134,16 +138,6 @@ class mysql_string_imp {
                             (my_h_string in_string, char *out_buffer,
                              uint64 length, const char *charset_name));
 
-  /* mysql_string_converter_v2 service. */
-
-  static DEFINE_BOOL_METHOD(convert_from_buffer_v2,
-                            (my_h_string dest_string, const char *src_buffer,
-                             uint64 src_length, CHARSET_INFO_h src_charset));
-
-  static DEFINE_BOOL_METHOD(convert_to_buffer_v2,
-                            (my_h_string src_string, char *dest_buffer,
-                             uint64 dest_length, CHARSET_INFO_h dest_charset));
-
   /**
     Gets character code of character on specified index position in
     string to a specified buffer.
@@ -229,6 +223,9 @@ class mysql_string_imp {
     Releases the string iterator object specified.
 
     @param iter String iterator object handle to release.
+    @return Status of performed operation
+    @retval false success
+    @retval true failure
   */
   static DEFINE_METHOD(void, iterator_destroy, (my_h_string_iterator iter));
 
@@ -268,45 +265,5 @@ class mysql_string_imp {
     @retval true failure
   */
   static DEFINE_BOOL_METHOD(is_digit, (my_h_string_iterator iter, bool *out));
-
-  /**
-    Retrieves character value at current iterator position
-
-    @param iter       String iterator object handle
-    @param [out] out  Pointer to long value to store character to
-
-    @return Status of performed operation
-      @retval false success
-      @retval true failure
-  */
-  static DEFINE_BOOL_METHOD(get, (my_h_string_iterator iter, ulong *out));
-
-  static DEFINE_BOOL_METHOD(reset, (my_h_string s));
-
-  static DEFINE_BOOL_METHOD(append, (my_h_string s1, my_h_string s2));
-
-  /**
-    Allocates a string object and sets it value as substring of the input
-    string. Caller must free the allocated string by calling destroy().
-
-    @param [in]  in_string   String handle to extract substring from.
-    @param [in]  offset      Character offset of the substring.
-    @param [in]  count       Number of characters of the substring.
-    @param [out] out_string  Pointer to string handle holding the created result
-    string.
-    @return Status of performed operation
-    @retval false success
-    @retval true failure
-  */
-  static DEFINE_BOOL_METHOD(substr, (my_h_string in_string, uint offset,
-                                     uint count, my_h_string *out_string));
-
-  static DEFINE_BOOL_METHOD(compare,
-                            (my_h_string s1, my_h_string s2, int *cmp));
-
-  static DEFINE_BOOL_METHOD(get_data,
-                            (my_h_string s, const char **buffer_pointer,
-                             size_t *buffer_length,
-                             CHARSET_INFO_h *buffer_charset));
 };
 #endif /* MYSQL_SERVER_STRING_SERVICE_H */
