@@ -1,4 +1,4 @@
-/* Copyright (c) 2002, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,7 +23,6 @@
 #include "sql/sp_rcontext.h"
 
 #include <atomic>
-#include <memory>
 #include <new>
 
 #include "my_alloc.h"
@@ -44,7 +43,7 @@
 #include "sql/sql_tmp_table.h"  // create_tmp_table_from_fields
 #include "template_utils.h"     // delete_container_pointers
 
-class Query_expression;
+class SELECT_LEX_UNIT;
 
 extern "C" void sql_alloc_error_handler(void);
 
@@ -65,14 +64,14 @@ sp_rcontext::sp_rcontext(const sp_pcontext *root_parsing_ctx,
       m_ccount(0) {}
 
 sp_rcontext::~sp_rcontext() {
-  if (m_var_table != nullptr) {
+  if (m_var_table) {
     free_blobs(m_var_table);
-    ::destroy_at(m_var_table);
+    destroy(m_var_table);
   }
 
   delete_container_pointers(m_activated_handlers);
   delete_container_pointers(m_visible_handlers);
-  assert(m_ccount == 0);
+  DBUG_ASSERT(m_ccount == 0);
 
   // Leave m_var_items and m_case_expr_holders untouched.
   // They are allocated in mem roots and will be freed accordingly.
@@ -87,7 +86,7 @@ sp_rcontext *sp_rcontext::create(THD *thd, const sp_pcontext *root_parsing_ctx,
 
   if (ctx->alloc_arrays(thd) || ctx->init_var_table(thd) ||
       ctx->init_var_items(thd)) {
-    ::destroy_at(ctx);
+    destroy(ctx);
     return nullptr;
   }
 
@@ -96,13 +95,13 @@ sp_rcontext *sp_rcontext::create(THD *thd, const sp_pcontext *root_parsing_ctx,
 
 bool sp_rcontext::alloc_arrays(THD *thd) {
   {
-    const size_t n = m_root_parsing_ctx->max_cursor_index();
+    size_t n = m_root_parsing_ctx->max_cursor_index();
     m_cstack.reset(
         static_cast<sp_cursor **>(thd->alloc(n * sizeof(sp_cursor *))), n);
   }
 
   {
-    const size_t n = m_root_parsing_ctx->get_num_case_exprs();
+    size_t n = m_root_parsing_ctx->get_num_case_exprs();
     m_case_expr_holders.reset(
         static_cast<Item_cache **>(thd->mem_calloc(n * sizeof(Item_cache *))),
         n);
@@ -118,7 +117,7 @@ bool sp_rcontext::init_var_table(THD *thd) {
 
   m_root_parsing_ctx->retrieve_field_definitions(&field_def_lst);
 
-  assert(field_def_lst.elements == m_root_parsing_ctx->max_var_index());
+  DBUG_ASSERT(field_def_lst.elements == m_root_parsing_ctx->max_var_index());
 
   if (!(m_var_table = create_tmp_table_from_fields(thd, field_def_lst)))
     return true;
@@ -130,7 +129,7 @@ bool sp_rcontext::init_var_table(THD *thd) {
 }
 
 bool sp_rcontext::init_var_items(THD *thd) {
-  const uint num_vars = m_root_parsing_ctx->max_var_index();
+  uint num_vars = m_root_parsing_ctx->max_var_index();
 
   m_var_items.reset(static_cast<Item **>(thd->alloc(num_vars * sizeof(Item *))),
                     num_vars);
@@ -146,7 +145,7 @@ bool sp_rcontext::init_var_items(THD *thd) {
 }
 
 bool sp_rcontext::set_return_value(THD *thd, Item **return_value_item) {
-  assert(m_return_value_fld);
+  DBUG_ASSERT(m_return_value_fld);
 
   m_return_value_set = true;
 
@@ -173,7 +172,7 @@ bool sp_rcontext::push_cursor(sp_instr_cpush *i) {
 }
 
 void sp_rcontext::pop_cursors(uint count) {
-  assert(m_ccount >= count);
+  DBUG_ASSERT(m_ccount >= count);
 
   while (count--) {
     m_ccount--;
@@ -217,7 +216,7 @@ void sp_rcontext::pop_handler_frame(THD *thd) {
   m_activated_handlers.pop_back();
 
   // Also pop matching DA and copy new conditions.
-  assert(thd->get_stmt_da() == &frame->handler_da);
+  DBUG_ASSERT(thd->get_stmt_da() == &frame->handler_da);
   thd->pop_diagnostics_area();
   // Out with the old, in with the new!
   thd->get_stmt_da()->reset_condition_info(thd);
@@ -307,7 +306,7 @@ bool sp_rcontext::handle_sql_condition(THD *thd, uint *ip,
       declared within the one we're calling. At that point, the
       catcher for our throw would become very hard to predict!)
       One benefit of not simply clearing the DA as we enter a handler
-      (instead of resetting the condition count further down in this
+      (instead of resetting the condition cound further down in this
       exact function as we do now) and forcing the user to utilize
       GET STACKED DIAGNOSTICS is that this way, we can make
       SHOW WARNINGS|ERRORS work.
@@ -334,7 +333,7 @@ bool sp_rcontext::handle_sql_condition(THD *thd, uint *ip,
   //  - there is a pending SQL-condition (error or warning);
   //  - there is an SQL-handler for it.
 
-  assert(found_condition);
+  DBUG_ASSERT(found_condition);
 
   sp_handler_entry *handler_entry = nullptr;
   for (size_t i = 0; i < m_visible_handlers.size(); ++i) {
@@ -364,9 +363,9 @@ bool sp_rcontext::handle_sql_condition(THD *thd, uint *ip,
   */
   if (!handler_entry) return false;
 
-  const uint continue_ip = handler_entry->handler->type == sp_handler::CONTINUE
-                               ? cur_spi->get_cont_dest()
-                               : 0;
+  uint continue_ip = handler_entry->handler->type == sp_handler::CONTINUE
+                         ? cur_spi->get_cont_dest()
+                         : 0;
 
   /* Add a frame to handler-call-stack. */
   Handler_call_frame *frame = new (std::nothrow)
@@ -459,7 +458,7 @@ bool sp_cursor::open(THD *thd) {
     return true;
   }
 
-  const bool rc = mysql_open_cursor(thd, &m_result, &m_server_side_cursor);
+  bool rc = mysql_open_cursor(thd, &m_result, &m_server_side_cursor);
 
   // If execution failed, ensure that the cursor is closed.
   if (rc && m_server_side_cursor != nullptr) {
@@ -480,7 +479,7 @@ bool sp_cursor::close() {
   return false;
 }
 
-void sp_cursor::destroy() { assert(m_server_side_cursor == nullptr); }
+void sp_cursor::destroy() { DBUG_ASSERT(m_server_side_cursor == nullptr); }
 
 bool sp_cursor::fetch(List<sp_variable> *vars) {
   if (m_server_side_cursor == nullptr) {
@@ -522,7 +521,7 @@ bool sp_cursor::fetch(List<sp_variable> *vars) {
 ///////////////////////////////////////////////////////////////////////////
 
 bool sp_cursor::Query_fetch_into_spvars::prepare(
-    THD *thd, const mem_root_deque<Item *> &fields, Query_expression *u) {
+    THD *thd, const mem_root_deque<Item *> &fields, SELECT_LEX_UNIT *u) {
   /*
     Cache the number of columns in the result set in order to easily
     return an error if column count does not match value count.
@@ -537,8 +536,8 @@ bool sp_cursor::Query_fetch_into_spvars::send_data(
   auto item_iter = VisibleFields(items).begin();
   sp_variable *spvar;
 
-  assert(items.size() == CountVisibleFields(items));
-  assert(spvar_list->size() == items.size());
+  DBUG_ASSERT(items.size() == CountVisibleFields(items));
+  DBUG_ASSERT(spvar_list->size() == items.size());
 
   /*
     Assign the row fetched from a server side cursor to stored
