@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +29,7 @@
 
 class Alter_info;
 class THD;
-class Table_ref;
+struct TABLE_LIST;
 
 /**
   A base class for CREATE/ALTER TABLE commands and friends.
@@ -47,7 +46,7 @@ class Table_ref;
   * OPTIMIZE TABLE
   * REPAIR TABLE
 */
-class Sql_cmd_ddl_table : public Sql_cmd_ddl {
+class Sql_cmd_ddl_table : public Sql_cmd {
  public:
   explicit Sql_cmd_ddl_table(Alter_info *alter_info);
 
@@ -57,12 +56,12 @@ class Sql_cmd_ddl_table : public Sql_cmd_ddl {
   Alter_info *const m_alter_info;
 };
 
-inline Sql_cmd_ddl_table::~Sql_cmd_ddl_table() = default;
+inline Sql_cmd_ddl_table::~Sql_cmd_ddl_table() {}
 
 class Sql_cmd_create_table final : public Sql_cmd_ddl_table {
  public:
   Sql_cmd_create_table(Alter_info *alter_info,
-                       Table_ref *query_expression_tables)
+                       TABLE_LIST *query_expression_tables)
       : Sql_cmd_ddl_table(alter_info),
         query_expression_tables(query_expression_tables) {}
 
@@ -70,14 +69,11 @@ class Sql_cmd_create_table final : public Sql_cmd_ddl_table {
     return SQLCOM_CREATE_TABLE;
   }
 
-  const MYSQL_LEX_CSTRING *eligible_secondary_storage_engine(
-      THD *thd) const override;
-
   bool execute(THD *thd) override;
   bool prepare(THD *thd) override;
 
  private:
-  Table_ref *query_expression_tables;
+  TABLE_LIST *query_expression_tables;
 };
 
 class Sql_cmd_create_or_drop_index_base : public Sql_cmd_ddl_table {
@@ -89,8 +85,8 @@ class Sql_cmd_create_or_drop_index_base : public Sql_cmd_ddl_table {
   bool execute(THD *thd) override;
 };
 
-inline Sql_cmd_create_or_drop_index_base::~Sql_cmd_create_or_drop_index_base() =
-    default;
+inline Sql_cmd_create_or_drop_index_base::~Sql_cmd_create_or_drop_index_base() {
+}
 
 class Sql_cmd_create_index final : public Sql_cmd_create_or_drop_index_base {
  public:
@@ -101,15 +97,6 @@ class Sql_cmd_create_index final : public Sql_cmd_create_or_drop_index_base {
   }
 };
 
-class Sql_cmd_drop_table final : public Sql_cmd_ddl {
- public:
-  enum_sql_command sql_command_code() const override {
-    return SQLCOM_DROP_TABLE;
-  }
-
-  bool execute(THD *thd [[maybe_unused]]) override { return false; }
-};
-
 class Sql_cmd_drop_index final : public Sql_cmd_create_or_drop_index_base {
  public:
   using Sql_cmd_create_or_drop_index_base::Sql_cmd_create_or_drop_index_base;
@@ -131,7 +118,7 @@ class Sql_cmd_cache_index final : public Sql_cmd_ddl_table {
   bool execute(THD *thd) override;
 
  private:
-  bool assign_to_keycache(THD *thd, Table_ref *tables);
+  bool assign_to_keycache(THD *thd, TABLE_LIST *tables);
 
  private:
   const LEX_CSTRING m_key_cache_name;
@@ -148,7 +135,7 @@ class Sql_cmd_load_index final : public Sql_cmd_ddl_table {
   bool execute(THD *thd) override;
 
  private:
-  bool preload_keys(THD *thd, Table_ref *tables);
+  bool preload_keys(THD *thd, TABLE_LIST *tables);
 };
 
 #endif /* SQL_CMD_CREATE_TABLE_INCLUDED */
