@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,20 +33,19 @@
 
 #include "sql/sql_const_folding.h"
 
-#include <assert.h>
 #include <float.h>                         // DBL_MAX, FLT_MAX
 #include <stdint.h>                        // UINT64_MAX
 #include <sys/types.h>                     // uint
+#include <cstring>                         // memset
 #include <utility>                         // swap
 #include "decimal.h"                       // E_DEC_FATAL_ERROR
 #include "field_types.h"                   // MYSQL_TYPE_DATE
-                                           // assert
+#include "my_dbug.h"                       // DBUG_ASSERT
+#include "my_decimal.h"                    // my_decimal, my_decimal_cmp
 #include "my_inttypes.h"                   // longlong, ulonglong
 #include "my_time.h"                       // TIME_to_longlong_datetime_packed
-#include "mysql/strings/dtoa.h"            // DECIMAL_NOT_SPECIFIED
 #include "mysql/udf_registration_types.h"  // INT_RESULT, STRING_RESULT
 #include "mysql_time.h"                    // MYSQL_TIME
-#include "sql-common/my_decimal.h"         // my_decimal, my_decimal_cmp
 #include "sql/field.h"                     // Field_real, Field
 #include "sql/item.h"                      // Item, Item_field, Item_int
 #include "sql/item_cmpfunc.h"              // Item_bool_func2, Item_cond
@@ -237,7 +235,7 @@ static bool analyze_int_field_constant(THD *thd, Item_field *f,
     case INT_RESULT:
       break;
     case STRING_RESULT: {
-      if ((*const_val)->type() == Item::HEX_BIN_ITEM) {
+      if ((*const_val)->type() == Item::VARBIN_ITEM) {
         /*
           0x digits have STRING_RESULT but are ints in int
           context.
@@ -245,7 +243,7 @@ static bool analyze_int_field_constant(THD *thd, Item_field *f,
         break;
       }
     }
-      [[fallthrough]];
+      // fall-through
     case REAL_RESULT: {
       /*
         Try to convert to decimal. If that fails, we know the constant is out of
@@ -265,31 +263,32 @@ static bool analyze_int_field_constant(THD *thd, Item_field *f,
 
       if (err & E_DEC_TRUNCATED) {
         /*
-          Check for underflow, e.g. 1.7976931348623157E-308 would end up as
-          decimal 0.0, which means that the floating point values was
+          Check for underflow, e.g. 1.7976931348623157E-308 would end up
+          as decimal 0.0, which means that the floating point values was
           marginally greater than 0.0, so we "simulate" this by adding 0.1.
-          Correspondingly for negative underflow, we subtract 0.1. This is OK,
-          because we round later.  The value can also be truncated even if it
-          isn't quite as small as zero, but in such a case its absolute value
-          would always be smaller than 0.1, but not representable as a decimal,
-          so we use 0.1 for those as well.
+          Correspondingly for negative underflow, we subtract 0.1. This is
+          OK, because we round later.
         */
+        my_decimal n;
+        err = int2my_decimal(E_DEC_FATAL_ERROR, 0, false, &n);
+        DBUG_ASSERT(err == 0);
+        DBUG_ASSERT(my_decimal_cmp(&n, &dec) == 0);
         if (v > 0) {
           // underflow on the positive side
-          const String s("0.1", thd->charset());
+          String s("0.1", thd->charset());
           err = str2my_decimal(E_DEC_FATAL_ERROR, s.ptr(), s.length(),
                                s.charset(), &dec);
-          assert(err == 0);
+          DBUG_ASSERT(err == 0);
         } else {
-          const String s("-0.1", thd->charset());
+          String s("-0.1", thd->charset());
           err = str2my_decimal(E_DEC_FATAL_ERROR, s.ptr(), s.length(),
                                s.charset(), &dec);
-          assert(err == 0);
+          DBUG_ASSERT(err == 0);
         }
       }
       d = &dec;
     }
-      [[fallthrough]];
+      // fall-through
     case DECIMAL_RESULT: {
       /*
         If out of bounds of longlong, return RP_OUTSIDE_LOW or RP_OUTSIDE_HIGH
@@ -359,7 +358,7 @@ static bool analyze_int_field_constant(THD *thd, Item_field *f,
       // special treatment below
       break;
     default:
-      assert(false); /* purecov: inspected */
+      DBUG_ASSERT(false); /* purecov: inspected */
   }
 
   switch (f->field->type()) {
@@ -428,7 +427,7 @@ static bool analyze_int_field_constant(THD *thd, Item_field *f,
       }
     } break;
     default:
-      assert(false); /* purecov: inspected */
+      DBUG_ASSERT(false); /* purecov: inspected */
   }
 
   return false;
@@ -471,7 +470,7 @@ static bool analyze_decimal_field_constant(THD *thd, const Item_field *f,
     0xnnn numbers, which also have STRING result type, but should be treated
     the same as ints.
   */
-  if (ir == STRING_RESULT && (*const_val)->type() != Item::HEX_BIN_ITEM) {
+  if (ir == STRING_RESULT && (*const_val)->type() != Item::VARBIN_ITEM) {
     was_string_or_real = true;
     ir = DECIMAL_RESULT;
   }
@@ -481,9 +480,7 @@ static bool analyze_decimal_field_constant(THD *thd, const Item_field *f,
     case INT_RESULT: {
       my_decimal tmp;
       const auto *const d = (*const_val)->val_decimal(&tmp);
-      if (thd->is_error()) return true;
-      assert(d != nullptr);
-      assert(decimal_actual_fraction(d) == 0);
+      DBUG_ASSERT(decimal_actual_fraction(d) == 0);
       const int actual_intg = decimal_intg(d);
 
       if (actual_intg > f_intg) {  // overflow
@@ -501,7 +498,7 @@ static bool analyze_decimal_field_constant(THD *thd, const Item_field *f,
     } break;
     case REAL_RESULT: {
       my_decimal val_dec;
-      const double v = (*const_val)->val_real();
+      double v = (*const_val)->val_real();
       err = double2decimal(v, &val_dec);
 
       if (err & E_DEC_OVERFLOW) {
@@ -524,7 +521,7 @@ static bool analyze_decimal_field_constant(THD *thd, const Item_field *f,
         *negative = v < 0;
         my_decimal tmp;
         err = longlong2decimal(0, &tmp);
-        assert(err == 0);
+        DBUG_ASSERT(err == 0);
 
         widen_fraction(f_frac, &tmp);
         const auto new_dec = new (thd->mem_root) Item_decimal(&tmp);
@@ -534,7 +531,7 @@ static bool analyze_decimal_field_constant(THD *thd, const Item_field *f,
       }
       was_string_or_real = true;
     }
-      [[fallthrough]];
+      // fall-thru
     case DECIMAL_RESULT: {
       /*
         Decimal constant can have different range and precision
@@ -544,8 +541,6 @@ static bool analyze_decimal_field_constant(THD *thd, const Item_field *f,
       // Compute actual (minimal) decimal type of the constant
       my_decimal buff, *d;
       d = (*const_val)->val_decimal(&buff);
-      if ((*const_val)->null_value) return false;
-      assert(d != nullptr);
       const int actual_frac = decimal_actual_fraction(d);
       const int actual_intg = decimal_intg(d);
       const bool overflow = actual_intg > f_intg;
@@ -630,7 +625,7 @@ static bool analyze_real_field_constant(THD *thd, Item_field *f,
       */
       break;
     default:
-      assert(false); /* purecov: inspected */
+      DBUG_ASSERT(false); /* purecov: inspected */
       break;
   }
 
@@ -704,7 +699,7 @@ static bool analyze_year_field_constant(THD *thd, Item **const_val,
       allowed year values have been typed as MYSQL_TYPE_YEAR, cf.
       convert_constant_item called during type resolution.
     */
-    assert((*const_val)->result_type() == INT_RESULT);
+    DBUG_ASSERT((*const_val)->result_type() == INT_RESULT);
     const longlong year = (*const_val)->val_int();
 
     if (year == 0)
@@ -725,12 +720,11 @@ static bool analyze_year_field_constant(THD *thd, Item **const_val,
     case REAL_RESULT:
     case INT_RESULT: {
       const double year = (*const_val)->val_real();
-      if (year > static_cast<double>(Field_year::MAX_YEAR)) {
+      if (year > Field_year::MAX_YEAR) {
         *place = RP_OUTSIDE_HIGH;
       } else if (year < 0.0) {
         *place = RP_OUTSIDE_LOW;
-      } else if (year > 0.0 &&
-                 year < static_cast<double>(Field_year::MIN_YEAR)) {
+      } else if (year > 0.0 && year < Field_year::MIN_YEAR) {
         /*
           These values can be given as constants, but are not allowed to be
           stored in the field, so an = or <> comparison can be folded. For
@@ -753,7 +747,7 @@ static bool analyze_year_field_constant(THD *thd, Item **const_val,
       }
     } break;
     default:
-      assert(false); /* purecov: inspected */
+      DBUG_ASSERT(false); /* purecov: inspected */
       break;
   }
   return false;
@@ -784,7 +778,7 @@ static bool analyze_timestamp_field_constant(THD *thd, const Item_field *f,
     case INT_RESULT: {
       MYSQL_TIME ltime =
           my_time_set(0, 0, 0, 0, 0, 0, 0, false, MYSQL_TIMESTAMP_DATETIME);
-      MYSQL_TIME_STATUS status;
+      MYSQL_TIME_STATUS status{0, 0, 0};
       if (rtype == STRING_RESULT) {
         String buf, *res = (*const_val)->val_str(&buf);
         /*
@@ -827,11 +821,11 @@ static bool analyze_timestamp_field_constant(THD *thd, const Item_field *f,
       if (ft == MYSQL_TYPE_TIMESTAMP) {
         /*
           Convert constant to timeval, if it fits. If not, we are out of
-          range for a TIMESTAMP. The timeval is UTC since epoch, using 32
-          bits range.
+          range for a TIMESTAMP. The timeval is UTC since epoch.
         */
         int warnings = 0;
-        my_timeval tm = {0, 0};
+        struct timeval tm;
+        std::memset(&tm, 0, sizeof(tm));
         int zeros = 0;
         zeros += ltime.year == 0;
         zeros += ltime.month == 0;
@@ -855,7 +849,7 @@ static bool analyze_timestamp_field_constant(THD *thd, const Item_field *f,
           compare with min/max, unless it is 0 or has a zero date part (year,
           month or day)
         */
-        if (tm.m_tv_sec != 0) {
+        if (tm.tv_sec != 0) {
           /* '2038-01-19 03:14:07.[999999]' */
           MYSQL_TIME max_timestamp = my_time_set(
               TIMESTAMP_MAX_YEAR, 1, 19, 3, 14, 7,
@@ -864,8 +858,8 @@ static bool analyze_timestamp_field_constant(THD *thd, const Item_field *f,
               false, MYSQL_TIMESTAMP_DATETIME);
 
           /* '1970-01-01 00:00:01.[000000]' */
-          const MYSQL_TIME min_timestamp = my_time_set(
-              1970, 1, 1, 0, 0, 1, 0, false, MYSQL_TIMESTAMP_DATETIME);
+          MYSQL_TIME min_timestamp = my_time_set(1970, 1, 1, 0, 0, 1, 0, false,
+                                                 MYSQL_TIMESTAMP_DATETIME);
 
           // We store in UTC, so use as is
           const longlong max_t =
@@ -1080,7 +1074,7 @@ static bool fold_or_simplify(THD *thd, Item *ref_or_field,
       ft == Item_func::MULT_EQUAL_FUNC ||
       down_cast<Item_bool_func2 *>(*retcond)->ignore_unknown();
   if (always_true) {
-    if (ref_or_field->is_nullable()) {
+    if (ref_or_field->maybe_null) {
       if (is_top_level) {
         i = new (thd->mem_root) Item_func_isnotnull(ref_or_field);
       } else {
@@ -1107,7 +1101,7 @@ static bool fold_or_simplify(THD *thd, Item *ref_or_field,
       return false;
     }
 
-    if (ref_or_field->is_nullable() && ft != Item_func::EQUAL_FUNC) {
+    if (ref_or_field->maybe_null && ft != Item_func::EQUAL_FUNC) {
       i = new (thd->mem_root) Item_func_ne(ref_or_field, ref_or_field);
     } else {
       i = new (thd->mem_root) Item_func_false();
@@ -1277,7 +1271,7 @@ bool fold_condition(THD *thd, Item *cond, Item **retcond,
 
   switch (func_type) {
     case Item_func::ISNOTNULL_FUNC:
-      if (func->arguments()[0]->is_nullable()) {
+      if (func->arguments()[0]->maybe_null) {
         return fold_arguments(thd, func);
       }
 
@@ -1318,7 +1312,7 @@ bool fold_condition(THD *thd, Item *cond, Item **retcond,
     const auto equal = down_cast<Item_equal *>(func);
     // Use first field:  any one will do: they have the same type
     equal->m_const_folding[0] = equal->get_first();
-    equal->m_const_folding[1] = equal->const_arg();
+    equal->m_const_folding[1] = equal->get_const();
     args = equal->m_const_folding;
   }
 
@@ -1338,9 +1332,9 @@ bool fold_condition(THD *thd, Item *cond, Item **retcond,
         [1] See `create_tmp_field_from_item' case INT_RESULT.
       */
       seen_field = true;
-    } else if (args[i]->const_for_execution() && type != Item::SUBQUERY_ITEM) {
+    } else if (args[i]->const_for_execution() && type != Item::SUBSELECT_ITEM) {
       /*
-        Re test on Item::SUBQUERY_ITEM above: we exclude optimize time
+        Re test on Item::SUBSELECT_ITEM above: we exclude optimize time
         evaluation of constant subqueries for now; since it could still be
         expensive to evaluate and we have no cost model to decide whether
         folding it would save total time spent. It may turn out not to be
@@ -1441,8 +1435,8 @@ bool fold_condition(THD *thd, Item *cond, Item **retcond,
       if (func_type == Item_func::MULT_EQUAL_FUNC && (*retcond != nullptr)) {
         // The constant may have been modified, update the multi-equal
         const auto equal = down_cast<Item_equal *>(func);
-        assert(equal->m_const_folding[1] != nullptr);  // the constant
-        equal->set_const_arg(equal->m_const_folding[1]);
+        DBUG_ASSERT(equal->m_const_folding[1] != nullptr);  // the constant
+        equal->set_const(equal->m_const_folding[1]);
       }
       break;
     case Item_func::LT_FUNC:
