@@ -1,4 +1,4 @@
-/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2002, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -55,9 +55,11 @@
 #include "sql/sql_executor.h"  // QEP_TAB
 #include "sql/sql_lex.h"
 #include "sql/sql_list.h"
+#include "sql/sql_optimizer.h"
 #include "sql/sql_table.h"  // primary_key_name
 #include "sql/table.h"
 #include "sql_string.h"
+#include "template_utils.h"
 #include "thr_lock.h"
 #include "typelib.h"
 
@@ -135,10 +137,10 @@ static bool init_fields(THD *thd, TABLE_LIST *tables,
     find_fields->field->table->pos_in_table_list->select_lex =
         thd->lex->select_lex;
     bitmap_set_bit(find_fields->field->table->read_set,
-                   find_fields->field->field_index);
+                   find_fields->field->field_index());
     /* To make life easier when setting values in keys */
     bitmap_set_bit(find_fields->field->table->write_set,
-                   find_fields->field->field_index);
+                   find_fields->field->field_index());
   }
   return false;
 }
@@ -217,9 +219,9 @@ static int search_topics(THD *thd, QEP_TAB *topics,
   int count = 0;
   DBUG_TRACE;
 
-  unique_ptr_destroy_only<RowIterator> iterator =
-      init_table_iterator(thd, nullptr, topics, false,
-                          /*ignore_not_found_rows=*/false);
+  unique_ptr_destroy_only<RowIterator> iterator = init_table_iterator(
+      thd, nullptr, topics,
+      /*ignore_not_found_rows=*/false, /*count_examined_rows=*/false);
   if (iterator == nullptr) return 0;
 
   while (!iterator->Read()) {
@@ -256,9 +258,9 @@ static int search_keyword(THD *thd, QEP_TAB *keywords,
   int count = 0;
   DBUG_TRACE;
 
-  unique_ptr_destroy_only<RowIterator> iterator =
-      init_table_iterator(thd, nullptr, keywords, false,
-                          /*ignore_not_found_rows=*/false);
+  unique_ptr_destroy_only<RowIterator> iterator = init_table_iterator(
+      thd, nullptr, keywords,
+      /*ignore_not_found_rows=*/false, /*count_examined_rows=*/false);
   if (iterator == nullptr) return 0;
 
   while (!iterator->Read() && count < 2) {
@@ -374,9 +376,9 @@ static int search_categories(THD *thd, QEP_TAB *categories,
 
   DBUG_TRACE;
 
-  unique_ptr_destroy_only<RowIterator> iterator =
-      init_table_iterator(thd, nullptr, categories, false,
-                          /*ignore_not_found_rows=*/false);
+  unique_ptr_destroy_only<RowIterator> iterator = init_table_iterator(
+      thd, nullptr, categories,
+      /*ignore_not_found_rows=*/false, /*count_examined_rows=*/false);
   if (iterator == nullptr) return 0;
 
   while (!iterator->Read()) {
@@ -406,9 +408,9 @@ static void get_all_items_for_category(THD *thd, QEP_TAB *items, Field *pfname,
                                        List<String> *res) {
   DBUG_TRACE;
 
-  unique_ptr_destroy_only<RowIterator> iterator =
-      init_table_iterator(thd, nullptr, items, false,
-                          /*ignore_not_found_rows=*/false);
+  unique_ptr_destroy_only<RowIterator> iterator = init_table_iterator(
+      thd, nullptr, items,
+      /*ignore_not_found_rows=*/false, /*count_examined_rows=*/false);
   if (iterator == nullptr) return;
   while (!iterator->Read()) {
     if (!items->condition()->val_int()) continue;
@@ -444,12 +446,12 @@ static void get_all_items_for_category(THD *thd, QEP_TAB *items, Field *pfname,
 
 static int send_answer_1(THD *thd, String *s1, String *s2, String *s3) {
   DBUG_TRACE;
-  List<Item> field_list;
+  mem_root_deque<Item *> field_list(thd->mem_root);
   field_list.push_back(new Item_empty_string("name", 64));
   field_list.push_back(new Item_empty_string("description", 1000));
   field_list.push_back(new Item_empty_string("example", 1000));
 
-  if (thd->send_result_metadata(&field_list,
+  if (thd->send_result_metadata(field_list,
                                 Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF))
     return 1;
 
@@ -484,13 +486,13 @@ static int send_answer_1(THD *thd, String *s1, String *s2, String *s3) {
 
 static int send_header_2(THD *thd, bool for_category) {
   DBUG_TRACE;
-  List<Item> field_list;
+  mem_root_deque<Item *> field_list(thd->mem_root);
   if (for_category)
     field_list.push_back(new Item_empty_string("source_category_name", 64));
   field_list.push_back(new Item_empty_string("name", 64));
   field_list.push_back(new Item_empty_string("is_it_category", 1));
   return thd->send_result_metadata(
-      &field_list, Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF);
+      field_list, Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF);
 }
 
 /*
@@ -586,7 +588,7 @@ static bool prepare_simple_select(THD *thd, Item *cond, TABLE *table,
   const bool impossible =
       test_quick_select(thd, keys_to_use, 0, HA_POS_ERROR, false,
                         ORDER_NOT_RELEVANT, tab, cond, &needed_reg_dummy, &qck,
-                        tab->table()->force_index) < 0;
+                        tab->table()->force_index, thd->lex->select_lex) < 0;
   tab->set_quick(qck);
 
   return impossible || (tab->quick() && tab->quick()->reset());
@@ -639,40 +641,45 @@ bool mysqld_help(THD *thd, const char *mask) {
   SELECT_LEX *const select_lex = thd->lex->select_lex;
   DBUG_TRACE;
 
-  TABLE_LIST tables[4] = {TABLE_LIST("mysql", "help_topic", TL_READ),
-                          TABLE_LIST("mysql", "help_category", TL_READ),
-                          TABLE_LIST("mysql", "help_relation", TL_READ),
-                          TABLE_LIST("mysql", "help_keyword", TL_READ)};
+  TABLE_LIST *tables[4];
+  tables[0] = new (thd->mem_root) TABLE_LIST("mysql", "help_topic", TL_READ);
+  if (tables[0] == nullptr) return true;
+  tables[1] = new (thd->mem_root) TABLE_LIST("mysql", "help_category", TL_READ);
+  if (tables[1] == nullptr) return true;
+  tables[2] = new (thd->mem_root) TABLE_LIST("mysql", "help_relation", TL_READ);
+  if (tables[2] == nullptr) return true;
+  tables[3] = new (thd->mem_root) TABLE_LIST("mysql", "help_keyword", TL_READ);
+  if (tables[3] == nullptr) return true;
 
-  tables[0].next_global = tables[0].next_local =
-      tables[0].next_name_resolution_table = &tables[1];
-  tables[1].next_global = tables[1].next_local =
-      tables[1].next_name_resolution_table = &tables[2];
-  tables[2].next_global = tables[2].next_local =
-      tables[2].next_name_resolution_table = &tables[3];
+  tables[0]->next_global = tables[0]->next_local =
+      tables[0]->next_name_resolution_table = tables[1];
+  tables[1]->next_global = tables[1]->next_local =
+      tables[1]->next_name_resolution_table = tables[2];
+  tables[2]->next_global = tables[2]->next_local =
+      tables[2]->next_name_resolution_table = tables[3];
 
   /*
     HELP must be available under LOCK TABLES.
   */
-  if (open_trans_system_tables_for_read(thd, tables)) goto error2;
+  if (open_trans_system_tables_for_read(thd, tables[0])) goto error2;
 
   /*
     Init tables and fields to be usable from items
     tables do not contain VIEWs => we can pass 0 as conds
   */
   select_lex->context.table_list =
-      select_lex->context.first_name_resolution_table = &tables[0];
-  if (select_lex->setup_tables(thd, tables, false)) goto error;
+      select_lex->context.first_name_resolution_table = tables[0];
+  if (select_lex->setup_tables(thd, tables[0], false)) goto error;
   memcpy((char *)used_fields, (char *)init_used_fields, sizeof(used_fields));
-  if (init_fields(thd, tables, used_fields, array_elements(used_fields)))
+  if (init_fields(thd, tables[0], used_fields, array_elements(used_fields)))
     goto error;
-  for (i = 0; i < sizeof(tables) / sizeof(TABLE_LIST); i++)
-    tables[i].table->file->init_table_handle_for_HANDLER();
+  for (i = 0; i < array_elements(tables); i++)
+    tables[i]->table->file->init_table_handle_for_HANDLER();
 
   {
     QEP_TAB_standalone qep_tab_st;
     QEP_TAB &tab = qep_tab_st.as_QEP_TAB();
-    if (prepare_select_for_name(thd, mask, mlen, tables[0].table,
+    if (prepare_select_for_name(thd, mask, mlen, tables[0]->table,
                                 used_fields[help_topic_name].field, &tab))
       goto error;
 
@@ -685,7 +692,7 @@ bool mysqld_help(THD *thd, const char *mask) {
     QEP_TAB_standalone qep_tab_st;
     QEP_TAB &tab = qep_tab_st.as_QEP_TAB();
 
-    if (prepare_select_for_name(thd, mask, mlen, tables[3].table,
+    if (prepare_select_for_name(thd, mask, mlen, tables[3]->table,
                                 used_fields[help_keyword_name].field, &tab))
       goto error;
 
@@ -693,7 +700,7 @@ bool mysqld_help(THD *thd, const char *mask) {
     count_topics =
         (count_topics != 1)
             ? 0
-            : get_topics_for_keyword(thd, tables[0].table, tables[2].table,
+            : get_topics_for_keyword(thd, tables[0]->table, tables[2]->table,
                                      used_fields, key_id, &topics_list, &name,
                                      &description, &example);
   }
@@ -705,7 +712,7 @@ bool mysqld_help(THD *thd, const char *mask) {
       QEP_TAB_standalone qep_tab_st;
       QEP_TAB &tab = qep_tab_st.as_QEP_TAB();
 
-      if (prepare_select_for_name(thd, mask, mlen, tables[1].table,
+      if (prepare_select_for_name(thd, mask, mlen, tables[1]->table,
                                   used_fields[help_category_name].field, &tab))
         goto error;
 
@@ -732,7 +739,7 @@ bool mysqld_help(THD *thd, const char *mask) {
         QEP_TAB_standalone qep_tab_st;
         QEP_TAB &tab = qep_tab_st.as_QEP_TAB();
 
-        if (prepare_simple_select(thd, cond_topic_by_cat, tables[0].table,
+        if (prepare_simple_select(thd, cond_topic_by_cat, tables[0]->table,
                                   &tab))
           goto error;
         get_all_items_for_category(
@@ -742,7 +749,7 @@ bool mysqld_help(THD *thd, const char *mask) {
         QEP_TAB_standalone qep_tab_st;
         QEP_TAB &tab = qep_tab_st.as_QEP_TAB();
 
-        if (prepare_simple_select(thd, cond_cat_by_cat, tables[1].table, &tab))
+        if (prepare_simple_select(thd, cond_cat_by_cat, tables[1]->table, &tab))
           goto error;
         get_all_items_for_category(thd, &tab,
                                    used_fields[help_category_name].field,
@@ -766,7 +773,7 @@ bool mysqld_help(THD *thd, const char *mask) {
     QEP_TAB_standalone qep_tab_st;
     QEP_TAB &tab = qep_tab_st.as_QEP_TAB();
 
-    if (prepare_select_for_name(thd, mask, mlen, tables[1].table,
+    if (prepare_select_for_name(thd, mask, mlen, tables[1]->table,
                                 used_fields[help_category_name].field, &tab))
       goto error;
     search_categories(thd, &tab, used_fields, &categories_list, nullptr);
