@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,8 +29,8 @@
 
 class Compiled_in_command_iterator : public bootstrap::Command_iterator {
  public:
-  Compiled_in_command_iterator() = default;
-  virtual ~Compiled_in_command_iterator() = default;
+  Compiled_in_command_iterator() {}
+  virtual ~Compiled_in_command_iterator() {}
   bool begin(void) override;
   int next(std::string &query) override;
   void report_error_details(log_function_t log) override;
@@ -46,20 +45,4 @@ extern bool opt_initialize_insecure;
 bool initialize_create_data_directory(const char *data_home);
 extern bool mysql_initialize_directory_freshly_created;
 
-/* Declarations below are for unit testing. */
-extern bool generate_password(char *password, int size);
-
-#define ALLOWED_PWD_UPCHARS "QWERTYUIOPASDFGHJKLZXCVBNM"
-#define ALLOWED_PWD_LOWCHARS "qwertyuiopasdfghjklzxcvbnm"
-#define ALLOWED_PWD_NUMCHARS "1234567890"
-#define ALLOWED_PWD_SYMCHARS ",.-+*;:_!#%&/()=?><"
-
-static constexpr const char g_allowed_pwd_chars[] =
-    ALLOWED_PWD_LOWCHARS ALLOWED_PWD_SYMCHARS ALLOWED_PWD_UPCHARS
-        ALLOWED_PWD_NUMCHARS;
-static constexpr const char g_upper_case_chars[] = ALLOWED_PWD_UPCHARS;
-static constexpr const char g_lower_case_chars[] = ALLOWED_PWD_LOWCHARS;
-static constexpr const char g_numeric_chars[] = ALLOWED_PWD_NUMCHARS;
-static constexpr const char g_special_chars[] = ALLOWED_PWD_SYMCHARS;
-
 #endif /* SQL_INITIALIZE_H */
