@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,6 +32,7 @@
 // LOG_EVENT_UPDATE_TABLE_MAP_VERSION_F
 #include <string.h>
 #include <sys/types.h>
+
 #include <algorithm>
 #include <atomic>
 
@@ -147,6 +148,7 @@ class READ_INFO {
   /* load xml */
   List<XML_TAG> taglist;
   int read_value(int delim, String *val);
+  int read_cdata(String *val, bool *have_cdata);
   bool read_xml();
   void clear_level(int level);
 
@@ -291,10 +293,15 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
   is_concurrent =
       (table_list->lock_descriptor().type == TL_WRITE_CONCURRENT_INSERT);
 
-  if (m_opt_fields_or_vars.is_empty()) {
+  if (m_opt_fields_or_vars.empty()) {
     Field_iterator_table_ref field_iterator;
     field_iterator.set(table_list);
     for (; !field_iterator.end_of_fields(); field_iterator.next()) {
+      // Do not include user hidden fields.
+      if (field_iterator.field() != nullptr &&
+          field_iterator.field()->is_hidden())
+        continue;
+
       Item *item;
       if (!(item = field_iterator.create_item(thd))) return true;
 
@@ -309,29 +316,35 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
       Let us also prepare SET clause, altough it is probably empty
       in this case.
     */
-    if (setup_fields(thd, Ref_item_array(), m_opt_set_fields, INSERT_ACL,
-                     nullptr, false, true) ||
-        setup_fields(thd, Ref_item_array(), m_opt_set_exprs, SELECT_ACL,
-                     nullptr, false, false))
+    if (setup_fields(thd, /*want_privilege=*/INSERT_ACL,
+                     /*allow_sum_func=*/false, /*split_sum_funcs=*/false,
+                     /*column_update=*/true, /*typed_items=*/nullptr,
+                     &m_opt_set_fields, Ref_item_array()) ||
+        setup_fields(thd, /*want_privilege=*/SELECT_ACL,
+                     /*allow_sum_func=*/false, /*split_sum_funcs=*/false,
+                     /*column_update=*/false, /*typed_items=*/nullptr,
+                     &m_opt_set_exprs, Ref_item_array()))
       return true;
   } else {  // Part field list
     /*
       Because m_opt_fields_or_vars may contain user variables,
       pass false for column_update in first call below.
     */
-    if (setup_fields(thd, Ref_item_array(), m_opt_fields_or_vars, INSERT_ACL,
-                     nullptr, false, false) ||
-        setup_fields(thd, Ref_item_array(), m_opt_set_fields, INSERT_ACL,
-                     nullptr, false, true))
+    if (setup_fields(thd, /*want_privilege=*/INSERT_ACL,
+                     /*allow_sum_func=*/false, /*split_sum_funcs=*/false,
+                     /*column_update=*/false, /*typed_items=*/nullptr,
+                     &m_opt_fields_or_vars, Ref_item_array()) ||
+        setup_fields(thd, /*want_privilege=*/INSERT_ACL,
+                     /*allow_sum_func=*/false, /*split_sum_funcs=*/false,
+                     /*column_update=*/true, /*typed_items=*/nullptr,
+                     &m_opt_set_fields, Ref_item_array()))
       return true;
 
     /*
       Special updatability test is needed because m_opt_fields_or_vars may
       contain a mix of column references and user variables.
     */
-    Item *item;
-    List_iterator<Item> it(m_opt_fields_or_vars);
-    while ((item = it++)) {
+    for (Item *item : m_opt_fields_or_vars) {
       if ((item->type() == Item::FIELD_ITEM ||
            item->type() == Item::REF_ITEM) &&
           item->field_for_view_update() == nullptr) {
@@ -345,8 +358,8 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
           that corresponding Item_func_get_user_var items are resolved as
           non-const items.
         */
-        Item_func_set_user_var *user_var = new (thd->mem_root)
-            Item_func_set_user_var(item->item_name, item, false);
+        Item_func_set_user_var *user_var =
+            new (thd->mem_root) Item_func_set_user_var(item->item_name, item);
         if (user_var == nullptr) return true;
         thd->lex->set_var_list.push_back(user_var);
       }
@@ -374,8 +387,10 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
     if (check_that_all_fields_are_given_values(thd, table, table_list))
       return true;
     /* Fix the expressions in SET clause */
-    if (setup_fields(thd, Ref_item_array(), m_opt_set_exprs, SELECT_ACL,
-                     nullptr, false, false))
+    if (setup_fields(thd, /*want_privilege=*/SELECT_ACL,
+                     /*allow_sum_func=*/false, /*split_sum_funcs=*/false,
+                     /*column_update=*/false, /*typed_items=*/nullptr,
+                     &m_opt_set_exprs, Ref_item_array()))
       return true;
   }
 
@@ -394,7 +409,7 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
     * LOAD DATA INFILE fff INTO TABLE xxx (columns1) SET columns2=
     may need a default for columns other than columns1 and columns2.
   */
-  const bool manage_defaults = m_opt_fields_or_vars.elements != 0;
+  const bool manage_defaults = !m_opt_fields_or_vars.empty();
   COPY_INFO info(COPY_INFO::INSERT_OPERATION, &m_opt_fields_or_vars,
                  &m_opt_set_fields, manage_defaults, handle_duplicates,
                  escape_char);
@@ -409,15 +424,13 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
 
   uint tot_length = 0;
   bool use_blobs = false, use_vars = false;
-  List_iterator_fast<Item> it(m_opt_fields_or_vars);
-  Item *item;
 
-  while ((item = it++)) {
+  for (Item *item : m_opt_fields_or_vars) {
     const Item *real_item = item->real_item();
 
     if (real_item->type() == Item::FIELD_ITEM) {
       const Field *field = down_cast<const Item_field *>(real_item)->field;
-      if (field->flags & BLOB_FLAG) {
+      if (field->is_flag_set(BLOB_FLAG)) {
         use_blobs = true;
         tot_length += 256;  // Will be extended if needed
       } else
@@ -700,16 +713,43 @@ bool Sql_cmd_load_table::write_execute_load_query_log_event(
   return mysql_bin_log.write_event(&e);
 }
 
+namespace {
+/**
+  Checks if an item is a hidden generated column.
+
+  @param table       Pointer to TABLE object
+  @param item        Item to check
+
+  @returns true if checked item is a hidden generated column.
+*/
+inline bool is_hidden_generated_column(TABLE *table, Item *item) {
+  Item *real_item = item->real_item();
+  if (table->has_gcol() && real_item->type() == Item::FIELD_ITEM) {
+    const Field *field = down_cast<Item_field *>(real_item)->field;
+    if (bitmap_is_set(&table->fields_for_functional_indexes,
+                      field->field_index()))
+      return true;
+  }
+  return false;
+}
+}  // namespace
+
 /**
   Read of rows of fixed size + optional garbage + optional newline
 
+  @param thd         Pointer to THD object
+  @param info        Pointer to COPY_INFO object
+  @param table_list  Pointer to TABLE_LIST object
+  @param read_info   Pointer to READ_INFO object
+  @param skip_lines  Number of ignored lines
+                     at the start of the file.
+
   @returns true if error
 */
 bool Sql_cmd_load_table::read_fixed_length(THD *thd, COPY_INFO &info,
                                            TABLE_LIST *table_list,
                                            READ_INFO &read_info,
                                            ulong skip_lines) {
-  List_iterator_fast<Item> it(m_opt_fields_or_vars);
   TABLE *table = table_list->table;
   bool err;
   DBUG_TRACE;
@@ -729,7 +769,6 @@ bool Sql_cmd_load_table::read_fixed_length(THD *thd, COPY_INFO &info,
       skip_lines--;
       continue;
     }
-    it.rewind();
     uchar *pos = read_info.row_start;
 
     restore_record(table, s->default_values);
@@ -744,8 +783,9 @@ bool Sql_cmd_load_table::read_fixed_length(THD *thd, COPY_INFO &info,
 
     Autoinc_field_has_explicit_non_null_value_reset_guard after_each_row(table);
 
-    Item *item;
-    while ((item = it++)) {
+    for (Item *item : m_opt_fields_or_vars) {
+      // Skip hidden generated columns.
+      if (is_hidden_generated_column(table, item)) continue;
       /*
         There is no variables in fields_vars list in this format so
         this conversion is safe (no need to check for STRING_ITEM).
@@ -857,6 +897,16 @@ class Field_tmp_nullability_guard {
 };
 
 /**
+  Read rows in delimiter-separated formats.
+
+  @param thd         Pointer to THD object
+  @param info        Pointer to COPY_INFO object
+  @param table_list  Pointer to TABLE_LIST object
+  @param read_info   Pointer to READ_INFO object
+  @param enclosed    ENCLOSED BY character
+  @param skip_lines  Number of ignored lines
+                     at the start of the file.
+
   @returns true if error
 */
 bool Sql_cmd_load_table::read_sep_field(THD *thd, COPY_INFO &info,
@@ -864,8 +914,6 @@ bool Sql_cmd_load_table::read_sep_field(THD *thd, COPY_INFO &info,
                                         READ_INFO &read_info,
                                         const String &enclosed,
                                         ulong skip_lines) {
-  List_iterator_fast<Item> it(m_opt_fields_or_vars);
-  Item *item;
   TABLE *table = table_list->table;
   size_t enclosed_length;
   bool err;
@@ -873,7 +921,7 @@ bool Sql_cmd_load_table::read_sep_field(THD *thd, COPY_INFO &info,
 
   enclosed_length = enclosed.length();
 
-  for (;; it.rewind()) {
+  for (;;) {
     if (thd->killed) {
       thd->send_kill_message();
       return true;
@@ -891,11 +939,16 @@ bool Sql_cmd_load_table::read_sep_field(THD *thd, COPY_INFO &info,
 
     Autoinc_field_has_explicit_non_null_value_reset_guard after_each_row(table);
 
-    while ((item = it++)) {
+    auto it = m_opt_fields_or_vars.begin();
+    for (; it != m_opt_fields_or_vars.end(); ++it) {
+      Item *item = *it;
       uint length;
       uchar *pos;
       Item *real_item;
 
+      // Skip hidden generated columns.
+      if (is_hidden_generated_column(table, item)) continue;
+
       if (read_info.read_field()) break;
 
       /* If this line is to be skipped we don't want to fill field or var */
@@ -961,10 +1014,12 @@ bool Sql_cmd_load_table::read_sep_field(THD *thd, COPY_INFO &info,
       skip_lines--;
       continue;
     }
-    if (item) {
+    if (it != m_opt_fields_or_vars.end()) {
       /* Have not read any field, thus input file is simply ended */
-      if (item == m_opt_fields_or_vars.head()) break;
-      for (; item; item = it++) {
+      if (it == m_opt_fields_or_vars.begin()) break;
+
+      for (; it != m_opt_fields_or_vars.end(); ++it) {
+        Item *item = *it;
         Item *real_item = item->real_item();
         if (real_item->type() == Item::FIELD_ITEM) {
           Field *field = ((Item_field *)real_item)->field;
@@ -1015,9 +1070,7 @@ bool Sql_cmd_load_table::read_sep_field(THD *thd, COPY_INFO &info,
         fill_record_n_invoke_before_triggers() after all trigger instructions
         has been executed.
       */
-      it.rewind();
-
-      while ((item = it++)) {
+      for (Item *item : m_opt_fields_or_vars) {
         Item *real_item = item->real_item();
         if (real_item->type() == Item::FIELD_ITEM)
           ((Item_field *)real_item)
@@ -1067,19 +1120,24 @@ bool Sql_cmd_load_table::read_sep_field(THD *thd, COPY_INFO &info,
 /**
   Read rows in xml format
 
+  @param thd         Pointer to THD object
+  @param info        Pointer to COPY_INFO object
+  @param table_list  Pointer to TABLE_LIST object
+  @param read_info   Pointer to READ_INFO object
+  @param skip_lines  Number of ignored lines
+                     at the start of the file.
+
   @returns true if error
 */
 bool Sql_cmd_load_table::read_xml_field(THD *thd, COPY_INFO &info,
                                         TABLE_LIST *table_list,
                                         READ_INFO &read_info,
                                         ulong skip_lines) {
-  List_iterator_fast<Item> it(m_opt_fields_or_vars);
-  Item *item;
   TABLE *table = table_list->table;
   const CHARSET_INFO *cs = read_info.read_charset;
   DBUG_TRACE;
 
-  for (;; it.rewind()) {
+  for (;;) {
     if (thd->killed) {
       thd->send_kill_message();
       return true;
@@ -1112,10 +1170,16 @@ bool Sql_cmd_load_table::read_xml_field(THD *thd, COPY_INFO &info,
 
     Autoinc_field_has_explicit_non_null_value_reset_guard after_each_row(table);
 
-    while ((item = it++)) {
+    auto it = m_opt_fields_or_vars.begin();
+    Item *item = nullptr;
+    for (; it != m_opt_fields_or_vars.end(); ++it) {
+      item = *it;
       /* If this line is to be skipped we don't want to fill field or var */
       if (skip_lines) continue;
 
+      // Skip hidden generated columns.
+      if (is_hidden_generated_column(table, item)) continue;
+
       /* find field in tag list */
       xmlit.rewind();
       tag = xmlit++;
@@ -1170,11 +1234,11 @@ bool Sql_cmd_load_table::read_xml_field(THD *thd, COPY_INFO &info,
       continue;
     }
 
-    if (item) {
+    if (item != nullptr) {
       /* Have not read any field, thus input file is simply ended */
-      if (item == m_opt_fields_or_vars.head()) break;
+      if (it == m_opt_fields_or_vars.begin()) break;
 
-      for (; item; item = it++) {
+      for (; it != m_opt_fields_or_vars.end(); item = *it++) {
         if (item->type() == Item::FIELD_ITEM) {
           /*
             QQ: We probably should not throw warning for each field.
@@ -1796,6 +1860,63 @@ int READ_INFO::read_value(int delim, String *val) {
   return my_tospace(chr);
 }
 
+/*
+  Read CDATA value if any.
+  Ignore multibyte and XML escape.
+  Note: the last character read must be '<' before calling this function.
+
+  @param[out] val           Resulting CDATA string.
+  @param[out] have_cdata    Set if really read CDATA.
+
+  @returns    Last character read or
+              my_b_EOF in case of unexpected EOF.
+*/
+int READ_INFO::read_cdata(String *val, bool *have_cdata) {
+  const char cdata_head[] = "![CDATA[";
+  const char *head_ptr = cdata_head;
+
+  /* Check for CDATA head "![CDATA[" */
+  for (size_t i = 0; i < strlen(cdata_head); i++) {
+    int chr = GET;
+
+    if (chr != *head_ptr++) {
+      /*
+        Didn't find "![CDATA[" head,
+        push back the last (unmatched) character
+      */
+      PUSH(chr);
+      /* and all matched from the head. */
+      while (i--) PUSH(*--head_ptr);
+
+      *have_cdata = false;
+      return '<';
+    }
+  }
+
+  int tail[3]{0};
+  for (tail[2] = GET; tail[2] != my_b_EOF; tail[2] = GET) {
+    /* Check for CDATA tail "]]>" */
+    if (tail[0] == ']' && tail[1] == ']' && tail[2] == '>') {
+      /* Cut last two characters ("]]") which were appended to val. */
+      DBUG_ASSERT(val->length() >= 2);
+      val->length(val->length() - 2);
+
+      *have_cdata = true;
+      return '>';
+    }
+    /* Shift the tail */
+    tail[0] = tail[1];
+    tail[1] = tail[2];
+
+    val->append(tail[2]);
+  }
+
+  /* Didn't find CDATA tail "]]>", the last character read must be my_b_EOF. */
+  DBUG_ASSERT(tail[2] == my_b_EOF);
+  *have_cdata = false;
+  return my_b_EOF;
+}
+
 /*
   Read a record in xml format
   tags and attributes are stored in taglist
@@ -1890,8 +2011,16 @@ bool READ_INFO::read_xml() {
           read in the upcoming call to read_value()
          */
         PUSH(chr);
-        chr = read_value('<', &value);
-        if (chr == my_b_EOF) goto found_eof;
+
+        /* Read <![CDATA[ ... ]]> and tag's value. */
+        bool have_cdata;
+        do {
+          chr = read_value('<', &value);
+          if (chr == my_b_EOF) goto found_eof;
+
+          chr = read_cdata(&value, &have_cdata);
+          if (chr == my_b_EOF) goto found_eof;
+        } while (have_cdata);
 
         /* save value to list */
         if (tag.length() > 0 && value.length() > 0) {
