@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2004, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,15 +22,18 @@
 
 #include "sql/sql_view.h"
 
-#include <limits.h>
-#include <stdio.h>
-#include <string.h>
 #include <sys/types.h>
+
+#include <climits>
+#include <cstdio>
+#include <cstring>
 #include <utility>
 
 #include "lex_string.h"
 #include "m_ctype.h"
 #include "m_string.h"
+#include "mem_root_deque.h"  // mem_root_deque
+#include "my_alloc.h"        // operator new
 #include "my_base.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
@@ -45,13 +48,15 @@
 #include "sql/auth/sql_security_ctx.h"
 #include "sql/binlog.h"  // mysql_bin_log
 #include "sql/dd/cache/dictionary_client.h"
-#include "sql/dd/dd.h"          // dd::get_dictionary
-#include "sql/dd/dd_schema.h"   // dd::schema_exists
-#include "sql/dd/dd_view.h"     // dd::create_view
-#include "sql/dd/dictionary.h"  // dd::Dictionary
+#include "sql/dd/dd.h"           // dd::get_dictionary
+#include "sql/dd/dd_schema.h"    // dd::schema_exists
+#include "sql/dd/dd_view.h"      // dd::create_view
+#include "sql/dd/dictionary.h"   // dd::Dictionary
+#include "sql/dd/string_type.h"  // String_type
 #include "sql/dd/types/abstract_table.h"
-#include "sql/dd_sql_view.h"  // update_referencing_views_metadata
-#include "sql/derror.h"       // ER_THD
+#include "sql/dd/types/view.h"  // View
+#include "sql/dd_sql_view.h"    // update_referencing_views_metadata
+#include "sql/derror.h"         // ER_THD
 #include "sql/enum_query_type.h"
 #include "sql/error_handler.h"  // Internal_error_handler
 #include "sql/field.h"
@@ -61,6 +66,7 @@
 #include "sql/mysqld.h"     // stage_end reg_ext key_file_frm
 #include "sql/opt_trace.h"  // opt_trace_disable_if_no_view_access
 #include "sql/parse_tree_node_base.h"
+#include "sql/parser_yystype.h"  // Create_col_name_list
 #include "sql/query_options.h"
 #include "sql/sp_cache.h"   // sp_cache_invalidate
 #include "sql/sql_base.h"   // get_table_def_key
@@ -83,7 +89,6 @@
 
 namespace dd {
 class Schema;
-class View;
 }  // namespace dd
 
 /*
@@ -95,40 +100,41 @@ class View;
     last_element  the last element of the list above
 
   NOTE
-    Unique names are generated by adding 'My_exp_' to the old name of the
+    Unique names are generated by adding a fixed prefix to the old name of the
     column. In case the name that was created this way already exists, we
     add a numeric postfix to its end (i.e. "1") and increase the number
     until the name becomes unique. If the generated name is longer than
     NAME_LEN, it is truncated.
 */
 
-static void make_unique_view_field_name(Item *target, List<Item> &item_list,
+static void make_unique_view_field_name(Item *target,
+                                        const mem_root_deque<Item *> &item_list,
                                         Item *last_element) {
   const char *name = (target->orig_name.is_set() ? target->orig_name.ptr()
                                                  : target->item_name.ptr());
   size_t name_len;
   uint attempt;
   char buff[NAME_LEN + 1];
-  List_iterator_fast<Item> itc(item_list);
 
   for (attempt = 0;; attempt++) {
     Item *check;
     bool ok = true;
 
     if (attempt)
-      name_len = snprintf(buff, NAME_LEN, "My_exp_%d_%s", attempt, name);
+      name_len =
+          snprintf(buff, NAME_LEN, SYNTHETIC_FIELD_NAME "%d_%s", attempt, name);
     else
-      name_len = snprintf(buff, NAME_LEN, "My_exp_%s", name);
+      name_len = snprintf(buff, NAME_LEN, SYNTHETIC_FIELD_NAME "%s", name);
 
+    auto itc = item_list.begin();
     do {
-      check = itc++;
+      check = *itc++;
       if (check != target && check->item_name.eq(buff)) {
         ok = false;
         break;
       }
     } while (check != last_element);
     if (ok) break;
-    itc.rewind();
   }
 
   target->orig_name = target->item_name;
@@ -156,11 +162,12 @@ static void make_unique_view_field_name(Item *target, List<Item> &item_list,
 */
 
 bool check_duplicate_names(const Create_col_name_list *column_names,
-                           List<Item> &item_list, bool gen_unique_view_name) {
+                           const mem_root_deque<Item *> &item_list,
+                           bool gen_unique_view_name) {
   DBUG_TRACE;
   if (column_names) {
     const uint count = column_names->size();
-    if (count != item_list.elements) {
+    if (count != CountVisibleFields(item_list)) {
       my_error(ER_VIEW_WRONG_LIST, MYF(0));
       return true;
     }
@@ -175,33 +182,29 @@ bool check_duplicate_names(const Create_col_name_list *column_names,
     return false;
   }
 
-  Item *item;
-  List_iterator_fast<Item> it(item_list);
-  List_iterator_fast<Item> itc(item_list);
-
-  while ((item = it++)) {
-    Item *check;
+  for (Item *item : VisibleFields(item_list)) {
     /* treat underlying fields like set by user names */
     if (item->real_item()->type() == Item::FIELD_ITEM)
       item->item_name.set_autogenerated(false);
-    itc.rewind();
-    while ((check = itc++) && check != item) {
+    for (Item *check : VisibleFields(item_list)) {
+      if (item == check) break;
       if (item->item_name.eq(check->item_name)) {
-        if (!gen_unique_view_name) goto err;
+        if (!gen_unique_view_name) {
+          my_error(ER_DUP_FIELDNAME, MYF(0), item->item_name.ptr());
+          return true;
+        }
         if (item->item_name.is_autogenerated())
           make_unique_view_field_name(item, item_list, item);
         else if (check->item_name.is_autogenerated())
           make_unique_view_field_name(check, item_list, item);
-        else
-          goto err;
+        else {
+          my_error(ER_DUP_FIELDNAME, MYF(0), item->item_name.ptr());
+          return true;
+        }
       }
     }
   }
   return false;
-
-err:
-  my_error(ER_DUP_FIELDNAME, MYF(0), item->item_name.ptr());
-  return true;
 }
 
 /**
@@ -211,19 +214,18 @@ err:
   @param  lex   LEX for this thread.
 */
 
-static void make_valid_column_names(LEX *lex) {
-  Item *item;
+void make_valid_column_names(LEX *lex) {
   size_t name_len;
   char buff[NAME_LEN];
-  uint column_no = 1;
+  uint column_no = 0;
 
   for (SELECT_LEX *sl = lex->select_lex; sl; sl = sl->next_select()) {
-    for (List_iterator_fast<Item> it(sl->item_list); (item = it++);
-         column_no++) {
+    for (Item *item : sl->visible_fields()) {
+      ++column_no;
       if (!item->item_name.is_autogenerated() ||
           !check_column_name(item->item_name.ptr()))
         continue;
-      name_len = snprintf(buff, NAME_LEN, "Name_exp_%u", column_no);
+      name_len = snprintf(buff, NAME_LEN, SYNTHETIC_FIELD_NAME "%u", column_no);
       item->orig_name = item->item_name;
       item->item_name.copy(buff, name_len);
     }
@@ -378,9 +380,7 @@ bool create_view_precheck(THD *thd, TABLE_LIST *tables, TABLE_LIST *view,
     Mark fields for special privilege check ("any" privilege)
   */
   for (SELECT_LEX *sl = select_lex; sl; sl = sl->next_select()) {
-    List_iterator_fast<Item> it(sl->item_list);
-    Item *item;
-    while ((item = it++)) {
+    for (Item *item : sl->visible_fields()) {
       Item_field *field;
       if ((field = item->field_for_view_update())) {
         /*
@@ -555,9 +555,9 @@ bool mysql_create_view(THD *thd, TABLE_LIST *views,
   */
   for (tbl = lex->query_tables; tbl; tbl = tbl->next_global) {
     /* is this table view and the same view which we creates now? */
-    if (tbl->is_view() && strcmp(tbl->view_db.str, view->db) == 0 &&
-        strcmp(tbl->view_name.str, view->table_name) == 0) {
-      my_error(ER_NO_SUCH_TABLE, MYF(0), tbl->view_db.str, tbl->view_name.str);
+    if (tbl->is_view() && strcmp(tbl->db, view->db) == 0 &&
+        strcmp(tbl->table_name, view->table_name) == 0) {
+      my_error(ER_NO_SUCH_TABLE, MYF(0), tbl->db, tbl->table_name);
       res = true;
       goto err;
     }
@@ -571,8 +571,8 @@ bool mysql_create_view(THD *thd, TABLE_LIST *views,
     */
     if (tbl->table) {
       /* is this table temporary and is not view? */
-      if (tbl->table->s->tmp_table != NO_TMP_TABLE && !tbl->is_view() &&
-          !tbl->schema_table) {
+      if (tbl->table->s->tmp_table != NO_TMP_TABLE &&
+          !tbl->is_view_or_derived() && !tbl->schema_table) {
         my_error(ER_VIEW_SELECT_TMPTABLE, MYF(0), tbl->alias);
         res = true;
         goto err;
@@ -582,37 +582,45 @@ bool mysql_create_view(THD *thd, TABLE_LIST *views,
 
   /* prepare select to resolve all fields */
   lex->context_analysis_only |= CONTEXT_ANALYSIS_ONLY_VIEW;
-  if (unit->prepare(thd, nullptr, 0, 0)) {
+  if (!unit->is_prepared()) {
     /*
-      some errors from prepare are reported to user, if is not then
-      it will be checked after err: label
+      @todo - the following code is duplicated in mysql_test_create_view.
+              ensure that we have a single preparation function for create view.
     */
-    res = true;
-    goto err;
-  }
+    if (unit->prepare(thd, nullptr, nullptr, 0, 0)) {
+      /*
+        some errors from prepare are reported to user, if is not then
+        it will be checked after err: label
+      */
+      res = true;
+      goto err;
+    }
+    /* Check if the auto generated column names are conforming. */
+    make_valid_column_names(lex);
 
-  /* Check if the auto generated column names are conforming. */
-  make_valid_column_names(lex);
+    /*
+      Only column names of the first query block should be checked for
+      duplication; any further UNION-ed part isn't used for determining
+      names of the view's columns.
+    */
+    if (check_duplicate_names(view->derived_column_names(), select_lex->fields,
+                              true)) {
+      res = true;
+      goto err;
+    }
 
-  /*
-    Only column names of the first select_lex should be checked for
-    duplication; any further UNION-ed part isn't used for determining
-    names of the view's columns.
-  */
-  if (check_duplicate_names(view->derived_column_names(), select_lex->item_list,
-                            true)) {
-    res = true;
-    goto err;
-  }
-
-  /*
-    Make sure the view doesn't have so many columns that we hit the
-    64k header limit if the view is materialized as a MyISAM table.
-  */
-  if (select_lex->item_list.elements > MAX_FIELDS) {
-    my_error(ER_TOO_MANY_FIELDS, MYF(0));
-    res = true;
-    goto err;
+    /*
+      Make sure the view doesn't have so many columns that we hit the
+      64k header limit if the view is materialized as a MyISAM table.
+    */
+    if (select_lex->fields.size() > MAX_FIELDS) {
+      my_error(ER_TOO_MANY_FIELDS, MYF(0));
+      res = true;
+      goto err;
+    }
+  } else {
+    lex->restore_cmd_properties();
+    bind_fields(thd->stmt_arena->item_list());
   }
 
   /*
@@ -641,9 +649,7 @@ bool mysql_create_view(THD *thd, TABLE_LIST *views,
 
     for (sl = select_lex; sl; sl = sl->next_select()) {
       DBUG_ASSERT(view->db); /* Must be set in the parser */
-      List_iterator_fast<Item> it(sl->item_list);
-      Item *item;
-      while ((item = it++)) {
+      for (Item *item : sl->visible_fields()) {
         Item_field *fld = item->field_for_view_update();
         uint priv = (get_column_grant(thd, &view->grant, view->db,
                                       view->table_name, item->item_name.ptr()) &
@@ -788,9 +794,7 @@ bool is_updatable_view(THD *thd, TABLE_LIST *view) {
     if (updatable) {
       // check that at least one column in view is updatable.
       bool view_has_updatable_column = false;
-      List_iterator_fast<Item> it(lex->select_lex->item_list);
-      Item *item;
-      while ((item = it++)) {
+      for (Item *item : lex->select_lex->visible_fields()) {
         Item_field *item_field = item->field_for_view_update();
         if (item_field && !item_field->table_ref->schema_table) {
           view_has_updatable_column = true;
@@ -900,7 +904,9 @@ bool mysql_register_view(THD *thd, TABLE_LIST *view,
     // view definition.
     Sql_mode_parse_guard parse_guard(thd);
 
-    lex->unit->print(thd, &view_query, QT_TO_ARGUMENT_CHARSET);
+    lex->unit->print(
+        thd, &view_query,
+        enum_query_type(QT_TO_ARGUMENT_CHARSET | QT_HIDE_ROLLUP_FUNCTIONS));
     lex->unit->print(
         thd, &is_query,
         enum_query_type(QT_TO_SYSTEM_CHARSET | QT_WITHOUT_INTRODUCERS));
@@ -1079,7 +1085,7 @@ bool open_and_read_view(THD *thd, TABLE_SHARE *share, TABLE_LIST *view_ref) {
   if (view_ref->is_view()) {
     DBUG_PRINT("info",
                ("VIEW %s.%s is already processed on previous PS/SP execution",
-                view_ref->view_db.str, view_ref->view_name.str));
+                view_ref->db, view_ref->table_name));
     return false;
   }
 
@@ -1092,14 +1098,12 @@ bool open_and_read_view(THD *thd, TABLE_SHARE *share, TABLE_LIST *view_ref) {
   // Check that view is not referenced recursively
   for (TABLE_LIST *precedent = view_ref->referencing_view; precedent;
        precedent = precedent->referencing_view) {
-    if (precedent->view_name.length == view_ref->table_name_length &&
-        precedent->view_db.length == view_ref->db_length &&
-        my_strcasecmp(system_charset_info, precedent->view_name.str,
+    if (precedent->table_name_length == view_ref->table_name_length &&
+        precedent->db_length == view_ref->db_length &&
+        my_strcasecmp(system_charset_info, precedent->table_name,
                       view_ref->table_name) == 0 &&
-        my_strcasecmp(system_charset_info, precedent->view_db.str,
-                      view_ref->db) == 0) {
-      my_error(ER_VIEW_RECURSIVE, MYF(0), top_view->view_db.str,
-               top_view->view_name.str);
+        my_strcasecmp(system_charset_info, precedent->db, view_ref->db) == 0) {
+      my_error(ER_VIEW_RECURSIVE, MYF(0), top_view->db, top_view->table_name);
       return true;
     }
   }
@@ -1146,9 +1150,9 @@ class DD_table_access_error_handler : public Internal_error_handler {
  public:
   DD_table_access_error_handler() {}
 
-  virtual bool handle_condition(THD *, uint sql_errno, const char *,
-                                Sql_condition::enum_severity_level *,
-                                const char *) {
+  bool handle_condition(THD *, uint sql_errno, const char *,
+                        Sql_condition::enum_severity_level *,
+                        const char *) override {
     return (sql_errno == ER_NO_SYSTEM_TABLE_ACCESS);
   }
 };
@@ -1208,9 +1212,6 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
     return false;
   }
 
-  // Save view's name, which will be wiped out by materialization
-  view_ref->save_name_temporary();
-
   /*
     We don't invalidate a prepared statement when a view changes,
     or when someone creates a temporary table.
@@ -1250,7 +1251,7 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
     that the view is parsed and prepared correctly.
   */
   mysql_mutex_lock(&thd->LOCK_thd_data);
-  thd->reset_db(view_ref->view_db);
+  thd->reset_db({view_ref->db, view_ref->db_length});
   mysql_mutex_unlock(&thd->LOCK_thd_data);
 
   lex_start(thd);
@@ -1282,8 +1283,9 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
     ER_NO_ACCESS_TO_NATIVE_FCT is reported otherwise.
   */
   bool parsing_system_view_saved = thd->parsing_system_view;
-  thd->parsing_system_view = dd::get_dictionary()->is_system_view_name(
+  bool parsing_system_view = dd::get_dictionary()->is_system_view_name(
       view_ref->db, view_ref->table_name);
+  thd->parsing_system_view = parsing_system_view;
 
   if (thd->parsing_system_view) thd->push_internal_handler(&dd_access_handler);
 
@@ -1318,9 +1320,10 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
   /*
     Check rights to run commands (EXPLAIN SELECT & SHOW CREATE) which show
     underlying tables.
-    Skip this step if we are opening view for prelocking only.
+    Skip this step if we are opening view for prelocking only or if this is
+    a DD table used under INFORMATION_SCHEMA system view.
   */
-  if (!view_ref->prelocking_placeholder) {
+  if (!view_ref->prelocking_placeholder && !parsing_system_view) {
     // If executing prepared statement: see "Optimizer trace" note above.
     opt_trace_disable_if_no_view_access(thd, view_ref, view_tables);
 
@@ -1409,8 +1412,14 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
       For LOCK TABLES we need to acquire "strong" metadata lock to ensure
       that we properly protect underlying tables for storage engines which
       don't use THR_LOCK locks.
+      Note that we should not acquire strong locks on underlying tables of
+      INFORMATION_SCHEMA views, which are data-dictionary tables, as this
+      will stall most of concurrent DDL in the system.
+      It is safe to do so since these tables and views are handled in a
+      special way when opened under LOCK TABLES. You can access them even
+      without mentioning them in LOCK TABLES statement.
     */
-    if (old_lex->sql_command == SQLCOM_LOCK_TABLES)
+    if (old_lex->sql_command == SQLCOM_LOCK_TABLES && !parsing_system_view)
       tbl->mdl_request.set_type(MDL_SHARED_READ_ONLY);
 
     /*
@@ -1480,13 +1489,25 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
          tbl = tbl->next_local) {
       enum_mdl_type mdl_lock_type;
 
-      tbl->set_lock(view_ref->lock_descriptor());
+      if (!parsing_system_view) {
+        /*
+          Play safe. Do not even try to propagate possibly write locks
+          to underlying tables of INFORMATION_SCHEMA views. Marking them
+          for write might cause part of prelocking algorithm responsible
+          for foreign key handling to consider such tables as to be updated
+          (even though INFORMATION_SCHEMA views are not updatable) and
+          force LOCK TABLES to acquire strong metadata locks on some
+          data-dictionary tables blocking concurrent DDL.
+        */
+        tbl->set_lock(view_ref->lock_descriptor());
+      }
 
-      if (old_lex->sql_command == SQLCOM_LOCK_TABLES) {
+      if (old_lex->sql_command == SQLCOM_LOCK_TABLES && !parsing_system_view) {
         /*
           For LOCK TABLES we need to acquire "strong" metadata lock to
           ensure that we properly protect underlying tables for storage
-          engines which don't use THR_LOCK locks.
+          engines which don't use THR_LOCK locks (but see the above
+          comment about INFORMATION_SCHEMA views/data-dictionary tables).
           OTOH for mergeable views we want to respect LOCAL clause in
           LOCK TABLES ... READ LOCAL.
         */
@@ -1533,7 +1554,7 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
   // Move nondeterminism information to whole query.
   old_lex->safe_to_cache_query &= view_lex->safe_to_cache_query;
 
-  old_lex->subqueries = true;
+  if (view_lex->has_udf()) old_lex->set_has_udf();
 
   Security_context *security_ctx;
 
@@ -1544,8 +1565,7 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
     */
     try {
       DBUG_ASSERT(thd->stmt_arena->mem_root);
-      view_ref->view_sctx = new (thd->stmt_arena->mem_root)
-          Security_context(thd->stmt_arena->mem_root);
+      view_ref->view_sctx = new (thd->stmt_arena->mem_root) Security_context();
       if (view_ref->view_sctx == nullptr) return true;
     } catch (...) {
       return true;
@@ -1609,11 +1629,61 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
   view_lex->all_selects_list->include_chain_in_global(
       &old_lex->all_selects_list);
 
-  view_ref->derived_key_list.empty();
+  view_ref->derived_key_list.clear();
 
   DBUG_ASSERT(view_lex == thd->lex);
   thd->lex = old_lex;  // Needed for prepare_security
-  result = !view_ref->prelocking_placeholder && view_ref->prepare_security(thd);
+
+  result = view_ref->prepare_security(thd);
+
+  if (!result && old_lex->sql_command == SQLCOM_LOCK_TABLES && view_tables) {
+    /*
+      For LOCK TABLES we need to check if user which security context is used
+      for view execution has necessary privileges for acquiring strong locks
+      on its underlying tables. These are LOCK TABLES and SELECT privileges.
+      Note that we only require SELECT and not LOCK TABLES on underlying
+      tables at view creation time. And these privileges might have been
+      revoked from user since then in any case.
+    */
+    DBUG_ASSERT(view_tables_tail);
+    for (TABLE_LIST *tbl = view_tables; tbl != view_tables_tail->next_global;
+         tbl = tbl->next_global) {
+      bool fake_lock_tables_acl;
+      if (check_lock_view_underlying_table_access(thd, tbl,
+                                                  &fake_lock_tables_acl)) {
+        result = true;
+        break;
+      }
+
+      if (fake_lock_tables_acl) {
+        /*
+          Do not acquire strong metadata locks for I_S and read-only/
+          truncatable-only P_S tables (i.e. for cases when we override
+          refused LOCK_TABLES_ACL). It is safe to do this since:
+          1) For I_S tables which are views on top of data-dictionary
+             tables and read-only/truncatable-only P_S tables under
+             LOCK TABLES we do not look at locked tables, and open
+             them separately.
+          2) I_S tables which are implemented as temporary tables
+             populated by special hook, we do not acquire metadata
+             locks or do normal open at all.
+        */
+        DBUG_ASSERT(tbl->is_system_view || belongs_to_p_s(tbl) ||
+                    tbl->schema_table);
+        tbl->mdl_request.set_type(MDL_SHARED_READ);
+        /*
+          We must override thr_lock_type (which can be a write type) as
+          well. This is necessary to keep consistency with MDL, to avoid
+          confusing part of prelocking algorithm which handles FKs and
+          to allow concurrent access to read-only and truncatable-only
+          P_S tables. Overriding TL_READ used by LOCK TABLES READ LOCAL
+          also helps to avoid upgrading metadata lock to SRO on both I_S
+          view and its underlying data-dictionary tables.
+        */
+        tbl->set_lock({TL_READ_DEFAULT, THR_DEFAULT});
+      }
+    }
+  }
 
   lex_end(view_lex);
 
@@ -1917,7 +1987,7 @@ bool check_key_in_view(THD *thd, TABLE_LIST *view,
     true  error (is not sent to cliet)
 */
 
-bool insert_view_fields(List<Item> *list, TABLE_LIST *view) {
+bool insert_view_fields(mem_root_deque<Item *> *list, TABLE_LIST *view) {
   Field_translator *trans_end;
   Field_translator *trans;
   DBUG_TRACE;
