@@ -347,7 +347,6 @@ extern int MYSQLdebug;
     MASTER_PASSWORD_SYM = 557,
     MASTER_PORT_SYM = 558,
     MASTER_RETRY_COUNT_SYM = 559,
-    MASTER_SERVER_ID_SYM = 560,
     MASTER_SSL_CAPATH_SYM = 561,
     MASTER_TLS_VERSION_SYM = 562,
     MASTER_SSL_CA_SYM = 563,
@@ -786,12 +785,51 @@ extern int MYSQLdebug;
     REQUIRE_TABLE_PRIMARY_KEY_CHECK_SYM = 996,
     STREAM_SYM = 997,
     OFF_SYM = 998,
-    NOT_A_TOKEN_SYM = 1150,
-    KEYWORD_USED_AS_IDENT = 1152,
-    KEYWORD_USED_AS_KEYWORD = 1153,
-    CONDITIONLESS_JOIN = 1154,
-    SUBQUERY_AS_EXPR = 1155,
-    EMPTY_FROM_CLAUSE = 1156
+    RETURNING_SYM = 999,
+    YYUNDEF = 1150,
+    JSON_VALUE_SYM = 1151,
+    TLS_SYM = 1152,
+    ATTRIBUTE_SYM = 1153,
+    ENGINE_ATTRIBUTE_SYM = 1154,
+    SECONDARY_ENGINE_ATTRIBUTE_SYM = 1155,
+    SOURCE_CONNECTION_AUTO_FAILOVER_SYM = 1156,
+    ZONE_SYM = 1157,
+    GRAMMAR_SELECTOR_DERIVED_EXPR = 1158,
+    REPLICA_SYM = 1159,
+    REPLICAS_SYM = 1160,
+    ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_SYM = 1161,
+    GET_SOURCE_PUBLIC_KEY_SYM = 1162,
+    SOURCE_AUTO_POSITION_SYM = 1163,
+    SOURCE_BIND_SYM = 1164,
+    SOURCE_COMPRESSION_ALGORITHM_SYM = 1165,
+    SOURCE_CONNECT_RETRY_SYM = 1166,
+    SOURCE_DELAY_SYM = 1167,
+    SOURCE_HEARTBEAT_PERIOD_SYM = 1168,
+    SOURCE_HOST_SYM = 1169,
+    SOURCE_LOG_FILE_SYM = 1170,
+    SOURCE_LOG_POS_SYM = 1171,
+    SOURCE_PASSWORD_SYM = 1172,
+    SOURCE_PORT_SYM = 1173,
+    SOURCE_PUBLIC_KEY_PATH_SYM = 1174,
+    SOURCE_RETRY_COUNT_SYM = 1175,
+    SOURCE_SSL_SYM = 1176,
+    SOURCE_SSL_CA_SYM = 1177,
+    SOURCE_SSL_CAPATH_SYM = 1178,
+    SOURCE_SSL_CERT_SYM = 1179,
+    SOURCE_SSL_CIPHER_SYM = 1180,
+    SOURCE_SSL_CRL_SYM = 1181,
+    SOURCE_SSL_CRLPATH_SYM = 1182,
+    SOURCE_SSL_KEY_SYM = 1183,
+    SOURCE_SSL_VERIFY_SERVER_CERT_SYM = 1184,
+    SOURCE_TLS_CIPHERSUITES_SYM = 1185,
+    SOURCE_TLS_VERSION_SYM = 1186,
+    SOURCE_USER_SYM = 1187,
+    SOURCE_ZSTD_COMPRESSION_LEVEL_SYM = 1188,
+    KEYWORD_USED_AS_IDENT = 1190,
+    KEYWORD_USED_AS_KEYWORD = 1191,
+    CONDITIONLESS_JOIN = 1192,
+    SUBQUERY_AS_EXPR = 1193,
+    EMPTY_FROM_CLAUSE = 1194
   };
 #endif
 /* Tokens.  */
@@ -1097,7 +1135,6 @@ extern int MYSQLdebug;
 #define MASTER_PASSWORD_SYM 557
 #define MASTER_PORT_SYM 558
 #define MASTER_RETRY_COUNT_SYM 559
-#define MASTER_SERVER_ID_SYM 560
 #define MASTER_SSL_CAPATH_SYM 561
 #define MASTER_TLS_VERSION_SYM 562
 #define MASTER_SSL_CA_SYM 563
@@ -1536,12 +1573,51 @@ extern int MYSQLdebug;
 #define REQUIRE_TABLE_PRIMARY_KEY_CHECK_SYM 996
 #define STREAM_SYM 997
 #define OFF_SYM 998
-#define NOT_A_TOKEN_SYM 1150
-#define KEYWORD_USED_AS_IDENT 1152
-#define KEYWORD_USED_AS_KEYWORD 1153
-#define CONDITIONLESS_JOIN 1154
-#define SUBQUERY_AS_EXPR 1155
-#define EMPTY_FROM_CLAUSE 1156
+#define RETURNING_SYM 999
+#define YYUNDEF 1150
+#define JSON_VALUE_SYM 1151
+#define TLS_SYM 1152
+#define ATTRIBUTE_SYM 1153
+#define ENGINE_ATTRIBUTE_SYM 1154
+#define SECONDARY_ENGINE_ATTRIBUTE_SYM 1155
+#define SOURCE_CONNECTION_AUTO_FAILOVER_SYM 1156
+#define ZONE_SYM 1157
+#define GRAMMAR_SELECTOR_DERIVED_EXPR 1158
+#define REPLICA_SYM 1159
+#define REPLICAS_SYM 1160
+#define ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_SYM 1161
+#define GET_SOURCE_PUBLIC_KEY_SYM 1162
+#define SOURCE_AUTO_POSITION_SYM 1163
+#define SOURCE_BIND_SYM 1164
+#define SOURCE_COMPRESSION_ALGORITHM_SYM 1165
+#define SOURCE_CONNECT_RETRY_SYM 1166
+#define SOURCE_DELAY_SYM 1167
+#define SOURCE_HEARTBEAT_PERIOD_SYM 1168
+#define SOURCE_HOST_SYM 1169
+#define SOURCE_LOG_FILE_SYM 1170
+#define SOURCE_LOG_POS_SYM 1171
+#define SOURCE_PASSWORD_SYM 1172
+#define SOURCE_PORT_SYM 1173
+#define SOURCE_PUBLIC_KEY_PATH_SYM 1174
+#define SOURCE_RETRY_COUNT_SYM 1175
+#define SOURCE_SSL_SYM 1176
+#define SOURCE_SSL_CA_SYM 1177
+#define SOURCE_SSL_CAPATH_SYM 1178
+#define SOURCE_SSL_CERT_SYM 1179
+#define SOURCE_SSL_CIPHER_SYM 1180
+#define SOURCE_SSL_CRL_SYM 1181
+#define SOURCE_SSL_CRLPATH_SYM 1182
+#define SOURCE_SSL_KEY_SYM 1183
+#define SOURCE_SSL_VERIFY_SERVER_CERT_SYM 1184
+#define SOURCE_TLS_CIPHERSUITES_SYM 1185
+#define SOURCE_TLS_VERSION_SYM 1186
+#define SOURCE_USER_SYM 1187
+#define SOURCE_ZSTD_COMPRESSION_LEVEL_SYM 1188
+#define KEYWORD_USED_AS_IDENT 1190
+#define KEYWORD_USED_AS_KEYWORD 1191
+#define CONDITIONLESS_JOIN 1192
+#define SUBQUERY_AS_EXPR 1193
+#define EMPTY_FROM_CLAUSE 1194
 
 /* Value type.  */
 
