@@ -1,4 +1,4 @@
-/*  Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+/*  Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -43,6 +43,7 @@
 #include "my_psi_config.h"
 #include "my_thread.h"
 #include "my_thread_local.h"  // my_get_thread_local & my_set_thread_local
+#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/mysql_mutex_bits.h"
 #include "mysql/components/services/mysql_rwlock_bits.h"
@@ -52,7 +53,6 @@
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/psi/mysql_rwlock.h"
 #include "mysql/psi/mysql_statement.h"
-#include "mysql/psi/psi_base.h"
 #include "mysql_time.h"
 #include "mysqld_error.h"
 #include "pfs_thread_provider.h"
@@ -60,6 +60,7 @@
 #include "sql/auth/sql_security_ctx.h"
 #include "sql/conn_handler/connection_handler_manager.h"
 #include "sql/current_thd.h"
+#include "sql/debug_sync.h"          // DEBUG_SYNC
 #include "sql/derror.h"              // ER_DEFAULT
 #include "sql/log.h"                 // Query log
 #include "sql/mysqld.h"              // current_thd
@@ -153,7 +154,7 @@ class Thread_to_plugin_map {
       std::map<my_thread_t, const void *>::iterator it =
           collection.find(thread);
       if (it == collection.end()) collection[thread] = plugin;
-    } catch (const std::bad_alloc &e) {
+    } catch (const std::bad_alloc &) {
       return true;
     }
     return false;
@@ -344,9 +345,8 @@ class Mutexed_map_thd_srv_session {
 
     @param key Key of the element
 
-    @return
-      value of the element
-      NULL  if not found
+    @return value of the element
+    @retval NULL  if not found
   */
   Srv_session *find(const THD *key) {
     rwlock_scoped_lock lock(&LOCK_collection, false, __FILE__, __LINE__);
@@ -360,17 +360,16 @@ class Mutexed_map_thd_srv_session {
 
     @param key     key
     @param plugin  secondary key
-    @param session
+    @param session object to be added to the collection
 
-    @return
-      false  success
-      true   failure
+    @retval false  success
+    @retval true   failure
   */
   bool add(const THD *key, const void *plugin, Srv_session *session) {
     rwlock_scoped_lock lock(&LOCK_collection, true, __FILE__, __LINE__);
     try {
       collection[key] = std::make_pair(plugin, session);
-    } catch (const std::bad_alloc &e) {
+    } catch (const std::bad_alloc &) {
       return true;
     }
     return false;
@@ -565,6 +564,8 @@ static void err_handle_error(void *ctx, uint err_errno, const char *err_msg,
 
 static void err_shutdown(void *, int) {}
 
+static bool err_alive(void *) { return true; }
+
 const struct st_command_service_cbs error_protocol_callbacks = {
     err_start_result_metadata,
     err_field_metadata,
@@ -584,7 +585,8 @@ const struct st_command_service_cbs error_protocol_callbacks = {
     err_get_string,
     err_handle_ok,
     err_handle_error,
-    err_shutdown};
+    err_shutdown,
+    err_alive};
 
 /**
   Modifies the PSI structures to (de)install a THD
@@ -748,16 +750,25 @@ bool Srv_session::module_deinit() {
 /**
   Checks if the session is valid.
 
-  Checked is if session is NULL, or in the list of opened sessions. If the
-  session is not in this list it was either closed or the address is invalid.
+  Checked is if session is NULL, or the state of the session is
+  SRV_SESSION_OPENED, SRV_SESSION_ATTACHED or SRV_SESSION_DETACHED.
 
   @return
     true  valid
     false not valid
 */
 bool Srv_session::is_valid(const Srv_session *session) {
-  const THD *thd = session ? &session->thd : nullptr;
-  return thd ? (bool)server_session_list.find(thd) : false;
+  DBUG_ASSERT(session != nullptr);
+  const bool is_valid_session = ((session->state > SRV_SESSION_CREATED) &&
+                                 (session->state < SRV_SESSION_CLOSED));
+  /*
+    Make sure valid session exists and invalid sessions doesn't exists in the
+    list of opened sessions.
+  */
+  DBUG_ASSERT((is_valid_session && server_session_list.find(&session->thd)) ||
+              (!is_valid_session && !server_session_list.find(&session->thd)));
+
+  return is_valid_session;
 }
 
 /**
@@ -775,6 +786,7 @@ Srv_session::Srv_session(srv_session_error_cb err_cb, void *err_cb_ctx)
       state(SRV_SESSION_CREATED),
       vio_type(NO_VIO_TYPE) {
   thd.mark_as_srv_session();
+  thd.m_audited = false;
 }
 
 /**
@@ -835,11 +847,7 @@ bool Srv_session::open() {
 
   server_session_list.add(&thd, plugin, this);
 
-  if (mysql_audit_notify(
-          &thd, AUDIT_EVENT(MYSQL_AUDIT_CONNECTION_PRE_AUTHENTICATE))) {
-    Connection_handler_manager::dec_connection_count();
-    return true;
-  }
+  state = SRV_SESSION_OPENED;
 
   return false;
 }
@@ -852,9 +860,10 @@ bool Srv_session::open() {
     true    failure
 */
 bool Srv_session::attach() {
-  const bool first_attach = (state == SRV_SESSION_CREATED);
+  const bool first_attach = (state == SRV_SESSION_OPENED);
   DBUG_TRACE;
   DBUG_PRINT("info", ("current_thd=%p", current_thd));
+  DBUG_ASSERT(state > SRV_SESSION_CREATED && state < SRV_SESSION_CLOSED);
 
   if (is_attached()) {
     if (!my_thread_equal(thd.real_id, my_thread_self())) {
@@ -908,9 +917,6 @@ bool Srv_session::attach() {
       At first attach the security context should have been already set and
       and this will report corect information.
     */
-    if (mysql_audit_notify(&thd, AUDIT_EVENT(MYSQL_AUDIT_CONNECTION_CONNECT)))
-      return true;
-
 #ifdef HAVE_PSI_THREAD_INTERFACE
     PSI_THREAD_CALL(notify_session_connect)(thd.get_psi());
 #endif /* HAVE_PSI_THREAD_INTERFACE */
@@ -1001,14 +1007,13 @@ bool Srv_session::close() {
     current_thd will be different then.
   */
   query_logger.general_log_print(&thd, COM_QUIT, NullS);
-  mysql_audit_notify(&thd, AUDIT_EVENT(MYSQL_AUDIT_CONNECTION_DISCONNECT), 0);
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
   PSI_THREAD_CALL(notify_session_disconnect)(thd.get_psi());
 #endif /* HAVE_PSI_THREAD_INTERFACE */
 
   thd.security_context()->logout();
-  thd.m_view_ctx_list.empty();
+  thd.m_view_ctx_list.clear();
   close_mysql_tables(&thd);
 
   thd.set_plugin(nullptr);
@@ -1016,6 +1021,13 @@ bool Srv_session::close() {
 
   thd.get_stmt_da()->reset_diagnostics_area();
 
+  // DEBUG_SYNC control block is released under call to
+  // `thd.release_resource`, thus we can't put this sync
+  // point directly before `pop_protocol`.
+  // Second constrain is that `THD::disconnect` marks
+  // this connection as killed, which disables DEBUG_SYNC.
+  DEBUG_SYNC(&thd, "srv_session_close");
+
   thd.disconnect();
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
@@ -1024,12 +1036,12 @@ bool Srv_session::close() {
 
   thd.release_resources();
 
+  Global_THD_manager::get_instance()->remove_thd(&thd);
+
   mysql_mutex_lock(&thd.LOCK_thd_protocol);
   thd.pop_protocol();
   mysql_mutex_unlock(&thd.LOCK_thd_protocol);
 
-  Global_THD_manager::get_instance()->remove_thd(&thd);
-
   Connection_handler_manager::dec_connection_count();
 
   return false;
@@ -1096,7 +1108,7 @@ int Srv_session::execute_command(enum enum_server_command command,
   mysql_audit_release(&thd);
 
   /*
-    The server does it for COM_QUERY in mysql_parse() but not for
+    The server does it for COM_QUERY in dispatch_sql_command() but not for
     COM_INIT_DB, for example
   */
   if (command != COM_QUERY) thd.reset_for_next_command();
@@ -1158,7 +1170,7 @@ class Find_thd_by_id_with_callback_set : public Find_THD_Impl {
     When a thread is found the callback function passed to the constructor
     is invoked under THD::Lock_thd_data
   */
-  virtual bool operator()(THD *thd) {
+  bool operator()(THD *thd) override {
     if (thd->thread_id() == thread_id) {
       MUTEX_LOCK(lock, &thd->LOCK_thd_data);
       callback(thd, &input);
