@@ -1,16 +1,15 @@
-/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,7 +20,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include "my_dbug.h"                                /* assert */
+#include "my_dbug.h"                                /* DBUG_ASSERT */
 #include "mysql/components/services/log_builtins.h" /* LogErr */
 #include "mysql/status_var.h"                       /* SHOW_VAR */
 #include "mysqld_error.h"                           /* Error/Warning macros */
@@ -49,7 +48,7 @@ void Ssl_acceptor_context_container::switch_data(
 }
 
 bool TLS_channel::singleton_init(Ssl_acceptor_context_container **out,
-                                 std::string channel,
+                                 std::string channel, bool use_ssl_arg,
                                  Ssl_init_callback *callbacks, bool db_init) {
   if (out == nullptr || callbacks == nullptr) return true;
   *out = nullptr;
@@ -57,11 +56,19 @@ bool TLS_channel::singleton_init(Ssl_acceptor_context_container **out,
     No need to take the ssl_ctx_lock lock here since it's being called
     from singleton_init().
   */
-  if (callbacks->provision_certs()) return true;
+  if (use_ssl_arg && callbacks->provision_certs()) return true;
 
-  enum enum_ssl_init_error error = SSL_INITERR_NOERROR;
+  /*
+    No real need for opt_use_ssl to be enabled in bootstrap mode,
+    but we want the SSL material generation and/or validation
+    (if supplied). So, we keep it on.
+
+    We don't hush the option since it would indicate a failure
+    in auto-generation, bad key material explicitly specified or
+    auto-generation disabled explcitly while SSL is still on.
+  */
   Ssl_acceptor_context_data *news =
-      new Ssl_acceptor_context_data(channel, callbacks, true, &error);
+      new Ssl_acceptor_context_data(channel, use_ssl_arg, callbacks);
   Ssl_acceptor_context_container *new_container =
       new Ssl_acceptor_context_container(news);
   if (news == nullptr || new_container == nullptr) {
@@ -71,13 +78,6 @@ bool TLS_channel::singleton_init(Ssl_acceptor_context_container **out,
     return true;
   }
 
-  if (opt_tls_certificates_enforced_validation &&
-      error != SSL_INITERR_NOERROR) {
-    LogErr(ERROR_LEVEL, ER_FAILED_TO_VALIDATE_CERTIFICATES_SERVER_EXIT);
-    delete new_container;
-    return true;
-  }
-
   if (news->have_ssl() && callbacks->warn_self_signed_ca()) {
     /* This would delete Ssl_acceptor_context_data too */
     delete new_container;
@@ -102,7 +102,7 @@ void TLS_channel::singleton_flush(Ssl_acceptor_context_container *container,
                                   enum enum_ssl_init_error *error, bool force) {
   if (container == nullptr) return;
   Ssl_acceptor_context_data *news =
-      new Ssl_acceptor_context_data(channel, callbacks, false, error);
+      new Ssl_acceptor_context_data(channel, true, callbacks, false, error);
   if (*error != SSL_INITERR_NOERROR && !force) {
     delete news;
     return;
@@ -234,12 +234,6 @@ int Ssl_mysql_main_status::show_ssl_ctx_sess_timeouts(THD *, SHOW_VAR *var,
       var, buff, Ssl_acceptor_context_property_type::session_cache_timeouts);
 }
 
-int Ssl_mysql_main_status::show_ssl_ctx_sess_timeout(THD *, SHOW_VAR *var,
-                                                     char *buff) {
-  return show_long_status(
-      var, buff, Ssl_acceptor_context_property_type::session_cache_timeout);
-}
-
 int Ssl_mysql_main_status::show_ssl_ctx_sess_number(THD *, SHOW_VAR *var,
                                                     char *buff) {
   return show_long_status(
