@@ -1,4 +1,4 @@
-/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,14 +32,13 @@ extern std::atomic_bool g_admin_ssl_configured;
 
 extern std::string mysql_main_channel;
 extern std::string mysql_admin_channel;
-extern bool opt_tls_certificates_enforced_validation;
 
 /** helper class to deal with optionally empty strings */
 class OptionalString {
  public:
   OptionalString() : value_(), empty_(true) {}
   OptionalString(const char *s) : value_(s ? s : ""), empty_(!s) {}
-  ~OptionalString() = default;
+  ~OptionalString() {}
   OptionalString(const OptionalString &) = default;
 
   const char *c_str() const { return empty_ ? nullptr : value_.c_str(); }
@@ -62,15 +61,13 @@ class Ssl_init_callback {
                                OptionalString *cipher,
                                OptionalString *ciphersuites,
                                OptionalString *key, OptionalString *crl,
-                               OptionalString *crl_path,
-                               bool *session_cache_mode,
-                               long *session_cache_timeout) = 0;
+                               OptionalString *crl_path) = 0;
 
   virtual bool provision_certs() = 0;
 
   virtual bool warn_self_signed_ca() = 0;
 
-  virtual ~Ssl_init_callback() = default;
+  virtual ~Ssl_init_callback() {}
 };
 
 /**
@@ -83,14 +80,13 @@ class Ssl_init_callback_server_main final : public Ssl_init_callback {
                        OptionalString *version, OptionalString *cert,
                        OptionalString *cipher, OptionalString *ciphersuites,
                        OptionalString *key, OptionalString *crl,
-                       OptionalString *crl_path, bool *session_cache_mode,
-                       long *session_cache_timeout) override;
+                       OptionalString *crl_path) override;
 
   bool provision_certs() override;
 
   bool warn_self_signed_ca() override;
 
-  ~Ssl_init_callback_server_main() override = default;
+  ~Ssl_init_callback_server_main() override {}
 
  private:
   ssl_artifacts_status auto_detect_ssl();
@@ -106,8 +102,7 @@ class Ssl_init_callback_server_admin final : public Ssl_init_callback {
                        OptionalString *version, OptionalString *cert,
                        OptionalString *cipher, OptionalString *ciphersuites,
                        OptionalString *key, OptionalString *crl,
-                       OptionalString *crl_path, bool *session_cache_mode,
-                       long *session_cache_timeout) override;
+                       OptionalString *crl_path) override;
 
   bool provision_certs() override {
     /*
@@ -119,20 +114,10 @@ class Ssl_init_callback_server_admin final : public Ssl_init_callback {
 
   bool warn_self_signed_ca() override;
 
-  ~Ssl_init_callback_server_admin() override = default;
+  ~Ssl_init_callback_server_admin() override {}
 };
 
 extern Ssl_init_callback_server_main server_main_callback;
 extern Ssl_init_callback_server_admin server_admin_callback;
 
-/**
-  Helper method to validate values of --tls-version and --admin-tls-version
-*/
-bool validate_tls_version(const char *val);
-
-enum class TLS_version { TLSv12 = 0, TLSv13 };
-/**
-  Helper method to validate values of --ssl-cipher and --admin-ssl-cipher
-*/
-void validate_ciphers(const char *option, const char *val, TLS_version version);
 #endif  // !SSL_INIT_CALLBACK_INCLUDED
