@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,21 +24,20 @@
 #define SYSTEM_VARIABLES_INCLUDED
 
 #include <stddef.h>
-#include <stdint.h>
 #include <sys/types.h>
 
+#include "m_ctype.h"
 #include "my_base.h"  // ha_rows
 #include "my_inttypes.h"
 #include "my_sqlcommand.h"
-#include "my_thread_local.h"  // my_thread_id
-#include "mysql/strings/m_ctype.h"
+#include "my_thread_local.h"     // my_thread_id
 #include "sql/rpl_gtid.h"        // Gitd_specification
 #include "sql/sql_plugin_ref.h"  // plugin_ref
 
 class MY_LOCALE;
 class Time_zone;
 
-using sql_mode_t = uint64_t;
+typedef ulonglong sql_mode_t;
 struct LIST;
 
 // Values for binlog_format sysvar
@@ -80,6 +79,13 @@ enum enum_binlog_row_metadata {
   BINLOG_ROW_METADATA_FULL = 1
 };
 
+// Values for transaction_write_set_extraction sysvar
+enum enum_transaction_write_set_hashing_algorithm {
+  HASH_ALGORITHM_OFF = 0,
+  HASH_ALGORITHM_MURMUR32 = 1,
+  HASH_ALGORITHM_XXHASH64 = 2
+};
+
 // Values for session_track_gtids sysvar
 enum enum_session_track_gtids {
   SESSION_TRACK_GTIDS_OFF = 0,
@@ -94,47 +100,20 @@ enum use_secondary_engine {
   SECONDARY_ENGINE_FORCED = 2
 };
 
-/**
-  Values for explain_format sysvar.
-
-  The value "TRADITIONAL_STRICT" is meant only to be used by the mtr test
-  suite. With hypergraph optimizer, if explain_format value is TRADITIONAL,
-  EXPLAIN without a format specifier prints in TREE format. The mtr tests were
-  written *before* this traditional-tree conversion was introduced. So mtr was
-  designed to just ignore the "format not supported with hypergraph" error when
-  a test runs an EXPLAIN without format specifier with --hypergraph. With the
-  conversion introduced, EXPLAIN without format specifier therefore would have
-  output in different formats with and without the mtr --hypergraph option.  In
-  order for the mtr tests to be able to continue to pass, mtr internally sets
-  explain_format to TRADITIONAL_STRICT so that these statements continue to
-  error out rather than print TREE format as they would do with TRADITIONAL
-  format. This is a temporary stuff.  Once all tests start using TREE format,
-  we will deprecate this value.
-*/
-enum class Explain_format_type : ulong {
-  TRADITIONAL = 0,
-  TRADITIONAL_STRICT = 1,
-  TREE = 2,
-  JSON = 3
-};
-
 /* Bits for different SQL modes modes (including ANSI mode) */
-inline constexpr sql_mode_t MODE_REAL_AS_FLOAT = 1;
-inline constexpr sql_mode_t MODE_PIPES_AS_CONCAT = 2;
-inline constexpr sql_mode_t MODE_ANSI_QUOTES = 4;
-inline constexpr sql_mode_t MODE_IGNORE_SPACE = 8;
-inline constexpr sql_mode_t MODE_NOT_USED = 16;
-inline constexpr sql_mode_t MODE_ONLY_FULL_GROUP_BY = 32;
-inline constexpr sql_mode_t MODE_NO_UNSIGNED_SUBTRACTION = 64;
-inline constexpr sql_mode_t MODE_NO_DIR_IN_CREATE = 128;
-inline constexpr sql_mode_t MODE_ANSI = 0x40000;
-inline constexpr sql_mode_t MODE_NO_AUTO_VALUE_ON_ZERO = MODE_ANSI * 2;
-inline constexpr sql_mode_t MODE_NO_BACKSLASH_ESCAPES =
-    MODE_NO_AUTO_VALUE_ON_ZERO * 2;
-inline constexpr sql_mode_t MODE_STRICT_TRANS_TABLES =
-    MODE_NO_BACKSLASH_ESCAPES * 2;
-inline constexpr sql_mode_t MODE_STRICT_ALL_TABLES =
-    MODE_STRICT_TRANS_TABLES * 2;
+#define MODE_REAL_AS_FLOAT 1
+#define MODE_PIPES_AS_CONCAT 2
+#define MODE_ANSI_QUOTES 4
+#define MODE_IGNORE_SPACE 8
+#define MODE_NOT_USED 16
+#define MODE_ONLY_FULL_GROUP_BY 32
+#define MODE_NO_UNSIGNED_SUBTRACTION 64
+#define MODE_NO_DIR_IN_CREATE 128
+#define MODE_ANSI 262144L
+#define MODE_NO_AUTO_VALUE_ON_ZERO (MODE_ANSI * 2)
+#define MODE_NO_BACKSLASH_ESCAPES (MODE_NO_AUTO_VALUE_ON_ZERO * 2)
+#define MODE_STRICT_TRANS_TABLES (MODE_NO_BACKSLASH_ESCAPES * 2)
+#define MODE_STRICT_ALL_TABLES (MODE_STRICT_TRANS_TABLES * 2)
 /*
  * NO_ZERO_DATE, NO_ZERO_IN_DATE and ERROR_FOR_DIVISION_BY_ZERO modes are
  * removed in 5.7 and their functionality is merged with STRICT MODE.
@@ -142,34 +121,31 @@ inline constexpr sql_mode_t MODE_STRICT_ALL_TABLES =
  * but they are not used. Setting these modes in 5.7 will give warning and
  * have no effect.
  */
-inline constexpr sql_mode_t MODE_NO_ZERO_IN_DATE = MODE_STRICT_ALL_TABLES * 2;
-inline constexpr sql_mode_t MODE_NO_ZERO_DATE = MODE_NO_ZERO_IN_DATE * 2;
-inline constexpr sql_mode_t MODE_INVALID_DATES = MODE_NO_ZERO_DATE * 2;
-inline constexpr sql_mode_t MODE_ERROR_FOR_DIVISION_BY_ZERO =
-    MODE_INVALID_DATES * 2;
-inline constexpr sql_mode_t MODE_TRADITIONAL =
-    MODE_ERROR_FOR_DIVISION_BY_ZERO * 2;
-inline constexpr sql_mode_t MODE_HIGH_NOT_PRECEDENCE = 1ULL << 29;
-inline constexpr sql_mode_t MODE_NO_ENGINE_SUBSTITUTION =
-    MODE_HIGH_NOT_PRECEDENCE * 2;
-inline constexpr sql_mode_t MODE_PAD_CHAR_TO_FULL_LENGTH = 1ULL << 31;
+#define MODE_NO_ZERO_IN_DATE (MODE_STRICT_ALL_TABLES * 2)
+#define MODE_NO_ZERO_DATE (MODE_NO_ZERO_IN_DATE * 2)
+#define MODE_INVALID_DATES (MODE_NO_ZERO_DATE * 2)
+#define MODE_ERROR_FOR_DIVISION_BY_ZERO (MODE_INVALID_DATES * 2)
+#define MODE_TRADITIONAL (MODE_ERROR_FOR_DIVISION_BY_ZERO * 2)
+#define MODE_HIGH_NOT_PRECEDENCE (1ULL << 29)
+#define MODE_NO_ENGINE_SUBSTITUTION (MODE_HIGH_NOT_PRECEDENCE * 2)
+#define MODE_PAD_CHAR_TO_FULL_LENGTH (1ULL << 31)
 /*
   If this mode is set the fractional seconds which cannot fit in given fsp will
   be truncated.
 */
-inline constexpr sql_mode_t MODE_TIME_TRUNCATE_FRACTIONAL = 1ULL << 32;
+#define MODE_TIME_TRUNCATE_FRACTIONAL (1ULL << 32)
 
-inline constexpr sql_mode_t MODE_LAST = 1ULL << 33;
+#define MODE_LAST (1ULL << 33)
 
-inline constexpr sql_mode_t MODE_ALLOWED_MASK =
-    (MODE_REAL_AS_FLOAT | MODE_PIPES_AS_CONCAT | MODE_ANSI_QUOTES |
-     MODE_IGNORE_SPACE | MODE_NOT_USED | MODE_ONLY_FULL_GROUP_BY |
-     MODE_NO_UNSIGNED_SUBTRACTION | MODE_NO_DIR_IN_CREATE | MODE_ANSI |
-     MODE_NO_AUTO_VALUE_ON_ZERO | MODE_NO_BACKSLASH_ESCAPES |
-     MODE_STRICT_TRANS_TABLES | MODE_STRICT_ALL_TABLES | MODE_NO_ZERO_IN_DATE |
-     MODE_NO_ZERO_DATE | MODE_INVALID_DATES | MODE_ERROR_FOR_DIVISION_BY_ZERO |
-     MODE_TRADITIONAL | MODE_HIGH_NOT_PRECEDENCE | MODE_NO_ENGINE_SUBSTITUTION |
-     MODE_PAD_CHAR_TO_FULL_LENGTH | MODE_TIME_TRUNCATE_FRACTIONAL);
+#define MODE_ALLOWED_MASK                                                      \
+  (MODE_REAL_AS_FLOAT | MODE_PIPES_AS_CONCAT | MODE_ANSI_QUOTES |              \
+   MODE_IGNORE_SPACE | MODE_NOT_USED | MODE_ONLY_FULL_GROUP_BY |               \
+   MODE_NO_UNSIGNED_SUBTRACTION | MODE_NO_DIR_IN_CREATE | MODE_ANSI |          \
+   MODE_NO_AUTO_VALUE_ON_ZERO | MODE_NO_BACKSLASH_ESCAPES |                    \
+   MODE_STRICT_TRANS_TABLES | MODE_STRICT_ALL_TABLES | MODE_NO_ZERO_IN_DATE |  \
+   MODE_NO_ZERO_DATE | MODE_INVALID_DATES | MODE_ERROR_FOR_DIVISION_BY_ZERO |  \
+   MODE_TRADITIONAL | MODE_HIGH_NOT_PRECEDENCE | MODE_NO_ENGINE_SUBSTITUTION | \
+   MODE_PAD_CHAR_TO_FULL_LENGTH | MODE_TIME_TRUNCATE_FRACTIONAL)
 
 /*
   We can safely ignore and reset these obsolete mode bits while replicating:
@@ -207,7 +183,7 @@ struct System_variables {
     They both should have the same 'version' and 'size'.
     When attempting to access a dynamic variable, if the session version
     is out of date, then the session version is updated and realloced if
-    necessary and bytes copied from global to make up for missing data.
+    neccessary and bytes copied from global to make up for missing data.
   */
   ulong dynamic_variables_version;
   char *dynamic_variables_ptr;
@@ -253,7 +229,6 @@ struct System_variables {
   ulong net_write_timeout;
   ulong optimizer_prune_level;
   ulong optimizer_search_depth;
-  ulong optimizer_max_subgraph_pairs;
   ulonglong parser_max_mem_size;
   ulong range_optimizer_max_mem_size;
   ulong preload_buff_size;
@@ -280,6 +255,8 @@ struct System_variables {
   uint binlog_trx_compression_level_zstd;
   ulonglong binlog_row_value_options;
   bool sql_log_bin;
+  // see enum_transaction_write_set_hashing_algorithm
+  ulong transaction_write_set_extraction;
   ulong completion_type;
   ulong transaction_isolation;
   ulong updatable_views_with_limit;
@@ -341,7 +318,7 @@ struct System_variables {
 
   double long_query_time_double;
 
-  bool pseudo_replica_mode;
+  bool pseudo_slave_mode;
 
   Gtid_specification gtid_next;
   Gtid_set_or_null gtid_next_list;
@@ -396,16 +373,6 @@ struct System_variables {
 
   bool sql_require_primary_key;
 
-  /**
-    @sa Sys_sql_generate_invisible_primary_key
-  */
-  bool sql_generate_invisible_primary_key;
-
-  /**
-    @sa Sys_show_gipk_in_create_table_and_information_schema
-  */
-  bool show_gipk_in_create_table_and_information_schema;
-
   /**
     Used in replication to determine the server version of the original server
     where the transaction was executed.
@@ -455,67 +422,8 @@ struct System_variables {
     @sa Sys_select_disk_sync_delay
   */
   uint select_into_disk_sync_delay;
-
-  /**
-    @sa Sys_terminology_use_previous
-  */
-  ulong terminology_use_previous;
-
-  /**
-    @sa Sys_connection_memory_limit
-  */
-  ulonglong conn_mem_limit;
-  /**
-    @sa Sys_connection_memory_chunk_size
-  */
-  ulong conn_mem_chunk_size;
-  /**
-    @sa Sys_connection_global_memory_tracking
-  */
-  bool conn_global_mem_tracking;
-
-  /**
-    Switch which controls whether XA transactions are detached
-    (made accessible to other connections for commit/rollback)
-    as part of XA PREPARE (true), or at session disconnect (false, default).
-    An important side effect of setting this to true is that temporary tables
-    are disallowed in XA transactions. This is necessary because temporary
-    tables and their contents (and thus changes to them) is bound to
-    specific connections, so they don't make sense if XA transaction is
-    committed or rolled back from another connection.
-   */
-  bool xa_detach_on_prepare;
-
-  /**
-    @sa Sys_debug_sensitive_session_string
-  */
-  char *debug_sensitive_session_str;
-
-  /**
-    Used to specify the format in which the EXPLAIN statement should display
-    information if the FORMAT option is not explicitly specified.
-    @sa Sys_explain_format
-   */
-  Explain_format_type explain_format;
-
-  /**
-    Used to specify the JSON format version used by EXPLAIN FORMAT=JSON with
-    the old (non-hypergraph) join optimizer.
-    @sa Sys_explain_json_format_version
-   */
-  uint explain_json_format_version;
-
-  /// Max size of set operations hash buffer size.
-  ulonglong set_operations_buffer_size;
-#ifndef NDEBUG
-  char *debug_set_operations_secondary_overflow_at;
-#endif
 };
 
-static_assert(std::is_trivially_copyable<System_variables>::value);
-static_assert(std::is_trivial<System_variables>::value);
-static_assert(std::is_standard_layout<System_variables>::value);
-
 /**
   Per thread status variables.
   Must be long/ulong up to last_system_status_var so that
