@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -35,7 +35,6 @@
 
 class KEY;
 class Item;
-class Item_copy;
 class Window;
 struct CHARSET_INFO;
 struct MEM_ROOT;
@@ -73,14 +72,6 @@ typedef Mem_root_array<Func_ptr> Func_ptr_array;
 
 class Temp_table_param {
  public:
-  /**
-    Used to store the values of grouped non-column-reference expressions in
-    between groups, so they can be retreived when the group changes.
-
-    @see setup_copy_fields
-    @see copy_fields
-  */
-  Mem_root_vector<Item_copy *> grouped_expressions;
   Mem_root_vector<Copy_field> copy_fields;
 
   uchar *group_buff;
@@ -196,8 +187,7 @@ class Temp_table_param {
   Window *m_window;
 
   Temp_table_param(MEM_ROOT *mem_root = *THR_MALLOC)
-      : grouped_expressions(Mem_root_allocator<Item_copy *>(mem_root)),
-        copy_fields(Mem_root_allocator<Copy_field>(mem_root)),
+      : copy_fields(Mem_root_allocator<Copy_field>(mem_root)),
         group_buff(nullptr),
         items_to_copy(nullptr),
         keyinfo(nullptr),
@@ -220,10 +210,7 @@ class Temp_table_param {
         m_window_short_circuit(false),
         m_window(nullptr) {}
 
-  void cleanup() {
-    grouped_expressions.clear();
-    copy_fields.clear();
-  }
+  void cleanup() { copy_fields.clear(); }
 };
 
 #endif  // TEMP_TABLE_PARAM_INCLUDED
