@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -225,7 +225,7 @@ class Transaction_ctx {
     m_savepoints = nullptr;
     m_xid_state.cleanup();
     m_rpl_transaction_ctx.cleanup();
-    m_transaction_write_set_ctx.clear_write_set();
+    m_transaction_write_set_ctx.reset_state();
     trans_begin_hook_invoked = false;
     free_root(&m_mem_root, MYF(MY_KEEP_PREALLOC));
     return;
@@ -256,7 +256,7 @@ class Transaction_ctx {
 
   void *allocate_memory(unsigned int size) { return m_mem_root.Alloc(size); }
 
-  void claim_memory_ownership() { m_mem_root.Claim(); }
+  void claim_memory_ownership(bool claim) { m_mem_root.Claim(claim); }
 
   void free_memory(myf root_alloc_flags) {
     free_root(&m_mem_root, root_alloc_flags);
