@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004, 2020, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2004, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,10 +27,6 @@
    (We will refer to this code as to elsie-code further.)
 */
 
-#ifdef TZINFO2SQL
-#define DISABLE_PSI_FILE 1
-#endif
-
 #include "sql/tztime.h"
 
 #include <algorithm>
@@ -61,6 +57,7 @@
 #include "my_psi_config.h"
 #include "my_sys.h"
 #include "my_time.h"  // MY_TIME_T_MIN
+#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
 #include "mysql/components/services/mysql_mutex_bits.h"
@@ -69,7 +66,6 @@
 #include "mysql/psi/mysql_file.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_mutex.h"
-#include "mysql/psi/psi_base.h"
 #include "mysqld_error.h"
 #include "sql/dd/types/event.h"
 #include "sql/field.h"
@@ -79,12 +75,12 @@
 #include "sql/sql_error.h"
 #include "sql/system_variables.h"
 #include "sql/thr_malloc.h"
-#include "sql/tzfile.h"  // TZ_MAX_REV_RANGES
+#include "sql/time_zone_common.h"  // ABBR_ARE_USED
+#include "sql/tzfile.h"            // TZ_MAX_REV_RANGES
 #include "template_utils.h"
 #include "thr_lock.h"
 #include "thr_mutex.h"
 
-#if !defined(TZINFO2SQL)
 #include "sql/debug_sync.h"  // DEBUG_SYNC
 #include "sql/log.h"
 #include "sql/mysqld.h"     // global_system_variables
@@ -93,7 +89,6 @@
 #include "sql/sql_time.h"   // localtime_to_TIME
 #include "sql/table.h"      // TABLE_LIST
 #include "sql_string.h"     // String
-#endif
 
 #include <algorithm>
 #include <string>
@@ -103,452 +98,6 @@
 #include "print_version.h"
 #include "welcome_copyright_notice.h" /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
-using std::min;
-
-/*
-  Macro for reading 32-bit integer from network byte order (big-endian)
-  from a unaligned memory location.
-*/
-#define int4net(A)                                                       \
-  (int32)(((uint32)((uchar)(A)[3])) | (((uint32)((uchar)(A)[2])) << 8) | \
-          (((uint32)((uchar)(A)[1])) << 16) |                            \
-          (((uint32)((uchar)(A)[0])) << 24))
-
-/*
-  Now we don't use abbreviations in server but we will do this in future.
-*/
-#if defined(TZINFO2SQL)
-#define ABBR_ARE_USED
-#else
-#if !defined(DBUG_OFF)
-/* Let use abbreviations for debug purposes */
-#undef ABBR_ARE_USED
-#define ABBR_ARE_USED
-#endif /* !defined(DBUG_OFF) */
-#endif /* defined(TZINFO2SQL) */
-
-/* Structure describing local time type (e.g. Moscow summer time (MSD)) */
-typedef struct ttinfo {
-  long tt_gmtoff;  // Offset from UTC in seconds
-  uint tt_isdst;   // Is daylight saving time or not. Used to set tm_isdst
-#ifdef ABBR_ARE_USED
-  uint tt_abbrind;  // Index of start of abbreviation for this time type.
-#endif
-  /*
-    We don't use tt_ttisstd and tt_ttisgmt members of original elsie-code
-    struct since we don't support POSIX-style TZ descriptions in variables.
-  */
-} TRAN_TYPE_INFO;
-
-/* Structure describing leap-second corrections. */
-typedef struct lsinfo {
-  my_time_t ls_trans;  // Transition time
-  long ls_corr;        // Correction to apply
-} LS_INFO;
-
-/*
-  Structure with information describing ranges of my_time_t shifted to local
-  time (my_time_t + offset). Used for local MYSQL_TIME -> my_time_t conversion.
-  See comments for TIME_to_gmt_sec() for more info.
-*/
-typedef struct revtinfo {
-  long rt_offset;  // Offset of local time from UTC in seconds
-  uint rt_type;    // Type of period 0 - Normal period. 1 - Spring time-gap
-} REVT_INFO;
-
-#ifdef TZNAME_MAX
-#define MY_TZNAME_MAX TZNAME_MAX
-#endif
-#ifndef TZNAME_MAX
-#define MY_TZNAME_MAX 255
-#endif
-
-/*
-  Structure which fully describes time zone which is
-  described in our db or in zoneinfo files.
-*/
-struct TIME_ZONE_INFO {
-  uint leapcnt;    // Number of leap-second corrections
-  uint timecnt;    // Number of transitions between time types
-  uint typecnt;    // Number of local time types
-  size_t charcnt;  // Number of characters used for abbreviations
-  uint revcnt;     // Number of transition descr. for TIME->my_time_t conversion
-  /* The following are dynamical arrays are allocated in MEM_ROOT */
-  my_time_t *ats;        // Times of transitions between time types
-  uchar *types;          // Local time types for transitions
-  TRAN_TYPE_INFO *ttis;  // Local time types descriptions
-#ifdef ABBR_ARE_USED
-  /* Storage for local time types abbreviations. They are stored as ASCIIZ */
-  char *chars;
-#endif
-  /*
-    Leap seconds corrections descriptions, this array is shared by
-    all time zones who use leap seconds.
-  */
-  LS_INFO *lsis;
-  /*
-    Starting points and descriptions of shifted my_time_t (my_time_t + offset)
-    ranges on which shifted my_time_t -> my_time_t mapping is linear or
-    undefined. Used for tm -> my_time_t conversion.
-  */
-  my_time_t *revts;
-  REVT_INFO *revtis;
-  /*
-    Time type which is used for times smaller than first transition or if
-    there are no transitions at all.
-  */
-  TRAN_TYPE_INFO *fallback_tti;
-};
-
-static bool prepare_tz_info(TIME_ZONE_INFO *sp, MEM_ROOT *storage);
-
-#if defined(TZINFO2SQL)
-
-#ifdef ABBR_ARE_USED
-static const char *const MAGIC_STRING_FOR_INVALID_ZONEINFO_FILE =
-    "Local time zone must be set--see zic manual page";
-#endif
-
-/*
-  Load time zone description from zoneinfo (TZinfo) file.
-
-  SYNOPSIS
-    tz_load()
-      name - path to zoneinfo file
-      sp   - TIME_ZONE_INFO structure to fill
-
-  RETURN VALUES
-    0 - Ok
-    1 - Error
-*/
-static bool tz_load(const char *name, TIME_ZONE_INFO *sp, MEM_ROOT *storage) {
-  uchar *p;
-  size_t read_from_file;
-  uint i;
-  MYSQL_FILE *file;
-
-  if (!(file =
-            mysql_file_fopen(0, name, O_RDONLY | MY_FOPEN_BINARY, MYF(MY_WME))))
-    return true;
-  {
-    union {
-      struct tzhead tzhead;
-      uchar buf[sizeof(struct tzhead) + sizeof(my_time_t) * TZ_MAX_TIMES +
-                TZ_MAX_TIMES + sizeof(TRAN_TYPE_INFO) * TZ_MAX_TYPES +
-#ifdef ABBR_ARE_USED
-                std::max(TZ_MAX_CHARS + 1, (2 * (MY_TZNAME_MAX + 1))) +
-#endif
-                sizeof(LS_INFO) * TZ_MAX_LEAPS];
-    } u;
-    uint ttisstdcnt;
-    uint ttisgmtcnt;
-    char *tzinfo_buf;
-
-    read_from_file = mysql_file_fread(file, u.buf, sizeof(u.buf), MYF(MY_WME));
-
-    if (mysql_file_fclose(file, MYF(MY_WME)) != 0) return true;
-
-    if (read_from_file < sizeof(struct tzhead)) return true;
-
-    ttisstdcnt = int4net(u.tzhead.tzh_ttisgmtcnt);
-    ttisgmtcnt = int4net(u.tzhead.tzh_ttisstdcnt);
-    sp->leapcnt = int4net(u.tzhead.tzh_leapcnt);
-    sp->timecnt = int4net(u.tzhead.tzh_timecnt);
-    sp->typecnt = int4net(u.tzhead.tzh_typecnt);
-    sp->charcnt = int4net(u.tzhead.tzh_charcnt);
-    p = u.tzhead.tzh_charcnt + sizeof(u.tzhead.tzh_charcnt);
-    if (sp->leapcnt > TZ_MAX_LEAPS || sp->typecnt == 0 ||
-        sp->typecnt > TZ_MAX_TYPES || sp->timecnt > TZ_MAX_TIMES ||
-        sp->charcnt > TZ_MAX_CHARS ||
-        (ttisstdcnt != sp->typecnt && ttisstdcnt != 0) ||
-        (ttisgmtcnt != sp->typecnt && ttisgmtcnt != 0))
-      return true;
-    if ((uint)(read_from_file - (p - u.buf)) <
-        sp->timecnt * 4 +           /* ats */
-            sp->timecnt +           /* types */
-            sp->typecnt * (4 + 2) + /* ttinfos */
-            sp->charcnt +           /* chars */
-            sp->leapcnt * (4 + 4) + /* lsinfos */
-            ttisstdcnt +            /* ttisstds */
-            ttisgmtcnt)             /* ttisgmts */
-      return true;
-
-#ifdef ABBR_ARE_USED
-    size_t start_of_zone_abbrev = sizeof(struct tzhead) +
-                                  sp->timecnt * 4 +      /* ats */
-                                  sp->timecnt +          /* types */
-                                  sp->typecnt * (4 + 2); /* ttinfos */
-
-    /*
-      Check that timezone file doesn't contain junk timezone data.
-    */
-    if (!memcmp(u.buf + start_of_zone_abbrev,
-                MAGIC_STRING_FOR_INVALID_ZONEINFO_FILE,
-                std::min(sizeof(MAGIC_STRING_FOR_INVALID_ZONEINFO_FILE) - 1,
-                         sp->charcnt)))
-      return true;
-
-    size_t abbrs_buf_len = sp->charcnt + 1;
-#endif
-
-    if (!(tzinfo_buf = (char *)storage->Alloc(
-              ALIGN_SIZE(sp->timecnt * sizeof(my_time_t)) +
-              ALIGN_SIZE(sp->timecnt) +
-              ALIGN_SIZE(sp->typecnt * sizeof(TRAN_TYPE_INFO)) +
-#ifdef ABBR_ARE_USED
-              ALIGN_SIZE(abbrs_buf_len) +
-#endif
-              sp->leapcnt * sizeof(LS_INFO))))
-      return true;
-
-    sp->ats = (my_time_t *)tzinfo_buf;
-    tzinfo_buf += ALIGN_SIZE(sp->timecnt * sizeof(my_time_t));
-    sp->types = (uchar *)tzinfo_buf;
-    tzinfo_buf += ALIGN_SIZE(sp->timecnt);
-    sp->ttis = (TRAN_TYPE_INFO *)tzinfo_buf;
-    tzinfo_buf += ALIGN_SIZE(sp->typecnt * sizeof(TRAN_TYPE_INFO));
-#ifdef ABBR_ARE_USED
-    sp->chars = tzinfo_buf;
-    tzinfo_buf += ALIGN_SIZE(abbrs_buf_len);
-#endif
-    sp->lsis = (LS_INFO *)tzinfo_buf;
-
-    for (i = 0; i < sp->timecnt; i++, p += 4) sp->ats[i] = int4net(p);
-
-    for (i = 0; i < sp->timecnt; i++) {
-      sp->types[i] = *p++;
-      if (sp->types[i] >= sp->typecnt) return true;
-    }
-    for (i = 0; i < sp->typecnt; i++) {
-      TRAN_TYPE_INFO *ttisp;
-
-      ttisp = &sp->ttis[i];
-      ttisp->tt_gmtoff = int4net(p);
-      p += 4;
-      ttisp->tt_isdst = *p++;
-      if (ttisp->tt_isdst != 0 && ttisp->tt_isdst != 1) return true;
-      ttisp->tt_abbrind = *p++;
-      if (ttisp->tt_abbrind > sp->charcnt) return true;
-    }
-    for (i = 0; i < sp->charcnt; i++) sp->chars[i] = *p++;
-    sp->chars[i] = '\0'; /* ensure '\0' at end */
-    for (i = 0; i < sp->leapcnt; i++) {
-      LS_INFO *lsisp;
-
-      lsisp = &sp->lsis[i];
-      lsisp->ls_trans = int4net(p);
-      p += 4;
-      lsisp->ls_corr = int4net(p);
-      p += 4;
-    }
-    /*
-      Since we don't support POSIX style TZ definitions in variables we
-      don't read further like glibc or elsie code.
-    */
-  }
-
-  return prepare_tz_info(sp, storage);
-}
-#endif /* defined(TZINFO2SQL) */
-
-/*
-  Finish preparation of time zone description for use in TIME_to_gmt_sec()
-  and gmt_sec_to_TIME() functions.
-
-  SYNOPSIS
-    prepare_tz_info()
-      sp - pointer to time zone description
-      storage - pointer to MEM_ROOT where arrays for map allocated
-
-  DESCRIPTION
-    First task of this function is to find fallback time type which will
-    be used if there are no transitions or we have moment in time before
-    any transitions.
-    Second task is to build "shifted my_time_t" -> my_time_t map used in
-    MYSQL_TIME -> my_time_t conversion.
-    Note: See description of TIME_to_gmt_sec() function first.
-    In order to perform MYSQL_TIME -> my_time_t conversion we need to build
-  table which defines "shifted by tz offset and leap seconds my_time_t" ->
-    my_time_t function wich is almost the same (except ranges of ambiguity)
-    as reverse function to piecewise linear function used for my_time_t ->
-    "shifted my_time_t" conversion and which is also specified as table in
-    zoneinfo file or in our db (It is specified as start of time type ranges
-    and time type offsets). So basic idea is very simple - let us iterate
-    through my_time_t space from one point of discontinuity of my_time_t ->
-    "shifted my_time_t" function to another and build our approximation of
-    reverse function. (Actually we iterate through ranges on which
-    my_time_t -> "shifted my_time_t" is linear function).
-
-  RETURN VALUES
-    0	Ok
-    1	Error
-*/
-static bool prepare_tz_info(TIME_ZONE_INFO *sp, MEM_ROOT *storage) {
-  my_time_t cur_t = MY_TIME_T_MIN;
-  my_time_t cur_l, end_t, end_l = 0;
-  my_time_t cur_max_seen_l = MY_TIME_T_MIN;
-  long cur_offset, cur_corr, cur_off_and_corr;
-  uint next_trans_idx, next_leap_idx;
-  uint i;
-  /*
-    Temporary arrays where we will store tables. Needed because
-    we don't know table sizes ahead. (Well we can estimate their
-    upper bound but this will take extra space.)
-  */
-  my_time_t revts[TZ_MAX_REV_RANGES];
-  REVT_INFO revtis[TZ_MAX_REV_RANGES];
-
-  /*
-    Let us setup fallback time type which will be used if we have not any
-    transitions or if we have moment of time before first transition.
-    We will find first non-DST local time type and use it (or use first
-    local time type if all of them are DST types).
-  */
-  for (i = 0; i < sp->typecnt && sp->ttis[i].tt_isdst; i++) /* no-op */
-    ;
-  if (i == sp->typecnt) i = 0;
-  sp->fallback_tti = &(sp->ttis[i]);
-
-  /*
-    Let us build shifted my_time_t -> my_time_t map.
-  */
-  sp->revcnt = 0;
-
-  /* Let us find initial offset */
-  if (sp->timecnt == 0 || cur_t < sp->ats[0]) {
-    /*
-      If we have not any transitions or t is before first transition we are
-      using already found fallback time type which index is already in i.
-    */
-    next_trans_idx = 0;
-  } else {
-    /* cur_t == sp->ats[0] so we found transition */
-    i = sp->types[0];
-    next_trans_idx = 1;
-  }
-
-  cur_offset = sp->ttis[i].tt_gmtoff;
-
-  /* let us find leap correction... unprobable, but... */
-  for (next_leap_idx = 0;
-       next_leap_idx < sp->leapcnt && cur_t >= sp->lsis[next_leap_idx].ls_trans;
-       ++next_leap_idx)
-    continue;
-
-  if (next_leap_idx > 0)
-    cur_corr = sp->lsis[next_leap_idx - 1].ls_corr;
-  else
-    cur_corr = 0;
-
-  /* Iterate trough t space */
-  while (sp->revcnt < TZ_MAX_REV_RANGES - 1) {
-    cur_off_and_corr = cur_offset - cur_corr;
-
-    /*
-      We assuming that cur_t could be only overflowed downwards,
-      we also assume that end_t won't be overflowed in this case.
-    */
-    if (cur_off_and_corr < 0 && cur_t < MY_TIME_T_MIN - cur_off_and_corr)
-      cur_t = MY_TIME_T_MIN - cur_off_and_corr;
-
-    cur_l = cur_t + cur_off_and_corr;
-
-    /*
-      Let us choose end_t as point before next time type change or leap
-      second correction.
-    */
-    end_t =
-        min((next_trans_idx < sp->timecnt) ? sp->ats[next_trans_idx] - 1
-                                           : MY_TIME_T_MAX,
-            (next_leap_idx < sp->leapcnt) ? sp->lsis[next_leap_idx].ls_trans - 1
-                                          : MY_TIME_T_MAX);
-    /*
-      again assuming that end_t can be overlowed only in positive side
-      we also assume that end_t won't be overflowed in this case.
-    */
-    if (cur_off_and_corr > 0 && end_t > MY_TIME_T_MAX - cur_off_and_corr)
-      end_t = MY_TIME_T_MAX - cur_off_and_corr;
-
-    end_l = end_t + cur_off_and_corr;
-
-    if (end_l > cur_max_seen_l) {
-      /* We want special handling in the case of first range */
-      if (cur_max_seen_l == MY_TIME_T_MIN) {
-        revts[sp->revcnt] = cur_l;
-        revtis[sp->revcnt].rt_offset = cur_off_and_corr;
-        revtis[sp->revcnt].rt_type = 0;
-        sp->revcnt++;
-        cur_max_seen_l = end_l;
-      } else {
-        if (cur_l > cur_max_seen_l + 1) {
-          /* We have a spring time-gap and we are not at the first range */
-          revts[sp->revcnt] = cur_max_seen_l + 1;
-          revtis[sp->revcnt].rt_offset = revtis[sp->revcnt - 1].rt_offset;
-          revtis[sp->revcnt].rt_type = 1;
-          sp->revcnt++;
-          if (sp->revcnt == TZ_MAX_TIMES + TZ_MAX_LEAPS + 1)
-            break; /* That was too much */
-          cur_max_seen_l = cur_l - 1;
-        }
-
-        /* Assume here end_l > cur_max_seen_l (because end_l>=cur_l) */
-
-        revts[sp->revcnt] = cur_max_seen_l + 1;
-        revtis[sp->revcnt].rt_offset = cur_off_and_corr;
-        revtis[sp->revcnt].rt_type = 0;
-        sp->revcnt++;
-        cur_max_seen_l = end_l;
-      }
-    }
-
-    if (end_t == MY_TIME_T_MAX ||
-        ((cur_off_and_corr > 0) && (end_t >= MY_TIME_T_MAX - cur_off_and_corr)))
-      /* end of t space */
-      break;
-
-    cur_t = end_t + 1;
-
-    /*
-      Let us find new offset and correction. Because of our choice of end_t
-      cur_t can only be point where new time type starts or/and leap
-      correction is performed.
-    */
-    if (sp->timecnt != 0 && cur_t >= sp->ats[0]) /* else reuse old offset */
-      if (next_trans_idx < sp->timecnt && cur_t == sp->ats[next_trans_idx]) {
-        /* We are at offset point */
-        cur_offset = sp->ttis[sp->types[next_trans_idx]].tt_gmtoff;
-        ++next_trans_idx;
-      }
-
-    if (next_leap_idx < sp->leapcnt &&
-        cur_t == sp->lsis[next_leap_idx].ls_trans) {
-      /* we are at leap point */
-      cur_corr = sp->lsis[next_leap_idx].ls_corr;
-      ++next_leap_idx;
-    }
-  }
-
-  /* check if we have had enough space */
-  if (sp->revcnt == TZ_MAX_REV_RANGES - 1) return true;
-
-  /* set maximum end_l as finisher */
-  revts[sp->revcnt] = end_l;
-
-  /* Allocate arrays of proper size in sp and copy result there */
-  if (!(sp->revts = (my_time_t *)storage->Alloc(sizeof(my_time_t) *
-                                                (sp->revcnt + 1))) ||
-      !(sp->revtis =
-            (REVT_INFO *)storage->Alloc(sizeof(REVT_INFO) * sp->revcnt)))
-    return true;
-
-  memcpy(sp->revts, revts, sizeof(my_time_t) * (sp->revcnt + 1));
-  memcpy(sp->revtis, revtis, sizeof(REVT_INFO) * sp->revcnt);
-
-  return false;
-}
-
-#if !defined(TZINFO2SQL)
-
 static const uint mon_lengths[2][MONS_PER_YEAR] = {
     {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
     {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}};
@@ -781,6 +330,31 @@ static void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t sec_in_utc,
   tmp->second += hit;
 }
 
+/**
+  Converts time from a MYSQL_TIME struct to a unix timestamp-like 64 bit
+  integer. The function is guaranteed to use 64 bits on any platform.
+
+  @todo Make sec_since_epoch() call this function instead of duplicating the
+  code.
+
+  @param mt The time to convert.
+  @return A value compatible with a 64 bit Unix timestamp.
+*/
+static int64_t sec_since_epoch64(const MYSQL_TIME &mt) {
+  DBUG_ASSERT(mt.month > 0 && mt.month < 13);
+  // The year can be negative wrt to the epoch, hence the cast to signed.
+  auto year = static_cast<int64_t>(mt.year);
+  int64_t days = year * DAYS_PER_NYEAR - EPOCH_YEAR * DAYS_PER_NYEAR +
+                 LEAPS_THRU_END_OF(year - 1) -
+                 LEAPS_THRU_END_OF(EPOCH_YEAR - 1);
+  days += mon_starts[isleap(year)][mt.month - 1];
+  days += mt.day - 1;
+
+  return ((days * HOURS_PER_DAY + mt.hour) * MINS_PER_HOUR + mt.minute) *
+             SECS_PER_MIN +
+         mt.second;
+}
+
 /*
   Converts local time in broken down representation to local
   time zone analog of my_time_t represenation.
@@ -984,9 +558,6 @@ static my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, const TIME_ZONE_INFO *sp,
 /*
   End of elsie derived code.
 */
-#endif /* !defined(TZINFO2SQL) */
-
-#if !defined(TZINFO2SQL)
 
 /*
   String with names of SYSTEM time zone.
@@ -995,27 +566,76 @@ static const String tz_SYSTEM_name("SYSTEM", 6, &my_charset_latin1);
 
 Time_zone *my_tz_find(const int64 displacement);
 
+static void raise_time_zone_conversion_error(const MYSQL_TIME &mt) {
+  char str[MAX_DATE_STRING_REP_LENGTH];
+  // TODO(mhansson) Get the correct number of decimal places into the error
+  // message.  This is non-trivial, as this is part of the meta-data, which
+  // (for some reason) is not included in a MYSQL_TIME.
+  my_datetime_to_str(mt, str, 0);
+
+  my_error(ER_TRUNCATED_WRONG_VALUE, myf(0), "temporal", str);
+}
+
+/**
+  Checks that this temporal value can be converted from its specified time
+  zone (if any) to the current time zone. Specifically, temporal values with
+  zero months or days cannot be converted between time zones.
+
+  @param mt The time to check.
+  @retval false The temporal value has no time zone or can be converted.
+  @retval true Otherwise, and an error was raised.
+*/
+bool check_time_zone_convertibility(const MYSQL_TIME &mt) {
+  if (mt.time_type == MYSQL_TIMESTAMP_DATETIME_TZ &&
+      (mt.month < 1 || mt.day < 1)) {
+    raise_time_zone_conversion_error(mt);
+    return true;
+  }
+  return false;
+}
+
 /**
   Converts a date/time value with time zone to the corresponding date/time value
-  without time zone, adjusted to be in time zone specified by argument @p tz.
+  without time zone, converted to be in time zone specified by argument @p tz.
 
-  This function is intended only for the types with time zone, and is a no-op
+  Since MySQL doesn't have a data type for temporal values with time zone
+  information, all such values are converted to a value without time zone
+  using this function.
+
+  This function is intended only for values with a time zone, and is a no-op
   for all other types.
 
-  @param tz The time zone to adjust according to.
-  @param[in,out] mt Date/Time value to be adjusted.
+  The converted value may not fall outside the range of the `DATETIME` type.
+  Also some invalid values cannot be converted because the conversion result
+  would be undefined. In these cases an error is raised.
+
+  @param tz The time zone to convert according to.
+  @param[in,out] mt Date/Time value to be converted.
+
+  @return false on success. true if an error was raised.
 */
-void adjust_time_zone_displacement(const Time_zone *tz, MYSQL_TIME *mt) {
-  if (mt->time_type != MYSQL_TIMESTAMP_DATETIME_TZ) return;
+bool convert_time_zone_displacement(const Time_zone *tz, MYSQL_TIME *mt) {
+  if (mt->time_type != MYSQL_TIMESTAMP_DATETIME_TZ) return false;
+
+  if (check_time_zone_convertibility(*mt)) return true;
+
+  MYSQL_TIME out;
+  std::int64_t epoch_secs_in_utc =
+      sec_since_epoch64(*mt) - mt->time_zone_displacement;
 
-  my_time_t epoch_secs_in_utc =
-      sec_since_epoch(*mt) - mt->time_zone_displacement;
   ulong microseconds = mt->second_part;
 
-  tz->gmt_sec_to_TIME(mt, epoch_secs_in_utc);
-  mt->second_part = microseconds;
+  tz->gmt_sec_to_TIME(&out, epoch_secs_in_utc);
+  out.second_part = microseconds;
 
+  if (check_datetime_range(out)) {
+    raise_time_zone_conversion_error(out);
+    return true;
+  }
+
+  *mt = out;
   DBUG_ASSERT(mt->time_type == MYSQL_TIMESTAMP_DATETIME);
+  return false;
 }
 
 /*
@@ -1030,10 +650,10 @@ void adjust_time_zone_displacement(const Time_zone *tz, MYSQL_TIME *mt) {
 */
 class Time_zone_system : public Time_zone {
  public:
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    bool *in_dst_time_gap) const;
-  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String *get_name() const;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                            bool *in_dst_time_gap) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String *get_name() const override;
 };
 
 /*
@@ -1116,10 +736,10 @@ const String *Time_zone_system::get_name() const { return &tz_SYSTEM_name; }
 */
 class Time_zone_utc : public Time_zone {
  public:
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    bool *in_dst_time_gap) const;
-  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String *get_name() const;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                            bool *in_dst_time_gap) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String *get_name() const override;
 };
 
 /*
@@ -1190,10 +810,10 @@ const String *Time_zone_utc::get_name() const {
 class Time_zone_db : public Time_zone {
  public:
   Time_zone_db(TIME_ZONE_INFO *tz_info_arg, const String *tz_name_arg);
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    bool *in_dst_time_gap) const;
-  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String *get_name() const;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                            bool *in_dst_time_gap) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String *get_name() const override;
 
  private:
   TIME_ZONE_INFO *tz_info;
@@ -1273,10 +893,10 @@ const String *Time_zone_db::get_name() const { return tz_name; }
 class Time_zone_offset : public Time_zone {
  public:
   Time_zone_offset(long tz_offset_arg);
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    bool *in_dst_time_gap) const;
-  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String *get_name() const;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                            bool *in_dst_time_gap) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String *get_name() const override;
 
  private:
   /* Extra reserve because of snprintf */
@@ -1385,12 +1005,28 @@ Time_zone *my_tz_SYSTEM = &tz_SYSTEM;
 
 static MEM_ROOT tz_storage;
 
-/*
-  These mutex protects offset_tzs and tz_storage.
-  These protection needed only when we are trying to set
-  time zone which is specified as offset, and searching for existing
-  time zone in offset_tzs or creating if it didn't existed before in
-  tz_storage. So contention is low.
+/**
+  This mutex has two orthogonal purposes:
+
+  -# When the caller of my_tz_find() needs a Time_zone object
+     representing a time zone specified as a numeric displacement. The mutex is
+     then taken in order to protect the offset_tzs map and the
+     performance_schema key tz_storage. my_tz_find() uses a shared pool of
+     Time_zone objects and will search to see if there is an existing time zone,
+     and will otherwise create and insert one. So contention is low.
+
+  -# When the caller of my_tz_find() needs a Time_zone object
+     by name. First the tz_names map is searched, and if nothing is found, the
+     database tables are consulted. If nothing is found there either, an
+     error is thrown. If one is found, tz_load_from_open_tables() tries to
+     insert it in the map, and if it is already there, it fails, logging an "out
+     of memory" event. And that's the reason the whole procedure must take place
+     under a mutex, so that another session couldn't have inserted it in the
+     mean time.
+
+  It is not clear why the same mutex is used for both operations, or for that
+  matter why it is taken even before we have decided which of the two paths
+  above to take.
 */
 static mysql_mutex_t tz_LOCK;
 static bool tz_inited = false;
@@ -1709,6 +1345,7 @@ void my_tz_free() {
     tz_tables should be time_zone_name, next time_zone, then
     time_zone_transition_type and time_zone_transition should be last).
     It will also update information in hash used for time zones lookup.
+    Therefore, it is assumed that this function is called while tz_LOCK is held.
 
   RETURN VALUES
     Returns pointer to newly created Time_zone object or 0 in case of error.
@@ -1757,7 +1394,8 @@ static Time_zone *tz_load_from_open_tables(const String *tz_name,
 
   if (table->file->ha_index_init(0, true)) goto end;
 
-  res = table->file->ha_index_read_map(table->record[0], table->field[0]->ptr,
+  res = table->file->ha_index_read_map(table->record[0],
+                                       table->field[0]->field_ptr(),
                                        HA_WHOLE_KEY, HA_READ_KEY_EXACT);
   if (res) {
     /*
@@ -1792,7 +1430,8 @@ static Time_zone *tz_load_from_open_tables(const String *tz_name,
   table->field[0]->store((longlong)tzid, true);
   if (table->file->ha_index_init(0, true)) goto end;
 
-  res = table->file->ha_index_read_map(table->record[0], table->field[0]->ptr,
+  res = table->file->ha_index_read_map(table->record[0],
+                                       table->field[0]->field_ptr(),
                                        HA_WHOLE_KEY, HA_READ_KEY_EXACT);
   if (res) {
     DBUG_ASSERT(res != HA_ERR_LOCK_WAIT_TIMEOUT && res != HA_ERR_LOCK_DEADLOCK);
@@ -1820,7 +1459,8 @@ static Time_zone *tz_load_from_open_tables(const String *tz_name,
   table->field[0]->store((longlong)tzid, true);
   if (table->file->ha_index_init(0, true)) goto end;
 
-  res = table->file->ha_index_read_map(table->record[0], table->field[0]->ptr,
+  res = table->file->ha_index_read_map(table->record[0],
+                                       table->field[0]->field_ptr(),
                                        (key_part_map)1, HA_READ_KEY_EXACT);
   while (!res) {
     ttid = (uint)table->field[1]->val_int();
@@ -1867,7 +1507,7 @@ static Time_zone *tz_load_from_open_tables(const String *tz_name,
     tmp_tz_info.typecnt = ttid + 1;
 
     res = table->file->ha_index_next_same(table->record[0],
-                                          table->field[0]->ptr, 4);
+                                          table->field[0]->field_ptr(), 4);
   }
 
   if (res != HA_ERR_END_OF_FILE) {
@@ -1887,7 +1527,8 @@ static Time_zone *tz_load_from_open_tables(const String *tz_name,
   table->field[0]->store((longlong)tzid, true);
   if (table->file->ha_index_init(0, true)) goto end;
 
-  res = table->file->ha_index_read_map(table->record[0], table->field[0]->ptr,
+  res = table->file->ha_index_read_map(table->record[0],
+                                       table->field[0]->field_ptr(),
                                        (key_part_map)1, HA_READ_KEY_EXACT);
   while (!res) {
     ttime = (my_time_t)table->field[1]->val_int();
@@ -1912,7 +1553,7 @@ static Time_zone *tz_load_from_open_tables(const String *tz_name,
          (ulong)ttime, ttid));
 
     res = table->file->ha_index_next_same(table->record[0],
-                                          table->field[0]->ptr, 4);
+                                          table->field[0]->field_ptr(), 4);
   }
 
   /*
@@ -1994,6 +1635,10 @@ static Time_zone *tz_load_from_open_tables(const String *tz_name,
       (tmp_tzname->name.set(tz_name_buff, tz_name->length(),
                             &my_charset_latin1),
        !tz_names.emplace(to_string(tmp_tzname->name), tmp_tzname).second)) {
+    /*
+       We get here if either some new operator or String::set() returned
+       nullptr, or if *the time zone is already in the map*.
+    */
     LogErr(ERROR_LEVEL, ER_TZ_OOM_WHILE_LOADING_TIME_ZONE);
     goto end;
   }
@@ -2189,213 +1834,3 @@ Time_zone *my_tz_find(THD *thd, const String *name) {
 void Time_zone::adjust_leap_second(MYSQL_TIME *t) {
   if (t->second == 60 || t->second == 61) t->second = 59;
 }
-
-#endif /* !defined(TZINFO2SQL) */
-
-#ifdef TZINFO2SQL
-/*
-  This code belongs to mysql_tzinfo_to_sql converter command line utility.
-  This utility should be used by db admin for populating mysql.time_zone
-  tables.
-*/
-
-/*
-  Print info about time zone described by TIME_ZONE_INFO struct as
-  SQL statements populating mysql.time_zone* tables.
-
-  SYNOPSIS
-    print_tz_as_sql()
-      tz_name - name of time zone
-      sp      - structure describing time zone
-*/
-static void print_tz_as_sql(const char *tz_name, const TIME_ZONE_INFO *sp) {
-  uint i;
-
-  /* Here we assume that all time zones have same leap correction tables */
-  printf("INSERT INTO time_zone (Use_leap_seconds) VALUES ('%s');\n",
-         sp->leapcnt ? "Y" : "N");
-  printf("SET @time_zone_id= LAST_INSERT_ID();\n");
-  printf(
-      "INSERT INTO time_zone_name (Name, Time_zone_id) VALUES \
-('%s', @time_zone_id);\n",
-      tz_name);
-
-  if (sp->timecnt) {
-    printf(
-        "INSERT INTO time_zone_transition \
-(Time_zone_id, Transition_time, Transition_type_id) VALUES\n");
-    for (i = 0; i < sp->timecnt; i++)
-      printf("%s(@time_zone_id, %ld, %u)\n", (i == 0 ? " " : ","), sp->ats[i],
-             (uint)sp->types[i]);
-    printf(";\n");
-  }
-
-  printf(
-      "INSERT INTO time_zone_transition_type \
-(Time_zone_id, Transition_type_id, Offset, Is_DST, Abbreviation) VALUES\n");
-
-  for (i = 0; i < sp->typecnt; i++)
-    /*
-      Since the column time_zone_transition_type.Abbreviation
-      is declared as CHAR(8) we have to limit the number of characters
-      for the column abbreviation in the next output by 8 chars.
-    */
-    printf("%s(@time_zone_id, %u, %ld, %d, '%.8s')\n", (i == 0 ? " " : ","), i,
-           sp->ttis[i].tt_gmtoff, sp->ttis[i].tt_isdst,
-           sp->chars + sp->ttis[i].tt_abbrind);
-  printf(";\n");
-}
-
-/*
-  Print info about leap seconds in time zone as SQL statements
-  populating mysql.time_zone_leap_second table.
-
-  SYNOPSIS
-    print_tz_leaps_as_sql()
-      sp      - structure describing time zone
-*/
-static void print_tz_leaps_as_sql(const TIME_ZONE_INFO *sp) {
-  uint i;
-
-  /*
-    We are assuming that there are only one list of leap seconds
-    For all timezones.
-  */
-  printf("TRUNCATE TABLE time_zone_leap_second;\n");
-  printf("START TRANSACTION;\n");
-  if (sp->leapcnt) {
-    printf(
-        "INSERT INTO time_zone_leap_second \
-(Transition_time, Correction) VALUES\n");
-    for (i = 0; i < sp->leapcnt; i++)
-      printf("%s(%ld, %ld)\n", (i == 0 ? " " : ","), sp->lsis[i].ls_trans,
-             sp->lsis[i].ls_corr);
-    printf(";\n");
-  }
-  printf("COMMIT;\n");
-}
-
-/*
-  Some variables used as temporary or as parameters
-  in recursive scan_tz_dir() code.
-*/
-TIME_ZONE_INFO tz_info;
-MEM_ROOT tz_storage;
-char fullname[FN_REFLEN + 1];
-char *root_name_end;
-
-/*
-  Recursively scan zoneinfo directory and print all found time zone
-  descriptions as SQL.
-
-  SYNOPSIS
-    scan_tz_dir()
-      name_end - pointer to end of path to directory to be searched.
-
-  DESCRIPTION
-    This auxiliary recursive function also uses several global
-    variables as in parameters and for storing temporary values.
-
-    fullname      - path to directory that should be scanned.
-    root_name_end - pointer to place in fullname where part with
-                    path to initial directory ends.
-    current_tz_id - last used time zone id
-
-  RETURN VALUE
-    0 - Ok, 1 - Fatal error
-
-*/
-static bool scan_tz_dir(char *name_end) {
-  MY_DIR *cur_dir;
-  char *name_end_tmp;
-  uint i;
-
-  if (!(cur_dir = my_dir(fullname, MYF(MY_WANT_STAT)))) return true;
-
-  name_end = strmake(name_end, "/", FN_REFLEN - (name_end - fullname));
-
-  for (i = 0; i < cur_dir->number_off_files; i++) {
-    if (cur_dir->dir_entry[i].name[0] != '.') {
-      name_end_tmp = strmake(name_end, cur_dir->dir_entry[i].name,
-                             FN_REFLEN - (name_end - fullname));
-
-      if (MY_S_ISDIR(cur_dir->dir_entry[i].mystat->st_mode)) {
-        if (scan_tz_dir(name_end_tmp)) {
-          my_dirend(cur_dir);
-          return true;
-        }
-      } else if (MY_S_ISREG(cur_dir->dir_entry[i].mystat->st_mode)) {
-        init_alloc_root(PSI_NOT_INSTRUMENTED, &tz_storage, 32768, 0);
-        if (!tz_load(fullname, &tz_info, &tz_storage))
-          print_tz_as_sql(root_name_end + 1, &tz_info);
-        else
-          fprintf(stderr,
-                  "Warning: Unable to load '%s' as time zone. Skipping it.\n",
-                  fullname);
-        free_root(&tz_storage, MYF(0));
-      } else
-        fprintf(stderr, "Warning: '%s' is not regular file or directory\n",
-                fullname);
-    }
-  }
-
-  my_dirend(cur_dir);
-
-  return false;
-}
-
-int main(int argc, char **argv) {
-  MY_INIT(argv[0]);
-
-  if (argc != 2 && argc != 3) {
-    print_version();
-    puts(ORACLE_WELCOME_COPYRIGHT_NOTICE("2004"));
-    fprintf(stderr, "Usage:\n");
-    fprintf(stderr, " %s timezonedir\n", argv[0]);
-    fprintf(stderr, " %s timezonefile timezonename\n", argv[0]);
-    fprintf(stderr, " %s --leap timezonefile\n", argv[0]);
-    return 1;
-  }
-
-  if (argc == 2) {
-    root_name_end = strmake(fullname, argv[1], FN_REFLEN);
-
-    printf("TRUNCATE TABLE time_zone;\n");
-    printf("TRUNCATE TABLE time_zone_name;\n");
-    printf("TRUNCATE TABLE time_zone_transition;\n");
-    printf("TRUNCATE TABLE time_zone_transition_type;\n");
-
-    printf("START TRANSACTION;\n");
-    if (scan_tz_dir(root_name_end)) {
-      fprintf(stderr,
-              "There were fatal errors during processing "
-              "of zoneinfo directory\n");
-      return 1;
-    }
-    printf("COMMIT;\n");
-  } else {
-    init_alloc_root(PSI_NOT_INSTRUMENTED, &tz_storage, 32768, 0);
-
-    if (strcmp(argv[1], "--leap") == 0) {
-      if (tz_load(argv[2], &tz_info, &tz_storage)) {
-        fprintf(stderr, "Problems with zoneinfo file '%s'\n", argv[2]);
-        return 1;
-      }
-      print_tz_leaps_as_sql(&tz_info);
-    } else {
-      if (tz_load(argv[1], &tz_info, &tz_storage)) {
-        fprintf(stderr, "Problems with zoneinfo file '%s'\n", argv[2]);
-        return 1;
-      }
-      printf("START TRANSACTION;\n");
-      print_tz_as_sql(argv[2], &tz_info);
-      printf("COMMIT;\n");
-    }
-
-    free_root(&tz_storage, MYF(0));
-  }
-
-  return 0;
-}
-
-#endif /* defined(TZINFO2SQL) */
