@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+Copyright (c) 2008, 2018, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -54,17 +53,17 @@ dberr_t ib_trx_lock_table_with_retry(
 
   return (lock_table_for_trx(table, trx, mode));
 }
-/** Handles user errors and lock waits detected by the database engine.
-@param[out]     new_err possible new error encountered in lock wait, or if no
+/* Handles user errors and lock waits detected by the database engine.
+@param[out]	new_err	possible new error encountered in lock wait, or if no
 new error, the value of trx->error_state at the entry of this function
-@param[in]      trx     transaction
-@param[in]      thr     query thread
-@param[in]      savept  savepoint or NULL
-@param[in]      is_sdi  true if table is SDI
-@return true if it was a lock wait and we should continue running
+@param[in]	trx	transaction
+@param[in]	thr	query thread
+@param[in]	savept	savepoint or NULL
+@param[in]	is_sdi	true if table is SDI
+@return TRUE if it was a lock wait and we should continue running
 the query thread. */
-bool ib_handle_errors(dberr_t *new_err, trx_t *trx, que_thr_t *thr,
-                      trx_savept_t *savept, bool is_sdi) {
+ibool ib_handle_errors(dberr_t *new_err, trx_t *trx, que_thr_t *thr,
+                       trx_savept_t *savept, bool is_sdi) {
   dberr_t err;
 handle_new_error:
   err = trx->error_state;
@@ -79,7 +78,7 @@ handle_new_error:
         trx_rollback_for_mysql(trx);
       }
       break;
-      [[fallthrough]];
+      /* fall through */
     case DB_DUPLICATE_KEY:
     case DB_FOREIGN_DUPLICATE_KEY:
     case DB_TOO_BIG_RECORD:
@@ -106,7 +105,7 @@ handle_new_error:
 
       *new_err = err;
 
-      return true; /* Operation needs to be retried. */
+      return (TRUE); /* Operation needs to be retried. */
 
     case DB_DEADLOCK:
     case DB_LOCK_TABLE_FULL:
@@ -135,5 +134,5 @@ handle_new_error:
 
   trx->error_state = DB_SUCCESS;
 
-  return false;
+  return (FALSE);
 }
