@@ -58,10 +58,12 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "ut0vec.h"
 
 /** Build a table definition without updating SYSTEM TABLES
-@param[in,out]	table	dict table object
-@param[in,out]	trx	transaction instance
+@param[in,out]	table		dict table object
+@param[in]	create_info	HA_CREATE_INFO object
+@param[in,out]	trx		transaction instance
 @return DB_SUCCESS or error code */
-dberr_t dict_build_table_def(dict_table_t *table, trx_t *trx) {
+dberr_t dict_build_table_def(dict_table_t *table,
+                             const HA_CREATE_INFO *create_info, trx_t *trx) {
   std::string db_name;
   std::string tbl_name;
   dict_name::get_table(table->name.m_name, db_name, tbl_name);
@@ -84,12 +86,12 @@ dberr_t dict_build_table_def(dict_table_t *table, trx_t *trx) {
     dict_table_assign_new_id(table, trx);
   }
 
-  dberr_t err = dict_build_tablespace_for_table(table, trx);
+  dberr_t err = dict_build_tablespace_for_table(table, create_info, trx);
 
   return (err);
 }
 
-/** Build a tablespace to store various objects.
+/** Builds a tablespace to store various objects.
 @param[in,out]	trx		DD transaction
 @param[in,out]	tablespace	Tablespace object describing what to build.
 @return DB_SUCCESS or error code. */
@@ -137,8 +139,15 @@ dberr_t dict_build_tablespace(trx_t *trx, Tablespace *tablespace) {
   - page 3 will contain the root of the clustered index of the
   first table we create here. */
 
+  /* Set the initial size of the file being created. */
+  page_no_t size{};
+
+  size = tablespace->get_autoextend_size() > 0
+             ? (tablespace->get_autoextend_size() / srv_page_size)
+             : FIL_IBD_FILE_INITIAL_SIZE;
+
   err = fil_ibd_create(space, tablespace->name(), datafile->filepath(),
-                       tablespace->flags(), FIL_IBD_FILE_INITIAL_SIZE);
+                       tablespace->flags(), size);
 
   DBUG_INJECT_CRASH("ddl_crash_after_create_tablespace",
                     crash_injection_after_create_counter++);
@@ -156,7 +165,7 @@ dberr_t dict_build_tablespace(trx_t *trx, Tablespace *tablespace) {
   mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO); */
   ut_a(!FSP_FLAGS_GET_TEMPORARY(tablespace->flags()));
 
-  bool ret = fsp_header_init(space, FIL_IBD_FILE_INITIAL_SIZE, &mtr, false);
+  bool ret = fsp_header_init(space, size, &mtr, false);
   mtr_commit(&mtr);
 
   DBUG_EXECUTE_IF("fil_ibd_create_log",
@@ -171,10 +180,13 @@ dberr_t dict_build_tablespace(trx_t *trx, Tablespace *tablespace) {
 }
 
 /** Builds a tablespace to contain a table, using file-per-table=1.
-@param[in,out]	table	Table to build in its own tablespace.
-@param[in,out]	trx	Transaction
+@param[in,out]	table		Table to build in its own tablespace.
+@param[in]	create_info	HA_CREATE_INFO object
+@param[in,out]	trx		Transaction
 @return DB_SUCCESS or error code */
-dberr_t dict_build_tablespace_for_table(dict_table_t *table, trx_t *trx) {
+dberr_t dict_build_tablespace_for_table(dict_table_t *table,
+                                        const HA_CREATE_INFO *create_info,
+                                        trx_t *trx) {
   dberr_t err = DB_SUCCESS;
   mtr_t mtr;
   space_id_t space = 0;
@@ -256,8 +268,13 @@ dberr_t dict_build_tablespace_for_table(dict_table_t *table, trx_t *trx) {
     std::string tablespace_name(table->name.m_name);
     dict_name::convert_to_space(tablespace_name);
 
+    page_no_t size{};
+    size = create_info && create_info->m_implicit_tablespace_autoextend_size > 0
+               ? (create_info->m_implicit_tablespace_autoextend_size /
+                  srv_page_size)
+               : FIL_IBD_FILE_INITIAL_SIZE;
     err = fil_ibd_create(space, tablespace_name.c_str(), filepath, fsp_flags,
-                         FIL_IBD_FILE_INITIAL_SIZE);
+                         size);
 
     ut_free(filepath);
 
@@ -270,8 +287,15 @@ dberr_t dict_build_tablespace_for_table(dict_table_t *table, trx_t *trx) {
 
     mtr_start(&mtr);
 
-    bool ret =
-        fsp_header_init(table->space, FIL_IBD_FILE_INITIAL_SIZE, &mtr, false);
+    bool ret = fsp_header_init(table->space, size, &mtr, false);
+
+    if (ret) {
+      fil_set_autoextend_size(
+          table->space,
+          (create_info ? create_info->m_implicit_tablespace_autoextend_size
+                       : 0));
+    }
+
     mtr_commit(&mtr);
 
     DBUG_EXECUTE_IF("fil_ibd_create_log",
@@ -679,14 +703,15 @@ dict_index_t *dict_sdi_create_idx_in_mem(space_id_t space, bool space_discarded,
       dict_mem_table_create(table_name, space, 5, 0, 0, table_flags, 0);
 
   dict_mem_table_add_col(table, heap, "type", DATA_INT,
-                         DATA_NOT_NULL | DATA_UNSIGNED, 4);
+                         DATA_NOT_NULL | DATA_UNSIGNED, 4, true);
   dict_mem_table_add_col(table, heap, "id", DATA_INT,
-                         DATA_NOT_NULL | DATA_UNSIGNED, 8);
+                         DATA_NOT_NULL | DATA_UNSIGNED, 8, true);
   dict_mem_table_add_col(table, heap, "compressed_len", DATA_INT,
-                         DATA_NOT_NULL | DATA_UNSIGNED, 4);
+                         DATA_NOT_NULL | DATA_UNSIGNED, 4, true);
   dict_mem_table_add_col(table, heap, "uncompressed_len", DATA_INT,
-                         DATA_NOT_NULL | DATA_UNSIGNED, 4);
-  dict_mem_table_add_col(table, heap, "data", DATA_BLOB, DATA_NOT_NULL, 0);
+                         DATA_NOT_NULL | DATA_UNSIGNED, 4, true);
+  dict_mem_table_add_col(table, heap, "data", DATA_BLOB, DATA_NOT_NULL, 0,
+                         true);
 
   table->id = dict_sdi_get_table_id(space);
 
