@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2019, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2013, 2020, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -390,7 +390,7 @@ dberr_t SysTablespace::check_size(Datafile &file) {
 }
 
 /** Set the size of the file.
-@param[in]	file	data file object
+@param[in,out]	file	data file object
 @return DB_SUCCESS or error code */
 dberr_t SysTablespace::set_size(Datafile &file) {
   ut_a(!srv_read_only_mode || m_ignore_read_only);
@@ -423,7 +423,7 @@ dberr_t SysTablespace::set_size(Datafile &file) {
 }
 
 /** Create a data file.
-@param[in]	file	data file object
+@param[in,out]	file	data file object
 @return DB_SUCCESS or error code */
 dberr_t SysTablespace::create_file(Datafile &file) {
   dberr_t err = DB_SUCCESS;
@@ -460,7 +460,7 @@ dberr_t SysTablespace::create_file(Datafile &file) {
 }
 
 /** Open a data file.
-@param[in]	file	data file object
+@param[in,out]	file	data file object
 @return DB_SUCCESS or error code */
 dberr_t SysTablespace::open_file(Datafile &file) {
   dberr_t err = DB_SUCCESS;
@@ -584,7 +584,7 @@ dberr_t SysTablespace::read_lsn_and_check_flags(lsn_t *flushed_lsn) {
 }
 
 /** Check if a file can be opened in the correct mode.
-@param[in]	file	data file object
+@param[in,out]	file	data file object
 @param[out]	reason	exact reason if file_status check failed.
 @return DB_SUCCESS or error code. */
 dberr_t SysTablespace::check_file_status(const Datafile &file,
@@ -718,8 +718,8 @@ void SysTablespace::file_found(Datafile &file) {
 }
 
 /** Check the data file specification.
-@param[out] create_new_db	true if a new database is to be created
-@param[in] min_expected_size	Minimum expected tablespace size in bytes
+@param[in]  create_new_db     True if a new database is to be created
+@param[in]  min_expected_size Minimum expected tablespace size in bytes
 @return DB_SUCCESS if all OK else error code */
 dberr_t SysTablespace::check_file_spec(bool create_new_db,
                                        ulint min_expected_size) {
@@ -772,7 +772,7 @@ dberr_t SysTablespace::check_file_spec(bool create_new_db,
       ut_a(err != DB_FAIL);
       break;
 
-    } else if (create_new_db) {
+    } else if (create_new_db && !(*it).is_raw_type()) {
       ib::error(ER_IB_MSG_454)
           << "The " << name() << " data file '" << begin->m_name
           << "' was not found but"
@@ -783,6 +783,7 @@ dberr_t SysTablespace::check_file_spec(bool create_new_db,
       break;
 
     } else {
+      ut_ad(err == DB_SUCCESS);
       file_found(*it);
     }
   }
