@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2007, 2024, Oracle and/or its affiliates.
+Copyright (c) 2007, 2020, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -55,8 +54,7 @@ enum fts_ast_visit_pass_t {
 static fts_ast_node_t *fts_ast_node_create(void) {
   fts_ast_node_t *node;
 
-  node = (fts_ast_node_t *)ut::zalloc_withkey(UT_NEW_THIS_FILE_PSI_KEY,
-                                              sizeof(*node));
+  node = (fts_ast_node_t *)ut_zalloc_nokey(sizeof(*node));
 
   return (node);
 }
@@ -324,7 +322,7 @@ fts_ast_node_t *fts_ast_free_node(
   /*!< Get next node before freeing the node itself */
   next_node = node->next;
 
-  ut::free(node);
+  ut_free(node);
 
   return (next_node);
 }
@@ -375,7 +373,7 @@ void fts_ast_term_set_wildcard(fts_ast_node_t *node) /*!< in/out: set attribute
   ut_a(node->type == FTS_AST_TERM);
   ut_a(!node->term.wildcard);
 
-  node->term.wildcard = true;
+  node->term.wildcard = TRUE;
 }
 
 /** Set the proximity attribute of a text node. */
@@ -410,7 +408,7 @@ void fts_ast_state_free(fts_ast_state_t *state) /*!< in: ast state to free */
       node->term.ptr = nullptr;
     }
 
-    ut::free(node);
+    ut_free(node);
     node = next;
   }
 
@@ -418,7 +416,7 @@ void fts_ast_state_free(fts_ast_state_t *state) /*!< in: ast state to free */
 }
 
 /** Print the ast string
-@param[in]      ast_str string to print */
+@param[in]	ast_str	string to print */
 static void fts_ast_string_print(const fts_ast_string_t *ast_str) {
   for (ulint i = 0; i < ast_str->len; ++i) {
     printf("%c", ast_str->str[i]);
@@ -488,7 +486,7 @@ void fts_ast_node_print(fts_ast_node_t *node) /*!< in: ast node to print */
 }
 
 /** Check only union operation involved in the node
-@param[in]      node    ast node to check
+@param[in]	node	ast node to check
 @return true if the node contains only union else false. */
 bool fts_ast_node_check_union(fts_ast_node_t *node) {
   if (node->type == FTS_AST_LIST || node->type == FTS_AST_SUBEXP_LIST) {
@@ -609,7 +607,7 @@ dberr_t fts_ast_visit(fts_ast_oper_t oper,      /*!< in: current operator */
           continue;
         }
 
-        /* Process leaf node according to its pass.*/
+        /* Process leaf node accroding to its pass.*/
         if (oper == FTS_EXIST_SKIP && visit_pass == FTS_PASS_EXIST) {
           error = visitor(FTS_EXIST, node, arg);
           node->visited = true;
@@ -650,8 +648,8 @@ dberr_t fts_ast_visit(fts_ast_oper_t oper,      /*!< in: current operator */
 /**
 Create an ast string object, with NUL-terminator, so the string
 has one more byte than len
-@param[in] str          pointer to string
-@param[in] len          length of the string
+@param[in] str		pointer to string
+@param[in] len		length of the string
 @return ast string with NUL-terminator */
 fts_ast_string_t *fts_ast_string_create(const byte *str, ulint len) {
   fts_ast_string_t *ast_str;
@@ -659,10 +657,9 @@ fts_ast_string_t *fts_ast_string_create(const byte *str, ulint len) {
   ut_ad(len > 0);
 
   ast_str = static_cast<fts_ast_string_t *>(
-      ut::malloc_withkey(UT_NEW_THIS_FILE_PSI_KEY, sizeof(fts_ast_string_t)));
+      ut_malloc_nokey(sizeof(fts_ast_string_t)));
 
-  ast_str->str = static_cast<byte *>(
-      ut::malloc_withkey(UT_NEW_THIS_FILE_PSI_KEY, len + 1));
+  ast_str->str = static_cast<byte *>(ut_malloc_nokey(len + 1));
 
   ast_str->len = len;
   memcpy(ast_str->str, str, len);
@@ -673,18 +670,18 @@ fts_ast_string_t *fts_ast_string_create(const byte *str, ulint len) {
 
 /**
 Free an ast string instance
-@param[in,out] ast_str          string to free */
+@param[in,out] ast_str		string to free */
 void fts_ast_string_free(fts_ast_string_t *ast_str) {
   if (ast_str != nullptr) {
-    ut::free(ast_str->str);
-    ut::free(ast_str);
+    ut_free(ast_str->str);
+    ut_free(ast_str);
   }
 }
 
 /**
 Translate ast string of type FTS_AST_NUMB to unsigned long by strtoul
-@param[in]      ast_str string to translate
-@param[in]      base    the base
+@param[in]	ast_str	string to translate
+@param[in]	base	the base
 @return translated number */
 ulint fts_ast_string_to_ul(const fts_ast_string_t *ast_str, int base) {
   return (strtoul(reinterpret_cast<const char *>(ast_str->str), nullptr, base));
@@ -708,7 +705,7 @@ const char *fts_ast_node_type_get(fts_ast_type_t type) {
     case FTS_AST_PARSER_PHRASE_LIST:
       return ("FTS_AST_PARSER_PHRASE_LIST");
   }
-  ut_d(ut_error);
-  ut_o(return ("FTS_UNKNOWN"));
+  ut_ad(0);
+  return ("FTS_UNKNOWN");
 }
 #endif /* UNIV_DEBUG */
