@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2019, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2013, 2020, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -56,6 +56,7 @@ class Tablespace {
         m_space_id(SPACE_UNKNOWN),
         m_path(),
         m_flags(),
+        m_autoextend_size(),
         m_ignore_read_only(false) {
     /* No op */
   }
@@ -178,6 +179,12 @@ class Tablespace {
     return (&m_files.front());
   }
 
+  /* Set the autoextend size for the tablespace */
+  void set_autoextend_size(uint64_t size) { m_autoextend_size = size; }
+
+  /* Get the autoextend size for the tablespace */
+  uint64_t get_autoextend_size() const { return m_autoextend_size; }
+
  private:
   /**
   @param[in]	filename	Name to lookup in the data files.
@@ -185,7 +192,7 @@ class Tablespace {
   bool find(const char *filename);
 
   /** Note that the data file was found.
-  @param[in]	file	data file object */
+  @param[in,out] file	Data file object to set */
   void file_found(Datafile &file);
 
   /* DATA MEMBERS */
@@ -202,6 +209,9 @@ class Tablespace {
   /** Tablespace flags */
   uint32_t m_flags;
 
+  /** Autoextend size */
+  uint64_t m_autoextend_size;
+
  protected:
   /** Ignore server read only configuration for this tablespace. */
   bool m_ignore_read_only;
