@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2013, 2020, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -65,7 +65,7 @@ class SysTablespace : public Tablespace {
     /* No op */
   }
 
-  ~SysTablespace() { shutdown(); }
+  ~SysTablespace() override { shutdown(); }
 
   /** Set tablespace full status
   @param[in]	is_full		true if full */
@@ -86,18 +86,16 @@ class SysTablespace : public Tablespace {
   bool get_sanity_check_status() { return (m_sanity_checks_done); }
 
   /** Parse the input params and populate member variables.
-  @param	filepath_spec	path to data files
-  @param	supports_raw	true if it supports raw devices
-  @param	filenames_only	true to parse only file names
+  @param[in]	filepath_spec	path to data files
+  @param[in]	supports_raw	true if the tablespace supports raw devices
+  @param[in]	filenames_only	true to parse only file names
   @return true on success parse */
   bool parse_params(const char *filepath_spec, bool supports_raw,
                     bool filenames_only);
 
   /** Check the data file specification.
-  @param[in]	create_new_db		true if a new database
-  is to be created
-  @param[in]	min_expected_size	expected tablespace
-  size in bytes
+  @param[in]  create_new_db     True if a new database is to be created
+  @param[in]  min_expected_size Minimum expected tablespace size in bytes
   @return DB_SUCCESS if all OK else error code */
   dberr_t check_file_spec(bool create_new_db, ulint min_expected_size);
 
