@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2019, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 1995, 2020, Oracle and/or its affiliates. All rights reserved.
 Copyright (c) 2009, Google Inc.
 
 Portions of this file contain modifications contributed and copyrighted by
@@ -52,6 +52,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "log0test.h"
 #include "log0types.h"
+#include "my_compiler.h"
 
 /** Prefix for name of log file, e.g. "ib_logfile" */
 constexpr const char *const ib_logfile_basename = "ib_logfile";
@@ -68,7 +69,7 @@ constexpr uint32_t LOG_CHECKPOINT_EXTRA_FREE = 8;
 
 /** Per thread margin for the free space in the log, before a new query step
 which modifies the database, is started. It's multiplied by maximum number
-of threads, that can concurrently enter mini transactions. Expressed in
+of threads, that can concurrently enter mini-transactions. Expressed in
 number of pages. */
 constexpr uint32_t LOG_CHECKPOINT_FREE_PER_THREAD = 4;
 
@@ -149,6 +150,9 @@ static_assert(LOG_BLOCK_HDR_SIZE + LOG_BLOCK_TRL_SIZE < LOG_BLOCK_DATA_SIZE,
 /** Maximum possible sn value. */
 constexpr sn_t SN_MAX = (1ULL << 62) - 1;
 
+/** The sn bit to express locked state. */
+constexpr sn_t SN_LOCKED = 1ULL << 63;
+
 /** Maximum possible lsn value is slightly higher than the maximum sn value,
 because lsn sequence enumerates also bytes used for headers and footers of
 all log blocks. However, still 64-bits are enough to represent the maximum
@@ -204,6 +208,40 @@ constexpr uint32_t LOG_HEADER_CREATOR = 16;
 /** End of the log file creator field. */
 constexpr uint32_t LOG_HEADER_CREATOR_END = LOG_HEADER_CREATOR + 32;
 
+/** 32 BITs flag */
+constexpr uint32_t LOG_HEADER_FLAGS = LOG_HEADER_CREATOR_END;
+
+/** Flag at BIT-1 to indicate if redo logging is disabled or not. */
+constexpr uint32_t LOG_HEADER_FLAG_NO_LOGGING = 1;
+
+/** Flag at BIT-2 to indicate if server is not recoverable on crash. This
+is set only when redo logging is disabled and unset on slow shutdown after
+all pages are flushed to disk. */
+constexpr uint32_t LOG_HEADER_FLAG_CRASH_UNSAFE = 2;
+
+/** Maximum BIT position number. Should be set to the latest added. */
+constexpr uint32_t LOG_HEADER_FLAG_MAX = LOG_HEADER_FLAG_CRASH_UNSAFE;
+
+/** Current total size of LOG header. */
+constexpr uint32_t LOG_HEADER_SIZE = LOG_HEADER_FLAGS + 4;
+
+/** Set a specific bit in flag.
+@param[in]      flag    bit flag
+@param[in]      bit     set bit */
+inline void LOG_HEADER_SET_FLAG(uint32_t &flag, uint32_t bit) {
+  ut_ad(bit > 0);
+  ut_ad(bit <= LOG_HEADER_FLAG_MAX);
+  flag |= static_cast<uint32_t>(1UL << (bit - 1));
+}
+
+/** Check a specific bit in flag.
+@param[in]      flag    bit flag
+@param[in]      bit     check bit
+@return true, iff bit is set in flag. */
+inline bool LOG_HEADER_CHECK_FLAG(uint32_t flag, uint32_t bit) {
+  return ((flag & (1ULL << (bit - 1))) > 0);
+}
+
 /** Contents of the LOG_HEADER_CREATOR field */
 #define LOG_HEADER_CREATOR_CURRENT "MySQL " INNODB_VERSION_STR
 #define LOG_HEADER_CREATOR_8018  "MySQL 8.0.18" // (v3 format)
@@ -299,12 +337,6 @@ constexpr ulong INNODB_LOG_FLUSH_NOTIFIER_SPIN_DELAY_DEFAULT = 0;
 /** Default value of innodb_log_flush_notifier_timeout (in microseconds). */
 constexpr ulong INNODB_LOG_FLUSH_NOTIFIER_TIMEOUT_DEFAULT = 10;
 
-/** Default value of innodb_log_closer_spin_delay (in spin rounds). */
-constexpr ulong INNODB_LOG_CLOSER_SPIN_DELAY_DEFAULT = 0;
-
-/** Default value of innodb_log_closer_timeout (in microseconds). */
-constexpr ulong INNODB_LOG_CLOSER_TIMEOUT_DEFAULT = 1000;
-
 /** Default value of innodb_log_buffer_size (in bytes). */
 constexpr ulong INNODB_LOG_BUFFER_SIZE_DEFAULT = 16 * 1024 * 1024UL;
 
@@ -586,8 +618,14 @@ NOTE that the link is added after data is written to the reserved
 space in the log buffer. It is very critical to do all these steps
 as fast as possible, because very likely the log writer thread is
 waiting for the link.
-
+*/
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_buf_reserve
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in,out]	log	redo log
 @param[in]	len	number of data bytes to reserve for write
 @return handle that represents the reservation */
@@ -602,7 +640,14 @@ overflow it. If it does not cover, then returned value should be used
 to start the next write operation. Note that finally we must use exactly
 all the reserved space.
 
+*/
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_buf_write
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in,out]	log		redo log
 @param[in]	handle		handle for the reservation of space
 @param[in]	str		memory to write data from
@@ -624,7 +669,14 @@ After the link is added, the log writer may write the data to disk.
 NOTE that still dirty pages for the [start_lsn, end_lsn) are not added
 to flush lists when this function is called.
 
+*/
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_buf_add_links_to_recent_written
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in,out]	log		redo log
 @param[in]	handle		handle for the reservation of space
 @param[in]	start_lsn	start_lsn of the link to add
@@ -658,7 +710,15 @@ This is called after all dirty pages related to [start_lsn, end_lsn)
 have been added to corresponding flush lists.
 For detailed explanation - @see log0write.cc.
 
+*/
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_add_link_to_recent_closed
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
+
 @param[in,out]	log		redo log
 @param[in]	handle		handle for the reservation of space */
 void log_buffer_close(log_t &log, const Log_handle &handle);
@@ -669,6 +729,11 @@ void log_buffer_close(log_t &log, const Log_handle &handle);
 also to be flushed to disk. */
 void log_buffer_flush_to_disk(log_t &log, bool sync = true);
 
+/** Writes the log buffer to the log file. It is intended to be called from
+background master thread periodically. If the log writer threads are active,
+this function writes nothing. */
+void log_buffer_sync_in_background();
+
 /** Requests flush of the log buffer.
 @param[in]	sync	true: wait until the flush is done */
 inline void log_buffer_flush_to_disk(bool sync = true);
@@ -694,15 +759,8 @@ void log_buffer_get_last_block(log_t &log, lsn_t &last_lsn, byte *last_block,
 
 /** Advances log.buf_ready_for_write_lsn using links in the recent written
 buffer. It's used by the log writer thread only.
-@param[in]	log	redo log
-@return true if and only if the lsn has been advanced */
-bool log_advance_ready_for_write_lsn(log_t &log);
-
-/** Advances log.buf_dirty_pages_added_up_to_lsn using links in the recent
-closed buffer. It's used by the log closer thread only.
-@param[in]	log	redo log
-@return true if and only if the lsn has been advanced */
-bool log_advance_dirty_pages_added_up_to_lsn(log_t &log);
+@param[in,out]	log	redo log */
+void log_advance_ready_for_write_lsn(log_t &log);
 
 /** Validates that all slots in log recent written buffer for lsn values
 in range between begin and end, are empty. Used during tests, crashes the
@@ -732,14 +790,28 @@ are added to flush lists. That's because we need to guarantee,
 that the delay until dirty page is added to flush list is limited.
 For detailed explanation - @see log0write.cc.
 
+*/
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_add_dirty_pages
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in,out]	log   redo log
 @param[in]      lsn   lsn on which we wait (for any link: lsn -> x) */
 void log_wait_for_space_in_log_recent_closed(log_t &log, lsn_t lsn);
 
 /** Waits until there is free space in the log buffer. The free space has to be
 available for range of sn values ending at the provided sn.
+*/
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_waiting_for_writer
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in]     log     redo log
 @param[in]     end_sn  end of the range of sn values */
 void log_wait_for_space_in_log_buf(log_t &log, sn_t end_sn);
@@ -765,7 +837,14 @@ lsn_t log_get_max_modified_age_async(const log_t &log);
 /** Waits until there is free space in log files which includes
 concurrency margin required for all threads. You should rather
 use log_free_check().
+*/
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_reclaim_space
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in]     log   redo log */
 void log_free_check_wait(log_t &log);
 
@@ -816,6 +895,13 @@ bool log_rotate_encryption();
 /** Rotate default master key for redo log encryption. */
 void redo_rotate_default_master_key();
 
+/** Computes lsn up to which sync flush should be done or returns 0
+if there is no need to execute sync flush now.
+@param[in,out]  log  redo log
+@return lsn for which we want to have oldest_lsn >= lsn in each BP,
+        or 0 if there is no need for sync flush */
+lsn_t log_sync_flush_lsn(log_t &log);
+
 /** Requests a sharp checkpoint write for provided or greater lsn.
 @param[in,out]	log	redo log
 @param[in]	sync	true -> wait until it is finished
@@ -849,8 +935,11 @@ void log_files_header_read(log_t &log, uint32_t header);
 /** Fill redo log header.
 @param[out]	buf		filled buffer
 @param[in]	start_lsn	log start LSN
-@param[in]	creator		creator of the header */
-void log_files_header_fill(byte *buf, lsn_t start_lsn, const char *creator);
+@param[in]	creator		creator of the header
+@param[in]	no_logging	redo logging is disabled
+@param[in]	crash_unsafe	it is not safe to crash */
+void log_files_header_fill(byte *buf, lsn_t start_lsn, const char *creator,
+                           bool no_logging, bool crash_unsafe);
 
 /** Writes a log file header to the log file space.
 @param[in]	log		redo log
@@ -879,16 +968,6 @@ initialized to correspond to some lsn, for instance, a checkpoint lsn.
 @param[in]	lsn	log sequence number to set files_start_lsn at */
 void log_files_update_offsets(log_t &log, lsn_t lsn);
 
-/** Acquires the log buffer s-lock.
-@param[in,out]	log	redo log
-@return lock no, must be passed to s_lock_exit() */
-size_t log_buffer_s_lock_enter(log_t &log);
-
-/** Releases the log buffer s-lock.
-@param[in,out]	log	redo log
-@param[in]	lock_no	lock no received from s_lock_enter() */
-void log_buffer_s_lock_exit(log_t &log, size_t lock_no);
-
 /** Acquires the log buffer x-lock.
 @param[in,out]	log	redo log */
 void log_buffer_x_lock_enter(log_t &log);
@@ -1015,11 +1094,6 @@ Used only to assert, that the state is correct.
 @param[in]	log	redo log */
 void log_writer_thread_active_validate(const log_t &log);
 
-/** Validates that the log closer thread is active.
-Used only to assert, that the state is correct.
-@param[in]	log	redo log */
-void log_closer_thread_active_validate(const log_t &log);
-
 /** Validates that the log writer, flusher threads are active.
 Used only to assert, that the state is correct.
 @param[in]	log	redo log */
@@ -1055,41 +1129,76 @@ void log_stop_background_threads_nowait(log_t &log);
 /** Wakes up all log threads which are alive. */
 void log_wake_threads(log_t &log);
 
+/** Pause/Resume the log writer threads based on innodb_log_writer_threads
+value.
+NOTE: These pause/resume functions should be protected by mutex while serving.
+The caller innodb_log_writer_threads_update() is protected
+by LOCK_global_system_variables in mysqld. */
+void log_control_writer_threads(log_t &log);
+
 /** Free the log system data structures. Deallocate all the related memory. */
 void log_sys_close();
 
 /** The log writer thread co-routine.
+ */
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_writer
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in,out]	log_ptr		pointer to redo log */
 void log_writer(log_t *log_ptr);
 
 /** The log flusher thread co-routine.
+ */
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_flusher
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in,out]	log_ptr		pointer to redo log */
 void log_flusher(log_t *log_ptr);
 
 /** The log flush notifier thread co-routine.
+ */
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_flush_notifier
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in,out]	log_ptr		pointer to redo log */
 void log_flush_notifier(log_t *log_ptr);
 
 /** The log write notifier thread co-routine.
+ */
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_write_notifier
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in,out]	log_ptr		pointer to redo log */
 void log_write_notifier(log_t *log_ptr);
 
-/** The log closer thread co-routine.
-@see @ref sect_redo_log_closer
-@param[in,out]	log_ptr		pointer to redo log */
-void log_closer(log_t *log_ptr);
-
 /** The log checkpointer thread co-routine.
+ */
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_WORKAROUND_REF_DOCBUG()
+/**
 @see @ref sect_redo_log_checkpointer
+*/
+MY_COMPILER_DIAGNOSTIC_POP()
+/**
 @param[in,out]	log_ptr		pointer to redo log */
 void log_checkpointer(log_t *log_ptr);
 
-#define log_buffer_x_lock_own(log) log.sn_lock.x_own()
-
 #define log_checkpointer_mutex_enter(log) \
   mutex_enter(&((log).checkpointer_mutex))
 
@@ -1100,10 +1209,10 @@ void log_checkpointer(log_t *log_ptr);
 
 #define log_closer_mutex_enter(log) mutex_enter(&((log).closer_mutex))
 
-#define log_closer_mutex_exit(log) mutex_exit(&((log).closer_mutex))
+#define log_closer_mutex_enter_nowait(log) \
+  mutex_enter_nowait(&((log).closer_mutex))
 
-#define log_closer_mutex_own(log) \
-  (mutex_own(&((log).closer_mutex)) || !log_closer_is_active())
+#define log_closer_mutex_exit(log) mutex_exit(&((log).closer_mutex))
 
 #define log_flusher_mutex_enter(log) mutex_enter(&((log).flusher_mutex))
 
@@ -1190,10 +1299,6 @@ inline bool log_flusher_is_active();
 @return true if and only if the log flush notifier thread is active */
 inline bool log_flush_notifier_is_active();
 
-/** Checks if log closer thread is active.
-@return true if and only if the log closer thread is active */
-inline bool log_closer_is_active();
-
 /** Checks if log checkpointer thread is active.
 @return true if and only if the log checkpointer thread is active */
 inline bool log_checkpointer_is_active();
@@ -1207,6 +1312,18 @@ inline bool log_checkpointer_is_active();
 bool log_file_header_fill_encryption(byte *buf, byte *key, byte *iv,
                                      bool is_boot, bool encrypt_key);
 
+/** Disable redo logging and persist the information.
+@param[in,out]	log	redo log */
+void log_persist_disable(log_t &log);
+
+/** Enable redo logging and persist the information.
+@param[in,out]	log	redo log */
+void log_persist_enable(log_t &log);
+
+/** Persist the information that it is safe to restart server.
+@param[in,out]	log	redo log */
+void log_persist_crash_safe(log_t &log);
+
 #else /* !UNIV_HOTBACKUP */
 
 #ifdef UNIV_DEBUG
