@@ -1,6 +1,6 @@
 /***********************************************************************
 
-Copyright (c) 1995, 2020, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2020, Oracle and/or its affiliates.
 Copyright (c) 2009, Percona Inc.
 
 Portions of this file contain modifications contributed and copyrighted
@@ -72,9 +72,9 @@ struct fil_node_t;
 extern bool os_has_said_disk_full;
 
 /** Number of pending read operations */
-extern ulint os_n_pending_reads;
+extern std::atomic<ulint> os_n_pending_reads;
 /** Number of pending write operations */
-extern ulint os_n_pending_writes;
+extern std::atomic<ulint> os_n_pending_writes;
 
 /* Flush after each os_fsync_threshold bytes */
 extern unsigned long long os_fsync_threshold;
@@ -87,12 +87,20 @@ namespace file {
 and encryption code. */
 struct Block {
   /** Default constructor */
-  Block() : m_ptr(), m_in_use() {}
+  Block() noexcept : m_ptr(nullptr), m_in_use() {}
 
+  /** Free the given memory block.
+  @param[in]  obj  the memory block to be freed. */
+  static void free(file::Block *obj) noexcept;
+
+  /** Pointer to the memory block. */
   byte *m_ptr;
-
-  byte pad[INNOBASE_CACHE_LINE_SIZE - sizeof(ulint)];
-  lock_word_t m_in_use;
+  /** This padding is needed to avoid false sharing. TBD: of what exactly? We
+  can't use alignas because std::vector<Block> uses std::allocator which in
+  C++14 doesn't have to handle overaligned types. (see  20.7.9.1.5 of N4140
+  draft) */
+  byte pad[ut::INNODB_CACHE_LINE_SIZE];
+  std::atomic<bool> m_in_use;
 };
 }  // namespace file
 
@@ -206,7 +214,7 @@ static const ulint OS_FILE_READ_ALLOW_DELETE = 555;
 /* Options for file_create */
 static const ulint OS_FILE_AIO = 61;
 static const ulint OS_FILE_NORMAL = 62;
-/* @} */
+/** @} */
 
 /** Types for file create @{ */
 static const ulint OS_DATA_FILE = 100;
@@ -225,7 +233,7 @@ static const ulint OS_DBLWR_FILE = 105;
 
 /** Redo log archive file. */
 static const ulint OS_REDO_LOG_ARCHIVE_FILE = 105;
-/* @} */
+/** @} */
 
 /** Error codes from os_file_get_last_error @{ */
 static const ulint OS_FILE_NOT_FOUND = 71;
@@ -244,7 +252,7 @@ static const ulint OS_FILE_OPERATION_ABORTED = 80;
 static const ulint OS_FILE_ACCESS_VIOLATION = 81;
 static const ulint OS_FILE_NAME_TOO_LONG = 82;
 static const ulint OS_FILE_ERROR_MAX = 100;
-/* @} */
+/** @} */
 
 /** Types for AIO operations @{ */
 
@@ -298,7 +306,12 @@ class IORequest {
     This can be used to force a read and write without any
     compression e.g., for redo log, merge sort temporary files
     and the truncate redo log. */
-    NO_COMPRESSION = 512
+    NO_COMPRESSION = 512,
+
+    /** We optimise cases where punch hole is not done if the compressed length
+    of the page is the same as the original size of the page. Ignore such
+    optimisations if this flag is set. */
+    DISABLE_PUNCH_HOLE_OPTIMISATION = 1024
   };
 
   /** Default constructor */
@@ -306,7 +319,9 @@ class IORequest {
       : m_block_size(UNIV_SECTOR_SIZE),
         m_type(READ),
         m_compression(),
-        m_encryption() {
+        m_encryption(),
+        m_eblock(nullptr),
+        m_elen(0) {
     /* No op */
   }
 
@@ -317,7 +332,9 @@ class IORequest {
       : m_block_size(UNIV_SECTOR_SIZE),
         m_type(static_cast<uint16_t>(type)),
         m_compression(),
-        m_encryption() {
+        m_encryption(),
+        m_eblock(nullptr),
+        m_elen(0) {
     if (is_log()) {
       disable_compression();
     }
@@ -372,9 +389,19 @@ class IORequest {
     return ((m_type & PUNCH_HOLE) == PUNCH_HOLE);
   }
 
+  /** @return true if punch hole needs to be done always if it's supported and
+  if the page is to be compressed. */
+  bool is_punch_hole_optimisation_disabled() const
+      MY_ATTRIBUTE((warn_unused_result)) {
+    ut_ad(is_compressed() && punch_hole());
+
+    return (m_type & DISABLE_PUNCH_HOLE_OPTIMISATION) ==
+           DISABLE_PUNCH_HOLE_OPTIMISATION;
+  }
+
   /** @return true if the read should be validated */
   bool validate() const MY_ATTRIBUTE((warn_unused_result)) {
-    ut_a(is_read() ^ is_write());
+    ut_ad(is_read() ^ is_write());
 
     return (!is_read() || !punch_hole());
   }
@@ -386,6 +413,13 @@ class IORequest {
     }
   }
 
+  /** Set the force punch hole flag */
+  void disable_punch_hole_optimisation() {
+    if (is_punch_hole_supported()) {
+      m_type |= DISABLE_PUNCH_HOLE_OPTIMISATION;
+    }
+  }
+
   /** Clear the do not wake flag */
   void clear_do_not_wake() { m_type &= ~DO_NOT_WAKE; }
 
@@ -553,21 +587,47 @@ class IORequest {
     return (os.str());
   }
 
+  /** Get a reference to the underlying encryption information.
+  @return reference to the encryption information. */
+  Encryption &get_encryption_info() noexcept
+      MY_ATTRIBUTE((warn_unused_result)) {
+    return m_encryption;
+  }
+
+  /** Set the encrypted block to the given value.
+  @param[in]  eblock  the encrypted block. */
+  void set_encrypted_block(const file::Block *eblock) noexcept {
+    m_eblock = eblock;
+  }
+
+  /** Get the encrypted block.
+  @return the encrypted block. */
+  const file::Block *get_encrypted_block() const noexcept
+      MY_ATTRIBUTE((warn_unused_result)) {
+    return m_eblock;
+  }
+
  private:
   /* File system best block size */
-  uint32_t m_block_size;
+  uint32_t m_block_size{};
 
   /** Request type bit flags */
-  uint16_t m_type;
+  uint16_t m_type{};
 
   /** Compression algorithm */
-  Compression m_compression;
+  Compression m_compression{};
 
   /** Encryption algorithm */
-  Encryption m_encryption;
+  Encryption m_encryption{};
+
+  /** The encrypted block. */
+  const file::Block *m_eblock{};
+
+  /** The length of data in encrypted block. */
+  uint32_t m_elen{};
 };
 
-/* @} */
+/** @} */
 
 /** Sparse file size information. */
 struct os_file_size_t {
@@ -601,7 +661,7 @@ enum class AIO_mode : size_t {
   or write, causing a bottleneck for parallelism. */
   SYNC = 24
 };
-/* @} */
+/** @} */
 
 extern ulint os_n_file_reads;
 extern ulint os_n_file_writes;
@@ -668,22 +728,22 @@ struct os_file_stat_t {
 #ifndef UNIV_HOTBACKUP
 /** Create a temporary file. This function is like tmpfile(3), but
 the temporary file is created in the given parameter path. If the path
-is null then it will create the file in the mysql server configuration
+is NULL then it will create the file in the MySQL server configuration
 parameter (--tmpdir).
 @param[in]	path	location for creating temporary file
 @return temporary file handle, or NULL on error */
 FILE *os_file_create_tmpfile(const char *path);
 #endif /* !UNIV_HOTBACKUP */
 
-/**
-This function attempts to create a directory named pathname. The new directory
-gets default permissions. On Unix, the permissions are (0770 & ~umask). If the
-directory exists already, nothing is done and the call succeeds, unless the
-fail_if_exists arguments is true.
-
+/** This function attempts to create a directory named pathname. The new
+directory gets default permissions. On Unix the permissions are
+(0770 & ~umask). If the directory exists already, nothing is done and
+the call succeeds, unless the fail_if_exists arguments is true.
+If another error occurs, such as a permission error, this does not crash,
+but reports the error and returns false.
 @param[in]	pathname	directory name as null-terminated string
-@param[in]	fail_if_exists	if true, pre-existing directory is treated
-                                as an error.
+@param[in]	fail_if_exists	if true, pre-existing directory is treated as
+                                an error.
 @return true if call succeeds, false on error */
 bool os_file_create_directory(const char *pathname, bool fail_if_exists);
 
@@ -709,9 +769,9 @@ A simple function to open or create a file.
                                 string
 @param[in]	create_mode	create mode
 @param[in]	access_type	OS_FILE_READ_ONLY or OS_FILE_READ_WRITE
-@param[in]	read_only	if true read only mode checks are enforced
+@param[in]	read_only	if true, read only checks are enforced
 @param[out]	success		true if succeed, false if error
-@return own: handle to the file, not defined if error, error number
+@return handle to the file, not defined if error, error number
         can be retrieved with os_file_get_last_error */
 os_file_t os_file_create_simple_func(const char *name, ulint create_mode,
                                      ulint access_type, bool read_only,
@@ -784,11 +844,11 @@ file is closed before calling this function.
 @return true if success */
 bool os_file_rename_func(const char *oldpath, const char *newpath);
 
-/** NOTE! Use the corresponding macro os_file_close(), not directly this
-function!
+/** NOTE! Use the corresponding macro os_file_close(), not directly
+this function!
 Closes a file handle. In case of error, error number can be retrieved with
 os_file_get_last_error.
-@param[in]	file		own: handle to a file
+@param[in]	file		Handle to a file
 @return true if success */
 bool os_file_close_func(os_file_t file);
 
@@ -1302,15 +1362,16 @@ to original un-instrumented file I/O APIs */
                                            n, o)                               \
   os_file_read_no_error_handling_func(type, file_name, file, buf, offset, n, o)
 
-#define os_file_read_no_error_handling_int_fd(type, file_name, file, buf, \
-                                              offset, n, o)               \
-  os_file_read_no_error_handling_func(type, file_name, file, buf, offset, n, o)
+#define os_file_read_no_error_handling_int_fd(type, file_name, file, buf,     \
+                                              offset, n, o)                   \
+  os_file_read_no_error_handling_func(type, file_name, OS_FILE_FROM_FD(file), \
+                                      buf, offset, n, o)
 
 #define os_file_write_pfs(type, name, file, buf, offset, n) \
   os_file_write_func(type, name, file, buf, offset, n)
 
 #define os_file_write_int_fd(type, name, file, buf, offset, n) \
-  os_file_write_func(type, name, file, buf, offset, n)
+  os_file_write_func(type, name, OS_FILE_FROM_FD(file), buf, offset, n)
 
 #define os_file_flush_pfs(file) os_file_flush_func(file)
 
@@ -1387,14 +1448,14 @@ bool os_file_close_no_error_handling(os_file_t file);
 #endif /* UNIV_HOTBACKUP */
 
 /** Gets a file size.
-@param[in]	filename	handle to a file
-@return file size if OK, else set m_total_size to ~0 and m_alloc_size
-        to errno */
+@param[in]	filename	Full path to the filename to check
+@return file size if OK, else set m_total_size to ~0 and m_alloc_size to
+        errno */
 os_file_size_t os_file_get_size(const char *filename)
     MY_ATTRIBUTE((warn_unused_result));
 
 /** Gets a file size.
-@param[in]	file		handle to a file
+@param[in]	file		Handle to a file
 @return file size, or (os_offset_t) -1 on failure */
 os_offset_t os_file_get_size(pfs_os_file_t file)
     MY_ATTRIBUTE((warn_unused_result));
@@ -1431,11 +1492,12 @@ bool os_file_set_size(const char *name, pfs_os_file_t file, os_offset_t offset,
 @return true if success */
 bool os_file_set_eof(FILE *file); /*!< in: file to be truncated */
 
-/** Truncates a file to a specified size in bytes. Do nothing if the size
-preserved is smaller or equal than current size of file.
+/** Truncates a file to a specified size in bytes.
+Do nothing if the size to preserve is greater or equal to the current
+size of the file.
 @param[in]	pathname	file path
 @param[in]	file		file to be truncated
-@param[in]	size		size preserved in bytes
+@param[in]	size		size to preserve in bytes
 @return true if success */
 bool os_file_truncate(const char *pathname, pfs_os_file_t file,
                       os_offset_t size);
@@ -1463,16 +1525,16 @@ the OS error number + 100 is returned.
 @return error number, or OS error number + 100 */
 ulint os_file_get_last_error(bool report_all_errors);
 
-/** NOTE! Use the corresponding macro os_file_read(), not directly this
-function!
-Requests a synchronous read operation.
+/** NOTE! Use the corresponding macro os_file_read_first_page(), not directly
+this function!
+Requests a synchronous read operation of page 0 of IBD file.
 @param[in]	type		IO request context
 @param[in]  file_name file name
 @param[in]	file		Open file handle
 @param[out]	buf		buffer where to read
 @param[in]	offset		file offset where to read
 @param[in]	n		number of bytes to read
-@return DB_SUCCESS if request was successful */
+@return DB_SUCCESS if request was successful, DB_IO_ERROR on failure */
 dberr_t os_file_read_func(IORequest &type, const char *file_name,
                           os_file_t file, void *buf, os_offset_t offset,
                           ulint n) MY_ATTRIBUTE((warn_unused_result));
@@ -1485,12 +1547,12 @@ Requests a synchronous read operation of page 0 of IBD file
 @param[in]	file		Open file handle
 @param[out]	buf		buffer where to read
 @param[in]	n		number of bytes to read
-@return DB_SUCCESS if request was successful */
+@return DB_SUCCESS if request was successful, DB_IO_ERROR on failure */
 dberr_t os_file_read_first_page_func(IORequest &type, const char *file_name,
                                      os_file_t file, void *buf, ulint n)
     MY_ATTRIBUTE((warn_unused_result));
 
-/** copy data from one file to another file. Data is read/written
+/** Copy data from one file to another file. Data is read/written
 at current file offset.
 @param[in]	src_file	file handle to copy from
 @param[in]	src_offset	offset to copy from
@@ -1505,9 +1567,9 @@ dberr_t os_file_copy_func(os_file_t src_file, os_offset_t src_offset,
 /** Rewind file to its start, read at most size - 1 bytes from it to str, and
 NUL-terminate str. All errors are silently ignored. This function is
 mostly meant to be used with temporary files.
-@param[in,out]	file		file to read from
-@param[in,out]	str		buffer where to read
-@param[in]	size		size of buffer */
+@param[in,out]	file		File to read from
+@param[in,out]	str		Buffer where to read
+@param[in]	size		Size of buffer */
 void os_file_read_string(FILE *file, char *str, ulint size);
 
 /** NOTE! Use the corresponding macro os_file_read_no_error_handling(),
@@ -1541,13 +1603,20 @@ dberr_t os_file_write_func(IORequest &type, const char *name, os_file_t file,
                            const void *buf, os_offset_t offset, ulint n)
     MY_ATTRIBUTE((warn_unused_result));
 
-/** Check the existence and type of the given file.
-@param[in]	path		pathname of the file
-@param[out]	exists		true if file exists
-@param[out]	type		type of the file (if it exists)
+/** Check the existence and type of a given path.
+@param[in]   path    pathname of the file
+@param[out]  exists  true if file exists
+@param[out]  type    type of the file (if it exists)
 @return true if call succeeded */
 bool os_file_status(const char *path, bool *exists, os_file_type_t *type);
 
+/** Check the existence and usefulness of a given path.
+@param[in]  path  path name
+@retval true if the path exists and can be used
+@retval false if the path does not exist or if the path is
+unuseable to get to a possibly existing file or directory. */
+bool os_file_exists(const char *path);
+
 /** Create all missing subdirectories along the given path.
 @return DB_SUCCESS if OK, otherwise error code. */
 dberr_t os_file_create_subdirs_if_needed(const char *path);
@@ -1574,14 +1643,13 @@ void os_create_block_cache();
 
 /** Initializes the asynchronous io system. Creates one array each for ibuf
 and log i/o. Also creates one array each for read and write where each
-array is divided logically into n_read_segs and n_write_segs
+array is divided logically into n_readers and n_writers
 respectively. The caller must create an i/o handler thread for each
 segment in these arrays. This function also creates the sync array.
 No i/o handler thread needs to be created for that
 @param[in]	n_readers	number of reader threads
 @param[in]	n_writers	number of writer threads
-@param[in]	n_slots_sync	number of slots in the dblwr aio array */
-
+@param[in]	n_slots_sync	number of dblwr slots in the sync aio array */
 bool os_aio_init(ulint n_readers, ulint n_writers, ulint n_slots_sync);
 
 /**
@@ -1628,24 +1696,23 @@ call os_aio_simulated_wake_handler_threads later to ensure the threads
 are not left sleeping! */
 void os_aio_simulated_put_read_threads_to_sleep();
 
-/** This is the generic AIO handler interface function.
-Waits for an aio operation to complete. This function is used to wait the
+/** Waits for an AIO operation to complete. This function is used to wait the
 for completed requests. The AIO array of pending requests is divided
 into segments. The thread specifies which segment or slot it wants to wait
-for. NOTE: this function will also take care of freeing the aio slot,
+for. NOTE: this function will also take care of freeing the AIO slot,
 therefore no other thread is allowed to do the freeing!
-@param[in]	segment		the number of the segment in the aio arrays to
+@param[in]	segment		The number of the segment in the AIO arrays to
                                 wait for; segment 0 is the ibuf I/O thread,
                                 segment 1 the log I/O thread, then follow the
                                 non-ibuf read threads, and as the last are the
                                 non-ibuf write threads; if this is
                                 ULINT_UNDEFINED, then it means that sync AIO
                                 is used, and this parameter is ignored
-@param[out]	m1		the messages passed with the AIO request;
-                                note that also in the case where the AIO
-                                operation failed, these output parameters
-                                are valid and can be used to restart the
-                                operation, for example
+@param[out]	m1		the messages passed with the AIO request; note
+                                that also in the case where the AIO operation
+                                failed, these output parameters are valid and
+                                can be used to restart the operation,
+                                for example
 @param[out]	m2		callback message
 @param[out]	request		OS_FILE_WRITE or ..._READ
 @return DB_SUCCESS or error code */
@@ -1666,7 +1733,7 @@ bool os_aio_all_slots_free();
 #ifdef UNIV_DEBUG
 
 /** Prints all pending IO
-@param[in]	file	file where to print */
+@param[in]	file		File where to print */
 void os_aio_print_pending_io(FILE *file);
 
 #endif /* UNIV_DEBUG */
@@ -1679,10 +1746,10 @@ dberr_t os_get_free_space(const char *path, uint64_t &free_space);
 
 /** This function returns information about the specified file
 @param[in]	path		pathname of the file
-@param[in]	stat_info	information of a file in a directory
+@param[out]	stat_info	information of a file in a directory
 @param[in]	check_rw_perm	for testing whether the file can be opened
                                 in RW mode
-@param[in]	read_only	if true read only mode checks are enforced
+@param[in]	read_only	true if file is opened in read-only mode
 @return DB_SUCCESS if all OK */
 dberr_t os_file_get_status(const char *path, os_file_stat_t *stat_info,
                            bool check_rw_perm, bool read_only);
@@ -1691,12 +1758,13 @@ dberr_t os_file_get_status(const char *path, os_file_stat_t *stat_info,
 
 /** return any of the tmpdir path */
 char *innobase_mysql_tmpdir();
+
 /** Creates a temporary file in the location specified by the parameter
-path. If the path is NULL then it will be created on --tmpdir location.
-This function is defined in ha_innodb.cc.
+path. If the path is NULL, then it will be created in --tmpdir.
 @param[in]	path	location for creating temporary file
 @return temporary file descriptor, or < 0 on error */
 int innobase_mysql_tmpfile(const char *path);
+
 #endif /* !UNIV_HOTBACKUP */
 
 /** If it is a compressed page return the compressed page data + footer size
@@ -1746,8 +1814,10 @@ not then the source contents are left unchanged and DB_SUCCESS is returned.
 @param[in]	dblwr_read	true of double write recovery in progress
 @param[in,out]	src		Data read from disk, decompressed data will be
                                 copied to this page
-@param[in,out]	dst		Scratch area to use for decompression
-@param[in]	dst_len		Size of the scratch area in bytes
+@param[in,out]	dst		Scratch area to use for decompression or
+                                nullptr.
+@param[in]	dst_len		If dst is valid, then size of the scratch area
+                                in bytes
 @return DB_SUCCESS or error code */
 dberr_t os_file_decompress_page(bool dblwr_read, byte *src, byte *dst,
                                 ulint dst_len)
@@ -1829,9 +1899,9 @@ class Dir_Walker {
   using Function = std::function<void(const Path &, size_t)>;
 
   /** Depth first traversal of the directory starting from basedir
-  @param[in]      basedir    Start scanning from this directory
-  @param[in]      recursive  `true` if scan should be recursive
-  @param[in]      f          Callback for each entry found */
+  @param[in]  basedir     Start scanning from this directory
+  @param[in]  recursive  `true` if scan should be recursive
+  @param[in]  f           Function to call for each entry */
 #ifdef _WIN32
   static void walk_win32(const Path &basedir, bool recursive, Function &&f);
 #else
@@ -1841,12 +1911,52 @@ class Dir_Walker {
 
 /** Allocate a page for sync IO
 @return pointer to page */
-file::Block *os_alloc_block() noexcept;
+file::Block *os_alloc_block() noexcept MY_ATTRIBUTE((warn_unused_result));
+
+/** Get the sector aligned frame pointer.
+@param[in]  block   the memory block containing the page frame.
+@return the sector aligned frame pointer. */
+byte *os_block_get_frame(const file::Block *block) noexcept
+    MY_ATTRIBUTE((warn_unused_result));
 
 /** Free a page after sync IO
 @param[in,out]	block		The block to free/release */
 void os_free_block(file::Block *block) noexcept;
 
+inline void file::Block::free(file::Block *obj) noexcept { os_free_block(obj); }
+
+/** Encrypt a page content when write it to disk.
+@param[in]	type		IO flags
+@param[out]	buf		buffer to read or write
+@param[in,out]	n		number of bytes to read/write, starting from
+                                offset
+@return pointer to the encrypted page */
+file::Block *os_file_encrypt_page(const IORequest &type, void *&buf, ulint *n);
+
+/** Allocate the buffer for IO on a transparently compressed table.
+@param[in]	type		IO flags
+@param[out]	buf		buffer to read or write
+@param[in,out]	n		number of bytes to read/write, starting from
+                                offset
+@return pointer to allocated page, compressed data is written to the offset
+        that is aligned on the disk sector size */
+file::Block *os_file_compress_page(IORequest &type, void *&buf, ulint *n);
+
+/** This is a wrapper function for the os_file_write() function call.  The
+purpose of this wrapper function is to retry on i/o error. On I/O error
+(perhaps because of disk full situation) keep retrying the write operation
+till it succeeds.
+@param[in]  type     IO flags
+@param[in]  name     name of the file or path as a null-terminated string
+@param[in]  file     handle to an open file
+@param[out] buf      buffer from which to write
+@param[in]  offset   file offset from the start where to read
+@param[in]  n        number of bytes to read, starting from offset
+@return DB_SUCCESS if request was successful, false if fail */
+dberr_t os_file_write_retry(IORequest &type, const char *name,
+                            pfs_os_file_t file, const void *buf,
+                            os_offset_t offset, ulint n);
+
 #include "os0file.ic"
 #endif /* UNIV_NONINL */
 
