@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1997, 2023, Oracle and/or its affiliates.
+Copyright (c) 1997, 2020, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -43,7 +43,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 class MVCC {
  public:
   /** Constructor
-  @param size           Number of views to pre-allocate */
+  @param size		Number of views to pre-allocate */
   explicit MVCC(ulint size);
 
   /** Destructor.
@@ -51,23 +51,29 @@ class MVCC {
   ~MVCC();
 
   /** Allocate and create a view.
-  @param view   View owned by this class created for the caller. Must be
+  @param view	View owned by this class created for the caller. Must be
   freed by calling view_close()
-  @param trx    Transaction instance of caller */
+  @param trx	Transaction instance of caller */
   void view_open(ReadView *&view, trx_t *trx);
 
   /**
   Close a view created by the above function.
-  @param view           view allocated by trx_open.
-  @param own_mutex      true if caller owns trx_sys_t::mutex */
+  @param view		view allocated by trx_open.
+  @param own_mutex	true if caller owns trx_sys_t::mutex */
   void view_close(ReadView *&view, bool own_mutex);
 
+  /**
+  Release a view that is inactive but not closed. Caller must own
+  the trx_sys_t::mutex.
+  @param view		View to release */
+  void view_release(ReadView *&view);
+
   /** Clones the oldest view and stores it in view. No need to
   call view_close(). The caller owns the view that is passed in.
   It will also move the closed views from the m_views list to the
   m_free list. This function is called by Purge to determine whether it should
   purge the delete marked record or not.
-  @param view           Preallocated view, owned by the caller */
+  @param view		Preallocated view, owned by the caller */
   void clone_oldest_view(ReadView *view);
 
   /**
@@ -83,15 +89,9 @@ class MVCC {
   }
 
   /**
-  Set the view creator transaction id. Note: This should be set only
-  for views created by RW transactions.
-  @param view   Set the creator trx id for this view
-  @param id     Transaction id to set */
-  static void set_view_creator_trx_id(ReadView *view, trx_id_t id) {
-    ut_ad(id > 0);
-
-    view->creator_trx_id(id);
-  }
+  Set the view creator transaction id. Note: This shouldbe set only
+  for views created by RW transactions. */
+  static void set_view_creator_trx_id(ReadView *view, trx_id_t id);
 
  private:
   /**
@@ -110,6 +110,7 @@ class MVCC {
   marked read views from the views list to the freed list.
   @return oldest view if found or NULL */
   inline ReadView *get_oldest_view() const;
+  ReadView *get_view_created_by_trx_id(trx_id_t trx_id) const;
 
  private:
   // Prevent copying
@@ -117,7 +118,7 @@ class MVCC {
   MVCC &operator=(const MVCC &);
 
  private:
-  typedef UT_LIST_BASE_NODE_T(ReadView, m_view_list) view_list_t;
+  typedef UT_LIST_BASE_NODE_T(ReadView) view_list_t;
 
   /** Free views ready for reuse. */
   view_list_t m_free;
