@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1997, 2023, Oracle and/or its affiliates.
+Copyright (c) 1997, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -48,7 +48,8 @@ this program; if not, write to the Free Software Foundation, Inc.,
  if it figures out that an index record will be removed in the purge
  anyway, it will remove it in the rollback.
  @return DB_SUCCESS */
-[[nodiscard]] dberr_t row_undo_ins(undo_node_t *node, /*!< in: row undo node */
-                                   que_thr_t *thr);   /*!< in: query thread */
+dberr_t row_undo_ins(undo_node_t *node, /*!< in: row undo node */
+                     que_thr_t *thr)    /*!< in: query thread */
+    MY_ATTRIBUTE((warn_unused_result));
 
 #endif
