@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2020, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2020, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -33,6 +33,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #ifndef trx0purge_h
 #define trx0purge_h
 
+#include <unordered_set>
 #include "fil0fil.h"
 #include "mtr0mtr.h"
 #include "page0page.h"
@@ -116,17 +117,20 @@ struct purge_iter_t {
     // Do nothing
   }
 
-  trx_id_t trx_no;   /*!< Purge has advanced past all
-                     transactions whose number is less
-                     than this */
-  undo_no_t undo_no; /*!< Purge has advanced past all records
-                     whose undo number is less than this */
+  /** Purge has advanced past all transactions whose number
+  is less than this */
+  trx_id_t trx_no;
+
+  /** Purge has advanced past all records whose undo number
+  is less than this. */
+  undo_no_t undo_no;
+
+  /** The last undo record resided in this space id */
   space_id_t undo_rseg_space;
-  /*!< Last undo record resided in this
-  space id. */
+
+  /** The transaction that created the undo log record,
+  the Modifier trx id */
   trx_id_t modifier_trx_id;
-  /*!< the transaction that created the
-  undo log record. Modifier trx id.*/
 };
 
 /* Namespace to hold all the related functions and variables needed
@@ -170,7 +174,7 @@ undo number.
 inline space_id_t num2id(space_id_t space_num, size_t ndx) {
   ut_ad(space_num > 0);
   ut_ad(space_num <= FSP_MAX_UNDO_TABLESPACES);
-  ut_ad(ndx < dict_sys_t::undo_space_id_range);
+  ut_ad(ndx < dict_sys_t::s_undo_space_id_range);
 
   space_id_t space_id = dict_sys_t::s_max_undo_space_id + 1 - space_num -
                         static_cast<space_id_t>(ndx * FSP_MAX_UNDO_TABLESPACES);
@@ -179,7 +183,7 @@ inline space_id_t num2id(space_id_t space_num, size_t ndx) {
 }
 
 /** Convert an undo space number (from 1 to 127) into an undo space_id.
-Use the undo::space_id_bank to return the curent space_id assigned to
+Use the undo::space_id_bank to return the current space_id assigned to
 that undo number.
 @param[in]  space_num   undo tablespace number
 @return space_id of the undo tablespace */
@@ -203,11 +207,11 @@ NOTE: This may be an undo space_id from a pre-exisiting 5.7
 database which used space_ids from 1 to 127.  If so, the
 space_id is the space_num.
 The space_ids are assigned to number ranges in reverse from high to low.
-In addition, the first space IDs for each undo number occur sequentionally
+In addition, the first space IDs for each undo number occur sequentially
 and descending before the second space_id.
 
 Since s_max_undo_space_id = 0xFFFFFFEF, FSP_MAX_UNDO_TABLESPACES = 127
-and undo_space_id_range = 512:
+and s_undo_space_id_range = 400,000:
   Space ID   Space Num    Space ID   Space Num   ...  Space ID   Space Num
   0xFFFFFFEF      1       0xFFFFFFEe       2     ...  0xFFFFFF71    127
   0xFFFFFF70      1       0xFFFFFF6F       2     ...  0xFFFFFEF2    127
@@ -237,7 +241,7 @@ inline space_id_t id2next_id(space_id_t space_id) {
   space_id_t space_num = id2num(space_id);
   space_id_t first_id = dict_sys_t::s_max_undo_space_id + 1 - space_num;
   space_id_t last_id = first_id - (FSP_MAX_UNDO_TABLESPACES *
-                                   (dict_sys_t::undo_space_id_range - 1));
+                                   (dict_sys_t::s_undo_space_id_range - 1));
 
   return (space_id == SPACE_UNKNOWN || space_id == last_id
               ? first_id
@@ -263,7 +267,7 @@ return the next available space_id for that space number.
 space_id_t use_next_space_id(space_id_t space_num);
 
 /** Mark an undo number associated with a given space_id as unused and
-available to be resused.  This happens when the fil_space_t is closed
+available to be reused.  This happens when the fil_space_t is closed
 associated with a drop undo tablespace.
 @param[in] space_id  Undo Tablespace ID */
 void unuse_space_id(space_id_t space_id);
@@ -282,9 +286,9 @@ space number.
 space_id_t next_space_id(space_id_t space_id);
 
 /** Return the next available undo space ID to be used for a new explicit
-undo tablespaces.
-@retval if success, next available undo space number.
-@retval if failure, SPACE_UNKNOWN */
+undo tablespaces. The slot will be marked as in-use.
+@return next available undo space number if successful.
+@return SPACE_UNKNOWN if failed */
 space_id_t get_next_available_space_num();
 
 /** Build a standard undo tablespace name from a space_id.
@@ -293,6 +297,8 @@ space_id_t get_next_available_space_num();
 char *make_space_name(space_id_t space_id);
 
 /** Build a standard undo tablespace file name from a space_id.
+This will create a name like 'undo_001' if the space_id is in the
+reserved range, else it will be like 'undo001'.
 @param[in]	space_id	id of the undo tablespace.
 @return file_name of the undo tablespace file */
 char *make_file_name(space_id_t space_id);
@@ -364,34 +370,7 @@ struct Tablespace {
 
   /* Determine if this undo space needs to be truncated.
   @return true if it should be truncated, false if not. */
-  bool needs_truncation() {
-    /* If it is already inactive, even implicitly, then proceed. */
-    m_rsegs->s_lock();
-    if (m_rsegs->is_inactive_implicit() || m_rsegs->is_inactive_explicit()) {
-      m_rsegs->s_unlock();
-      return (true);
-    }
-
-    /* If implicit undo truncation is turned off, or if the rsegs don't exist
-    yet, don't bother checking the size. */
-    if (!srv_undo_log_truncate || m_rsegs == nullptr || m_rsegs->is_empty() ||
-        m_rsegs->is_init()) {
-      m_rsegs->s_unlock();
-      return (false);
-    }
-
-    ut_ad(m_rsegs->is_active());
-    m_rsegs->s_unlock();
-
-    page_no_t trunc_size = ut_max(
-        static_cast<page_no_t>(srv_max_undo_tablespace_size / srv_page_size),
-        static_cast<page_no_t>(SRV_UNDO_TABLESPACE_SIZE_IN_PAGES));
-    if (fil_space_get_size(id()) > trunc_size) {
-      return (true);
-    }
-
-    return (false);
-  }
+  bool needs_truncation();
 
   /** Change the space_id from its current value.
   @param[in]  space_id  The new undo tablespace ID */
@@ -421,7 +400,7 @@ struct Tablespace {
   of the file name provided. This name can come in three forms:
   absolute path, relative path, and basename.  Undo ADD DATAFILE
   does not accept a relative path.  So if that comes in here, it
-  was the scaneed name and is relative to the datadir.
+  was the scanned name and is relative to the datadir.
   If this is just a basename, add it to srv_undo_dir.
   @param[in]  file_name  explicit undo file name */
   void set_file_name(const char *file_name);
@@ -741,6 +720,31 @@ class Tablespaces {
     return (nullptr);
   }
 
+  /** Find the first undo space that is marked inactive explicitly.
+  @param[in,out]  num_active  If there are no inactive_explicit spaces
+                              found, this will contain the number of
+                              active spaces found.
+  @return pointer to an undo::Tablespace struct */
+  Tablespace *find_first_inactive_explicit(size_t *num_active) {
+    ut_ad(own_latch());
+
+    if (m_spaces.empty()) {
+      return (nullptr);
+    }
+
+    for (auto undo_space : m_spaces) {
+      if (undo_space->is_inactive_explicit()) {
+        return (undo_space);
+      }
+
+      if (num_active != nullptr && undo_space->is_active()) {
+        (*num_active)++;
+      }
+    }
+
+    return (nullptr);
+  }
+
 #ifdef UNIV_DEBUG
   /** Determine if this thread owns a lock on m_latch. */
   bool own_latch() {
@@ -768,13 +772,39 @@ class Tablespaces {
   rw_lock_t *m_latch;
 };
 
-/** Mutext for serializing undo tablespace related DDL.  These have to do with
+/** Mutex for serializing undo tablespace related DDL.  These have to do with
 creating and dropping undo tablespaces. */
 extern ib_mutex_t ddl_mutex;
 
 /** A global object that contains a vector of undo::Tablespace structs. */
 extern Tablespaces *spaces;
 
+#ifdef UNIV_DEBUG
+/**  Inject a crash if a certain SET GLOBAL DEBUG has been set.
+Before DBUG_SUICIDE(), write an entry about this crash to the error log
+and flush the redo log. */
+void inject_crash(const char *injection_point_name);
+
+/** Inject a failure in the undo truncation debug compiled code at various
+places so that it fails the first time it hits and succeeds after that. */
+class Inject_failure_once {
+  bool m_already_failed;
+  const char *m_inject_name;
+
+ public:
+  Inject_failure_once(const char *inject_name)
+      : m_already_failed{false}, m_inject_name{inject_name} {}
+
+  /**  If a certain SET GLOBAL DEBUG has been set and this is the first time
+  this has been called for that injection point, write an entry to the
+  error log and return true so that the caller can cause the failure.
+  @return true iff compiled with debug and the debug point has been set
+          and this it the first call for this debug point. */
+  bool should_fail();
+};
+
+#endif /* UNIV_DEBUG */
+
 /** Create the truncate log file. Needed to track the state of truncate during
 a crash. An auxiliary redo log file undo_<space_id>_trunc.log will be created
 while the truncate of the UNDO is in progress. This file is required during
@@ -814,7 +844,7 @@ void add_space_to_construction_list(space_id_t space_id);
 /** Clear the s_under_construction vector. */
 void clear_construction_list();
 
-/** Is an undo tablespace under constuction at the moment.
+/** Is an undo tablespace under construction at the moment.
 @param[in]	space_id	space id to check
 @return true if marked for truncate, else false. */
 bool is_under_construction(space_id_t space_id);
@@ -830,15 +860,16 @@ will not be under construction, so this function will return true.
 @return true if active (non-undo spaces are always active) */
 bool is_active(space_id_t space_id, bool get_latch = true);
 
+constexpr ulint TRUNCATE_FREQUENCY = 128;
+
 /** Track an UNDO tablespace marked for truncate. */
 class Truncate {
  public:
-  Truncate()
-      : m_space_id_marked(SPACE_UNKNOWN),
-        m_purge_rseg_truncate_frequency(
-            static_cast<ulint>(srv_purge_rseg_truncate_frequency)) {
-    /* Do Nothing. */
-  }
+  /** Constructor. */
+  Truncate() : m_space_id_marked(SPACE_UNKNOWN), m_timer() {}
+
+  /** Destructor. */
+  ~Truncate() = default;
 
   /** Is tablespace selected for truncate.
   @return true if undo tablespace is marked for truncate */
@@ -848,31 +879,13 @@ class Truncate {
   so that it will be truncated next. */
   void set_marked_space_empty() { m_marked_space_is_empty = true; }
 
-  /** Is tablespace selected for truncate empty of undo logs yet?
+  /** Is the tablespace selected for truncate empty of undo logs yet?
   @return true if the marked undo tablespace has no more undo logs */
   bool is_marked_space_empty() const { return (m_marked_space_is_empty); }
 
   /** Mark the tablespace for truncate.
   @param[in]  undo_space  undo tablespace to truncate. */
-  void mark(Tablespace *undo_space) {
-    /* Set the internal state of this undo space to inactive_implicit
-    so that its rsegs will not be allocated to any new transaction.
-    If the space is already in the inactive_explicit state, it will
-    stay there.
-    Note that the DD is not modified since in case of crash, the
-    action must be completed before the DD is available.
-    Set both the state and this marked id while this routine has
-    an x_lock on m_rsegs because a concurrent user thread might issue
-    undo_space->alter_active(). */
-    undo_space->set_inactive_implicit(&m_space_id_marked);
-
-    m_marked_space_is_empty = false;
-
-    /* We found an UNDO-tablespace to truncate so set the
-    local purge rseg truncate frequency to 3. This will help
-    accelerate the purge action and in turn truncate. */
-    set_rseg_truncate_frequency(3);
-  }
+  void mark(Tablespace *undo_space);
 
   /** Get the ID of the tablespace marked for truncate.
   @return tablespace ID marked for truncate. */
@@ -882,18 +895,14 @@ class Truncate {
 
   /** Reset for next rseg truncate. */
   void reset() {
-    /* Sync with global value as we are done with
-    truncate now. */
-    set_rseg_truncate_frequency(
-        static_cast<ulint>(srv_purge_rseg_truncate_frequency));
-
+    reset_timer();
     m_marked_space_is_empty = false;
     m_space_id_marked = SPACE_UNKNOWN;
   }
 
   /** Get the undo tablespace number to start a scan.
   Re-adjust in case the spaces::size() went down.
-  @return	UNDO space_num to start scanning. */
+  @return undo space_num to start scanning. */
   space_id_t get_scan_space_num() const {
     s_scan_pos = s_scan_pos % undo::spaces->size();
 
@@ -903,29 +912,17 @@ class Truncate {
   }
 
   /** Increment the scanning position in a round-robin fashion.
-  @return	UNDO space_num at incremented scanning position. */
+  @return undo space_num at incremented scanning position. */
   space_id_t increment_scan() const {
-    /** Round-robin way of selecting an undo tablespace
-    for the truncate operation. Once we reach the end of
-    the list of known undo tablespace IDs, move back to
-    the first undo tablespace ID. This will scan active
+    /** Round-robin way of selecting an undo tablespace for the truncate
+    operation. Once we reach the end of the list of known undo tablespace
+    IDs, move back to the first undo tablespace ID. This will scan active
     as well as inactive undo tablespaces. */
     s_scan_pos = (s_scan_pos + 1) % undo::spaces->size();
 
     return (get_scan_space_num());
   }
 
-  /** Get local rseg purge truncate frequency
-  @return rseg purge truncate frequency. */
-  ulint get_rseg_truncate_frequency() const {
-    return (m_purge_rseg_truncate_frequency);
-  }
-
-  /** Set local rseg purge truncate frequency */
-  void set_rseg_truncate_frequency(ulint frequency) {
-    m_purge_rseg_truncate_frequency = frequency;
-  }
-
   /** Check if the given space id is equal to the space ID that is marked for
   truncation.
   @return true if they are equal, false otherwise. */
@@ -933,26 +930,27 @@ class Truncate {
     return (m_space_id_marked == space_id);
   }
 
+  /** @return the number of milliseconds since last reset. */
+  int64_t check_timer() const { return (m_timer.elapsed()); }
+
+  /** Reset the timer. */
+  void reset_timer() { m_timer.reset(); }
+
  private:
   /** UNDO space ID that is marked for truncate. */
   space_id_t m_space_id_marked;
 
-  /** This is true if the marked space is empty of undo logs
-  and ready to truncate.  We leave the rsegs object 'inactive'
-  until after it is truncated and rebuilt.  This allow the
-  code to do the check for undo logs only once. */
+  /** This is true if the marked space is empty of undo logs and ready
+  to truncate.  We leave the rsegs object 'inactive' until after it is
+  truncated and rebuilt.  This allow the code to do the check for undo
+  logs only once. */
   bool m_marked_space_is_empty;
 
-  /** Rollback segment(s) purge frequency. This is a local
-  value maintained along with the global value. It is set
-  to the global value in the before each truncate.  But when
-  a tablespace is marked for truncate it is updated to 1 and
-  then minimum value among 2 is used by the purge action. */
-  ulint m_purge_rseg_truncate_frequency;
+  /** Elapsed time since last truncate check. */
+  ib::Timer m_timer;
 
-  /** Start scanning for UNDO tablespace from this
-  vector position. This is to avoid bias selection
-  of one tablespace always. */
+  /** Start scanning for UNDO tablespace from this vector position. This is
+  to avoid bias selection of one tablespace always. */
   static size_t s_scan_pos;
 
 }; /* class Truncate */
@@ -961,85 +959,103 @@ class Truncate {
 
 /** The control structure used in the purge operation */
 struct trx_purge_t {
-  sess_t *sess; /*!< System session running the purge
-                query */
-  trx_t *trx;   /*!< System transaction running the
-                purge query: this trx is not in the
-                trx list of the trx system and it
-                never ends */
+  /** System session running the purge query */
+  sess_t *sess;
+
+  /** System transaction running the purge query: this trx is not in the trx
+  list of the trx system and it never ends */
+  trx_t *trx;
 #ifndef UNIV_HOTBACKUP
-  rw_lock_t latch;              /*!< The latch protecting the purge
-                                view. A purge operation must acquire an
-                                x-latch here for the instant at which
-                                it changes the purge view: an undo
-                                log operation can prevent this by
-                                obtaining an s-latch here. It also
-                                protects state and running */
-#endif                          /* !UNIV_HOTBACKUP */
-  os_event_t event;             /*!< State signal event */
-  ulint n_stop;                 /*!< Counter to track number stops */
-  volatile bool running;        /*!< true, if purge is active,
-                                we check this without the latch too */
-  volatile purge_state_t state; /*!< Purge coordinator thread states,
-                                we check this in several places
-                                without holding the latch. */
-  que_t *query;                 /*!< The query graph which will do the
-                                parallelized purge operation */
-  ReadView view;                /*!< The purge will not remove undo logs
-                                which are >= this view (purge view) */
-  bool view_active;             /*!< true if view is active */
-  volatile ulint n_submitted;   /*!< Count of total tasks submitted
-                                to the task queue */
-  volatile ulint n_completed;   /*!< Count of total tasks completed */
+  /** The latch protecting the purge view. A purge operation must acquire an
+  x-latch here for the instant at which it changes the purge view: an undo
+  log operation can prevent this by obtaining an s-latch here. It also
+  protects state and running */
+  rw_lock_t latch;
+#endif /* !UNIV_HOTBACKUP */
+
+  /** State signal event */
+  os_event_t event;
+
+  /** Counter to track number stops */
+  ulint n_stop;
+
+  /** true, if purge is active, we check this without the latch too */
+  volatile bool running;
+
+  /** Purge coordinator thread states, we check this in several places without
+  holding the latch. */
+  volatile purge_state_t state;
+
+  /** The query graph which will do the parallelized purge operation */
+  que_t *query;
+
+  /** The purge will not remove undo logs which are >= this view (purge view) */
+  ReadView view;
+
+  /** true if view is active */
+  bool view_active;
+
+  /** Count of total tasks submitted to the task queue */
+  volatile ulint n_submitted;
+
+  /** Count of total tasks completed */
+  std::atomic<ulint> n_completed;
 
-  /*------------------------------*/
   /* The following two fields form the 'purge pointer' which advances
   during a purge, and which is used in history list truncation */
 
-  purge_iter_t iter;  /* Limit up to which we have read and
-                      parsed the UNDO log records.  Not
-                      necessarily purged from the indexes.
-                      Note that this can never be less than
-                      the limit below, we check for this
-                      invariant in trx0purge.cc */
-  purge_iter_t limit; /* The 'purge pointer' which advances
-                      during a purge, and which is used in
-                      history list truncation */
+  /** Limit up to which we have read and parsed the UNDO log records.  Not
+  necessarily purged from the indexes.  Note that this can never be less than
+  the limit below, we check for this invariant in trx0purge.cc */
+  purge_iter_t iter;
+
+  /** The 'purge pointer' which advances during a purge, and which is used in
+  history list truncation */
+  purge_iter_t limit;
 #ifdef UNIV_DEBUG
-  purge_iter_t done; /* Indicate 'purge pointer' which have
-                     purged already accurately. */
-#endif               /* UNIV_DEBUG */
-  /*-----------------------------*/
-  ibool next_stored;     /*!< TRUE if the info of the next record
-                         to purge is stored below: if yes, then
-                         the transaction number and the undo
-                         number of the record are stored in
-                         purge_trx_no and purge_undo_no above */
-  trx_rseg_t *rseg;      /*!< Rollback segment for the next undo
-                         record to purge */
-  page_no_t page_no;     /*!< Page number for the next undo
-                         record to purge, page number of the
-                         log header, if dummy record */
-  ulint offset;          /*!< Page offset for the next undo
-                         record to purge, 0 if the dummy
-                         record */
-  page_no_t hdr_page_no; /*!< Header page of the undo log where
-                         the next record to purge belongs */
-  ulint hdr_offset;      /*!< Header byte offset on the page */
+  /** Indicate 'purge pointer' which have purged already accurately. */
+  purge_iter_t done;
+#endif /* UNIV_DEBUG */
 
-  TrxUndoRsegsIterator *rseg_iter; /*!< Iterator to get the next rseg
-                                   to process */
+  /** true if the info of the next record to purge is stored below: if yes, then
+  the transaction number and the undo number of the record are stored in
+  purge_trx_no and purge_undo_no above */
+  bool next_stored;
 
-  purge_pq_t *purge_queue; /*!< Binary min-heap, ordered on
-                           TrxUndoRsegs::trx_no. It is protected
-                           by the pq_mutex */
-  PQMutex pq_mutex;        /*!< Mutex protecting purge_queue */
+  /** Rollback segment for the next undo record to purge */
+  trx_rseg_t *rseg;
 
-  undo::Truncate undo_trunc; /*!< Track UNDO tablespace marked
-                             for truncate. */
+  /** Page number for the next undo record to purge, page number of the log
+  header, if dummy record */
+  page_no_t page_no;
 
-  mem_heap_t *heap; /*!< Heap for reading the undo log
-                    records */
+  /** Page offset for the next undo record to purge, 0 if the dummy record */
+  ulint offset;
+
+  /** Header page of the undo log where the next record to purge belongs */
+  page_no_t hdr_page_no;
+
+  /** Header byte offset on the page */
+  ulint hdr_offset;
+
+  /** Iterator to get the next rseg to process */
+  TrxUndoRsegsIterator *rseg_iter;
+
+  /** Binary min-heap, ordered on TrxUndoRsegs::trx_no. It is protected
+  by the pq_mutex */
+  purge_pq_t *purge_queue;
+
+  /** Mutex protecting purge_queue */
+  PQMutex pq_mutex;
+
+  /** Track UNDO tablespace marked for truncate. */
+  undo::Truncate undo_trunc;
+
+  /** Heap for reading the undo log records */
+  mem_heap_t *heap;
+
+  /** Set of all THDs allocated by the purge system. */
+  ut::unordered_set<THD *> thds;
 };
 
 /** Choose the rollback segment with the smallest trx_no. */
