@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2019, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2020, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Google Inc.
 
 This program is free software; you can redistribute it and/or modify
@@ -90,13 +90,11 @@ write the checkpoint (e.g. coming from log_make_latest_checkpoint()).
 static bool log_should_checkpoint(log_t &log);
 
 /** Considers writing next checkpoint. Checks if checkpoint should be written
-(using log_should_checkpoint()) and writes the checkpoint if that's the case.
-@return true if checkpoint has been written */
-static bool log_consider_checkpoint(log_t &log);
+(using log_should_checkpoint()) and writes the checkpoint if that's the case. */
+static void log_consider_checkpoint(log_t &log);
 
-/** Considers requesting page cleaners to execute sync flush.
-@return true if request has been made */
-static bool log_consider_sync_flush(log_t &log);
+/** Considers requesting page cleaners to execute sync flush. */
+static void log_consider_sync_flush(log_t &log);
 
 /** Makes a checkpoint. Note that this function does not flush dirty blocks
 from the buffer pool. It only checks what is lsn of the oldest modification
@@ -129,13 +127,37 @@ out of space in redo log (close to free_check_limit_sn).
 @param[in]  new_oldest  oldest_lsn to stop flush at (or greater) */
 static bool log_request_sync_flush(const log_t &log, lsn_t new_oldest);
 
+/** Sync log file changes to disk if required. */
+static void log_fsync() {
+#ifdef _WIN32
+  switch (srv_win_file_flush_method) {
+    case SRV_WIN_IO_UNBUFFERED:
+      break;
+    case SRV_WIN_IO_NORMAL:
+      fil_flush_file_redo();
+      break;
+  }
+#else
+  switch (srv_unix_file_flush_method) {
+    case SRV_UNIX_O_DSYNC:
+    case SRV_UNIX_NOSYNC:
+      break;
+    case SRV_UNIX_FSYNC:
+    case SRV_UNIX_LITTLESYNC:
+    case SRV_UNIX_O_DIRECT:
+    case SRV_UNIX_O_DIRECT_NO_FSYNC:
+      fil_flush_file_redo();
+  }
+#endif /* _WIN32 */
+}
+
 /**************************************************/ /**
 
  @name Coordination with buffer pool and oldest_lsn
 
  *******************************************************/
 
-/* @{ */
+/** @{ */
 
 /** Calculates lsn at which we might write a next checkpoint. It does the
 best effort, but possibly the maximum allowed lsn, could be even bigger.
@@ -180,23 +202,15 @@ static lsn_t log_compute_available_for_checkpoint_lsn(const log_t &log) {
 
   lsn_t lwm_lsn = buf_pool_get_oldest_modification_lwm();
 
-  /* Empty flush list. */
+  /* We cannot return lsn larger than dpa_lsn,
+  because some mtr's commit could be in the middle, after
+  its log records have been written to log buffer, but before
+  its dirty pages have been added to flush lists. */
 
   if (lwm_lsn == 0) {
-    /* There are no dirty pages. We cannot return current lsn,
-    because some mtr's commit could be in the middle, after
-    its log records have been written to log buffer, but before
-    its dirty pages have been added to flush lists. */
-
+    /* Empty flush list. */
     lwm_lsn = dpa_lsn;
-
   } else {
-    /* Cannot go beyound dpa_lsn.
-
-    Note that log_closer might still not advance dpa enough,
-    because it might be scheduled out. Yes, the returned lwm
-    guarantees it is able to advance, but it needs to do it! */
-
     lwm_lsn = std::min(lwm_lsn, dpa_lsn);
   }
 
@@ -251,6 +265,7 @@ static lsn_t log_compute_available_for_checkpoint_lsn(const log_t &log) {
 
 static lsn_t log_update_available_for_checkpoint_lsn(log_t &log) {
   /* Update lsn available for checkpoint. */
+  log.recent_closed.advance_tail();
   const lsn_t oldest_lsn = log_compute_available_for_checkpoint_lsn(log);
 
   log_limits_mutex_enter(log);
@@ -273,7 +288,7 @@ static lsn_t log_update_available_for_checkpoint_lsn(log_t &log) {
   return (result);
 }
 
-/* @} */
+/** @} */
 
 /**************************************************/ /**
 
@@ -281,9 +296,10 @@ static lsn_t log_update_available_for_checkpoint_lsn(log_t &log) {
 
  *******************************************************/
 
-/* @{ */
+/** @{ */
 
-void log_files_header_fill(byte *buf, lsn_t start_lsn, const char *creator) {
+void log_files_header_fill(byte *buf, lsn_t start_lsn, const char *creator,
+                           bool no_logging, bool crash_unsafe) {
   memset(buf, 0, OS_FILE_LOG_BLOCK_SIZE);
 
   if (log_detected_format != UINT32_MAX) {
@@ -303,6 +319,16 @@ void log_files_header_fill(byte *buf, lsn_t start_lsn, const char *creator) {
 
   ut_ad(LOG_HEADER_CREATOR_END - LOG_HEADER_CREATOR >= strlen(creator));
 
+  uint32_t header_flags = 0;
+
+  if (no_logging) {
+    LOG_HEADER_SET_FLAG(header_flags, LOG_HEADER_FLAG_NO_LOGGING);
+  }
+  if (crash_unsafe) {
+    LOG_HEADER_SET_FLAG(header_flags, LOG_HEADER_FLAG_CRASH_UNSAFE);
+  }
+  mach_write_to_4(buf + LOG_HEADER_FLAGS, header_flags);
+
   log_block_set_checksum(buf, log_block_calc_checksum_crc32(buf));
 }
 
@@ -315,7 +341,13 @@ void log_files_header_flush(log_t &log, uint32_t nth_file, lsn_t start_lsn) {
 
   byte *buf = log.file_header_bufs[nth_file];
 
-  log_files_header_fill(buf, start_lsn, LOG_HEADER_CREATOR_CURRENT);
+  log_files_header_fill(buf, start_lsn, LOG_HEADER_CREATOR_CURRENT,
+                        log.m_disable, log.m_crash_unsafe);
+
+  /* Save start LSN for first file. */
+  if (nth_file == 0) {
+    log.m_first_file_lsn = start_lsn;
+  }
 
   DBUG_PRINT("ib_log", ("write " LSN_PF " file " ULINTPF " header", start_lsn,
                         ulint(nth_file)));
@@ -348,6 +380,52 @@ void log_files_header_read(log_t &log, uint32_t header) {
   ut_a(err == DB_SUCCESS);
 }
 
+void log_persist_enable(log_t &log) {
+  log_writer_mutex_enter(log);
+
+  log.m_disable = false;
+  log.m_crash_unsafe = false;
+
+  ut_ad(!srv_read_only_mode);
+  log_files_header_flush(log, 0, log.m_first_file_lsn);
+
+  log_writer_mutex_exit(log);
+  log_fsync();
+}
+
+void log_persist_disable(log_t &log) {
+  log_writer_mutex_enter(log);
+
+  log.m_disable = true;
+
+  /* If server is restarted in read only mode, we should
+  skip writing to redo header. */
+  if (!srv_read_only_mode) {
+    log.m_crash_unsafe = true;
+    log_files_header_flush(log, 0, log.m_first_file_lsn);
+  }
+
+  log_writer_mutex_exit(log);
+  log_fsync();
+}
+
+void log_persist_crash_safe(log_t &log) {
+  if (srv_read_only_mode) {
+    ut_ad(!log.m_crash_unsafe);
+    return;
+  }
+
+  log_writer_mutex_enter(log);
+
+  ut_ad(log.m_disable);
+  log.m_crash_unsafe = false;
+
+  log_files_header_flush(log, 0, log.m_first_file_lsn);
+
+  log_writer_mutex_exit(log);
+  log_fsync();
+}
+
 #endif /* UNIV_HOTBACKUP */
 #ifdef UNIV_HOTBACKUP
 
@@ -374,7 +452,7 @@ void meb_log_print_file_hdr(byte *block) {
 #ifndef UNIV_HOTBACKUP
 
 void log_files_downgrade(log_t &log) {
-  ut_ad(srv_is_being_shutdown);
+  ut_ad(srv_shutdown_state.load() >= SRV_SHUTDOWN_LAST_PHASE);
   ut_a(!log_checkpointer_is_active());
 
   const uint32_t nth_file = 0;
@@ -399,7 +477,7 @@ void log_files_downgrade(log_t &log) {
   ut_a(err == DB_SUCCESS);
 }
 
-/* @} */
+/** @} */
 
 /**************************************************/ /**
 
@@ -407,7 +485,7 @@ void log_files_downgrade(log_t &log) {
 
  *******************************************************/
 
-/* @{ */
+/** @{ */
 
 static lsn_t log_determine_checkpoint_lsn(log_t &log) {
   ut_ad(log_checkpointer_mutex_own(log));
@@ -473,26 +551,7 @@ void log_files_write_checkpoint(log_t &log, lsn_t next_checkpoint_lsn) {
 
   LOG_SYNC_POINT("log_before_checkpoint_flush");
 
-#ifdef _WIN32
-  switch (srv_win_file_flush_method) {
-    case SRV_WIN_IO_UNBUFFERED:
-      break;
-    case SRV_WIN_IO_NORMAL:
-      fil_flush_file_redo();
-      break;
-  }
-#else
-  switch (srv_unix_file_flush_method) {
-    case SRV_UNIX_O_DSYNC:
-    case SRV_UNIX_NOSYNC:
-      break;
-    case SRV_UNIX_FSYNC:
-    case SRV_UNIX_LITTLESYNC:
-    case SRV_UNIX_O_DIRECT:
-    case SRV_UNIX_O_DIRECT_NO_FSYNC:
-      fil_flush_file_redo();
-  }
-#endif /* _WIN32 */
+  log_fsync();
 
   DBUG_PRINT("ib_log", ("checkpoint info written"));
 
@@ -767,17 +826,22 @@ static bool log_request_sync_flush(const log_t &log, lsn_t new_oldest) {
   } else if (srv_flush_sync) {
     /* Wake up page cleaner asking to perform sync flush
     (unless user explicitly disabled sync-flushes). */
-    new_oldest += log_buffer_flush_order_lag(log);
 
-    return (buf_flush_request_force(new_oldest));
+    int64_t sig_count = os_event_reset(buf_flush_tick_event);
+
+    os_event_set(buf_flush_event);
+
+    os_event_wait_time_low(buf_flush_tick_event, 1000000, sig_count);
+
+    return (true);
 
   } else {
     return (false);
   }
 }
 
-static bool log_consider_sync_flush(log_t &log) {
-  ut_ad(log_checkpointer_mutex_own(log));
+lsn_t log_sync_flush_lsn(log_t &log) {
+  log_update_available_for_checkpoint_lsn(log);
 
   /* We acquire limits mutex only for a short period. Afterwards these
   values might be changed (advanced to higher values). However, in the
@@ -800,7 +864,7 @@ static bool log_consider_sync_flush(log_t &log) {
   ut_a(flush_up_to <= current_lsn);
 
   if (current_lsn == flush_up_to) {
-    return (false);
+    return 0;
   }
 
   const lsn_t margin = log_free_check_margin(log);
@@ -820,9 +884,23 @@ static bool log_consider_sync_flush(log_t &log) {
   }
 
   if (flush_up_to > oldest_lsn) {
+    flush_up_to += log_buffer_flush_order_lag(log);
+
+    return flush_up_to;
+  }
+
+  return 0;
+}
+
+static void log_consider_sync_flush(log_t &log) {
+  ut_ad(log_checkpointer_mutex_own(log));
+
+  const auto flush_up_to = log_sync_flush_lsn(log);
+
+  if (flush_up_to != 0) {
     log_checkpointer_mutex_exit(log);
 
-    const bool result = log_request_sync_flush(log, flush_up_to);
+    log_request_sync_flush(log, flush_up_to);
 
     log_checkpointer_mutex_enter(log);
 
@@ -830,11 +908,7 @@ static bool log_consider_sync_flush(log_t &log) {
     lsn available for creating a new checkpoint, just try to update
     it to not wait for next checkpointer loop. */
     log_update_available_for_checkpoint_lsn(log);
-
-    return (result);
   }
-
-  return (false);
 }
 
 static uint64_t log_checkpoint_time_elapsed(const log_t &log) {
@@ -927,11 +1001,11 @@ static bool log_should_checkpoint(log_t &log) {
   return (false);
 }
 
-static bool log_consider_checkpoint(log_t &log) {
+static void log_consider_checkpoint(log_t &log) {
   ut_ad(log_checkpointer_mutex_own(log));
 
   if (!log_should_checkpoint(log)) {
-    return (false);
+    return;
   }
 
   /* It's clear that a new checkpoint should be written.
@@ -953,12 +1027,10 @@ static bool log_consider_checkpoint(log_t &log) {
   have changed, we follow a simple way and perform a full
   re-check of all conditions. */
   if (!log_should_checkpoint(log)) {
-    return (false);
+    return;
   }
 
   log_checkpoint(log);
-
-  return (true);
 }
 
 void log_checkpointer(log_t *log_ptr) {
@@ -966,69 +1038,96 @@ void log_checkpointer(log_t *log_ptr) {
 
   log_t &log = *log_ptr;
 
-  log_checkpointer_mutex_enter(log);
+  static const ulint log_busy_checkpoint_interval =
+      7; /*SRV_MASTER_CHECKPOINT_INTERVAL*/
+  ulint old_activity_count = srv_get_activity_count();
+  ulint error = OS_SYNC_TIME_EXCEEDED;
 
-  while (true) {
-    auto do_some_work = [&log] {
-      ut_ad(log_checkpointer_mutex_own(log));
+  for (;;) {
+    log_checkpointer_mutex_enter(log);
 
-      /* We will base our next decisions on maximum lsn
-      available for creating a new checkpoint. It would
-      be great to have it updated beforehand. Also, this
-      is the only thread that relies on that value, so we
-      don't need to update it in other threads. */
-      log_update_available_for_checkpoint_lsn(log);
+    const auto sig_count = os_event_reset(log.checkpointer_event);
+    const lsn_t requested_checkpoint_lsn = log.requested_checkpoint_lsn;
 
+    bool system_is_busy = false;
+    if (error == OS_SYNC_TIME_EXCEEDED &&
+        srv_check_activity(old_activity_count)) {
+      old_activity_count = srv_get_activity_count();
+      /* system is busy. takes longer interval. */
+      system_is_busy = true;
+    }
+
+    if (error != OS_SYNC_TIME_EXCEEDED || !system_is_busy ||
+        requested_checkpoint_lsn >
+            log.last_checkpoint_lsn.load(std::memory_order_acquire) ||
+        log_checkpoint_time_elapsed(log) >=
+            log_busy_checkpoint_interval * srv_log_checkpoint_every * 1000ULL) {
       /* Consider flushing some dirty pages. */
-      const bool sync_flushed = log_consider_sync_flush(log);
+      log_consider_sync_flush(log);
 
       LOG_SYNC_POINT("log_checkpointer_before_consider_checkpoint");
 
       /* Consider writing checkpoint. */
-      const bool checkpointed = log_consider_checkpoint(log);
+      log_consider_checkpoint(log);
+    }
 
-      if (sync_flushed || checkpointed) {
-        return (true);
-      }
-
-      if (log.should_stop_threads.load()) {
-        if (!log_closer_is_active()) {
-          return (true);
-        }
-      }
-
-      return (false);
-    };
-
-    const auto sig_count = os_event_reset(log.checkpointer_event);
-
-    if (!do_some_work()) {
-      log_checkpointer_mutex_exit(log);
-
-      os_event_wait_time_low(log.checkpointer_event, 10 * 1000, sig_count);
-
-      log_checkpointer_mutex_enter(log);
+    log_checkpointer_mutex_exit(log);
 
+    if (requested_checkpoint_lsn >
+        log.last_checkpoint_lsn.load(std::memory_order_relaxed)) {
+      /* not satisfied. retry. */
+      error = 0;
     } else {
-      log_checkpointer_mutex_exit(log);
-
-      os_thread_sleep(0);
-
-      log_checkpointer_mutex_enter(log);
+      error = os_event_wait_time_low(
+          log.checkpointer_event, srv_log_checkpoint_every * 1000, sig_count);
     }
 
     /* Check if we should close the thread. */
     if (log.should_stop_threads.load()) {
-      if (!log_closer_is_active()) {
-        break;
+      ut_ad(!log.writer_threads_paused.load());
+      if (!log_flusher_is_active() && !log_writer_is_active()) {
+        lsn_t end_lsn = log.write_lsn.load();
+
+        ut_a(log_lsn_validate(end_lsn));
+        ut_a(end_lsn == log.flushed_to_disk_lsn.load());
+        ut_a(end_lsn == log_buffer_ready_for_write_lsn(log));
+
+        ut_a(end_lsn >= log_buffer_dirty_pages_added_up_to_lsn(log));
+
+        if (log_buffer_dirty_pages_added_up_to_lsn(log) == end_lsn) {
+          /* All confirmed reservations have been written
+          to redo and all dirty pages related to those
+          writes have been added to flush lists.
+
+          However, there could be user threads, which are
+          in the middle of log_buffer_reserve(), reserved
+          range of sn values, but could not confirm.
+
+          Note that because log_writer is already not alive,
+          the only possible reason guaranteed by its death,
+          is that there is x-lock at end_lsn, in which case
+          end_lsn separates two regions in log buffer:
+          completely full and completely empty. */
+          const lsn_t ready_lsn = log_buffer_ready_for_write_lsn(log);
+
+          const lsn_t current_lsn = log_get_lsn(log);
+
+          if (current_lsn > ready_lsn) {
+            log.recent_written.validate_no_links(ready_lsn, current_lsn);
+            log.recent_closed.validate_no_links(ready_lsn, current_lsn);
+          }
+
+          break;
+        }
+        /* We need to wait until remaining dirty pages
+        have been added. */
       }
+      /* We prefer to wait until all writing is done. */
     }
   }
-
-  log_checkpointer_mutex_exit(log);
 }
 
-/* @} */
+/** @} */
 
 /**************************************************/ /**
 
@@ -1036,7 +1135,7 @@ void log_checkpointer(log_t *log_ptr) {
 
  *******************************************************/
 
-/* @{ */
+/** @{ */
 
 bool log_calc_concurrency_margin(log_t &log) {
   uint64_t concurrency_margin;
@@ -1247,6 +1346,6 @@ lsn_t log_get_max_modified_age_async(const log_t &log) {
                                OS_FILE_LOG_BLOCK_SIZE));
 }
 
-/* @} */
+/** @} */
 
 #endif /* !UNIV_HOTBACKUP */
