@@ -1,6 +1,6 @@
 /***********************************************************************
 
-Copyright (c) 2019, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2019, 2020, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -35,6 +35,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 #include "log0log.h"
 #include "mach0data.h"
 #include "os0file.h"
+#include "page0page.h"
 #include "ut0crc32.h"
 
 #include <errno.h>
@@ -46,11 +47,15 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 constexpr char Encryption::KEY_MAGIC_V1[];
 constexpr char Encryption::KEY_MAGIC_V2[];
 constexpr char Encryption::KEY_MAGIC_V3[];
+constexpr char Encryption::KEY_MAGIC_EMPTY[];
 constexpr char Encryption::MASTER_KEY_PREFIX[];
 constexpr char Encryption::DEFAULT_MASTER_KEY[];
 
+/** Minimum length needed for encryption */
+constexpr size_t MIN_ENCRYPTION_LEN = 2 * MY_AES_BLOCK_SIZE + FIL_PAGE_DATA;
+
 /** Current master key id */
-ulint Encryption::s_master_key_id = 0;
+uint32_t Encryption::s_master_key_id = Encryption::DEFAULT_MASTER_KEY_ID;
 
 /** Current uuid of server instance */
 char Encryption::s_uuid[Encryption::SERVER_UUID_LEN + 1] = {0};
@@ -87,7 +92,7 @@ void Encryption::create_master_key(byte **master_key) noexcept {
   }
 
   /* Generate new master key */
-  snprintf(key_name, MASTER_KEY_NAME_MAX_LEN, "%s-%s-" ULINTPF,
+  snprintf(key_name, MASTER_KEY_NAME_MAX_LEN, "%s-%s-" UINT32PF,
            MASTER_KEY_PREFIX, s_uuid, s_master_key_id + 1);
 
   /* We call key ring API to generate master key here. */
@@ -113,7 +118,7 @@ void Encryption::create_master_key(byte **master_key) noexcept {
 #endif /* !UNIV_HOTBACKUP */
 }
 
-void Encryption::get_master_key(ulint master_key_id, char *srv_uuid,
+void Encryption::get_master_key(uint32_t master_key_id, char *srv_uuid,
                                 byte **master_key) noexcept {
   size_t key_len = 0;
   char *key_type = nullptr;
@@ -124,13 +129,13 @@ void Encryption::get_master_key(ulint master_key_id, char *srv_uuid,
   if (srv_uuid != nullptr) {
     ut_ad(strlen(srv_uuid) > 0);
 
-    snprintf(key_name, MASTER_KEY_NAME_MAX_LEN, "%s-%s-" ULINTPF,
+    snprintf(key_name, MASTER_KEY_NAME_MAX_LEN, "%s-%s-" UINT32PF,
              MASTER_KEY_PREFIX, srv_uuid, master_key_id);
   } else {
     /* For compitable with 5.7.11, we need to get master key with
     server id. */
 
-    snprintf(key_name, MASTER_KEY_NAME_MAX_LEN, "%s-%lu-" ULINTPF,
+    snprintf(key_name, MASTER_KEY_NAME_MAX_LEN, "%s-%lu-" UINT32PF,
              MASTER_KEY_PREFIX, server_id, master_key_id);
   }
 
@@ -167,7 +172,7 @@ void Encryption::get_master_key(ulint master_key_id, char *srv_uuid,
 #endif /* UNIV_ENCRYPT_DEBUG */
 }
 
-void Encryption::get_master_key(ulint *master_key_id,
+void Encryption::get_master_key(uint32_t *master_key_id,
                                 byte **master_key) noexcept {
 #ifndef UNIV_HOTBACKUP
   int ret;
@@ -224,7 +229,7 @@ void Encryption::get_master_key(ulint *master_key_id,
   } else {
     *master_key_id = s_master_key_id;
 
-    snprintf(key_name, MASTER_KEY_NAME_MAX_LEN, "%s-%s-" ULINTPF,
+    snprintf(key_name, MASTER_KEY_NAME_MAX_LEN, "%s-%s-" UINT32PF,
              MASTER_KEY_PREFIX, s_uuid, *master_key_id);
 
     /* We call key ring API to get master key here. */
@@ -239,7 +244,7 @@ void Encryption::get_master_key(ulint *master_key_id,
         my_free(key_type);
       }
 
-      snprintf(key_name, MASTER_KEY_NAME_MAX_LEN, "%s-%lu-" ULINTPF,
+      snprintf(key_name, MASTER_KEY_NAME_MAX_LEN, "%s-%lu-" UINT32PF,
                MASTER_KEY_PREFIX, server_id, *master_key_id);
 
       ret = my_key_fetch(key_name, &key_type, nullptr,
@@ -278,7 +283,7 @@ void Encryption::get_master_key(ulint *master_key_id,
 bool Encryption::fill_encryption_info(byte *key, byte *iv, byte *encrypt_info,
                                       bool is_boot, bool encrypt_key) noexcept {
   byte *master_key = nullptr;
-  ulint master_key_id = 0;
+  uint32_t master_key_id = DEFAULT_MASTER_KEY_ID;
   bool is_default_master_key = false;
 
   /* Get master key from key ring. For bootstrap, we use a default
@@ -289,8 +294,6 @@ bool Encryption::fill_encryption_info(byte *key, byte *iv, byte *encrypt_info,
         || (strlen(server_uuid) == 0)
 #endif
     ) {
-      master_key_id = 0;
-
       master_key = static_cast<byte *>(ut_zalloc_nokey(KEY_LEN));
 
       ut_ad(KEY_LEN >= sizeof(DEFAULT_MASTER_KEY));
@@ -318,7 +321,7 @@ bool Encryption::fill_encryption_info(byte *key, byte *iv, byte *encrypt_info,
 
   /* Write master key id. */
   mach_write_to_4(ptr, master_key_id);
-  ptr += sizeof(uint32);
+  ptr += sizeof(uint32_t);
 
   /* Write server uuid. */
   memcpy(reinterpret_cast<char *>(ptr), s_uuid, sizeof(s_uuid));
@@ -455,16 +458,12 @@ bool Encryption::decode_encryption_info(byte *key, byte *iv,
                                         bool decrypt_key) noexcept {
   byte *ptr;
   byte *master_key = nullptr;
-  uint32 master_key_id = 0;
+  uint32_t master_key_id = DEFAULT_MASTER_KEY_ID;
   byte key_info[KEY_LEN * 2];
   ulint crc1;
   ulint crc2;
   char srv_uuid[SERVER_UUID_LEN + 1];
   Version version;
-#ifdef UNIV_ENCRYPT_DEBUG
-  const byte *data;
-  ulint i;
-#endif
 
   ptr = encryption_info;
 
@@ -476,14 +475,12 @@ bool Encryption::decode_encryption_info(byte *key, byte *iv,
     version = VERSION_2;
   } else if (memcmp(ptr, KEY_MAGIC_V3, MAGIC_SIZE) == 0) {
     version = VERSION_3;
-  } else {
+  } else if (memcmp(ptr, KEY_MAGIC_EMPTY, MAGIC_SIZE) == 0) {
     /* We don't report an error during recovery, since the
     encryption info maybe hasn't writen into datafile when
     the table is newly created. */
-    if (recv_recovery_is_on()) {
-      return (true);
-    }
-
+    return (recv_recovery_is_on() ? true : false);
+  } else {
     ib::error(ER_IB_MSG_837) << "Failed to decrypt encryption information,"
                              << " found unexpected version of it!";
     return (false);
@@ -505,10 +502,12 @@ bool Encryption::decode_encryption_info(byte *key, byte *iv,
     {
       std::ostringstream msg;
 
+      msg << "Master Key ID: " << master_key_id;
+      msg << " hex: {";
       ut_print_buf_hex(msg, master_key, KEY_LEN);
+      msg << "}";
 
-      ib::info(ER_IB_MSG_838)
-          << "Key ID: " << key_id << " hex: {" << msg.str() << "}";
+      ib::info(ER_IB_MSG_838) << msg.str();
     }
 #endif /* UNIV_ENCRYPT_DEBUG */
 
@@ -516,7 +515,7 @@ bool Encryption::decode_encryption_info(byte *key, byte *iv,
     auto len = my_aes_decrypt(ptr, sizeof(key_info), key_info, master_key,
                               KEY_LEN, my_aes_256_ecb, nullptr, false);
 
-    if (master_key_id == 0) {
+    if (master_key_id == DEFAULT_MASTER_KEY_ID) {
       ut_free(master_key);
     } else {
       my_free(master_key);
@@ -611,11 +610,12 @@ bool Encryption::encrypt_log_block(const IORequest &type, byte *src_ptr,
   {
     std::ostringstream msg;
 
+    msg << "Encrypting block: " << log_block_get_hdr_no(src_ptr);
+    msg << "{";
     ut_print_buf_hex(msg, src_ptr, OS_FILE_LOG_BLOCK_SIZE);
+    msg << "}";
 
-    ib::info(ER_IB_MSG_842)
-        << "Encrypting block: " << log_block_get_hdr_no(src_ptr) << "{"
-        << msg.str() << "}";
+    ib::info(ER_IB_MSG_842) << msg.str();
   }
 #endif /* UNIV_ENCRYPT_DEBUG */
 
@@ -685,26 +685,35 @@ bool Encryption::encrypt_log_block(const IORequest &type, byte *src_ptr,
   }
 
 #ifdef UNIV_ENCRYPT_DEBUG
-  fprintf(stderr, "Encrypted block %lu.\n", log_block_get_hdr_no(dst_ptr));
-  ut_print_buf_hex(stderr, dst_ptr, OS_FILE_LOG_BLOCK_SIZE);
-  fprintf(stderr, "\n");
+  {
+    std::ostringstream os{};
+    os << "Encrypted block " << log_block_get_hdr_no(dst_ptr) << "."
+       << std::endl;
+    ut_print_buf_hex(os, dst_ptr, OS_FILE_LOG_BLOCK_SIZE);
+    os << std::endl;
+    ib::info() << os.str();
 
-  byte *check_buf =
-      static_cast<byte *>(ut_malloc_nokey(OS_FILE_LOG_BLOCK_SIZE));
-  byte *buf2 = static_cast<byte *>(ut_malloc_nokey(OS_FILE_LOG_BLOCK_SIZE));
+    byte *check_buf =
+        static_cast<byte *>(ut_malloc_nokey(OS_FILE_LOG_BLOCK_SIZE));
+    byte *buf2 = static_cast<byte *>(ut_malloc_nokey(OS_FILE_LOG_BLOCK_SIZE));
 
-  memcpy(check_buf, dst_ptr, OS_FILE_LOG_BLOCK_SIZE);
-  dberr_t err = decrypt_log(type, check_buf, OS_FILE_LOG_BLOCK_SIZE, buf2,
-                            OS_FILE_LOG_BLOCK_SIZE);
-  log_block_set_encrypt_bit(check_buf, true);
-  if (err != DB_SUCCESS ||
-      memcmp(src_ptr, check_buf, OS_FILE_LOG_BLOCK_SIZE) != 0) {
-    ut_print_buf_hex(stderr, src_ptr, OS_FILE_LOG_BLOCK_SIZE);
-    ut_print_buf_hex(stderr, check_buf, OS_FILE_LOG_BLOCK_SIZE);
-    ut_ad(0);
+    memcpy(check_buf, dst_ptr, OS_FILE_LOG_BLOCK_SIZE);
+    log_block_set_encrypt_bit(check_buf, true);
+    dberr_t err = decrypt_log(type, check_buf, OS_FILE_LOG_BLOCK_SIZE, buf2,
+                              OS_FILE_LOG_BLOCK_SIZE);
+    if (err != DB_SUCCESS ||
+        memcmp(src_ptr, check_buf, OS_FILE_LOG_BLOCK_SIZE) != 0) {
+      std::ostringstream msg{};
+      ut_print_buf_hex(msg, src_ptr, OS_FILE_LOG_BLOCK_SIZE);
+      ib::error() << msg.str();
+
+      msg.seekp(0);
+      ut_print_buf_hex(msg, check_buf, OS_FILE_LOG_BLOCK_SIZE);
+      ib::fatal() << msg.str();
+    }
+    ut_free(buf2);
+    ut_free(check_buf);
   }
-  ut_free(buf2);
-  ut_free(check_buf);
 #endif /* UNIV_ENCRYPT_DEBUG */
 
   /* Set the encrypted flag. */
@@ -736,57 +745,50 @@ byte *Encryption::encrypt_log(const IORequest &type, byte *src, ulint src_len,
   }
 
 #ifdef UNIV_ENCRYPT_DEBUG
-  byte *check_buf = static_cast<byte *>(ut_malloc_nokey(src_len));
-  byte *buf2 = static_cast<byte *>(ut_malloc_nokey(src_len));
+  {
+    byte *check_buf = static_cast<byte *>(ut_malloc_nokey(src_len));
+    byte *buf2 = static_cast<byte *>(ut_malloc_nokey(src_len));
 
-  memcpy(check_buf, dst, src_len);
+    memcpy(check_buf, dst, src_len);
 
-  dberr_t err = decrypt_log(type, check_buf, src_len, buf2, src_len);
-  if (err != DB_SUCCESS || memcmp(src, check_buf, src_len) != 0) {
-    ut_print_buf_hex(stderr, src, src_len);
-    ut_print_buf_hex(stderr, check_buf, src_len);
-    ut_ad(0);
+    dberr_t err = decrypt_log(type, check_buf, src_len, buf2, src_len);
+    if (err != DB_SUCCESS || memcmp(src, check_buf, src_len) != 0) {
+      std::ostringstream msg{};
+      ut_print_buf_hex(msg, src, src_len);
+      ib::error() << msg.str();
+
+      msg.seekp(0);
+      ut_print_buf_hex(msg, check_buf, src_len);
+      ib::fatal() << msg.str();
+    }
+    ut_free(buf2);
+    ut_free(check_buf);
   }
-  ut_free(buf2);
-  ut_free(check_buf);
 #endif /* UNIV_ENCRYPT_DEBUG */
 
   return (dst);
 }
 
-byte *Encryption::encrypt(const IORequest &type, byte *src, ulint src_len,
-                          byte *dst, ulint *dst_len) noexcept {
-  ulint len = 0;
-  ulint page_type = mach_read_from_2(src + FIL_PAGE_TYPE);
-  ulint data_len;
-  ulint main_len;
-  ulint remain_len;
-  byte remain_buf[MY_AES_BLOCK_SIZE * 2];
+bool Encryption::encrypt_low(byte *src, ulint src_len, byte *dst,
+                             ulint *dst_len) noexcept {
+  const uint16_t page_type = mach_read_from_2(src + FIL_PAGE_TYPE);
 
-  /* For encrypting redo log, take another way. */
-  ut_ad(!type.is_log());
-
-#ifdef UNIV_ENCRYPT_DEBUG
-  ulint space_id = mach_read_from_4(src + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID);
-  ulint page_no = mach_read_from_4(src + FIL_PAGE_OFFSET);
-
-  fprintf(stderr, "Encrypting page:%lu.%lu len:%lu\n", space_id, page_no,
-          src_len);
-  ut_print_buf(stderr, m_key, 32);
-  ut_print_buf(stderr, m_iv, 32);
-#endif /* UNIV_ENCRYPT_DEBUG */
-
-  /* Shouldn't encrypte an already encrypted page. */
-  ut_ad(page_type != FIL_PAGE_ENCRYPTED &&
-        page_type != FIL_PAGE_COMPRESSED_AND_ENCRYPTED &&
-        page_type != FIL_PAGE_ENCRYPTED_RTREE);
-
-  ut_ad(m_type != NONE);
+  /* Shouldn't encrypt an already encrypted page. */
+  ut_ad(!is_encrypted_page(src));
 
   /* This is data size which need to encrypt. */
-  data_len = src_len - FIL_PAGE_DATA;
-  main_len = (data_len / MY_AES_BLOCK_SIZE) * MY_AES_BLOCK_SIZE;
-  remain_len = data_len - main_len;
+  auto src_enc_len = src_len;
+
+  /* In FIL_PAGE_VERSION_2, we encrypt the actual compressed data length. */
+  if (page_type == FIL_PAGE_COMPRESSED) {
+    src_enc_len =
+        mach_read_from_2(src + FIL_PAGE_COMPRESS_SIZE_V1) + FIL_PAGE_DATA;
+    /* Extend src_enc_len if needed */
+    if (src_enc_len < MIN_ENCRYPTION_LEN) {
+      src_enc_len = MIN_ENCRYPTION_LEN;
+    }
+    ut_a(src_enc_len <= src_len);
+  }
 
   /* Only encrypt the data + trailer, leave the header alone */
 
@@ -795,58 +797,65 @@ byte *Encryption::encrypt(const IORequest &type, byte *src, ulint src_len,
       ut_error;
 
     case AES: {
-      lint elen;
-
       ut_ad(m_klen == KEY_LEN);
 
-      elen = my_aes_encrypt(src + FIL_PAGE_DATA, static_cast<uint32>(main_len),
-                            dst + FIL_PAGE_DATA,
-                            reinterpret_cast<unsigned char *>(m_key),
-                            static_cast<uint32>(m_klen), my_aes_256_cbc,
-                            reinterpret_cast<unsigned char *>(m_iv), false);
+      /* Total length of the data to encrypt. */
+      const auto data_len = src_enc_len - FIL_PAGE_DATA;
+
+      /* Server encryption functions expect input data to be in multiples
+      of MY_AES_BLOCK SIZE. Therefore we encrypt the overlapping data of
+      the chunk_len and trailer_len twice. First we encrypt the bigger
+      chunk of data then we do the trailer. The trailer encryption block
+      starts at 2 * MY_AES_BLOCK_SIZE bytes offset from the end of the
+      enc_len.  During decryption we do the reverse of the above process. */
+      ut_ad(data_len >= 2 * MY_AES_BLOCK_SIZE);
+
+      const auto chunk_len = (data_len / MY_AES_BLOCK_SIZE) * MY_AES_BLOCK_SIZE;
+      const auto remain_len = data_len - chunk_len;
+
+      auto elen =
+          my_aes_encrypt(src + FIL_PAGE_DATA, static_cast<uint32>(chunk_len),
+                         dst + FIL_PAGE_DATA, reinterpret_cast<byte *>(m_key),
+                         static_cast<uint32>(m_klen), my_aes_256_cbc,
+                         reinterpret_cast<byte *>(m_iv), false);
 
       if (elen == MY_AES_BAD_DATA) {
-        ulint page_no = mach_read_from_4(src + FIL_PAGE_OFFSET);
-        ulint space_id =
-            mach_read_from_4(src + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID);
+        const auto page_id = page_get_page_id(src);
+
+        ib::error(ER_IB_MSG_844) << " Can't encrypt data of page " << page_id;
         *dst_len = src_len;
-        ib::error(ER_IB_MSG_844)
-            << " Can't encrypt data of page,"
-            << " page no:" << page_no << " space id:" << space_id;
-        return (src);
+        return false;
       }
 
-      len = static_cast<ulint>(elen);
-      ut_ad(len == main_len);
+      const auto len = static_cast<size_t>(elen);
+      ut_a(len == chunk_len);
 
-      /* Copy remain bytes and page tailer. */
-      memcpy(dst + FIL_PAGE_DATA + len, src + FIL_PAGE_DATA + len,
-             src_len - FIL_PAGE_DATA - len);
-
-      /* Encrypt the remain bytes. */
+      /* Encrypt the trailing bytes. */
       if (remain_len != 0) {
-        remain_len = MY_AES_BLOCK_SIZE * 2;
+        /* Copy remaining bytes and page tailer. */
+        memcpy(dst + FIL_PAGE_DATA + len, src + FIL_PAGE_DATA + len,
+               remain_len);
 
-        elen = my_aes_encrypt(dst + FIL_PAGE_DATA + data_len - remain_len,
-                              static_cast<uint32>(remain_len), remain_buf,
-                              reinterpret_cast<unsigned char *>(m_key),
+        constexpr size_t trailer_len = MY_AES_BLOCK_SIZE * 2;
+        byte buf[trailer_len];
+
+        elen = my_aes_encrypt(dst + FIL_PAGE_DATA + data_len - trailer_len,
+                              static_cast<uint32>(trailer_len), buf,
+                              reinterpret_cast<byte *>(m_key),
                               static_cast<uint32>(m_klen), my_aes_256_cbc,
-                              reinterpret_cast<unsigned char *>(m_iv), false);
+                              reinterpret_cast<byte *>(m_iv), false);
 
         if (elen == MY_AES_BAD_DATA) {
-          ulint page_no = mach_read_from_4(src + FIL_PAGE_OFFSET);
-          ulint space_id =
-              mach_read_from_4(src + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID);
+          const auto page_id = page_get_page_id(src);
 
-          ib::error(ER_IB_MSG_845)
-              << " Can't encrypt data of page,"
-              << " page no:" << page_no << " space id:" << space_id;
+          ib::error(ER_IB_MSG_845) << " Can't encrypt data of page," << page_id;
           *dst_len = src_len;
-          return (src);
+          return false;
         }
 
-        memcpy(dst + FIL_PAGE_DATA + data_len - remain_len, remain_buf,
-               remain_len);
+        ut_a(static_cast<size_t>(elen) == trailer_len);
+
+        memcpy(dst + FIL_PAGE_DATA + data_len - trailer_len, buf, trailer_len);
       }
 
       break;
@@ -868,37 +877,68 @@ byte *Encryption::encrypt(const IORequest &type, byte *src, ulint src_len,
     ut_ad(memcmp(src + FIL_PAGE_TYPE + 2, dst + FIL_PAGE_TYPE + 2,
                  FIL_PAGE_DATA - FIL_PAGE_TYPE - 2) == 0);
   } else if (page_type == FIL_PAGE_RTREE) {
-    /* If the page is R-tree page, we need to save original
-    type. */
+    /* If the page is R-tree page, we need to save original type. */
     mach_write_to_2(dst + FIL_PAGE_TYPE, FIL_PAGE_ENCRYPTED_RTREE);
   } else {
     mach_write_to_2(dst + FIL_PAGE_TYPE, FIL_PAGE_ENCRYPTED);
     mach_write_to_2(dst + FIL_PAGE_ORIGINAL_TYPE_V1, page_type);
   }
 
-#ifdef UNIV_ENCRYPT_DEBUG
-  byte *check_buf = static_cast<byte *>(ut_malloc_nokey(src_len));
-  byte *buf2 = static_cast<byte *>(ut_malloc_nokey(src_len));
-
-  memcpy(check_buf, dst, src_len);
-
-  dberr_t err = decrypt(type, check_buf, src_len, buf2, src_len);
-  if (err != DB_SUCCESS ||
-      memcmp(src + FIL_PAGE_DATA, check_buf + FIL_PAGE_DATA,
-             src_len - FIL_PAGE_DATA) != 0) {
-    ut_print_buf(stderr, src, src_len);
-    ut_print_buf(stderr, check_buf, src_len);
-    ut_ad(0);
+  /* Add padding 0 for unused portion */
+  if (src_len > src_enc_len) {
+    memset(dst + src_enc_len, 0, src_len - src_enc_len);
   }
-  ut_free(buf2);
-  ut_free(check_buf);
-
-  fprintf(stderr, "Encrypted page:%lu.%lu\n", space_id, page_no);
-#endif /* UNIV_ENCRYPT_DEBUG */
 
   *dst_len = src_len;
 
-  return (dst);
+  return (true);
+}
+
+byte *Encryption::encrypt(const IORequest &type, byte *src, ulint src_len,
+                          byte *dst, ulint *dst_len) noexcept {
+  /* For encrypting redo log, take another way. */
+  ut_ad(!type.is_log());
+
+#ifdef UNIV_ENCRYPT_DEBUG
+  {
+    ulint space_id = mach_read_from_4(src + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID);
+    ulint page_no = mach_read_from_4(src + FIL_PAGE_OFFSET);
+
+    fprintf(stderr, "Encrypting page:%lu.%lu len:%lu\n", space_id, page_no,
+            src_len);
+    ut_print_buf(stderr, m_key, 32);
+    ut_print_buf(stderr, m_iv, 32);
+  }
+#endif /* UNIV_ENCRYPT_DEBUG */
+
+  ut_ad(m_type != NONE);
+
+  if (!encrypt_low(src, src_len, dst, dst_len)) {
+    return (src);
+  }
+
+#ifdef UNIV_ENCRYPT_DEBUG
+  {
+    byte *check_buf = static_cast<byte *>(ut_malloc_nokey(src_len));
+    byte *buf2 = static_cast<byte *>(ut_malloc_nokey(src_len));
+
+    memcpy(check_buf, dst, src_len);
+
+    dberr_t err = decrypt(type, check_buf, src_len, buf2, src_len);
+    if (err != DB_SUCCESS ||
+        memcmp(src + FIL_PAGE_DATA, check_buf + FIL_PAGE_DATA,
+               src_len - FIL_PAGE_DATA) != 0) {
+      ut_print_buf(stderr, src, src_len);
+      ut_print_buf(stderr, check_buf, src_len);
+      ut_ad(0);
+    }
+    ut_free(buf2);
+    ut_free(check_buf);
+
+    fprintf(stderr, "Encrypted page:%lu.%lu\n", space_id, page_no);
+  }
+#endif /* UNIV_ENCRYPT_DEBUG */
+  return dst;
 }
 
 dberr_t Encryption::decrypt_log_block(const IORequest &type, byte *src,
@@ -973,9 +1013,13 @@ dberr_t Encryption::decrypt_log_block(const IORequest &type, byte *src,
   ptr -= LOG_BLOCK_HDR_SIZE;
 
 #ifdef UNIV_ENCRYPT_DEBUG
-  fprintf(stderr, "Decrypted block %lu.\n", log_block_get_hdr_no(ptr));
-  ut_print_buf_hex(stderr, ptr, OS_FILE_LOG_BLOCK_SIZE);
-  fprintf(stderr, "\n");
+  {
+    std::ostringstream msg{};
+    msg << "Decrypted block " << log_block_get_hdr_no(ptr) << "." << std::endl;
+    ut_print_buf_hex(msg, ptr, OS_FILE_LOG_BLOCK_SIZE);
+    msg << std::endl;
+    ib::info() << msg.str();
+  }
 #endif
 
   /* Reset the encrypted flag. */
@@ -1007,13 +1051,12 @@ dberr_t Encryption::decrypt_log(const IORequest &type, byte *src, ulint src_len,
     {
       std::ostringstream msg;
 
+      msg << "Decrypting block: " << log_block_get_hdr_no(ptr) << std::endl;
+      msg << "data={" << std::endl;
       ut_print_buf_hex(msg, ptr, OS_FILE_LOG_BLOCK_SIZE);
+      msg << std::endl << "}";
 
-      ib::info(ER_IB_MSG_847)
-          << "Decrypting block: " << log_block_get_hdr_no(ptr) << std::endl
-          << "data={" << std::endl
-          << msg.str << std::endl
-          << "}";
+      ib::info(ER_IB_MSG_847) << msg.str();
     }
 #endif /* UNIV_ENCRYPT_DEBUG */
 
@@ -1049,42 +1092,58 @@ dberr_t Encryption::decrypt(const IORequest &type, byte *src, ulint src_len,
   ulint main_len;
   ulint remain_len;
   ulint original_type;
-  ulint page_type;
   byte remain_buf[MY_AES_BLOCK_SIZE * 2];
   file::Block *block;
 
+  /* If the page is encrypted, then we need key to decrypt it. */
+  if (is_encrypted_page(src) && m_type == NONE) {
+    return DB_IO_DECRYPT_FAIL;
+  }
+
   if (!is_encrypted_page(src) || m_type == NONE) {
     /* There is nothing we can do. */
-    return (DB_SUCCESS);
+    return DB_SUCCESS;
   }
 
   /* For compressed page, we need to get the compressed size
   for decryption */
-  page_type = mach_read_from_2(src + FIL_PAGE_TYPE);
+  const ulint page_type = mach_read_from_2(src + FIL_PAGE_TYPE);
+
+  /* Actual length of the compressed data */
+  uint16_t z_len = 0;
+
   if (page_type == FIL_PAGE_COMPRESSED_AND_ENCRYPTED) {
-    src_len = static_cast<uint16_t>(
-                  mach_read_from_2(src + FIL_PAGE_COMPRESS_SIZE_V1)) +
-              FIL_PAGE_DATA;
-    src_len = ut_calc_align(src_len, type.block_size());
+    z_len = mach_read_from_2(src + FIL_PAGE_COMPRESS_SIZE_V1);
+    src_len = z_len + FIL_PAGE_DATA;
+
+    Compression::meta_t header;
+    Compression::deserialize_header(src, &header);
+    if (header.m_version == Compression::FIL_PAGE_VERSION_1) {
+      src_len = ut_calc_align(src_len, type.block_size());
+    } else {
+      /* Extend src_len if needed */
+      if (src_len < MIN_ENCRYPTION_LEN) {
+        src_len = MIN_ENCRYPTION_LEN;
+      }
+    }
   }
 
 #ifdef UNIV_ENCRYPT_DEBUG
+  const page_id_t page_id(
+      mach_read_from_4(src + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID),
+      mach_read_from_4(src + FIL_PAGE_OFFSET));
+
   {
-    auto space_id = mach_read_from_4(src + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID);
-
-    auto page_no = mach_read_from_4(src + FIL_PAGE_OFFSET);
-
     std::ostringstream msg;
 
+    msg << "Decrypting page: " << page_id << " len: " << src_len << std::endl;
     msg << "key={";
     ut_print_buf(msg, m_key, 32);
     msg << "}" << std::endl << "iv= {";
     ut_print_buf(msg, m_iv, 32);
     msg << "}";
 
-    ib::info(ER_IB_MSG_848) << "Decrypting page: " << space_id << "." << page_no
-                            << " len: " << src_len << "\n"
-                            << msg.str();
+    ib::info(ER_IB_MSG_848) << msg.str();
   }
 #endif /* UNIV_ENCRYPT_DEBUG */
 
@@ -1124,6 +1183,7 @@ dberr_t Encryption::decrypt(const IORequest &type, byte *src, ulint src_len,
                               reinterpret_cast<unsigned char *>(m_key),
                               static_cast<uint32>(m_klen), my_aes_256_cbc,
                               reinterpret_cast<unsigned char *>(m_iv), false);
+
         if (elen == MY_AES_BAD_DATA) {
           if (block != nullptr) {
             os_free_block(block);
@@ -1132,6 +1192,8 @@ dberr_t Encryption::decrypt(const IORequest &type, byte *src, ulint src_len,
           return (DB_IO_DECRYPT_FAIL);
         }
 
+        ut_ad(static_cast<ulint>(elen) == remain_len);
+
         /* Copy the other data bytes to temp area. */
         memcpy(dst, ptr, data_len - remain_len);
       } else {
@@ -1193,8 +1255,21 @@ dberr_t Encryption::decrypt(const IORequest &type, byte *src, ulint src_len,
     os_free_block(block);
   }
 
+#ifdef UNIV_DEBUG
+  {
+    /* Check if all the padding bytes are zeroes. */
+    if (page_type == FIL_PAGE_COMPRESSED_AND_ENCRYPTED) {
+      uint32_t padding = src_len - FIL_PAGE_DATA - z_len;
+      for (uint32_t i = 0; i < padding; ++i) {
+        byte *pad = src + z_len + FIL_PAGE_DATA + i;
+        ut_ad(*pad == 0x0);
+      }
+    }
+  }
+#endif /* UNIV_DEBUG */
+
 #ifdef UNIV_ENCRYPT_DEBUG
-  ib::info(ER_IB_MSG_850) << "Decrypted page: " << space_id << "." << page_no;
+  ib::info(ER_IB_MSG_850) << "Decrypted page: " << page_id;
 #endif /* UNIV_ENCRYPT_DEBUG */
 
   DBUG_EXECUTE_IF("ib_crash_during_decrypt_page", DBUG_SUICIDE(););
@@ -1204,37 +1279,57 @@ dberr_t Encryption::decrypt(const IORequest &type, byte *src, ulint src_len,
 
 #ifndef UNIV_HOTBACKUP
 bool Encryption::check_keyring() noexcept {
-  size_t key_len;
   bool ret = false;
-  char *key_type = nullptr;
-  char *master_key = nullptr;
-  char key_name[MASTER_KEY_NAME_MAX_LEN];
+  byte *master_key = nullptr;
 
-  key_name[sizeof(DEFAULT_MASTER_KEY)] = 0;
+  /* During bootstrap, default master key is used */
+  if (s_master_key_id == DEFAULT_MASTER_KEY_ID) {
+    static bool checked = false;
+    if (checked) {
+      return true;
+    }
 
-  strncpy(key_name, DEFAULT_MASTER_KEY, sizeof(key_name));
+    /* Generate/Fetch/Delete a dummy master key to confirm keyring is up and
+    running. */
+    size_t key_len;
+    char *key_type = nullptr;
+    char key_name[MASTER_KEY_NAME_MAX_LEN];
 
-  /* We call key ring API to generate master key here. */
-  int my_ret = my_key_generate(key_name, "AES", nullptr, KEY_LEN);
+    key_name[sizeof(DEFAULT_MASTER_KEY)] = 0;
 
-  /* We call key ring API to get master key here. */
-  my_ret = my_key_fetch(key_name, &key_type, nullptr,
-                        reinterpret_cast<void **>(&master_key), &key_len);
+    strncpy(key_name, DEFAULT_MASTER_KEY, sizeof(key_name));
 
-  if (my_ret != 0) {
-    ib::error(ER_IB_MSG_851) << "Check keyring plugin fail, please check the"
-                             << " keyring plugin is loaded.";
+    /* We call key ring API to generate master key here. */
+    int my_ret = my_key_generate(key_name, "AES", nullptr, KEY_LEN);
+
+    /* We call key ring API to get master key here. */
+    my_ret = my_key_fetch(key_name, &key_type, nullptr,
+                          reinterpret_cast<void **>(&master_key), &key_len);
+
+    if (my_ret != 0) {
+      ib::error(ER_IB_MSG_851) << "Check keyring plugin fail, please check the"
+                               << " keyring plugin is loaded.";
+    } else {
+      my_key_remove(key_name, nullptr);
+      ret = true;
+      checked = true;
+    }
+
+    if (key_type != nullptr) {
+      my_free(key_type);
+    }
+
+    if (master_key != nullptr) {
+      my_free(master_key);
+    }
   } else {
-    my_key_remove(key_name, nullptr);
-    ret = true;
-  }
+    uint32_t master_key_id;
 
-  if (key_type != nullptr) {
-    my_free(key_type);
-  }
-
-  if (master_key != nullptr) {
-    my_free(master_key);
+    Encryption::get_master_key(&master_key_id, &master_key);
+    if (master_key != nullptr) {
+      my_free(master_key);
+      ret = true;
+    }
   }
 
   return (ret);
@@ -1257,4 +1352,4 @@ byte *Encryption::get_initial_vector() const { return m_iv; }
 
 void Encryption::set_initial_vector(byte *iv) { m_iv = iv; }
 
-ulint Encryption::get_master_key_id() { return s_master_key_id; }
+uint32_t Encryption::get_master_key_id() { return s_master_key_id; }
