@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2014, 2020, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2012, 2020, Oracle and/or its affiliates.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -138,7 +138,7 @@ struct LatchDebug {
   @return	pointer to a thread's acquired latches. */
   Latches *thread_latches(bool add = false) UNIV_NOTHROW;
 
-  /** Check that all the latches already owned by a thread have a lower
+  /** Check that all the latches already owned by a thread have a higher
   level than limit.
   @param[in]	latches		the thread's existing (acquired) latches
   @param[in]	limit		to check against
@@ -153,10 +153,9 @@ struct LatchDebug {
   const latch_t *find(const Latches *latches,
                       latch_level_t level) const UNIV_NOTHROW;
 
-  /**
-  Checks if the level value exists in the thread's acquired latches.
-  @param[in]	level		to lookup
-  @return	latch if found or 0 */
+  /** Checks if the level value exists in the thread's acquired latches.
+  @param[in]	level		The level to lookup
+  @return	latch if found or NULL */
   const latch_t *find(latch_level_t level) UNIV_NOTHROW;
 
   /** Report error and abort.
@@ -168,13 +167,12 @@ struct LatchDebug {
   void crash(const Latches *latches, const Latched *latched,
              latch_level_t level) const UNIV_NOTHROW;
 
-  /** Do a basic ordering check.
+  /** Do a basic ordering check. Asserts that all the existing latches have a
+  level higher than the in_level.
   @param[in]	latches		thread's existing latches
   @param[in]	requested_level	Level requested by latch
-  @param[in]	in_level	declared ulint so that we can
-                                  do level - 1. The level of the
-                                  latch that the thread is trying
-                                  to acquire
+  @param[in]	in_level	The level of the latch that the thread is trying
+  to acquire. Declared ulint so that we can do level - 1.
   @return true if passes, else crash with error message. */
   bool basic_check(const Latches *latches, latch_level_t requested_level,
                    ulint in_level) const UNIV_NOTHROW;
@@ -202,7 +200,11 @@ struct LatchDebug {
         latch_level_t back_latch_level = latches->back().m_latch->get_level();
         latch_level_t back_level = latches->back().m_latch->get_level();
 
+#ifdef UNIV_NO_ERR_MSGS
         ib::error()
+#else
+        ib::error(ER_IB_LOCK_VALIDATE_LATCH_ORDER_VIOLATION)
+#endif
             << "LatchDebug::lock_validate() latch order violation. level="
             << level << ", back_latch_level=" << back_latch_level
             << ", back_level=" << back_level << ".";
@@ -251,9 +253,14 @@ struct LatchDebug {
         latch_level_t back_latch_level = latches->back().m_latch->get_level();
         latch_level_t back_level = latches->back().m_latch->get_level();
 
-        ib::error() << "LatchDebug::relock() latch order violation. level="
-                    << level << ", back_latch_level=" << back_latch_level
-                    << ", back_level=" << back_level << ".";
+#ifdef UNIV_NO_ERR_MSGS
+        ib::error()
+#else
+        ib::error(ER_IB_RELOCK_LATCH_ORDER_VIOLATION)
+#endif
+            << "LatchDebug::relock() latch order violation. level=" << level
+            << ", back_latch_level=" << back_latch_level
+            << ", back_level=" << back_level << ".";
         ut_error;
       }
 
@@ -287,8 +294,7 @@ struct LatchDebug {
   }
 
   /** Removes a latch from the thread level array if it is found there.
-  @param[in]	latch		The latch that was released
-  */
+  @param[in]	latch		The latch that was released */
   void unlock(const latch_t *latch) UNIV_NOTHROW;
 
   /** Get the level name
@@ -427,6 +433,7 @@ LatchDebug::LatchDebug() {
   LEVEL_MAP_INSERT(SYNC_RECV);
   LEVEL_MAP_INSERT(SYNC_RECV_WRITER);
   LEVEL_MAP_INSERT(SYNC_LOG_SN);
+  LEVEL_MAP_INSERT(SYNC_LOG_SN_MUTEX);
   LEVEL_MAP_INSERT(SYNC_LOG_LIMITS);
   LEVEL_MAP_INSERT(SYNC_LOG_WRITER);
   LEVEL_MAP_INSERT(SYNC_LOG_WRITE_NOTIFIER);
@@ -441,11 +448,11 @@ LatchDebug::LatchDebug() {
   LEVEL_MAP_INSERT(SYNC_PAGE_CLEANER);
   LEVEL_MAP_INSERT(SYNC_PURGE_QUEUE);
   LEVEL_MAP_INSERT(SYNC_TRX_SYS_HEADER);
-  LEVEL_MAP_INSERT(SYNC_REC_LOCK);
   LEVEL_MAP_INSERT(SYNC_THREADS);
   LEVEL_MAP_INSERT(SYNC_TRX);
   LEVEL_MAP_INSERT(SYNC_TRX_SYS);
-  LEVEL_MAP_INSERT(SYNC_LOCK_SYS);
+  LEVEL_MAP_INSERT(SYNC_LOCK_SYS_GLOBAL);
+  LEVEL_MAP_INSERT(SYNC_LOCK_SYS_SHARDED);
   LEVEL_MAP_INSERT(SYNC_LOCK_WAIT_SYS);
   LEVEL_MAP_INSERT(SYNC_INDEX_ONLINE_LOG);
   LEVEL_MAP_INSERT(SYNC_IBUF_BITMAP);
@@ -546,11 +553,6 @@ void LatchDebug::crash(const Latches *latches, const Latched *latched,
   ut_error;
 }
 
-/** Check that all the latches already owned by a thread have a lower
-level than limit.
-@param[in]	latches		the thread's existing (acquired) latches
-@param[in]	limit		to check against
-@return latched info if there is one with a level <= limit . */
 const Latched *LatchDebug::less(const Latches *latches,
                                 latch_level_t limit) const UNIV_NOTHROW {
   Latches::const_iterator end = latches->end();
@@ -564,12 +566,12 @@ const Latched *LatchDebug::less(const Latches *latches,
   return (nullptr);
 }
 
-/** Do a basic ordering check.
+/** Do a basic ordering check. Asserts that all the existing latches have a
+level higher than the in_level.
 @param[in]	latches		thread's existing latches
 @param[in]	requested_level	Level requested by latch
-@param[in]	in_level	declared ulint so that we can do level - 1.
-                                The level of the latch that the thread is
-                                trying to acquire
+@param[in]	in_level	The level of the latch that the thread is trying
+to acquire. Declared ulint so that we can do level - 1.
 @return true if passes, else crash with error message. */
 bool LatchDebug::basic_check(const Latches *latches,
                              latch_level_t requested_level,
@@ -645,7 +647,7 @@ const latch_t *LatchDebug::find(const Latches *latches,
 }
 
 /** Checks if the level value exists in the thread's acquired latches.
-@param[in]	 level		The level to lookup
+@param[in]	level		The level to lookup
 @return	latch if found or NULL */
 const latch_t *LatchDebug::find(latch_level_t level) UNIV_NOTHROW {
   return (find(thread_latches(), level));
@@ -677,6 +679,7 @@ Latches *LatchDebug::check_order(const latch_t *latch,
       break;
 
     case SYNC_LOG_SN:
+    case SYNC_LOG_SN_MUTEX:
     case SYNC_TRX_SYS_HEADER:
     case SYNC_LOCK_FREE_HASH:
     case SYNC_MONITOR_MUTEX:
@@ -702,7 +705,7 @@ Latches *LatchDebug::check_order(const latch_t *latch,
     case SYNC_PAGE_ARCH_CLIENT:
     case SYNC_SEARCH_SYS:
     case SYNC_THREADS:
-    case SYNC_LOCK_SYS:
+    case SYNC_LOCK_SYS_GLOBAL:
     case SYNC_LOCK_WAIT_SYS:
     case SYNC_TRX_SYS:
     case SYNC_IBUF_BITMAP_MUTEX:
@@ -755,12 +758,13 @@ Latches *LatchDebug::check_order(const latch_t *latch,
 
     case SYNC_TRX:
 
-      /* Either the thread must own the lock_sys->mutex, or
-      it is allowed to own only ONE trx_t::mutex. */
+      /* Either the thread must own the lock_sys global latch, or
+      it is allowed to own only ONE trx_t::mutex. There are additional rules
+      for holding more than one trx_t::mutex @see trx_before_mutex_enter(). */
 
       if (less(latches, level) != nullptr) {
         basic_check(latches, level, level - 1);
-        ut_a(find(latches, SYNC_LOCK_SYS) != nullptr);
+        ut_a(find(latches, SYNC_LOCK_SYS_GLOBAL) != nullptr);
       }
       break;
 
@@ -774,6 +778,9 @@ Latches *LatchDebug::check_order(const latch_t *latch,
     case SYNC_BUF_ZIP_HASH:
     case SYNC_BUF_FLUSH_STATE:
     case SYNC_RSEG_ARRAY_HEADER:
+    case SYNC_LOCK_SYS_SHARDED:
+    case SYNC_BUF_PAGE_HASH:
+    case SYNC_BUF_BLOCK:
 
       /* We can have multiple mutexes of this type therefore we
       can only check whether the greater than condition holds. */
@@ -781,24 +788,6 @@ Latches *LatchDebug::check_order(const latch_t *latch,
       basic_check(latches, level, level - 1);
       break;
 
-    case SYNC_BUF_PAGE_HASH:
-      /* Fall through */
-    case SYNC_BUF_BLOCK:
-
-      if (less(latches, level) != nullptr) {
-        basic_check(latches, level, level - 1);
-      }
-      break;
-
-    case SYNC_REC_LOCK:
-
-      if (find(latches, SYNC_LOCK_SYS) != nullptr) {
-        basic_check(latches, level, SYNC_REC_LOCK - 1);
-      } else {
-        basic_check(latches, level, SYNC_REC_LOCK);
-      }
-      break;
-
     case SYNC_IBUF_BITMAP:
 
       /* Either the thread must own the master mutex to all
@@ -939,10 +928,7 @@ Latches *LatchDebug::check_order(const latch_t *latch,
 }
 
 /** Removes a latch from the thread level array if it is found there.
-@param[in]	latch		that was released/unlocked
-@return true if found in the array; it is not an error if the latch is
-not found, as we presently are not able to determine the level for
-every latch reservation the program does */
+@param[in]	latch		The latch that was released */
 void LatchDebug::unlock(const latch_t *latch) UNIV_NOTHROW {
   if (latch->get_level() == SYNC_LEVEL_VARYING) {
     // We don't have varying level mutexes
@@ -1028,7 +1014,7 @@ latch_id_t sync_latch_get_id(const char *name) {
 
 /** Get the latch name from a sync level
 @param[in]	level		Latch level to lookup
-@return NULL if not found. */
+@return nullptr if not found. */
 const char *sync_latch_get_name(latch_level_t level) {
   LatchMetaData::const_iterator end = latch_meta.end();
 
@@ -1064,7 +1050,7 @@ void sync_check_lock_granted(const latch_t *latch) {
 
 /** Check if it is OK to acquire the latch.
 @param[in]	latch	latch type
-@param[in]	level	Latch level */
+@param[in]	level	the level of the mutex */
 void sync_check_lock(const latch_t *latch, latch_level_t level) {
   if (LatchDebug::instance() != nullptr) {
     ut_ad(latch->get_level() == SYNC_LEVEL_VARYING);
@@ -1103,10 +1089,10 @@ const latch_t *sync_check_find(latch_level_t level) {
   return (nullptr);
 }
 
-/** Iterate over the thread's latches.
-@param[in,out]	functor		called for each element.
-@return false if the sync debug hasn't been initialised
-@return the value returned by the functor */
+/** Checks that the level array for the current thread is empty.
+Terminate iteration if the functor returns true.
+@param[in,out]	 functor	called for each element.
+@return true if the functor returns true */
 bool sync_check_iterate(sync_check_functor_t &functor) {
   if (LatchDebug::instance() != nullptr) {
     return (LatchDebug::instance()->for_each(functor));
@@ -1135,7 +1121,7 @@ void LatchDebug::init() UNIV_NOTHROW {
 
   mutex_create(LATCH_ID_RW_LOCK_DEBUG, &rw_lock_debug_mutex);
 
-  rw_lock_debug_event = os_event_create("rw_lock_debug_event");
+  rw_lock_debug_event = os_event_create();
 
   rw_lock_debug_waiters.store(false, std::memory_order_relaxed);
 }
@@ -1318,6 +1304,8 @@ static void sync_latch_meta_init() UNIV_NOTHROW {
 
   LATCH_ADD_RWLOCK(LOG_SN, SYNC_LOG_SN, log_sn_lock_key);
 
+  LATCH_ADD_MUTEX(LOG_SN_MUTEX, SYNC_LOG_SN_MUTEX, log_sn_mutex_key);
+
   LATCH_ADD_MUTEX(LOG_ARCH, SYNC_LOG_ARCH, log_sys_arch_mutex_key);
 
   LATCH_ADD_MUTEX(PAGE_ARCH, SYNC_PAGE_ARCH, page_sys_arch_mutex_key);
@@ -1363,8 +1351,6 @@ static void sync_latch_meta_init() UNIV_NOTHROW {
 
   LATCH_ADD_MUTEX(RW_LOCK_LIST, SYNC_NO_ORDER_CHECK, rw_lock_list_mutex_key);
 
-  LATCH_ADD_MUTEX(RW_LOCK_MUTEX, SYNC_NO_ORDER_CHECK, rw_lock_mutex_key);
-
   LATCH_ADD_MUTEX(SRV_INNODB_MONITOR, SYNC_NO_ORDER_CHECK,
                   srv_innodb_monitor_mutex_key);
 
@@ -1391,7 +1377,11 @@ static void sync_latch_meta_init() UNIV_NOTHROW {
 
   LATCH_ADD_MUTEX(TRX, SYNC_TRX, trx_mutex_key);
 
-  LATCH_ADD_MUTEX(LOCK_SYS, SYNC_LOCK_SYS, lock_mutex_key);
+  LATCH_ADD_MUTEX(LOCK_SYS_PAGE, SYNC_LOCK_SYS_SHARDED,
+                  lock_sys_page_mutex_key);
+
+  LATCH_ADD_MUTEX(LOCK_SYS_TABLE, SYNC_LOCK_SYS_SHARDED,
+                  lock_sys_table_mutex_key);
 
   LATCH_ADD_MUTEX(LOCK_SYS_WAIT, SYNC_LOCK_WAIT_SYS, lock_wait_mutex_key);
 
@@ -1455,6 +1445,9 @@ static void sync_latch_meta_init() UNIV_NOTHROW {
 
   LATCH_ADD_RWLOCK(RSEGS, SYNC_RSEGS, rsegs_lock_key);
 
+  LATCH_ADD_RWLOCK(LOCK_SYS_GLOBAL, SYNC_LOCK_SYS_GLOBAL,
+                   lock_sys_global_rw_lock_key);
+
   LATCH_ADD_RWLOCK(UNDO_SPACES, SYNC_UNDO_SPACES, undo_spaces_lock_key);
 
   LATCH_ADD_MUTEX(UNDO_DDL, SYNC_UNDO_DDL, PFS_NOT_INSTRUMENTED);
