@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2020, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2020, Oracle and/or its affiliates.
 Copyright (c) 2008, Google Inc.
 
 Portions of this file contain modifications contributed and copyrighted by
@@ -211,16 +211,12 @@ void rw_lock_create_func(
   /* If this is the very first time a synchronization object is
   created, then the following call initializes the sync system. */
 
-#ifndef INNODB_RW_LOCKS_USE_ATOMICS
-  mutex_create(LATCH_ID_RW_LOCK_MUTEX, rw_lock_get_mutex(lock));
-#else /* INNODB_RW_LOCKS_USE_ATOMICS */
 #ifdef UNIV_DEBUG
   UT_NOT_USED(cmutex_name);
 #endif
-#endif /* INNODB_RW_LOCKS_USE_ATOMICS */
 
   lock->lock_word = X_LOCK_DECR;
-  lock->waiters = 0;
+  lock->waiters = false;
 
   lock->recursive.store(false, std::memory_order_relaxed);
   lock->sx_recursive = 0;
@@ -241,19 +237,19 @@ void rw_lock_create_func(
 
   /* This should hold in practice. If it doesn't then we need to
   split the source file anyway. Or create the locks on lines
-  less than 8192. cline is unsigned:13. */
-  ut_ad(cline <= 8192);
-  lock->cline = (unsigned int)cline;
+  less than 65536. cline is uint16_t. */
+  ut_ad(cline <= std::numeric_limits<decltype(lock->cline)>::max());
+  lock->cline = cline;
 
   lock->count_os_wait = 0;
   lock->last_s_file_name = "not yet reserved";
   lock->last_x_file_name = "not yet reserved";
   lock->last_s_line = 0;
   lock->last_x_line = 0;
-  lock->event = os_event_create(nullptr);
-  lock->wait_ex_event = os_event_create(nullptr);
+  lock->event = os_event_create();
+  lock->wait_ex_event = os_event_create();
 
-  lock->is_block_lock = 0;
+  lock->is_block_lock = false;
 
   mutex_enter(&rw_lock_list_mutex);
 
@@ -276,10 +272,6 @@ void rw_lock_free_func(rw_lock_t *lock) /*!< in/out: rw-lock */
 
   mutex_enter(&rw_lock_list_mutex);
 
-#ifndef INNODB_RW_LOCKS_USE_ATOMICS
-  mutex_free(rw_lock_get_mutex(lock));
-#endif /* !INNODB_RW_LOCKS_USE_ATOMICS */
-
   os_event_destroy(lock->event);
 
   os_event_destroy(lock->wait_ex_event);
@@ -486,7 +478,7 @@ void rw_lock_x_lock_wait_func(
 /** Low-level function for acquiring an exclusive lock.
  @return false if did not succeed, true if success. */
 UNIV_INLINE
-ibool rw_lock_x_lock_low(
+bool rw_lock_x_lock_low(
     rw_lock_t *lock,       /*!< in: pointer to rw-lock */
     ulint pass,            /*!< in: pass value; != 0, if the lock will
                            be passed to another thread to unlock */
@@ -536,21 +528,22 @@ ibool rw_lock_x_lock_low(
       }
     } else {
       /* Another thread locked before us */
-      return (FALSE);
+      return false;
     }
   }
 
   ut_d(rw_lock_add_debug_info(lock, pass, RW_LOCK_X, file_name, line));
 
   lock->last_x_file_name = file_name;
-  lock->last_x_line = (unsigned int)line;
+  ut_ad(line <= std::numeric_limits<decltype(lock->last_x_line)>::max());
+  lock->last_x_line = line;
 
-  return (TRUE);
+  return true;
 }
 
 /** Low-level function for acquiring an sx lock.
  @return false if did not succeed, true if success. */
-ibool rw_lock_sx_lock_low(
+bool rw_lock_sx_lock_low(
     rw_lock_t *lock,       /*!< in: pointer to rw-lock */
     ulint pass,            /*!< in: pass value; != 0, if the lock will
                            be passed to another thread to unlock */
@@ -604,16 +597,18 @@ ibool rw_lock_sx_lock_low(
       }
     } else {
       /* Another thread locked before us */
-      return (FALSE);
+      return false;
     }
   }
 
   ut_d(rw_lock_add_debug_info(lock, pass, RW_LOCK_SX, file_name, line));
 
   lock->last_x_file_name = file_name;
-  lock->last_x_line = (unsigned int)line;
 
-  return (TRUE);
+  ut_ad(line <= std::numeric_limits<decltype(lock->last_x_line)>::max());
+  lock->last_x_line = line;
+
+  return true;
 }
 
 /** NOTE! Use the corresponding macro, not directly this function! Lock an
@@ -813,16 +808,12 @@ lock_loop:
  @return true */
 bool rw_lock_validate(const rw_lock_t *lock) /*!< in: rw-lock */
 {
-  ulint waiters;
-  lint lock_word;
+  int32_t lock_word;
 
   ut_ad(lock);
-
-  waiters = rw_lock_get_waiters(lock);
   lock_word = lock->lock_word;
 
   ut_ad(lock->magic_n == rw_lock_t::MAGIC_N);
-  ut_ad(waiters == 0 || waiters == 1);
   ut_ad(lock_word > -(2 * X_LOCK_DECR));
   ut_ad(lock_word <= X_LOCK_DECR);
 
@@ -1043,10 +1034,6 @@ void rw_lock_list_print_info(FILE *file) /*!< in: file where to print */
        lock = UT_LIST_GET_NEXT(list, lock)) {
     count++;
 
-#ifndef INNODB_RW_LOCKS_USE_ATOMICS
-    mutex_enter(&lock->mutex);
-#endif /* INNODB_RW_LOCKS_USE_ATOMICS */
-
     if (lock->lock_word != X_LOCK_DECR) {
       fprintf(file, "RW-LOCK: %p ", (void *)lock);
 
@@ -1067,20 +1054,16 @@ void rw_lock_list_print_info(FILE *file) /*!< in: file where to print */
 
       rw_lock_debug_mutex_exit();
     }
-
-#ifndef INNODB_RW_LOCKS_USE_ATOMICS
-    mutex_exit(&lock->mutex);
-#endif /* INNODB_RW_LOCKS_USE_ATOMICS */
   }
 
   fprintf(file, "Total number of rw-locks " ULINTPF "\n", count);
   mutex_exit(&rw_lock_list_mutex);
 }
 
-/** Prints info of a debug struct. */
-void rw_lock_debug_print(FILE *f,                     /*!< in: output stream */
-                         const rw_lock_debug_t *info) /*!< in: debug struct */
-{
+/** Prints info of a debug struct.
+@param[in] f Output stream
+@param[in] info Debug struct */
+void rw_lock_debug_print(FILE *f, const rw_lock_debug_t *info) {
   ulint rwt = info->lock_type;
 
   fprintf(f, "Locked: thread " UINT64PF " file %s line " ULINTPF "  ",
