@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2019, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2020, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -35,6 +35,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "current_thd.h"
 #include "ha_prototypes.h"
+#include "mysql/plugin.h"
 #include "sql_error.h"
 #include "trx0sys.h"
 
@@ -182,10 +183,10 @@ static bool read_binlog_position(const byte *binlog_buf, const char *&file_name,
 }
 
 /** Write binary log position into passed buffer.
-@param[in]	file_name	binary log file name
-@param[in]	offset		binary log offset
-@param[out]	binlog_buf	buffer from trx sys page to write to
-@param[in,out]	mtr		mini transaction */
+@param[in]	file_name	Binary log file name
+@param[in]	offset		Binary log offset
+@param[out]	binlog_buf	Buffer from trx sys page to write to
+@param[in,out]	mtr		Mini-transaction */
 static void write_binlog_position(const char *file_name, uint64_t offset,
                                   byte *binlog_buf, mtr_t *mtr) {
   if (file_name == nullptr ||
@@ -602,12 +603,15 @@ void trx_sys_close(void) {
 
   trx_purge_sys_close();
 
-  /* Only prepared transactions may be left in the system. Free them. */
-  ut_a(UT_LIST_GET_LEN(trx_sys->rw_trx_list) == trx_sys->n_prepared_trx);
+  /* Only prepared or active-recovered transactions may be left in the system.
+  The active-recovered transactions are allowed only if we did not force to
+  rollback them during shutdown (which might happen if e.g. it is fast
+  shutdown). Free all of them. */
+  trx_sys_after_background_threads_shutdown_validate();
 
   for (trx_t *trx = UT_LIST_GET_FIRST(trx_sys->rw_trx_list); trx != nullptr;
        trx = UT_LIST_GET_FIRST(trx_sys->rw_trx_list)) {
-    trx_free_prepared(trx);
+    trx_free_prepared_or_active_recovered(trx);
   }
 
   /* There can't be any active transactions. */
@@ -633,65 +637,83 @@ void trx_sys_close(void) {
   trx_sys = nullptr;
 }
 
-/** @brief Convert an undo log to TRX_UNDO_PREPARED state on shutdown.
-
-If any prepared ACTIVE transactions exist, and their rollback was
-prevented by innodb_force_recovery, we convert these transactions to
-XA PREPARE state in the main-memory data structures, so that shutdown
-will proceed normally. These transactions will again recover as ACTIVE
-on the next restart, and they will be rolled back unless
-innodb_force_recovery prevents it again.
-
-@param[in]	trx	transaction
-@param[in,out]	undo	undo log to convert to TRX_UNDO_PREPARED */
-static void trx_undo_fake_prepared(const trx_t *trx, trx_undo_t *undo) {
-  ut_ad(srv_force_recovery >= SRV_FORCE_NO_TRX_UNDO);
-  ut_ad(trx_state_eq(trx, TRX_STATE_ACTIVE));
-  ut_ad(trx->is_recovered);
-
-  if (undo != nullptr) {
-    ut_ad(undo->state == TRX_UNDO_ACTIVE);
-    undo->state = TRX_UNDO_PREPARED;
+void trx_sys_before_pre_dd_shutdown_validate() {
+  /** All connections are closed and close_connection unregisters
+  associated trx from mysql_trx_list. We still might have some non
+  started transactions in mysql_trx_list.
+  Purge threads are an exception, they create their own trx_t objects which are
+  not real transactions but are needed for the purge sys to use the query
+  threads framework. Purge sys shutdown happens at a later point in the shutdown
+  sequence so we need to skip its transactions here.
+  IMPORTANT: If allocating transactions for background threads, please use
+  trx_allocate_for_background. This function does not add the trx to the
+  mysql_trx_list so we don't have to add logic to skip these at shutdown.
+  */
+  trx_sys_mutex_enter();
+  for (trx_t *trx = UT_LIST_GET_FIRST(trx_sys->mysql_trx_list); trx != nullptr;
+       trx = UT_LIST_GET_NEXT(mysql_trx_list, trx)) {
+    /** Skip purge thread trx, it will be cleared after purge sys shutdown */
+    if (trx->purge_sys_trx) {
+      continue;
+    }
+    ut_a(trx->state == TRX_STATE_NOT_STARTED);
   }
+  trx_sys_mutex_exit();
 }
 
-/*********************************************************************
-Check if there are any active (non-prepared) transactions.
-@return total number of active transactions or 0 if none */
-ulint trx_sys_any_active_transactions(void) {
+void trx_sys_after_pre_dd_shutdown_validate() {
   trx_sys_mutex_enter();
-
-  ulint total_trx = UT_LIST_GET_LEN(trx_sys->mysql_trx_list);
-
-  if (total_trx == 0) {
-    total_trx = UT_LIST_GET_LEN(trx_sys->rw_trx_list);
-    ut_a(total_trx >= trx_sys->n_prepared_trx);
-
-    if (total_trx > trx_sys->n_prepared_trx &&
-        srv_force_recovery >= SRV_FORCE_NO_TRX_UNDO) {
-      for (trx_t *trx = UT_LIST_GET_FIRST(trx_sys->rw_trx_list); trx != nullptr;
-           trx = UT_LIST_GET_NEXT(trx_list, trx)) {
-        if (!trx_state_eq(trx, TRX_STATE_ACTIVE) || !trx->is_recovered) {
-          continue;
-        }
-        /* This was a recovered transaction whose rollback was disabled by
-        the innodb_force_recovery setting. Pretend that it is in XA PREPARE
-        state so that shutdown will work. */
-        trx_undo_fake_prepared(trx, trx->rsegs.m_redo.insert_undo);
-        trx_undo_fake_prepared(trx, trx->rsegs.m_redo.update_undo);
-        trx_undo_fake_prepared(trx, trx->rsegs.m_noredo.insert_undo);
-        trx_undo_fake_prepared(trx, trx->rsegs.m_noredo.update_undo);
-        trx->state = TRX_STATE_PREPARED;
-        trx_sys->n_prepared_trx++;
-      }
-    }
-
-    ut_a(total_trx >= trx_sys->n_prepared_trx);
-    total_trx -= trx_sys->n_prepared_trx;
+  /** At this point we check the mysql_trx_list again, now we don't expect purge
+  thread transactions in the list */
+  for (trx_t *trx = UT_LIST_GET_FIRST(trx_sys->mysql_trx_list); trx != nullptr;
+       trx = UT_LIST_GET_NEXT(mysql_trx_list, trx)) {
+    ut_a(trx->state == TRX_STATE_NOT_STARTED);
   }
-
   trx_sys_mutex_exit();
 
+  /* We assert that all transactions are rolled back if
+  [1] Not force recovery mode.
+  [2] Not fast shutdown
+  [3] The rollback thread has started and stopped gracefully.
+
+  The only left transactions are those that have state == TRX_STATE_PREPARED.
+
+  Above, [3] could be false during error exit, when the rollback thread might
+  never have started and we don't rollback the recovered transactions in that
+  case. */
+
+  const auto active_recovered_trxs = trx_sys_recovered_active_trxs_count();
+  if (srv_shutdown_waits_for_rollback_of_recovered_transactions() &&
+      srv_thread_is_stopped(srv_threads.m_trx_recovery_rollback)) {
+    ut_a(active_recovered_trxs == 0);
+  }
+
+  trx_sys_mutex_enter();
+  ut_a(UT_LIST_GET_LEN(trx_sys->rw_trx_list) ==
+       trx_sys->n_prepared_trx + active_recovered_trxs);
+  trx_sys_mutex_exit();
+}
+
+void trx_sys_after_background_threads_shutdown_validate() {
+  trx_sys_after_pre_dd_shutdown_validate();
+
+  trx_sys_mutex_enter();
+  ut_a(UT_LIST_GET_LEN(trx_sys->mysql_trx_list) == 0);
+  trx_sys_mutex_exit();
+}
+
+size_t trx_sys_recovered_active_trxs_count() {
+  size_t total_trx = 0;
+  trx_sys_mutex_enter();
+  /* Recovered transactions are never citizens of mysql_trx_list,
+  so it's enough to check rw_trx_list. */
+  for (trx_t *trx = UT_LIST_GET_FIRST(trx_sys->rw_trx_list); trx != nullptr;
+       trx = UT_LIST_GET_NEXT(trx_list, trx)) {
+    if (trx_state_eq(trx, TRX_STATE_ACTIVE) && trx->is_recovered) {
+      total_trx++;
+    }
+  }
+  trx_sys_mutex_exit();
   return (total_trx);
 }
 
