@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2019, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2020, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -207,13 +207,13 @@ trx_undo_rec_t *trx_undo_get_prev_rec(
 }
 
 /** Gets the next record in an undo log from the next page.
-@param[in]	space		undo log header space
-@param[in]	page_size	page size
-@param[in]	undo_page	undo log page
-@param[in]	page_no		undo log header page number
-@param[in]	offset		undo log header offset on page
-@param[in]	mode		latch mode: RW_S_LATCH or RW_X_LATCH
-@param[in,out]	mtr		mini-transaction
+@param[in]	space		Undo log header space
+@param[in]	page_size	Page size
+@param[in]	undo_page	Undo log page
+@param[in]	page_no		Undo log header page number
+@param[in]	offset		Undo log header offset on page
+@param[in]	mode		Latch mode: RW_S_LATCH or RW_X_LATCH
+@param[in,out]	mtr		Mini-transaction
 @return undo log record, the page latched, NULL if none */
 static trx_undo_rec_t *trx_undo_get_next_rec_from_next_page(
     space_id_t space, const page_size_t &page_size, const page_t *undo_page,
@@ -280,13 +280,13 @@ trx_undo_rec_t *trx_undo_get_next_rec(
 }
 
 /** Gets the first record in an undo log.
-@param[out]	modifier_trx_id	the modifier trx identifier.
-@param[in]	space		undo log header space
-@param[in]	page_size	page size
-@param[in]	page_no		undo log header page number
-@param[in]	offset		undo log header offset on page
-@param[in]	mode		latching mode: RW_S_LATCH or RW_X_LATCH
-@param[in,out]	mtr		mini-transaction
+@param[out]	modifier_trx_id	The modifier trx identifier.
+@param[in]	space		Undo log header space
+@param[in]	page_size	Page size
+@param[in]	page_no		Undo log header page number
+@param[in]	offset		Undo log header offset on page
+@param[in]	mode		Latching mode: RW_S_LATCH or RW_X_LATCH
+@param[in,out]	mtr		Mini-transaction
 @return undo log record, the page latched, NULL if none */
 trx_undo_rec_t *trx_undo_get_first_rec(trx_id_t *modifier_trx_id,
                                        space_id_t space,
@@ -516,7 +516,7 @@ static ulint trx_undo_header_create(
 
   new_free = free + TRX_UNDO_LOG_OLD_HDR_SIZE;
 
-  ut_a(free + TRX_UNDO_LOG_HDR_SIZE < UNIV_PAGE_SIZE - 100);
+  ut_a(free + TRX_UNDO_LOG_GTID_XA_HDR_SIZE < UNIV_PAGE_SIZE - 100);
 
   mach_write_to_2(page_hdr + TRX_UNDO_PAGE_START, new_free);
 
@@ -578,37 +578,33 @@ static void trx_undo_write_xid(
                     XIDDATASIZE, mtr);
 }
 
-void trx_undo_gtid_flush_prepare(trx_t *trx) {
-  /* Only relevant for prepared transaction. */
-  if (!trx_state_eq(trx, TRX_STATE_PREPARED)) {
-    return;
-  }
-  /* Only external transactions have GTID for XA PREPARE. */
-  if (trx_is_mysql_xa(trx)) {
-    return;
-  }
-  /* Wait for XA Prepare GTID to flush. */
-  auto &gtid_persistor = clone_sys->get_gtid_persistor();
-  gtid_persistor.wait_flush(true, false, false, nullptr);
-}
-
 dberr_t trx_undo_gtid_add_update_undo(trx_t *trx, bool prepare, bool rollback) {
   ut_ad(!(prepare && rollback));
   /* Check if GTID persistence is needed. */
   auto &gtid_persistor = clone_sys->get_gtid_persistor();
-  bool alloc = gtid_persistor.trx_check_set(trx, prepare, rollback);
 
-  if (!alloc) {
+  /* Caller could request GTID persistence explicitly for non-Innodb operations
+  using an empty transaction. We also allocate undo for such cases. */
+  bool gtid_explicit = false;
+
+  bool alloc =
+      gtid_persistor.trx_check_set(trx, prepare, rollback, gtid_explicit);
+
+  auto undo_ptr = &trx->rsegs.m_redo;
+
+  /* If update undo is already allocated, nothing to do. */
+  if (!alloc || undo_ptr->is_update()) {
     return (DB_SUCCESS);
   }
 
   /* For GTID persistence we need update undo segment. Allocate update
   undo segment here if it is insert only transaction. If no undo segment
   is allocated yet, then transaction didn't do any modification and
-  no GTID would be allotted to it. */
-  auto undo_ptr = &trx->rsegs.m_redo;
+  no GTID would be allotted to it. One exception is the explicit GTID
+  request. */
   dberr_t db_err = DB_SUCCESS;
-  if (undo_ptr->is_insert_only()) {
+
+  if (undo_ptr->is_insert_only() || gtid_explicit) {
     ut_ad(!rollback);
     mutex_enter(&trx->undo_mutex);
     db_err = trx_undo_assign_undo(trx, undo_ptr, TRX_UNDO_UPDATE);
@@ -628,45 +624,86 @@ dberr_t trx_undo_gtid_add_update_undo(trx_t *trx, bool prepare, bool rollback) {
   return (db_err);
 }
 
-void trx_undo_gtid_set(trx_t *trx, trx_undo_t *undo) {
+bool trx_undo_t::gtid_allocated(bool is_prepare) const {
+  if (is_prepare) {
+    return m_gtid_storage == Gtid_storage::PREPARE_AND_COMMIT;
+  }
+  return m_gtid_storage == Gtid_storage::COMMIT ||
+         m_gtid_storage == Gtid_storage::PREPARE_AND_COMMIT;
+}
+
+std::tuple<int, size_t> trx_undo_t::gtid_get_details(bool is_prepare) const {
+  int flag = is_prepare ? TRX_UNDO_FLAG_XA_PREPARE_GTID : TRX_UNDO_FLAG_GTID;
+  size_t size = is_prepare ? TRX_UNDO_LOG_GTID_XA : TRX_UNDO_LOG_GTID;
+
+  return std::make_tuple(flag, size);
+}
+
+void trx_undo_gtid_set(trx_t *trx, trx_undo_t *undo, bool is_xa_prepare) {
+  int gtid_flag;
+  /* For XA prepare we store the GTID separately. */
+  std::tie(gtid_flag, std::ignore) = undo->gtid_get_details(is_xa_prepare);
+
   /* Reset GTID flag */
-  undo->flag &= ~TRX_UNDO_FLAG_GTID;
+  undo->flag &= ~gtid_flag;
 
   if (!trx->persists_gtid) {
     return;
   }
 
   /* Verify that we have allocated for GTID */
-  if (!undo->gtid_allocated) {
+  if (!undo->gtid_allocated(is_xa_prepare)) {
     ut_ad(false);
     ib::error(ER_IB_CLONE_GTID_PERSIST)
         << "Could not persist GTID as space for GTID is not allocated.";
     return;
   }
-  undo->flag |= TRX_UNDO_FLAG_GTID;
+  undo->flag |= gtid_flag;
 }
 
 void trx_undo_gtid_read_and_persist(trx_ulogf_t *undo_header) {
   /* Check if undo log has GTID. */
   auto flag = mach_read_ulint(undo_header + TRX_UNDO_FLAGS, MLOG_1BYTE);
+
+  /* Extract and add GTID information of the transaction to the persister. */
+  Gtid_desc gtid_desc;
+
+  /* Get GTID persister */
+  auto &gtid_persistor = clone_sys->get_gtid_persistor();
+
+  /* Extract and add XA prepare GTID, if there. */
+  if ((flag & TRX_UNDO_FLAG_XA_PREPARE_GTID) != 0) {
+    /* Get GTID format version. */
+    gtid_desc.m_version = static_cast<uint32_t>(
+        mach_read_from_1(undo_header + TRX_UNDO_LOG_GTID_VERSION));
+
+    /* Get GTID information string. */
+    memcpy(&gtid_desc.m_info[0], undo_header + TRX_UNDO_LOG_GTID_XA,
+           TRX_UNDO_LOG_GTID_LEN);
+    /* Mark GTID valid. */
+    gtid_desc.m_is_set = true;
+
+    /* No concurrency is involved during recovery but satisfy
+    the interface requirement. */
+    trx_sys_mutex_enter();
+    gtid_persistor.add(gtid_desc);
+    trx_sys_mutex_exit();
+  }
+
   if ((flag & TRX_UNDO_FLAG_GTID) == 0) {
     return;
   }
-  /* Extract and add GTID information of the transaction to the persister. */
-  Gtid_desc gtid_desc;
 
   /* Get GTID format version. */
   gtid_desc.m_version = static_cast<uint32_t>(
       mach_read_from_1(undo_header + TRX_UNDO_LOG_GTID_VERSION));
+
   /* Get GTID information string. */
   memcpy(&gtid_desc.m_info[0], undo_header + TRX_UNDO_LOG_GTID,
          TRX_UNDO_LOG_GTID_LEN);
   /* Mark GTID valid. */
   gtid_desc.m_is_set = true;
 
-  /* Get GTID persister */
-  auto &gtid_persistor = clone_sys->get_gtid_persistor();
-
   /* No concurrency is involved during recovery but satisfy
   the interface requirement. */
   trx_sys_mutex_enter();
@@ -675,16 +712,21 @@ void trx_undo_gtid_read_and_persist(trx_ulogf_t *undo_header) {
 }
 
 void trx_undo_gtid_write(trx_t *trx, trx_ulogf_t *undo_header, trx_undo_t *undo,
-                         mtr_t *mtr) {
-  if ((undo->flag & TRX_UNDO_FLAG_GTID) == 0) {
+                         mtr_t *mtr, bool is_xa_prepare) {
+  int gtid_flag;
+  size_t gtid_offset;
+
+  std::tie(gtid_flag, gtid_offset) = undo->gtid_get_details(is_xa_prepare);
+
+  if ((undo->flag & gtid_flag) == 0) {
     return;
   }
 
   /* Reset GTID flag */
-  undo->flag &= ~TRX_UNDO_FLAG_GTID;
+  undo->flag &= ~gtid_flag;
 
   /* We must have allocated for GTID but add a safe check. */
-  if (!undo->gtid_allocated) {
+  if (!undo->gtid_allocated(is_xa_prepare)) {
     ut_ad(false);
     return;
   }
@@ -700,9 +742,9 @@ void trx_undo_gtid_write(trx_t *trx, trx_ulogf_t *undo_header, trx_undo_t *undo,
                      gtid_desc.m_version, MLOG_1BYTE, mtr);
     /* Persist fixed length GTID */
     ut_ad(TRX_UNDO_LOG_GTID_LEN == GTID_INFO_SIZE);
-    mlog_write_string(undo_header + TRX_UNDO_LOG_GTID, &gtid_desc.m_info[0],
+    mlog_write_string(undo_header + gtid_offset, &gtid_desc.m_info[0],
                       TRX_UNDO_LOG_GTID_LEN, mtr);
-    undo->flag |= TRX_UNDO_FLAG_GTID;
+    undo->flag |= gtid_flag;
   }
   mlog_write_ulint(undo_header + TRX_UNDO_FLAGS, undo->flag, MLOG_1BYTE, mtr);
 }
@@ -725,27 +767,40 @@ static void trx_undo_read_xid(
 }
 
 /** Adds space for the XA XID after an undo log old-style header.
-@param[in,out]	undo_page	undo log segment header page
-@param[in,out]	log_hdr		undo log header
-@param[in,out]	mtr		mini transaction
-@param[in]	add_gtid	add space for GTID */
-static void trx_undo_header_add_space_for_xid(page_t *undo_page,
-                                              trx_ulogf_t *log_hdr, mtr_t *mtr,
-                                              bool add_gtid) {
-  trx_upagef_t *page_hdr;
-  ulint free;
-  ulint new_free;
+@param[in,out]	undo_page	Undo log segment header page
+@param[in,out]	log_hdr		Undo log header
+@param[in,out]	mtr		Mini-transaction
+@param[in]      gtid_storage    GTID storage type */
+static void trx_undo_header_add_space_for_xid(
+    page_t *undo_page, trx_ulogf_t *log_hdr, mtr_t *mtr,
+    trx_undo_t::Gtid_storage gtid_storage) {
+  trx_upagef_t *page_hdr = undo_page + TRX_UNDO_PAGE_HDR;
 
-  page_hdr = undo_page + TRX_UNDO_PAGE_HDR;
-
-  free = mach_read_from_2(page_hdr + TRX_UNDO_PAGE_FREE);
+  ulint free = mach_read_from_2(page_hdr + TRX_UNDO_PAGE_FREE);
 
   /* free is now the end offset of the old style undo log header */
   ut_a(free == (ulint)(log_hdr - undo_page) + TRX_UNDO_LOG_OLD_HDR_SIZE);
 
-  ulint new_limit = add_gtid ? TRX_UNDO_LOG_HDR_SIZE : TRX_UNDO_LOG_XA_HDR_SIZE;
+  ulint new_limit = TRX_UNDO_LOG_XA_HDR_SIZE;
 
-  new_free = free + (new_limit - TRX_UNDO_LOG_OLD_HDR_SIZE);
+  switch (gtid_storage) {
+    case trx_undo_t::Gtid_storage::COMMIT:
+      new_limit = TRX_UNDO_LOG_GTID_HDR_SIZE;
+      break;
+
+    case trx_undo_t::Gtid_storage::PREPARE_AND_COMMIT:
+      new_limit = TRX_UNDO_LOG_GTID_XA_HDR_SIZE;
+      break;
+
+    case trx_undo_t::Gtid_storage::NONE:
+      new_limit = TRX_UNDO_LOG_XA_HDR_SIZE;
+      break;
+
+    default:
+      ut_ad(false);
+  }
+
+  ulint new_free = free + (new_limit - TRX_UNDO_LOG_OLD_HDR_SIZE);
 
   /* Add space for a XID after the header, update the free offset
   fields on the undo log page and in the undo log header */
@@ -773,11 +828,11 @@ void trx_undo_insert_header_reuse_log(
 #endif /* !UNIV_HOTBACKUP */
 
 /** Parse the redo log entry of an undo log page header create or reuse.
-@param[in]	type	MLOG_UNDO_HDR_CREATE or MLOG_UNDO_HDR_REUSE
-@param[in]	ptr	redo log record
-@param[in]	end_ptr	end of log buffer
-@param[in,out]	page	page frame or NULL
-@param[in,out]	mtr	mini-transaction or NULL
+@param[in]      type     MLOG_UNDO_HDR_CREATE or MLOG_UNDO_HDR_REUSE
+@param[in]      ptr      Redo log record
+@param[in]      end_ptr  End of log buffer
+@param[in,out]  page     Page frame or NULL
+@param[in,out]  mtr      Mini-transaction or NULL
 @return end of log record or NULL */
 byte *trx_undo_parse_page_header(mlog_id_t type, const byte *ptr,
                                  const byte *end_ptr, page_t *page,
@@ -824,7 +879,7 @@ static ulint trx_undo_insert_header_reuse(
 
   free = TRX_UNDO_SEG_HDR + TRX_UNDO_SEG_HDR_SIZE;
 
-  ut_a(free + TRX_UNDO_LOG_HDR_SIZE < UNIV_PAGE_SIZE - 100);
+  ut_a(free + TRX_UNDO_LOG_GTID_XA_HDR_SIZE < UNIV_PAGE_SIZE - 100);
 
   log_hdr = undo_page + free;
 
@@ -997,12 +1052,12 @@ void trx_undo_free_last_page_func(
 /** Empties an undo log header page of undo records for that undo log.
 Other undo logs may still have records on that page, if it is an update
 undo log.
-@param[in]	space		space
-@param[in]	page_size	page size
-@param[in]	hdr_page_no	header page number
-@param[in]	hdr_offset	header offset
-@param[in,out]	mtr		mini-transaction */
-static void trx_undo_empty_header_page(space_id_t space,
+@param[in]      space_id     Tablespace ID
+@param[in]      page_size    Page size
+@param[in]      hdr_page_no  Header page number
+@param[in]      hdr_offset   Header offset
+@param[in,out]  mtr          Mini-transaction */
+static void trx_undo_empty_header_page(space_id_t space_id,
                                        const page_size_t &page_size,
                                        page_no_t hdr_page_no, ulint hdr_offset,
                                        mtr_t *mtr) {
@@ -1011,7 +1066,7 @@ static void trx_undo_empty_header_page(space_id_t space,
   ulint end;
 
   header_page =
-      trx_undo_page_get(page_id_t(space, hdr_page_no), page_size, mtr);
+      trx_undo_page_get(page_id_t(space_id, hdr_page_no), page_size, mtr);
 
   log_hdr = header_page + hdr_offset;
 
@@ -1020,74 +1075,111 @@ static void trx_undo_empty_header_page(space_id_t space,
   mlog_write_ulint(log_hdr + TRX_UNDO_LOG_START, end, MLOG_2BYTES, mtr);
 }
 
+/** Get page offset up to which undo logs can be truncated.
+There are three possibilities.
+1. Truncate nothing on this page. Return -1
+2. Truncate part of the page. Return the offset
+3. Truncate the whole page. Return 0
+@param[in]  undo       undo log to truncate
+@param[in]  undo_page  undo log page to check
+@param[in]  limit      limit up to which undo logs to be truncated
+@return page offset to truncate to, 0 for whole page, -1 for nothing. */
+int trx_undo_page_truncate_offset(trx_undo_t *undo, page_t *undo_page,
+                                  undo_no_t limit) {
+  auto rec = trx_undo_page_get_last_rec(undo_page, undo->hdr_page_no,
+                                        undo->hdr_offset);
+  trx_undo_rec_t *trunc_rec = nullptr;
+
+  while (rec != nullptr) {
+    /* Check if current record has gone below the limit. */
+    if (trx_undo_rec_get_undo_no(rec) < limit) {
+      /* If this is the first record on the page, don't truncate anything */
+      if (trunc_rec == nullptr) {
+        return (-1);
+      }
+
+      /* Return an offset within the page. */
+      return (trunc_rec - undo_page);
+    }
+
+    /* Truncate at least up to this record, maybe more */
+    trunc_rec = rec;
+    rec = trx_undo_page_get_prev_rec(rec, undo->hdr_page_no, undo->hdr_offset);
+  }
+
+  /* All records on the page are >= limit */
+  if (undo->last_page_no == undo->hdr_page_no) {
+    /* This is the header page. Return an offset
+    if there are any records on the page. */
+    if (trunc_rec != nullptr) {
+      return (trunc_rec - undo_page);
+    }
+
+    /* Header page is empty.  Do not truncate anything. */
+    return (-1);
+  }
+
+  /* Truncate the whole page. */
+  return (0);
+}
+
 /** Truncates an undo log from the end. This function is used during a rollback
  to free space from an undo log. */
+#ifdef UNIV_DEBUG
+/**
+@param[in]  trx    transaction for this undo log */
+#endif /* UNIV_DEBUG */
+/**
+@param[in]  undo   undo log
+@param[in]  limit  all undo records with undo number;
+                   This value should be truncated. */
 void trx_undo_truncate_end_func(
 #ifdef UNIV_DEBUG
-    const trx_t *trx, /*!< in: transaction whose undo log it is */
-#endif                /* UNIV_DEBUG */
-    trx_undo_t *undo, /*!< in: undo log */
-    undo_no_t limit)  /*!< in: all undo records with undo number
-                      >= this value should be truncated */
-{
-  page_t *undo_page;
-  page_no_t last_page_no;
-  trx_undo_rec_t *rec;
-  trx_undo_rec_t *trunc_here;
-  mtr_t mtr;
-
-  ut_ad(mutex_own(&(trx->undo_mutex)));
-
+    const trx_t *trx,
+#endif /* UNIV_DEBUG */
+    trx_undo_t *undo, undo_no_t limit) {
+  ut_ad(mutex_own(&trx->undo_mutex));
   ut_ad(mutex_own(&undo->rseg->mutex));
 
+  mtr_t mtr;
+
   for (;;) {
-    mtr_start(&mtr);
+    mtr.start();
+
+    /* Set NO_REDO for temporary undo logs. */
     if (fsp_is_system_temporary(undo->rseg->space_id)) {
-      mtr.set_log_mode(MTR_LOG_NO_REDO);
       ut_ad(trx->rsegs.m_noredo.rseg == undo->rseg);
+      mtr.set_log_mode(MTR_LOG_NO_REDO);
     } else {
       ut_ad(trx->rsegs.m_redo.rseg == undo->rseg);
     }
 
-    trunc_here = nullptr;
+    const page_id_t page_id(undo->space, undo->last_page_no);
 
-    last_page_no = undo->last_page_no;
+    auto undo_page = trx_undo_page_get(page_id, undo->page_size, &mtr);
 
-    undo_page = trx_undo_page_get(page_id_t(undo->space, last_page_no),
-                                  undo->page_size, &mtr);
+    int trunc_offset = trx_undo_page_truncate_offset(undo, undo_page, limit);
 
-    rec = trx_undo_page_get_last_rec(undo_page, undo->hdr_page_no,
-                                     undo->hdr_offset);
-    while (rec) {
-      if (trx_undo_rec_get_undo_no(rec) >= limit) {
-        /* Truncate at least this record off, maybe
-        more */
-        trunc_here = rec;
-      } else {
-        goto function_exit;
-      }
-
-      rec =
-          trx_undo_page_get_prev_rec(rec, undo->hdr_page_no, undo->hdr_offset);
+    /* If offset is within the page, truncate part of the page and quit.*/
+    if (trunc_offset > 0) {
+      mlog_write_ulint(undo_page + TRX_UNDO_PAGE_HDR + TRX_UNDO_PAGE_FREE,
+                       trunc_offset, MLOG_2BYTES, &mtr);
+      break;
     }
 
-    if (last_page_no == undo->hdr_page_no) {
-      goto function_exit;
+    /* If all recs are < limit, don't truncate anything. */
+    if (trunc_offset < 0) {
+      break;
     }
 
-    ut_ad(last_page_no == undo->last_page_no);
+    /* Free the last page and move on to the next. */
+    ut_ad(undo->last_page_no != undo->hdr_page_no);
     trx_undo_free_last_page(trx, undo, &mtr);
 
-    mtr_commit(&mtr);
+    mtr.commit();
   }
 
-function_exit:
-  if (trunc_here) {
-    mlog_write_ulint(undo_page + TRX_UNDO_PAGE_HDR + TRX_UNDO_PAGE_FREE,
-                     trunc_here - undo_page, MLOG_2BYTES, &mtr);
-  }
-
-  mtr_commit(&mtr);
+  mtr.commit();
 }
 
 /** Truncate the head of an undo log.
@@ -1112,7 +1204,7 @@ void trx_undo_truncate_start(trx_rseg_t *rseg, page_no_t hdr_page_no,
     return;
   }
 loop:
-  mtr_start(&mtr);
+  mtr.start();
 
   if (fsp_is_system_temporary(rseg->space_id)) {
     mtr.set_log_mode(MTR_LOG_NO_REDO);
@@ -1123,7 +1215,7 @@ loop:
   if (rec == nullptr) {
     /* Already empty */
 
-    mtr_commit(&mtr);
+    mtr.commit();
 
     return;
   }
@@ -1132,7 +1224,7 @@ loop:
 
   last_rec = trx_undo_page_get_last_rec(undo_page, hdr_page_no, hdr_offset);
   if (trx_undo_rec_get_undo_no(last_rec) >= limit) {
-    mtr_commit(&mtr);
+    mtr.commit();
 
     return;
   }
@@ -1146,7 +1238,7 @@ loop:
     trx_undo_free_page(rseg, TRUE, rseg->space_id, hdr_page_no, page_no, &mtr);
   }
 
-  mtr_commit(&mtr);
+  mtr.commit();
 
   goto loop;
 }
@@ -1165,7 +1257,7 @@ static void trx_undo_seg_free(const trx_undo_t *undo, bool noredo) {
   rseg = undo->rseg;
 
   do {
-    mtr_start(&mtr);
+    mtr.start();
 
     if (noredo) {
       mtr.set_log_mode(MTR_LOG_NO_REDO);
@@ -1192,7 +1284,7 @@ static void trx_undo_seg_free(const trx_undo_t *undo, bool noredo) {
     }
 
     rseg->unlatch();
-    mtr_commit(&mtr);
+    mtr.commit();
   } while (!finished);
 }
 
@@ -1243,8 +1335,6 @@ static trx_undo_t *trx_undo_mem_init(
 
   bool xid_exists = ((flag & TRX_UNDO_FLAG_XID) != 0);
 
-  bool gtid_exists = ((flag & TRX_UNDO_FLAG_GTID) != 0);
-
   /* Read X/Open XA transaction identification if it exists, or
   set it to NULL. */
   xid.reset();
@@ -1259,7 +1349,17 @@ static trx_undo_t *trx_undo_mem_init(
       mtr_read_ulint(undo_header + TRX_UNDO_DICT_TRANS, MLOG_1BYTE, mtr);
 
   undo->flag = flag;
-  undo->gtid_allocated = gtid_exists;
+
+  undo->m_gtid_storage = trx_undo_t::Gtid_storage::NONE;
+
+  if ((flag & TRX_UNDO_FLAG_XA_PREPARE_GTID) != 0) {
+    /* Prepare GTID implies space is allocated for both
+    prepare and commit GTID. */
+    undo->m_gtid_storage = trx_undo_t::Gtid_storage::PREPARE_AND_COMMIT;
+  } else if ((flag & TRX_UNDO_FLAG_GTID) != 0) {
+    /* Space is allocated for only commit GTID. */
+    undo->m_gtid_storage = trx_undo_t::Gtid_storage::COMMIT;
+  }
 
   undo->state = state;
   undo->size = flst_get_len(seg_header + TRX_UNDO_PAGE_LIST);
@@ -1326,7 +1426,7 @@ ulint trx_undo_lists_init(
   ulint i;
   mtr_t mtr;
 
-  mtr_start(&mtr);
+  mtr.start();
 
   rseg_header =
       trx_rsegf_get_new(rseg->space_id, rseg->page_no, rseg->page_size, &mtr);
@@ -1349,9 +1449,9 @@ ulint trx_undo_lists_init(
 
       size += undo->size;
 
-      mtr_commit(&mtr);
+      mtr.commit();
 
-      mtr_start(&mtr);
+      mtr.start();
 
       rseg_header =
           trx_rsegf_get(rseg->space_id, rseg->page_no, rseg->page_size, &mtr);
@@ -1361,7 +1461,7 @@ ulint trx_undo_lists_init(
     }
   }
 
-  mtr_commit(&mtr);
+  mtr.commit();
 
   return (size);
 }
@@ -1397,7 +1497,7 @@ static trx_undo_t *trx_undo_mem_create(trx_rseg_t *rseg, ulint id, ulint type,
 
   undo->dict_operation = FALSE;
   undo->flag = 0;
-  undo->gtid_allocated = false;
+  undo->m_gtid_storage = trx_undo_t::Gtid_storage::NONE;
 
   undo->rseg = rseg;
 
@@ -1411,7 +1511,6 @@ static trx_undo_t *trx_undo_mem_create(trx_rseg_t *rseg, ulint id, ulint type,
   undo->empty = TRUE;
   undo->top_page_no = page_no;
   undo->guess_block = nullptr;
-  undo->withdraw_clock = 0;
 
   return (undo);
 }
@@ -1435,7 +1534,7 @@ static void trx_undo_mem_init_for_reuse(
 
   undo->dict_operation = FALSE;
   undo->flag = 0;
-  undo->gtid_allocated = false;
+  undo->m_gtid_storage = trx_undo_t::Gtid_storage::NONE;
 
   undo->hdr_offset = offset;
   undo->empty = TRUE;
@@ -1455,7 +1554,7 @@ void trx_undo_mem_free(trx_undo_t *undo) /*!< in: the undo object to be freed */
 @param[in]   type   type of the log: TRX_UNDO_INSERT or TRX_UNDO_UPDATE
 @param[in]   trx_id  id of the trx for which the undo log is created
 @param[in]   xid     X/Open transaction identification
-@param[in]   is_gtid if transaction has GTID
+@param[in]   gtid_storage GTID storage type
 @param[out]  undo    the new undo log object, undefined if did not succeed
 @param[in]   mtr     mini-transation
 @retval DB_SUCCESS if successful in creating the new undo lob object,
@@ -1464,19 +1563,18 @@ void trx_undo_mem_free(trx_undo_t *undo) /*!< in: the undo object to be freed */
 @retval DB_OUT_OF_MEMORY */
 static MY_ATTRIBUTE((warn_unused_result)) dberr_t
     trx_undo_create(trx_t *trx, trx_rseg_t *rseg, ulint type, trx_id_t trx_id,
-                    const XID *xid, bool is_gtid, trx_undo_t **undo,
-                    mtr_t *mtr) {
+                    const XID *xid, trx_undo_t::Gtid_storage gtid_storage,
+                    trx_undo_t **undo, mtr_t *mtr) {
   trx_rsegf_t *rseg_header;
   page_no_t page_no;
   ulint offset;
   ulint id;
   page_t *undo_page;
-  dberr_t err;
 
   ut_ad(mutex_own(&(rseg->mutex)));
 
   if (rseg->get_curr_size() == rseg->max_size) {
-    return (DB_OUT_OF_FILE_SPACE);
+    return DB_OUT_OF_FILE_SPACE;
   }
 
   rseg->incr_curr_size();
@@ -1484,48 +1582,50 @@ static MY_ATTRIBUTE((warn_unused_result)) dberr_t
   rseg_header =
       trx_rsegf_get(rseg->space_id, rseg->page_no, rseg->page_size, mtr);
 
-  err = trx_undo_seg_create(rseg, rseg_header, type, &id, &undo_page, mtr);
+  auto err = trx_undo_seg_create(rseg, rseg_header, type, &id, &undo_page, mtr);
 
   if (err != DB_SUCCESS) {
     /* Did not succeed */
-
     rseg->decr_curr_size();
-
-    return (err);
+    return err;
   }
 
   page_no = page_get_page_no(undo_page);
 
   offset = trx_undo_header_create(undo_page, trx_id, mtr);
 
-  bool add_space_gtid = (is_gtid && type == TRX_UNDO_UPDATE);
+  /* GTID storage is needed only for update undo log. */
+  if (type != TRX_UNDO_UPDATE) {
+    gtid_storage = trx_undo_t::Gtid_storage::NONE;
+  }
+
   trx_undo_header_add_space_for_xid(undo_page, undo_page + offset, mtr,
-                                    add_space_gtid);
+                                    gtid_storage);
 
   *undo = trx_undo_mem_create(rseg, id, type, trx_id, xid, page_no, offset);
   if (*undo == nullptr) {
-    err = DB_OUT_OF_MEMORY;
-  } else {
-    (*undo)->gtid_allocated = add_space_gtid;
+    return DB_OUT_OF_MEMORY;
   }
 
-  return (err);
+  (*undo)->m_gtid_storage = gtid_storage;
+  return DB_SUCCESS;
 }
 
 /*================ UNDO LOG ASSIGNMENT AND CLEANUP =====================*/
 
 /** Reuses a cached undo log.
-@param[in,out]	trx	transaction
-@param[in,out]	rseg	rollback segment memory object
-@param[in]	type	type of the log: TRX_UNDO_INSERT or TRX_UNDO_UPDATE
-@param[in]	trx_id	id of the trx for which the undo log is used
+@param[in,out]	trx	Transaction
+@param[in,out]	rseg	Rollback segment memory object
+@param[in]	type	Type of the log: TRX_UNDO_INSERT or TRX_UNDO_UPDATE
+@param[in]	trx_id	Id of the trx for which the undo log is used
 @param[in]	xid	X/Open XA transaction identification
-@param[in]	is_gtid	if transaction has GTID
-@param[in,out]	mtr	mini transaction
+@param[in]      gtid_storage GTID storage type
+@param[in,out]	mtr	Mini-transaction
 @return the undo log memory object, NULL if none cached */
 static trx_undo_t *trx_undo_reuse_cached(trx_t *trx, trx_rseg_t *rseg,
                                          ulint type, trx_id_t trx_id,
-                                         const XID *xid, bool is_gtid,
+                                         const XID *xid,
+                                         trx_undo_t::Gtid_storage gtid_storage,
                                          mtr_t *mtr) {
   trx_undo_t *undo;
 
@@ -1558,28 +1658,25 @@ static trx_undo_t *trx_undo_reuse_cached(trx_t *trx, trx_rseg_t *rseg,
 
   auto undo_page = trx_undo_page_get(page_id_t(undo->space, undo->hdr_page_no),
                                      undo->page_size, mtr);
-
-  bool add_space_gtid = false;
   ulint offset;
 
   if (type == TRX_UNDO_INSERT) {
     offset = trx_undo_insert_header_reuse(undo_page, trx_id, mtr);
+    gtid_storage = trx_undo_t::Gtid_storage::NONE;
 
-    trx_undo_header_add_space_for_xid(undo_page, undo_page + offset, mtr,
-                                      false);
   } else {
     ut_a(mach_read_from_2(undo_page + TRX_UNDO_PAGE_HDR + TRX_UNDO_PAGE_TYPE) ==
          TRX_UNDO_UPDATE);
 
     offset = trx_undo_header_create(undo_page, trx_id, mtr);
-
-    trx_undo_header_add_space_for_xid(undo_page, undo_page + offset, mtr,
-                                      is_gtid);
-    add_space_gtid = is_gtid;
   }
 
+  trx_undo_header_add_space_for_xid(undo_page, undo_page + offset, mtr,
+                                    gtid_storage);
+
   trx_undo_mem_init_for_reuse(undo, trx_id, xid, offset);
-  undo->gtid_allocated = add_space_gtid;
+
+  undo->m_gtid_storage = gtid_storage;
 
   return (undo);
 }
@@ -1643,17 +1740,18 @@ dberr_t trx_undo_assign_undo(
   Innodb would persist GTID. Call it before any undo segment
   is assigned for transaction. We allocate space for GTID
   only if GTID is persisted. */
-  bool is_gtid = false;
+  trx_undo_t::Gtid_storage gtid_storage{trx_undo_t::Gtid_storage::NONE};
+
   if (!no_redo) {
     auto &gtid_persistor = clone_sys->get_gtid_persistor();
     if (is_first) {
       gtid_persistor.set_persist_gtid(trx, true);
     }
     /* Check if the undo segment needs to allocate for GTID. */
-    is_gtid = gtid_persistor.persists_gtid(trx);
+    gtid_storage = gtid_persistor.persists_gtid(trx);
   }
 
-  mtr_start(&mtr);
+  mtr.start();
   if (no_redo) {
     mtr.set_log_mode(MTR_LOG_NO_REDO);
   } else {
@@ -1671,12 +1769,12 @@ dberr_t trx_undo_assign_undo(
           ? nullptr
           :
 #endif
-          trx_undo_reuse_cached(trx, rseg, type, trx->id, trx->xid, is_gtid,
-                                &mtr);
+          trx_undo_reuse_cached(trx, rseg, type, trx->id, trx->xid,
+                                gtid_storage, &mtr);
 
   if (undo == nullptr) {
-    err = trx_undo_create(trx, rseg, type, trx->id, trx->xid, is_gtid, &undo,
-                          &mtr);
+    err = trx_undo_create(trx, rseg, type, trx->id, trx->xid, gtid_storage,
+                          &undo, &mtr);
     if (err != DB_SUCCESS) {
       goto func_exit;
     }
@@ -1710,7 +1808,7 @@ dberr_t trx_undo_assign_undo(
 
 func_exit:
   rseg->unlatch();
-  mtr_commit(&mtr);
+  mtr.commit();
 
   return (err);
 }
@@ -1752,10 +1850,10 @@ page_t *trx_undo_set_state_at_finish(
 }
 
 /** Set the state of the undo log segment at a XA PREPARE or XA ROLLBACK.
-@param[in,out]	trx		transaction
-@param[in,out]	undo		insert_undo or update_undo log
+@param[in,out]	trx		Transaction
+@param[in,out]	undo		Insert_undo or update_undo log
 @param[in]	rollback	false=XA PREPARE, true=XA ROLLBACK
-@param[in,out]	mtr		mini-transaction
+@param[in,out]	mtr		Mini-transaction
 @return undo log segment header page, x-latched */
 page_t *trx_undo_set_state_at_prepare(trx_t *trx, trx_undo_t *undo,
                                       bool rollback, mtr_t *mtr) {
@@ -1777,7 +1875,7 @@ page_t *trx_undo_set_state_at_prepare(trx_t *trx, trx_undo_t *undo,
   undo_header = undo_page + offset;
 
   /* Write GTID information if there. */
-  trx_undo_gtid_write(trx, undo_header, undo, mtr);
+  trx_undo_gtid_write(trx, undo_header, undo, mtr, !rollback);
 
   if (rollback) {
     ut_ad(undo->state == TRX_UNDO_PREPARED);
@@ -1800,19 +1898,18 @@ page_t *trx_undo_set_state_at_prepare(trx_t *trx, trx_undo_t *undo,
 
 /** Adds the update undo log header as the first in the history list, and
  frees the memory object, or puts it to the list of cached update undo log
- segments. */
-void trx_undo_update_cleanup(
-    trx_t *trx,               /*!< in: trx owning the update
-                              undo log */
-    trx_undo_ptr_t *undo_ptr, /*!< in: update undo log. */
-    page_t *undo_page,        /*!< in: update undo log header page,
-                              x-latched */
-    bool update_rseg_history_len,
-    /*!< in: if true: update rseg history
-    len else skip updating it. */
-    ulint n_added_logs, /*!< in: number of logs added */
-    mtr_t *mtr)         /*!< in: mtr */
-{
+ segments.
+@param[in] trx Trx owning the update undo log
+@param[in] undo_ptr Update undo log.
+@param[in] undo_page Update undo log header page, x-latched
+@param[in] update_rseg_history_len If true: update rseg history len else
+skip updating it.
+@param[in] n_added_logs Number of logs added
+@param[in] mtr Mini-transaction */
+void trx_undo_update_cleanup(trx_t *trx, trx_undo_ptr_t *undo_ptr,
+                             page_t *undo_page, bool update_rseg_history_len,
+
+                             ulint n_added_logs, mtr_t *mtr) {
   trx_rseg_t *rseg;
   trx_undo_t *undo;
 
@@ -1881,13 +1978,15 @@ void trx_undo_insert_cleanup(trx_undo_ptr_t *undo_ptr, bool noredo) {
   rseg->unlatch();
 }
 
-/** At shutdown, frees the undo logs of a PREPARED transaction. */
-void trx_undo_free_prepared(trx_t *trx) /*!< in/out: PREPARED transaction */
-{
-  ut_ad(srv_shutdown_state.load() == SRV_SHUTDOWN_EXIT_THREADS);
+void trx_undo_free_trx_with_prepared_or_active_logs(trx_t *trx,
+                                                    ulint expected_undo_state) {
+  ut_a(expected_undo_state == TRX_UNDO_ACTIVE ||
+       expected_undo_state == TRX_UNDO_PREPARED);
+
+  ut_a(srv_shutdown_state.load() == SRV_SHUTDOWN_EXIT_THREADS);
 
   if (trx->rsegs.m_redo.update_undo) {
-    ut_a(trx->rsegs.m_redo.update_undo->state == TRX_UNDO_PREPARED);
+    ut_a(trx->rsegs.m_redo.update_undo->state == expected_undo_state);
     UT_LIST_REMOVE(trx->rsegs.m_redo.rseg->update_undo_list,
                    trx->rsegs.m_redo.update_undo);
     trx_undo_mem_free(trx->rsegs.m_redo.update_undo);
@@ -1896,7 +1995,7 @@ void trx_undo_free_prepared(trx_t *trx) /*!< in/out: PREPARED transaction */
   }
 
   if (trx->rsegs.m_redo.insert_undo) {
-    ut_a(trx->rsegs.m_redo.insert_undo->state == TRX_UNDO_PREPARED);
+    ut_a(trx->rsegs.m_redo.insert_undo->state == expected_undo_state);
     UT_LIST_REMOVE(trx->rsegs.m_redo.rseg->insert_undo_list,
                    trx->rsegs.m_redo.insert_undo);
     trx_undo_mem_free(trx->rsegs.m_redo.insert_undo);
@@ -1905,7 +2004,7 @@ void trx_undo_free_prepared(trx_t *trx) /*!< in/out: PREPARED transaction */
   }
 
   if (trx->rsegs.m_noredo.update_undo) {
-    ut_a(trx->rsegs.m_noredo.update_undo->state == TRX_UNDO_PREPARED);
+    ut_a(trx->rsegs.m_noredo.update_undo->state == expected_undo_state);
 
     UT_LIST_REMOVE(trx->rsegs.m_noredo.rseg->update_undo_list,
                    trx->rsegs.m_noredo.update_undo);
@@ -1914,7 +2013,7 @@ void trx_undo_free_prepared(trx_t *trx) /*!< in/out: PREPARED transaction */
     trx->rsegs.m_noredo.update_undo = nullptr;
   }
   if (trx->rsegs.m_noredo.insert_undo) {
-    ut_a(trx->rsegs.m_noredo.insert_undo->state == TRX_UNDO_PREPARED);
+    ut_a(trx->rsegs.m_noredo.insert_undo->state == expected_undo_state);
 
     UT_LIST_REMOVE(trx->rsegs.m_noredo.rseg->insert_undo_list,
                    trx->rsegs.m_noredo.insert_undo);
@@ -1926,68 +2025,82 @@ void trx_undo_free_prepared(trx_t *trx) /*!< in/out: PREPARED transaction */
 
 bool trx_undo_truncate_tablespace(undo::Tablespace *marked_space) {
 #ifdef UNIV_DEBUG
-  static int truncate_fail_count;
-  DBUG_EXECUTE_IF(
-      "ib_undo_trunc_fail_truncate", if (++truncate_fail_count == 1) {
-        ib::info() << "ib_undo_trunc_fail_truncate";
-        return (false);
-      });
+  static undo::Inject_failure_once injector("ib_undo_trunc_fail_truncate");
+  if (injector.should_fail()) {
+    return (false);
+  };
 #endif /* UNIV_DEBUG */
 
   bool success = true;
-  space_id_t old_space_id = marked_space->id();
-  space_id_t space_num = undo::id2num(old_space_id);
-  Rsegs *marked_rsegs = marked_space->rsegs();
+
+  auto old_space_id = marked_space->id();
+  auto space_num = undo::id2num(old_space_id);
+  auto marked_rsegs = marked_space->rsegs();
 
   undo::unuse_space_id(old_space_id);
 
-  space_id_t new_space_id = undo::use_next_space_id(space_num);
+  auto new_space_id = undo::use_next_space_id(space_num);
+
+  auto space = fil_space_get(old_space_id);
+
+  auto n_pages = INITIAL_UNDO_SPACE_SIZE_IN_PAGES;
+
+  /* If the default extend amount has been increased greater than the default
+  and it has been less than 1 second since the last time the file was extended,
+  then we consider the undo tablespace to be growing aggressively. */
+  if (space->m_undo_extend > INITIAL_UNDO_SPACE_SIZE_IN_PAGES &&
+      space->m_last_extended.elapsed() < 1000) {
+    /* UNDO is beeing extended aggressively, dont' reduce size to default. */
+    n_pages = fil_space_get_size(old_space_id) / 4;
+  }
 
-  fil_space_t *space = fil_space_get(old_space_id);
   bool is_encrypted = FSP_FLAGS_GET_ENCRYPTION(space->flags);
 
   /* Step-1: Truncate tablespace by replacement with a new space_id. */
-  success = fil_replace_tablespace(old_space_id, new_space_id,
-                                   SRV_UNDO_TABLESPACE_SIZE_IN_PAGES);
+  success = fil_replace_tablespace(old_space_id, new_space_id, n_pages);
+
   if (!success) {
     return (success);
   }
 
-  DBUG_EXECUTE_IF("ib_undo_trunc_empty_file",
-                  ib::info(ER_IB_MSG_UNDO_TRUNC_EMPTY_FILE);
-                  DBUG_SUICIDE(););
+  ut_d(undo::inject_crash("ib_undo_trunc_empty_file"));
 
   /* This undo tablespace is unused. Lock the Rsegs before the
   file_space because SYNC_RSEGS > SYNC_FSP. */
   marked_rsegs->x_lock();
 
-  /* Step-2: Re-initialize tablespace header.
-  Avoid REDO logging as we don't want to apply the action if server
-  crashes. For fix-up we have UNDO-truncate-ddl-log. */
+  /* Step-2: Re-initialize tablespace header. */
   log_free_check();
+
   mtr_t mtr;
-  mtr_start(&mtr);
-  mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO);
-  /* Why return value is not being checked here? */
-  fsp_header_init(new_space_id, SRV_UNDO_TABLESPACE_SIZE_IN_PAGES, &mtr, false);
+
+  mtr.start();
+
+  fsp_header_init(new_space_id, n_pages, &mtr, false);
+
+  /* If tablespace is to be encrypted, encrypt it now */
+  if (is_encrypted && srv_undo_log_encrypt) {
+    ut_d(bool ret =) set_undo_tablespace_encryption(new_space_id, &mtr, false);
+    /* Don't expect any error here (unless keyring plugin is uninstalled). In
+    that case too, continue truncation processing of tablespace. */
+    ut_ad(!ret);
+  }
 
   /* Step-3: Add the RSEG_ARRAY page. */
   trx_rseg_array_create(new_space_id, &mtr);
-  mtr_commit(&mtr);
+
+  mtr.commit();
 
   /* Step-4: Re-initialize rollback segment header that resides
   in truncated tablespaces. */
 
-  DBUG_EXECUTE_IF("ib_undo_trunc_before_rsegs",
-                  ib::info(ER_IB_MSG_UNDO_TRUNK_BEFORE_RSEG);
-                  DBUG_SUICIDE(););
+  ut_d(undo::inject_crash("ib_undo_trunc_before_rsegs"));
 
   for (auto rseg : *marked_rsegs) {
-    trx_rsegf_t *rseg_header;
-
     log_free_check();
-    mtr_start(&mtr);
-    mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO);
+
+    mtr.start();
+
     mtr_x_lock(fil_space_get_latch(new_space_id), &mtr);
 
     rseg->space_id = new_space_id;
@@ -1997,7 +2110,7 @@ bool trx_undo_truncate_tablespace(undo::Tablespace *marked_space) {
 
     ut_a(rseg->page_no != FIL_NULL);
 
-    rseg_header =
+    auto rseg_header =
         trx_rsegf_get_new(new_space_id, rseg->page_no, rseg->page_size, &mtr);
 
     /* Before re-initialization ensure that we free the existing
@@ -2036,7 +2149,7 @@ bool trx_undo_truncate_tablespace(undo::Tablespace *marked_space) {
         mtr_read_ulint(rseg_header + TRX_RSEG_HISTORY_SIZE, MLOG_4BYTES, &mtr) +
         1);
 
-    mtr_commit(&mtr);
+    mtr.commit();
 
     ut_ad(rseg->get_curr_size() == 1);
     ut_ad(rseg->trx_ref_count == 0);
@@ -2047,17 +2160,6 @@ bool trx_undo_truncate_tablespace(undo::Tablespace *marked_space) {
     rseg->last_del_marks = FALSE;
   }
 
-  /* If tablespace is to be encrypted, encrypt it now */
-  if (is_encrypted && srv_undo_log_encrypt) {
-    mtr_t mtr;
-    mtr_start(&mtr);
-    ut_d(bool ret =) set_undo_tablespace_encryption(new_space_id, &mtr, false);
-    /* Don't expect any error here (unless keyring plugin is uninstalled). In
-    that case too, continue truncation processing of tablespace. */
-    ut_ad(!ret);
-    mtr_commit(&mtr);
-  }
-
   marked_rsegs->x_unlock();
 
   /* Increment the space ID for this undo space now so that if anyone refers
