@@ -1377,12 +1377,6 @@ static bool backup_rocksdb_checkpoint(Backup_context &context, bool final) {
 @return true if success. */
 bool backup_start(Backup_context &context) {
   if (!opt_no_lock) {
-    if (opt_safe_slave_backup) {
-      if (!wait_for_safe_slave(mysql_connection)) {
-        return (false);
-      }
-    }
-
     if (!backup_files(MySQL_datadir_path.path().c_str(), true)) {
       return (false);
     }
@@ -1444,6 +1438,8 @@ bool backup_start(Backup_context &context) {
 
   context.log_status = log_status_get(mysql_connection);
 
+  debug_sync_point("xtrabackup_after_query_log_status");
+
   if (!write_current_binlog_file(mysql_connection)) {
     return (false);
   }
@@ -2126,7 +2122,7 @@ bool copy_back(int argc, char **argv) {
       return (false);
     }
     auto key = Encryption::get_master_key_id();
-    int ret = fscanf(f, "%lu", &key);
+    int ret = fscanf(f, "%u", &key);
     ut_a(ret == 1);
     fclose(f);
 
@@ -2153,7 +2149,7 @@ bool copy_back(int argc, char **argv) {
 
     my_free(master_key);
 
-    ulint master_key_id;
+    uint32_t master_key_id;
     Encryption::get_master_key(&master_key_id, &master_key);
 
     msg_ts("Generated new master key");
@@ -2540,6 +2536,11 @@ bool decrypt_decompress() {
 
 #ifdef HAVE_VERSION_CHECK
 void version_check() {
+  if (system("which perl > /dev/null 2>&1")) {
+    msg("xtrabackup: perl binary not found. Skipping the version check\n");
+    return;
+  }
+
   if (opt_password != NULL) {
     setenv("option_mysql_password", opt_password, 1);
   }
