@@ -66,8 +66,11 @@ Place, Suite 330, Boston, MA 02111-1307 USA
 #include "xtrabackup_version.h"
 
 #include "backup_mysql.h"
+#include "fsp0fsp.h"
+#include "xb_regex.h"
 
 extern uint opt_ssl_mode;
+extern bool opt_no_server_version_check;
 extern char *opt_ssl_ca;
 extern char *opt_ssl_capath;
 extern char *opt_ssl_cert;
@@ -361,6 +364,25 @@ void parse_show_engine_innodb_status(MYSQL *connection) {
   mysql_free_result(mysql_result);
 }
 
+/* find the pxb base version */
+static unsigned long pxb_base_version() {
+  std::string pxb_base = MYSQL_SERVER_VERSION;
+  unsigned long major = 0, minor = 0, version = 0;
+  std::size_t major_p = pxb_base.find(".");
+  if (major_p != std::string::npos) major = stoi(pxb_base.substr(0, major_p));
+
+  std::size_t minor_p = pxb_base.find(".", major_p + 1);
+  if (minor_p != std::string::npos)
+    minor = stoi(pxb_base.substr(major_p + 1, minor_p - major_p));
+
+  std::size_t version_p = pxb_base.find(".", minor_p + 1);
+  if (version_p != std::string::npos)
+    version = stoi(pxb_base.substr(minor_p + 1, version_p - minor_p));
+  else
+    version = stoi(pxb_base.substr(minor_p + 1));
+  return major * 10000 + minor * 100 + version;
+}
+
 static bool check_server_version(unsigned long version_number,
                                  const char *version_string,
                                  const char *version_comment,
@@ -390,6 +412,7 @@ static bool check_server_version(unsigned long version_number,
   pxb24 = pxb24 || ((version_number > 100000 && version_number < 100300) &&
                     server_flavor == FLAVOR_MARIADB);
 
+
   if (!version_supported) {
     msg("Error: Unsupported server version: '%s'.\n"
         "This version of Percona XtraBackup can only perform backups and "
@@ -405,6 +428,20 @@ static bool check_server_version(unsigned long version_number,
     }
   }
 
+
+  auto pxb_base_ver = pxb_base_version();
+
+  DBUG_EXECUTE_IF("simulate_lower_version", pxb_base_ver = 80014;);
+
+  if (!opt_no_server_version_check && pxb_base_ver < version_number) {
+    msg("Error: Unsupported server version %s.\n"
+        "Please upgrade PXB, if a new "
+        "version is available. To continue with risk, use the option "
+        "--no-server-version-check.\n",
+        mysql_connection->server_version);
+    version_supported = false;
+  }
+
   return (version_supported);
 }
 
@@ -987,9 +1024,9 @@ stop_thread:
 }
 
 static void start_query_killer() {
-  kill_query_thread_stop = os_event_create("kill_query_thread_stop");
-  kill_query_thread_started = os_event_create("kill_query_thread_started");
-  kill_query_thread_stopped = os_event_create("kill_query_thread_stopped");
+  kill_query_thread_stop = os_event_create();
+  kill_query_thread_started = os_event_create();
+  kill_query_thread_stopped = os_event_create();
 
   os_thread_create(PSI_NOT_INSTRUMENTED, kill_query_thread).start();
 
@@ -1138,11 +1175,6 @@ bool lock_tables_maybe(MYSQL *connection, int timeout, int retry_count) {
  Releases the lock acquired with FTWRL/LOCK TABLES FOR BACKUP, depending on
  the locking strategy being used */
 void unlock_all(MYSQL *connection) {
-  if (opt_debug_sleep_before_unlock) {
-    msg_ts("Debug sleep for %u seconds\n", opt_debug_sleep_before_unlock);
-    os_thread_sleep(opt_debug_sleep_before_unlock * 1000);
-  }
-
   if (instance_locked) {
     msg_ts("Executing UNLOCK INSTANCE\n");
     xb_mysql_query(connection, "UNLOCK INSTANCE", false);
@@ -2155,60 +2187,89 @@ void backup_cleanup() {
   }
 }
 
-static ib_mutex_t mdl_lock_con_mutex;
 static MYSQL *mdl_con = NULL;
-
-void mdl_lock_init() {
-  mutex_create(LATCH_ID_XTRA_DATAFILES_ITER_MUTEX, &mdl_lock_con_mutex);
-
-  mdl_con = xb_mysql_connect();
-
-  if (mdl_con != NULL) {
-    xb_mysql_query(mdl_con, "BEGIN", false, true);
-  }
-}
-
-void mdl_lock_table(ulint space_id) {
+void mdl_lock_tables() {
+  msg_ts("Initializing MDL on all current tables.\n");
   MYSQL_RES *mysql_result = NULL;
   MYSQL_ROW row;
-  char *query;
+  mdl_con = xb_mysql_connect();
+  if (mdl_con != NULL) {
+    xb_mysql_query(mdl_con, "BEGIN", false, true);
+    mysql_result = xb_mysql_query(mdl_con,
+                                  "SELECT NAME, SPACE FROM "
+                                  "INFORMATION_SCHEMA.INNODB_TABLES",
+                                  true, true);
+    while ((row = mysql_fetch_row(mysql_result))) {
+      if (fsp_is_ibd_tablespace(atoi(row[1]))) {
+        char full_table_name[MAX_FULL_NAME_LEN + 1];
+        innobase_format_name(full_table_name, sizeof(full_table_name), row[0]);
+        if (is_fts_index(full_table_name)) {
+          // We will eventually get to the row to lock the main table
+          msg_ts("%s is a Full-Text Index. Skipping\n", full_table_name);
+          continue;
+        } else if (is_tmp_table(full_table_name)) {
+          // We cannot run SELECT ... #sql-; Skipped to avoid invalid query.
+          msg_ts("%s is a temporary table. Skipping\n", full_table_name);
+          continue;
+        }
 
-  mutex_enter(&mdl_lock_con_mutex);
+        msg_ts("Locking MDL for %s\n", full_table_name);
+        char *lock_query;
+        xb_a(
+            asprintf(&lock_query, "SELECT 1 FROM %s LIMIT 0", full_table_name));
 
-  xb_a(asprintf(&query,
-                "SELECT NAME FROM INFORMATION_SCHEMA.INNODB_TABLES "
-                "WHERE SPACE = %lu",
-                space_id));
+        xb_mysql_query(mdl_con, lock_query, false, false);
 
-  mysql_result = xb_mysql_query(mdl_con, query, true);
-
-  while ((row = mysql_fetch_row(mysql_result))) {
-    char *lock_query;
-    char table_name[MAX_FULL_NAME_LEN + 1];
-
-    innobase_format_name(table_name, sizeof(table_name), row[0]);
-
-    msg_ts("Locking MDL for %s\n", table_name);
-
-    xb_a(asprintf(&lock_query, "SELECT * FROM %s LIMIT 1", table_name));
-
-    xb_mysql_query(mdl_con, lock_query, false, false);
-
-    free(lock_query);
+        free(lock_query);
+      }
+    }
+    mysql_free_result(mysql_result);
   }
-
-  mysql_free_result(mysql_result);
-  free(query);
-
-  mutex_exit(&mdl_lock_con_mutex);
 }
 
 void mdl_unlock_all() {
   msg_ts("Unlocking MDL for all tables\n");
+  if (mdl_con != NULL) {
+    xb_mysql_query(mdl_con, "COMMIT", false, true);
+    mysql_close(mdl_con);
+  }
+}
+bool is_fts_index(const std::string &tablespace) {
+  const char *pattern =
+      "^(FTS|fts)_[0-9a-fA-f]{16}_(([0-9a-fA-]{16}_(INDEX|index)_[1-6])|"
+      "DELETED_CACHE|deleted_cache|DELETED|deleted|CONFIG|config|BEING_DELETED|"
+      "being_deleted|BEING_DELETED_CACHE|beign_deleted_cache)$";
+  const char *error_context = "is_fts_index";
+  return check_regexp_table_name(tablespace, error_context, pattern);
+}
 
-  xb_mysql_query(mdl_con, "COMMIT", false, true);
+bool is_tmp_table(const std::string &tablespace) {
+  const char *pattern = "^#sql";
+  const char *error_context = "is_tmp_table";
+  return check_regexp_table_name(tablespace, error_context, pattern);
+}
 
-  mutex_free(&mdl_lock_con_mutex);
+bool check_regexp_table_name(std::string tablespace, const char *error_context,
+                             const char *pattern) {
+  bool result = false;
+  get_table_name_from_tablespace(tablespace);
+  xb_regex_t preg;
+  size_t nmatch = 1;
+  xb_regmatch_t pmatch[1];
+  compile_regex(pattern, error_context, &preg);
+
+  if (xb_regexec(&preg, tablespace.c_str(), nmatch, pmatch, 0) != REG_NOMATCH) {
+    result = true;
+  }
+  xb_regfree(&preg);
+  return result;
+}
+void get_table_name_from_tablespace(std::string &tablespace) {
+  std::size_t pos = tablespace.find("`.`");  //`db`.`table` separator
+  if (pos != std::string::npos) {
+    tablespace = tablespace.substr(pos + 3);
+    tablespace.erase(tablespace.size() - 1);  // remove leading `
+  }
 }
 
 bool has_innodb_buffer_pool_dump() {
