@@ -43,10 +43,9 @@ class Redo_Log_Reader {
   @return true if success. */
   bool find_start_checkpoint_lsn();
 
-  /** Find last checkpoint lsn.
-  @param[out] lsn               last checkpoint lsn
-  @return true if success. */
-  bool find_last_checkpoint_lsn(lsn_t *lsn);
+  /** Check if redo logs are disabled during the backup.
+  @return true if redo logs are disabled */
+  bool validate_redo_log_file();
 
   /** Get log header. */
   byte *get_header() const;
@@ -231,7 +230,7 @@ class Archived_Redo_Log_Monitor {
   uint32_t get_first_log_block_checksum() const;
 
   /** Read archived log until the given log block. */
-  void skip_for_block(lsn_t lsn, uint32_t no, uint32_t checksum);
+  void skip_for_block(lsn_t lsn, const byte *redo_buf);
 
  private:
   /** Parse the value of innodb_redo_log_archive_dirs. */
@@ -278,7 +277,7 @@ class Redo_Log_Data_Manager {
   void abort();
 
   /** Stop log copying at specific lsn. */
-  bool stop_at(lsn_t lsn);
+  bool stop_at(lsn_t lsn, lsn_t checkpoint_lsn);
 
   /** Close log file. */
   void close();
