@@ -49,6 +49,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 using namespace xbcloud;
 
 #define XBCLOUD_VERSION XTRABACKUP_VERSION
+#define XBCLOUD_REVISION XTRABACKUP_REVISION
 
 /*****************************************************************************/
 
@@ -86,6 +87,7 @@ static char *opt_s3_endpoint = nullptr;
 static char *opt_s3_access_key = nullptr;
 static char *opt_s3_secret_key = nullptr;
 static char *opt_s3_session_token = nullptr;
+static char *opt_s3_storage_class = nullptr;
 static char *opt_s3_bucket = nullptr;
 static ulong opt_s3_bucket_lookup;
 static ulong opt_s3_api_version = 0;
@@ -95,6 +97,7 @@ static char *opt_google_endpoint = nullptr;
 static char *opt_google_access_key = nullptr;
 static char *opt_google_secret_key = nullptr;
 static char *opt_google_session_token = nullptr;
+static char *opt_google_storage_class = nullptr;
 static char *opt_google_bucket = nullptr;
 
 static std::string backup_name;
@@ -145,6 +148,7 @@ enum {
   OPT_S3_ACCESS_KEY,
   OPT_S3_SECRET_KEY,
   OPT_S3_SESSION_TOKEN,
+  OPT_S3_STORAGE_CLASS,
   OPT_S3_BUCKET,
   OPT_S3_BUCKET_LOOKUP,
   OPT_S3_API_VERSION,
@@ -154,6 +158,7 @@ enum {
   OPT_GOOGLE_ACCESS_KEY,
   OPT_GOOGLE_SECRET_KEY,
   OPT_GOOGLE_SESSION_TOKEN,
+  OPT_GOOGLE_STORAGE_CLASS,
   OPT_GOOGLE_BUCKET,
 
   OPT_PARALLEL,
@@ -173,6 +178,9 @@ static struct my_option my_long_options[] = {
     {"help", '?', "Display this help and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0,
      0, 0, 0, 0, 0},
 
+     {"version", 'V', "Display version and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0,
+      0, 0, 0, 0, 0},
+
     {"storage", OPT_STORAGE, "Specify storage type S3/SWIFT.", &opt_storage,
      &opt_storage, &storage_typelib, GET_ENUM, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
@@ -269,6 +277,14 @@ static struct my_option my_long_options[] = {
      &opt_s3_session_token, &opt_s3_session_token, 0, GET_STR_ALLOC,
      REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
+    {"s3-storage-class", OPT_S3_STORAGE_CLASS,
+     "S3 storage class. STANDARD|STANDARD_IA|GLACIER|...     "
+     "... is meant for passing "
+     "custom storage class names provided by other S3 implementations such as "
+     "MinIO CephRadosGW, etc.",
+     &opt_s3_storage_class, &opt_s3_storage_class, 0, GET_STR_ALLOC,
+     REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+
     {"s3-bucket", OPT_S3_BUCKET, "S3 bucket.", &opt_s3_bucket, &opt_s3_bucket,
      0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
@@ -289,19 +305,24 @@ static struct my_option my_long_options[] = {
      0, 0, 0, 0, 0, 0},
 
     {"google-access-key", OPT_GOOGLE_ACCESS_KEY,
-     "Goolge cloud storage access key.", &opt_google_access_key,
+     "Google cloud storage access key.", &opt_google_access_key,
      &opt_google_access_key, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
     {"google-secret-key", OPT_GOOGLE_SECRET_KEY,
-     "Goolge cloud storage secret key.", &opt_google_secret_key,
+     "Google cloud storage secret key.", &opt_google_secret_key,
      &opt_google_secret_key, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
     {"google-session-token", OPT_GOOGLE_SESSION_TOKEN,
-     "Goolge cloud storage session token.", &opt_google_session_token,
+     "Google cloud storage session token.", &opt_google_session_token,
      &opt_google_session_token, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0,
      0},
 
-    {"google-bucket", OPT_GOOGLE_BUCKET, "Goolge cloud storage bucket.",
+    {"google-storage-class", OPT_GOOGLE_STORAGE_CLASS,
+     "Google cloud storage class. STANDARD|NEARLINE|COLDLINE|ARCHIVE",
+     &opt_google_storage_class, &opt_google_storage_class, 0, GET_STR_ALLOC,
+     REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+
+    {"google-bucket", OPT_GOOGLE_BUCKET, "Google cloud storage bucket.",
      &opt_google_bucket, &opt_google_bucket, 0, GET_STR_ALLOC, REQUIRED_ARG, 0,
      0, 0, 0, 0, 0},
 
@@ -323,8 +344,8 @@ static struct my_option my_long_options[] = {
     {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
 
 static void print_version() {
-  printf("%s  Ver %s for %s (%s)\n", my_progname, XBCLOUD_VERSION, SYSTEM_TYPE,
-         MACHINE_TYPE);
+  printf("%s  Ver %s for %s (%s) (revision id: %s)\n", my_progname, XBCLOUD_VERSION, SYSTEM_TYPE,
+         MACHINE_TYPE, XBCLOUD_REVISION);
 }
 
 static void usage() {
@@ -362,6 +383,9 @@ static bool get_one_option(int optid,
     case '?':
       usage();
       exit(0);
+    case 'V':
+      print_version();
+      exit(0);
     case OPT_SWIFT_PASSWORD:
     case OPT_SWIFT_KEY:
     case OPT_SWIFT_TENANT:
@@ -415,6 +439,7 @@ static void get_env_args() {
   get_env_value(opt_s3_access_key, "S3_ACCESS_KEY_ID");
   get_env_value(opt_s3_secret_key, "S3_SECRET_ACCESS_KEY");
   get_env_value(opt_s3_session_token, "S3_SESSION_TOKEN");
+  get_env_value(opt_s3_storage_class, "S3_STORAGE_CLASS");
   get_env_value(opt_s3_region, "S3_DEFAULT_REGION");
   get_env_value(opt_cacert, "S3_CA_BUNDLE");
   get_env_value(opt_s3_endpoint, "S3_ENDPOINT");
@@ -422,6 +447,7 @@ static void get_env_args() {
   get_env_value(opt_s3_access_key, "AWS_ACCESS_KEY_ID");
   get_env_value(opt_s3_secret_key, "AWS_SECRET_ACCESS_KEY");
   get_env_value(opt_s3_session_token, "AWS_SESSION_TOKEN");
+  get_env_value(opt_s3_storage_class, "AWS_STORAGE_CLASS");
   get_env_value(opt_s3_region, "AWS_DEFAULT_REGION");
   get_env_value(opt_cacert, "AWS_CA_BUNDLE");
   get_env_value(opt_s3_endpoint, "AWS_ENDPOINT");
@@ -698,20 +724,43 @@ bool xbcloud_put(Object_store *store, const std::string &container,
 
   xb_stream_read_done(stream);
 
-  bool ret = (res != XB_STREAM_READ_ERROR) && (!has_errors);
 
-  if (ret && upload_md5) {
-    msg_ts("%s: Uploading md5\n", my_progname);
-    ret = store->upload_object(container, backup_name + ".md5", buf_md5);
-  }
-
-  if (!ret) {
+  // check if backup directory exists and it has some files
+  if (res == XB_STREAM_READ_ERROR || has_errors ||
+      !store->list_objects_in_directory(container, backup_name, object_list) ||
+      object_list.size() == 0) {
     msg_ts("%s: Upload failed.\n", my_progname);
-  } else {
-    msg_ts("%s: Upload completed.\n", my_progname);
+    return false;
   }
 
-  return ret;
+  /* check if the last_file (xtrabackup_tablespaces.gz.00000000000 or
+  xtrabackup_tablespaces.00000000000000000000) is uploaded to cloud storage to
+  determine successful xbcloud "put" operation. */
+  std::string last_file_prefix = backup_name + "/xtrabackup_tablespaces";
+  bool file_found = false;
+
+  if (!object_list.empty()) {
+    auto cur_file = object_list.back();
+    auto last_file_size = last_file_prefix.size();
+    if (cur_file.size() >= last_file_size &&
+        cur_file.substr(0, last_file_size).compare(last_file_prefix) == 0)
+      file_found = true;
+  }
+  if (!file_found) {
+    msg_ts("%s: Upload failed: backup is incomplete.\n", my_progname);
+    return false;
+  }
+
+  if (upload_md5) {
+    msg_ts("%s: Uploading md5\n", my_progname);
+    if (!store->upload_object(container, backup_name + ".md5", buf_md5)) {
+      msg_ts("%s: Upload failed: Error uploading md5.\n", my_progname);
+      return false;
+    }
+  }
+
+  msg_ts("%s: Upload completed.\n", my_progname);
+  return true;
 }
 
 bool chunk_name_to_file_name(const std::string &chunk_name,
@@ -1067,9 +1116,11 @@ int main(int argc, char **argv) {
         opt_s3_secret_key != nullptr ? opt_s3_secret_key : "";
     std::string session_token =
         opt_s3_session_token != nullptr ? opt_s3_session_token : "";
+    std::string storage_class =
+        opt_s3_storage_class != nullptr ? opt_s3_storage_class : "";
     object_store = std::unique_ptr<Object_store>(new S3_object_store(
         &http_client, region, access_key, secret_key, session_token,
-        opt_s3_endpoint != nullptr ? opt_s3_endpoint : "",
+        storage_class, opt_s3_endpoint != nullptr ? opt_s3_endpoint : "",
         static_cast<s3_bucket_lookup_t>(opt_s3_bucket_lookup),
         static_cast<s3_api_version_t>(opt_s3_api_version)));
 
@@ -1096,11 +1147,15 @@ int main(int argc, char **argv) {
         opt_google_secret_key != nullptr ? opt_google_secret_key : "";
     std::string session_token =
         opt_google_session_token != nullptr ? opt_google_session_token : "";
+    std::string storage_class =
+        opt_google_storage_class != nullptr ? opt_google_storage_class : "";
+
     object_store = std::unique_ptr<Object_store>(new S3_object_store(
         &http_client, region, access_key, secret_key, session_token,
+        storage_class,
         opt_google_endpoint != nullptr ? opt_google_endpoint
                                        : "https://storage.googleapis.com/",
-        LOOKUP_DNS, S3_V2));
+        LOOKUP_DNS, S3_V4));
 
     if (opt_google_bucket == nullptr) {
       msg_ts("%s: Google bucket is not specified.\n", my_progname);
