@@ -34,11 +34,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 #include "ds_decompress.h"
 #include "ds_decompress_lz4.h"
 #include "ds_decrypt.h"
+#include "file_utils.h"
 #include "template_utils.h"
 #include "xbcrypt_common.h"
 #include "xtrabackup_version.h"
 
 #define XBSTREAM_VERSION XTRABACKUP_VERSION
+#define XBSTREAM_REVISION XTRABACKUP_REVISION
 #define XBSTREAM_BUFFER_SIZE (10 * 1024 * 1024UL)
 
 typedef enum { RUN_MODE_NONE, RUN_MODE_CREATE, RUN_MODE_EXTRACT } run_mode_t;
@@ -68,12 +70,15 @@ static char *opt_encrypt_key = NULL;
 static int opt_encrypt_threads = 1;
 static bool opt_decompress = 0;
 static uint opt_decompress_threads = 1;
+static bool opt_absolute_names = 0;
 
 enum { OPT_DECOMPRESS = 256, OPT_DECOMPRESS_THREADS, OPT_ENCRYPT_THREADS };
 
 static struct my_option my_long_options[] = {
     {"help", '?', "Display this help and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0,
      0, 0, 0, 0, 0},
+    {"version", 'V', "Display version and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG,
+     0, 0, 0, 0, 0, 0},
     {"create", 'c', "Stream the specified files to the standard output.", 0, 0,
      0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"decompress", OPT_DECOMPRESS, "Decompress individual backup files.",
@@ -110,6 +115,10 @@ static struct my_option my_long_options[] = {
      "The default value is 1.",
      &opt_encrypt_threads, &opt_encrypt_threads, 0, GET_INT, REQUIRED_ARG, 1, 1,
      INT_MAX, 0, 0, 0},
+    {"absolute-names", 'P',
+     "Don't strip leading slashes from file names when creating archives.",
+     &opt_absolute_names, &opt_absolute_names, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0,
+     0, 0},
 
     {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
 
@@ -194,8 +203,8 @@ static int get_options(int *argc, char ***argv) {
 }
 
 static void print_version(void) {
-  printf("%s  Ver %s for %s (%s)\n", my_progname, XBSTREAM_VERSION, SYSTEM_TYPE,
-         MACHINE_TYPE);
+  printf("%s  Ver %s for %s (%s) (revision id: %s)\n", my_progname,
+         XBSTREAM_VERSION, SYSTEM_TYPE, MACHINE_TYPE, XBSTREAM_REVISION);
 }
 
 static void usage(void) {
@@ -250,6 +259,9 @@ static bool get_one_option(int optid,
     case 'k':
       hide_option(argument, &opt_encrypt_key);
       break;
+    case 'V':
+      print_version();
+      exit(0);
     case '?':
       usage();
       exit(0);
@@ -306,6 +318,8 @@ static int mode_create(int argc, char **argv) {
 
   for (i = 0; i < argc; i++) {
     char *filepath = argv[i];
+    const char *filepath_dst;
+    int filepath_prefix_len;
     File src_file;
     xb_wstream_file_t *file;
 
@@ -322,7 +336,10 @@ static int mode_create(int argc, char **argv) {
       goto err;
     }
 
-    file = xb_stream_write_open(stream, filepath, &mystat, NULL, NULL);
+    filepath_dst = opt_absolute_names
+                       ? filepath
+                       : safer_name_suffix(filepath, &filepath_prefix_len);
+    file = xb_stream_write_open(stream, filepath_dst, &mystat, NULL, NULL);
     if (file == NULL) {
       goto err;
     }
@@ -445,6 +462,18 @@ static void *extract_worker_thread_func(void *arg) {
       continue;
     }
 
+    if (!opt_absolute_names) {
+      int filepath_prefix_len;
+      safer_name_suffix(chunk.path, &filepath_prefix_len);
+      if (filepath_prefix_len != 0) {
+        msg("%s: absolute path not allowed: %.*s.\n", my_progname,
+            chunk.pathlen, chunk.path);
+        res = XB_STREAM_READ_ERROR;
+        pthread_mutex_unlock(ctxt->mutex);
+        break;
+      }
+    }
+
     /* See if we already have this file open */
     entry = ctxt->filehash[chunk.path];
 
