@@ -783,6 +783,13 @@ $MYSQL_VERSION_MINOR $MYSQL_VERSION_PATCH`
     [[ $server_str > $version_str ]]
 }
 
+function is_debug_server()
+{
+
+[[ $MYSQL_VERSION =~ .*debug ]]
+
+}
+
 #########################################################################
 # Require a server version higher than the first argument
 ########################################################################
@@ -792,6 +799,16 @@ function require_server_version_higher_than()
         skip_test "Requires server version higher than $1"
 }
 
+#########################################################################
+# Requires debug pxb version
+########################################################################
+function require_debug_pxb_version()
+{
+    if ! $XB_BIN --help 2>&1 | grep -q debug.=name; then
+        skip_test "Requires debug build"
+    fi
+}
+
 ########################################################################
 # Return 0 if the server version is lower than the first argument
 #########################################################################
@@ -1103,6 +1120,11 @@ function innodb_wait_for_flush_all()
     while ! innodb_n_dirty_pages ; do
         sleep 1
     done
+
+    flushed_lsn=`innodb_flushed_lsn`
+    while [ `innodb_checkpoint_lsn` -lt "$flushed_lsn" ] ; do
+      sleep 1
+    done
 }
 
 # Return current LSN
