@@ -4,12 +4,12 @@
 
 start_server
 
-err=$(${XB_BIN} ${XB_ARGS} --backup --skip-myoption --target-dir=$topdir/backup 2>&1 | grep "unknown option")
+err=$(${XB_BIN} ${XB_ARGS} --strict --backup --skip-myoption --target-dir=$topdir/backup 2>&1 | grep "unknown option")
 if [ -z "$err" ] ; then
     die "xtrabackup did not report error (--myoption)"
 fi
 
-err=$(${XB_BIN} ${XB_ARGS} --backup --compress --copmress-threads=4 --target-dir=$topdir/backup 2>&1 | grep "unknown variable")
+err=$(${XB_BIN} ${XB_ARGS} --strict --backup --compress --copmress-threads=4 --target-dir=$topdir/backup 2>&1 | grep "unknown variable")
 if [ -z "$err" ] ; then
     die "xtrabackup did not report error (--copmress-threads)"
 fi
@@ -26,7 +26,7 @@ if [ -z "$err" ] ; then
     die "xtrabackup did not report error (--myvariable)"
 fi
 
-err=$(${XB_BIN} ${XB_ARGS} --skip-strict --backup --skip-myoption --target-dir=$topdir/backup 2>&1 | grep "unknown option")
+err=$(${XB_BIN} ${XB_ARGS} --backup --skip-myoption --target-dir=$topdir/backup 2>&1 | grep "WARNING: unknown option")
 if [ -z "$err" ] ; then
     die "xtrabackup did not report warning (--myoption)"
 fi
