@@ -64,7 +64,7 @@ ulonglong myisam_recover_options;
 static ulong opt_myisam_block_size;
 
 /* Interface to mysqld, to check system tables supported by SE */
-static bool myisam_is_supported_system_table(const char *,
+static bool myisam_is_supported_system_table(const char *db,
                                              const char *table_name,
                                              bool is_sql_layer_system_table);
 
@@ -294,7 +294,7 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
              (type == HA_KEYTYPE_BINARY && !field->zero_pack()))) {
           /* No blobs here */
           if (j == 0) keydef[i].flag |= HA_PACK_KEY;
-          if (!(field->flags & ZEROFILL_FLAG) &&
+          if (!field->is_flag_set(ZEROFILL_FLAG) &&
               (field->type() == MYSQL_TYPE_STRING ||
                field->type() == MYSQL_TYPE_VAR_STRING ||
                ((int)(pos->key_part[j].length - field->decimals())) >= 4))
@@ -366,7 +366,7 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
     }
     if (!found) break;
 
-    if (found->flags & BLOB_FLAG)
+    if (found->is_flag_set(BLOB_FLAG))
       recinfo_pos->type = (int)FIELD_BLOB;
     else if (found->type() == MYSQL_TYPE_VARCHAR)
       recinfo_pos->type = FIELD_VARCHAR;
@@ -376,7 +376,7 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
       recinfo_pos->type = (int)FIELD_SKIP_ZERO;
     else
       recinfo_pos->type =
-          (int)((length <= 3 || (found->flags & ZEROFILL_FLAG))
+          (int)((length <= 3 || (found->is_flag_set(ZEROFILL_FLAG)))
                     ? FIELD_NORMAL
                     : found->type() == MYSQL_TYPE_STRING ||
                               found->type() == MYSQL_TYPE_VAR_STRING
@@ -659,66 +659,52 @@ static const char *ha_myisam_exts[] = {".MYI", ".MYD", NullS};
 /**
   @brief Check if the given db.tablename is a system table for this SE.
 
+  @param db                         database name.
   @param table_name                 table name to check.
   @param is_sql_layer_system_table  if the supplied db.table_name is a SQL
                                     layer system table.
 
-  @note As for 5.7, mysql doesn't support MyISAM as an engine for the following
-        system tables: columns_priv, db, procs_priv, proxies_priv, tables_priv,
-        user.
+  @note As for 8.0, MySQL doesn't support MyISAM as an engine for the system
+        tables. Altering engine of system table to MyISAM is not allowed.
+        Creating system tables in MyISAM is allowed to upgrade from older
+        versions through mysqldump.
 
-  @note In case there is a need to define MYISAM specific system
-        database, then please see reference implementation in
-        ha_example.cc.
+  @note In case there is a need to define MYISAM specific system database, then
+        please see reference implementation in ha_example.cc.
 
   @retval true   Given db.table_name is supported system table.
   @retval false  Given db.table_name is not a supported system table.
 */
 
-static bool myisam_is_supported_system_table(const char *,
+static bool myisam_is_supported_system_table(const char *db,
                                              const char *table_name,
                                              bool is_sql_layer_system_table) {
+  /*
+    Currently MYISAM does not support any other SE specific system tables. If
+    in future it does, please see ha_example.cc for reference implementation
+  */
+  if (!is_sql_layer_system_table) return false;
+
+  /*
+    During upgrade, server side execution of SQL statement is allowed to use
+    this SE for system tables.
+  */
   THD *thd = current_thd;
+  if (thd->is_server_upgrade_thread()) return true;
 
-  if (thd->lex->sql_command == SQLCOM_CREATE_TABLE ||
-      thd->lex->sql_command == SQLCOM_ALTER_TABLE) {
-    /*
-      We allow creation of ACL tables in MyISAM to allow upgrade from
-      older versions through mysqldump and downgrade.
-    */
-    // Does MYISAM support "ALL" SQL layer system tables ?
-    if (is_sql_layer_system_table) return true;
-
-    /*
-      Currently MYISAM does not support any other SE specific
-      system tables. If in future it does, please see ha_example.cc
-      for reference implementation
-    */
-
-    return false;
-  } else {
-    static const char *unsupported_system_tables[] = {
-        "columns_priv", "db",   "procs_priv",         "proxies_priv",
-        "tables_priv",  "user", (const char *)nullptr};
-
-    if (is_sql_layer_system_table) {
-      for (unsigned i = 0; unsupported_system_tables[i] != nullptr; ++i) {
-        if (!strcmp(table_name, unsupported_system_tables[i]))
-          // Doesn't support MYISAM for this table name
-          return false;
-      }
-      // Support MYISAM for other system tables not listed explicitly
-      return true;
-    }
-
-    /*
-      Currently MYISAM does not support any other SE specific
-      system tables. If in future it does, please see ha_example.cc
-      for reference implementation
-    */
-
-    return false;
+  /*
+    For client side SQL, creating system tables in this SE is allowed to support
+    logical upgrade.
+  */
+  if (thd->lex->sql_command == SQLCOM_CREATE_TABLE) {
+    push_warning_printf(thd, Sql_condition::SL_WARNING, ER_UNSUPPORTED_ENGINE,
+                        ER_THD(thd, ER_UNSUPPORTED_ENGINE), "MyISAM", db,
+                        table_name);
+    return true;
   }
+
+  // Any other usage is not allowed.
+  return false;
 }
 
 /* Name is here without an extension */
