@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,12 +29,13 @@
 
 #define JAM_FILE_ID 185
 
+
 class AttrInfo {
   /**
    * Sender(s)
    */
   friend class DbUtil;
-
+  
   /**
    * Receiver(s)
    */
@@ -50,14 +51,14 @@ class AttrInfo {
   friend class NdbOperation;
 
   friend bool printATTRINFO(FILE *, const Uint32 *, Uint32, Uint16);
+  
+public:
+  STATIC_CONST( HeaderLength = 3 );
+  STATIC_CONST( DataLength = 22 );
+  STATIC_CONST( MaxSignalLength = HeaderLength + DataLength );
+  STATIC_CONST( SectionSizeInfoLength = 5 );
 
- public:
-  static constexpr Uint32 HeaderLength = 3;
-  static constexpr Uint32 DataLength = 22;
-  static constexpr Uint32 MaxSignalLength = HeaderLength + DataLength;
-  static constexpr Uint32 SectionSizeInfoLength = 5;
-
- private:
+private:
   Uint32 connectPtr;
   Uint32 transId[2];
   Uint32 attrData[DataLength];
@@ -73,9 +74,9 @@ class AttrInfo {
   signal. For SCAN_TABREQ, all attribute info words are sent in ATTRINFO
   signals).
 
-  For SCAN_TABREQ, and TCKEYREQ for read or update with interpreted code,
-  the attribute information can have up to five sections. The initial
-  five words of the stream defines the length of the sections, followed
+  For SCAN_TABREQ, and TCKEYREQ for read or update with interpreted code, 
+  the attribute information can have up to five sections. The initial 
+  five words of the stream defines the length of the sections, followed 
   by the words of each section in sequence.
 
   The sections are:
@@ -96,6 +97,7 @@ class AttrInfo {
   all attributes in the underlying table being indexed are accessible.
 */
 
+
 #undef JAM_FILE_ID
 
 #endif
