@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,11 +25,12 @@
 #ifndef BACKUP_IMPL_HPP
 #define BACKUP_IMPL_HPP
 
-#include <NodeBitmask.hpp>
 #include "SignalData.hpp"
+#include <NodeBitmask.hpp>
 
 #define JAM_FILE_ID 78
 
+
 /*
  * DefineBackupReq
  *
@@ -42,7 +43,7 @@ class DefineBackupReq {
    * Sender(s)
    */
   friend class BackupMaster;
-
+  
   /**
    * Reciver(s)
    */
@@ -51,11 +52,10 @@ class DefineBackupReq {
   friend class Dblqh;
 
   friend bool printDEFINE_BACKUP_REQ(FILE *, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength_v1 = 11 + NdbNodeBitmask48::Size);
 
- public:
-  static constexpr Uint32 SignalLength_v1 = 11 + NdbNodeBitmask48::Size;
-
- private:
+private:
   /**
    * i - value of backup object
    */
@@ -71,12 +71,12 @@ class DefineBackupReq {
    * Note: Only to support versions < 8.0.18
    */
   NdbNodeBitmask48 nodes;
-
+  
   /**
    * Generated random number
    */
   Uint32 backupKey[2];
-
+  
   /**
    * Length of backup data
    */
@@ -110,10 +110,9 @@ class DefineBackupRef {
   friend class BackupMaster;
 
   friend bool printDEFINE_BACKUP_REF(FILE *, const Uint32 *, Uint32, Uint16);
-
- public:
-  static constexpr Uint32 SignalLength = 4;
-
+public:
+  STATIC_CONST( SignalLength = 4 );
+  
   enum ErrorCode {
     Undefined = 1340,
     FailedToAllocateBuffers = 1342,
@@ -126,8 +125,7 @@ class DefineBackupRef {
     FailedToAllocateAttributeRecord = 1349,
     FailedForBackupFilesAleadyExist = 1350
   };
-
- private:
+private:
   Uint32 backupId;
   Uint32 backupPtr;
   Uint32 errorCode;
@@ -148,11 +146,10 @@ class DefineBackupConf {
   friend class BackupMaster;
 
   friend bool printDEFINE_BACKUP_CONF(FILE *, const Uint32 *, Uint32, Uint16);
-
- public:
-  static constexpr Uint32 SignalLength = 2;
-
- private:
+public:
+  STATIC_CONST( SignalLength = 2 );
+  
+private:
   Uint32 backupId;
   Uint32 backupPtr;
 };
@@ -162,7 +159,7 @@ class StartBackupReq {
    * Sender(s)
    */
   friend class BackupMaster;
-
+  
   /**
    * Reciver(s)
    */
@@ -170,11 +167,11 @@ class StartBackupReq {
   friend class BackupProxy;
 
   friend bool printSTART_BACKUP_REQ(FILE *, const Uint32 *, Uint32, Uint16);
+public:
 
- public:
-  static constexpr Uint32 SignalLength = 4;
+  STATIC_CONST( SignalLength = 4 );
 
- private:
+private:
   Uint32 backupId;
   Uint32 backupPtr;
   Uint32 senderRef;
@@ -186,7 +183,7 @@ class StartBackupRef {
    * Sender(s)
    */
   friend class Backup;
-
+  
   /**
    * Reciver(s)
    */
@@ -194,16 +191,14 @@ class StartBackupRef {
   friend class BackupProxy;
 
   friend bool printSTART_BACKUP_REF(FILE *, const Uint32 *, Uint32, Uint16);
-
- public:
-  static constexpr Uint32 SignalLength = 4;
+public:
+  STATIC_CONST( SignalLength = 4 );
 
   enum ErrorCode {
     FailedToAllocateTriggerRecord = 1,
     FailedStartSinceDefineFailed = 1351
   };
-
- private:
+private:
   Uint32 backupId;
   Uint32 backupPtr;
   Uint32 errorCode;
@@ -215,18 +210,17 @@ class StartBackupConf {
    * Sender(s)
    */
   friend class Backup;
-
+  
   /**
    * Reciver(s)
    */
   friend class BackupMaster;
 
   friend bool printSTART_BACKUP_CONF(FILE *, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength = 2 );
 
- public:
-  static constexpr Uint32 SignalLength = 2;
-
- private:
+private:
   Uint32 backupId;
   Uint32 backupPtr;
 };
@@ -237,7 +231,7 @@ class BackupFragmentReq {
    */
   friend class BackupMaster;
   friend class BackupProxy;
-
+  
   /**
    * Reciver(s)
    */
@@ -245,11 +239,10 @@ class BackupFragmentReq {
   friend class Dblqh;
 
   friend bool printBACKUP_FRAGMENT_REQ(FILE *, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength = 7 );
 
- public:
-  static constexpr Uint32 SignalLength = 7;
-
- private:
+private:
   Uint32 backupId;
   Uint32 backupPtr;
   Uint32 tableId;
@@ -265,7 +258,7 @@ class BackupFragmentRef {
    */
   friend class Backup;
   friend class Dblqh;
-
+  
   /**
    * Reciver(s)
    */
@@ -273,11 +266,10 @@ class BackupFragmentRef {
   friend class BackupProxy;
 
   friend bool printBACKUP_FRAGMENT_REF(FILE *, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength = 5 );
 
- public:
-  static constexpr Uint32 SignalLength = 5;
-
- private:
+private:
   Uint32 backupId;
   Uint32 backupPtr;
   Uint32 errorCode;
@@ -290,18 +282,17 @@ class BackupFragmentConf {
    */
   friend class Backup;
   friend class Dblqh;
-
+  
   /**
    * Reciver(s)
    */
   friend class BackupMaster;
 
   friend bool printBACKUP_FRAGMENT_CONF(FILE *, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength = 8 );
 
- public:
-  static constexpr Uint32 SignalLength = 8;
-
- private:
+private:
   Uint32 backupId;
   Uint32 backupPtr;
   Uint32 tableId;
@@ -313,8 +304,8 @@ class BackupFragmentConf {
 };
 
 class BackupFragmentCompleteRep {
- public:
-  static constexpr Uint32 SignalLength = 8;
+public:
+  STATIC_CONST( SignalLength = 8 );
 
   Uint32 backupId;
   Uint32 backupPtr;
@@ -331,7 +322,7 @@ class StopBackupReq {
    * Sender(s)
    */
   friend class BackupMaster;
-
+  
   /**
    * Reciver(s)
    */
@@ -339,11 +330,10 @@ class StopBackupReq {
   friend class BackupProxy;
 
   friend bool printSTOP_BACKUP_REQ(FILE *, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength = 6 );
 
- public:
-  static constexpr Uint32 SignalLength = 6;
-
- private:
+private:
   Uint32 backupId;
   Uint32 backupPtr;
   Uint32 startGCP;
@@ -357,7 +347,7 @@ class StopBackupRef {
    * Sender(s)
    */
   friend class Backup;
-
+  
   /**
    * Reciver(s)
    */
@@ -365,11 +355,10 @@ class StopBackupRef {
   friend class BackupProxy;
 
   friend bool printSTOP_BACKUP_REF(FILE *, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength = 4 );
 
- public:
-  static constexpr Uint32 SignalLength = 4;
-
- private:
+private:
   Uint32 backupId;
   Uint32 backupPtr;
   Uint32 errorCode;
@@ -381,18 +370,17 @@ class StopBackupConf {
    * Sender(s)
    */
   friend class Backup;
-
+  
   /**
    * Reciver(s)
    */
   friend class BackupMaster;
 
   friend bool printSTOP_BACKUP_CONF(FILE *, const Uint32 *, Uint32, Uint16);
-
- public:
-  static constexpr Uint32 SignalLength = 4;
-
- private:
+public:
+  STATIC_CONST( SignalLength = 4 );
+  
+private:
   Uint32 backupId;
   Uint32 backupPtr;
   Uint32 noOfLogBytes;
@@ -404,18 +392,17 @@ class BackupStatusReq {
    * Sender(s)
    */
   friend class BackupMaster;
-
+  
   /**
    * Reciver(s)
    */
   friend class Backup;
 
   friend bool printBACKUP_STATUS_REQ(FILE *, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength = 1 );
 
- public:
-  static constexpr Uint32 SignalLength = 1;
-
- private:
+private:
 };
 
 class BackupStatusConf {
@@ -423,20 +410,21 @@ class BackupStatusConf {
    * Sender(s)
    */
   friend class Backup;
-
+  
   /**
    * Reciver(s)
    */
   friend class BackupMaster;
 
   friend bool printBACKUP_STATUS_CONF(FILE *, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength = 1 );
 
- public:
-  static constexpr Uint32 SignalLength = 1;
-
- private:
+private:
 };
 
+
+
 #undef JAM_FILE_ID
 
 #endif
