@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,7 @@
 
 #define JAM_FILE_ID 175
 
+
 class CopyActiveReq {
   /**
    * Sender(s)
@@ -39,13 +40,17 @@ class CopyActiveReq {
    * Receiver(s)
    */
   friend class Dblqh;
+public:
+  STATIC_CONST( SignalLength = 6 );
 
- public:
-  static constexpr Uint32 SignalLength = 6;
+  enum Flags
+  {
+    CAR_NO_WAIT = 0x1
+    ,CAR_NO_LOGGING = 0x2
+    ,CAR_LOCAL_SEND = 0x4
+  };
 
-  enum Flags { CAR_NO_WAIT = 0x1, CAR_NO_LOGGING = 0x2, CAR_LOCAL_SEND = 0x4 };
-
- private:
+private:
   Uint32 userPtr;
   Uint32 userRef;
   Uint32 tableId;
@@ -64,11 +69,10 @@ class CopyActiveConf {
    * Receiver(s)
    */
   friend class Dbdih;
+public:
+  STATIC_CONST( SignalLength = 5 );
 
- public:
-  static constexpr Uint32 SignalLength = 5;
-
- private:
+private:
   Uint32 userPtr;
   Uint32 startingNodeId;
   Uint32 tableId;
@@ -85,11 +89,10 @@ class CopyActiveRef {
    * Receiver(s)
    */
   friend class Dbdih;
+public:
+  STATIC_CONST( SignalLength = 5 );
 
- public:
-  static constexpr Uint32 SignalLength = 5;
-
- private:
+private:
   Uint32 userPtr;
   Uint32 startingNodeId;
   Uint32 tableId;
@@ -97,6 +100,7 @@ class CopyActiveRef {
   Uint32 errorCode;
 };
 
+
 #undef JAM_FILE_ID
 
 #endif
