@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,13 +25,14 @@
 #ifndef DIGETNODES_HPP
 #define DIGETNODES_HPP
 
-#include <ndb_limits.h>
 #include <NodeBitmask.hpp>
+#include <ndb_limits.h>
 
 #define JAM_FILE_ID 90
 
+
 /**
- *
+ * 
  */
 struct DiGetNodesConf {
   /**
@@ -47,17 +48,17 @@ struct DiGetNodesConf {
    */
   friend class Dbdih;
 
-  static constexpr Uint32 SignalLength = 4 + MAX_REPLICAS;
-  static constexpr Uint32 REORG_MOVING = 0x80000000;
+  STATIC_CONST( SignalLength = 4 + MAX_REPLICAS );
+  STATIC_CONST( REORG_MOVING = 0x80000000);
 
   Uint32 zero;
   Uint32 fragId;
   Uint32 reqinfo;
   Uint32 instanceKey;
-  Uint32 nodes[MAX_REPLICAS + (3 + MAX_REPLICAS)];  //+1
+  Uint32 nodes[MAX_REPLICAS + (3 + MAX_REPLICAS)]; //+1
 };
 /**
- *
+ * 
  */
 class DiGetNodesReq {
   /**
@@ -71,12 +72,10 @@ class DiGetNodesReq {
    * Receiver(s)
    */
   friend class Dbdih;
-
- public:
-  static constexpr Uint32 SignalLength = 6 + (sizeof(void *) / sizeof(Uint32));
-  static constexpr Uint32 MAX_DIGETNODESREQS = 16;
-
- private:
+public:
+  STATIC_CONST( SignalLength = 6 + (sizeof(void*) / sizeof(Uint32)) );
+  STATIC_CONST(MAX_DIGETNODESREQS = 16);
+private:
   Uint32 tableId;
   Uint32 hashValue;
   Uint32 distr_key_indicator;
@@ -84,11 +83,12 @@ class DiGetNodesReq {
   Uint32 get_next_fragid_indicator;
   Uint32 anyNode;
   union {
-    void *jamBufferPtr;
+    void * jamBufferPtr;
     Uint32 jamBufferStorage[2];
   };
 };
 
+
 #undef JAM_FILE_ID
 
 #endif
