@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -86,7 +86,7 @@ public:
   /**
    * Length of signal
    */
-  static constexpr Uint32 FixedLength = 6;
+  STATIC_CONST( FixedLength = 6 );
 
 private:
 
@@ -104,7 +104,8 @@ private:
 // Variable sized part. Those will contain 
 // info about memory/file pages to read/write
 //-------------------------------------------------------------  
-  union { // DATA 6 - 21
+  union {
+    UintR pageData[16];        // DATA 6 - 21
     struct {
       Uint32 varIndex;   // In unit cluster size
       Uint32 fileOffset; // In unit page size
@@ -119,15 +120,9 @@ private:
       Uint32 size;
     } memoryAddress;
     struct {
+      Uint32 varIndex[1]; // Size = numberOfPages
       Uint32 fileOffset;
-      Uint32 varIndex[15]; // Size = numberOfPages
     } listOfMemPages;
-    struct {
-      Uint32 pageNumber;
-    } globalPage;
-    struct {
-      Uint32 pageNumber;
-    } sharedPage;
   } data;
 
   static Uint8 getSyncFlag(const UintR & opFlag);
@@ -140,9 +135,6 @@ private:
   static void setPartialReadFlag(UintR & opFlag, Uint32 flag);
 };
 
-DECLARE_SIGNAL_SCOPE(GSN_FSREADREQ, Local);
-DECLARE_SIGNAL_SCOPE(GSN_FSWRITEREQ, Local);
-
 /**
  * Operation flag
  *
@@ -205,7 +197,7 @@ struct FsSuspendOrd
   UintR filePointer;          // DATA 0
   Uint32 milliseconds;
 
-  static constexpr Uint32 SignalLength = 2;
+  STATIC_CONST(SignalLength = 2);
 };
 
 
