@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +29,9 @@
 
 #define JAM_FILE_ID 180
 
+
 class SystemError {
+
   /**
    * Reciver(s)
    */
@@ -45,11 +46,10 @@ class SystemError {
   /**
    * For printing
    */
-  friend bool printSYSTEM_ERROR(FILE *output, const Uint32 *theData, Uint32 len,
-                                Uint16 receiverBlockNo);
+  friend bool printSYSTEM_ERROR(FILE * output, const Uint32 * theData, Uint32 len, Uint16 receiverBlockNo);
 
- public:
-  static constexpr Uint32 SignalLength = 4;
+public:
+  STATIC_CONST( SignalLength = 4 );
 
   enum ErrorCode {
     GCPStopDetected = 3,
@@ -60,12 +60,14 @@ class SystemError {
     StartFragRefError = 9,
     ProtocolError = 10
   };
-
+  
   Uint32 errorRef;
   Uint32 errorCode;
   Uint32 data[1];
 };
 
+
 #undef JAM_FILE_ID
 
 #endif
+
