@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2007, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -99,7 +99,20 @@ public:
                      Uint32 *buffer= 0, 
                      Uint32 buffer_word_size= 0);
 
+  /* Constructor variant that obtains table from NdbRecord
+  */
+  NdbInterpretedCode(const NdbRecord &,
+                     Uint32 *buffer= 0,
+                     Uint32 buffer_word_size= 0);
+
+
   ~NdbInterpretedCode();
+
+  /**
+   * Discard any NdbInterpreter program constructed so far
+   * and allow construction to start over again.
+   */
+  void reset();
   
   /* Register constant loads
    * -----------------------
@@ -202,7 +215,7 @@ public:
    * @param RegLValue register to use as left hand side of condition
    * @param RegRValue register to use as right hand side of condition
    * @param Label Program label to jump to if condition is true
-   * @return 0 if successfull, -1 otherwise.
+   * @return 0 if successful, -1 otherwise.
    */
   int branch_ge(Uint32 RegLvalue, Uint32 RegRvalue, Uint32 Label);
   int branch_gt(Uint32 RegLvalue, Uint32 RegRvalue, Uint32 Label);
@@ -259,6 +272,24 @@ public:
                     Uint32 Label);
   int branch_col_ge(const void * val, Uint32 unused, Uint32 attrId,
                     Uint32 Label);
+
+  /* Variants of above methods allowing us to compare two Attr
+   * from the same table. Both Attr's has to be of the exact same
+   * data type, including length, precision, scale, etc.
+   *
+   * NOTE that NULL values are compared such that NULL is
+   * less that any non-NULL value, and NULL is equal to NULL.
+   *
+   * BEWARE, that this is not according to the specified SQL
+   * std spec, which is also implemented by MySql.
+   */
+  int branch_col_eq(Uint32 attrId1, Uint32 attrId2, Uint32 label);
+  int branch_col_ne(Uint32 attrId1, Uint32 attrId2, Uint32 label);
+  int branch_col_lt(Uint32 attrId1, Uint32 attrId2, Uint32 label);
+  int branch_col_le(Uint32 attrId1, Uint32 attrId2, Uint32 label);
+  int branch_col_gt(Uint32 attrId1, Uint32 attrId2, Uint32 label);
+  int branch_col_ge(Uint32 attrId1, Uint32 attrId2, Uint32 label);
+
   int branch_col_eq_null(Uint32 attrId, Uint32 Label);
   int branch_col_ne_null(Uint32 attrId, Uint32 Label);
 
@@ -674,6 +705,8 @@ private:
   int write_attr_impl(const NdbColumnImpl *c, Uint32 RegSource);
   int branch_col(Uint32 branch_type, Uint32 attrId, const void * val,
                  Uint32 len, Uint32 label);
+  int branch_col(Uint32 branch_type, Uint32 attrId1, Uint32 attrId2,
+                 Uint32 label);
   int getInfo(Uint32 number, CodeMetaInfo &info) const;
   static int compareMetaInfo(const void *a, 
                              const void *b);
