@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,9 +27,10 @@
 
 #include <ndb_global.h>
 
-// Enable DEBUG_RSS to measure memory consumption in various parts
+//Enable DEBUG_RSS to measure memory consumption in various parts
 //#define DEBUG_RSS 1
-struct ndb_rusage {
+struct ndb_rusage
+{
   Uint64 ru_utime;
   Uint64 ru_stime;
   Uint64 ru_minflt;
@@ -41,16 +42,16 @@ struct ndb_rusage {
 #endif
 };
 
-#ifdef __cplusplus
+#ifdef	__cplusplus
 extern "C" {
 #endif
 
-/**
- * Get resource usage for calling thread
- */
-int Ndb_GetRUsage(ndb_rusage *dst, bool process);
+  /**
+   * Get resource usage for calling thread
+   */
+  int Ndb_GetRUsage(ndb_rusage * dst, bool process);
 
-#ifdef __cplusplus
+#ifdef	__cplusplus
 }
 #endif
 
