@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -76,6 +76,20 @@ static inline int ndb_socket_errno()
   return WSAGetLastError();
 }
 
+static inline
+int ndb_getsockopt(ndb_socket_t s, int level, int optname,
+                   void *optval, ndb_socket_len_t *optlen)
+{
+  return getsockopt(s.s, level, optname, (char*)optval, optlen);
+}
+
+static inline
+int ndb_setsockopt(ndb_socket_t s, int level, int optname,
+                   void *optval, ndb_socket_len_t optlen)
+{
+  return setsockopt(s.s, level, optname, (char*)optval, optlen);
+}
+
 static inline
 ndb_socket_t ndb_socket_create(int domain, int type,int protocol)
 {
@@ -85,6 +99,21 @@ ndb_socket_t ndb_socket_create(int domain, int type,int protocol)
   return s;
 }
 
+static inline
+ndb_socket_t ndb_socket_create_dual_stack(int type, int protocol)
+{
+  ndb_socket_t s;
+  int on = 0;
+  s.s= socket(AF_INET6, type, protocol);
+
+  if (ndb_setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY,
+                     (char *)&on, sizeof(on)) == -1)
+  {
+    ndb_socket_invalidate(&s);
+  }
+  return s;
+}
+
 static inline
 ssize_t ndb_recv(ndb_socket_t s, char* buf, size_t len, int flags)
 {
@@ -123,21 +152,21 @@ int ndb_socket_nonblock(ndb_socket_t s, int enable)
 }
 
 static inline
-int ndb_bind_inet(ndb_socket_t s, const struct sockaddr_in *addr)
+int ndb_bind_inet(ndb_socket_t s, const struct sockaddr_in6 *addr)
 {
-  return bind(s.s, (const struct sockaddr*)addr, sizeof(struct sockaddr_in));
+  return bind(s.s, (const struct sockaddr*)addr, sizeof(struct sockaddr_in6));
 }
 
 static inline
 int ndb_socket_get_port(ndb_socket_t s, unsigned short *port)
 {
-  struct sockaddr_in servaddr;
+  struct sockaddr_in6 servaddr;
   ndb_socket_len_t sock_len = sizeof(servaddr);
   if(getsockname(s.s, (struct sockaddr*)&servaddr, &sock_len) < 0) {
     return 1;
   }
 
-  *port= ntohs(servaddr.sin_port);
+  *port= ntohs(servaddr.sin6_port);
   return 0;
 }
 
@@ -164,28 +193,21 @@ int ndb_connect_inet(ndb_socket_t s, const struct sockaddr_in *addr)
 }
 
 static inline
-int ndb_getsockopt(ndb_socket_t s, int level, int optname,
-                   void *optval, ndb_socket_len_t *optlen)
+int ndb_connect_inet6(ndb_socket_t s, const struct sockaddr_in6 *addr)
 {
-  return getsockopt(s.s, level, optname, (char*)optval, optlen);
+  return connect(s.s, (const struct sockaddr*) addr,
+                 sizeof(struct sockaddr_in6));
 }
 
 static inline
-int ndb_setsockopt(ndb_socket_t s, int level, int optname,
-                   void *optval, ndb_socket_len_t optlen)
+int ndb_socket_connect_address(ndb_socket_t s, struct in6_addr *a)
 {
-  return setsockopt(s.s, level, optname, (char*)optval, optlen);
-}
-
-static inline
-int ndb_socket_connect_address(ndb_socket_t s, struct in_addr *a)
-{
-  struct sockaddr_in addr;
+  struct sockaddr_in6 addr;
   ndb_socket_len_t addrlen= sizeof(addr);
   if(getpeername(s.s, (struct sockaddr*)&addr, &addrlen)==SOCKET_ERROR)
     return ndb_socket_errno();
 
-  *a= addr.sin_addr;
+  *a= addr.sin6_addr;
   return 0;
 }
 
