@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+Copyright (c) 2020 Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -25,27 +25,29 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 #ifndef NODEJS_ADAPTER_INCLUDE_JSVALUEACCESS_H
 #define NODEJS_ADAPTER_INCLUDE_JSVALUEACCESS_H
 
-#include <node_buffer.h>
-#include <cstddef>  // size_t
+#include <cstddef>              // size_t
 
-using v8::Boolean;
-using v8::Date;
-using v8::Integer;
-using v8::Isolate;
+using v8::String;
+using v8::Number;
+using v8::NewStringType;
 using v8::Local;
+using v8::Object;
+using v8::Isolate;
+using v8::Value;
 using v8::Maybe;
 using v8::MaybeLocal;
-using v8::Message;
-using v8::Name;
-using v8::NewStringType;
+using v8::Boolean;
+using v8::Integer;
 using v8::Number;
-using v8::Object;
-using v8::String;
+using v8::Date;
+using v8::Name;
 using v8::TryCatch;
-using v8::Value;
+using v8::Message;
 
 typedef v8::FunctionCallbackInfo<Value> Arguments;
 
+#include "node_buffer.h"
+
 // HasProperty
 template <typename KEY>
 inline bool HasProperty(Isolate *isolate, Local<Object> obj, KEY key) {
@@ -75,14 +77,15 @@ inline Local<Value> Get(Local<Object> obj, T key) {
 
 // Create New JavaScript Strings
 inline Local<String> NewUtf8String(Isolate *isolate, const char *str,
-                                   size_t len) {
-  return String::NewFromUtf8(isolate, str, NewStringType::kNormal, len)
-      .ToLocalChecked();
+                                   size_t len)
+{
+  return  String::NewFromUtf8(isolate, str, NewStringType::kNormal, len).
+    ToLocalChecked();
 }
 
-inline Local<String> NewUtf8String(
-    Isolate *isolate, const char *str,
-    NewStringType type = NewStringType::kNormal) {
+inline Local<String> NewUtf8String(Isolate *isolate, const char *str,
+                                   NewStringType type = NewStringType::kNormal)
+{
   return String::NewFromUtf8(isolate, str, type).ToLocalChecked();
 }
 
@@ -104,6 +107,7 @@ inline Local<String> NewExternalTwoByteString(Isolate *isolate, EXT ext) {
   return String::NewExternalTwoByte(isolate, ext).ToLocalChecked();
 }
 
+
 inline Local<String> ToString(const Arguments &args, const char *str) {
   return NewUtf8String(args.GetIsolate(), str);
 }
@@ -143,14 +147,15 @@ inline Local<Object> ToObject(Local<Value> val) {
 }
 
 template <typename T>
-inline Local<Object> ElementToObject(Local<Object> obj, T key) {
+inline Local<Object> ElementToObject(Local<Object> obj, T key)
+{
   return ToObject(obj->GetIsolate(), Get(obj->GetIsolate(), obj, key));
 }
 
+
 // SetProp
 template <typename T>
-inline void SetProp(Isolate *isolate, T obj, const char *key,
-                    Local<Value> value) {
+inline void SetProp(Isolate *isolate, T obj, const char * key, Local<Value> value) {
   Maybe<bool> r = obj->Set(isolate->GetCurrentContext(),
                            NewStringSymbol(isolate, key), value);
   r.Check();
@@ -162,8 +167,8 @@ inline void SetProp(Isolate *isolate, T obj, int i, Local<Value> value) {
   r.Check();
 }
 
-inline void SetProp(Isolate *isolate, Local<Object> obj, Local<String> key,
-                    Local<Value> value) {
+inline void SetProp(Isolate *isolate, Local<Object> obj,
+                    Local<String> key, Local<Value> value) {
   Maybe<bool> r = obj->Set(isolate->GetCurrentContext(), key, value);
   r.Check();
 }
@@ -188,15 +193,15 @@ inline void SetProp(Local<Object> obj, KEY key, Local<Value> value) {
 }
 
 // Node::Buffer
-inline Local<Object> CopyToJsBuffer(Isolate *i, const char *data, size_t len) {
+inline Local<Object> CopyToJsBuffer(Isolate* i, const char* data, size_t len) {
   return node::Buffer::Copy(i, data, len).ToLocalChecked();
 }
 
-inline Local<Object> NewJsBuffer(Isolate *iso, char *data, size_t len) {
+inline Local<Object> NewJsBuffer(Isolate* iso, char* data, size_t len) {
   return node::Buffer::New(iso, data, len).ToLocalChecked();
 }
 
-inline Local<Object> NewJsBuffer(Isolate *isolate, char *data, size_t len,
+inline Local<Object> NewJsBuffer(Isolate* isolate, char* data, size_t len,
                                  node::Buffer::FreeCallback cb) {
   /* nullptr is hint */
   return node::Buffer::New(isolate, data, len, cb, nullptr).ToLocalChecked();
@@ -206,7 +211,7 @@ inline Local<Object> NewJsBuffer(Isolate *isolate, Local<String> str) {
   return node::Buffer::New(isolate, str).ToLocalChecked();
 }
 
-inline Local<Object> NewJsBuffer(Isolate *iso, size_t len) {
+inline Local<Object> NewJsBuffer(Isolate* iso, size_t len) {
   return node::Buffer::New(iso, len).ToLocalChecked();
 }
 
@@ -214,7 +219,7 @@ inline size_t GetBufferLength(Local<Object> obj) {
   return node::Buffer::Length(obj);
 }
 
-inline char *GetBufferData(Local<Object> obj) {
+inline char * GetBufferData(Local<Object> obj) {
   return node::Buffer::Data(obj);
 }
 
@@ -314,8 +319,8 @@ inline bool GetBoolProperty(Local<Object> obj, T key) {
 
 // StackTrace
 inline Local<Value> GetStackTrace(Isolate *isolate, TryCatch *err) {
-  return ToString(
-      isolate, err->StackTrace(isolate->GetCurrentContext()).ToLocalChecked());
+  return ToString(isolate,
+    err->StackTrace(isolate->GetCurrentContext()).ToLocalChecked());
 }
 
 #endif
