@@ -1,6 +1,6 @@
 /*
- Copyright (c) 2013, 2023, Oracle and/or its affiliates.
-
+ Copyright (c) 2013, 2020 Oracle and/or its affiliates.
+ 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
@@ -26,50 +26,57 @@
 #include "unified_debug.h"
 
 using v8::Local;
-using v8::Object;
 using v8::Value;
+using v8::Object;
 
 class BlobWriteHandler;
 
 class ColumnHandler {
- public:
+public:
   ColumnHandler();
   ~ColumnHandler();
   void init(v8::Isolate *, const NdbDictionary::Column *, uint32_t);
   Local<Value> read(char *, Local<Object>) const;
   Local<Value> write(Local<Value>, char *) const;
-  BlobWriteHandler *createBlobWriteHandle(Local<Value>, int fieldNo) const;
+  BlobWriteHandler * createBlobWriteHandle(Local<Value>, int fieldNo) const;
   bool isBlob() const;
 
- public:
+public:
   const NdbDictionary::Column *column;
-
- private:
+private: 
   const NdbTypeEncoder *encoder;
   v8::Isolate *isolate;
   uint32_t offset;
   bool isLob, isText;
 };
 
-inline bool ColumnHandler::isBlob() const { return isLob; }
+inline bool ColumnHandler::isBlob() const {
+  return isLob;
+}
 
 class ColumnHandlerSet {
- public:
+public:
   ColumnHandlerSet(int);
   ~ColumnHandlerSet();
-  ColumnHandler *getHandler(int);
-
- private:
+  ColumnHandler * getHandler(int);
+private:
   int size;
-  ColumnHandler *const handlers;
+  ColumnHandler * const handlers;
 };
 
-inline ColumnHandlerSet::ColumnHandlerSet(int _size)
-    : size(_size), handlers(new ColumnHandler[size]) {}
 
-inline ColumnHandlerSet::~ColumnHandlerSet() { delete[] handlers; }
+inline ColumnHandlerSet::ColumnHandlerSet(int _size) :
+  size(_size),
+  handlers(new ColumnHandler[size])
+{ }
 
-inline ColumnHandler *ColumnHandlerSet::getHandler(int i) {
-  assert(i < size);
-  return &handlers[i];
+inline ColumnHandlerSet::~ColumnHandlerSet() {
+  delete[] handlers;
 }
+
+inline ColumnHandler * ColumnHandlerSet::getHandler(int i) {
+  assert(i < size);
+  return & handlers[i];
+}
+
+
