@@ -1,5 +1,5 @@
 /*
- Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2013, 2020 Oracle and/or its affiliates.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -25,7 +25,6 @@
 #include <NdbApi.hpp>
 #include <node.h>
 
-using v8::Handle;
 using v8::Local;
 using v8::Value;
 using v8::Object;
@@ -35,7 +34,7 @@ typedef Local<Value> EncoderReader(const NdbDictionary::Column *,
                                    char *, uint32_t);
 
 typedef Local<Value> EncoderWriter(const NdbDictionary::Column *,
-                                   Handle<Value>, char *, uint32_t);
+                                   Local<Value>, char *, uint32_t);
 
 typedef struct {
   EncoderReader * read;
@@ -45,5 +44,5 @@ typedef struct {
 
 const NdbTypeEncoder * getEncoderForColumn(const NdbDictionary::Column *);
 
-Local<Object> getBufferForText(const NdbDictionary::Column *, Handle<String>);
-Local<String> getTextFromBuffer(const NdbDictionary::Column *, Handle<Object>);
+Local<Object> getBufferForText(const NdbDictionary::Column *, Local<String>);
+Local<String> getTextFromBuffer(const NdbDictionary::Column *, Local<Object>);
