@@ -1,5 +1,5 @@
 /*
- Copyright (c) 2013, 2016, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2013, 2020 Oracle and/or its affiliates.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -27,38 +27,40 @@
 #include "ColumnProxy.h"
 #include "JsWrapper.h"
 
-using namespace v8;
 
-Handle<Value> ColumnProxy::get(v8::Isolate *isolate, char *buffer) {
-  Handle<Value> val = ToLocal(& jsValue);
+Local<Value> ColumnProxy::get(v8::Isolate *isolate, char *buffer) {
+  Local<Value> val = jsValue.Get(isolate);
 
   if(! isLoaded) {
-    val = handler->read(buffer, ToLocal(& blobBuffer));
+    val = handler->read(buffer, blobBuffer.Get(isolate));
     jsValue.Reset(isolate, val);
     isLoaded = true;
   }
   return val;
 }
 
-void ColumnProxy::set(v8::Isolate *isolate, Handle<Value> newValue) {
+void ColumnProxy::set(v8::Isolate *isolate, Local<Value> newValue) {
   isNull = (newValue->IsNull());
   isLoaded = isDirty = true;
   jsValue.Reset(isolate, newValue);
   DEBUG_PRINT("set %s", handler->column->getName());
 }
 
-Handle<Value> ColumnProxy::write(v8::Isolate *isolate, char *buffer) {
-  Handle<Value> rval = Undefined(isolate);
+Local<Value> ColumnProxy::write(v8::Isolate *isolate, char *buffer) {
+  Local<Value> rval = Undefined(isolate);
 
   if(isDirty && ! (handler->isBlob())) {
-    rval = handler->write(ToLocal(& jsValue), buffer);
+    rval = handler->write(jsValue.Get(isolate), buffer);
   }
   isDirty = false;
 
   return rval;
 }
 
-BlobWriteHandler * ColumnProxy::createBlobWriteHandle(int i) {
-  return isNull ? 0 : handler->createBlobWriteHandle(ToLocal(& jsValue), i);
+BlobWriteHandler * ColumnProxy::createBlobWriteHandle(v8::Isolate * isolate,
+                                                      int i) {
+  if(isNull) return nullptr;
+  Local<Value> columnValue = jsValue.Get(isolate);
+  return handler->createBlobWriteHandle(columnValue, i);
 }
 
