@@ -1,17 +1,16 @@
 /*
- Copyright (c) 2013, 2024, Oracle and/or its affiliates.
-
+ Copyright (c) 2013, 2020 Oracle and/or its affiliates.
+ 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is designed to work with certain software (including
+ This program is also distributed with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have either included with
- the program or referenced in the documentation.
+ separately licensed software that they have included with MySQL.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,13 +24,14 @@
 
 #include <cstddef>
 
-#include <NdbApi.hpp>
+#include <NdbApi.hpp> 
 
-#include "NativeMethodCall.h"
-#include "NdbJsConverters.h"
-#include "NdbWrapperErrors.h"
 #include "adapter_global.h"
 #include "js_wrapper_macros.h"
+#include "NdbWrapperErrors.h"
+#include "NativeMethodCall.h"
+#include "NdbJsConverters.h"
+
 
 V8WrapperFn load_const_null;
 V8WrapperFn load_const_u16;
@@ -74,8 +74,7 @@ V8WrapperFn def_sub;
 V8WrapperFn call_sub;
 V8WrapperFn ret_sub;
 V8WrapperFn finalise;
-V8WrapperFn
-    NdbInterpretedCode_getTable_wrapper;  // rename to avoid duplicate symbol
+V8WrapperFn NdbInterpretedCode_getTable_wrapper;   // rename to avoid duplicate symbol
 V8WrapperFn getNdbError;
 V8WrapperFn getWordsUsed;
 // V8WrapperFn copy; // not wrapped
@@ -83,52 +82,52 @@ V8WrapperFn getWordsUsed;
 #define WRAPPER_FUNCTION(A) addMethod(#A, A)
 
 class NdbInterpretedCodeEnvelopeClass : public Envelope {
- public:
+public:
   NdbInterpretedCodeEnvelopeClass() : Envelope("NdbInterpretedCode") {
     EscapableHandleScope scope(v8::Isolate::GetCurrent());
-    WRAPPER_FUNCTION(load_const_null);
-    WRAPPER_FUNCTION(load_const_u16);
-    WRAPPER_FUNCTION(load_const_u32);
-    // WRAPPER_FUNCTION( load_const_u64);
-    WRAPPER_FUNCTION(read_attr);
-    WRAPPER_FUNCTION(write_attr);
-    WRAPPER_FUNCTION(add_reg);
-    WRAPPER_FUNCTION(sub_reg);
-    WRAPPER_FUNCTION(def_label);
-    WRAPPER_FUNCTION(branch_label);
-    WRAPPER_FUNCTION(branch_ge);
-    WRAPPER_FUNCTION(branch_gt);
-    WRAPPER_FUNCTION(branch_le);
-    WRAPPER_FUNCTION(branch_lt);
-    WRAPPER_FUNCTION(branch_eq);
-    WRAPPER_FUNCTION(branch_ne);
-    WRAPPER_FUNCTION(branch_ne_null);
-    WRAPPER_FUNCTION(branch_eq_null);
-    WRAPPER_FUNCTION(branch_col_eq);
-    WRAPPER_FUNCTION(branch_col_ne);
-    WRAPPER_FUNCTION(branch_col_lt);
-    WRAPPER_FUNCTION(branch_col_le);
-    WRAPPER_FUNCTION(branch_col_gt);
-    WRAPPER_FUNCTION(branch_col_ge);
-    WRAPPER_FUNCTION(branch_col_eq_null);
-    WRAPPER_FUNCTION(branch_col_ne_null);
-    WRAPPER_FUNCTION(branch_col_like);
-    WRAPPER_FUNCTION(branch_col_notlike);
-    WRAPPER_FUNCTION(branch_col_and_mask_eq_mask);
-    WRAPPER_FUNCTION(branch_col_and_mask_ne_mask);
-    WRAPPER_FUNCTION(branch_col_and_mask_eq_zero);
-    WRAPPER_FUNCTION(branch_col_and_mask_ne_zero);
-    WRAPPER_FUNCTION(interpret_exit_ok);
-    WRAPPER_FUNCTION(interpret_exit_nok);
-    WRAPPER_FUNCTION(interpret_exit_last_row);
-    WRAPPER_FUNCTION(add_val);
-    WRAPPER_FUNCTION(sub_val);
-    WRAPPER_FUNCTION(def_sub);
-    WRAPPER_FUNCTION(call_sub);
-    WRAPPER_FUNCTION(ret_sub);
-    WRAPPER_FUNCTION(finalise);
-    WRAPPER_FUNCTION(getWordsUsed);
-    // WRAPPER_FUNCTION( copy);   // not wrapped
+    WRAPPER_FUNCTION( load_const_null);
+    WRAPPER_FUNCTION( load_const_u16);
+    WRAPPER_FUNCTION( load_const_u32);
+    // WRAPPER_FUNCTION( load_const_u64); 
+    WRAPPER_FUNCTION( read_attr);
+    WRAPPER_FUNCTION( write_attr);
+    WRAPPER_FUNCTION( add_reg);
+    WRAPPER_FUNCTION( sub_reg);
+    WRAPPER_FUNCTION( def_label);
+    WRAPPER_FUNCTION( branch_label);
+    WRAPPER_FUNCTION( branch_ge);
+    WRAPPER_FUNCTION( branch_gt);
+    WRAPPER_FUNCTION( branch_le);
+    WRAPPER_FUNCTION( branch_lt);
+    WRAPPER_FUNCTION( branch_eq);
+    WRAPPER_FUNCTION( branch_ne);
+    WRAPPER_FUNCTION( branch_ne_null);
+    WRAPPER_FUNCTION( branch_eq_null);
+    WRAPPER_FUNCTION( branch_col_eq);
+    WRAPPER_FUNCTION( branch_col_ne);
+    WRAPPER_FUNCTION( branch_col_lt);
+    WRAPPER_FUNCTION( branch_col_le);
+    WRAPPER_FUNCTION( branch_col_gt);
+    WRAPPER_FUNCTION( branch_col_ge);
+    WRAPPER_FUNCTION( branch_col_eq_null);
+    WRAPPER_FUNCTION( branch_col_ne_null);
+    WRAPPER_FUNCTION( branch_col_like);
+    WRAPPER_FUNCTION( branch_col_notlike);
+    WRAPPER_FUNCTION( branch_col_and_mask_eq_mask);
+    WRAPPER_FUNCTION( branch_col_and_mask_ne_mask);
+    WRAPPER_FUNCTION( branch_col_and_mask_eq_zero);
+    WRAPPER_FUNCTION( branch_col_and_mask_ne_zero);
+    WRAPPER_FUNCTION( interpret_exit_ok);
+    WRAPPER_FUNCTION( interpret_exit_nok);
+    WRAPPER_FUNCTION( interpret_exit_last_row);
+    WRAPPER_FUNCTION( add_val);
+    WRAPPER_FUNCTION( sub_val);
+    WRAPPER_FUNCTION( def_sub);
+    WRAPPER_FUNCTION( call_sub);
+    WRAPPER_FUNCTION( ret_sub);
+    WRAPPER_FUNCTION( finalise);
+    WRAPPER_FUNCTION( getWordsUsed);
+    // WRAPPER_FUNCTION( copy);   // not wrapped 
     addMethod("getTable", NdbInterpretedCode_getTable_wrapper);
     addMethod("getNdbError", getNdbError<NdbInterpretedCode>);
   }
@@ -139,11 +138,11 @@ NdbInterpretedCodeEnvelopeClass NdbInterpretedCodeEnvelope;
 /* The const version has no methods attached: */
 Envelope ConstNdbInterpretedCodeEnvelope("const NdbInterpretedCode");
 
-Envelope *getConstNdbInterpretedCodeEnvelope() {
-  return &ConstNdbInterpretedCodeEnvelope;
+Envelope * getConstNdbInterpretedCodeEnvelope() {
+  return & ConstNdbInterpretedCodeEnvelope;
 }
 
-void newNdbInterpretedCode(const Arguments &args) {
+void newNdbInterpretedCode(const Arguments & args) {
   EscapableHandleScope scope(args.GetIsolate());
 
   DEBUG_MARKER(UDEB_DETAIL);
@@ -151,7 +150,7 @@ void newNdbInterpretedCode(const Arguments &args) {
   REQUIRE_ARGS_LENGTH(1);
 
   JsValueConverter<const NdbDictionary::Table *> arg0(args[0]);
-  NdbInterpretedCode *c = new NdbInterpretedCode(arg0.toC());
+  NdbInterpretedCode * c = new NdbInterpretedCode(arg0.toC());
   Local<Value> jsObject = NdbInterpretedCodeEnvelope.wrap(c);
   NdbInterpretedCodeEnvelope.freeFromGC(c, jsObject);
   args.GetReturnValue().Set(scope.Escape(jsObject));
@@ -161,7 +160,7 @@ void load_const_null(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeMethodCall_1_<int, NdbInterpretedCode, uint32_t> NCALL;
-  NCALL ncall(&NdbInterpretedCode::load_const_null, args);
+  NCALL ncall(& NdbInterpretedCode::load_const_null, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -169,9 +168,8 @@ void load_const_null(const Arguments &args) {
 void load_const_u16(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::load_const_u16, args);
+  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::load_const_u16, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -179,21 +177,20 @@ void load_const_u16(const Arguments &args) {
 void load_const_u32(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::load_const_u32, args);
+  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::load_const_u32, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
 
+
 /* TODO: read_attr and write_attr have two forms */
 void read_attr(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t,
-                              const NdbDictionary::Column *>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::read_attr, args);
+  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, 
+                              const NdbDictionary::Column *> NCALL;
+  NCALL ncall(& NdbInterpretedCode::read_attr, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -201,10 +198,10 @@ void read_attr(const Arguments &args) {
 void write_attr(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_2_<int, NdbInterpretedCode,
-                              const NdbDictionary::Column *, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::write_attr, args);
+  typedef NativeMethodCall_2_<int, NdbInterpretedCode,  
+                              const NdbDictionary::Column *,
+                              uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::write_attr, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -212,10 +209,9 @@ void write_attr(const Arguments &args) {
 void add_reg(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_3_<int, NdbInterpretedCode, uint32_t, uint32_t,
-                              uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::add_reg, args);
+  typedef NativeMethodCall_3_<int, NdbInterpretedCode, 
+                              uint32_t, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::add_reg, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -223,10 +219,9 @@ void add_reg(const Arguments &args) {
 void sub_reg(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_3_<int, NdbInterpretedCode, uint32_t, uint32_t,
-                              uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::sub_reg, args);
+  typedef NativeMethodCall_3_<int, NdbInterpretedCode, 
+                              uint32_t, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::sub_reg, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -235,7 +230,7 @@ void def_label(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeMethodCall_1_<int, NdbInterpretedCode, int> NCALL;
-  NCALL ncall(&NdbInterpretedCode::def_label, args);
+  NCALL ncall(& NdbInterpretedCode::def_label, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -244,7 +239,7 @@ void branch_label(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeMethodCall_1_<int, NdbInterpretedCode, uint32_t> NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_label, args);
+  NCALL ncall(& NdbInterpretedCode::branch_label, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -252,10 +247,9 @@ void branch_label(const Arguments &args) {
 void branch_ge(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_3_<int, NdbInterpretedCode, uint32_t, uint32_t,
-                              uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_ge, args);
+  typedef NativeMethodCall_3_<int, NdbInterpretedCode, 
+                              uint32_t, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_ge, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -263,10 +257,9 @@ void branch_ge(const Arguments &args) {
 void branch_gt(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_3_<int, NdbInterpretedCode, uint32_t, uint32_t,
-                              uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_gt, args);
+  typedef NativeMethodCall_3_<int, NdbInterpretedCode, 
+                              uint32_t, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_gt, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -274,10 +267,9 @@ void branch_gt(const Arguments &args) {
 void branch_le(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_3_<int, NdbInterpretedCode, uint32_t, uint32_t,
-                              uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_le, args);
+  typedef NativeMethodCall_3_<int, NdbInterpretedCode, 
+                              uint32_t, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_le, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -285,10 +277,9 @@ void branch_le(const Arguments &args) {
 void branch_lt(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_3_<int, NdbInterpretedCode, uint32_t, uint32_t,
-                              uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_lt, args);
+  typedef NativeMethodCall_3_<int, NdbInterpretedCode, 
+                              uint32_t, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_lt, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -296,10 +287,9 @@ void branch_lt(const Arguments &args) {
 void branch_eq(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_3_<int, NdbInterpretedCode, uint32_t, uint32_t,
-                              uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_eq, args);
+  typedef NativeMethodCall_3_<int, NdbInterpretedCode, 
+                              uint32_t, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_eq, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -307,10 +297,9 @@ void branch_eq(const Arguments &args) {
 void branch_ne(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_3_<int, NdbInterpretedCode, uint32_t, uint32_t,
-                              uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_ne, args);
+  typedef NativeMethodCall_3_<int, NdbInterpretedCode, 
+                              uint32_t, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_ne, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -318,9 +307,9 @@ void branch_ne(const Arguments &args) {
 void branch_ne_null(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_ne_null, args);
+  typedef NativeMethodCall_2_<int, NdbInterpretedCode,  
+                              uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_ne_null, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -328,16 +317,17 @@ void branch_ne_null(const Arguments &args) {
 void branch_eq_null(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_eq_null, args);
+  typedef NativeMethodCall_2_<int, NdbInterpretedCode,  
+                              uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_eq_null, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
 
+
 /****************************************************************
- *     BRANCH ON COLUMN AND VALUE
- *     These have hand-written wrappers
+ *     BRANCH ON COLUMN AND VALUE 
+ *     These have hand-written wrappers 
  *     ARG0: Buffer
  *     ARG1: Offset
  *     ARG2: AttrID
@@ -345,7 +335,7 @@ void branch_eq_null(const Arguments &args) {
  ****************************************************************/
 
 /* Utility function */
-const void *getValueAddr(const Arguments &args) {
+const void * getValueAddr(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   Local<Object> buffer = ArgToObject(args, 0);
@@ -356,112 +346,107 @@ const void *getValueAddr(const Arguments &args) {
 void branch_col_eq(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  const void *val = getValueAddr(args);
-  NdbInterpretedCode *code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
-  int rval =
-      code->branch_col_eq(val, 0, GetUint32Arg(args, 2), GetUint32Arg(args, 3));
+  const void * val = getValueAddr(args);
+  NdbInterpretedCode * code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
+  int rval = code->branch_col_eq(val, 0, GetUint32Arg(args, 2), GetUint32Arg(args, 3));
   args.GetReturnValue().Set(rval);
 }
 
 void branch_col_ne(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  const void *val = getValueAddr(args);
-  NdbInterpretedCode *code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
-  int rval =
-      code->branch_col_ne(val, 0, GetUint32Arg(args, 2), GetUint32Arg(args, 3));
+  const void * val = getValueAddr(args);
+  NdbInterpretedCode * code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
+  int rval = code->branch_col_ne(val, 0, GetUint32Arg(args, 2),GetUint32Arg(args, 3));
   args.GetReturnValue().Set(rval);
 }
 
 void branch_col_lt(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  const void *val = getValueAddr(args);
-  NdbInterpretedCode *code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
-  int rval =
-      code->branch_col_lt(val, 0, GetUint32Arg(args, 2), GetUint32Arg(args, 3));
+  const void * val = getValueAddr(args);
+  NdbInterpretedCode * code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
+  int rval = code->branch_col_lt(val, 0, GetUint32Arg(args, 2),GetUint32Arg(args, 3));
   args.GetReturnValue().Set(rval);
 }
 
 void branch_col_le(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  const void *val = getValueAddr(args);
-  NdbInterpretedCode *code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
-  int rval =
-      code->branch_col_le(val, 0, GetUint32Arg(args, 2), GetUint32Arg(args, 3));
+  const void * val = getValueAddr(args);
+  NdbInterpretedCode * code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
+  int rval = code->branch_col_le(val, 0, GetUint32Arg(args, 2),GetUint32Arg(args, 3));
   args.GetReturnValue().Set(rval);
 }
 
 void branch_col_gt(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  const void *val = getValueAddr(args);
-  NdbInterpretedCode *code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
-  int rval =
-      code->branch_col_gt(val, 0, GetUint32Arg(args, 2), GetUint32Arg(args, 3));
+  const void * val = getValueAddr(args);
+  NdbInterpretedCode * code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
+  int rval = code->branch_col_gt(val, 0,GetUint32Arg(args, 2),GetUint32Arg(args, 3));
   args.GetReturnValue().Set(rval);
 }
 
 void branch_col_ge(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  const void *val = getValueAddr(args);
-  NdbInterpretedCode *code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
-  int rval =
-      code->branch_col_ge(val, 0, GetUint32Arg(args, 2), GetUint32Arg(args, 3));
+  const void * val = getValueAddr(args);
+  NdbInterpretedCode * code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
+  int rval = code->branch_col_ge(val, 0, GetUint32Arg(args, 2),GetUint32Arg(args, 3));
   args.GetReturnValue().Set(rval);
 }
 
 void branch_col_and_mask_eq_mask(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  const void *val = getValueAddr(args);
-  NdbInterpretedCode *code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
-  int rval = code->branch_col_and_mask_eq_mask(val, 0, GetUint32Arg(args, 2),
-                                               GetUint32Arg(args, 3));
+  const void * val = getValueAddr(args);
+  NdbInterpretedCode * code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
+  int rval = code->branch_col_and_mask_eq_mask(
+    val, 0, GetUint32Arg(args, 2),GetUint32Arg(args, 3));
   args.GetReturnValue().Set(rval);
 }
 
 void branch_col_and_mask_ne_mask(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  const void *val = getValueAddr(args);
-  NdbInterpretedCode *code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
-  int rval = code->branch_col_and_mask_ne_mask(val, 0, GetUint32Arg(args, 2),
-                                               GetUint32Arg(args, 3));
+  const void * val = getValueAddr(args);
+  NdbInterpretedCode * code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
+  int rval = code->branch_col_and_mask_ne_mask(
+    val, 0,GetUint32Arg(args, 2),GetUint32Arg(args, 3));
   args.GetReturnValue().Set(rval);
 }
 
 void branch_col_and_mask_eq_zero(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  const void *val = getValueAddr(args);
-  NdbInterpretedCode *code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
-  int rval = code->branch_col_and_mask_eq_zero(val, 0, GetUint32Arg(args, 2),
-                                               GetUint32Arg(args, 3));
+  const void * val = getValueAddr(args);
+  NdbInterpretedCode * code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
+  int rval = code->branch_col_and_mask_eq_zero(
+    val, 0, GetUint32Arg(args, 2),GetUint32Arg(args, 3));
   args.GetReturnValue().Set(rval);
 }
 
 void branch_col_and_mask_ne_zero(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  const void *val = getValueAddr(args);
-  NdbInterpretedCode *code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
-  int rval = code->branch_col_and_mask_ne_zero(val, 0, GetUint32Arg(args, 2),
-                                               GetUint32Arg(args, 3));
+  const void * val = getValueAddr(args);
+  NdbInterpretedCode * code = unwrapPointer<NdbInterpretedCode *>(args.Holder());
+  int rval = code->branch_col_and_mask_ne_zero(
+    val, 0, GetUint32Arg(args, 2),GetUint32Arg(args, 3));
   args.GetReturnValue().Set(rval);
 }
 
+
 /****************************************************************
  *    Back to generic wrappers
  ****************************************************************/
 void branch_col_eq_null(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_col_eq_null, args);
+  typedef NativeMethodCall_2_<int, NdbInterpretedCode,  
+                              uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_col_eq_null, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -469,9 +454,9 @@ void branch_col_eq_null(const Arguments &args) {
 void branch_col_ne_null(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_col_ne_null, args);
+  typedef NativeMethodCall_2_<int, NdbInterpretedCode,  
+                              uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_col_ne_null, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -480,10 +465,9 @@ void branch_col_ne_null(const Arguments &args) {
 void branch_col_like(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_4_<int, NdbInterpretedCode, const void *, uint32_t,
-                              uint32_t, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_col_like, args);
+  typedef NativeMethodCall_4_<int, NdbInterpretedCode,  
+                              const void *, uint32_t, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_col_like, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -492,14 +476,14 @@ void branch_col_like(const Arguments &args) {
 void branch_col_notlike(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_4_<int, NdbInterpretedCode, const void *, uint32_t,
-                              uint32_t, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::branch_col_notlike, args);
+  typedef NativeMethodCall_4_<int, NdbInterpretedCode,  
+                              const void *, uint32_t, uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::branch_col_notlike, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
 
+
 /****************************************************************
  *   End of column/value branch instructions
  ****************************************************************/
@@ -508,7 +492,7 @@ void interpret_exit_ok(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeMethodCall_0_<int, NdbInterpretedCode> NCALL;
-  NCALL ncall(&NdbInterpretedCode::interpret_exit_ok, args);
+  NCALL ncall(& NdbInterpretedCode::interpret_exit_ok, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -517,7 +501,7 @@ void interpret_exit_nok(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeMethodCall_1_<int, NdbInterpretedCode, uint32_t> NCALL;
-  NCALL ncall(&NdbInterpretedCode::interpret_exit_nok, args);
+  NCALL ncall(& NdbInterpretedCode::interpret_exit_nok, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -526,7 +510,7 @@ void interpret_exit_last_row(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeMethodCall_0_<int, NdbInterpretedCode> NCALL;
-  NCALL ncall(&NdbInterpretedCode::interpret_exit_last_row, args);
+  NCALL ncall(& NdbInterpretedCode::interpret_exit_last_row, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -534,9 +518,9 @@ void interpret_exit_last_row(const Arguments &args) {
 void add_val(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::add_val, args);
+  typedef NativeMethodCall_2_<int, NdbInterpretedCode,  
+                              uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::add_val, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -544,9 +528,9 @@ void add_val(const Arguments &args) {
 void sub_val(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeMethodCall_2_<int, NdbInterpretedCode, uint32_t, uint32_t>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::sub_val, args);
+  typedef NativeMethodCall_2_<int, NdbInterpretedCode,  
+                              uint32_t, uint32_t> NCALL;
+  NCALL ncall(& NdbInterpretedCode::sub_val, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -555,7 +539,7 @@ void def_sub(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeMethodCall_1_<int, NdbInterpretedCode, uint32_t> NCALL;
-  NCALL ncall(&NdbInterpretedCode::def_sub, args);
+  NCALL ncall(& NdbInterpretedCode::def_sub, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -564,7 +548,7 @@ void call_sub(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeMethodCall_1_<int, NdbInterpretedCode, uint32_t> NCALL;
-  NCALL ncall(&NdbInterpretedCode::call_sub, args);
+  NCALL ncall(& NdbInterpretedCode::call_sub, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -573,7 +557,7 @@ void ret_sub(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeMethodCall_0_<int, NdbInterpretedCode> NCALL;
-  NCALL ncall(&NdbInterpretedCode::ret_sub, args);
+  NCALL ncall(& NdbInterpretedCode::ret_sub, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -582,7 +566,7 @@ void finalise(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeMethodCall_0_<int, NdbInterpretedCode> NCALL;
-  NCALL ncall(&NdbInterpretedCode::finalise, args);
+  NCALL ncall(& NdbInterpretedCode::finalise, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -590,10 +574,9 @@ void finalise(const Arguments &args) {
 void NdbInterpretedCode_getTable_wrapper(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
-  typedef NativeConstMethodCall_0_<const NdbDictionary::Table *,
-                                   NdbInterpretedCode>
-      NCALL;
-  NCALL ncall(&NdbInterpretedCode::getTable, args);
+  typedef NativeConstMethodCall_0_<const NdbDictionary::Table*, 
+                                   NdbInterpretedCode> NCALL;
+  NCALL ncall(& NdbInterpretedCode::getTable, args);
   ncall.wrapReturnValueAs(getNdbDictTableEnvelope());
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
@@ -603,7 +586,7 @@ void getWordsUsed(const Arguments &args) {
   DEBUG_MARKER(UDEB_DETAIL);
   EscapableHandleScope scope(args.GetIsolate());
   typedef NativeConstMethodCall_0_<uint32_t, NdbInterpretedCode> NCALL;
-  NCALL ncall(&NdbInterpretedCode::getWordsUsed, args);
+  NCALL ncall(& NdbInterpretedCode::getWordsUsed, args);
   ncall.run();
   args.GetReturnValue().Set(scope.Escape(ncall.jsReturnVal()));
 }
@@ -613,3 +596,4 @@ void NdbInterpretedCode_initOnLoad(Local<Object> target) {
   SetProp(target, "NdbInterpretedCode", ic_obj);
   DEFINE_JS_FUNCTION(ic_obj, "create", newNdbInterpretedCode);
 }
+
