@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -37,18 +37,14 @@
 #include "storage/ndb/plugin/ndb_log.h"
 #include "storage/ndb/plugin/ndb_thd.h"
 
-// Typedefs for long names
-typedef NdbDictionary::Column NDBCOL;
-typedef NdbDictionary::Table NDBTAB;
-
-typedef enum ndb_item_type {
+enum NDB_ITEM_TYPE {
   NDB_VALUE = 0,     // Qualified more with Item::Type
   NDB_FIELD = 1,     // Qualified from table definition
   NDB_FUNCTION = 2,  // Qualified from Item_func::Functype
   NDB_END_COND = 3   // End marker for condition group
-} NDB_ITEM_TYPE;
+};
 
-typedef enum ndb_func_type {
+enum NDB_FUNC_TYPE {
   NDB_EQ_FUNC = 0,
   NDB_NE_FUNC = 1,
   NDB_LT_FUNC = 2,
@@ -63,9 +59,9 @@ typedef enum ndb_func_type {
   NDB_COND_AND_FUNC = 11,
   NDB_COND_OR_FUNC = 12,
   NDB_UNSUPPORTED_FUNC = 13
-} NDB_FUNC_TYPE;
+};
 
-typedef union ndb_item_value {
+union NDB_ITEM_VALUE {
   const Item *item;  // NDB_VALUE
   struct {           // NDB_FIELD
     Field *field;
@@ -75,7 +71,7 @@ typedef union ndb_item_value {
     NDB_FUNC_TYPE func_type;
     uint arg_count;
   };
-} NDB_ITEM_VALUE;
+};
 
 /*
   Mapping defining the negated and swapped function equivalent
@@ -166,7 +162,7 @@ class Ndb_item {
 
   uint32 pack_length() const { return get_field()->pack_length(); }
 
-  const uchar *get_val() const { return get_field()->ptr; }
+  const uchar *get_val() const { return get_field()->field_ptr(); }
 
   const CHARSET_INFO *get_field_charset() const {
     const Field *field = get_field();
@@ -192,7 +188,11 @@ class Ndb_item {
         const_cast<Item *>(item)->save_in_field(field, false);
     dbug_tmp_restore_column_map(field->table->write_set, old_map);
 
-    if (unlikely(status != TYPE_OK)) return -1;
+    if (unlikely(status != TYPE_OK) &&
+        // 'TYPE_NOTE*': Minor truncation considered insignificant -> Still ok
+        status != TYPE_NOTE_TRUNCATED && status != TYPE_NOTE_TIME_TRUNCATED) {
+      return -1;
+    }
 
     return 0;  // OK
   }
@@ -874,7 +874,7 @@ static void ndb_serialize_cond(const Item *item, void *arg) {
               // Found a Field_item of a supported type. and from 'this' table
               DBUG_ASSERT(context->table == field->table);
 
-              const NDBCOL *col =
+              const NdbDictionary::Column *col =
                   context->ndb_table->getColumn(field->field_name);
               DBUG_ASSERT(col);
               ndb_item = new (*THR_MALLOC) Ndb_item(field, col->getColumnNo());
@@ -1265,6 +1265,18 @@ void ha_ndbcluster_cond::cond_clear() {
   m_unpushed_cond = nullptr;
 }
 
+/*
+  Clean up condition state after handler closed the table.
+  Could possible be reopen later, in which case the same condition
+  prepared for push should still be valid.
+*/
+void ha_ndbcluster_cond::cond_close() {
+  if (m_pushed_cond != nullptr && !isGeneratedCodeReusable()) {
+    m_scan_filter_code.reset();
+    m_unpushed_cond = nullptr;
+  }
+}
+
 /**
   Construct the AND conjunction of the pushed- and remainder
   predicate terms. If the the original condition was either
@@ -1378,17 +1390,15 @@ static int create_or_conditions(Item_cond *cond, List<Item> pushed_list,
                        from the condition terms pushed down.
   @param pushed_cond   The (part of) the condition term we may push
                        down to the ndbcluster storage engine.
-  @param remainder     The remainder (part of) the condition term
+  @param remainder_cond The remainder (part of) the condition term
                        still needed to be evaluated by the server.
 
   @return a List of Ndb_item objects representing the serialized
           form of the 'pushed_cond'.
  */
-static List<const Ndb_item> cond_push_boolean_term(Item *term, TABLE *table,
-                                                   const NDBTAB *ndb_table,
-                                                   bool other_tbls_ok,
-                                                   Item *&pushed_cond,
-                                                   Item *&remainder_cond)
+static List<const Ndb_item> cond_push_boolean_term(
+    Item *term, TABLE *table, const NdbDictionary::Table *ndb_table,
+    bool other_tbls_ok, Item *&pushed_cond, Item *&remainder_cond)
 
 {
   DBUG_TRACE;
@@ -1509,17 +1519,11 @@ static List<const Ndb_item> cond_push_boolean_term(Item *term, TABLE *table,
       }
     }
   }
-  /*
-    There are some used_tables() 'out of reach', tagged with special
-    *_TABLE_BIT. These can not be referred from a pushed condition.
-  */
-  const table_map dont_use_tables =
-      INNER_TABLE_BIT |  // Condition contain a subquery
-      RAND_TABLE_BIT;    // 'non-stable' value
 
-  if (term->used_tables() & dont_use_tables) {
-  } else if (other_tbls_ok ||
-             !(term->used_tables() & ~table->pos_in_table_list->map())) {
+  if (term->used_tables() & RAND_TABLE_BIT) {
+    // RAND_TABLE_BIT: Produce non deterministic results, dont push
+  } else if (other_tbls_ok || !(term->used_tables() & ~PSEUDO_TABLE_BITS &
+                                ~table->pos_in_table_list->map())) {
     // Has broken down the condition into predicate terms, or sub conditions,
     // which either has to be accepted or rejected for pushdown
     Ndb_cond_traverse_context context(table, ndb_table);
@@ -1565,6 +1569,11 @@ void ha_ndbcluster_cond::prep_cond_push(const Item *cond, bool other_tbls_ok) {
   m_remainder_cond = remainder;
 }
 
+bool ha_ndbcluster_cond::isGeneratedCodeReusable() const {
+  const TABLE *const table = m_handler->table;
+  return (m_pushed_cond->used_tables() & ~table->pos_in_table_list->map()) == 0;
+}
+
 /*
   Make a pushed condition prepared with prep_cond_push() available for
   the handler to really be used against the storage engine.
@@ -1572,10 +1581,7 @@ void ha_ndbcluster_cond::prep_cond_push(const Item *cond, bool other_tbls_ok) {
 int ha_ndbcluster_cond::use_cond_push(const Item *&pushed_cond,
                                       const Item *&remainder_cond) {
   DBUG_TRACE;
-
-  const TABLE *const table = m_handler->table;
-  if (m_pushed_cond != nullptr &&
-      !(m_pushed_cond->used_tables() & ~table->pos_in_table_list->map())) {
+  if (m_pushed_cond != nullptr && isGeneratedCodeReusable()) {
     /**
      * pushed_cond had no dependencies outside of this 'table'.
      * Code for pushed condition can be generated now, and reused
@@ -1599,6 +1605,24 @@ int ha_ndbcluster_cond::use_cond_push(const Item *&pushed_cond,
   return 0;
 }
 
+int ha_ndbcluster_cond::build_cond_push() {
+  DBUG_TRACE;
+  if (m_pushed_cond != nullptr && !isGeneratedCodeReusable()) {
+    NdbInterpretedCode code(m_handler->m_table);
+    NdbScanFilter filter(&code);
+    const int ret = generate_scan_filter_from_cond(filter);
+    if (unlikely(ret != 0)) {
+      set_condition(m_pushed_cond);
+      return ret;
+    } else {
+      // Success, keep the generated code.
+      DBUG_ASSERT(code.getWordsUsed() > 0);
+      m_scan_filter_code.copy(code);
+    }
+  }
+  return 0;
+}
+
 int ha_ndbcluster_cond::build_scan_filter_predicate(
     List_iterator<const Ndb_item> &cond, NdbScanFilter *filter,
     bool negated) const {
@@ -1611,7 +1635,7 @@ int ha_ndbcluster_cond::build_scan_filter_predicate(
       const Ndb_item *a = cond++;
       if (a == nullptr) break;
 
-      enum ndb_func_type function_type =
+      NDB_FUNC_TYPE function_type =
           (negated) ? Ndb_item::negate(ndb_item->get_func_type())
                     : ndb_item->get_func_type();
 
