@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -45,14 +44,13 @@ class Ndb_dist_priv_util {
         "columns_priv",
         "procs_priv",
         "proxies_priv"
-#ifndef NDEBUG
+#ifndef DBUG_OFF
         ,
         "mtr__acl_test_table"  // For test ndb_ddl.dist_priv_migration
 #endif
     };
 
-    if (m_iter_curr_table >= (sizeof(tables) / sizeof(tables[0])))
-      return nullptr;
+    if (m_iter_curr_table >= (sizeof(tables) / sizeof(tables[0]))) return NULL;
     m_iter_curr_table++;
     return tables[m_iter_curr_table - 1];
   }
