@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,8 +24,6 @@
 
 #include "storage/ndb/plugin/ndb_ndbapi_util.h"
 
-#include <charconv>
-
 #include <string.h>  // memcpy
 
 #include "my_byteorder.h"
@@ -167,7 +164,7 @@ bool ndb_table_has_hidden_pk(const NdbDictionary::Table *ndbtab) {
 
 bool ndb_table_has_tablespace(const NdbDictionary::Table *ndbtab) {
   // NOTE! There is a slight ambiguity in the NdbDictionary::Table.
-  // Depending on whether it has been retrieved from NDB or created
+  // Depending on wheter it has been retrieved from NDB or created
   // by user as part of defining a new table in NDB, different methods
   // need to be used for determining if table has tablespace
 
@@ -178,7 +175,7 @@ bool ndb_table_has_tablespace(const NdbDictionary::Table *ndbtab) {
 
   if (ndbtab->getTablespace()) {
     // Retrieved from NDB, the tablespace id and version
-    // are available in the table definition -> has tablespace.
+    // are avaliable in the table definition -> has tablespace.
     // NOTE! Fetching the name would require another roundtrip to NDB
     return true;
   }
@@ -254,8 +251,7 @@ bool ndb_get_tablespace_names(
 
 bool ndb_get_table_names_in_schema(
     const NdbDictionary::Dictionary *dict, const std::string &schema_name,
-    std::unordered_set<std::string> *table_names,
-    std::unordered_set<std::string> *temp_names) {
+    std::unordered_set<std::string> *table_names) {
   NdbDictionary::Dictionary::List list;
   if (dict->listObjects(list, NdbDictionary::Object::UserTable) != 0) {
     return false;
@@ -268,26 +264,26 @@ bool ndb_get_table_names_in_schema(
       continue;
     }
 
-    if (ndb_name_is_blob_prefix(elmt.name) ||
-        ndb_name_is_fk_mock_prefix(elmt.name)) {
-      continue;
-    }
-
-    // Skip temporary named tables if container for them is not provided
-    const bool is_temp = ndb_name_is_temp(elmt.name);
-    if (temp_names == nullptr && is_temp) {
+    if (ndb_name_is_temp(elmt.name) || ndb_name_is_blob_prefix(elmt.name)) {
       continue;
     }
 
     if (schema_name == "mysql" &&
         (strcmp(elmt.name, "ndb_schema") == 0 ||
          strcmp(elmt.name, "ndb_schema_result") == 0 ||
-         strcmp(elmt.name, "ndb_apply_status") == 0 ||
          strcmp(elmt.name, "ndb_sql_metadata") == 0 ||
          strcmp(elmt.name, "ndb_index_stat_head") == 0 ||
          strcmp(elmt.name, "ndb_index_stat_sample") == 0)) {
-      // Skip NDB utility tables. They are marked as hidden in the DD and are
-      // specially handled by the binlog thread.
+      // Skip NDB utility tables.
+      //
+      // The first three tables and marked as hidden in the DD and are handled
+      // specifically by the binlog thread.
+      //
+      // The index stat tables are created by the index stat thread and are not
+      // installed in the DD at all. The contents of these tables are
+      // incomprehensible without some kind of parsing and are thus not exposed
+      // to the MySQL Server. They remain visible and accessible via the
+      // ndb_select_all tool.
       continue;
     }
 
@@ -297,7 +293,7 @@ bool ndb_get_table_names_in_schema(
       // Only return the table if they're already usable i.e. StateOnline or
       // StateBackup or if they're expected to be usable soon which is denoted
       // by StateBuilding
-      is_temp ? temp_names->insert(elmt.name) : table_names->insert(elmt.name);
+      table_names->insert(elmt.name);
     }
   }
   return true;
@@ -523,36 +519,6 @@ bool ndb_table_index_count(const NdbDictionary::Dictionary *dict,
   return true;
 }
 
-bool ndb_table_have_unique_or_fk(const NdbDictionary::Dictionary *dict,
-                                 const NdbDictionary::Table *ndbtab,
-                                 bool &found) {
-  DBUG_TRACE;
-  NdbDictionary::Dictionary::List list;
-  if (dict->listDependentObjects(list, *ndbtab) != 0) {
-    // List dependent failed
-    return false;
-  }
-  for (uint i = 0; i < list.count; i++) {
-    NdbDictionary::Dictionary::List::Element &elmt = list.elements[i];
-    DBUG_PRINT("info", ("elmt[%d]: %d '%s'", i, elmt.type, elmt.name));
-
-    if (elmt.type == NdbDictionary::Object::UniqueHashIndex) {
-      DBUG_PRINT("info", ("Found UniqueHashIndex"));
-      found = true;
-      return true;
-    }
-
-    if (elmt.type == NdbDictionary::Object::ForeignKey) {
-      DBUG_PRINT("info", ("Found ForeignKey"));
-      found = true;
-      return true;
-    }
-  }
-
-  DBUG_PRINT("info", ("No unique or fk found"));
-  return true;
-}
-
 bool ndb_table_scan_and_delete_rows(
     Ndb *ndb, const THD *thd, const NdbDictionary::Table *ndb_table,
     NdbError &ndb_err,
@@ -605,7 +571,7 @@ bool ndb_table_scan_and_delete_rows(
 
         case 1:
           DBUG_PRINT("info", ("No more rows"));
-          // No more rows, commit the transaction
+          // No more rows, commit the transation
           if (trans->execute(NdbTransaction::Commit) != 0) {
             // Failed to commit
             return &trans->getNdbError();
@@ -634,74 +600,3 @@ bool ndb_table_scan_and_delete_rows(
 
   return ndb_trans_retry(ndb, thd, ndb_err, scan_and_delete_func);
 }
-
-bool ndb_get_parent_table_ids_in_dictionary(
-    const NdbDictionary::Dictionary *dict,
-    std::unordered_set<unsigned> &table_ids) {
-  DBUG_TRACE;
-
-  NdbDictionary::Dictionary::List list;
-  if (dict->listObjects(list, NdbDictionary::Object::ForeignKey) != 0) {
-    // List objects failed
-    return false;
-  }
-
-  for (unsigned i = 0; i < list.count; i++) {
-    const NdbDictionary::Dictionary::List::Element &elmt = list.elements[i];
-    DBUG_PRINT("info", ("elmt[%d]: %d '%s'", i, elmt.id, elmt.name));
-
-    // Extract parent_id from name format "<parent_id>/<child_id>/<name>"
-    unsigned parent_id = 0;
-    std::from_chars(elmt.name, elmt.name + 10, parent_id);
-    if (parent_id == 0) {
-      // unexpected format, skip
-      assert(false);
-      continue;
-    }
-
-    DBUG_PRINT("info", ("parent_id: %d", parent_id));
-    table_ids.insert(parent_id);
-  }
-  return true;
-}
-
-#ifndef NDEBUG
-#include "storage/ndb/plugin/ndb_table_guard.h"
-
-bool ndb_dump_NDB_tables(Ndb *ndb) {
-  // Get list of tables from NDB and read database names
-
-  NdbDictionary::Dictionary::List list;
-  if (ndb->getDictionary()->listObjects(list,
-                                        NdbDictionary::Object::UserTable) != 0)
-    return true;
-
-  fprintf(stderr, "ndb_dump_NDB_tables\n");
-  fprintf(stderr, "| table_id | db_name | table_name | object_version |\n");
-
-  for (uint i = 0; i < list.count; i++) {
-    NdbDictionary::Dictionary::List::Element &elmt = list.elements[i];
-    DBUG_PRINT("info", ("elmt[%d]: %d '%s'", i, elmt.id, elmt.name));
-
-    const int table_id = elmt.id;
-    const char *table_database = elmt.database;
-    assert(elmt.schema == std::string("def"));  // always "<db>/def/<name>"
-    const char *table_name = elmt.name;
-
-    Ndb_table_guard ndbtab_g(ndb, table_database, table_name);
-    const NdbDictionary::Table *ndbtab = ndbtab_g.get_table();
-    if (!ndbtab) {
-      DBUG_PRINT("info",
-                 ("failed to open table %s.%s", table_database, table_name));
-      // skip tables that cannot be opened with Ndb_table_guard from server
-      continue;
-    }
-    assert(table_id == ndbtab->getObjectId());  // same as listed
-    const unsigned table_version = ndbtab->getObjectVersion();
-
-    fprintf(stderr, "| %d | %s | %s | %u |\n", table_id, table_database,
-            table_name, table_version);
-  }
-  return false;
-}
-#endif
