@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,6 +26,7 @@
 #define NDB_NDBAPI_UTIL_H
 
 #include <stddef.h>
+#include <functional>
 #include <string>
 #include <unordered_set>
 #include <vector>
@@ -34,6 +35,9 @@
 #include "storage/ndb/include/ndbapi/NdbDictionary.hpp"
 #include "storage/ndb/include/ndbapi/NdbRecAttr.hpp"
 
+class THD;
+class NdbScanFilter;
+
 union NdbValue {
   const NdbRecAttr *rec;
   NdbBlob *blob;
@@ -82,7 +86,6 @@ void ndb_unpack_varchar(const NdbDictionary::Column *column, size_t offset,
 /**
    @brief ndb_get_extra_metadata_version, returns the version of the
           extra metadata attached to the table in NDB.
-   @param ndbtab
    @return version of extra metadata or 0 if none
  */
 Uint32 ndb_get_extra_metadata_version(const NdbDictionary::Table *ndbtab);
@@ -92,7 +95,7 @@ Uint32 ndb_get_extra_metadata_version(const NdbDictionary::Table *ndbtab);
    table in NDB.
 
    @param ndbtab The NDB table
-   @param serialized_metadata[out] variabel to receive the serialized metadata
+   @param[out] serialized_metadata variable to receive the serialized metadata
 
    @return true if table has extra metadata version 2
 */
@@ -101,7 +104,6 @@ bool ndb_table_get_serialized_metadata(const NdbDictionary::Table *ndbtab,
 
 /**
  * @brief ndb_table_has_blobs, check if the NDB table has blobs
- * @param ndbtab
  * @return true if the table have blobs
  */
 bool ndb_table_has_blobs(const NdbDictionary::Table *ndbtab);
@@ -110,14 +112,12 @@ bool ndb_table_has_blobs(const NdbDictionary::Table *ndbtab);
  * @brief ndb_table_has_hidden_pk, check if the NDB table has a hidden
  *        primary key(as created by ndbcluster to support having table
  *        without primary key in NDB)
- * @param ndbtab
  * @return true if the table has a hidden primary key
  */
 bool ndb_table_has_hidden_pk(const NdbDictionary::Table *ndbtab);
 
 /**
  * @brief check if the NDB table has tablespace
- * @param ndbtab
  * @return true if the table has a tablespace
  *
  * @note This is indicated either by the table having a tablespace name
@@ -128,7 +128,6 @@ bool ndb_table_has_tablespace(const NdbDictionary::Table *ndbtab);
 /**
  * @brief check if the NDB table has tablespace name indicating
  *        that is has a tablespace
- * @param ndbtab
  * @return nullptr or tablespace name
  *
  * @note The NdbApi function getTablespaceName() is peculiar as it
@@ -179,15 +178,11 @@ bool ndb_get_tablespace_names(
  * @param dict              NDB Dictionary
  * @param schema_name       Schema name
  * @param table_names [out] List of table names
- * @param skip_util_tables  Controls whether the NDB utility tables i.e.
- *                          ndb_schema, ndb_schema_result, and ndb_sql_metadata
- *                          are included in list of table names returned or not
  * @return true on success, false on failure
  */
-bool ndb_get_table_names_in_schema(const NdbDictionary::Dictionary *dict,
-                                   const std::string &schema_name,
-                                   std::unordered_set<std::string> *table_names,
-                                   bool skip_util_tables = true);
+bool ndb_get_table_names_in_schema(
+    const NdbDictionary::Dictionary *dict, const std::string &schema_name,
+    std::unordered_set<std::string> *table_names);
 
 /**
  * @brief Retrieves list of undofile names assigned to a logfile group from NDB
@@ -305,5 +300,20 @@ bool ndb_get_tablespace_id_and_version(NdbDictionary::Dictionary *dict,
 bool ndb_table_index_count(const NdbDictionary::Dictionary *dict,
                            const NdbDictionary::Table *ndbtab,
                            unsigned int &index_count);
+/**
+ * @brief Scan the given table and delete the rows returned
+ * @param ndb                    The Ndb Object
+ * @param thd                    The THD object
+ * @param ndb_table              NDB Table object whose rows are to be deleted
+ * @param ndb_err [out]          NdbError object to send back the error during
+ *                               a failure
+ * @param ndb_scan_filter_defn   An optional std::function defining a
+ *                               NdbScanFilter to be used by the scan
+ * @return true if the rows were successfully deleted, false if not
+ */
+bool ndb_table_scan_and_delete_rows(
+    Ndb *ndb, const THD *thd, const NdbDictionary::Table *ndb_table,
+    NdbError &ndb_err,
+    const std::function<void(NdbScanFilter &)> &ndb_scan_filter_defn = nullptr);
 
 #endif
