@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,18 +35,24 @@
 #include "sql/sql_error.h"
 #include "sql/sql_thd_internal_api.h"
 #include "storage/ndb/plugin/ndb_anyvalue.h"
-#include "storage/ndb/plugin/ndb_applier.h"
 #include "storage/ndb/plugin/ndb_dist_priv_util.h"
 #include "storage/ndb/plugin/ndb_name_util.h"
 #include "storage/ndb/plugin/ndb_require.h"
 #include "storage/ndb/plugin/ndb_schema_dist_table.h"
-#include "storage/ndb/plugin/ndb_schema_object.h"
 #include "storage/ndb/plugin/ndb_schema_result_table.h"
 #include "storage/ndb/plugin/ndb_share.h"
 #include "storage/ndb/plugin/ndb_thd.h"
 #include "storage/ndb/plugin/ndb_thd_ndb.h"
 #include "storage/ndb/plugin/ndb_upgrade_util.h"
 
+// Temporarily use a fixed string on the form "./mysql/ndb_schema" as key
+// for retrieving the NDB_SHARE for mysql.ndb_schema. This will subsequently
+// be removed when a NDB_SHARE can be acquired using db+table_name and the
+// key is formatted behind the curtains in NDB_SHARE without using
+// build_table_filename() etc.
+static constexpr const char *NDB_SCHEMA_TABLE_KEY =
+    IF_WIN(".\\mysql\\ndb_schema", "./mysql/ndb_schema");
+
 bool Ndb_schema_dist::is_ready(void *requestor) {
   DBUG_TRACE;
 
@@ -55,10 +60,8 @@ bool Ndb_schema_dist::is_ready(void *requestor) {
   ss << "is_ready_" << std::hex << requestor;
   const std::string reference = ss.str();
 
-  // Acquire reference on mysql.ndb_schema
-  NDB_SHARE *schema_share = NDB_SHARE::acquire_reference(
-      Ndb_schema_dist_table::DB_NAME.c_str(),
-      Ndb_schema_dist_table::TABLE_NAME.c_str(), reference.c_str());
+  NDB_SHARE *schema_share = NDB_SHARE::acquire_reference_by_key(
+      NDB_SCHEMA_TABLE_KEY, reference.c_str());
   if (schema_share == nullptr) return false;  // Not ready
 
   if (!schema_share->have_event_operation()) {
@@ -119,17 +122,9 @@ Ndb_schema_dist_client::Ndb_schema_dist_client(THD *thd)
 bool Ndb_schema_dist_client::prepare(const char *db, const char *tabname) {
   DBUG_TRACE;
 
-  // Check local schema distribution state
-  if (!check_local_schema_dist_available()) {
-    push_warning(m_thd, Sql_condition::SL_WARNING, ER_GET_ERRMSG,
-                 "Schema distribution is not ready");
-    return false;
-  }
-
   // Acquire reference on mysql.ndb_schema
-  m_share = NDB_SHARE::acquire_reference(
-      Ndb_schema_dist_table::DB_NAME.c_str(),
-      Ndb_schema_dist_table::TABLE_NAME.c_str(), m_share_reference.c_str());
+  m_share = NDB_SHARE::acquire_reference_by_key(NDB_SCHEMA_TABLE_KEY,
+                                                m_share_reference.c_str());
 
   if (m_share == nullptr || m_share->have_event_operation() == false ||
       DBUG_EVALUATE_IF("ndb_schema_dist_not_ready_early", true, false)) {
@@ -140,19 +135,6 @@ bool Ndb_schema_dist_client::prepare(const char *db, const char *tabname) {
     return false;
   }
 
-  // Acquire reference also on mysql.ndb_schema_result
-  m_result_share = NDB_SHARE::acquire_reference(
-      Ndb_schema_result_table::DB_NAME.c_str(),
-      Ndb_schema_result_table::TABLE_NAME.c_str(), m_share_reference.c_str());
-  if (m_result_share == nullptr ||
-      m_result_share->have_event_operation() == false) {
-    // The mysql.ndb_schema_result hasn't been created or not setup yet ->
-    // schema distribution is not ready
-    push_warning(m_thd, Sql_condition::SL_WARNING, ER_GET_ERRMSG,
-                 "Schema distribution is not ready (ndb_schema_result)");
-    return false;
-  }
-
   if (unlikely(m_ddl_blocked)) {
     // If a data node gets upgraded after this MySQL Server is upgraded, this
     // MySQL Server will not be aware of the upgrade due to Bug#30930132.
@@ -170,7 +152,7 @@ bool Ndb_schema_dist_client::prepare(const char *db, const char *tabname) {
         // If the tablename is not empty, it is a non database DDL. Block it.
         my_printf_error(
             ER_DISALLOWED_OPERATION,
-            "DDLs are disallowed on NDB SE as there is at least one node "
+            "DDLs are disallowed on NDB SE as there is atleast one node "
             "without MySQL DD support connected to the cluster.",
             MYF(0));
         return false;
@@ -264,7 +246,7 @@ bool Ndb_schema_dist_client::check_identifier_limits(
 
   // Check that identifiers does not exceed the limits imposed
   // by the ndb_schema table layout
-  for (const auto &key : m_prepared_keys.keys()) {
+  for (auto key : m_prepared_keys.keys()) {
     // db
     if (!schema_dist_table.check_column_identifier_limit(
             Ndb_schema_dist_table::COL_DB, key.first)) {
@@ -288,7 +270,7 @@ void Ndb_schema_dist_client::Prepared_keys::add_key(const char *db,
 
 bool Ndb_schema_dist_client::Prepared_keys::check_key(
     const char *db, const char *tabname) const {
-  for (const auto &key : m_keys) {
+  for (auto key : m_keys) {
     if (key.first == db && key.second == tabname) {
       return true;  // OK, key has been prepared
     }
@@ -296,22 +278,20 @@ bool Ndb_schema_dist_client::Prepared_keys::check_key(
   return false;
 }
 
+extern void update_slave_api_stats(const Ndb *);
+
 Ndb_schema_dist_client::~Ndb_schema_dist_client() {
   if (m_share) {
     // Release the reference to mysql.ndb_schema table
     NDB_SHARE::release_reference(m_share, m_share_reference.c_str());
   }
-  if (m_result_share) {
-    // Release the reference to mysql.ndb_schema_result table
-    NDB_SHARE::release_reference(m_result_share, m_share_reference.c_str());
-  }
 
-  if (m_thd_ndb) {
-    // Inform Applier that one schema distribution has completed
-    Ndb_applier *const applier = m_thd_ndb->get_applier();
-    if (applier) {
-      applier->atSchemaDistCompleted();
-    }
+  if (m_thd_ndb && m_thd_ndb->is_slave_thread()) {
+    // Copy-out slave thread statistics
+    // NOTE! This is just a "convenient place" to call this
+    // function, it could be moved to "end of statement"(if there
+    // was such a place..).
+    update_slave_api_stats(m_thd_ndb->ndb);
   }
 
   if (m_holding_acl_mutex) {
@@ -331,7 +311,7 @@ uint32 Ndb_schema_dist_client::unique_id() const {
   if (id == 0) {
     id = ++schema_dist_id_sequence;
   }
-  assert(id != 0);
+  DBUG_ASSERT(id != 0);
   return id;
 }
 
@@ -342,19 +322,10 @@ uint32 Ndb_schema_dist_client::unique_id() const {
 */
 uint32 Ndb_schema_dist_client::unique_version() const {
   const uint32 ver = m_thd_ndb->connection->node_id();
-  assert(ver != 0);
+  DBUG_ASSERT(ver != 0);
   return ver;
 }
 
-void Ndb_schema_dist_client::save_schema_op_results(
-    const NDB_SCHEMA_OBJECT *ndb_schema_object) {
-  std::vector<NDB_SCHEMA_OBJECT::Result> participant_results;
-  ndb_schema_object->client_get_schema_op_results(participant_results);
-  for (const auto &it : participant_results) {
-    m_schema_op_results.push_back({it.nodeid, it.result, it.message});
-  }
-}
-
 void Ndb_schema_dist_client::push_and_clear_schema_op_results() {
   if (m_schema_op_results.empty()) {
     return;
@@ -362,7 +333,7 @@ void Ndb_schema_dist_client::push_and_clear_schema_op_results() {
 
   // Push results received from participant(s) as warnings. These are meant to
   // indicate that schema distribution has failed on one of the nodes. For more
-  // information on how and why the failure occurred, the relevant error log
+  // information on how and why the failure occured, the relevant error log
   // remains the place to look
   for (const Schema_op_result &op_result : m_schema_op_results) {
     // Warning consists of the node id and message but not result code since
@@ -382,24 +353,24 @@ bool Ndb_schema_dist_client::log_schema_op(const char *query,
                                            uint32 version, SCHEMA_OP_TYPE type,
                                            bool log_query_on_participant) {
   DBUG_TRACE;
-  assert(db && table_name);
-  assert(id != 0 && version != 0);
-  assert(m_thd_ndb);
+  DBUG_ASSERT(db && table_name);
+  DBUG_ASSERT(id != 0 && version != 0);
+  DBUG_ASSERT(m_thd_ndb);
 
   // Never allow temporary names when communicating with participant
   if (ndb_name_is_temp(db) || ndb_name_is_temp(table_name)) {
-    assert(false);
+    DBUG_ASSERT(false);
     return false;
   }
 
-  // Require that references to schema distribution tables has been initialized
+  // Require that m_share has been initialized to reference the
+  // schema distribution table
   ndbcluster::ndbrequire(m_share);
-  ndbcluster::ndbrequire(m_result_share);
 
   // Check that prepared keys match
   if (!m_prepared_keys.check_key(db, table_name)) {
     m_thd_ndb->push_warning("INTERNAL ERROR: prepared keys didn't match");
-    assert(false);  // Catch in debug
+    DBUG_ASSERT(false);  // Catch in debug
     return false;
   }
 
@@ -414,7 +385,7 @@ bool Ndb_schema_dist_client::log_schema_op(const char *query,
     std::string invalid_identifier;
     if (!check_identifier_limits(invalid_identifier)) {
       m_thd_ndb->push_warning("INTERNAL ERROR: identifier limits exceeded");
-      assert(false);  // Catch in debug
+      DBUG_ASSERT(false);  // Catch in debug
       return false;
     }
   }
@@ -495,10 +466,6 @@ bool Ndb_schema_dist_client::rename_table_prepare(
   // pass the "new key"(i.e db/table_name) for the table to be renamed,
   // that's since there isn't enough placeholders in the subsequent rename
   // table phase.
-  // NOTE2! The "new key" is sent in filesystem format where multibyte or
-  // characters who are deemed not suitable as filenames have been encoded. This
-  // differs from the db and tablename parameters in the schema dist
-  // protocol who are just passed as they are.
   return log_schema_op(new_key_for_table, strlen(new_key_for_table), db,
                        table_name, id, version, SOT_RENAME_TABLE_PREPARE);
 }
@@ -545,7 +512,7 @@ bool Ndb_schema_dist_client::drop_table(const char *db, const char *table_name,
        the participants get the DROP DATABASE it will remove
        any tables from the DD and then remove the database.
   */
-  assert(thd_sql_command(m_thd) != SQLCOM_DROP_DB);
+  DBUG_ASSERT(thd_sql_command(m_thd) != SQLCOM_DROP_DB);
 
   /*
     Rewrite the query, the original query may contain several tables but
@@ -633,7 +600,7 @@ bool Ndb_schema_dist_client::acl_notify(const char *database, const char *query,
                                         uint query_length,
                                         bool participant_refresh) {
   DBUG_TRACE;
-  assert(m_holding_acl_mutex);
+  DBUG_ASSERT(m_holding_acl_mutex);
   auto key = m_prepared_keys.keys()[0];
   std::string new_query("use ");
   if (database != nullptr && strcmp(database, "mysql")) {
@@ -650,7 +617,7 @@ bool Ndb_schema_dist_client::acl_notify(const char *database, const char *query,
 /* SNAPSHOT-style ACL change distribution */
 bool Ndb_schema_dist_client::acl_notify(std::string user_list) {
   DBUG_TRACE;
-  assert(m_holding_acl_mutex);
+  DBUG_ASSERT(m_holding_acl_mutex);
   auto key = m_prepared_keys.keys()[0];
 
   return log_schema_op(user_list.c_str(), user_list.length(), key.first.c_str(),
@@ -658,6 +625,20 @@ bool Ndb_schema_dist_client::acl_notify(std::string user_list) {
                        SOT_ACL_SNAPSHOT);
 }
 
+bool Ndb_schema_dist_client::tablespace_changed(const char *tablespace_name,
+                                                int id, int version) {
+  DBUG_TRACE;
+  return log_schema_op(ndb_thd_query(m_thd), ndb_thd_query_length(m_thd), "",
+                       tablespace_name, id, version, SOT_TABLESPACE);
+}
+
+bool Ndb_schema_dist_client::logfilegroup_changed(const char *logfilegroup_name,
+                                                  int id, int version) {
+  DBUG_TRACE;
+  return log_schema_op(ndb_thd_query(m_thd), ndb_thd_query_length(m_thd), "",
+                       logfilegroup_name, id, version, SOT_LOGFILE_GROUP);
+}
+
 bool Ndb_schema_dist_client::create_tablespace(const char *tablespace_name,
                                                int id, int version) {
   DBUG_TRACE;
@@ -763,13 +744,13 @@ const char *Ndb_schema_dist_client::type_name(SCHEMA_OP_TYPE type) {
     default:
       break;
   }
-  assert(false);
+  DBUG_ASSERT(false);
   return "<unknown>";
 }
 
 uint32 Ndb_schema_dist_client::calculate_anyvalue(bool force_nologging) const {
   Uint32 anyValue = 0;
-  if (!m_thd_ndb->get_applier()) {
+  if (!thd_slave_thread(m_thd)) {
     /* Schema change originating from this MySQLD, check SQL_LOG_BIN
      * variable and pass 'setting' to all logging MySQLDs via AnyValue
      */
@@ -801,13 +782,13 @@ uint32 Ndb_schema_dist_client::calculate_anyvalue(bool force_nologging) const {
     anyValue = thd_unmasked_server_id(m_thd);
   }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   /*
     MySQLD will set the user-portion of AnyValue (if any) to all 1s
     This tests code filtering ServerIds on the value of server-id-bits.
   */
   const char *p = getenv("NDB_TEST_ANYVALUE_USERDATA");
-  if (p != nullptr && *p != 0 && *p != '0' && *p != 'n' && *p != 'N') {
+  if (p != 0 && *p != 0 && *p != '0' && *p != 'n' && *p != 'N') {
     dbug_ndbcluster_anyvalue_set_userbits(anyValue);
   }
 #endif
