@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +31,6 @@
 #include <string>
 #include <unordered_map>
 
-#include "my_dbug.h"
 #include "storage/ndb/plugin/ndb_bitmap.h"
 #include "storage/ndb/plugin/ndb_require.h"
 
@@ -67,10 +65,10 @@ class Ndb_schema_objects {
      there is only one entry in the hash.
    */
   NDB_SCHEMA_OBJECT *find(uint32 nodeid, uint32 schema_op_id) const {
-    assert(nodeid);
+    DBUG_ASSERT(nodeid);
 
     // Make sure that own nodeid has been set
-    assert(m_own_nodeid);
+    DBUG_ASSERT(m_own_nodeid);
 
     if (nodeid != m_own_nodeid) {
       // Looking for a schema operation started in another node,  the
@@ -87,10 +85,10 @@ class Ndb_schema_objects {
 } active_schema_clients;
 
 void NDB_SCHEMA_OBJECT::init(uint32 nodeid) {
-  assert(nodeid);
+  DBUG_ASSERT(nodeid);
   std::lock_guard<std::mutex> lock_hash(active_schema_clients.m_lock);
   // Make sure that no active schema clients exist when function is called
-  assert(active_schema_clients.m_hash.size() == 0);
+  DBUG_ASSERT(active_schema_clients.m_hash.size() == 0);
   active_schema_clients.m_own_nodeid = nodeid;
 }
 
@@ -101,7 +99,7 @@ static uint32 next_schema_op_id() {
   if (id == 0) {
     id = schema_op_id_sequence++;
   }
-  assert(id != 0);
+  DBUG_ASSERT(id != 0);
   return id;
 }
 
@@ -132,11 +130,11 @@ NDB_SCHEMA_OBJECT::NDB_SCHEMA_OBJECT(const char *key, const char *db,
       m_started(std::chrono::steady_clock::now()) {}
 
 NDB_SCHEMA_OBJECT::~NDB_SCHEMA_OBJECT() {
-  assert(state.m_use_count == 0);
+  DBUG_ASSERT(state.m_use_count == 0);
   // Check that all participants have completed
-  assert(state.m_participants.size() == count_completed_participants());
+  DBUG_ASSERT(state.m_participants.size() == count_completed_participants());
   // Coordinator should have completed
-  assert(state.m_coordinator_completed);
+  DBUG_ASSERT(state.m_coordinator_completed);
 }
 
 NDB_SCHEMA_OBJECT *NDB_SCHEMA_OBJECT::get(const char *db,
@@ -290,23 +288,10 @@ size_t NDB_SCHEMA_OBJECT::count_completed_participants() const {
   return count;
 }
 
-bool NDB_SCHEMA_OBJECT::register_participants(
-    const std::unordered_set<uint32> &nodes) {
+void NDB_SCHEMA_OBJECT::register_participants(
+    const std::unordered_set<uint32> &nodes) const {
   std::lock_guard<std::mutex> lock_state(state.m_lock);
 
-  if (state.m_participants.size()) {
-    // There are already participants registered, this means that the client has
-    // failed the schema operation (most likley due to timeout).
-    // As part of failing it has inserted one participant where it's assigned
-    // result + error message.
-    ndbcluster::ndbrequire(
-        state.m_participants.size() == 1 &&
-        state.m_participants.count(active_schema_clients.m_own_nodeid) == 1 &&
-        state.m_coordinator_completed);
-
-    return false;
-  }
-
   // Assume the list of participants is empty
   ndbcluster::ndbrequire(state.m_participants.size() == 0);
   // Assume coordinator have not completed
@@ -317,11 +302,9 @@ bool NDB_SCHEMA_OBJECT::register_participants(
 
   // Double check that there are as many participants as nodes
   ndbcluster::ndbrequire(nodes.size() == state.m_participants.size());
-
-  return true;
 }
 
-bool NDB_SCHEMA_OBJECT::result_received_from_node(
+void NDB_SCHEMA_OBJECT::result_received_from_node(
     uint32 participant_node_id, uint32 result,
     const std::string &message) const {
   std::lock_guard<std::mutex> lock_state(state.m_lock);
@@ -331,17 +314,14 @@ bool NDB_SCHEMA_OBJECT::result_received_from_node(
     // Received reply from node not registered as participant, may happen
     // when a node hears the schema op but this node hasn't registered it as
     // subscriber yet.
-    return false;  // Not registered
+    return;
   }
 
   // Mark participant as completed and save result
   State::Participant &participant = it->second;
-  if (!participant.m_completed) {
-    participant.m_completed = true;
-    participant.m_result = result;
-    participant.m_message = message;
-  }
-  return true;
+  participant.m_completed = true;
+  participant.m_result = result;
+  participant.m_message = message;
 }
 
 void NDB_SCHEMA_OBJECT::result_received_from_nodes(
@@ -396,7 +376,7 @@ bool NDB_SCHEMA_OBJECT::check_for_failed_subscribers(
     const char *message) const {
   std::unique_lock<std::mutex> lock_state(state.m_lock);
   // Can be called only after the coordinator has registered participants
-  assert(state.m_participants.size() > 0);
+  DBUG_ASSERT(state.m_participants.size() > 0);
 
   // Fail participants not in list of nodes
   fail_participants_not_in_list(new_subscribers, result, message);
@@ -410,30 +390,13 @@ bool NDB_SCHEMA_OBJECT::check_for_failed_subscribers(
   return true;
 }
 
-bool NDB_SCHEMA_OBJECT::check_timeout(bool is_client, int timeout_seconds,
-                                      uint32 result,
+bool NDB_SCHEMA_OBJECT::check_timeout(int timeout_seconds, uint32 result,
                                       const char *message) const {
   std::unique_lock<std::mutex> lock_state(state.m_lock);
 
-  if (is_client && state.m_participants.size()) {
-    // The client is checking for timeout but participants have been registered,
-    // this means that coordinator has taken over timeout checking
-    return false;
-  }
-
   if (m_started + std::chrono::seconds(timeout_seconds) >
       std::chrono::steady_clock::now())
-    return false;  // Timeout has not occurred
-
-  if (is_client) {
-    // Timeout has expired, since this is the client no participants are
-    // registered as the coordinator haven't yet got the first schema event,
-    // add own node as participant
-    state.m_participants[active_schema_clients.m_own_nodeid];
-
-    // Mark coordinator as completed
-    state.m_coordinator_completed = true;
-  }
+    return false;  // Timeout has not occured
 
   // Mark all participants who hasn't already completed as timedout
   for (auto &it : state.m_participants) {
@@ -474,7 +437,7 @@ void NDB_SCHEMA_OBJECT::fail_schema_op(uint32 result,
   // All participant should now have been marked as completed
   ndbcluster::ndbrequire(state.m_participants.size() ==
                          count_completed_participants());
-  // Mark also coordinator as completed
+  // Mark als coordinator as completed
   state.m_coordinator_completed = true;
 }
 
