@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,7 +30,6 @@
 
 #include "m_string.h"
 #include "sql/sql_class.h"
-#include "sql/sql_table.h"
 #include "sql/strfunc.h"
 #include "sql/table.h"
 #include "storage/ndb/include/ndbapi/NdbEventOperation.hpp"
@@ -42,90 +41,74 @@
 #include "storage/ndb/plugin/ndb_table_map.h"
 
 extern Ndb *g_ndb;
-
-// Protecting the list of NDB_SHARE's as well as each shares refcount and list
-// of references
-static mysql_mutex_t shares_mutex;
+extern mysql_mutex_t ndbcluster_mutex;
 
 // List of NDB_SHARE's which correspond to an open table.
-static std::unique_ptr<collation_unordered_map<std::string, NDB_SHARE *>>
-    ndb_shares;
+std::unique_ptr<collation_unordered_map<std::string, NDB_SHARE *>>
+    ndbcluster_open_tables;
 
 // List of NDB_SHARE's which have been dropped, they are kept in this list
 // until all references to them have been released.
 static std::unordered_set<NDB_SHARE *> dropped_shares;
 
-// Format the key used for storing NDB_SHARE's in the list of open tables
-static inline void format_key(char *buf, size_t buf_size, const char *db_name,
-                              const char *table_name) {
-  // Format key in filesystem format, detect temporary named tables and use
-  // FN_IS_TMP since those doesn't need to be prefixed with db name or
-  // translated to filesystem format.
-  // NOTE! This function is still using filesystem format in order to be
-  // compatible with the NDB_SHARE_KEY format that's used as key in the list of
-  // open NDB_SHARE's.
-  build_table_filename(buf, buf_size, db_name, table_name, "",
-                       ndb_name_is_temp(table_name) ? FN_IS_TMP : 0);
-}
-
-NDB_SHARE::NDB_SHARE(const char *key_arg) {
-  /* Allocates enough space for key, db, and table_name */
-  key = NDB_SHARE::create_key(key_arg);
-
-  db = NDB_SHARE::key_get_db_name(key);
-  table_name = NDB_SHARE::key_get_table_name(key);
-
-  thr_lock_init(&lock);
-  mysql_mutex_init(PSI_INSTRUMENT_ME, &mutex, MY_MUTEX_INIT_FAST);
-
-#ifndef NDEBUG
-  refs = new Ndb_share_references();
-#endif
-
-  // Turn on special flag for mysql.ndb_apply_status table
-  if (strcmp(db, "mysql") == 0 && strcmp(table_name, "ndb_apply_status") == 0) {
-    DBUG_PRINT("info", ("Setting FLAG_TABLE_IS_APPLY_STATUS"));
-    flags |= FLAG_TABLE_IS_APPLY_STATUS;
-  }
-
-  // Initialize to preserve same behaviour as zerofilling malloc
-  // in prior versions
-  tuple_id_range.m_first_tuple_id = 0;
-  tuple_id_range.m_last_tuple_id = 0;
-  tuple_id_range.m_highest_seen = 0;
-}
-
 NDB_SHARE *NDB_SHARE::create(const char *key) {
   if (DBUG_EVALUATE_IF("ndb_share_create_fail1", true, false)) {
     // Simulate failure to create NDB_SHARE
     return nullptr;
   }
 
-  return new (std::nothrow) NDB_SHARE(key);
+  NDB_SHARE *share;
+  if (!(share = (NDB_SHARE *)my_malloc(PSI_INSTRUMENT_ME, sizeof(*share),
+                                       MYF(MY_WME | MY_ZEROFILL))))
+    return nullptr;
+
+  share->flags = 0;
+  share->state = NSS_INITIAL;
+
+  /* Allocates enough space for key, db, and table_name */
+  share->key = NDB_SHARE::create_key(key);
+
+  share->db = NDB_SHARE::key_get_db_name(share->key);
+  share->table_name = NDB_SHARE::key_get_table_name(share->key);
+
+  thr_lock_init(&share->lock);
+  mysql_mutex_init(PSI_INSTRUMENT_ME, &share->mutex, MY_MUTEX_INIT_FAST);
+
+  share->m_cfn_share = nullptr;
+
+  share->op = 0;
+
+#ifndef DBUG_OFF
+  DBUG_ASSERT(share->m_use_count == 0);
+  share->refs = new Ndb_share_references();
+#endif
+
+  share->inplace_alter_new_table_def = nullptr;
+
+  return share;
 }
 
-NDB_SHARE::~NDB_SHARE() {
-  thr_lock_delete(&lock);
-  mysql_mutex_destroy(&mutex);
+void NDB_SHARE::destroy(NDB_SHARE *share) {
+  thr_lock_delete(&share->lock);
+  mysql_mutex_destroy(&share->mutex);
 
   // ndb_index_stat_free() should have cleaned up:
-  assert(index_stat_list == nullptr);
+  assert(share->index_stat_list == NULL);
 
-  teardown_conflict_fn(g_ndb, m_cfn_share);
+  teardown_conflict_fn(g_ndb, share->m_cfn_share);
 
-#ifndef NDEBUG
-  assert(m_use_count == 0);
-  assert(refs->check_empty());
-  delete refs;
+#ifndef DBUG_OFF
+  DBUG_ASSERT(share->m_use_count == 0);
+  DBUG_ASSERT(share->refs->check_empty());
+  delete share->refs;
 #endif
 
   // Release memory for the variable length strings held by
   // key but also referenced by db, table_name and shadow_table->db etc.
-  free_key(key);
+  free_key(share->key);
+  my_free(share);
 }
 
-void NDB_SHARE::destroy(NDB_SHARE *share) { delete share; }
-
 /*
   Struct holding dynamic length strings for NDB_SHARE. The type is
   opaque to the user of NDB_SHARE and should
@@ -224,7 +207,7 @@ const char *NDB_SHARE::key_string() const {
   return key->m_buffer;
 }
 
-const char *NDB_SHARE::share_state_string() const {
+const char *NDB_SHARE::share_state_string(void) const {
   switch (state) {
     case NSS_INITIAL:
       return "NSS_INITIAL";
@@ -237,97 +220,73 @@ const char *NDB_SHARE::share_state_string() const {
 
 void NDB_SHARE::free_share(NDB_SHARE **share) {
   DBUG_TRACE;
-  mysql_mutex_assert_owner(&shares_mutex);
+  mysql_mutex_assert_owner(&ndbcluster_mutex);
 
   if (!(*share)->decrement_use_count()) {
-    // No one is using the NDB_SHARE anymore, release it
+    // Noone is using the NDB_SHARE anymore, release it
     NDB_SHARE::real_free_share(share);
   }
 }
 
-NDB_SHARE *NDB_SHARE::acquire_or_create_reference(const char *db_name,
-                                                  const char *table_name,
-                                                  const char *reference) {
+NDB_SHARE *NDB_SHARE::create_and_acquire_reference(const char *key,
+                                                   const char *reference) {
   DBUG_TRACE;
-
-  char key[FN_REFLEN + 1];
-  format_key(key, sizeof(key) - 1, db_name, table_name);
-
   DBUG_PRINT("enter", ("key: '%s'", key));
 
-  mysql_mutex_lock(&shares_mutex);
+  mysql_mutex_assert_owner(&ndbcluster_mutex);
 
-  NDB_SHARE *share = acquire_reference_impl(key);
-  if (share) {
-    share->refs_insert(reference);
-    DBUG_PRINT("NDB_SHARE",
-               ("'%s', reference: '%s', use_count: %u", share->key_string(),
-                reference, share->use_count()));
-    mysql_mutex_unlock(&shares_mutex);
-    return share;
-  }
+  // Make sure that the SHARE does not already exist
+  DBUG_ASSERT(!acquire_reference_impl(key));
 
-  // No NDB_SHARE existed, create one
-  share = NDB_SHARE::create(key);
+  NDB_SHARE *share = NDB_SHARE::create(key);
   if (share == nullptr) {
     DBUG_PRINT("error", ("failed to create NDB_SHARE"));
-    mysql_mutex_unlock(&shares_mutex);
     return nullptr;
   }
 
   // Insert the new share in list of open shares
-  ndb_shares->emplace(key, share);
+  ndbcluster_open_tables->emplace(key, share);
 
-  // Add share refcount from 'ndb_shares', the list owns a reference
+  // Add share refcount from 'ndbcluster_open_tables'
   share->increment_use_count();
-  share->refs_insert("ndb_shares");
+  share->refs_insert("ndbcluster_open_tables");
 
   // Add refcount for returned 'share'.
   share->increment_use_count();
   share->refs_insert(reference);
 
-  mysql_mutex_unlock(&shares_mutex);
-
   return share;
 }
 
-NDB_SHARE *NDB_SHARE::create_for_handler(const char *db_name,
-                                         const char *table_name,
-                                         const ha_ndbcluster *reference) {
-  char key[FN_REFLEN + 1];
-  format_key(key, sizeof(key) - 1, db_name, table_name);
-
-  mysql_mutex_lock(&shares_mutex);
+NDB_SHARE *NDB_SHARE::create_and_acquire_reference(
+    const char *key, const class ha_ndbcluster *reference) {
+  mysql_mutex_lock(&ndbcluster_mutex);
 
   NDB_SHARE *share = NDB_SHARE::create(key);
   if (share == nullptr)
     ndb_log_error("failed to create NDB_SHARE for key: %s", key);
   else {
     // Insert the new share in list of open shares
-    ndb_shares->emplace(key, share);
+    ndbcluster_open_tables->emplace(key, share);
 
-    // Add share refcount from 'ndb_shares', the list owns a reference
+    // Add share refcount from 'ndbcluster_open_tables'
     share->increment_use_count();
-    share->refs_insert("ndb_shares");
+    share->refs_insert("ndbcluster_open_tables");
 
     // Add refcount for returned 'share'.
     share->increment_use_count();
     share->refs_insert(reference);
   }
 
-  mysql_mutex_unlock(&shares_mutex);
+  mysql_mutex_unlock(&ndbcluster_mutex);
   return share;
 }
 
-NDB_SHARE *NDB_SHARE::acquire_for_handler(const char *db_name,
-                                          const char *table_name,
-                                          const ha_ndbcluster *reference) {
+NDB_SHARE *NDB_SHARE::acquire_for_handler(
+    const char *key, const class ha_ndbcluster *reference) {
   DBUG_TRACE;
 
-  char key[FN_REFLEN + 1];
-  format_key(key, sizeof(key) - 1, db_name, table_name);
-
-  mysql_mutex_lock(&shares_mutex);
+  mysql_mutex_lock(&ndbcluster_mutex);
   NDB_SHARE *share = acquire_reference_impl(key);
   if (share) {
     share->refs_insert(reference);
@@ -336,7 +295,7 @@ NDB_SHARE *NDB_SHARE::acquire_for_handler(const char *db_name,
                 "use_count: %u",
                 share->key_string(), reference, share->use_count()));
   }
-  mysql_mutex_unlock(&shares_mutex);
+  mysql_mutex_unlock(&ndbcluster_mutex);
 
   return share;
 }
@@ -345,7 +304,7 @@ void NDB_SHARE::release_for_handler(NDB_SHARE *share,
                                     const ha_ndbcluster *reference) {
   DBUG_TRACE;
 
-  mysql_mutex_lock(&shares_mutex);
+  mysql_mutex_lock(&ndbcluster_mutex);
 
   DBUG_PRINT("NDB_SHARE", ("release '%s', reference: 'ha_ndbcluster(%p)', "
                            "use_count: %u",
@@ -353,7 +312,7 @@ void NDB_SHARE::release_for_handler(NDB_SHARE *share,
 
   share->refs_erase(reference);
   NDB_SHARE::free_share(&share);
-  mysql_mutex_unlock(&shares_mutex);
+  mysql_mutex_unlock(&ndbcluster_mutex);
 }
 
 /*
@@ -362,12 +321,12 @@ void NDB_SHARE::release_for_handler(NDB_SHARE *share,
 
 NDB_SHARE *NDB_SHARE::acquire_reference_on_existing(NDB_SHARE *share,
                                                     const char *reference) {
-  mysql_mutex_lock(&shares_mutex);
+  mysql_mutex_lock(&ndbcluster_mutex);
 
   // Should already be referenced
-  assert(share->use_count() > 0);
+  DBUG_ASSERT(share->use_count() > 0);
   // Number of references should match use_count
-  assert(share->use_count() == share->refs->size());
+  DBUG_ASSERT(share->use_count() == share->refs->size());
 
   share->increment_use_count();
   share->refs_insert(reference);
@@ -375,20 +334,33 @@ NDB_SHARE *NDB_SHARE::acquire_reference_on_existing(NDB_SHARE *share,
   DBUG_PRINT("NDB_SHARE", ("'%s', reference: '%s', use_count: %u",
                            share->key_string(), reference, share->use_count()));
 
-  mysql_mutex_unlock(&shares_mutex);
+  mysql_mutex_unlock(&ndbcluster_mutex);
   return share;
 }
 
 /*
-  Acquire reference using db and table name
+  Acquire reference using key.
 */
-NDB_SHARE *NDB_SHARE::acquire_reference(const char *db_name,
-                                        const char *table_name,
-                                        const char *reference) {
-  char key[FN_REFLEN + 1];
-  format_key(key, sizeof(key) - 1, db_name, table_name);
 
-  mysql_mutex_lock(&shares_mutex);
+NDB_SHARE *NDB_SHARE::acquire_reference_by_key(const char *key,
+                                               const char *reference) {
+  mysql_mutex_lock(&ndbcluster_mutex);
+
+  NDB_SHARE *share = acquire_reference_impl(key);
+  if (share) {
+    share->refs_insert(reference);
+    DBUG_PRINT("NDB_SHARE",
+               ("'%s', reference: '%s', use_count: %u", share->key_string(),
+                reference, share->use_count()));
+  }
+
+  mysql_mutex_unlock(&ndbcluster_mutex);
+  return share;
+}
+
+NDB_SHARE *NDB_SHARE::acquire_reference_by_key_have_lock(
+    const char *key, const char *reference) {
+  mysql_mutex_assert_owner(&ndbcluster_mutex);
 
   NDB_SHARE *share = acquire_reference_impl(key);
   if (share) {
@@ -398,12 +370,11 @@ NDB_SHARE *NDB_SHARE::acquire_reference(const char *db_name,
                 reference, share->use_count()));
   }
 
-  mysql_mutex_unlock(&shares_mutex);
   return share;
 }
 
 void NDB_SHARE::release_reference(NDB_SHARE *share, const char *reference) {
-  mysql_mutex_lock(&shares_mutex);
+  mysql_mutex_lock(&ndbcluster_mutex);
 
   DBUG_PRINT("NDB_SHARE", ("release '%s', reference: '%s', use_count: %u",
                            share->key_string(), reference, share->use_count()));
@@ -411,10 +382,21 @@ void NDB_SHARE::release_reference(NDB_SHARE *share, const char *reference) {
   share->refs_erase(reference);
   NDB_SHARE::free_share(&share);
 
-  mysql_mutex_unlock(&shares_mutex);
+  mysql_mutex_unlock(&ndbcluster_mutex);
 }
 
-#ifndef NDEBUG
+void NDB_SHARE::release_reference_have_lock(NDB_SHARE *share,
+                                            const char *reference) {
+  mysql_mutex_assert_owner(&ndbcluster_mutex);
+
+  DBUG_PRINT("NDB_SHARE", ("release '%s', reference: '%s', use_count: %u",
+                           share->key_string(), reference, share->use_count()));
+
+  share->refs_erase(reference);
+  NDB_SHARE::free_share(&share);
+}
+
+#ifndef DBUG_OFF
 
 bool NDB_SHARE::Ndb_share_references::check_empty() const {
   if (size() == 0) {
@@ -476,13 +458,13 @@ void NDB_SHARE::debug_print(std::string &out,
      << "  state: " << share_state_string() << ", " << line_separator
      << "  op: " << op << ", " << line_separator;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   std::string refs_string;
   refs->debug_print(refs_string, line_separator);
   ss << refs_string.c_str();
 
   // There should be as many refs as the use_count says
-  assert(use_count() == refs->size());
+  DBUG_ASSERT(use_count() == refs->size());
 #endif
 
   ss << "}";
@@ -492,10 +474,10 @@ void NDB_SHARE::debug_print(std::string &out,
 
 void NDB_SHARE::debug_print_shares(std::string &out) {
   std::stringstream ss;
-  ss << "ndb_shares {"
+  ss << "ndbcluster_open_tables {"
      << "\n";
 
-  for (const auto &key_and_value : *ndb_shares) {
+  for (const auto &key_and_value : *ndbcluster_open_tables) {
     const NDB_SHARE *share = key_and_value.second;
     std::string s;
     share->debug_print(s, "\n");
@@ -513,14 +495,14 @@ uint NDB_SHARE::decrement_use_count() {
   return --m_use_count;
 }
 
-void NDB_SHARE::print_remaining_open_shares() {
-  mysql_mutex_lock(&shares_mutex);
-  if (!ndb_shares->empty()) {
+void NDB_SHARE::print_remaining_open_tables(void) {
+  mysql_mutex_lock(&ndbcluster_mutex);
+  if (!ndbcluster_open_tables->empty()) {
     std::string s;
     NDB_SHARE::debug_print_shares(s);
     std::cerr << s << std::endl;
   }
-  mysql_mutex_unlock(&shares_mutex);
+  mysql_mutex_unlock(&ndbcluster_mutex);
 }
 
 int NDB_SHARE::rename_share(NDB_SHARE *share, NDB_SHARE_KEY *new_key) {
@@ -529,10 +511,11 @@ int NDB_SHARE::rename_share(NDB_SHARE *share, NDB_SHARE_KEY *new_key) {
   DBUG_PRINT("enter",
              ("new_key: '%s'", NDB_SHARE::key_get_key(new_key).c_str()));
 
-  mysql_mutex_lock(&shares_mutex);
+  mysql_mutex_lock(&ndbcluster_mutex);
 
   // Make sure that no NDB_SHARE with new_key already exists
-  if (find_or_nullptr(*ndb_shares, NDB_SHARE::key_get_key(new_key))) {
+  if (find_or_nullptr(*ndbcluster_open_tables,
+                      NDB_SHARE::key_get_key(new_key))) {
     // Dump the list of open NDB_SHARE's since new_key already exists
     ndb_log_error(
         "INTERNAL ERROR: Found existing NDB_SHARE for "
@@ -547,14 +530,15 @@ int NDB_SHARE::rename_share(NDB_SHARE *share, NDB_SHARE_KEY *new_key) {
   /* Update the share hash key. */
   NDB_SHARE_KEY *old_key = share->key;
   share->key = new_key;
-  ndb_shares->erase(NDB_SHARE::key_get_key(old_key));
-  ndb_shares->emplace(NDB_SHARE::key_get_key(new_key), share);
+  ndbcluster_open_tables->erase(NDB_SHARE::key_get_key(old_key));
+  ndbcluster_open_tables->emplace(NDB_SHARE::key_get_key(new_key), share);
 
   // Make sure that NDB_SHARE with old key does not exist
-  assert(find_or_nullptr(*ndb_shares, NDB_SHARE::key_get_key(old_key)) ==
-         nullptr);
+  DBUG_ASSERT(find_or_nullptr(*ndbcluster_open_tables,
+                              NDB_SHARE::key_get_key(old_key)) == nullptr);
   // Make sure that NDB_SHARE with new key does exist
-  assert(find_or_nullptr(*ndb_shares, NDB_SHARE::key_get_key(new_key)));
+  DBUG_ASSERT(find_or_nullptr(*ndbcluster_open_tables,
+                              NDB_SHARE::key_get_key(new_key)));
 
   DBUG_PRINT("info", ("setting db and table_name to point at new key"));
   share->db = NDB_SHARE::key_get_db_name(share->key);
@@ -562,7 +546,7 @@ int NDB_SHARE::rename_share(NDB_SHARE *share, NDB_SHARE_KEY *new_key) {
 
   if (share->op) {
     Ndb_event_data *event_data =
-        Ndb_event_data::get_event_data(share->op->getCustomData());
+        static_cast<Ndb_event_data *>(share->op->getCustomData());
     if (event_data && event_data->shadow_table) {
       if (!ndb_name_is_temp(share->table_name)) {
         DBUG_PRINT("info", ("Renaming shadow table"));
@@ -584,7 +568,7 @@ int NDB_SHARE::rename_share(NDB_SHARE *share, NDB_SHARE_KEY *new_key) {
       }
     }
   }
-  mysql_mutex_unlock(&shares_mutex);
+  mysql_mutex_unlock(&ndbcluster_mutex);
   return 0;
 }
 
@@ -605,10 +589,10 @@ NDB_SHARE *NDB_SHARE::acquire_reference_impl(const char *key) {
     return nullptr;
   }
 
-  mysql_mutex_assert_owner(&shares_mutex);
+  mysql_mutex_assert_owner(&ndbcluster_mutex);
 
-  auto it = ndb_shares->find(key);
-  if (it == ndb_shares->end()) {
+  auto it = ndbcluster_open_tables->find(key);
+  if (it == ndbcluster_open_tables->end()) {
     DBUG_PRINT("error", ("%s does not exist", key));
     return nullptr;
   }
@@ -622,13 +606,13 @@ NDB_SHARE *NDB_SHARE::acquire_reference_impl(const char *key) {
 }
 
 void NDB_SHARE::initialize(CHARSET_INFO *charset) {
-  mysql_mutex_init(PSI_INSTRUMENT_ME, &shares_mutex, MY_MUTEX_INIT_FAST);
-  ndb_shares.reset(new collation_unordered_map<std::string, NDB_SHARE *>(
-      charset, PSI_INSTRUMENT_ME));
+  ndbcluster_open_tables.reset(
+      new collation_unordered_map<std::string, NDB_SHARE *>(charset,
+                                                            PSI_INSTRUMENT_ME));
 }
 
-void NDB_SHARE::deinitialize() {
-  mysql_mutex_lock(&shares_mutex);
+void NDB_SHARE::deinitialize(void) {
+  mysql_mutex_lock(&ndbcluster_mutex);
 
   // There should not be any NDB_SHARE's left -> crash after logging in debug
   const class Debug_require {
@@ -636,19 +620,19 @@ void NDB_SHARE::deinitialize() {
 
    public:
     Debug_require(bool val) : m_required_val(val) {}
-    ~Debug_require() { assert(m_required_val); }
-  } shares_remaining(ndb_shares->empty() && dropped_shares.empty());
+    ~Debug_require() { DBUG_ASSERT(m_required_val); }
+  } shares_remaining(ndbcluster_open_tables->empty() && dropped_shares.empty());
 
   // Drop remaining open shares, drop one NDB_SHARE after the other
   // until open tables list is empty
-  while (!ndb_shares->empty()) {
-    NDB_SHARE *share = ndb_shares->begin()->second;
+  while (!ndbcluster_open_tables->empty()) {
+    NDB_SHARE *share = ndbcluster_open_tables->begin()->second;
     ndb_log_error("Still open NDB_SHARE '%s', use_count: %d, state: %s",
                   share->key_string(), share->use_count(),
                   share->share_state_string());
     // If last ref, share is destroyed immediately, else moved to list of
     // dropped shares
-    NDB_SHARE::mark_share_dropped_impl(&share);
+    NDB_SHARE::mark_share_dropped(&share);
   }
 
   // Release remaining dropped shares, release one NDB_SHARE after the other
@@ -661,30 +645,28 @@ void NDB_SHARE::deinitialize() {
     NDB_SHARE::real_free_share(&share);
   }
 
-  mysql_mutex_unlock(&shares_mutex);
-
-  mysql_mutex_destroy(&shares_mutex);
+  mysql_mutex_unlock(&ndbcluster_mutex);
 }
 
-void NDB_SHARE::release_extra_share_references() {
-  mysql_mutex_lock(&shares_mutex);
-  while (!ndb_shares->empty()) {
-    NDB_SHARE *share = ndb_shares->begin()->second;
+void NDB_SHARE::release_extra_share_references(void) {
+  mysql_mutex_lock(&ndbcluster_mutex);
+  while (!ndbcluster_open_tables->empty()) {
+    NDB_SHARE *share = ndbcluster_open_tables->begin()->second;
     /*
       The share kept by the server has not been freed, free it
       Will also take it out of _open_tables list
     */
-    assert(share->use_count() > 0);
-    assert(share->state != NSS_DROPPED);
-    NDB_SHARE::mark_share_dropped_impl(&share);
+    DBUG_ASSERT(share->use_count() > 0);
+    DBUG_ASSERT(share->state != NSS_DROPPED);
+    NDB_SHARE::mark_share_dropped(&share);
   }
-  mysql_mutex_unlock(&shares_mutex);
+  mysql_mutex_unlock(&ndbcluster_mutex);
 }
 
 void NDB_SHARE::real_free_share(NDB_SHARE **share_ptr) {
   NDB_SHARE *share = *share_ptr;
   DBUG_TRACE;
-  mysql_mutex_assert_owner(&shares_mutex);
+  mysql_mutex_assert_owner(&ndbcluster_mutex);
 
   // Share must already be marked as dropped
   ndbcluster::ndbrequire(share->state == NSS_DROPPED);
@@ -703,21 +685,21 @@ void NDB_SHARE::real_free_share(NDB_SHARE **share_ptr) {
 
 extern void ndb_index_stat_free(NDB_SHARE *);
 
-void NDB_SHARE::mark_share_dropped_impl(NDB_SHARE **share_ptr) {
+void NDB_SHARE::mark_share_dropped(NDB_SHARE **share_ptr) {
   NDB_SHARE *share = *share_ptr;
   DBUG_TRACE;
-  mysql_mutex_assert_owner(&shares_mutex);
+  mysql_mutex_assert_owner(&ndbcluster_mutex);
+
+  // The NDB_SHARE should not have any event operations, those
+  // should have been removed already _before_ marking the NDB_SHARE
+  // as dropped.
+  DBUG_ASSERT(share->op == nullptr);
 
   if (share->state == NSS_DROPPED) {
     // The NDB_SHARE was already marked as dropped
     return;
   }
 
-  // The NDB_SHARE should not have any event operation, it
-  // should have been removed already _before_ marking the NDB_SHARE
-  // as dropped.
-  assert(share->have_event_operation() == false);
-
   // The index_stat is not needed anymore, free it.
   ndb_index_stat_free(share);
 
@@ -725,13 +707,13 @@ void NDB_SHARE::mark_share_dropped_impl(NDB_SHARE **share_ptr) {
   share->state = NSS_DROPPED;
 
   // Remove share from list of open shares
-  ndbcluster::ndbrequire(ndb_shares->erase(share->key_string()));
+  ndbcluster::ndbrequire(ndbcluster_open_tables->erase(share->key_string()));
 
   // Remove reference from list of open shares and decrement use count
-  share->refs_erase("ndb_shares");
+  share->refs_erase("ndbcluster_open_tables");
   share->decrement_use_count();
 
-  // Destroy the NDB_SHARE if no one is using it, this is normally a special
+  // Destroy the NDB_SHARE if noone is using it, this is normally a special
   // case for shutdown code path. In all other cases the caller will hold
   // reference to the share.
   if (share->use_count() == 0) {
@@ -743,7 +725,7 @@ void NDB_SHARE::mark_share_dropped_impl(NDB_SHARE **share_ptr) {
   // to keep track of it until all references has been released
   dropped_shares.emplace(share);
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   std::string s;
   share->debug_print(s, "\n");
   std::cerr << "dropped_share: " << s << std::endl;
@@ -754,28 +736,10 @@ void NDB_SHARE::mark_share_dropped_impl(NDB_SHARE **share_ptr) {
   // NOTE! The refcount has not been incremented
 }
 
-void NDB_SHARE::mark_share_dropped_and_release(NDB_SHARE *share,
-                                               const char *reference) {
-  mysql_mutex_lock(&shares_mutex);
-  mark_share_dropped_impl(&share);
-
-  DBUG_PRINT("NDB_SHARE", ("release '%s', reference: '%s', use_count: %u",
-                           share->key_string(), reference, share->use_count()));
-
-  share->refs_erase(reference);
-  NDB_SHARE::free_share(&share);
-
-  // If this was the last share ref, it is now deleted. If there are more
-  // references, the share will remain in the list of dropped until
-  // remaining references are released.
-
-  mysql_mutex_unlock(&shares_mutex);
-}
-
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 void NDB_SHARE::dbg_check_shares_update() {
   ndb_log_info("dbug_check_shares open:");
-  for (const auto &key_and_value : *ndb_shares) {
+  for (const auto &key_and_value : *ndbcluster_open_tables) {
     const NDB_SHARE *share = key_and_value.second;
     ndb_log_info("  %s.%s: state: %s(%u) use_count: %u", share->db,
                  share->table_name, share->share_state_string(),
@@ -794,7 +758,7 @@ void NDB_SHARE::dbg_check_shares_update() {
   /**
    * Only shares in mysql database may be open...
    */
-  for (const auto &key_and_value : *ndb_shares) {
+  for (const auto &key_and_value : *ndbcluster_open_tables) {
     const NDB_SHARE *share = key_and_value.second;
     assert(strcmp(share->db, "mysql") == 0);
   }
@@ -807,26 +771,3 @@ void NDB_SHARE::dbg_check_shares_update() {
   }
 }
 #endif
-
-bool NDB_SHARE::install_event_op(NdbEventOperation *new_op, bool replace_op) {
-  // Require mutex to be held
-  mysql_mutex_assert_owner(&mutex);
-
-  // Only allow installing op with properly setup custom data
-  ndbcluster::ndbrequire(
-      Ndb_event_data::check_custom_data(new_op->getCustomData(), this));
-
-  // Only allow replacing op unless "replace_op" (which is used by inplace alter
-  // who will replace the old op with new in an atomic fashion)
-  ndbcluster::ndbrequire(op == nullptr || (op && replace_op));
-
-  // Don't allow installing op in other states than NSS_INITIAL. This will for
-  // example detect when a user thread tries to install event op on a share
-  // which has been "dropped" (by the binlog thread) during CREATE TABLE.
-  if (state != NSS_INITIAL) {
-    return false;
-  }
-
-  op = new_op;
-  return true;
-}
