@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2010, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,13 +32,13 @@
  *   by using a socketpair...
  * where theSocket is the receive part, and m_send_socket is the write part
  */
-class Loopback_Transporter : public TCP_Transporter {
+class Loopback_Transporter : public TCP_Transporter
+{
   friend class TransporterRegistry;
-
- private:
+private:
   // Initialize member variables
-  Loopback_Transporter(TransporterRegistry &,
-                       const TransporterConfiguration *conf);
+  Loopback_Transporter(TransporterRegistry&,
+                       const TransporterConfiguration* conf);
 
   // Disconnect, delete send buffers and receive buffer
   ~Loopback_Transporter() override;
@@ -58,16 +58,14 @@ class Loopback_Transporter : public TCP_Transporter {
    * @overload TCP_Transporter::disconnectImpl
    */
   void disconnectImpl() override;
+protected:
 
-  void releaseAfterDisconnect() override;
-
- protected:
- private:
+private:
   /**
    * m_send_socket is used to send
    * theSocket (in base class) is used for receive
    */
-  ndb_socket_t m_send_socket;
+  NDB_SOCKET_TYPE m_send_socket;
 
   /**
    * overloads TCP_Transporter::send_is_possible
@@ -75,4 +73,4 @@ class Loopback_Transporter : public TCP_Transporter {
   bool send_is_possible(int timeout_millisec) const override;
 };
 
-#endif  // Define of TCP_Transporter_H
+#endif // Define of TCP_Transporter_H
