@@ -1,24 +1,31 @@
 /*
-   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-/* -*- c-basic-offset: 4; -*- */
+#include <cstring>
+
 #include <ndb_global.h>
 #include <BaseString.hpp>
-#include "basestring_vsnprintf.h"
 
 BaseString::BaseString()
 {
@@ -216,7 +223,7 @@ BaseString::assfmt(const char *fmt, ...)
      * when called as vsnprintf(NULL, 0, ...).
      */
     va_start(ap, fmt);
-    l = basestring_vsnprintf(buf, sizeof(buf), fmt, ap) + 1;
+    l = std::vsnprintf(buf, sizeof(buf), fmt, ap) + 1;
     va_end(ap);
     if(l > (int)m_len) {
         char *t = new char[l];
@@ -229,7 +236,7 @@ BaseString::assfmt(const char *fmt, ...)
 	m_chr = t;
     }
     va_start(ap, fmt);
-    l = basestring_vsnprintf(m_chr, l, fmt, ap);
+    l = std::vsnprintf(m_chr, l, fmt, ap);
     assert(l == (int)strlen(m_chr));
     va_end(ap);
     m_len = (unsigned)strlen(m_chr);
@@ -248,7 +255,7 @@ BaseString::appfmt(const char *fmt, ...)
      * when called as vsnprintf(NULL, 0, ...).
      */
     va_start(ap, fmt);
-    l = basestring_vsnprintf(buf, sizeof(buf), fmt, ap) + 1;
+    l = std::vsnprintf(buf, sizeof(buf), fmt, ap) + 1;
     va_end(ap);
     char *tmp = new char[l];
     if (tmp == NULL)
@@ -257,7 +264,7 @@ BaseString::appfmt(const char *fmt, ...)
       return *this;
     }
     va_start(ap, fmt);
-    basestring_vsnprintf(tmp, l, fmt, ap);
+    std::vsnprintf(tmp, l, fmt, ap);
     va_end(ap);
     append(tmp);
     delete[] tmp;
@@ -296,49 +303,136 @@ BaseString::split(Vector<BaseString> &v,
     return num;
 }
 
-ssize_t
-BaseString::indexOf(char c, size_t pos) const {
+bool
+BaseString::splitKeyValue(BaseString& key, BaseString& value) const
+{
+  for (Uint32 i = 0; i < length(); i++)
+  {
+    if (m_chr[i] == '=')
+    {
+      if (i == 0)
+        key = BaseString();
+      else
+        key = BaseString(m_chr, i);
 
-  if (pos >= m_len)
-    return -1;
+      value = BaseString(m_chr + i + 1);
+      return true;
+    }
+  }
+  return false;
+}
 
-    char *p = strchr(m_chr + pos, c);
-    if(p == NULL)
-	return -1;
-    return (ssize_t)(p-m_chr);
+int
+BaseString::splitWithQuotedStrings(Vector<BaseString> &v,
+      const BaseString &separator,
+      int maxSize) const
+{
+  char *str = strdup(m_chr);
+  int i, start, len, num = 0;
+  len = (int)strlen(str);
+  const char* opening_quote = nullptr;
+
+  for(start = i = 0;
+      (i <= len) && ((maxSize < 0) || ((int)v.size() <= maxSize - 1));
+      i++)
+  {
+    if (str[i] != '\0')
+    {
+      const char* curr_quote = strchr("'\"", str[i]);
+      if (curr_quote != nullptr)
+      {
+        if (opening_quote == nullptr)
+        {
+          // Opening quote found, ignore separator till closing quote is found
+          opening_quote = curr_quote;
+        }
+        else if (*opening_quote ==  *curr_quote)
+        {
+          // Closing quote found, check for separator from now
+          opening_quote = nullptr;
+        }
+        continue;
+      }
+    }
+    if ((strchr(separator.c_str(), str[i]) && (opening_quote == nullptr)) ||
+        (i == len))
+    {
+      if ((maxSize < 0) || ((int)v.size() < (maxSize - 1)))
+        str[i] = '\0';
+      v.push_back(BaseString(str+start));
+      num++;
+      start = i+1;
+    }
+  }
+  free(str);
+
+  return num;
 }
 
 ssize_t
-BaseString::indexOf(const char * needle, size_t pos) const {
-
+BaseString::indexOf(char c, size_t pos) const
+{
   if (pos >= m_len)
     return -1;
 
-    char *p = strstr(m_chr + pos, needle);
-    if(p == NULL)
-	return -1;
-    return (ssize_t)(p-m_chr);
+  char *p = strchr(m_chr + pos, c);
+  if(p == NULL)
+    return -1;
+  return (ssize_t)(p-m_chr);
 }
 
 ssize_t
-BaseString::lastIndexOf(char c) const {
-    char *p;
-    p = strrchr(m_chr, c);
-    if(p == NULL)
-	return -1;
-    return (ssize_t)(p-m_chr);
+BaseString::indexOf(const char * needle, size_t pos) const
+{
+  if (pos >= m_len)
+    return -1;
+
+  char *p = strstr(m_chr + pos, needle);
+  if(p == NULL)
+    return -1;
+  return (ssize_t)(p-m_chr);
+}
+
+ssize_t
+BaseString::lastIndexOf(char c) const
+{
+  char *p;
+  p = strrchr(m_chr, c);
+  if(p == NULL)
+    return -1;
+  return (ssize_t)(p-m_chr);
+}
+
+bool
+BaseString::starts_with(const BaseString& str) const
+{
+  return std::strncmp(m_chr, str.m_chr, str.m_len) == 0;
+}
+
+bool
+BaseString::starts_with(const char* str) const
+{
+  const char* p = m_chr;
+  const char* q = str;
+  while (*q != 0 && *p != 0 && *p == *q)
+  {
+    p++;
+    q++;
+  }
+  return *q == 0;
 }
 
 BaseString
-BaseString::substr(ssize_t start, ssize_t stop) const {
-    if(stop < 0)
-	stop = length();
-    ssize_t len = stop-start;
-    if(len <= 0)
-	return BaseString("");
-    BaseString s;
-    s.assign(m_chr+start, len);
-    return s;
+BaseString::substr(ssize_t start, ssize_t stop) const
+{
+  if(stop < 0)
+    stop = length();
+  ssize_t len = stop-start;
+  if(len <= 0)
+    return BaseString("");
+  BaseString s;
+  s.assign(m_chr+start, len);
+  return s;
 }
 
 static bool
@@ -503,7 +597,7 @@ BaseString::trim(char * str, const char * delim){
 int
 BaseString::vsnprintf(char *str, size_t size, const char *format, va_list ap)
 {
-  return(basestring_vsnprintf(str, size, format, ap));
+  return(std::vsnprintf(str, size, format, ap));
 }
 
 int
@@ -511,7 +605,7 @@ BaseString::snprintf(char *str, size_t size, const char *format, ...)
 {
   va_list ap;
   va_start(ap, format);
-  int ret= basestring_vsnprintf(str, size, format, ap);
+  int ret= std::vsnprintf(str, size, format, ap);
   va_end(ap);
   return(ret);
 }
@@ -617,7 +711,7 @@ BaseString::hexdump(char * buf, size_t len, const Uint32 * wordbuf, size_t numwo
   return offset;
 }
 
-#ifdef TEST_BASE_STRING
+#ifdef TEST_BASESTRING
 
 #include <NdbTap.hpp>
 
@@ -678,6 +772,29 @@ TAPTEST(BaseString)
 	OK(BaseString("hej").indexOf('X') == -1);
     }
 
+    {
+      BaseString base("123abcdef");
+      BaseString sub("123abc");
+      OK(base.starts_with(sub) == true);
+
+      BaseString base1("123abc");
+      BaseString sub1("123abcdef");
+      OK(base1.starts_with(sub1) == false)
+
+      BaseString base2("123abcdef");
+      BaseString sub2("");
+      OK(base2.starts_with(sub2) == true);
+
+      BaseString base3("");
+      BaseString sub3("123abcdef");
+      OK(base3.starts_with(sub3) == false);
+
+      OK(base.starts_with("123abc") == true);
+      OK(base1.starts_with("123abcdef") == false);
+      OK(base2.starts_with("") == true);
+      OK(base3.starts_with("123abcdef") == false);
+    }
+
     {
 	OK(BaseString(" 1").trim(" ") == "1");
 	OK(BaseString("1 ").trim(" ") == "1");
@@ -715,6 +832,31 @@ TAPTEST(BaseString)
       BIG_ASSFMT_OK(20*1024*1024);
     }
 
+    {
+      printf("Testing splitWithQuotedStrings\n");
+      Vector<BaseString> v;
+
+      BaseString("key=value").splitWithQuotedStrings(v, "=");
+      OK(v[0] == "key");
+      v.clear();
+
+      BaseString("abcdef=\"ghi\"").splitWithQuotedStrings(v, "=");
+      OK(v[0] == "abcdef");
+      v.clear();
+
+      BaseString("abc=\"de=f\"").splitWithQuotedStrings(v, "=");
+      OK(v[1] == "\"de=f\"");
+      v.clear();
+
+      BaseString("abc=\"\"de=f\"\"").splitWithQuotedStrings(v, "=");
+      OK(v[1] == "\"\"de");
+      v.clear();
+
+      BaseString("abc=\"\'de=f\'\"").splitWithQuotedStrings(v, "=");
+      OK(v[1] == "\"\'de=f\'\"");
+      v.clear();
+    }
+
     return 1; // OK
 }
 
