@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2004, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -56,11 +56,10 @@ SocketClient::init()
   if (ndb_socket_valid(m_sockfd))
     ndb_socket_close(m_sockfd);
 
-  m_sockfd= ndb_socket_create(AF_INET, SOCK_STREAM, 0);
+  m_sockfd= ndb_socket_create_dual_stack(SOCK_STREAM, 0);
   if (!ndb_socket_valid(m_sockfd)) {
     return false;
   }
-
   DBUG_PRINT("info",("NDB_SOCKET: " MY_SOCKET_FORMAT,
                      MY_SOCKET_FORMAT_VALUE(m_sockfd)));
 
@@ -74,20 +73,20 @@ SocketClient::bind(const char* local_hostname,
   if (!ndb_socket_valid(m_sockfd))
     return -1;
 
-  struct sockaddr_in local;
+  struct sockaddr_in6 local;
   memset(&local, 0, sizeof(local));
-  local.sin_family = AF_INET;
-  local.sin_port = htons(local_port);
+  local.sin6_family = AF_INET6;
+  local.sin6_port = htons(local_port);
   if (local_port == 0 &&
       m_last_used_port != 0)
   {
     // Try to bind to the same port as last successful connect instead of
     // any ephemeral port. Intention is to reuse any previous TIME_WAIT TCB
-    local.sin_port = htons(m_last_used_port);
+    local.sin6_port = htons(m_last_used_port);
   }
 
   // Resolve local address
-  if (Ndb_getInAddr(&local.sin_addr, local_hostname))
+  if (Ndb_getInAddr6(&local.sin6_addr, local_hostname))
   {
     return errno ? errno : EINVAL;
   }
@@ -108,7 +107,7 @@ SocketClient::bind(const char* local_hostname,
       // Faild to bind same port as last, retry with any
       // ephemeral port(as originally requested)
       m_last_used_port = 0; // Reset last used port
-      local.sin_port = htons(0); // Try bind with any port
+      local.sin6_port = htons(0); // Try bind with any port
       continue;
     }
 
@@ -127,6 +126,7 @@ SocketClient::bind(const char* local_hostname,
 #define NONBLOCKERR(E) (E!=EINPROGRESS)
 #endif
 
+
 NDB_SOCKET_TYPE
 SocketClient::connect(const char* server_hostname,
                       unsigned short server_port)
@@ -143,13 +143,13 @@ SocketClient::connect(const char* server_hostname,
     }
   }
 
-  struct sockaddr_in server_addr;
+  struct sockaddr_in6 server_addr;
   memset(&server_addr, 0, sizeof(server_addr));
-  server_addr.sin_family = AF_INET;
-  server_addr.sin_port = htons(server_port);
+  server_addr.sin6_family = AF_INET6;
+  server_addr.sin6_port = htons(server_port);
 
   // Resolve server address
-  if (Ndb_getInAddr(&server_addr.sin_addr, server_hostname))
+  if (Ndb_getInAddr6(&server_addr.sin6_addr, server_hostname))
   {
     DEBUG_FPRINTF((stderr, "Failed Ndb_getInAddr in connect\n"));
     ndb_socket_close(m_sockfd);
@@ -169,7 +169,7 @@ SocketClient::connect(const char* server_hostname,
   // Start non blocking connect
   DEBUG_FPRINTF((stderr, "Connect to %s:%u\n",
                          server_hostname, server_port));
-  int r = ndb_connect_inet(m_sockfd, &server_addr);
+  int r = ndb_connect_inet6(m_sockfd, &server_addr);
   if (r == 0)
     goto done; // connected immediately.
 
