@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,7 +28,6 @@
 #include <InputStream.hpp>
 #include <Parser.hpp>
 #include <SocketServer.hpp>
-#include "util/NdbSocket.h"
 
 class CPCD;
 
@@ -36,7 +35,6 @@ class CPCDAPISession : public SocketServer::Session {
   typedef Parser<CPCDAPISession> Parser_t;
 
   class CPCD &m_cpcd;
-  NdbSocket m_secure_socket;
   InputStream *m_input;
   OutputStream *m_output;
   Parser_t *m_parser;
@@ -48,7 +46,7 @@ class CPCDAPISession : public SocketServer::Session {
   void printLongString(const char *key, const char *value);
 
  public:
-  CPCDAPISession(ndb_socket_t, class CPCD &);
+  CPCDAPISession(NDB_SOCKET_TYPE, class CPCD &);
   CPCDAPISession(FILE *f, CPCD &cpcd);
   ~CPCDAPISession() override;
 
@@ -76,7 +74,7 @@ class CPCDAPIService : public SocketServer::Service {
  public:
   CPCDAPIService(class CPCD &cpcd) : m_cpcd(cpcd) {}
 
-  CPCDAPISession *newSession(ndb_socket_t theSock) override {
+  CPCDAPISession *newSession(NDB_SOCKET_TYPE theSock) override {
     return new CPCDAPISession(theSock, m_cpcd);
   }
 };
