@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,78 +23,83 @@
 */
 
 #include "SimBlockList.hpp"
-#include <NdbEnv.h>
-#include <Backup.hpp>
-#include <BackupProxy.hpp>
+#include <Emulator.hpp>
+#include <SimulatedBlock.hpp>
 #include <Cmvmi.hpp>
-#include <DbUtil.hpp>
+#include <Ndbfs.hpp>
 #include <Dbacc.hpp>
-#include <DbaccProxy.hpp>
 #include <Dbdict.hpp>
 #include <Dbdih.hpp>
-#include <Dbinfo.hpp>
 #include <Dblqh.hpp>
-#include <DblqhProxy.hpp>
-#include <Dbqacc.hpp>
-#include <DbqaccProxy.hpp>
-#include <Dbqlqh.hpp>
-#include <DbqlqhProxy.hpp>
-#include <Dbqtup.hpp>
-#include <DbqtupProxy.hpp>
-#include <Dbqtux.hpp>
-#include <DbqtuxProxy.hpp>
 #include <Dbspj.hpp>
-#include <DbspjProxy.hpp>
 #include <Dbtc.hpp>
-#include <DbtcProxy.hpp>
 #include <Dbtup.hpp>
-#include <DbtupProxy.hpp>
-#include <Dbtux.hpp>
-#include <DbtuxProxy.hpp>
-#include <Emulator.hpp>
-#include <LocalProxy.hpp>
 #include <Ndbcntr.hpp>
-#include <Ndbfs.hpp>
-#include <PgmanProxy.hpp>
-#include <QBackup.hpp>
-#include <QBackupProxy.hpp>
-#include <QRestore.hpp>
-#include <QRestoreProxy.hpp>
 #include <Qmgr.hpp>
-#include <RestoreProxy.hpp>
-#include <SimulatedBlock.hpp>
-#include <Suma.hpp>
 #include <Trix.hpp>
+#include <Backup.hpp>
+#include <DbUtil.hpp>
+#include <Suma.hpp>
+#include <Dbtux.hpp>
+#include <tsman.hpp>
 #include <lgman.hpp>
-#include <mt.hpp>
 #include <pgman.hpp>
 #include <restore.hpp>
+#include <Dbinfo.hpp>
+#include <NdbEnv.h>
+#include <LocalProxy.hpp>
+#include <DblqhProxy.hpp>
+#include <DbspjProxy.hpp>
+#include <DbaccProxy.hpp>
+#include <DbtupProxy.hpp>
+#include <DbtuxProxy.hpp>
+#include <BackupProxy.hpp>
+#include <RestoreProxy.hpp>
+#include <PgmanProxy.hpp>
+#include <DbtcProxy.hpp>
+#include <DbspjProxy.hpp>
 #include <thrman.hpp>
 #include <trpman.hpp>
-#include <tsman.hpp>
+#include <Dbqlqh.hpp>
+#include <Dbqacc.hpp>
+#include <Dbqtup.hpp>
+#include <Dbqtux.hpp>
+#include <QBackup.hpp>
+#include <QRestore.hpp>
+#include <DbqlqhProxy.hpp>
+#include <DbqaccProxy.hpp>
+#include <DbqtupProxy.hpp>
+#include <DbqtuxProxy.hpp>
+#include <QBackupProxy.hpp>
+#include <QRestoreProxy.hpp>
+#include <mt.hpp>
 #include "portlib/NdbMem.h"
 
 #define JAM_FILE_ID 492
 
+
 #define NEW_BLOCK(B) new B
 
-void SimBlockList::load(EmulatorData &data) {
+void
+SimBlockList::load(EmulatorData& data){
   noOfBlocks = NO_OF_BLOCKS;
-  theList = new SimulatedBlock *[noOfBlocks];
-  if (!theList) {
-    ERROR_SET(fatal, NDBD_EXIT_MEMALLOC, "Failed to create the block list", "");
+  theList = new SimulatedBlock * [noOfBlocks];
+  if (!theList)
+  {
+    ERROR_SET(fatal, NDBD_EXIT_MEMALLOC,
+              "Failed to create the block list", "");
   }
 
   Block_context ctx(*data.theConfiguration, *data.m_mem_manager);
-
-  SimulatedBlock *fs = 0;
+  
+  SimulatedBlock * fs = 0;
   {
     Uint32 dl;
-    const ndb_mgm_configuration_iterator *p =
-        ctx.m_config.getOwnConfigIterator();
-    if (p && !ndb_mgm_get_int_parameter(p, CFG_DB_DISCLESS, &dl) && dl) {
+    const ndb_mgm_configuration_iterator * p = 
+      ctx.m_config.getOwnConfigIterator();
+    if(p && !ndb_mgm_get_int_parameter(p, CFG_DB_DISCLESS, &dl) && dl){
       fs = NEW_BLOCK(VoidFs)(ctx);
-    } else {
+    } else { 
       fs = NEW_BLOCK(Ndbfs)(ctx);
     }
   }
@@ -106,22 +110,22 @@ void SimBlockList::load(EmulatorData &data) {
     theList[0] = NEW_BLOCK(Pgman)(ctx);
   else
     theList[0] = NEW_BLOCK(PgmanProxy)(ctx);
-  theList[1] = NEW_BLOCK(Lgman)(ctx);
-  theList[2] = NEW_BLOCK(Tsman)(ctx);
+  theList[1]  = NEW_BLOCK(Lgman)(ctx);
+  theList[2]  = NEW_BLOCK(Tsman)(ctx);
   if (!mtLqh)
-    theList[3] = NEW_BLOCK(Dbacc)(ctx);
+    theList[3]  = NEW_BLOCK(Dbacc)(ctx);
   else
-    theList[3] = NEW_BLOCK(DbaccProxy)(ctx);
-  theList[4] = NEW_BLOCK(Cmvmi)(ctx);
-  theList[5] = fs;
-  theList[6] = NEW_BLOCK(Dbdict)(ctx);
-  theList[7] = NEW_BLOCK(Dbdih)(ctx);
+    theList[3]  = NEW_BLOCK(DbaccProxy)(ctx);
+  theList[4]  = NEW_BLOCK(Cmvmi)(ctx);
+  theList[5]  = fs;
+  theList[6]  = NEW_BLOCK(Dbdict)(ctx);
+  theList[7]  = NEW_BLOCK(Dbdih)(ctx);
   if (!mtLqh)
-    theList[8] = NEW_BLOCK(Dblqh)(ctx);
+    theList[8]  = NEW_BLOCK(Dblqh)(ctx);
   else
-    theList[8] = NEW_BLOCK(DblqhProxy)(ctx);
+    theList[8]  = NEW_BLOCK(DblqhProxy)(ctx);
   if (globalData.ndbMtTcWorkers == 0)
-    theList[9] = NEW_BLOCK(Dbtc)(ctx);
+    theList[9]  = NEW_BLOCK(Dbtc)(ctx);
   else
     theList[9] = NEW_BLOCK(DbtcProxy)(ctx);
   if (!mtLqh)
@@ -147,9 +151,9 @@ void SimBlockList::load(EmulatorData &data) {
     theList[18] = NEW_BLOCK(RestoreProxy)(ctx);
   theList[19] = NEW_BLOCK(Dbinfo)(ctx);
   if (globalData.ndbMtTcWorkers == 0)
-    theList[20] = NEW_BLOCK(Dbspj)(ctx);
+    theList[20]  = NEW_BLOCK(Dbspj)(ctx);
   else
-    theList[20] = NEW_BLOCK(DbspjProxy)(ctx);
+    theList[20]  = NEW_BLOCK(DbspjProxy)(ctx);
   if (NdbIsMultiThreaded() == false)
     theList[21] = NEW_BLOCK(Thrman)(ctx);
   else
@@ -169,13 +173,17 @@ void SimBlockList::load(EmulatorData &data) {
   assert(NO_OF_BLOCKS == 29);
 
   // Check that all blocks could be created
-  for (int i = 0; i < noOfBlocks; i++) {
-    if (!theList[i]) {
-      ERROR_SET(fatal, NDBD_EXIT_MEMALLOC, "Failed to create block", "");
+  for (int i = 0; i < noOfBlocks; i++)
+  {
+    if (!theList[i])
+    {
+      ERROR_SET(fatal, NDBD_EXIT_MEMALLOC,
+                "Failed to create block", "");
     }
   }
 
-  if (globalData.isNdbMt) {
+  if (globalData.isNdbMt)
+  {
     /**
       This is where we bind blocks to their respective threads.
       mt_init_thr_map binds the blocks to the two main threads,
@@ -189,46 +197,61 @@ void SimBlockList::load(EmulatorData &data) {
       instances.
     */
     mt_init_thr_map();
-    for (int i = 0; i < noOfBlocks; i++) {
-      if (theList[i]) theList[i]->loadWorkers();
+    for (int i = 0; i < noOfBlocks; i++)
+    {
+      if (theList[i])
+        theList[i]->loadWorkers();
     }
     mt_finalize_thr_map();
   }
 }
 
-void SimBlockList::unload() {
-  if (theList != 0) {
-    for (int i = 0; i < noOfBlocks; i++) {
-      if (theList[i] != 0) {
+void
+SimBlockList::unload(){
+  if(theList != 0){
+    for(int i = 0; i<noOfBlocks; i++){
+      if(theList[i] != 0){
 #ifdef VM_TRACE
-        theList[i]->~SimulatedBlock();
-        free(theList[i]);
+	theList[i]->~SimulatedBlock();
+	free(theList[i]);
 #else
-        delete (theList[i]);
+        delete(theList[i]);
 #endif
-        theList[i] = 0;
+	theList[i] = 0;
       }
     }
-    delete[] theList;
-    theList = 0;
+    delete [] theList;
+    theList    = 0;
     noOfBlocks = 0;
   }
 }
 
 Uint64 SimBlockList::getTransactionMemoryNeed(
-    const Uint32 dbtc_instance_count, const Uint32 ldm_instance_count,
-    const ndb_mgm_configuration_iterator *mgm_cfg,
-    const bool use_reserved) const {
-  Uint64 byte_count = Dbtc::getTransactionMemoryNeed(dbtc_instance_count,
-                                                     mgm_cfg, use_reserved);
-  byte_count += Dbacc::getTransactionMemoryNeed(ldm_instance_count, mgm_cfg,
-                                                use_reserved);
-  byte_count += Dblqh::getTransactionMemoryNeed(ldm_instance_count, mgm_cfg,
-                                                use_reserved);
-  byte_count += Dbtup::getTransactionMemoryNeed(ldm_instance_count, mgm_cfg,
-                                                use_reserved);
-  byte_count += Dbtux::getTransactionMemoryNeed(ldm_instance_count, mgm_cfg,
-                                                use_reserved);
+  const Uint32 dbtc_instance_count,
+  const Uint32 ldm_instance_count,
+  const ndb_mgm_configuration_iterator * mgm_cfg,
+  const bool use_reserved) const
+{
+  Uint64 byte_count = Dbtc::getTransactionMemoryNeed(
+    dbtc_instance_count,
+    mgm_cfg,
+    use_reserved);
+  byte_count += Dbacc::getTransactionMemoryNeed(
+    ldm_instance_count,
+    mgm_cfg,
+    use_reserved);
+  byte_count += Dblqh::getTransactionMemoryNeed(
+    ldm_instance_count,
+    mgm_cfg,
+    use_reserved);
+  byte_count += Dbtup::getTransactionMemoryNeed(
+    ldm_instance_count,
+    mgm_cfg,
+    use_reserved);
+  byte_count += Dbtux::getTransactionMemoryNeed(
+    ldm_instance_count,
+    mgm_cfg,
+    use_reserved);
 
   byte_count += Dbqacc::getTransactionMemoryNeed();
   byte_count += Dbqlqh::getTransactionMemoryNeed();
@@ -236,3 +259,4 @@ Uint64 SimBlockList::getTransactionMemoryNeed(
   byte_count += Dbqtux::getTransactionMemoryNeed();
   return byte_count;
 }
+
