@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2019, 2020 Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -31,14 +31,16 @@
 
 #define JAM_FILE_ID 537
 
-class QRestore : public Restore {
+class QRestore : public Restore
+{
   friend class QRestoreProxy;
 
- public:
-  QRestore(Block_context &ctx, Uint32 instanceNumber = 0);
+public:
+  QRestore(Block_context& ctx,
+           Uint32 instanceNumber = 0);
   virtual ~QRestore();
 
- private:
+private:
   BLOCK_DEFINES(QRestore);
 };
 
