@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2019, 2020 Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,10 +26,16 @@
 #include "QBackup.hpp"
 
 #include <EventLogger.hpp>
+extern EventLogger * g_eventLogger;
 
 #define JAM_FILE_ID 534
 
-QBackup::QBackup(Block_context &ctx, Uint32 instanceNumber)
-    : Backup(ctx, instanceNumber, QBACKUP) {}
+QBackup::QBackup(Block_context& ctx,
+                 Uint32 instanceNumber):
+  Backup(ctx, instanceNumber, QBACKUP)
+{
+}
 
-QBackup::~QBackup() {}
+QBackup::~QBackup()
+{
+}
