@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,7 +29,6 @@
 #define ACC_SAFE_QUEUE
 #endif
 
-#include "util/require.h"
 #include <pc.hpp>
 #include "Bitmask.hpp"
 #include <DynArr256.hpp>
@@ -42,6 +41,7 @@
 #include "TransientSlotPool.hpp"
 
 #include <EventLogger.hpp>
+extern EventLogger * g_eventLogger;
 
 #define JAM_FILE_ID 344
 
@@ -121,7 +121,7 @@
 #define ZDIR_RANGE_FULL_ERROR 633 // on fragment
 
 #define ZLOCAL_KEY_LENGTH_ERROR 634 // From Dbdict via Dblqh
-#define ZNOWAIT_ERROR 635 // Can't lock immediately, and nowait set
+#define ZNOWAIT_ERROR 635 // Cant lock immediately, and nowait set
 
 #endif
 
@@ -180,7 +180,7 @@ Uint16
 ElementHeader::getPageIdx(Uint32 data)
 {
   /* Bits 1-13 is reserved for page index */
-  static_assert(MAX_TUPLES_BITS <= 13);
+  NDB_STATIC_ASSERT(MAX_TUPLES_BITS <= 13);
   return (data >> 1) & MAX_TUPLES_PER_PAGE;
 }
 
@@ -421,9 +421,9 @@ struct Fragmentrec {
   Uint32 expSenderPageptr;
 
 //-----------------------------------------------------------------------------
-// Number of locks held on fragment, only for self-check
+// List of lock owners currently used only for self-check
 //-----------------------------------------------------------------------------
-  Uint32 lockCount;
+  Uint32 lockOwnersList;
 
 //-----------------------------------------------------------------------------
 // References to Directory Ranges (which in turn references directories, which
@@ -490,7 +490,7 @@ struct Fragmentrec {
 // elementLength: Length of element in bucket and overflow pages
 // keyLength: Length of key
 //-----------------------------------------------------------------------------
-  static constexpr Uint32 elementLength = 2;
+  STATIC_CONST( elementLength = 2 );
   Uint16 keyLength;
 
 //-----------------------------------------------------------------------------
@@ -502,9 +502,9 @@ struct Fragmentrec {
 // hashcheckbit is the bit to check whether to send element to split bucket or not
 // k (== 6) is the number of buckets per page
 //-----------------------------------------------------------------------------
-  static constexpr Uint32 k = 6;
-  static constexpr Uint32 MIN_HASH_COMPARE_BITS = 7;
-  static constexpr Uint32 MAX_HASH_VALUE_BITS = 31;
+  STATIC_CONST( k = 6 );
+  STATIC_CONST( MIN_HASH_COMPARE_BITS = 7 );
+  STATIC_CONST( MAX_HASH_VALUE_BITS = 31 );
 
 //-----------------------------------------------------------------------------
 // nodetype can only be STORED in this release. Is currently only set, never read
@@ -687,7 +687,7 @@ public:
 
 
 struct Operationrec {
-  static constexpr Uint32 TYPE_ID = RT_DBACC_OPERATION;
+  STATIC_CONST( TYPE_ID = RT_DBACC_OPERATION);
   Uint32 m_magic;
 
   enum OpBits {
@@ -696,7 +696,7 @@ struct Operationrec {
     ,OP_ACC_LOCK_MODE       = 0x00020 // Or:de lock mode of all operation
                                       // before me
     ,OP_LOCK_OWNER          = 0x00040
-    ,OP_RUN_QUEUE           = 0x00080 // In parallel queue of lock owner
+    ,OP_RUN_QUEUE           = 0x00080 // In parallell queue of lock owner
     ,OP_DIRTY_READ          = 0x00100
     ,OP_LOCK_REQ            = 0x00200 // isAccLockReq
     ,OP_COMMIT_DELETE_CHECK = 0x00400
@@ -742,12 +742,14 @@ struct Operationrec {
   Uint32 fid;
   Uint32 fragptr;
   LHBits32 hashValue;
+  Uint32 nextLockOwnerOp;
   Uint32 nextParallelQue;
   union {
     Uint32 nextSerialQue;      
     Uint32 m_lock_owner_ptr_i; // if nextParallelQue = RNIL, else undefined
   };
   Uint32 prevOp;
+  Uint32 prevLockOwnerOp;
   union {
     Uint32 prevParallelQue;
     Uint32 m_lo_last_parallel_op_ptr_i;
@@ -776,14 +778,14 @@ struct Operationrec {
 
   typedef Ptr<Operationrec> OperationrecPtr;
   typedef TransientPool<Operationrec> Operationrec_pool;
-  static constexpr Uint32 DBACC_OPERATION_RECORD_TRANSIENT_POOL_INDEX = 1;
+  STATIC_CONST(DBACC_OPERATION_RECORD_TRANSIENT_POOL_INDEX = 1);
   Operationrec_pool oprec_pool;
   OperationrecPtr operationRecPtr;
   OperationrecPtr queOperPtr;
   Uint32 cfreeopRec;
 
 struct ScanRec {
-  static constexpr Uint32 TYPE_ID = RT_DBACC_SCAN;
+  STATIC_CONST( TYPE_ID = RT_DBACC_SCAN );
   Uint32 m_magic;
 
   enum ScanState {
@@ -859,7 +861,7 @@ public:
 };
   typedef Ptr<ScanRec> ScanRecPtr;
   typedef TransientPool<ScanRec> ScanRec_pool;
-  static constexpr Uint32 DBACC_SCAN_RECORD_TRANSIENT_POOL_INDEX = 0;
+  STATIC_CONST(DBACC_SCAN_RECORD_TRANSIENT_POOL_INDEX = 0);
   ScanRec_pool scanRec_pool;
   ScanRecPtr scanPtr;
 
@@ -892,8 +894,6 @@ public:
 
   bool checkOpPendingAbort(Uint32 accConnectPtr) const;
 
-  bool getPrecedingOperation(OperationrecPtr& opPtr) const;
-
 private:
   BLOCK_DEFINES(Dbacc);
 
@@ -938,6 +938,7 @@ private:
   void initFragAdd(Signal*, FragmentrecPtr) const;
   void initFragPageZero(FragmentrecPtr, Page8Ptr) const;
   void initFragGeneral(FragmentrecPtr) const;
+  void verifyFragCorrect(FragmentrecPtr regFragPtr) const;
   void releaseFragResources(Signal* signal, Uint32 fragIndex);
   void releaseRootFragRecord(Signal* signal, RootfragmentrecPtr rootPtr) const;
   void releaseRootFragResources(Signal* signal, Uint32 tableId);
@@ -967,8 +968,7 @@ private:
 #endif
 #ifdef ACC_SAFE_QUEUE
   bool validate_lock_queue(OperationrecPtr opPtr) const;
-  bool validate_parallel_queue(OperationrecPtr opPtr,
-                               Uint32 ownerPtrI) const;
+  Uint32 get_parallel_head(OperationrecPtr opPtr) const;
   void dump_lock_queue(OperationrecPtr loPtr) const;
 #else
   bool validate_lock_queue(OperationrecPtr) const { return true;}
@@ -1109,6 +1109,8 @@ private:
 			  OperationrecPtr release_op) const;
   Uint32 allocOverflowPage();
   bool getfragmentrec(FragmentrecPtr&, Uint32 fragId);
+  void insertLockOwnersList(const OperationrecPtr&) const;
+  void takeOutLockOwnersList(const OperationrecPtr&) const;
 
   void initFsOpRec(Signal* signal) const;
   void initOverpage(Page8Ptr);
@@ -1544,7 +1546,7 @@ inline bool Dbacc::ScanRec::isScanned(Uint32 elemptr) const
    * number of available bits in elemScanned to get an unique bit index for
    * each element.
    */
-  static_assert(ZBUF_SIZE <= ELEM_SCANNED_BITS);
+  NDB_STATIC_ASSERT(ZBUF_SIZE <= ELEM_SCANNED_BITS);
   return (elemScanned >> (elemptr % ELEM_SCANNED_BITS)) & 1;
 }
 
