@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2019, 2020 Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,15 +26,21 @@
 #include "Dbqacc.hpp"
 
 #include <EventLogger.hpp>
+extern EventLogger * g_eventLogger;
 
 #define JAM_FILE_ID 522
 
-Dbqacc::Dbqacc(Block_context &ctx, Uint32 instanceNumber)
-    : Dbacc(ctx, instanceNumber, DBQACC) {}
+Dbqacc::Dbqacc(Block_context& ctx,
+               Uint32 instanceNumber):
+  Dbacc(ctx, instanceNumber, DBQACC)
+{
+}
 
-Uint64 Dbqacc::getTransactionMemoryNeed() {
+Uint64 Dbqacc::getTransactionMemoryNeed()
+{
   Uint32 query_instance_count =
-      globalData.ndbMtQueryThreads + globalData.ndbMtRecoverThreads;
+    globalData.ndbMtQueryThreads +
+    globalData.ndbMtRecoverThreads;
   Uint32 acc_scan_recs = 1;
   Uint32 acc_op_recs = 1;
 
@@ -49,4 +54,7 @@ Uint64 Dbqacc::getTransactionMemoryNeed() {
   return (scan_byte_count + op_byte_count);
 }
 
-Dbqacc::~Dbqacc() {}
+
+Dbqacc::~Dbqacc()
+{
+}
