@@ -1,4 +1,4 @@
-/* Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2019, 2020 Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,10 +25,17 @@
 
 #define JAM_FILE_ID 520
 
-DbqlqhProxy::DbqlqhProxy(Block_context &ctx) : LocalProxy(DBQLQH, ctx) {}
+DbqlqhProxy::DbqlqhProxy(Block_context& ctx) :
+  LocalProxy(DBQLQH, ctx)
+{
+}
 
-DbqlqhProxy::~DbqlqhProxy() {}
+DbqlqhProxy::~DbqlqhProxy()
+{
+}
 
-SimulatedBlock *DbqlqhProxy::newWorker(Uint32 instanceNo) {
+SimulatedBlock*
+DbqlqhProxy::newWorker(Uint32 instanceNo)
+{
   return new Dbqlqh(m_ctx, instanceNo);
 }
