@@ -1,4 +1,4 @@
-/* Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2019, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,14 +27,15 @@
 
 #define JAM_FILE_ID 519
 
-class DbqlqhProxy : public LocalProxy {
- public:
-  DbqlqhProxy(Block_context &ctx);
+class DbqlqhProxy : public LocalProxy
+{
+public:
+  DbqlqhProxy(Block_context& ctx);
   ~DbqlqhProxy() override;
   BLOCK_DEFINES(DbqlqhProxy);
 
- protected:
-  SimulatedBlock *newWorker(Uint32 instanceNo) override;
+protected:
+  SimulatedBlock* newWorker(Uint32 instanceNo) override;
 };
 
 #undef JAM_FILE_ID
