@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2022, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,7 +23,6 @@
 */
 
 #define DBTC_C
-#include "util/require.h"
 #include "Dbtc.hpp"
 #include "ndb_global.h"
 #include <pc.hpp>
@@ -35,6 +34,7 @@
 
 #define JAM_FILE_ID 349
 
+extern EventLogger * g_eventLogger;
 
 #if defined(VM_TRACE) || defined(ERROR_INSERT)
 #define DEBUG_MEM
@@ -711,7 +711,7 @@ Dbtc::Dbtc(Block_context& ctx, Uint32 instanceNo):
     &scanRecordPool;
   c_transient_pools[DBTC_COMMIT_ACK_MARKER_TRANSIENT_POOL_INDEX] =
     &m_commitAckMarkerPool;
-  static_assert(c_transient_pool_count == 13);
+  NDB_STATIC_ASSERT(c_transient_pool_count == 13);
   c_transient_pools_shrinking.clear();
 }//Dbtc::Dbtc()
 
