@@ -1,16 +1,15 @@
-/* Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2019, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,14 +27,15 @@
 
 #define JAM_FILE_ID 527
 
-class DbqtupProxy : public LocalProxy {
- public:
-  DbqtupProxy(Block_context &ctx);
+class DbqtupProxy : public LocalProxy
+{
+public:
+  DbqtupProxy(Block_context& ctx);
   ~DbqtupProxy() override;
   BLOCK_DEFINES(DbqtupProxy);
 
- protected:
-  SimulatedBlock *newWorker(Uint32 instanceNo) override;
+protected:
+  SimulatedBlock* newWorker(Uint32 instanceNo) override;
 };
 
 #undef JAM_FILE_ID
