@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,11 +30,11 @@
 #include <pc.hpp>
 #include <signaldata/TupCommit.hpp>
 #include <EventLogger.hpp>
-#include "../backup/Backup.hpp"
 #include "../dblqh/Dblqh.hpp"
 
 #define JAM_FILE_ID 416
 
+extern EventLogger *g_eventLogger;
 
 #if (defined(VM_TRACE) || defined(ERROR_INSERT))
 //#define DEBUG_DISK 1
@@ -622,7 +622,7 @@ static void dump_buf_hex(unsigned char *p, Uint32 bytes)
     }
     sprintf(q+3*i, " %02X", p[i]);
   }
-  g_eventLogger->info("%8p: %s", p, buf);
+  ndbout_c("%8p: %s", p, buf);
 }
 #endif
 
@@ -662,7 +662,7 @@ static void dump_buf_hex(unsigned char *p, Uint32 bytes)
  * The varpart entry header contains the actual length of the varpart
  * allocated from the page. This size might be equal or bigger than
  * the size of the varpart to be committed. We will always at COMMIT time
- * ensure that we shrink it to the minimum size. It might even be
+ * ensure that we shrink it to the minimum size. It migth even be
  * shrunk to 0 in which case we free the varpart entirely.
  *
  * Handling ABORT
@@ -1150,7 +1150,7 @@ Dbtup::disk_page_commit_callback(Signal* signal,
 
   regOperPtr.p->op_struct.bit_field.m_load_diskpage_on_commit= 0;
   regOperPtr.p->m_commit_disk_callback_page= page_id;
-  ndbrequire(m_global_page_pool.getPtr(diskPagePtr, page_id));
+  m_global_page_pool.getPtr(diskPagePtr, page_id);
   prepare_oper_ptr = regOperPtr;
   
   {
@@ -1812,7 +1812,7 @@ Dbtup::exec_tup_commit(Signal *signal)
   else
   {
     jamDebug();
-    ndbrequire(m_global_page_pool.getPtr(diskPagePtr, diskPagePtr.i));
+    m_global_page_pool.getPtr(diskPagePtr, diskPagePtr.i);
   }
   
   ptrCheckGuard(regTabPtr, no_of_tablerec, tablerec);
@@ -2011,7 +2011,7 @@ Dbtup::execute_real_commit(Signal *signal,
   Uint32 nextOp = leaderOperPtr.p->nextActiveOp;
   Uint32 prevOp = leaderOperPtr.p->prevActiveOp;
   /**
-   * The trigger code (which is shared between detached/immediate)
+   * The trigger code (which is shared between detached/imediate)
    *   check op-list to check were to read before values from
    *   detached triggers should always read from original tuple value
    *   from before transaction start, not from any intermediate update
