@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -40,6 +40,7 @@
 #include <signaldata/AlterIndxImpl.hpp>
 #include <signaldata/ScanFrag.hpp>
 #include "../dblqh/Dblqh.hpp"
+#include "../dbtux/Dbtux.hpp"
 
 #define JAM_FILE_ID 423
 
@@ -254,6 +255,7 @@ void
 Dbtup::execDROP_TRIG_IMPL_REQ(Signal* signal)
 {
   jamEntry();
+  ndbassert(!m_is_query_block);
   const DropTrigImplReq* req = (const DropTrigImplReq*)signal->getDataPtr();
   const Uint32 senderRef = req->senderRef;
   const Uint32 senderData = req->senderData;
@@ -1493,7 +1495,7 @@ void Dbtup::executeTrigger(KeyReqStruct *req_struct,
   ptrCheckGuard(regFragPtr, cnoOfFragrec, fragrecord);
   Fragrecord::FragState fragstatus = regFragPtr.p->fragStatus;
 
-  if (refToMain(ref) == BACKUP)
+  if (refToMain(ref) == getBACKUP())
   {
     jam();
     if (isNdbMtLqh())
@@ -1512,7 +1514,7 @@ void Dbtup::executeTrigger(KeyReqStruct *req_struct,
     */
     signal->theData[0] = trigPtr->triggerId;
     signal->theData[1] = regFragPtr.p->fragmentId;
-    EXECUTE_DIRECT(BACKUP, GSN_BACKUP_TRIG_REQ, signal, 2);
+    EXECUTE_DIRECT(getBACKUP(), GSN_BACKUP_TRIG_REQ, signal, 2);
     jamEntry();
     if (signal->theData[0] == 0) {
       jam();
@@ -1965,7 +1967,7 @@ bool Dbtup::readTriggerInfo(TupTriggerData* const trigPtr,
   if ((regOperPtr->op_struct.bit_field.m_triggers == 
          TupKeyReq::OP_NO_TRIGGERS) &&
       (refToMain(trigPtr->m_receiverRef) == SUMA ||
-       refToMain(trigPtr->m_receiverRef) == BACKUP))
+       refToMain(trigPtr->m_receiverRef) == getBACKUP()))
   {
     /* Operations that have no logical effect need not be backed up
      * or sent as an event. Eg. OPTIMIZE TABLE is performed as a
@@ -2241,10 +2243,10 @@ Dbtup::addTuxEntries(Signal* signal,
       failPtrI = triggerPtr.i;
       goto fail;
     }
-    EXECUTE_DIRECT(DBTUX, GSN_TUX_MAINT_REQ,
-        signal, TuxMaintReq::SignalLength);
+    c_tux->execTUX_MAINT_REQ(signal);
     jamEntryDebug();
-    if (req->errorCode != 0) {
+    if (unlikely(req->errorCode != 0))
+    {
       jam();
       terrorCode = req->errorCode;
       failPtrI = triggerPtr.i;
@@ -2257,12 +2259,11 @@ fail:
   req->opInfo = TuxMaintReq::OpRemove;
   triggerList.first(triggerPtr);
   while (triggerPtr.i != failPtrI) {
-    jam();
+    jamDebug();
     req->indexId = triggerPtr.p->indexId;
     req->errorCode = RNIL;
-    EXECUTE_DIRECT(DBTUX, GSN_TUX_MAINT_REQ,
-        signal, TuxMaintReq::SignalLength);
-    jamEntry();
+    c_tux->execTUX_MAINT_REQ(signal);
+    jamEntryDebug();
     ndbrequire(req->errorCode == 0);
     triggerList.next(triggerPtr);
   }
@@ -2364,13 +2365,13 @@ Dbtup::removeTuxEntries(Signal* signal,
   const TupTriggerData_list& triggerList = regTabPtr->tuxCustomTriggers;
   TriggerPtr triggerPtr;
   triggerList.first(triggerPtr);
-  while (triggerPtr.i != RNIL) {
-    jam();
+  while (triggerPtr.i != RNIL)
+  {
+    jamDebug();
     req->indexId = triggerPtr.p->indexId;
-    req->errorCode = RNIL,
-    EXECUTE_DIRECT(DBTUX, GSN_TUX_MAINT_REQ,
-        signal, TuxMaintReq::SignalLength);
-    jamEntry();
+    req->errorCode = RNIL;
+    c_tux->execTUX_MAINT_REQ(signal); 
+    jamEntryDebug();
     // must succeed
     ndbrequire(req->errorCode == 0);
     triggerList.next(triggerPtr);
