@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2019, 2020 Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,15 +26,21 @@
 #include "Dbqtux.hpp"
 
 #include <EventLogger.hpp>
+extern EventLogger * g_eventLogger;
 
 #define JAM_FILE_ID 530
 
-Dbqtux::Dbqtux(Block_context &ctx, Uint32 instanceNumber)
-    : Dbtux(ctx, instanceNumber, DBQTUX) {}
+Dbqtux::Dbqtux(Block_context& ctx,
+               Uint32 instanceNumber):
+  Dbtux(ctx, instanceNumber, DBQTUX)
+{
+}
 
-Uint64 Dbqtux::getTransactionMemoryNeed() {
+Uint64 Dbqtux::getTransactionMemoryNeed()
+{
   Uint32 query_instance_count =
-      globalData.ndbMtQueryThreads + globalData.ndbMtRecoverThreads;
+    globalData.ndbMtQueryThreads +
+    globalData.ndbMtRecoverThreads;
   Uint64 scan_op_byte_count = 1;
   Uint32 tux_scan_recs = 1;
   Uint32 tux_scan_lock_recs = 1;
@@ -52,4 +58,6 @@ Uint64 Dbqtux::getTransactionMemoryNeed() {
   return (scan_op_byte_count + scan_lock_byte_count + scan_bound_byte_count);
 }
 
-Dbqtux::~Dbqtux() {}
+Dbqtux::~Dbqtux()
+{
+}
