@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,14 +27,17 @@
 
 #define JAM_FILE_ID 375
 
+
 /*
  * Add entry.  Handle the case when there is room for one more.  This
  * is the common case given slack in nodes.
  *
  * Can be called from MT-build of ordered indexes.
  */
-void Dbtux::treeAdd(TuxCtx &ctx, Frag &frag, TreePos treePos, TreeEnt ent) {
-  TreeHead &tree = frag.m_tree;
+void
+Dbtux::treeAdd(TuxCtx& ctx, Frag& frag, TreePos treePos, TreeEnt ent)
+{
+  TreeHead& tree = frag.m_tree;
   NodeHandle node(frag);
   do {
     if (treePos.m_loc != NullTupLoc) {
@@ -66,11 +68,17 @@ void Dbtux::treeAdd(TuxCtx &ctx, Frag &frag, TreePos treePos, TreeEnt ent) {
  * node in left subtree with room for one more.  It will receive the min
  * entry of this node.  The min entry could be the entry to add.
  */
-void Dbtux::treeAddFull(TuxCtx &ctx, Frag &frag, NodeHandle lubNode,
-                        unsigned pos, TreeEnt ent) {
-  TreeHead &tree = frag.m_tree;
+void
+Dbtux::treeAddFull(TuxCtx& ctx,
+                   Frag& frag,
+                   NodeHandle lubNode,
+                   unsigned pos,
+                   TreeEnt ent)
+{
+  TreeHead& tree = frag.m_tree;
   TupLoc loc = lubNode.getLink(0);
-  if (loc != NullTupLoc) {
+  if (loc != NullTupLoc)
+  {
     // find g.l.b node
     NodeHandle glbNode(frag);
     do {
@@ -78,18 +86,30 @@ void Dbtux::treeAddFull(TuxCtx &ctx, Frag &frag, NodeHandle lubNode,
       selectNode(ctx, glbNode, loc);
       loc = glbNode.getLink(1);
     } while (loc != NullTupLoc);
-    if (glbNode.getOccup() < tree.m_maxOccup) {
+    if (glbNode.getOccup() < tree.m_maxOccup)
+    {
       // g.l.b node has room
       thrjamDebug(ctx.jamBuffer);
       Uint32 scanList = RNIL;
       Uint32 scanInstance = 0;
-      if (pos != 0) {
+      if (pos != 0)
+      {
         thrjamDebug(ctx.jamBuffer);
         // add the new entry and return min entry
-        nodePushDown(ctx, lubNode, pos - 1, ent, scanList, scanInstance);
+        nodePushDown(ctx,
+                     lubNode,
+                     pos - 1,
+                     ent,
+                     scanList,
+                     scanInstance);
       }
       // g.l.b node receives min entry from l.u.b node
-      nodePushUp(ctx, glbNode, glbNode.getOccup(), ent, scanList, scanInstance);
+      nodePushUp(ctx,
+                 glbNode,
+                 glbNode.getOccup(),
+                 ent,
+                 scanList,
+                 scanInstance);
       return;
     }
     treeAddNode(ctx, frag, lubNode, pos, ent, glbNode, 1);
@@ -105,9 +125,15 @@ void Dbtux::treeAddFull(TuxCtx &ctx, Frag &frag, NodeHandle lubNode,
  *
  * Can be called from MT-build of ordered indexes.
  */
-void Dbtux::treeAddNode(TuxCtx &ctx, Frag &frag, NodeHandle lubNode,
-                        unsigned pos, TreeEnt ent, NodeHandle parentNode,
-                        unsigned i) {
+void
+Dbtux::treeAddNode(TuxCtx& ctx,
+                   Frag& frag,
+                   NodeHandle lubNode,
+                   unsigned pos,
+                   TreeEnt ent,
+                   NodeHandle parentNode,
+                   unsigned i)
+{
   NodeHandle glbNode(frag);
   insertNode(ctx, glbNode);
   // connect parent and child
@@ -116,10 +142,16 @@ void Dbtux::treeAddNode(TuxCtx &ctx, Frag &frag, NodeHandle lubNode,
   glbNode.setSide(i);
   Uint32 scanList = RNIL;
   Uint32 scanInstance = 0;
-  if (pos != 0) {
+  if (pos != 0)
+  {
     thrjam(ctx.jamBuffer);
     // add the new entry and return min entry
-    nodePushDown(ctx, lubNode, pos - 1, ent, scanList, scanInstance);
+    nodePushDown(ctx,
+                 lubNode,
+                 pos - 1,
+                 ent,
+                 scanList,
+                 scanInstance);
   }
   // g.l.b node receives min entry from l.u.b node
   nodePushUp(ctx, glbNode, 0, ent, scanList, scanInstance);
@@ -133,8 +165,9 @@ void Dbtux::treeAddNode(TuxCtx &ctx, Frag &frag, NodeHandle lubNode,
  *
  * Can be called from MT-build of ordered indexes.
  */
-void Dbtux::treeAddRebalance(TuxCtx &ctx, Frag &frag, NodeHandle node,
-                             unsigned i) {
+void
+Dbtux::treeAddRebalance(TuxCtx & ctx, Frag& frag, NodeHandle node, unsigned i)
+{
   while (true) {
     // height of subtree i has increased by 1
     int j = (i == 0 ? -1 : +1);
@@ -188,8 +221,10 @@ void Dbtux::treeAddRebalance(TuxCtx &ctx, Frag &frag, NodeHandle node,
  * interior nodes this is a requirement.  For others it means that we do
  * not need to consider merge of semi-leaf and leaf.
  */
-void Dbtux::treeRemove(Frag &frag, TreePos treePos) {
-  TreeHead &tree = frag.m_tree;
+void
+Dbtux::treeRemove(Frag& frag, TreePos treePos)
+{
+  TreeHead& tree = frag.m_tree;
   unsigned pos = treePos.m_pos;
   NodeHandle node(frag);
   selectNode(c_ctx, node, treePos.m_loc);
@@ -229,7 +264,11 @@ void Dbtux::treeRemove(Frag &frag, TreePos treePos) {
  * left subtree to borrow an entry from.  The max entry of the g.l.b
  * node becomes the min entry of this node.
  */
-void Dbtux::treeRemoveInner(Frag &frag, NodeHandle lubNode, unsigned pos) {
+void
+Dbtux::treeRemoveInner(Frag& frag,
+                       NodeHandle lubNode,
+                       unsigned pos)
+{
   TreeEnt ent;
   // find g.l.b node
   NodeHandle glbNode(frag);
@@ -242,13 +281,18 @@ void Dbtux::treeRemoveInner(Frag &frag, NodeHandle lubNode, unsigned pos) {
   // borrow max entry from semi/leaf
   Uint32 scanList = RNIL;
   Uint32 scanInstance = 0;
-  nodePopDown(c_ctx, glbNode, glbNode.getOccup() - 1, ent, &scanList,
+  nodePopDown(c_ctx,
+              glbNode,
+              glbNode.getOccup() - 1,
+              ent,
+              &scanList,
               &scanInstance);
   // g.l.b may be empty now
   // a descending scan may try to enter the empty g.l.b
   // we prevent this in scanNext
   nodePopUp(c_ctx, lubNode, pos, ent, scanList, scanInstance);
-  if (glbNode.getLink(0) != NullTupLoc) {
+  if (glbNode.getLink(0) != NullTupLoc)
+  {
     jam();
     treeRemoveSemi(frag, glbNode, 0);
     return;
@@ -261,16 +305,17 @@ void Dbtux::treeRemoveInner(Frag &frag, NodeHandle lubNode, unsigned pos) {
  * semi-leaf to bring semi-leaf occupancy above minOccup, if possible.
  * The leaf may become empty.
  */
-void Dbtux::treeRemoveSemi(Frag &frag, NodeHandle semiNode, unsigned i) {
-  TreeHead &tree = frag.m_tree;
+void
+Dbtux::treeRemoveSemi(Frag& frag, NodeHandle semiNode, unsigned i)
+{
+  TreeHead& tree = frag.m_tree;
   ndbrequire(semiNode.getChilds() < 2);
   TupLoc leafLoc = semiNode.getLink(i);
   NodeHandle leafNode(frag);
   selectNode(c_ctx, leafNode, leafLoc);
   if (semiNode.getOccup() < tree.m_minOccup) {
     jam();
-    unsigned cnt =
-        min(leafNode.getOccup(), tree.m_minOccup - semiNode.getOccup());
+    unsigned cnt = min(leafNode.getOccup(), tree.m_minOccup - semiNode.getOccup());
     nodeSlide(c_ctx, semiNode, leafNode, cnt, i);
     if (leafNode.getOccup() == 0) {
       // remove empty leaf
@@ -285,8 +330,10 @@ void Dbtux::treeRemoveSemi(Frag &frag, NodeHandle semiNode, unsigned i) {
  * entries to it as in the semi-leaf case.  If parent is interior node,
  * do nothing.
  */
-void Dbtux::treeRemoveLeaf(Frag &frag, NodeHandle leafNode) {
-  TreeHead &tree = frag.m_tree;
+void
+Dbtux::treeRemoveLeaf(Frag& frag, NodeHandle leafNode)
+{
+  TreeHead& tree = frag.m_tree;
   TupLoc parentLoc = leafNode.getLink(2);
   if (parentLoc != NullTupLoc) {
     jam();
@@ -298,8 +345,7 @@ void Dbtux::treeRemoveLeaf(Frag &frag, NodeHandle leafNode) {
       jam();
       if (parentNode.getOccup() < tree.m_minOccup) {
         jam();
-        unsigned cnt =
-            min(leafNode.getOccup(), tree.m_minOccup - parentNode.getOccup());
+        unsigned cnt = min(leafNode.getOccup(), tree.m_minOccup - parentNode.getOccup());
         nodeSlide(c_ctx, parentNode, leafNode, cnt, i);
       }
     }
@@ -314,8 +360,10 @@ void Dbtux::treeRemoveLeaf(Frag &frag, NodeHandle leafNode) {
 /*
  * Remove empty leaf.
  */
-void Dbtux::treeRemoveNode(Frag &frag, NodeHandle leafNode) {
-  TreeHead &tree = frag.m_tree;
+void
+Dbtux::treeRemoveNode(Frag& frag, NodeHandle leafNode)
+{
+  TreeHead& tree = frag.m_tree;
   ndbrequire(leafNode.getChilds() == 0);
   TupLoc parentLoc = leafNode.getLink(2);
   unsigned i = leafNode.getSide();
@@ -339,7 +387,9 @@ void Dbtux::treeRemoveNode(Frag &frag, NodeHandle leafNode) {
  * Re-balance tree after removing a node.  The process starts with the
  * parent of the removed node.
  */
-void Dbtux::treeRemoveRebalance(Frag &frag, NodeHandle node, unsigned i) {
+void
+Dbtux::treeRemoveRebalance(Frag& frag, NodeHandle node, unsigned i)
+{
   while (true) {
     // height of subtree i has decreased by 1
     int j = (i == 0 ? -1 : +1);
@@ -407,9 +457,10 @@ void Dbtux::treeRemoveRebalance(Frag &frag, NodeHandle node, unsigned i) {
  * all optional. If 4 are there it changes side.
  *
  * Can be called from MT-build of ordered indexes.
- */
-void Dbtux::treeRotateSingle(TuxCtx &ctx, Frag &frag, NodeHandle &node,
-                             unsigned i) {
+*/
+void
+Dbtux::treeRotateSingle(TuxCtx& ctx, Frag& frag, NodeHandle& node, unsigned i)
+{
   ndbrequire(i <= 1);
   /*
   5 is the old top node that have been unbalanced due to an insert or
@@ -446,10 +497,11 @@ void Dbtux::treeRotateSingle(TuxCtx &ctx, Frag &frag, NodeHandle &node,
   if (loc4 != NullTupLoc) {
     thrjam(ctx.jamBuffer);
     selectNode(ctx, node4, loc4);
-    ndbrequire(node4.getSide() == (1 - i) && node4.getLink(2) == loc3);
+    ndbrequire(node4.getSide() == (1 - i) &&
+               node4.getLink(2) == loc3);
     node4.setSide(i);
     node4.setLink(2, loc5);
-  }  // if
+  }//if
 
   /*
   Retrieve the address of 5's parent before it is destroyed
@@ -457,7 +509,7 @@ void Dbtux::treeRotateSingle(TuxCtx &ctx, Frag &frag, NodeHandle &node,
   TupLoc loc0 = node5.getLink(2);
 
   /*
-  The next step is to perform the rotation. 3 will inherit 5's parent
+  The next step is to perform the rotation. 3 will inherit 5's parent 
   and side. 5 will become a child of 3 on the right side for LL and on
   the left side for RR.
   5 will get 3 as the parent. It will get 4 as a child and it will be
@@ -484,7 +536,7 @@ void Dbtux::treeRotateSingle(TuxCtx &ctx, Frag &frag, NodeHandle &node,
   } else {
     thrjam(ctx.jamBuffer);
     frag.m_tree.m_root = loc3;
-  }  // if
+  }//if
   /* The final step of the change is to update the balance of 3 and
   5 that changed places. There are two cases here. The first case is
   when 3 unbalanced in the same direction by an insert or a delete.
@@ -496,17 +548,22 @@ void Dbtux::treeRotateSingle(TuxCtx &ctx, Frag &frag, NodeHandle &node,
   In this case 5 will change balance but still be unbalanced and 3 will
   be unbalanced in the opposite direction of 5.
   */
-  if (bal3 == bal5) {
+  if (bal3 == bal5)
+  {
     thrjam(ctx.jamBuffer);
     node3.setBalance(0);
     node5.setBalance(0);
-  } else if (bal3 == 0) {
+  }
+  else if (bal3 == 0)
+  {
     thrjam(ctx.jamBuffer);
     node3.setBalance(-bal5);
     node5.setBalance(bal5);
-  } else {
+  }
+  else
+  {
     ndbabort();
-  }  // if
+  }//if
   /*
   Set node to 3 as return parameter for enabling caller to continue
   traversing the tree.
@@ -565,7 +622,7 @@ void Dbtux::treeRotateSingle(TuxCtx &ctx, Frag &frag, NodeHandle &node,
  * 5) Balance change:
  *    If balance(4) == 0 then balance(6) = 0
  *      since height(3) = height(5) = maxheight(3,5) = height(7)
- *    If balance(4) == +1 then balance(6) = 0
+ *    If balance(4) == +1 then balance(6) = 0 
  *      since height(5) = maxheight(3,5) = height(7)
  *    If balance(4) == -1 then balance(6) = 1
  *      since height(5) + 1 = maxheight(3,5) = height(7)
@@ -578,7 +635,7 @@ void Dbtux::treeRotateSingle(TuxCtx &ctx, Frag &frag, NodeHandle &node,
  * 5) Balance change:
  *    If balance(4) == 0 then balance(2) = 0
  *      since height(3) = height(5) = maxheight(3,5) = height(1)
- *    If balance(4) == -1 then balance(2) = 0
+ *    If balance(4) == -1 then balance(2) = 0 
  *      since height(3) = maxheight(3,5) = height(1)
  *    If balance(4) == +1 then balance(6) = 1
  *      since height(3) + 1 = maxheight(3,5) = height(1)
@@ -603,12 +660,12 @@ void Dbtux::treeRotateSingle(TuxCtx &ctx, Frag &frag, NodeHandle &node,
  * If Node 5 exists:
  * 1) Change parent from 4 to 6
  * 2) Change side from 1 - i to i
- *
+ * 
  * If Node 0 exists:
  * 1) previous link to 6 is replaced by link to 4 on proper side
  *
  * Node 1 and 7 needs no changes at all.
- *
+ * 
  * Some additional requires are that balance(2) = - balance(6) = -1/+1 since
  * otherwise we would do a single rotate.
  *
@@ -616,9 +673,10 @@ void Dbtux::treeRotateSingle(TuxCtx &ctx, Frag &frag, NodeHandle &node,
  *
  * Can be called from MT-build of ordered indexes.
  */
-void Dbtux::treeRotateDouble(TuxCtx &ctx, Frag &frag, NodeHandle &node,
-                             unsigned i) {
-  TreeHead &tree = frag.m_tree;
+void
+Dbtux::treeRotateDouble(TuxCtx& ctx, Frag& frag, NodeHandle& node, unsigned i)
+{
+  TreeHead& tree = frag.m_tree;
 
   // old top node
   NodeHandle node6 = node;
@@ -651,9 +709,11 @@ void Dbtux::treeRotateDouble(TuxCtx &ctx, Frag &frag, NodeHandle &node,
   TupLoc loc5 = node4.getLink(1 - i);
 
   // fill up leaf before it becomes internal
-  if (loc3 == NullTupLoc && loc5 == NullTupLoc) {
+  if (loc3 == NullTupLoc && loc5 == NullTupLoc)
+  {
     thrjam(ctx.jamBuffer);
-    if (node4.getOccup() < tree.m_minOccup) {
+    if (node4.getOccup() < tree.m_minOccup)
+    {
       thrjam(ctx.jamBuffer);
       unsigned cnt = tree.m_minOccup - node4.getOccup();
       ndbrequire(cnt < node2.getOccup());
@@ -661,15 +721,19 @@ void Dbtux::treeRotateDouble(TuxCtx &ctx, Frag &frag, NodeHandle &node,
       ndbrequire(node4.getOccup() >= tree.m_minOccup);
       ndbrequire(node2.getOccup() != 0);
     }
-  } else {
-    if (loc3 != NullTupLoc) {
+  }
+  else
+  {
+    if (loc3 != NullTupLoc)
+    {
       thrjam(ctx.jamBuffer);
       NodeHandle node3(frag);
       selectNode(ctx, node3, loc3);
       node3.setLink(2, loc2);
       node3.setSide(1 - i);
     }
-    if (loc5 != NullTupLoc) {
+    if (loc5 != NullTupLoc)
+    {
       thrjam(ctx.jamBuffer);
       NodeHandle node5(frag);
       selectNode(ctx, node5, loc5);
@@ -693,29 +757,39 @@ void Dbtux::treeRotateDouble(TuxCtx &ctx, Frag &frag, NodeHandle &node,
   node4.setLink(2, loc0);
   node4.setSide(side6);
 
-  if (loc0 != NullTupLoc) {
+  if (loc0 != NullTupLoc)
+  {
     thrjam(ctx.jamBuffer);
     selectNode(ctx, node0, loc0);
     node0.setLink(side6, loc4);
-  } else {
+  }
+  else
+  {
     thrjam(ctx.jamBuffer);
     frag.m_tree.m_root = loc4;
   }
   // set balance of changed nodes
   node4.setBalance(0);
-  if (bal4 == 0) {
+  if (bal4 == 0)
+  {
     thrjam(ctx.jamBuffer);
     node2.setBalance(0);
     node6.setBalance(0);
-  } else if (bal4 == -bal2) {
+  }
+  else if (bal4 == -bal2)
+  {
     thrjam(ctx.jamBuffer);
     node2.setBalance(0);
     node6.setBalance(bal2);
-  } else if (bal4 == bal2) {
+  }
+  else if (bal4 == bal2)
+  {
     thrjam(ctx.jamBuffer);
     node2.setBalance(-bal2);
     node6.setBalance(0);
-  } else {
+  }
+  else
+  {
     ndbabort();
   }
   // new top node
