@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -44,7 +44,7 @@ class Lgman : public SimulatedBlock
 {
 public:
   Lgman(Block_context& ctx);
-  virtual ~Lgman();
+  ~Lgman() override;
   BLOCK_DEFINES(Lgman);
   
 protected:
@@ -525,6 +525,7 @@ public:
     Uint32 len;
   };
 
+  Uint64 get_latest_lsn();
   Uint64 add_entry_simple(const Change*,
                           Uint32 cnt,
                           Uint32 alloc_size,
