@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -576,6 +576,20 @@ Ndbfs::execFSOPENREQ(Signal* signal)
     handle.getSection(ptr, FsOpenReq::FILENAME);
   }
   file->theFileName.set(this, userRef, fsOpenReq->fileNumber, false, ptr);
+  if (handle.m_cnt > FsOpenReq::PASSWORD)
+  {
+    jam();
+    SegmentedSectionPtr ptr;
+    handle.getSection(ptr, FsOpenReq::PASSWORD);
+    ndbrequire(ptr.sz * sizeof(Uint32) <= sizeof(file->m_password));
+    copy((Uint32*)&file->m_password, ptr);
+    ndbrequire(4 + file->m_password.password_length <= ptr.sz * sizeof(Uint32));
+    file->m_password.encryption_password[file->m_password.password_length] = 0;
+  }
+  else
+  {
+    file->m_password.password_length = 0;
+  }
   releaseSections(handle);
   
   if (fsOpenReq->fileFlags & FsOpenReq::OM_INIT)
@@ -615,6 +629,7 @@ Ndbfs::execFSOPENREQ(Signal* signal)
       fsRef->userPointer  = userPointer;
       fsRef->setErrorCode(fsRef->errorCode, FsRef::fsErrOutOfMemory);
       fsRef->osErrorCode  = ~0; // Indicate local error
+ndbabort();
       sendSignal(userRef, GSN_FSOPENREF, signal, 3, JBB);
       return;
     }
@@ -882,7 +897,6 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
       }//for
       request->par.readWrite.numberOfPages = fsRWReq->numberOfPages;
       break;
-      // make it a writev or readv
     }//case
 
     case FsReadWriteReq::fsFormatMemAddress:
@@ -1237,9 +1251,9 @@ Ndbfs::createAsyncFile()
     // Print info about all open files
     for (unsigned i = 0; i < theFiles.size(); i++){
       AsyncFile* file = theFiles[i];
-      ndbout_c("%2d (0x%lx): %s",
+      ndbout_c("%2d (%p): %s",
                i,
-               (long) file,
+               file,
                file->isOpen() ?"OPEN" : "CLOSED");
     }
     ndbout_c("m_maxFiles: %u, theFiles.size() = %u",
@@ -1379,6 +1393,7 @@ Ndbfs::report(Request * request, Signal* signal)
       jam();
       // Put the file back in idle files list
       pushIdleFile(request->file);
+//ndbabort();
       sendSignal(ref, GSN_FSOPENREF, signal, FsRef::SignalLength, JBB);
       break;
     }
@@ -1397,16 +1412,15 @@ Ndbfs::report(Request * request, Signal* signal)
       break;
     }
     case Request:: writeSync:
-    case Request:: writevSync:
     case Request:: write:
-    case Request:: writev: {
+    {
       jam();
       sendSignal(ref, GSN_FSWRITEREF, signal, FsRef::SignalLength, JBB);
       break;
     }
     case Request:: read: 
     case Request:: readPartial:
-    case Request:: readv: {
+    {
       jam();
       sendSignal(ref, GSN_FSREADREF, signal, FsRef::SignalLength, JBB);
       break;
@@ -1473,7 +1487,7 @@ Ndbfs::report(Request * request, Signal* signal)
 	m_maxOpenedFiles = theOpenFiles.size();
 
       fsConf->filePointer = request->theFilePointer;
-      fsConf->fileInfo = request->m_fileinfo;
+      fsConf->fileInfo = 0;
       fsConf->file_size_hi = request->m_file_size_hi;
       fsConf->file_size_lo = request->m_file_size_lo;
       sendSignal(ref, GSN_FSOPENCONF, signal, 5, JBA);
@@ -1490,15 +1504,14 @@ Ndbfs::report(Request * request, Signal* signal)
       break;
     }
     case Request:: writeSync:
-    case Request:: writevSync:
     case Request:: write:
-    case Request:: writev: {
+    {
       jam();
       sendSignal(ref, GSN_FSWRITECONF, signal, 1, JBA);
       break;
     }
     case Request:: read:
-    case Request:: readv: {
+    {
       jam();
       sendSignal(ref, GSN_FSREADCONF, signal, 1, JBA);
       break;
@@ -1598,11 +1611,16 @@ Uint32 Ndbfs::translateErrno(int aErrno)
       //none valid parameters
     case ERROR_INVALID_HANDLE:
     case ERROR_INVALID_DRIVE:
+    case ERROR_INVALID_DATA:
     case ERROR_INVALID_ACCESS:
     case ERROR_HANDLE_EOF:
     case ERROR_BUFFER_OVERFLOW:
 
       return FsRef::fsErrInvalidParameters;
+
+    case ERROR_FILE_EXISTS:
+      return FsRef::fsErrFileExists;
+
       //environment error
     case ERROR_CRC:
     case ERROR_ARENA_TRASHED:
@@ -1621,6 +1639,8 @@ Uint32 Ndbfs::translateErrno(int aErrno)
       return FsRef::fsErrNoMoreResources;
       //no file
     case ERROR_FILE_NOT_FOUND:
+    case ERROR_INVALID_NAME:
+    case ERROR_PATH_NOT_FOUND:
       return FsRef::fsErrFileDoesNotExist;
 
     case ERR_ReadUnderflow:
@@ -1663,9 +1683,11 @@ Uint32 Ndbfs::translateErrno(int aErrno)
     case EFAULT:
     case EISDIR:
     case ENOTDIR:
-    case EEXIST:
     case ETXTBSY:
       return FsRef::fsErrInvalidParameters;
+      // file exists
+    case EEXIST:
+      return FsRef::fsErrFileExists;
       //environment error
     case ELOOP:
 #ifdef ENOLINK
@@ -1784,10 +1806,10 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
     
     for (unsigned i = 0; i < theOpenFiles.size(); i++){
       AsyncFile* file = theOpenFiles.getFile(i);
-      infoEvent("%2d (0x%lx): %s thr: %lx", i,
-                (long)file,
+      infoEvent("%2d (%p): %s thr: %p", i,
+                file,
                 file->theFileName.c_str(),
-                (long)file->getThread());
+                file->getThread());
     }
     return;
   }
@@ -1796,7 +1818,7 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
     
     for (unsigned i = 0; i < theFiles.size(); i++){
       AsyncFile* file = theFiles[i];
-      infoEvent("%2d (0x%lx): %s", i, (long)file, file->isOpen()?"OPEN":"CLOSED");
+      infoEvent("%2d (%p): %s", i, file, file->isOpen()?"OPEN":"CLOSED");
     }
     return;
   }
@@ -1806,7 +1828,7 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
 
     for (unsigned i = 0; i < theIdleFiles.size(); i++){
       AsyncFile* file = theIdleFiles[i];
-      infoEvent("%2d (0x%lx): %s", i, (long)file, file->isOpen()?"OPEN":"CLOSED");
+      infoEvent("%2d (%p): %s", i, file, file->isOpen()?"OPEN":"CLOSED");
     }
 
     return;
@@ -1866,7 +1888,7 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
     ndbout << "All files: " << endl;
     for (unsigned i = 0; i < theFiles.size(); i++){
       AsyncFile* file = theFiles[i];
-      ndbout_c("%2d (0x%lx): %s", i, (long) file, file->isOpen()?"OPEN":"CLOSED");
+      ndbout_c("%2d (%p): %s", i, file, file->isOpen()?"OPEN":"CLOSED");
     }
 #endif
   }
@@ -1878,10 +1900,9 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
       AsyncFile* file = theFiles[i];
       if (file == 0)
         continue;
-      ndbout_c("%u : %s %s fileInfo=%08x", i,
+      ndbout_c("%u : %s %s", i,
                file->theFileName.c_str() ? file->theFileName.c_str() : "",
-               file->isOpen() ? "OPEN" : "CLOSED",
-               file->get_fileinfo());
+               file->isOpen() ? "OPEN" : "CLOSED");
     }
   }
 }//Ndbfs::execDUMP_STATE_ORD()
