@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2020, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -259,7 +259,7 @@ class Pgman : public SimulatedBlock
 {
 public:
   Pgman(Block_context& ctx, Uint32 instanceNumber = 0);
-  virtual ~Pgman();
+  ~Pgman() override;
 
   /* Special function to indicate the block is the extra PGMAN worker */
   void init_extra_pgman();
@@ -610,7 +610,7 @@ private:
 
   bool m_track_lcp_speed_loop_ongoing;
 public:
-  void lcp_end_point(Uint32 lcp_time_in_ms);
+  bool lcp_end_point(Uint32 lcp_time_in_ms, bool first, bool internal);
   void set_lcp_dd_percentage(Uint32 dd_percentage);
   void set_current_disk_write_speed(Uint64);
   void lcp_start_point(Signal*, Uint32, Uint32);
@@ -841,6 +841,7 @@ private:
                          Signal*,
                          Ptr<Page_entry>, 
                          Page_request page_req);
+  void set_lsn(Ptr<Page_entry>, Uint64 lsn);
   void update_lsn(Signal *signal,
                   EmulatedJamBuffer* jamBuf,
                   Ptr<Page_entry>,
@@ -943,6 +944,7 @@ public:
    */
   bool init_page_entry(Request&);
 
+  void set_lsn(Local_key, Uint64 lsn);
   void update_lsn(Signal*, Local_key, Uint64 lsn);
 
   /**
