@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2020, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -43,7 +43,7 @@ class Tsman : public SimulatedBlock
 {
 public:
   Tsman(Block_context&);
-  virtual ~Tsman();
+  ~Tsman() override;
   BLOCK_DEFINES(Tsman);
   
 protected:
