@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -38,7 +38,6 @@
 #include <NodeState.hpp>
 #include "ndbd_malloc_impl.hpp"
 
-#include <NdbMem.h>
 #include <NdbMutex.h>
 
 #include <EventLogger.hpp>
@@ -147,13 +146,13 @@ static const char* const jamFileNames[] =
    "GetTableId.hpp",                     // 10
    "NextScan.hpp",                       // 11
    "DihFragCount.hpp",                   // 12
-   "CmInit.hpp",                         // 13
+   "CmInit.hpp",                         // 13 DELETED FILE
    "DropTabFile.hpp",                    // 14
    "BuildIndx.hpp",                      // 15
    "TcContinueB.hpp",                    // 16
    "MasterGCP.hpp",                      // 17
    "UtilPrepare.hpp",                    // 18
-   "DictSizeAltReq.hpp",                 // 19
+   "DictSizeAltReq.hpp",                 // 19 DELETED FILE
    "TabCommit.hpp",                      // 20
    "LqhTransConf.hpp",                   // 21
    "CallbackSignal.hpp",                 // 22
@@ -166,7 +165,7 @@ static const char* const jamFileNames[] =
    "GetTabInfo.hpp",                     // 29
    "BuildIndxImpl.hpp",                  // 30
    "Sync.hpp",                           // 31
-   "CntrMasterReq.hpp",                  // 32
+   "CntrMasterReq.hpp",                  // 32 DELETED FILE
    "CreateIndxImpl.hpp",                 // 33
    "UtilLock.hpp",                       // 34
    "ApiVersion.hpp",                     // 35
@@ -200,7 +199,7 @@ static const char* const jamFileNames[] =
    "DropIndxImpl.hpp",                   // 63
    "CreateHashMap.hpp",                  // 64
    "CmRegSignalData.hpp",                // 65
-   "LqhSizeAltReq.hpp",                  // 66
+   "LqhSizeAltReq.hpp",                  // 66 DELETED FILE
    "StartFragReq.hpp",                   // 67
    "DictTakeover.hpp",                   // 68
    "FireTrigOrd.hpp",                    // 69
@@ -236,20 +235,20 @@ static const char* const jamFileNames[] =
    "ReleasePages.hpp",                   // 99
    "CreateTrig.hpp",                     // 100
    "BackupSignalData.hpp",               // 101
-   "TuxSizeAltReq.hpp",                  // 102
+   "TuxSizeAltReq.hpp",                  // 102 DELETED FILE
    "CreateEvnt.hpp",                     // 103
    "CreateTrigImpl.hpp",                 // 104
    "StartRec.hpp",                       // 105
    "ContinueFragmented.hpp",             // 106
    "CreateObj.hpp",                      // 107
    "DihScanTab.hpp",                     // 108
-   "AccSizeAltReq.hpp",                  // 109
+   "AccSizeAltReq.hpp",                  // 109 DELETED FILE
    "DropFK.hpp",                         // 110
-   "HotSpareRep.hpp",                    // 111
+   "HotSpareRep.hpp",                    // 111 DELETED FILE
    "AlterTable.hpp",                     // 112
    "DisconnectRep.hpp",                  // 113
    "DihContinueB.hpp",                   // 114
-   "TupSizeAltReq.hpp",                  // 115
+   "TupSizeAltReq.hpp",                  // 115 DELETED FILE
    "AllocMem.hpp",                       // 116
    "TamperOrd.hpp",                      // 117
    "ResumeReq.hpp",                      // 118
@@ -259,7 +258,7 @@ static const char* const jamFileNames[] =
    "DbinfoScan.hpp",                     // 122
    "SchemaTrans.hpp",                    // 123
    "UtilDelete.hpp",                     // 124
-   "TcSizeAltReq.hpp",                   // 125
+   "TcSizeAltReq.hpp",                   // 125 DELETED FILE
    "DictStart.hpp",                      // 126
    "TcKeyReq.hpp",                       // 127
    "SrFragidConf.hpp",                   // 128
@@ -288,10 +287,10 @@ static const char* const jamFileNames[] =
    "CreateTable.hpp",                    // 151
    "StartMe.hpp",                        // 152
    "AccLock.hpp",                        // 153
-   "CntrMasterConf.hpp",                 // 154
+   "CntrMasterConf.hpp",                 // 154 DELETED FILE
    "DbspjErr.hpp",                       // 155
    "FsReadWriteReq.hpp",                 // 156
-   "EmptyLcp.hpp",                       // 157
+   "EmptyLcp.hpp",                       // 157 DELETED FILE
    "DropNodegroupImpl.hpp",              // 158
    "InvalidateNodeLCPConf.hpp",          // 159
    "PrepFailReqRef.hpp",                 // 160
@@ -315,7 +314,7 @@ static const char* const jamFileNames[] =
    "TuxContinueB.hpp",                   // 178
    "PgmanContinueB.hpp",                 // 179
    "SystemError.hpp",                    // 180
-   "DihSizeAltReq.hpp",                  // 181
+   "DihSizeAltReq.hpp",                  // 181 DELETED FILE
    "TsmanContinueB.hpp",                 // 182
    "SetVarReq.hpp",                      // 183
    "StartOrd.hpp",                       // 184
@@ -374,13 +373,13 @@ static const char* const jamFileNames[] =
    "testLongSig.cpp",                    // 237
    "TransporterCallback_mt.cpp",         // 238
    "NdbinfoTables.cpp",                  // 239
-   "SuperPool.cpp",                      // 240
+   "SuperPool.cpp",                      // 240 DELETED FILE
    "LongSignal_nonmt.cpp",               // 241
    "FastScheduler.cpp",                  // 242
    "TransporterCallback_nonmt.cpp",      // 243
    "FastScheduler.hpp",                  // 244
    "CountingSemaphore.hpp",              // 245
-   "NdbdSuperPool.cpp",                  // 246
+   "NdbdSuperPool.cpp",                  // 246 DELETED FILE
    "TimeQueue.hpp",                      // 247
    "SimulatedBlock.hpp",                 // 248
    "IntrusiveList.cpp",                  // 249
@@ -390,7 +389,7 @@ static const char* const jamFileNames[] =
    "WatchDog.hpp",                       // 253
    "SimplePropertiesSection_mt.cpp",     // 254
    "Pool.cpp",                           // 255
-   "ClusterConfiguration.cpp",           // 256
+   "ClusterConfiguration.cpp",           // 256 DELETED FILE
    "DLCHashTable.hpp",                   // 257
    "KeyTable2.hpp",                      // 258
    "KeyDescriptor.hpp",                  // 259
@@ -398,13 +397,13 @@ static const char* const jamFileNames[] =
    "LHLevel.hpp",                        // 261
    "LongSignal.cpp",                     // 262
    "ThreadConfig.cpp",                   // 263
-   "LinearPool.hpp",                     // 264
+   "LinearPool.hpp",                     // 264 DELETED FILE
    "SafeMutex.cpp",                      // 265
    "SafeCounter.cpp",                    // 266
    "bench_pool.cpp",                     // 267
-   "DataBuffer2.hpp",                    // 268
+   "DataBuffer2.hpp",                    // 268 DELETED FILE
    "Mutex.hpp",                          // 269
-   "testSuperPool.cpp",                  // 270
+   "testSuperPool.cpp",                  // 270 DELETED FILE
    "CArray.hpp",                         // 271
    "mt_thr_config.hpp",                  // 272
    "TimeQueue.cpp",                      // 273
@@ -415,7 +414,7 @@ static const char* const jamFileNames[] =
    "RWPool.cpp",                         // 278
    "GlobalData.cpp",                     // 279
    "Prio.hpp",                           // 280
-   "SuperPool.hpp",                      // 281
+   "SuperPool.hpp",                      // 281 DELETED FILE
    "pc.hpp",                             // 282
    "LockQueue.hpp",                      // 283
    "ClusterConfiguration.hpp",           // 284
@@ -436,28 +435,28 @@ static const char* const jamFileNames[] =
    "DynArr256.hpp",                      // 299
    "LongSignal_mt.cpp",                  // 300
    "Configuration.cpp",                  // 301
-   "WaitQueue.hpp",                      // 302
+   "WaitQueue.hpp",                      // 302 DELETED FILE
    "WOPool.hpp",                         // 303
    "CountingPool.cpp",                   // 304
    "TransporterCallbackKernel.hpp",      // 305
-   "NdbdSuperPool.hpp",                  // 306
+   "NdbdSuperPool.hpp",                  // 306 DELETED FILE
    "DLHashTable2.hpp",                   // 307
    "VMSignal.cpp",                       // 308
    "ArenaPool.cpp",                      // 309
    "LHLevel.cpp",                        // 310
    "RWPool.hpp",                         // 311
-   "t.cpp",                              // 312
+   "mt-send-t.cpp",                      // 312
    "DLHashTable.hpp",                    // 313
    "VMSignal.hpp",                       // 314
    "Pool.hpp",                           // 315
    "Rope.hpp",                           // 316
-   "KeyTable2Ref.hpp",                   // 317
+   "KeyTable2Ref.hpp",                   // 317 DELETED FILE
    "LockQueue.cpp",                      // 318
-   "arrayListTest.cpp",                  // 319
-   "main.cpp",                           // 320
-   "arrayPoolTest.cpp",                  // 321
+   "arrayListTest.cpp",                  // 319 DELETED FILE
+   "main.cpp",                           // 320 DELETED FILE
+   "arrayPoolTest.cpp",                  // 321 DELETED FILE
    "SimBlockList.hpp",                   // 322
-   "lock.hpp",                           // 323
+   "mt-lock.hpp",                        // 323
    "rr.cpp",                             // 324
    "testCopy.cpp",                       // 325
    "Ndbinfo.cpp",                        // 326
@@ -497,7 +496,7 @@ static const char* const jamFileNames[] =
    "QmgrMain.cpp",                       // 360
    "QmgrInit.cpp",                       // 361
    "Qmgr.hpp",                           // 362
-   "timer.hpp",                          // 363
+   "timer.hpp",                          // 363 DELETED FILE
    "diskpage.cpp",                       // 364
    "DbtuxGen.cpp",                       // 365
    "DbtuxDebug.cpp",                     // 366
@@ -516,14 +515,14 @@ static const char* const jamFileNames[] =
    "Cmvmi.hpp",                          // 379
    "Cmvmi.cpp",                          // 380
    "AsyncIoThread.hpp",                  // 381
-   "MemoryChannelTest.cpp",              // 382
+   "MemoryChannelTest.cpp",              // 382 DELETED FILE
    "Filename.cpp",                       // 383
    "PosixAsyncFile.cpp",                 // 384
    "Ndbfs.hpp",                          // 385
    "OpenFiles.hpp",                      // 386
    "AsyncFile.cpp",                      // 387
    "AsyncIoThread.cpp",                  // 388
-   "AsyncFileTest.cpp",                  // 389
+   "AsyncFileTest.cpp",                  // 389 DELETED FILE
    "MemoryChannel.cpp",                  // 390
    "AsyncFile.hpp",                      // 391
    "Filename.hpp",                       // 392
@@ -581,7 +580,7 @@ static const char* const jamFileNames[] =
    "DblqhCommon.cpp",                    // 444
    "DblqhProxy.hpp",                     // 445
    "DblqhStateDesc.cpp",                 // 446
-   "records.hpp",                        // 447
+   "records.hpp",                        // 447 DELETED FILE
    "records.cpp",                        // 448
    "reader.cpp",                         // 449
    "Dblqh.hpp",                          // 450
@@ -627,11 +626,54 @@ static const char* const jamFileNames[] =
    "ErrorReporter.cpp",                  // 490
    "angel.hpp",                          // 491
    "SimBlockList.cpp",                   // 492
-   "CopyTab.cpp"                         // 493
+   "CopyTab.hpp",                        // 493
    "IsolateOrd.hpp",                     // 494
-   "IsolateOrd.cpp"                      // 495
+   "IsolateOrd.cpp",                     // 495
    "SegmentList.hpp",                    // 496
-   "SegmentList.cpp"                     // 497
+   "SegmentList.cpp",                    // 497
+   "LocalSysfile.hpp",                   // 498
+   "UndoLogLevel.hpp",                   // 499
+   "RedoStateRep.hpp",                   // 500
+   "printFragfile.cpp",                  // 501
+   "TransientPagePool.hpp",              // 502
+   "TransientPagePool.cpp",              // 503
+   "TransientSlotPool.hpp",              // 504
+   "TransientSlotPool.cpp",              // 505
+   "TransientPool.hpp",                  // 506
+   "Slot.hpp",                           // 507
+   "StaticSlotPool.hpp",                 // 508
+   "StaticSlotPool.cpp",                 // 509
+   "ComposedSlotPool.hpp",               // 510
+   "IntrusiveTags.hpp",                  // 511
+   "Sysfile.cpp",                        // 512
+   "BlockThreadBitmask.hpp",             // 513
+   "SyncThreadViaReqConf.hpp",           // 514
+   "TakeOverTcConf.hpp",                 // 515
+   "GetNumMultiTrp.hpp",                 // 516
+   "Dbqlqh.hpp",                         // 517
+   "Dbqlqh.cpp",                         // 518
+   "DbqlqhProxy.hpp",                    // 519
+   "DbqlqhProxy.cpp",                    // 520
+   "Dbqacc.hpp",                         // 521
+   "Dbqacc.cpp",                         // 522
+   "DbqaccProxy.hpp",                    // 523
+   "DbqaccProxy.cpp",                    // 524
+   "Dbqtup.hpp",                         // 525
+   "Dbqtup.cpp",                         // 526
+   "DbqtupProxy.hpp",                    // 527
+   "DbqtupProxy.cpp",                    // 528
+   "Dbqtux.hpp",                         // 529
+   "Dbqtux.cpp",                         // 530
+   "DbqtuxProxy.hpp",                    // 531
+   "DbqtuxProxy.cpp",                    // 532
+   "QBackup.hpp",                        // 533
+   "QBackup.cpp",                        // 534
+   "QBackupProxy.hpp",                   // 535
+   "QBackupProxy.cpp",                   // 536
+   "QRestore.hpp",                       // 537
+   "QRestore.cpp",                       // 538
+   "QRestoreProxy.hpp",                  // 539
+   "QRestoreProxy.cpp",                  // 540
    };
 
 bool 
@@ -701,13 +743,11 @@ EmulatorData::create(){
     For multithreaded ndbd, each thread will set a local jam buffer later.
   */
 #ifndef NO_EMULATED_JAM
-  void * jamBuffer = (void *)&theEmulatedJamBuffer;
+  EmulatedJamBuffer * jamBuffer = &theEmulatedJamBuffer;
 #else
-  void * jamBuffer = 0;
+  EmulatedJamBuffer * jamBuffer = nullptr;
 #endif
-  NdbThread_SetTlsKey(NDB_THREAD_TLS_JAM, jamBuffer);
-
-  NdbMem_Create();
+  NDB_THREAD_TLS_JAM = jamBuffer;
 
   theConfiguration = new Configuration();
   theWatchDog      = new WatchDog();
@@ -738,18 +778,22 @@ EmulatorData::create(){
 void
 EmulatorData::destroy(){
   if(theConfiguration)
-    delete theConfiguration; theConfiguration = 0;
+    delete theConfiguration;
+  theConfiguration = 0;
   if(theWatchDog)
-    delete theWatchDog; theWatchDog = 0;
+    delete theWatchDog;
+  theWatchDog = 0;
   if(theThreadConfig)
-    delete theThreadConfig; theThreadConfig = 0;
+    delete theThreadConfig;
+  theThreadConfig = 0;
   if(theSimBlockList)
-    delete theSimBlockList; theSimBlockList = 0;
+    delete theSimBlockList;
+  theSimBlockList = 0;
   if(m_socket_server)
-    delete m_socket_server; m_socket_server = 0;
+    delete m_socket_server;
+  m_socket_server = 0;
   NdbMutex_Destroy(theShutdownMutex);
   if (m_mem_manager)
-    delete m_mem_manager; m_mem_manager = 0;
-  
-  NdbMem_Destroy();
+    delete m_mem_manager;
+  m_mem_manager = 0;
 }
