@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2022, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,7 +26,6 @@
 #define GLOBAL_DATA_H
 
 #include <ndb_global.h>
-#include <cstring>
 #include <kernel_types.h>
 #include "Prio.hpp"
 #include "VMSignal.hpp"
@@ -131,7 +130,7 @@ struct GlobalData {
     theBufferFullMicrosSleep = 0;
     theMicrosSend = 0;
     theMicrosSpin = 0;
-    std::memset(m_hb_count, 0, sizeof(m_hb_count));
+    bzero(m_hb_count, sizeof(m_hb_count));
 #ifdef GCP_TIMER_HACK
     gcp_timer_limit = 0;
 #endif
