@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,8 +25,8 @@
 #ifndef PAGEPOOL_HPP
 #define PAGEPOOL_HPP
 
-#include "Pool.hpp"
 #include "ndb_limits.h"
+#include "Pool.hpp"
 
 #define JAM_FILE_ID 502
 
@@ -48,97 +48,85 @@
  * can handle slightly less than 2^26 pages.
  */
 
-class TransientPagePool {
- public:
+class TransientPagePool
+{
+public:
   class MapPage;
   class Page;
   class PageMap;
 
-  // Allow unit tests to access private data members by using class Test.
-  friend class Test;
-
   TransientPagePool();
-  TransientPagePool(Uint32 type_id, Ndbd_mem_manager *mem_manager);
-  void init(Uint32 type_id, Ndbd_mem_manager *mem_manager);
+  TransientPagePool(Uint32 type_id,
+                    Ndbd_mem_manager* mem_manager);
+  void init(Uint32 type_id, Ndbd_mem_manager* mem_manager);
 
-  bool seize(Ptr<Page> &p);
+  bool seize(Ptr<Page>& p);
   bool release(Uint32 i);
   bool release(Ptr<Page> p);
-  bool getPtr(Ptr<Page> &p) const;
-  bool getUncheckedPtr(Ptr<Page> &p) const;
-  bool getValidPtr(Ptr<Page> &p) const;
+  bool getPtr(Ptr<Page>& p) const;
+  bool getUncheckedPtr(Ptr<Page>& p) const;
+  bool getValidPtr(Ptr<Page>& p) const;
   Uint32 getTopPageNumber() const;
   bool canRelease(Uint32 i) const;
 
   static Uint64 getMemoryNeed(Uint32 pages);
-
- private:
+private:
   /* Page map methods */
-  static bool is_valid_index(Uint32 index);
-  static Uint32 get_next_index(Uint32 index);
-  static Uint32 get_prev_index(Uint32 index);
-  static bool on_same_map_page(Uint32 index1, Uint32 index2);
-  // Uint32 get_next_indexes(Uint32 index, Uint32 indexes[], Uint32 n) const;
+  Uint32 get_next_index(Uint32 index) const;
+  //Uint32 get_next_indexes(Uint32 index, Uint32 indexes[], Uint32 n) const;
   bool set(Uint32 index, Uint32 value);
   bool clear(Uint32 index);
   Uint32 get(Uint32 index) const;
   Uint32 get_valid(Uint32 i) const;
   bool shrink();
 
-  Ndbd_mem_manager *m_mem_manager;
+  Ndbd_mem_manager* m_mem_manager;
   /** PAGE MAP **/
-  MapPage *m_root_page;
+  MapPage* m_root_page;
   Uint32 m_top;
   Uint32 m_type_id;
 };
 
-class TransientPagePool::MapPage {
- public:
-  static constexpr Uint32 PAGE_ID_GAP = 8;
-  static constexpr Uint32 PAGE_WORDS = 8192 - PAGE_ID_GAP;
-  static constexpr Uint32 VALUE_INDEX_BITS = 13;
-  static constexpr Uint32 VALUE_INDEX_MASK = (1U << VALUE_INDEX_BITS) - 1;
-  static constexpr Uint32 NO_VALUE = 0;
-
-  /*
-   * Biggest page id supported by one MapPage.
-   */
-  static constexpr Uint32 MAX_PAGE_ID_1L = PAGE_WORDS - 1;
-  /*
-   * Biggest page id supported by two levels of MapPage.
-   */
-  static constexpr Uint32 MAX_PAGE_ID_2L =
-      (MAX_PAGE_ID_1L) + (MAX_PAGE_ID_1L)*8192;
+class TransientPagePool::MapPage
+{
+public:
+  STATIC_CONST( PAGE_WORDS = 8192 - 8 );
+  STATIC_CONST( VALUE_INDEX_BITS = 13 );
+  STATIC_CONST( VALUE_INDEX_MASK = (1U << VALUE_INDEX_BITS) - 1 );
+  STATIC_CONST( NO_VALUE = 0 );
 
   MapPage(Uint32 magic);
   Uint32 get(Uint32 i) const;
   void set(Uint32 i, Uint32 v);
+private:
 
- private:
   Uint32 m_magic;
   Uint32 m_reserved[7];
   Uint32 m_values[PAGE_WORDS];
 };
 
-class TransientPagePool::Page {
+class TransientPagePool::Page
+{
   friend class TransientPagePool;
-
- private:
+private:
   Uint32 m_magic;
   Uint32 m_page_id;
   Uint32 m_padding[8192 - 2];
 };
 
-inline Uint32 TransientPagePool::getTopPageNumber() const { return m_top; }
+inline Uint32 TransientPagePool::getTopPageNumber() const
+{
+  return m_top;
+}
 
-inline bool TransientPagePool::canRelease(Uint32 i) const {
+inline bool TransientPagePool::canRelease(Uint32 i) const
+{
   return (i != RNIL) && (i == m_top) && (m_top > 0);
 }
 
-inline bool TransientPagePool::release(Ptr<Page> p) { return release(p.i); }
-
-inline bool TransientPagePool::on_same_map_page(Uint32 index1, Uint32 index2) {
-  return (((index1 ^ index2) >> MapPage::VALUE_INDEX_BITS) == 0);
+inline bool TransientPagePool::release(Ptr<Page> p)
+{
+  return release(p.i);
 }
 
 #undef JAM_FILE_ID
