@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2005, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,11 +36,10 @@
  */
 void *ndbd_malloc(size_t size);
 bool ndbd_malloc_need_watchdog(size_t size);
-void *ndbd_malloc_watched(size_t size, volatile Uint32 *watch_dog);
+void *ndbd_malloc_watched(size_t size, volatile Uint32* watch_dog);
 void ndbd_free(void *p, size_t size);
-void ndbd_alloc_touch_mem(void *p, size_t sz, volatile Uint32 *watchCounter,
-                          bool make_readwritable);
+
 
 #undef JAM_FILE_ID
 
-#endif
+#endif 
