@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -67,8 +67,8 @@ private:
 
 public:
   MgmApiSession(class MgmtSrvr & mgm, NDB_SOCKET_TYPE sock, Uint64 session_id);
-  virtual ~MgmApiSession();
-  void runSession();
+  ~MgmApiSession() override;
+  void runSession() override;
 
   static const unsigned SOCKET_TIMEOUT = 30000;
 
@@ -155,7 +155,7 @@ public:
     m_mgmsrv(mgm),
     m_next_session_id(1) {}
 
-  SocketServer::Session * newSession(NDB_SOCKET_TYPE socket){
+  SocketServer::Session * newSession(NDB_SOCKET_TYPE socket) override{
     return new MgmApiSession(m_mgmsrv, socket, m_next_session_id++);
   }
 };
