@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,7 +26,7 @@
 #define NDB_IMPL_HPP
 
 #include <ndb_global.h>
-#include "my_config.h"
+#include "API.hpp"
 #include <NdbOut.hpp>
 #include <kernel/ndb_limits.h>
 #include <NdbTick.h>
@@ -41,7 +41,6 @@
 #include "trp_node.hpp"
 #include "NdbWaiter.hpp"
 #include "WakeupHandler.hpp"
-#include "kernel/signaldata/TcKeyConf.hpp"
 
 template <class T>
 struct Ndb_free_list_t 
@@ -67,64 +66,13 @@ private:
   Ndb_free_list_t& operator=(const Ndb_free_list_t&);
 
   /**
-   * update_stats() is called whenever a new local peak of 'm_used_cnt'
-   * objects has been observed.
-   *
-   * The high usage peaks are most interesting as we want to scale the
-   * free-list to accommodate these - The smaller peaks in between are mostly
-   * considered as 'noise' in this statistics. Which may cause a too low
-   * usage statistics to be collected, such that the high usage peaks could
-   * not be served from the free-list.
-   *
-   * In order to implement this we use a combination of statistics and
-   * heuristics. Heuristics is based on observing free-list behavior of
-   * an instrumented version of this code.
-   *
-   * 1) A 'high peak' is any peak value above or equal to the current
-   *    sampled mean value. -> Added to the statistics immediately .
-   * 2) A sampled peak value of 2 or less is considered as 'noise' and
-   *    just ignored.
-   * 3) Other peak values, less than the current mean:
-   *    These are observed over a period of such smaller peaks, and their
-   *    max value collected in 'm_sample_max'. When the windows size has expired,
-   *    the 'm_sample_max' value is sampled.
-   *    Intention with this heuristic is that temporary reduced usage of objects
-   *    should be ignored, but longer term changes should be acounted for.
-   *
-   * When we have taken a valid sample, we use the statistics to calculate the
-   * 95% percentile for max objects in use of 'class T'.
+   * Based on a serie of sampled max. values for m_used_cnt;
+   * calculate the 95% percentile for max objects in use of 'class T'.
    */
   void update_stats()
   {
-    const Uint32 mean = m_stats.getMean();
-    if (m_used_cnt >= mean)
-      // 1) A high-peak value, sample it
-      m_stats.update(m_used_cnt);
-    else if (m_used_cnt <= 2)
-      // 2) Ignore very low sampled values, is 'noise'
-      return;
-    else
-    {
-      // 3) A local peak, less than current 'mean'
-      if (m_sample_max < m_used_cnt)
-        m_sample_max = m_used_cnt;
-
-      // Use a decay function of current 'mean' to decide how many small samples
-      // we may ignore - Smaller samples are ignored for a longer time.
-      const Uint32 max_skipped = (mean*5) / m_used_cnt;
-      m_samples_skipped++;
-      if (m_samples_skipped < max_skipped && m_samples_skipped < 10)
-        return;
-
-      // Expired low-value observation period, sample max value seen.
-      m_stats.update(m_sample_max);
-    }
-    m_sample_max = 0;
-    m_samples_skipped = 0;
-
-    // Calculate upper 95% percentile from sampled values
-    const double upper = m_stats.getMean() + (2 * m_stats.getStdDev());
-    m_estm_max_used = (Uint32)(upper+0.999);
+    m_stats.update(m_used_cnt);
+    m_estm_max_used = (Uint32)(m_stats.getMean() + (2 * m_stats.getStdDev()));
   }
 
   /** Shrink m_free_list such that m_used_cnt+'free' <= 'm_estm_max_used' */
@@ -136,12 +84,6 @@ private:
   /** Last operation allocated, or grabbed a free object */
   bool m_is_growing;
 
-  /** Number of consecuitive 'low-peak' values skipped */
-  Uint32 m_samples_skipped;
-
-  /** Max sample value seen in the 'm_samples_skipped' period */
-  Uint32 m_sample_max;
-
   /** Statistics of peaks in number of obj 'T' in use */
   NdbStatistics m_stats;
 
@@ -211,6 +153,18 @@ public:
   BaseString m_dbname; // Database name
   BaseString m_schemaname; // Schema name
 
+  BaseString m_prefix; // Buffer for preformatted internal name <db>/<schema>/
+
+  int update_prefix()
+  {
+    if (!m_prefix.assfmt("%s%c%s%c", m_dbname.c_str(), table_name_separator,
+                         m_schemaname.c_str(), table_name_separator))
+    {
+      return -1;
+    }
+    return 0;
+  }
+
 /*
   We need this friend accessor function to work around a HP compiler problem,
   where template class friends are not working.
@@ -229,12 +183,14 @@ public:
   }
 
   Uint32 get_waitfor_timeout() const {
-    return m_ndb_cluster_connection.m_ndbapiconfig.m_waitfor_timeout;
+    return m_ndb_cluster_connection.m_config.m_waitfor_timeout;
   }
   const NdbApiConfig& get_ndbapi_config_parameters() const {
-    return m_ndb_cluster_connection.m_ndbapiconfig;
+    return m_ndb_cluster_connection.m_config;
   }
 
+  BaseString m_systemPrefix; // Buffer for preformatted for <sys>/<def>/
+  
   Uint64 customData;
 
   Uint64 clientStats[ Ndb::NumClientStatistics ];
@@ -258,7 +214,7 @@ public:
   }
 
   /* We don't record the sent/received bytes of some GSNs as they are 
-   * generated constantly and are not targeted to specific
+   * generated constantly and are not targetted to specific
    * Ndb instances.
    * See also TransporterFacade::TRACE_GSN
    */
@@ -323,10 +279,7 @@ public:
   Uint32 getNodeSequence(NodeId nodeId) const;
   Uint32 getNodeNdbVersion(NodeId nodeId) const;
   Uint32 getMinDbNodeVersion() const;
-  bool check_send_size(Uint32 /*node_id*/, Uint32 /*send_size*/) const
-  {
-    return true;
-  }
+  bool check_send_size(Uint32 node_id, Uint32 send_size) const { return true;}
 
   int sendSignal(NdbApiSignal*, Uint32 nodeId);
   int sendSignal(NdbApiSignal*, Uint32 nodeId,
@@ -417,7 +370,7 @@ NdbReceiver::getTransaction(ReceiverType type) const
   {
   case NDB_UNINITIALIZED:
     assert(false);
-    return nullptr;
+    return NULL;
   case NDB_QUERY_OPERATION:
     return &((NdbQueryOperationImpl*)m_owner)->getQuery().getNdbTransaction();
   default:
@@ -451,10 +404,8 @@ inline
 Ndb_free_list_t<T>::Ndb_free_list_t()
  : m_used_cnt(0),
    m_free_cnt(0),
-   m_free_list(nullptr),
+   m_free_list(NULL),
    m_is_growing(false),
-   m_samples_skipped(0),
-   m_sample_max(0),
    m_stats(),
    m_estm_max_used(0)
 {}
@@ -482,10 +433,10 @@ Ndb_free_list_t<T>::fill(Ndb* ndb, Uint32 cnt)
 {
 #ifndef HAVE_VALGRIND
   m_is_growing = true;
-  if (m_free_list == nullptr)
+  if (m_free_list == 0)
   {
     m_free_list = new T(ndb);
-    if (m_free_list == nullptr)
+    if (m_free_list == 0)
     {
       NdbImpl::setNdbError(*ndb, 4000);
       assert(false);
@@ -496,7 +447,7 @@ Ndb_free_list_t<T>::fill(Ndb* ndb, Uint32 cnt)
   while(m_free_cnt < cnt)
   {
     T* obj= new T(ndb);
-    if(obj == nullptr)
+    if(obj == 0)
     {
       NdbImpl::setNdbError(*ndb, 4000);
       assert(false);
@@ -508,10 +459,6 @@ Ndb_free_list_t<T>::fill(Ndb* ndb, Uint32 cnt)
   }
   return 0;
 #else
-  // Older versions of gcc do not like [[maybe_unused]] in templates.
-  // So disable the maybe-unused warning like this instead:
-  (void) ndb;
-  (void) cnt;
   return 0;
 #endif
 }
@@ -524,13 +471,13 @@ Ndb_free_list_t<T>::seize(Ndb* ndb)
 #ifndef HAVE_VALGRIND
   T* tmp = m_free_list;
   m_is_growing = true;
-  if (likely(tmp != nullptr))
+  if (likely(tmp != NULL))
   {
     m_free_list = (T*)tmp->next();
-    tmp->next(nullptr);
+    tmp->next(NULL);
     m_free_cnt--;
   }
-  else if (unlikely((tmp = new T(ndb)) == nullptr))
+  else if (unlikely((tmp = new T(ndb)) == NULL))
   {
     NdbImpl::setNdbError(*ndb, 4000);
     assert(false);
@@ -583,13 +530,13 @@ Ndb_free_list_t<T>::release(Uint32 cnt, T* head, T* tail)
   {
     T* tmp = head;
     Uint32 tmp_cnt = 0;
-    while (tmp != nullptr && tmp != tail)
+    while (tmp != 0 && tmp != tail)
     {
       tmp = (T*)tmp->next();
       tmp_cnt++;
     }
     assert(tmp == tail);
-    assert((tail==nullptr && tmp_cnt==0) || tmp_cnt+1 == cnt);
+    assert((tail==NULL && tmp_cnt==0) || tmp_cnt+1 == cnt);
   }
 #endif
 
