@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,15 +22,17 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#include "API.hpp"
 #include <AttributeHeader.hpp>
-#include <signaldata/IndxAttrInfo.hpp>
-#include <signaldata/IndxKeyInfo.hpp>
 #include <signaldata/TcIndx.hpp>
 #include <signaldata/TcKeyReq.hpp>
-#include "API.hpp"
+#include <signaldata/IndxKeyInfo.hpp>
+#include <signaldata/IndxAttrInfo.hpp>
 
-NdbIndexOperation::NdbIndexOperation(Ndb *aNdb)
-    : NdbOperation(aNdb, NdbOperation::UniqueIndexAccess), m_theIndex(nullptr) {
+NdbIndexOperation::NdbIndexOperation(Ndb* aNdb) :
+  NdbOperation(aNdb, NdbOperation::UniqueIndexAccess),
+  m_theIndex(NULL)
+{
   m_tcReqGSN = GSN_TCINDXREQ;
   m_attrInfoGSN = GSN_INDXATTRINFO;
   m_keyInfoGSN = GSN_INDXKEYINFO;
@@ -41,30 +43,34 @@ NdbIndexOperation::NdbIndexOperation(Ndb *aNdb)
   theReceiver.init(NdbReceiver::NDB_INDEX_OPERATION, this);
 }
 
-NdbIndexOperation::~NdbIndexOperation() {}
+NdbIndexOperation::~NdbIndexOperation()
+{
+}
 
 /*****************************************************************************
  * int indxInit();
  *
  * Return Value:  Return 0 : init was successful.
- *                Return -1: In all other case.
+ *                Return -1: In all other case.  
  * Remark:        Initiates operation record after allocation.
  *****************************************************************************/
-int NdbIndexOperation::indxInit(const NdbIndexImpl *anIndex,
-                                const NdbTableImpl *aTable,
-                                NdbTransaction *myConnection) {
+int
+NdbIndexOperation::indxInit(const NdbIndexImpl * anIndex,
+			    const NdbTableImpl * aTable, 
+			    NdbTransaction* myConnection)
+{
   NdbOperation::init(aTable, myConnection);
 
   switch ((NdbDictionary::Index::Type)anIndex->m_type) {
-    case (NdbDictionary::Index::UniqueHashIndex):
-      break;
-    case (NdbDictionary::Index::Undefined):
-    case (NdbDictionary::Index::OrderedIndex):
-      setErrorCodeAbort(4003);
-      return -1;
-    default:
-      assert(0);
-      break;
+  case(NdbDictionary::Index::UniqueHashIndex):
+    break;
+  case(NdbDictionary::Index::Undefined):
+  case(NdbDictionary::Index::OrderedIndex):
+    setErrorCodeAbort(4003);
+    return -1;
+  default:
+    DBUG_ASSERT(0);
+    break;
   }
   m_theIndex = anIndex;
   m_accessTable = anIndex->m_table;
@@ -72,107 +78,118 @@ int NdbIndexOperation::indxInit(const NdbIndexImpl *anIndex,
   return 0;
 }
 
-int NdbIndexOperation::readTuple(NdbOperation::LockMode lm) {
-  switch (lm) {
-    case LM_Read:
-      return readTuple();
-      break;
-    case LM_Exclusive:
-      return readTupleExclusive();
-      break;
-    case LM_CommittedRead:
-      return readTuple();
-      break;
-    case LM_SimpleRead:
-      return readTuple();
-      break;
-    default:
-      return -1;
+int NdbIndexOperation::readTuple(NdbOperation::LockMode lm)
+{ 
+  switch(lm) {
+  case LM_Read:
+    return readTuple();
+    break;
+  case LM_Exclusive:
+    return readTupleExclusive();
+    break;
+  case LM_CommittedRead:
+    return readTuple();
+    break;
+  case LM_SimpleRead:
+    return readTuple();
+    break;
+  default:
+    return -1;
   };
 }
 
-int NdbIndexOperation::insertTuple() {
+int NdbIndexOperation::insertTuple()
+{
   setErrorCode(4200);
   return -1;
 }
 
-int NdbIndexOperation::readTuple() {
+int NdbIndexOperation::readTuple()
+{
   // First check that index is unique
 
   return NdbOperation::readTuple();
 }
 
-int NdbIndexOperation::readTupleExclusive() {
+int NdbIndexOperation::readTupleExclusive()
+{
   // First check that index is unique
 
   return NdbOperation::readTupleExclusive();
 }
 
-int NdbIndexOperation::simpleRead() {
+int NdbIndexOperation::simpleRead()
+{
   // First check that index is unique
 
   return NdbOperation::readTuple();
 }
 
-int NdbIndexOperation::dirtyRead() {
+int NdbIndexOperation::dirtyRead()
+{
   // First check that index is unique
 
   return NdbOperation::readTuple();
 }
 
-int NdbIndexOperation::committedRead() {
+int NdbIndexOperation::committedRead()
+{
   // First check that index is unique
 
   return NdbOperation::readTuple();
 }
 
-int NdbIndexOperation::updateTuple() {
+int NdbIndexOperation::updateTuple()
+{
   // First check that index is unique
 
   return NdbOperation::updateTuple();
 }
 
-int NdbIndexOperation::deleteTuple() {
+int NdbIndexOperation::deleteTuple()
+{
   // First check that index is unique
 
   return NdbOperation::deleteTuple();
 }
 
-int NdbIndexOperation::dirtyUpdate() {
+int NdbIndexOperation::dirtyUpdate()
+{
   // First check that index is unique
 
   return NdbOperation::dirtyUpdate();
 }
 
-int NdbIndexOperation::interpretedWriteTuple() {
-  setErrorCode(4200);
-  return -1;
-}
-
-int NdbIndexOperation::interpretedUpdateTuple() {
+int NdbIndexOperation::interpretedUpdateTuple()
+{
   // First check that index is unique
 
   return NdbOperation::interpretedUpdateTuple();
 }
 
-int NdbIndexOperation::interpretedDeleteTuple() {
+int NdbIndexOperation::interpretedDeleteTuple()
+{
   // First check that index is unique
 
   return NdbOperation::interpretedDeleteTuple();
 }
 
-const NdbDictionary::Index *NdbIndexOperation::getIndex() const {
+const NdbDictionary::Index*
+NdbIndexOperation::getIndex() const
+{
   return m_theIndex;
 }
 
 /***************************************************************************
 int receiveTCINDXREF( NdbApiSignal* aSignal)
 
-Return Value:   Return 0 : send was successful.
-                Return -1: In all other case.
-Parameters:     aSignal: the signal object that contains the TCINDXREF signal
-from TC. Remark:         Handles the reception of the TCKEYREF signal.
+Return Value:   Return 0 : send was succesful.
+                Return -1: In all other case.   
+Parameters:     aSignal: the signal object that contains the TCINDXREF signal from TC.
+Remark:         Handles the reception of the TCKEYREF signal.
 ***************************************************************************/
-int NdbIndexOperation::receiveTCINDXREF(const NdbApiSignal *aSignal) {
+int
+NdbIndexOperation::receiveTCINDXREF(const NdbApiSignal* aSignal)
+{
   return receiveTCKEYREF(aSignal);
-}  // NdbIndexOperation::receiveTCINDXREF()
+}//NdbIndexOperation::receiveTCINDXREF()
