@@ -1,20 +1,25 @@
-/*
-  Copyright 2010 Sun Microsystems, Inc.
-  All rights reserved. Use is subject to license terms.
+/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  it under the terms of the GNU General Public License, version 2.0,
+  as published by the Free Software Foundation.
+
+  This program is also distributed with certain software (including
+  but not limited to OpenSSL) that is licensed under separate terms,
+  as designated in a particular file or component or in included license
+  documentation.  The authors of MySQL hereby grant you an additional
+  permission to link the program and your derivative works with the
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  GNU General Public License, version 2.0, for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+/*
 /*
  * MGMAPI.java
  */
@@ -141,6 +146,7 @@ import com.mysql.jtie.Wrapper;
     int CFG_DB_MICRO_GCP_TIMEOUT = 171;
     int CFG_DB_COMPRESSED_BACKUP = 172;
     int CFG_DB_COMPRESSED_LCP = 173;
+    int CFG_DB_REQUIRE_ENCRYPTED_BACKUP = 673;
     int CFG_DB_SCHED_EXEC_TIME = 174;
     int CFG_DB_SCHED_SPIN_TIME = 175;
     int CFG_DB_REALTIME_SCHEDULER = 176;
@@ -203,12 +209,6 @@ import com.mysql.jtie.Wrapper;
     int CFG_SHM_KEY = 502;
     int CFG_SHM_BUFFER_MEM = 503;
     int CFG_SHM_SIGNUM = 504;
-    int CFG_SCI_HOST1_ID_0 = 550;
-    int CFG_SCI_HOST1_ID_1 = 551;
-    int CFG_SCI_HOST2_ID_0 = 552;
-    int CFG_SCI_HOST2_ID_1 = 553;
-    int CFG_SCI_SEND_LIMIT = 554;
-    int CFG_SCI_BUFFER_MEM = 555;
     int CFG_602 = 602;
     int CFG_603 = 603;
     int CFG_604 = 604;
@@ -226,8 +226,6 @@ import com.mysql.jtie.Wrapper;
     int NODE_TYPE_MGM = 2;
     int CONNECTION_TYPE_TCP = 0;
     int CONNECTION_TYPE_SHM = 1;
-    int CONNECTION_TYPE_SCI = 2;
-    int CONNECTION_TYPE_OSE = 3;
 }
 /*!public!*/ class MGMAPI implements MGMAPI_CONFIG_PARAMTERS
 {
