@@ -1,15 +1,21 @@
 /*
- Copyright 2010 Sun Microsystems, Inc.
- All rights reserved. Use is subject to license terms.
+ Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ it under the terms of the GNU General Public License, version 2.0,
+ as published by the Free Software Foundation.
+
+ This program is also distributed with certain software (including
+ but not limited to OpenSSL) that is licensed under separate terms,
+ as designated in a particular file or component or in included license
+ documentation.  The authors of MySQL hereby grant you an additional
+ permission to link the program and your derivative works with the
+ separately licensed software that they have included with MySQL.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU General Public License, version 2.0, for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
@@ -437,7 +443,7 @@ public class MyLoadUnloadTest {
         final String cname = System.getProperty(cname_prop, "test.MyJapiTest");
         final String mname = System.getProperty(mname_prop, "test");
         out.println();
-        out.println("    using properties");
+        out.println("    settings:");
         out.println("        pprefixes = '" + pprefixes + "'");
         out.println("        cname = '" + cname + "'");
         out.println("        mname = '" + mname + "'");
@@ -461,7 +467,7 @@ public class MyLoadUnloadTest {
         // run test
         out.println();
         out.println("    load class ...");
-        Class cls = ucl.loadClass(cname);
+        Class<?> cls = ucl.loadClass(cname);
         out.println("    ... loaded " + cls
                     + " <" + cls.getClassLoader() + ">");
 
@@ -472,7 +478,7 @@ public class MyLoadUnloadTest {
 
         out.println();
         out.println("    create instance: ...");
-        Object o = cls.newInstance();
+        Object o = cls.getDeclaredConstructor().newInstance();
         out.println("    ... created instance: " + o);
         
         out.println();
