@@ -1,16 +1,15 @@
-/* Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,18 +24,20 @@
 #ifndef NDBT_CONFIG_HPP
 #define NDBT_CONFIG_HPP
 
-#include <mgmapi.h>
-#include <mgmapi_config_parameters.h>
 #include <ndb_types.h>
-#include <NdbRestarter.hpp>
+#include <mgmapi.h>
 #include <Vector.hpp>
+#include <NdbRestarter.hpp>
+#include <mgmapi_config_parameters.h>
 
 class NdbConfig : public NdbRestarter {
- public:
-  NdbConfig(const char *address = 0) : NdbRestarter(address) {}
+public:
+  NdbConfig(const char* address = 0)
+    : NdbRestarter(address)
+  {}
 
-  bool getProperty(unsigned nodeid, unsigned type, unsigned key, Uint32 *val);
-  bool getHostName(unsigned int node_id, const char **hostname);
+  bool getProperty(unsigned nodeid, unsigned type, unsigned key, Uint32 * val);
+  bool getHostName(unsigned int node_id, const char ** hostname);
 };
 
 #endif
