@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -19,7 +26,6 @@
 
 #include <NdbSleep.h>
 #include <NdbThread.h>
-#include <NdbMain.h>
 #include <NdbOut.hpp>
 #include <NdbEnv.h>
 #include <NdbTest.hpp>
@@ -87,7 +93,7 @@ static void usage(const char *prog)
 
 static
 int
-parse_args(int argc, const char **argv)
+parse_args(int argc, char **argv)
 {
    int i;
 
@@ -318,8 +324,7 @@ threadRoutine(void *arg)
   return NULL;
 }
 
-NDB_COMMAND(DbAsyncGenerator, "DbAsyncGenerator",
-	    "DbAsyncGenerator", "DbAsyncGenerator", 65535)
+int main(int argc, char** argv)
 {
   ndb_init();
   int i;
@@ -570,7 +575,7 @@ NDB_COMMAND(DbAsyncGenerator, "DbAsyncGenerator",
 			      65535, 
 			      threadName,
                               NDB_THREAD_PRIO_LOW);
-    if(pThread != 0 && pThread != NULL){
+    if(pThread){
       (&data[i*parallellism])->pThread = pThread;
     } else {      
       perror("Failed to create thread");
@@ -674,7 +679,7 @@ NDB_COMMAND(DbAsyncGenerator, "DbAsyncGenerator",
 * L O C A L   F U N C T I O N S                                *
 ***************************************************************/
 
-#ifndef NDB_WIN32
+#ifndef _WIN32
 #include <unistd.h>
 #endif
 
