@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,9 +22,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include "util/require.h"
 #include <ndb_global.h>
-#include <cstring>
 #include "NdbApi.hpp"
 #include <NdbSchemaCon.hpp>
 #include <md5_hash.hpp>
@@ -407,7 +405,7 @@ int main(int argc, char** argv)
     for (Uint32 i = 0; i<tNoOfThreads; i++)
     {
       pThreadData[i].record = (char*)malloc(sz);
-      std::memset(pThreadData[i].record, 0, sz);
+      bzero(pThreadData[i].record, sz);
     }
   }
 
@@ -1221,7 +1219,7 @@ createTables(Ndb* pMyNdb)
       {
         NdbDictionary::Index ndb_index(indexName[i][j]);
         ndb_index.setType(NdbDictionary::Index::OrderedIndex);
-        ndb_index.setLogging(false);
+        ndb_index.setLogging(FALSE);
         if (ndb_index.setTable(tableName[i]))
         {
           ndbout << "setTableError " << errno << endl;
@@ -1486,7 +1484,7 @@ init_thread_data(THREAD_DATA *my_thread_data, Uint32 thread_id)
 {
   Uint32 sz = NdbDictionary::getRecordRowLength(g_record[0]);
   my_thread_data->record = (char*)malloc(sz);
-  std::memset(my_thread_data->record, 0, sz);
+  memset(my_thread_data->record, 0, sz);
   init_list_headers(&my_thread_data->list_header, 1);
   my_thread_data->stop = false;
   my_thread_data->ready = false;
@@ -1638,11 +1636,11 @@ main_thread(RunType start_type, NdbTimer & timer)
 
   if (!insert_delete)
   {
-    NdbSleep_SecSleep(tWarmupTime);
+    sleep(tWarmupTime);
     tRunState = EXECUTING;
-    NdbSleep_SecSleep(tExecutionTime);
+    sleep(tExecutionTime);
     tRunState = COOLDOWN;
-    NdbSleep_SecSleep(tCooldownTime);
+    sleep(tCooldownTime);
     signal_definer_threads_to_stop();
   }
   wait_for_threads_ready(tNoOfDefinerThreads);
@@ -1998,7 +1996,7 @@ definer_thread(void *data)
   KEY_LIST_HEADER free_list_header;
   void *key_op_mem = malloc(sizeof(KEY_OPERATION) * max_outstanding);
   Uint32 *thread_id_mem = (Uint32*)malloc(total_records*sizeof(Uint32));
-  std::memset((char*)&thread_state[0], 0, sizeof(thread_state));
+  memset((char*)&thread_state[0], 0, sizeof(thread_state));
 
   init_key_op_list((char*)key_op_mem,
                    &free_list_header,
