@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,9 +25,6 @@
 #ifndef atrt_config_hpp
 #define atrt_config_hpp
 
-#include <time.h>
-#include "util/require.h"
-
 #include <NDBT_ReturnCodes.h>
 #include <mgmapi.h>
 #include <my_default.h>
@@ -46,6 +42,7 @@
 #include <string>
 #include "test_execution_resources.hpp"
 
+
 enum ErrorCodes {
   ERR_OK = 0,
   ERR_CRITICAL = 10,
@@ -72,7 +69,12 @@ struct TestResult {
   int result;
 };
 
-enum FailureMode : long { Restart, Abort, Skip, Continue };
+enum FailureMode : long {
+  Restart,
+  Abort,
+  Skip,
+  Continue
+};
 
 enum RestartMode : long {
   None,
@@ -81,19 +83,13 @@ enum RestartMode : long {
   Both,
 };
 
-namespace coverage {
-enum Coverage : long { None, Testcase, Testsuite };
-}
-
-enum CoverageTools : long { Lcov, Fastcov };
-
 struct atrt_host {
   unsigned m_index;
   BaseString m_user;
   BaseString m_basedir;
   BaseString m_hostname;
-  SimpleCpcClient *m_cpcd;
-  Vector<struct atrt_process *> m_processes;
+  SimpleCpcClient* m_cpcd;
+  Vector<struct atrt_process*> m_processes;
 };
 
 struct atrt_options {
@@ -109,8 +105,8 @@ struct atrt_process {
   BaseString m_name;
   unsigned int m_procno;
 
-  struct atrt_host *m_host;
-  struct atrt_cluster *m_cluster;
+  struct atrt_host* m_host;
+  struct atrt_cluster* m_cluster;
 
   enum Type {
     AP_ALL = 255,
@@ -126,11 +122,11 @@ struct atrt_process {
   SimpleCpcClient::Process m_proc;
   bool m_atrt_stopped;
 
-  NdbMgmHandle m_ndb_mgm_handle;     // if type == ndb_mgm
-  atrt_process *m_mysqld;            // if type == client
-  atrt_process *m_rep_src;           // if type == mysqld
-  Vector<atrt_process *> m_rep_dst;  // if type == mysqld
-  MYSQL m_mysql;                     // if type == mysqld
+  NdbMgmHandle m_ndb_mgm_handle;    // if type == ndb_mgm
+  atrt_process* m_mysqld;           // if type == client
+  atrt_process* m_rep_src;          // if type == mysqld
+  Vector<atrt_process*> m_rep_dst;  // if type == mysqld
+  MYSQL m_mysql;                    // if type == mysqld
   atrt_options m_options;
   uint m_nodeid;  // if m_fix_nodeid
 
@@ -143,7 +139,7 @@ struct atrt_process {
 struct atrt_cluster {
   BaseString m_name;
   BaseString m_dir;
-  Vector<atrt_process *> m_processes;
+  Vector<atrt_process*> m_processes;
   atrt_options m_options;
   uint m_next_nodeid;  // if m_fix_nodeid
 };
@@ -154,15 +150,15 @@ struct atrt_config {
   BaseString m_key;
   BaseString m_replication;
   BaseString m_site;
-  Vector<atrt_host *> m_hosts;
-  Vector<atrt_cluster *> m_clusters;
-  Vector<atrt_process *> m_processes;
+  Vector<atrt_host*> m_hosts;
+  Vector<atrt_cluster*> m_clusters;
+  Vector<atrt_process*> m_processes;
 };
 
 struct atrt_coverage_config {
-  int m_prefix_strip;
-  coverage::Coverage m_analysis;
-  CoverageTools m_tool;
+  int m_coverage_prefix_strip;
+  BaseString m_lcov_files_dir;
+  bool m_coverage;
 };
 
 struct atrt_testcase {
@@ -185,44 +181,44 @@ struct atrt_testcase {
 
 extern Logger g_logger;
 
-bool parse_args(int argc, char **argv, MEM_ROOT *alloc);
-bool setup_config(atrt_config &, atrt_coverage_config &, const char *mysqld,
+bool parse_args(int argc, char** argv, MEM_ROOT* alloc);
+bool setup_config(atrt_config&, atrt_coverage_config&, const char* mysqld,
                   bool);
-void setup_coverage_config(atrt_coverage_config &);
-void get_coverage_parameters(atrt_coverage_config &);
-bool load_deployment_options(atrt_config &);
-bool configure(atrt_config &, int setup);
-bool setup_directories(atrt_config &, int setup);
-bool setup_files(atrt_config &, int setup, int sshx);
+void setup_coverage_config(atrt_coverage_config&);
+void get_coverage_parameters(atrt_coverage_config&);
+bool load_deployment_options(atrt_config&);
+bool configure(atrt_config&, int setup);
+bool setup_directories(atrt_config&, int setup);
+bool setup_files(atrt_config&, int setup, int sshx);
 
-bool deploy(int, atrt_config &);
-bool sshx(atrt_config &, unsigned procmask);
+bool deploy(int, atrt_config&);
+bool sshx(atrt_config&, unsigned procmask);
 
-bool remove_dir(const char *, bool incl = true);
-bool exists_file(const char *path);
-bool connect_hosts(atrt_config &);
-const char *get_test_status(int result);
+bool remove_dir(const char*, bool incl = true);
+bool exists_file(const char* path);
+bool connect_hosts(atrt_config&);
+const char* get_test_status(int result);
 int atrt_exit(int return_code);
-void update_atrt_result_code(const TestResult &test_result,
-                             AtrtExitCodes *return_code);
+void update_atrt_result_code(const TestResult& test_result,
+                             AtrtExitCodes* return_code);
 
-bool is_client_running(atrt_config &);
-bool gather_result(atrt_config &, int *result);
+bool is_client_running(atrt_config&);
+bool gather_result(atrt_config&, int* result);
 
-int read_test_case(FILE *, int &line, atrt_testcase &);
-bool read_test_cases(FILE *, std::vector<atrt_testcase> *);
+int read_test_case(FILE*, int& line, atrt_testcase&);
+bool read_test_cases(FILE*, std::vector<atrt_testcase>*);
 
-bool setup_hosts(atrt_config &);
+bool setup_hosts(atrt_config&);
 
-bool connect_mysqld(atrt_process &proc);
-bool disconnect_mysqld(atrt_process &proc);
+bool connect_mysqld(atrt_process& proc);
+bool disconnect_mysqld(atrt_process& proc);
 
-NdbOut &operator<<(NdbOut &out, const atrt_process &proc);
+NdbOut& operator<<(NdbOut& out, const atrt_process& proc);
 
 /**
  * SQL
  */
-bool setup_db(atrt_config &);
+bool setup_db(atrt_config&);
 
 std::string getAtrtVersion();
 
@@ -231,35 +227,35 @@ std::string getAtrtVersion();
  */
 extern Logger g_logger;
 
-extern const char *g_cwd;
-extern const char *g_my_cnf;
-extern const char *g_user;
-extern const char *g_basedir;
-extern const char *g_prefix;
-extern const char *g_prefix0;
-extern const char *g_prefix1;
+extern const char* g_cwd;
+extern const char* g_my_cnf;
+extern const char* g_user;
+extern const char* g_basedir;
+extern const char* g_prefix;
+extern const char* g_prefix0;
+extern const char* g_prefix1;
 extern int g_baseport;
 extern int g_fqpn;
 extern int g_fix_nodeid;
 extern int g_default_ports;
 extern int g_restart;
 
-extern const char *g_site;
-extern const char *g_clusters;
+extern const char* g_site;
+extern const char* g_clusters;
 
 /**
  * Since binaries move location between 5.1 and 5.5
  *   we keep full path to them here
  */
-char *find_bin_path(const char *basename);
-char *find_bin_path(const char *prefix, const char *basename);
-extern const char *g_search_path[];
+char* find_bin_path(const char* basename);
+char* find_bin_path(const char* prefix, const char* basename);
+extern const char* g_search_path[];
 extern TestExecutionResources g_resources;
 
 #ifdef _WIN32
 #include <direct.h>
 
-inline int lstat(const char *name, struct stat *buf) { return stat(name, buf); }
+inline int lstat(const char* name, struct stat* buf) { return stat(name, buf); }
 
 inline int S_ISREG(int x) { return x & _S_IFREG; }
 
@@ -268,7 +264,7 @@ inline int S_ISDIR(int x) { return x & _S_IFDIR; }
 #endif
 
 /* in-place replace */
-static inline char *replace_chars(char *str, char from, char to) {
+static inline char* replace_chars(char* str, char from, char to) {
   int i;
 
   for (i = 0; str[i]; i++) {
@@ -279,26 +275,26 @@ static inline char *replace_chars(char *str, char from, char to) {
   return str;
 }
 
-static inline BaseString &replace_chars(BaseString &bs, char from, char to) {
-  replace_chars((char *)bs.c_str(), from, to);
+static inline BaseString& replace_chars(BaseString& bs, char from, char to) {
+  replace_chars((char*)bs.c_str(), from, to);
   return bs;
 }
-static inline BaseString &to_native(BaseString &bs) {
+static inline BaseString& to_native(BaseString& bs) {
   return replace_chars(bs, DIR_SEPARATOR[0] == '/' ? '\\' : '/',
                        DIR_SEPARATOR[0]);
 }
-static inline BaseString &to_fwd_slashes(BaseString &bs) {
+static inline BaseString& to_fwd_slashes(BaseString& bs) {
   return replace_chars(bs, '\\', '/');
 }
-static inline char *to_fwd_slashes(char *bs) {
+static inline char* to_fwd_slashes(char* bs) {
   return replace_chars(bs, '\\', '/');
 }
 
 // you must free() the result
-static inline char *replace_drive_letters(const char *path) {
+static inline char* replace_drive_letters(const char* path) {
   int i, j;
   int count;     // number of ':'s in path
-  char *retval;  // return value
+  char* retval;  // return value
   const char cygdrive[] = "/cygdrive";
   size_t cyglen = strlen(cygdrive), retval_len;
 
@@ -306,7 +302,7 @@ static inline char *replace_drive_letters(const char *path) {
     count += path[i] == ':';
   }
   retval_len = strlen(path) + count * cyglen + 1;
-  retval = (char *)malloc(retval_len);
+  retval = (char*)malloc(retval_len);
 
   for (i = j = 0; path[i]; i++) {
     if (path[i] && path[i + 1]) {
@@ -326,7 +322,7 @@ static inline char *replace_drive_letters(const char *path) {
   return retval;
 }
 
-static inline int sh(const char *script) {
+static inline int sh(const char* script) {
 #ifdef _WIN32
   g_logger.debug("sh('%s')", script);
 
@@ -348,7 +344,7 @@ static inline int sh(const char *script) {
     return -1;
   }
 
-  FILE *fp = fopen(tmp_file, "w");
+  FILE* fp = fopen(tmp_file, "w");
   if (fp == NULL) {
     g_logger.error("Cannot open file '%s', error: %d", tmp_file, errno);
     return -1;
@@ -356,7 +352,7 @@ static inline int sh(const char *script) {
 
   // cygwin'ify the script and write it to temp file
   {
-    char *cygwin_script = replace_drive_letters(script);
+    char* cygwin_script = replace_drive_letters(script);
     g_logger.debug(" - cygwin_script: '%s' ", cygwin_script);
     fprintf(fp, "%s", cygwin_script);
     free(cygwin_script);
