@@ -22,6 +22,8 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#include <algorithm>
+
 #include "Restore.hpp"
 #include <NdbTCP.h>
 #include <OutputStream.hpp>
@@ -39,6 +41,13 @@
 #include "restore_tables.h"
 #include <NdbThread.h>
 #include "../src/kernel/vm/Emulator.hpp"
+#include "kernel/signaldata/FsOpenReq.hpp"
+#include "portlib/ndb_file.h"
+#include "portlib/NdbMem.h"
+
+//#define DUMMY_PASSWORD
+
+using byte = unsigned char;
 
 extern thread_local EmulatedJamBuffer* NDB_THREAD_TLS_JAM;
 
@@ -47,6 +56,8 @@ extern bool ga_skip_unknown_objects;
 extern bool ga_skip_broken_objects;
 extern bool opt_include_stored_grants;
 
+extern char* g_backup_password;
+
 #define LOG_MSGLEN 1024
 
 Uint16 Twiddle16(Uint16 in); // Byte shift 16-bit data
@@ -387,7 +398,9 @@ RestoreMetaData::readMetaTableList() {
   
   Uint32 sectionInfo[2];
   
-  if (buffer_read(&sectionInfo, sizeof(sectionInfo), 1) != 1){
+  int r = buffer_read(&sectionInfo, sizeof(sectionInfo), 1);
+  if (r != 1)
+  {
     restoreLogger.log_error("readMetaTableList read header error");
     return 0;
   }
@@ -400,6 +413,18 @@ RestoreMetaData::readMetaTableList() {
   Uint32 tabsRead = 0;
   while (tabsRead < tabCount){
     int count = buffer_get_ptr(&tmp, 4, tabCount - tabsRead);
+    if (count < 0)
+    {
+      if (!m_xfile.is_encrypted())
+      {
+        restoreLogger.log_error("readMetaTableList read tabCount error, corrupt file?");
+      }
+      else
+      {
+        restoreLogger.log_error("readMetaTableList read tabCount error, wrong password or corrupt file?");
+      }
+      return 0;
+    }
     if(count == 0)
       break;
     tabsRead += count;
@@ -433,7 +458,9 @@ RestoreMetaData::readMetaTableDesc() {
     sz = 2;
     sectionInfo[2] = htonl(DictTabInfo::UserTable);
   }
-  if (buffer_read(&sectionInfo, 4*sz, 1) != 1){
+  int r = buffer_read(&sectionInfo, 4*sz, 1);
+  if (r != 1)
+  {
     restoreLogger.log_error("readMetaTableDesc read header error");
     return false;
   } // if
@@ -446,7 +473,9 @@ RestoreMetaData::readMetaTableDesc() {
   // Read dictTabInfo buffer
   const Uint32 len = (sectionInfo[1] - sz);
   void *ptr;
-  if (buffer_get_ptr(&ptr, 4, len) != len){
+  r = buffer_get_ptr(&ptr, 4, len);
+  if (r < 0 || Uint32(r) != len)
+  {
     restoreLogger.log_error("readMetaTableDesc read error");
     return false;
   } // if
@@ -722,12 +751,15 @@ RestoreMetaData::fixBlobs()
         }
       }
       assert(blobTable->m_dictTable != NULL);
+      assert(blobTable->m_blobTables.size() == 0);
       NdbTableImpl& bt = NdbTableImpl::getImpl(*blobTable->m_dictTable);
       const char* colName = c->m_blobVersion == 1 ? "DATA" : "NDB$DATA";
       const NdbColumnImpl* bc = bt.getColumn(colName);
       assert(bc != NULL);
       assert(c->m_storageType == NDB_STORAGETYPE_MEMORY);
       c->m_storageType = bc->m_storageType;
+
+      table->m_blobTables.push_back(blobTable);
     }
   }
   return true;
@@ -738,7 +770,9 @@ RestoreMetaData::readGCPEntry() {
 
   BackupFormat::CtlFile::GCPEntry dst;
   
-  if(buffer_read(&dst, 1, sizeof(dst)) != sizeof(dst)){
+  int r = buffer_read(&dst, 1, sizeof(dst));
+  if(r < 0 || size_t(r) != sizeof(dst))
+  {
     restoreLogger.log_error("readGCPEntry read error");
     return false;
   }
@@ -771,7 +805,8 @@ RestoreMetaData::readFragmentInfo()
   TableS * table = 0;
   Uint32 tableId = RNIL;
 
-  while (buffer_read(&fragInfo, 4, 2) == 2)
+  int r;
+  while ((r = buffer_read(&fragInfo, 4, 2)) == 2)
   {
     fragInfo.SectionType = ntohl(fragInfo.SectionType);
     fragInfo.SectionLength = ntohl(fragInfo.SectionLength);
@@ -783,7 +818,8 @@ RestoreMetaData::readFragmentInfo()
       return false;
     }
 
-    if (buffer_read(&fragInfo.TableId, (fragInfo.SectionLength-2)*4, 1) != 1)
+    int r = buffer_read(&fragInfo.TableId, (fragInfo.SectionLength-2)*4, 1);
+    if (r != 1)
     {
       restoreLogger.log_error("readFragmentInfo invalid section length: %u",
         fragInfo.SectionLength);
@@ -808,6 +844,11 @@ RestoreMetaData::readFragmentInfo()
     table->m_fragmentInfo.push_back(tmp);
     table->m_noOfRecords += tmp->noOfRecords;
   }
+  if (r < 0)
+  {
+    restoreLogger.log_error("readFragmentInfo invalid read");
+    return false;
+  }
   return true;
 }
 
@@ -832,6 +873,9 @@ TableS::TableS(Uint32 version, NdbTableImpl* tableImpl)
   m_staging = false;
   m_stagingTable = NULL;
   m_stagingFlags = 0;
+
+  m_pk_extended = false;
+  m_pk_index = NULL;
 }
 
 TableS::~TableS()
@@ -892,24 +936,23 @@ RestoreMetaData::parseTableDescriptor(const Uint32 * data, Uint32 len)
 
 // Constructor
 RestoreDataIterator::RestoreDataIterator(const RestoreMetaData & md, void (* _free_data_callback)(void*), void *ctx)
-  : BackupFile(_free_data_callback, ctx), m_metaData(md)
+  : BackupFile(_free_data_callback, ctx), m_metaData(md),
+    m_current_table_has_transforms(false)
 {
   restoreLogger.log_debug("RestoreDataIterator constructor");
   setDataFile(md, 0);
 
-  m_bitfield_storage_len = 8192;
-  m_bitfield_storage_ptr = (Uint32*)malloc(4*m_bitfield_storage_len);
-  m_bitfield_storage_curr_ptr = m_bitfield_storage_ptr;
-  m_row_bitfield_len = 0;
+  alloc_extra_storage(8192);
+  m_row_max_extra_wordlen = 0;
 }
 
 
 bool
 RestoreDataIterator::validateRestoreDataIterator()
 {
-    if (!m_bitfield_storage_ptr)
+    if (!m_extra_storage_ptr)
     {
-        restoreLogger.log_error("m_bitfield_storage_ptr is NULL");
+        restoreLogger.log_error("m_extra_storage_ptr is NULL");
         return false;
     }
     return true;
@@ -918,58 +961,107 @@ RestoreDataIterator::validateRestoreDataIterator()
 
 RestoreDataIterator::~RestoreDataIterator()
 {
-  free_bitfield_storage();
+  free_extra_storage();
 }
 
 void
-RestoreDataIterator::init_bitfield_storage(const NdbDictionary::Table* tab)
+RestoreDataIterator::calc_row_extra_storage_words(const TableS* tableSpec)
 {
-  Uint32 len = 0;
+  const NdbDictionary::Table* tab = tableSpec->m_dictTable;
+  Uint32 bitmap_words = 0;
+  Uint32 transform_words = 0;
   for (Uint32 i = 0; i<(Uint32)tab->getNoOfColumns(); i++)
   {
+    /* Space for bitmap-copy out from PACKED format */
     if (tab->getColumn(i)->getType() == NdbDictionary::Column::Bit)
     {
-      len += (tab->getColumn(i)->getLength() + 31) >> 5;
+      bitmap_words += (tab->getColumn(i)->getLength() + 31) >> 5;
+    }
+    /* Space for output from this column transform */
+    const AttributeDesc* attr_desc = tableSpec->getAttributeDesc(i);
+    if (attr_desc->transform != NULL)
+    {
+      transform_words += attr_desc->getSizeInWords();
     }
   }
 
-  m_row_bitfield_len = len;
+  m_current_table_has_transforms = (transform_words > 0);
+
+  m_row_max_extra_wordlen = bitmap_words + transform_words;
 }
 
 void
-RestoreDataIterator::reset_bitfield_storage()
+RestoreDataIterator::reset_extra_storage()
 {
-  m_bitfield_storage_curr_ptr = m_bitfield_storage_ptr;
+  m_extra_storage_curr_ptr = m_extra_storage_ptr;
 }
 
 void
-RestoreDataIterator::free_bitfield_storage()
+RestoreDataIterator::alloc_extra_storage(Uint32 words)
 {
-  if (m_bitfield_storage_ptr)
-    free(m_bitfield_storage_ptr);
-  m_bitfield_storage_ptr = 0;
-  m_bitfield_storage_curr_ptr = 0;
-  m_bitfield_storage_len = 0;
+  m_extra_storage_wordlen = words;
+  m_extra_storage_ptr = (Uint32*)malloc(4 * words);
+  m_extra_storage_curr_ptr = m_extra_storage_ptr;
+}
+
+void
+RestoreDataIterator::free_extra_storage()
+{
+  if (m_extra_storage_ptr)
+    free(m_extra_storage_ptr);
+  m_extra_storage_ptr = 0;
+  m_extra_storage_curr_ptr = 0;
 }
 
 Uint32
-RestoreDataIterator::get_free_bitfield_storage() const
+RestoreDataIterator::get_free_extra_storage() const
 {
-  
-  return Uint32((m_bitfield_storage_ptr + m_bitfield_storage_len) - 
-    m_bitfield_storage_curr_ptr);
+
+  return Uint32((m_extra_storage_ptr + m_extra_storage_wordlen) -
+    m_extra_storage_curr_ptr);
 }
 
-Uint32*
-RestoreDataIterator::get_bitfield_storage(Uint32 len)
+void
+RestoreDataIterator::check_extra_storage()
 {
-  Uint32 * currptr = m_bitfield_storage_curr_ptr;
+  assert(m_row_max_extra_wordlen <= m_extra_storage_wordlen);
+  if (m_row_max_extra_wordlen >= get_free_extra_storage())
+  {
+    /**
+     * No more space available to buffer rows, flush
+     * what is outstanding, then reset buffers and
+     * continue.
+     */
+    flush_and_reset_buffers();
+    assert(get_free_extra_storage() > m_row_max_extra_wordlen);
+    assert(m_extra_storage_ptr == m_extra_storage_curr_ptr);
+
+    /**
+     * We do not want to break up batching due to a lack of
+     * extra buffer storage, but that is what has happened
+     * here.
+     * So to avoid this in future we will take this chance
+     * to double the extra storage size, so that batching
+     * boundaries are eventually controlled by the file
+     * buffering only.
+     */
+    const Uint32 newWords = m_extra_storage_wordlen * 2;
+    free_extra_storage();
+    alloc_extra_storage(newWords);
+  }
+}
+
+
+Uint32*
+RestoreDataIterator::get_extra_storage(Uint32 len)
+{
+  Uint32 * currptr = m_extra_storage_curr_ptr;
   Uint32 * nextptr = currptr + len;
-  Uint32 * endptr = m_bitfield_storage_ptr + m_bitfield_storage_len;
+  Uint32 * endptr = m_extra_storage_ptr + m_extra_storage_wordlen;
 
   if (nextptr <= endptr)
   {
-    m_bitfield_storage_curr_ptr = nextptr;
+    m_extra_storage_curr_ptr = nextptr;
     return currptr;
   }
   
@@ -1049,29 +1141,72 @@ charpad:
   }
 }
 
+bool
+applyColumnTransform(const NdbDictionary::Column* col,
+                     const AttributeDesc* attr_desc,
+                     AttributeData* attr_data,
+                     void* dst_buf)
+{
+  assert(attr_desc->transform != NULL);
+
+  void* src_ptr = (attr_data->null? NULL : attr_data->void_value);
+  void* dst_ptr = dst_buf;
+
+  if (!attr_desc->transform->apply(col,
+                                   src_ptr,
+                                   &dst_ptr))
+  {
+    return false;
+  }
+
+  if (dst_ptr == NULL)
+  {
+    assert(col->getNullable());
+    attr_data->null = true;
+    attr_data->size = 0;
+    attr_data->void_value = NULL;
+  }
+  else
+  {
+    const uchar* dst_char = (const uchar*) dst_ptr;
+    attr_data->null = false;
+    attr_data->void_value = dst_ptr;
+    switch(col->getArrayType())
+    {
+    case NDB_ARRAYTYPE_SHORT_VAR:
+      attr_data->size = 1 + size_t(dst_char[0]);
+      break;
+    case NDB_ARRAYTYPE_MEDIUM_VAR:
+      attr_data->size = 2 + size_t(dst_char[0])
+        + (256 * size_t(dst_char[1]));
+      break;
+    default:
+      /* No change */
+      break;
+    }
+  }
+
+  /* Check size is within 'word length' of column type */
+  assert(attr_data->size <=
+         4 * ((((Uint32)col->getSizeInBytes()) +3)/4));
+
+  return true;
+}
+
+
 const TupleS *
 RestoreDataIterator::getNextTuple(int  & res, const bool skipFragment)
 {
-  if (m_currentTable->backupVersion >= NDBD_RAW_LCP)
-  {
-    if (m_row_bitfield_len >= get_free_bitfield_storage())
-    {
-      /**
-       * Informing buffer reader that it does not need to cache
-       * "old" data here would be clever...
-       * But I can't find a good/easy way to do this
-       */
-      if (free_data_callback)
-        (*free_data_callback)(m_ctx);
-      reset_bitfield_storage();
-    }
-  }
-  
+  /* Check that we have space to return another tuple */
+  check_extra_storage();
+
   while (true)
   {
     Uint32  dataLength = 0;
     // Read record length
-    if (buffer_read(&dataLength, sizeof(dataLength), 1) != 1){
+    int r = buffer_read(&dataLength, sizeof(dataLength), 1);
+    if (r != 1)
+    {
       restoreLogger.log_error("getNextTuple:Error reading length of data part");
       res = -1;
       return NULL;
@@ -1091,7 +1226,9 @@ RestoreDataIterator::getNextTuple(int  & res, const bool skipFragment)
 
     // Read tuple data
     void *_buf_ptr;
-    if (buffer_get_ptr(&_buf_ptr, 1, dataLenBytes) != dataLenBytes) {
+    r = buffer_get_ptr(&_buf_ptr, 1, dataLenBytes);
+    if (r < 0 || Uint32(r) != dataLenBytes)
+    {
       restoreLogger.log_error("getNextTuple:Read error: ");
       res = -1;
       return NULL;
@@ -1123,6 +1260,31 @@ RestoreDataIterator::getNextTuple(int  & res, const bool skipFragment)
       return NULL;
     }
 
+    /* Apply column transforms if the table has any defined */
+    if (m_current_table_has_transforms)
+    {
+      for (int i=0; i < m_currentTable->getNoOfAttributes(); i++)
+      {
+        const AttributeDesc* attr_desc = m_currentTable->getAttributeDesc(i);
+        if (attr_desc->transform == NULL)
+        {
+          continue;
+        }
+        const NdbDictionary::Column* col = m_currentTable->m_dictTable->getColumn(i);
+        void* dst_buf = get_extra_storage(attr_desc->getSizeInWords());
+        assert(dst_buf != NULL);
+
+        if (!applyColumnTransform(col,
+                                  attr_desc,
+                                  m_tuple.getData(i),
+                                  dst_buf))
+        {
+          res = -1;
+          return NULL;
+        }
+      }
+    }
+
     res = 0;
     return &m_tuple;
   }
@@ -1211,7 +1373,7 @@ RestoreDataIterator::readTupleData_packed(Uint32 *buf_ptr,
       Uint32* src32 = (Uint32*)src;
       
       Uint32 len32 = (len + 31) >> 5;
-      Uint32* tmp = get_bitfield_storage(len32);
+      Uint32* tmp = get_extra_storage(len32);
       attr_data->null = false;
       attr_data->void_value = tmp;
       attr_data->size = 4*len32;
@@ -1443,7 +1605,6 @@ RestoreDataIterator::readVarData_drop6(Uint32 *buf_ptr, Uint32 *ptr,
 BackupFile::BackupFile(void (* _free_data_callback)(void*), void *ctx)
   : free_data_callback(_free_data_callback), m_ctx(ctx)
 {
-  memset(&m_file,0,sizeof(m_file));
   m_path[0] = 0;
   m_fileName[0] = 0;
 
@@ -1475,27 +1636,44 @@ BackupFile::validateBackupFile()
 
 BackupFile::~BackupFile()
 {
-  (void)ndbzclose(&m_file);
+  int r = 0;
+  if (m_xfile.is_open())
+  {
+    r = m_xfile.close();
+  }
+
+  if (m_file.close() == -1)
+  {
+    r = -1;
+  }
+
+  if (r == -1)
+  {
+    restoreLogger.log_error("Warning: File did not close correctly.");
+  }
 
   if(m_buffer != 0)
+  {
     free(m_buffer);
+  }
 }
 
 bool
 BackupFile::openFile(){
-  (void)ndbzclose(&m_file);
+  int r;
   m_file_size = 0;
   m_file_pos = 0;
 
   info.setLevel(254);
   restoreLogger.log_info("Opening file '%s'", m_fileName);
-  int r= ndbzopen(&m_file, m_fileName, O_RDONLY);
+  r = m_file.open(m_fileName, FsOpenReq::OM_READONLY);
 
-  if(r != 1)
+  if(r == -1) {
     return false;
+  }
 
-  size_t size;
-  if (ndbz_file_size(&m_file, &size) == 0)
+  long long size = m_file.get_size();
+  if (size != -1)
   {
     m_file_size = (Uint64)size;
     restoreLogger.log_info("File size %llu bytes", m_file_size);
@@ -1507,25 +1685,66 @@ BackupFile::openFile(){
     m_file_size = 0;
   }
 
-  return true;
+#if !defined(DUMMY_PASSWORD)
+  r = m_xfile.open(m_file,
+                   reinterpret_cast<const byte*>(g_backup_password),
+                   g_backup_password ? strlen(g_backup_password) : 0);
+#else
+  r = m_xfile.open(m_file, reinterpret_cast<const byte*>("DUMMY"), 5);
+#endif
+  bool fail = (r == -1);
+  if (g_backup_password != nullptr)
+  {
+    if (!m_xfile.is_encrypted())
+    {
+      restoreLogger.log_error("Decryption requested but file not encrypted.");
+      fail = true;
+    }
+    else if (r == -1)
+    {
+      restoreLogger.log_error("Can not read decrypted file. Might be wrong password.");
+    }
+  }
+  else
+  {
+    if (m_xfile.is_encrypted())
+    {
+#if !defined(DUMMY_PASSWORD)
+      restoreLogger.log_error("File is encrypted but no decryption requested.");
+      fail = true;
+#endif
+    }
+    else if (r == -1)
+    {
+      restoreLogger.log_error("Can not read file. Might be corrupt.");
+    }
+  }
+
+  if (!fail)
+  {
+    return true;
+  }
+
+  if (r != -1)
+  {
+    m_xfile.close();
+  }
+  m_file.close();
+  return false;
 }
 
-Uint32 BackupFile::buffer_get_ptr_ahead(void **p_buf_ptr, Uint32 size, Uint32 nmemb)
+int BackupFile::buffer_get_ptr_ahead(void **p_buf_ptr, Uint32 size, Uint32 nmemb)
 {
   Uint32 sz = size*nmemb;
   if (sz > m_buffer_data_left) {
 
-    if (free_data_callback)
-      (*free_data_callback)(m_ctx);
-
-    reset_buffers();
+    flush_and_reset_buffers();
 
     if (m_is_undolog)
     {
       /* move the left data to the end of buffer
        */
-      size_t r = 0;
-      int error;
+      int r = 0;
       /* move the left data to the end of buffer
        * m_buffer_ptr point the end of the left data. buffer_data_start point the start of left data
        * m_buffer_data_left is the length of left data.
@@ -1570,24 +1789,24 @@ Uint32 BackupFile::buffer_get_ptr_ahead(void **p_buf_ptr, Uint32 size, Uint32 nm
 	file_left_entry_data = m_file_pos - sizeof(m_fileHeader);
         if (file_left_entry_data <= buffer_free_space)
         {
-          /* All remaining data fits in space available in buffer. 
-	   * Read data into buffer before existing data.
-	   */
-          // Move to the start of data to be read
-          ndbzseek(&m_file, sizeof(m_fileHeader), SEEK_SET);
-          r = ndbzread(&m_file,
-                       (char *)buffer_data_start - file_left_entry_data,
-                       Uint32(file_left_entry_data),
-                       &error);
-          //move back
-          ndbzseek(&m_file, sizeof(m_fileHeader), SEEK_SET);
+          ndbxfrm_output_reverse_iterator out((unsigned char*)buffer_data_start, (unsigned char*)buffer_data_start - file_left_entry_data, false);
+          byte* out_beg = out.begin();
+          r = m_xfile.read_backward(&out);
+          if (r == -1)
+            abort(); // TODO
+          r = out_beg - out.begin();
+          require(size_t(r) == file_left_entry_data);
         }
         else
         {
 	  // Fill remaing space at start of buffer with data from file.
-          ndbzseek(&m_file, m_file_pos-buffer_free_space, SEEK_SET);
-          r = ndbzread(&m_file, ((char *)m_buffer), buffer_free_space, &error);
-          ndbzseek(&m_file, m_file_pos-buffer_free_space, SEEK_SET);
+          ndbxfrm_output_reverse_iterator out((unsigned char*)m_buffer + buffer_free_space, (unsigned char*)m_buffer, false);
+          byte* out_beg = out.begin();
+          r = m_xfile.read_backward(&out);
+          if (r == -1)
+            abort(); // TODO
+          r = out_beg - out.begin();
+          require(size_t(r) == buffer_free_space);
         }
       }
       m_file_pos -= r;
@@ -1598,10 +1817,15 @@ Uint32 BackupFile::buffer_get_ptr_ahead(void **p_buf_ptr, Uint32 size, Uint32 nm
     else
     {
       memmove(m_buffer, m_buffer_ptr, m_buffer_data_left);
-      int error;
-      Uint32 r = ndbzread(&m_file,
-                          ((char *)m_buffer) + m_buffer_data_left,
-                          m_buffer_sz - m_buffer_data_left, &error);
+      ndbxfrm_output_iterator it = { static_cast<byte*>(m_buffer) + m_buffer_data_left,
+                                     static_cast<byte*>(m_buffer) + m_buffer_sz,
+                                     false };
+      int r = m_xfile.read_forward(&it);
+      if (r < 0)
+      {
+        return r;
+      }
+      r = it.begin() - (static_cast<byte*>(m_buffer) + m_buffer_data_left);
       m_file_pos += r;
       m_buffer_data_left += r;
       m_buffer_ptr = m_buffer;
@@ -1617,15 +1841,24 @@ Uint32 BackupFile::buffer_get_ptr_ahead(void **p_buf_ptr, Uint32 size, Uint32 nm
    *   So we should move m_buffer_ptr to the right place.
    */
   if(m_is_undolog)
+  {
     *p_buf_ptr = (char *)m_buffer_ptr - sz;
+  }
   else
+  {
     *p_buf_ptr = m_buffer_ptr;
+  }
 
+  require(sz % size == 0);
   return sz/size;
 }
-Uint32 BackupFile::buffer_get_ptr(void **p_buf_ptr, Uint32 size, Uint32 nmemb)
+int BackupFile::buffer_get_ptr(void **p_buf_ptr, Uint32 size, Uint32 nmemb)
 {
-  Uint32 r = buffer_get_ptr_ahead(p_buf_ptr, size, nmemb);
+  int r = buffer_get_ptr_ahead(p_buf_ptr, size, nmemb);
+  if (r < 0)
+  {
+    return r;
+  }
 
   if(m_is_undolog)
   {
@@ -1644,21 +1877,28 @@ Uint32 BackupFile::buffer_get_ptr(void **p_buf_ptr, Uint32 size, Uint32 nmemb)
   return r;
 }
 
-Uint32 BackupFile::buffer_read_ahead(void *ptr, Uint32 size, Uint32 nmemb)
+int BackupFile::buffer_read_ahead(void *ptr, Uint32 size, Uint32 nmemb)
 {
   void *buf_ptr;
-  Uint32 r = buffer_get_ptr_ahead(&buf_ptr, size, nmemb);
+  int r = buffer_get_ptr_ahead(&buf_ptr, size, nmemb);
+  if (r < 0)
+  {
+    return r;
+  }
   memcpy(ptr, buf_ptr, r*size);
 
   return r;
 }
 
-Uint32 BackupFile::buffer_read(void *ptr, Uint32 size, Uint32 nmemb)
+int BackupFile::buffer_read(void *ptr, Uint32 size, Uint32 nmemb)
 {
   void *buf_ptr;
-  Uint32 r = buffer_get_ptr(&buf_ptr, size, nmemb);
+  int r = buffer_get_ptr(&buf_ptr, size, nmemb);
+  if (r < 0)
+  {
+    return r;
+  }
   memcpy(ptr, buf_ptr, r*size);
-
   return r;
 }
 
@@ -1750,8 +1990,17 @@ BackupFile::readHeader(){
   }
   
   Uint32 oldsz = sizeof(BackupFormat::FileHeader_pre_backup_version);
-  if(buffer_read(&m_fileHeader, oldsz, 1) != 1){
-    restoreLogger.log_error("readDataFileHeader: Error reading header");
+  int r = buffer_read(&m_fileHeader, oldsz, 1);
+  if(r != 1)
+  {
+    if (!m_xfile.is_encrypted())
+    {
+      restoreLogger.log_error("readDataFileHeader: Error reading header");
+    }
+    else
+    {
+      restoreLogger.log_error("readDataFileHeader: Error reading header. Wrong password?");
+    }
     return false;
   }
   
@@ -1773,10 +2022,18 @@ BackupFile::readHeader(){
 
   if (backup_version >= NDBD_RAW_LCP)
   {
-    if (buffer_read(&m_fileHeader.NdbVersion, 
-                    sizeof(m_fileHeader) - oldsz, 1) != 1)
+    int r = buffer_read(&m_fileHeader.NdbVersion,
+                        sizeof(m_fileHeader) - oldsz, 1);
+    if (r != 1)
     {
-      restoreLogger.log_error("readDataFileHeader: Error reading header");
+      if (!m_xfile.is_encrypted())
+      {
+        restoreLogger.log_error("readDataFileHeader: Error reading header");
+      }
+      else
+      {
+        restoreLogger.log_error("readDataFileHeader: Error reading header. Wrong password?");
+      }
       return false;
     }
     
@@ -1811,20 +2068,47 @@ BackupFile::readHeader(){
     abort();
   }
   
-  if(m_fileHeader.FileType == BackupFormat::UNDO_FILE){
-      m_is_undolog = true;
-      /* move pointer to end of data part. 
-         move 4 bytes from the end of file 
-         because footer contain 4 bytes 0 at the end of file.
-         we discard the remain data stored in m_buffer.
-      */
-      size_t size;
-      if (ndbz_file_size(&m_file, &size) == 0)
-        m_file_size = (Uint64)size;
-      ndbzseek(&m_file, 4, SEEK_END);
-      m_file_pos = m_file_size - 4;
-      m_buffer_data_left = 0;
-      m_buffer_ptr = m_buffer;
+  if(m_fileHeader.FileType == BackupFormat::UNDO_FILE)
+  {
+    m_is_undolog = true;
+    /* move pointer to end of data part. 
+       move 4 bytes from the end of file 
+       because footer contain 4 bytes 0 at the end of file.
+       we discard the remain data stored in m_buffer.
+    */
+    m_file_size = m_xfile.move_to_end();
+    m_file_pos = m_file_size; // TODO??
+    require((int)m_file_size != -1);
+
+    size_t sz = m_buffer_sz;
+    if (m_file_size - sizeof(m_fileHeader) < sz)
+    {
+      sz = m_file_size - sizeof(m_fileHeader);
+    }
+    ndbxfrm_output_reverse_iterator out((unsigned char*)m_buffer + m_buffer_sz, (unsigned char*)m_buffer + m_buffer_sz - sz, false);
+    byte* out_beg = out.begin();
+    int r = m_xfile.read_backward(&out);
+    if (r == -1)
+    {
+      abort(); // TODO
+    }
+    int n = out_beg - out.begin();
+    require(n >= 4);
+    uint32_t last_word;
+    memcpy(&last_word, (char*)m_buffer + m_buffer_sz - 4, 4);
+    require(last_word == 0);
+
+    m_file_size = m_xfile.get_size();
+    require(n <= m_xfile.get_size());
+    m_file_pos = m_file_size - n;
+    if (m_file_pos < sizeof(m_fileHeader))
+    {
+      int d = sizeof(m_fileHeader) - m_file_pos;
+      m_file_pos += d;
+      n -= d;
+    }
+    m_buffer_data_left = n - 4;
+    m_buffer_ptr = (char*)m_buffer + m_buffer_sz - 4;
   }
 
   // Check for BackupFormat::FileHeader::ByteOrder if swapping is needed
@@ -1847,13 +2131,13 @@ BackupFile::validateFooter(){
 #ifdef ERROR_INSERT
 void BackupFile::error_insert(unsigned int code)
 {
+  m_error_insert = code;
   if(code == NDB_RESTORE_ERROR_INSERT_SMALL_BUFFER)
   {
     // Reduce size of buffer to test buffer overflow
     // handling. The buffer must still be large enough to
     // accommodate the file header.
     m_buffer_sz = 256;
-    m_error_insert = NDB_RESTORE_ERROR_INSERT_SMALL_BUFFER;
   }
 }
 #endif
@@ -1867,7 +2151,21 @@ bool RestoreDataIterator::readFragmentHeader(int & ret, Uint32 *fragmentId)
   while (1)
   {
     /* read first part of header */
-    if (buffer_read(&Header, 8, 1) != 1)
+    int r = buffer_read(&Header, 8, 1);
+    if (r < 0)
+    {
+      if (!m_xfile.is_encrypted())
+      {
+        restoreLogger.log_error("readFragmentHeader: Error reading header");
+      }
+      else
+      {
+        restoreLogger.log_error("readFragmentHeader: Error reading header. Wrong password?");
+      }
+      ret = -2;
+      return false;
+    }
+    if (r != 1)
     {
       ret = 0;
       return false;
@@ -1885,13 +2183,22 @@ bool RestoreDataIterator::readFragmentHeader(int & ret, Uint32 *fragmentId)
         return false;
       }
       if (Header.SectionLength > 2)
-        buffer_get_ptr(&tmp, Header.SectionLength*4-8, 1);
+      {
+        int r = buffer_get_ptr(&tmp, Header.SectionLength * 4 - 8, 1);
+        if (r < 0)
+        {
+          ret = -2;
+          return false;
+        }
+      }
       continue;
     }
     break;
   }
   /* read rest of header */
-  if (buffer_read(((char*)&Header)+8, Header.SectionLength*4-8, 1) != 1)
+  require(Header.SectionLength >= 2);
+  int r = buffer_read(((char*)&Header) + 8, Header.SectionLength * 4 - 8, 1);
+  if (r != 1)
   {
     ret = 0;
     return false;
@@ -1919,7 +2226,7 @@ bool RestoreDataIterator::readFragmentHeader(int & ret, Uint32 *fragmentId)
     return false;
   }
 
-  init_bitfield_storage(m_currentTable->m_dictTable);
+  calc_row_extra_storage_words(m_currentTable);
   info.setLevel(254);
   restoreLogger.log_info("_____________________________________________________"
                          "\nProcessing data in table: %s(%u) fragment %u",
@@ -1937,7 +2244,9 @@ bool
 RestoreDataIterator::validateFragmentFooter() {
   BackupFormat::DataFile::FragmentFooter footer;
   
-  if (buffer_read(&footer, sizeof(footer), 1) != 1){
+  int r = buffer_read(&footer, sizeof(footer), 1);
+  if (r != 1)
+  {
     restoreLogger.log_error("getFragmentFooter:Error reading fragment footer");
     return false;
   } 
@@ -1956,7 +2265,7 @@ RestoreDataIterator::validateFragmentFooter() {
 } // RestoreDataIterator::getFragmentFooter
 
 AttributeDesc::AttributeDesc(NdbDictionary::Column *c)
-  : m_column(c), truncation_detected(false)
+  : m_column(c), transform(NULL), truncation_detected(false)
 {
   size = 8*NdbColumnImpl::getImpl(* c).m_attrSize;
   arraySize = NdbColumnImpl::getImpl(* c).m_arraySize;
@@ -1964,6 +2273,12 @@ AttributeDesc::AttributeDesc(NdbDictionary::Column *c)
   parameterSz = 0;
 }
 
+AttributeDesc::~AttributeDesc()
+{
+  delete transform;
+  transform = NULL;
+}
+
 void TableS::createAttr(NdbDictionary::Column *column)
 {
   AttributeDesc * d = new AttributeDesc(column);
@@ -2087,6 +2402,7 @@ RestoreLogIterator::RestoreLogIterator(const RestoreMetaData & md)
 
   m_count = 0;
   m_last_gci = 0;
+  m_rowBuffIndex = 0;
 }
 
 const LogEntry *
@@ -2111,7 +2427,8 @@ RestoreLogIterator::getNextLogEntry(int & res) {
         res = 0;
         return 0;
       }
-      if (read_result != 1) {
+      if (read_result < 0 || read_result != 1)
+      {
         res= -1;
         return 0;
       }
@@ -2125,7 +2442,9 @@ RestoreLogIterator::getNextLogEntry(int & res) {
     len= ntohl(len);
 
     Uint32 data_len = sizeof(Uint32) + len*4;
-    if (buffer_get_ptr((void **)(&logEntryPtr), 1, data_len) != data_len) {
+    int r = buffer_get_ptr((void **)(&logEntryPtr), 1, data_len);
+    if (r < 0 || Uint32(r) != data_len)
+    {
       res= -2;
       return 0;
     }
@@ -2219,6 +2538,7 @@ RestoreLogIterator::getNextLogEntry(int & res) {
 
   const TableS * tab = m_logEntry.m_table;
   m_logEntry.clear();
+  m_rowBuffIndex = 0;
 
   AttributeHeader * ah = (AttributeHeader *)attr_data;
   AttributeHeader *end = (AttributeHeader *)(attr_data + attr_data_len);
@@ -2249,7 +2569,24 @@ RestoreLogIterator::getNextLogEntry(int & res) {
       attr->Data.size = sz;
       Twiddle(attr->Desc, &(attr->Data));
     }
-    
+
+    if (attr->Desc->transform)
+    {
+      const int col_idx = ah->getAttributeId();
+      const NdbDictionary::Column* col = tab->m_dictTable->getColumn(col_idx);
+      void* dst_buf = m_rowBuff + m_rowBuffIndex;
+      m_rowBuffIndex+= attr->Desc->getSizeInWords();
+      assert(m_rowBuffIndex <= RowBuffWords);
+
+      if (!applyColumnTransform(col,
+                                attr->Desc,
+                                &attr->Data,
+                                dst_buf))
+      {
+        res = -1;
+        return 0;
+      }
+    }
     
     ah = ah->getNext();
   }
