@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,6 +33,7 @@
 #include <ndb_limits.h>
 #include <ndb_opts.h>
 #include <ndb_version.h>
+#include "util/ndb_openssl_evp.h" // ndb_openssl_evp::library_init()
 
 #include "../src/ndbapi/NdbDictionaryImpl.hpp"
 #include "consumer_printer.hpp"
@@ -80,6 +81,11 @@ static unsigned opt_nodegroup_map_len= 0;
 static NODE_GROUP_MAP opt_nodegroup_map[MAX_NODE_GROUP_MAPS];
 #define OPT_NDB_NODEGROUP_MAP 'z'
 
+static bool opt_decrypt = false;
+
+// g_backup_password global, directly accessed in Restore.cpp.
+char* g_backup_password = nullptr;
+
 const char *opt_ndb_database= NULL;
 const char *opt_ndb_table= NULL;
 unsigned int opt_verbose;
@@ -139,6 +145,8 @@ static bool ga_disable_indexes = false;
 static bool ga_rebuild_indexes = false;
 bool ga_skip_unknown_objects = false;
 bool ga_skip_broken_objects = false;
+bool ga_allow_pk_changes = false;
+bool ga_ignore_extended_pk_updates = false;
 BaseString g_options("ndb_restore");
 static int ga_num_slices = 1;
 static int ga_slice_id = 0;
@@ -155,6 +163,7 @@ enum ndb_restore_options {
 #ifdef ERROR_INSERT
   ,OPT_ERROR_INSERT
 #endif
+  ,OPT_REMAP_COLUMN = 'x'
 };
 static const char *opt_fields_enclosed_by= NULL;
 static const char *opt_fields_terminated_by= NULL;
@@ -168,8 +177,98 @@ static const char *opt_include_tables= NULL;
 static const char *opt_exclude_databases= NULL;
 static const char *opt_include_databases= NULL;
 static const char *opt_rewrite_database= NULL;
+static const char *opt_one_remap_col_arg= NULL;
 static bool opt_restore_privilege_tables = false;
 
+/**
+ * ExtraTableInfo
+ *
+ * Container for information from user about how
+ * table should be restored
+ */
+class ExtraTableInfo
+{
+public:
+  ExtraTableInfo(const char* db_name,
+                 const char* table_name):
+    m_dbName(db_name),
+    m_tableName(table_name)
+  {
+  }
+
+  ~ExtraTableInfo() {}
+
+  const BaseString m_dbName;
+  const BaseString m_tableName;
+
+  /* Arguments related to column remappings */
+  Vector<BaseString> m_remapColumnArgs;
+};
+
+/**
+ * ExtraRestoreInfo
+ *
+ * Container for information from user about
+ * how to restore
+ */
+class ExtraRestoreInfo
+{
+public:
+  ExtraRestoreInfo()
+  {}
+  ~ExtraRestoreInfo()
+  {
+    for (Uint32 i=0; i<m_tables.size(); i++)
+    {
+      delete m_tables[i];
+      m_tables[i] = NULL;
+    }
+  }
+
+  /**
+   * findTable
+   *
+   * Lookup extra restore info for named table
+   */
+  ExtraTableInfo* findTable(const char* db_name,
+                            const char* table_name)
+  {
+    for (Uint32 i=0; i<m_tables.size(); i++)
+    {
+      ExtraTableInfo* tab = m_tables[i];
+      if ((strcmp(db_name, tab->m_dbName.c_str()) == 0) &&
+          (strcmp(table_name, tab->m_tableName.c_str()) == 0))
+      {
+        return tab;
+      }
+    }
+    return NULL;
+  }
+
+  /**
+   * findOrAddTable
+   *
+   * Lookup or Add empty extra restore info for named table
+   */
+  ExtraTableInfo* findOrAddTable(const char* db_name,
+                                 const char* table_name)
+  {
+    ExtraTableInfo* tab = findTable(db_name, table_name);
+    if (tab != NULL)
+    {
+      return tab;
+    }
+
+    tab = new ExtraTableInfo(db_name, table_name);
+    m_tables.push_back(tab);
+    return tab;
+  }
+
+  Vector<ExtraTableInfo*> m_tables;
+};
+
+static ExtraRestoreInfo g_extra_restore_info;
+
 static struct my_option my_long_options[] =
 {
   NDB_STD_OPTS("ndb_restore"),
@@ -182,6 +281,12 @@ static struct my_option my_long_options[] =
   { "backupid", 'b', "Backup id",
     (uchar**) &ga_backupId, (uchar**) &ga_backupId, 0,
     GET_INT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
+  { "decrypt", NDB_OPT_NOSHORT, "Decrypt file",
+    (uchar**) &opt_decrypt, (uchar**) &opt_decrypt, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "backup-password", NDB_OPT_NOSHORT, "Encryption password for backup file",
+    (uchar**) &g_backup_password, (uchar**) &g_backup_password, 0,
+    GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
   { "restore_data", 'r', 
     "Restore table data/logs into NDB Cluster using NDBAPI", 
     (uchar**) &_restore_data, (uchar**) &_restore_data,  0,
@@ -212,8 +317,8 @@ static struct my_option my_long_options[] =
     (uchar**) &_no_restore_disk, (uchar**) &_no_restore_disk,  0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "restore_epoch", 'e', 
-    "Restore epoch info into the status table. Convenient on a MySQL Cluster "
-    "replication slave, for starting replication. The row in "
+    "Restore epoch info into the status table. Convenient for starting MySQL "
+    "Cluster replication. The row in "
     NDB_REP_DB "." NDB_APPLY_TABLE " with id 0 will be updated/inserted.", 
     (uchar**) &ga_restore_epoch, (uchar**) &ga_restore_epoch,  0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
@@ -390,6 +495,23 @@ static struct my_option my_long_options[] =
     0,
     0,
     0 },
+  { "allow-pk-changes", NDB_OPT_NOSHORT,
+    "Allow changes to the set of columns making up a table's primary key.",
+    (uchar**) &ga_allow_pk_changes, (uchar**) &ga_allow_pk_changes, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "ignore-extended-pk-updates", NDB_OPT_NOSHORT,
+    "Ignore log entries containing updates to columns now included in an "
+    "extended primary key.",
+    (uchar**) &ga_ignore_extended_pk_updates,
+    (uchar**) &ga_ignore_extended_pk_updates,
+    0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "remap-column", OPT_REMAP_COLUMN, "Remap content for column while "
+    "restoring, format <database>.<table>.<column>:<function>:<function_args>."
+    "  <database> is remapped name, remapping applied before other conversions.",
+    (uchar**) &opt_one_remap_col_arg,
+    (uchar**) &opt_one_remap_col_arg, 0,
+    GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   { 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
@@ -502,6 +624,90 @@ static bool analyse_nodegroup_map(const char *ng_map_str,
   DBUG_RETURN(FALSE);
 }
 
+static bool parse_remap_option(const BaseString option,
+                               BaseString& db_name,
+                               BaseString& tab_name,
+                               BaseString& col_name,
+                               BaseString& func_name,
+                               BaseString& func_args,
+                               BaseString& error_msg)
+{
+  const char* expectedFormat = "<db>.<table>.<column>:function[:args]";
+
+  Vector<BaseString> optParts;
+  const int numOptParts = option.split(optParts,
+                                       BaseString(":"),
+                                       3);
+
+  if (numOptParts < 2)
+  {
+    error_msg.assfmt("remap-column : Badly formed option : %s.  "
+                     "Expected format : %s.",
+                     option.c_str(),
+                     expectedFormat);
+    return false;
+  }
+
+  Vector<BaseString> nameParts;
+  const int numNameParts = optParts[0].split(nameParts,
+                                             BaseString("."));
+  if (numNameParts != 3)
+  {
+    error_msg.assfmt("remap-column : Badly formed column specifier : %s "
+                     "in option %s.  "
+                     "Expected format : %s.",
+                     optParts[0].c_str(),
+                     option.c_str(),
+                     expectedFormat);
+    return false;
+  }
+
+  /* Copy out substrings */
+  db_name.assign(nameParts[0]);
+  tab_name.assign(nameParts[1]);
+  col_name.assign(nameParts[2]);
+  func_name.assign(optParts[1]);
+
+  if (numOptParts == 3)
+  {
+    func_args.assign(optParts[2]);
+  }
+  else
+  {
+    func_args.assign("");
+  }
+
+  return true;
+}
+
+
+static bool parse_remap_column(const char* argument)
+{
+  BaseString option(argument);
+  BaseString db, tab, col, func, args, error_msg;
+
+  if (!parse_remap_option(option,
+                          db,
+                          tab,
+                          col,
+                          func,
+                          args,
+                          error_msg))
+  {
+    restoreLogger.log_info("%s", error_msg.c_str());
+    return false;
+  }
+
+  /* Store this remapping + arguments against the db+table name */
+  ExtraTableInfo* eti = g_extra_restore_info.findOrAddTable(db.c_str(),
+                                                            tab.c_str());
+
+  /* We store the whole argument string to assist error reporting later */
+  eti->m_remapColumnArgs.push_back(option);
+
+  return true;
+}
+
 static void short_usage_sub(void)
 {
   ndb_short_usage_sub("[<path to backup files>]");
@@ -561,6 +767,8 @@ get_one_option(int optid, const struct my_option *opt MY_ATTRIBUTE((unused)),
   case OPT_REWRITE_DATABASE:
     parse_rewrite_database(argument);
     break;
+  case OPT_REMAP_COLUMN:
+    return (parse_remap_column(argument) ? 0 : 1);
   }
   return 0;
 }
@@ -624,23 +832,19 @@ makeExternalTableName(const BaseString &internalName)
   return externalName;
 }
 
-// Exclude the legacy list of six privilege tables from Cluster 7.x
-#include "storage/ndb/plugin/ndb_dist_priv_util.h"
-void
-exclude_privilege_tables()
-{
-  const char* table_name;
-  Ndb_dist_priv_util dist_priv;
-  while((table_name= dist_priv.iter_next_table()))
-  {
-    BaseString priv_tab;
-    priv_tab.assfmt("%s.%s", dist_priv.database(), table_name);
-    g_exclude_tables.push_back(priv_tab);
-    save_include_exclude(OPT_EXCLUDE_TABLES, (char *)priv_tab.c_str());
+// Exclude the legacy privilege tables from Cluster 7.x
+void exclude_privilege_tables() {
+  static const char *priv_tables[] = {
+      "mysql.user",         "mysql.db",         "mysql.tables_priv",
+      "mysql.columns_priv", "mysql.procs_priv", "mysql.proxies_priv"};
+
+  for (size_t i = 0; i < array_elements(priv_tables); i++) {
+    g_exclude_tables.push_back(priv_tables[i]);
+    save_include_exclude(OPT_EXCLUDE_TABLES,
+                         const_cast<char *>(priv_tables[i]));
   }
 }
 
-
 bool
 readArguments(Ndb_opts & opts, char*** pargv)
 {
@@ -657,6 +861,25 @@ readArguments(Ndb_opts & opts, char*** pargv)
   {
     exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
   }
+  if (!opt_decrypt)
+  {
+    if (g_backup_password != nullptr)
+    {
+      err <<
+        "Password (--backup-password) for decryption given, require also --decrypt."
+        << endl;
+      exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
+    }
+  }
+  else
+  {
+    if (g_backup_password == nullptr)
+    {
+      err << "Decrypting backup (--decrypt) requires password (--backup-password)." << endl;
+      exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
+    }
+  }
+
   if (ga_nodeId == 0)
   {
     err << "Backup file node ID not specified, please provide --nodeid" << endl;
@@ -864,6 +1087,14 @@ o verify nodegroup mapping
   {
     g_tableCompabilityMask |= TCM_EXCLUDE_MISSING_COLUMNS;
   }
+  if (ga_allow_pk_changes)
+  {
+    g_tableCompabilityMask |= TCM_ALLOW_PK_CHANGES;
+  }
+  if(ga_ignore_extended_pk_updates)
+  {
+    g_tableCompabilityMask |= TCM_IGNORE_EXTENDED_PK_UPDATES;
+  }
   return true;
 }
 
@@ -874,9 +1105,6 @@ bool create_consumers(RestoreThreadData *data)
   if (printer == NULL)
     return false;
 
-  if (g_restoring_in_parallel && (ga_nParallelism > ga_part_count))
-    ga_nParallelism /= ga_part_count;
-
   char threadname[20];
   BaseString::snprintf(threadname, sizeof(threadname), "%d-%u-%u",
                        ga_nodeId,
@@ -1305,6 +1533,490 @@ exclude_missing_tables(const RestoreMetaData& metaData, const Vector<BackupConsu
   }
 }
 
+static TableS*
+find_table_spec(RestoreMetaData& metaData,
+                const char* searchDbName,
+                const char* searchTableName,
+                bool rewrite_backup_db)
+{
+  for(Uint32 m = 0; m < metaData.getNoOfTables(); m++)
+  {
+    TableS *tableSpec= metaData[m];
+
+    BaseString externalName = makeExternalTableName(tableSpec->getTableName());
+    BaseString dbName, tabName;
+    {
+      Vector<BaseString> components;
+      if (externalName.split(components,
+                             BaseString(".")) != 2)
+      {
+        restoreLogger.log_info("Error processing table name from "
+                               "backup %s from %s",
+                               externalName.c_str(),
+                               tableSpec->getTableName());
+        return NULL;
+      }
+      dbName = components[0];
+      tabName = components[1];
+
+      if (rewrite_backup_db)
+      {
+        /* Check for rewrite db, as args are specified wrt new db names */
+        const char* rewrite_dbname;
+        if (g_rewrite_databases.get(dbName.c_str(), &rewrite_dbname))
+        {
+          dbName.assign(rewrite_dbname);
+        }
+      }
+    }
+
+    if (dbName == searchDbName &&
+        tabName == searchTableName)
+    {
+      return tableSpec;
+    }
+  }
+
+  return NULL;
+}
+
+class OffsetTransform : public ColumnTransform
+{
+public:
+  static
+  OffsetTransform* parse(const NdbDictionary::Column* col,
+                         const BaseString& func_name,
+                         const BaseString& func_args,
+                         BaseString& error_msg)
+  {
+    bool sig = true;
+    Uint64 bits = 0;
+    switch (col->getType())
+    {
+    case NdbDictionary::Column::Bigint:
+      sig = true;
+      bits = 64;
+      break;
+    case NdbDictionary::Column::Bigunsigned:
+      sig = false;
+      bits = 64;
+      break;
+    case NdbDictionary::Column::Int:
+      sig = true;
+      bits = 32;
+      break;
+    case NdbDictionary::Column::Unsigned:
+      sig = false;
+      bits = 32;
+      break;
+    default:
+      error_msg.assfmt("Column does not have supported integer type");
+      return NULL;
+    }
+
+    /* Todo : Use ndb type traits */
+    const Uint64 shift = bits - 1;
+    const Uint64 max_uval = ((Uint64(1) << shift) -1) | (Uint64(1) << shift);
+    const Int64 min_sval = 0 - (Uint64(1) << shift);
+    const Int64 max_sval = (Uint64(1) << shift) - 1;
+
+    Int64 offset_val;
+
+    int cnt = sscanf(func_args.c_str(), "%lld", &offset_val);
+    if (cnt != 1)
+    {
+      error_msg.assfmt("offset argument invalid");
+      return NULL;
+    }
+
+    {
+      /* Re-convert back to check for silent-saturation in sscanf */
+      char numbuf[22];
+      BaseString::snprintf(numbuf, sizeof(numbuf),
+                           "%lld", offset_val);
+      if (strncmp(func_args.c_str(), numbuf, sizeof(numbuf)) != 0)
+      {
+        error_msg.assfmt("Offset %s unreadable - out of range for type?",
+                         func_args.c_str());
+        return NULL;
+      }
+    }
+
+    if (offset_val < min_sval ||
+        offset_val > max_sval)
+    {
+      error_msg.assfmt("Offset %lld is out of range for type.",
+                       offset_val);
+      return NULL;
+    }
+
+    return new OffsetTransform(offset_val,
+                               sig,
+                               bits,
+                               min_sval,
+                               max_sval,
+                               max_uval);
+  }
+
+private:
+  Int64 m_offset_val;
+  Int64 m_sig_bound;
+  Uint64 m_unsig_bound;
+  bool m_offset_positive;
+  bool m_sig;
+  Uint32 m_bits;
+
+  OffsetTransform(Int64 offset_val,
+                  bool sig,
+                  Uint32 bits,
+                  Int64 min_sval,
+                  Int64 max_sval,
+                  Uint64 max_uval):
+    m_offset_val(offset_val),
+    m_sig(sig),
+    m_bits(bits)
+  {
+    m_offset_positive = offset_val >= 0;
+    if (sig)
+    {
+      if (m_offset_positive)
+      {
+        m_sig_bound = max_sval - offset_val;
+      }
+      else
+      {
+        m_sig_bound = min_sval - offset_val; // - - = +
+      }
+    }
+    else
+    {
+      if (m_offset_positive)
+      {
+        m_unsig_bound = max_uval - offset_val;
+      }
+      else
+      {
+        m_unsig_bound = (0 - offset_val);
+      }
+    }
+  }
+
+  ~OffsetTransform() override {}
+
+  static Uint64 readIntoU64(const void* src, Uint32 bits)
+  {
+    switch(bits)
+    {
+    case 64:
+      Uint64 dst;
+      memcpy(&dst, src, 8);
+      return dst;
+    case 32:
+    {
+      Uint32 u32;
+      memcpy(&u32, src, 4);
+      return u32;
+    }
+    default:
+      abort();
+    }
+    return 0;
+  }
+
+  static void writeFromU64(Uint64 src, void* dst, Uint32 bits)
+  {
+    switch(bits)
+    {
+    case 64:
+      memcpy(dst, &src, 8);
+      return;
+    case 32:
+    {
+      Uint32 u32 = (Uint32) src;
+      memcpy(dst, &u32, 4);
+      return;
+    }
+    default:
+      abort();
+    }
+  }
+
+  static Int64 readIntoS64(const void* src, Uint32 bits)
+  {
+    switch(bits)
+    {
+    case 64:
+      Int64 dst;
+      memcpy(&dst, src, 8);
+      return dst;
+    case 32:
+    {
+      Int32 i32;
+      memcpy(&i32, src, 4);
+      return i32;
+    }
+    default:
+      abort();
+    }
+    return 0;
+  }
+
+  static void writeFromS64(Int64 src, void* dst, Uint32 bits)
+  {
+    switch(bits)
+    {
+    case 64:
+      memcpy(dst, &src, 8);
+      return;
+    case 32:
+    {
+      Int32 i32 = (Int32) src;
+      memcpy(dst, &i32, 4);
+      return;
+    }
+    default:
+      abort();
+    }
+  }
+      
+  bool apply(const NdbDictionary::Column* col,
+                     const void* src_data,
+                     void** dst_data) override
+  {
+    if (src_data == NULL)
+    {
+      /* Offset(NULL, *) -> NULL */
+      *dst_data = NULL;
+      return true;
+    }
+
+    if (m_sig)
+    {
+      Int64 src_val = readIntoS64(src_data, m_bits);
+
+      bool src_in_bounds = true;
+      if (m_offset_positive)
+      {
+        src_in_bounds = (src_val <= m_sig_bound);
+      }
+      else
+      {
+        src_in_bounds = (src_val >= m_sig_bound);
+      }
+
+      if (unlikely(!src_in_bounds))
+      {
+        fprintf(stderr, "Offset : Source value out of bounds : adding %lld to %lld "
+                "gives an out of bounds value\n",
+                m_offset_val,
+                src_val);
+
+        return false;
+      }
+
+      src_val += m_offset_val;
+
+      writeFromS64(src_val, *dst_data, m_bits);
+    }
+    else
+    {
+      /* Unsigned */
+      Uint64 src_val = readIntoU64(src_data, m_bits);
+
+      bool src_in_bounds = true;
+      if (m_offset_positive)
+      {
+        src_in_bounds = (src_val <= m_unsig_bound);
+      }
+      else
+      {
+        src_in_bounds = (src_val >= m_unsig_bound);
+      }
+
+      if (unlikely(!src_in_bounds))
+      {
+        fprintf(stderr, "Offset : Source value out of bounds : adding %lld to %llu "
+                "gives an out of bounds value\n",
+                m_offset_val,
+                src_val);
+
+        return false;
+      }
+
+      src_val+= m_offset_val;
+
+      writeFromU64(src_val, *dst_data, m_bits);
+    }
+
+    return true;
+  }
+};
+
+
+static ColumnTransform*
+create_column_transform(const NdbDictionary::Column* col,
+                        const BaseString& func_name,
+                        const BaseString& func_args,
+                        BaseString& error_msg)
+{
+  BaseString lcfunc_name(func_name);
+  lcfunc_name.ndb_tolower();
+
+  if (lcfunc_name == "offset")
+  {
+    return OffsetTransform::parse(col, func_name, func_args, error_msg);
+  }
+  error_msg.assfmt("Function %s not defined", func_name.c_str());
+  return NULL;
+}
+
+static bool
+setup_one_remapping(TableS* tableSpec,
+                    const BaseString& col_name,
+                    const BaseString& func_name,
+                    const BaseString& func_args,
+                    BaseString& error_msg)
+{
+  const NdbDictionary::Column* col =
+    tableSpec->m_dictTable->getColumn(col_name.c_str());
+
+  if (!col)
+  {
+    error_msg.assfmt("Failed to find column %s in table",
+                     col_name.c_str());
+    return false;
+  }
+
+  AttributeDesc* ad = tableSpec->getAttributeDesc(col->getColumnNo());
+
+  if (ad->transform != NULL)
+  {
+    error_msg.assfmt("Duplicate remappings on column %s",
+                     col_name.c_str());
+    return false;
+  }
+
+  restoreLogger.log_debug("Initialising remap function "
+                          "\"%s:%s\" on column %s.%s",
+                          func_name.c_str(),
+                          func_args.c_str(),
+                          tableSpec->m_dictTable->getName(),
+                          col_name.c_str());
+
+  ColumnTransform* ct = create_column_transform(col,
+                                                func_name,
+                                                func_args,
+                                                error_msg);
+
+  if (ct == NULL)
+  {
+    return false;
+  }
+
+  ad->transform = ct;
+
+  return true;
+}
+
+static bool
+setup_column_remappings(RestoreMetaData& metaData)
+{
+  for (Uint32 t = 0; t < g_extra_restore_info.m_tables.size(); t++)
+  {
+    const ExtraTableInfo* eti = g_extra_restore_info.m_tables[t];
+
+    TableS* tableSpec = find_table_spec(metaData,
+                                        eti->m_dbName.c_str(),
+                                        eti->m_tableName.c_str(),
+                                        true); // Rewrite database
+    if (tableSpec)
+    {
+      const Vector<TableS*> blobTables = tableSpec->getBlobTables();
+      const bool haveBlobPartTables = blobTables.size() > 0;
+
+      for (Uint32 a=0; a < eti->m_remapColumnArgs.size(); a++)
+      {
+        BaseString db_name, tab_name, col_name, func_name, func_args, error_msg;
+        if (!parse_remap_option(eti->m_remapColumnArgs[a],
+                                db_name,
+                                tab_name,
+                                col_name,
+                                func_name,
+                                func_args,
+                                error_msg))
+        {
+          /* Should never happen as arg parsed on initial read */
+          restoreLogger.log_info("Unexpected - parse failed : \"%s\"",
+                                 eti->m_remapColumnArgs[a].c_str());
+          return false;
+        }
+
+        if (!setup_one_remapping(tableSpec,
+                                 col_name,
+                                 func_name,
+                                 func_args,
+                                 error_msg))
+        {
+          restoreLogger.log_info("remap_column : Failed with \"%s\" "
+                                 "while processing option \"%s\"",
+                                 error_msg.c_str(),
+                                 eti->m_remapColumnArgs[a].c_str());
+          return false;
+        }
+
+        const bool col_in_pk =
+          tableSpec->m_dictTable->getColumn(col_name.c_str())->getPrimaryKey();
+
+        if (col_in_pk &&
+            haveBlobPartTables)
+        {
+          /* This transform should be added on the Blob part table(s) */
+          for (Uint32 b=0; b < blobTables.size(); b++)
+          {
+            TableS* blobPartTableSpec = blobTables[b];
+            const NdbDictionary::Column* mainTabBlobCol =
+              tableSpec->m_dictTable->getColumn(blobPartTableSpec->getMainColumnId());
+
+            if (unlikely(mainTabBlobCol->getBlobVersion() == NDB_BLOB_V1))
+            {
+              restoreLogger.log_info("remap_column : Failed as table has "
+                                     "v1 Blob column %s when processing "
+                                     "option %s",
+                                     mainTabBlobCol->getName(),
+                                     eti->m_remapColumnArgs[a].c_str());
+              return false;
+            }
+
+            if (!setup_one_remapping(blobPartTableSpec,
+                                     col_name.c_str(),
+                                     func_name,
+                                     func_args,
+                                     error_msg))
+            {
+              restoreLogger.log_info("remap_column : Failed with error %s "
+                                     "while applying remapping to blob "
+                                     "parts table %s from option : %s",
+                                     error_msg.c_str(),
+                                     blobPartTableSpec->m_dictTable->getName(),
+                                     eti->m_remapColumnArgs[a].c_str());
+              return false;
+            }
+          }
+        }
+      }
+    }
+    else
+    {
+      restoreLogger.log_info("remap_column : Failed to find table in Backup "
+                             "matching option : \"%s\"",
+                             eti->m_remapColumnArgs[0].c_str());
+      return false;
+    }
+  }
+
+  return true;
+}
+
 static void
 free_data_callback(void *ctx)
 {
@@ -1328,6 +2040,7 @@ free_include_excludes_vector()
 
 static void exitHandler(int code)
 {
+  ndb_openssl_evp::library_end();
   free_include_excludes_vector();
   if (opt_core)
     abort();
@@ -1518,6 +2231,10 @@ int do_restore(RestoreThreadData *thrdata)
   {
     metaData.error_insert(_error_insert);
   }
+  for (Uint32 i = 0; i < g_consumers.size(); i++)
+  {
+    g_consumers[i]->error_insert(_error_insert);
+  }
 #endif 
   Logger::format_timestamp(time(NULL), timestamp, sizeof(timestamp));
   restoreLogger.log_info("%s [restore_metadata] Read meta data file header", timestamp);
@@ -1658,6 +2375,11 @@ int do_restore(RestoreThreadData *thrdata)
   if(ga_exclude_missing_tables)
     exclude_missing_tables(metaData, thrdata->m_consumers);
 
+  if (!setup_column_remappings(metaData))
+  {
+    return NdbToolsProgramExitCode::FAILED;
+  }
+
   /* report to clusterlog if applicable */
   for (i = 0; i < g_consumers.size(); i++)
     g_consumers[i]->report_started(ga_backupId, ga_nodeId);
@@ -1985,7 +2707,17 @@ int do_restore(RestoreThreadData *thrdata)
           OutputStream *tmp = ndbout.m_out;
           ndbout.m_out = output;
           for(Uint32 j= 0; j < g_consumers.size(); j++) 
-            g_consumers[j]->tuple(* tuple, fragmentId);
+          {
+            if (!g_consumers[j]->tuple(* tuple, fragmentId))
+            {
+              restoreLogger.log_error(
+                "Restore: error occurred while restoring data. Exiting...");
+              // wait for async transactions to complete
+              for (i= 0; i < g_consumers.size(); i++)
+                g_consumers[i]->endOfTuples();
+              return NdbToolsProgramExitCode::FAILED;
+            }
+          }
           ndbout.m_out =  tmp;
           if (check_progress())
             report_progress("Data file progress: ", dataIter);
@@ -1994,7 +2726,7 @@ int do_restore(RestoreThreadData *thrdata)
         if (res < 0)
         {
           restoreLogger.log_error(
-            "Restore: An error occurred while restoring data. Exiting...");
+            "Restore: An error occurred while reading data. Exiting...");
           return NdbToolsProgramExitCode::FAILED;
         }
         if (!dataIter.validateFragmentFooter())
@@ -2054,7 +2786,14 @@ int do_restore(RestoreThreadData *thrdata)
         if (check_slice_skip_fragment(table, logEntry->m_frag_id))
           continue;
         for(Uint32 j= 0; j < g_consumers.size(); j++)
-          g_consumers[j]->logEntry(* logEntry);
+        {
+          if (!g_consumers[j]->logEntry(* logEntry))
+          {
+            restoreLogger.log_error(
+              "Restore: Error restoring the data log. Exiting...");
+            return NdbToolsProgramExitCode::FAILED;
+          }
+        }
 
         if (check_progress())
           report_progress("Log file progress: ", logIter);
@@ -2062,7 +2801,7 @@ int do_restore(RestoreThreadData *thrdata)
       if (res < 0)
       {
         restoreLogger.log_error(
-          "Restore: An restoring the data log. Exiting... res = %u",
+          "Restore: Error reading the data log. Exiting... res = %d",
           res);
         return NdbToolsProgramExitCode::FAILED;
       }
@@ -2325,6 +3064,7 @@ static void* start_restore_worker(void *data)
 int
 main(int argc, char** argv)
 {
+  ndb_openssl_evp::library_init();
   NDB_INIT(argv[0]);
 
   const char *load_default_groups[]= { "mysql_cluster","ndb_restore",0 };
@@ -2374,6 +3114,10 @@ main(int argc, char** argv)
                      ga_num_slices,
                      ga_slice_id);
   }
+  if (ga_allow_pk_changes)
+    g_options.append(" --allow-pk-changes");
+  if (ga_ignore_extended_pk_updates)
+    g_options.append(" --ignore-extended-pk-updates");
 
   // determine backup format: simple or multi-part, and count parts
   int result = detect_backup_format();
@@ -2476,10 +3220,21 @@ main(int argc, char** argv)
    // create one restore thread per backup part
     Vector<RestoreThreadData*> thrdata;
     CyclicBarrier barrier(ga_part_count);
+
+    /**
+     * Divide data INSERT parallelism across parts, ensuring
+     * each part has at least 1
+     */
+    ga_nParallelism /= ga_part_count;
+    if (ga_nParallelism == 0)
+      ga_nParallelism = 1;
+
+    debug << "Part parallelism is " << ga_nParallelism << endl;
+
     for (int part_id=1; part_id<=ga_part_count; part_id++)
     {
       NDB_THREAD_PRIO prio = NDB_THREAD_PRIO_MEAN;
-      uint stack_size = 128*1024;
+      uint stack_size = 128 * 1024;
       char name[20];
       snprintf (name, sizeof(name), "restore%d", part_id);
       RestoreThreadData *data = new RestoreThreadData(part_id, &barrier);
@@ -2527,6 +3282,7 @@ main(int argc, char** argv)
     exitHandler(NdbToolsProgramExitCode::FAILED);
   }
 
+  ndb_openssl_evp::library_end();
   return NdbToolsProgramExitCode::OK;
 }  // main
 
