@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2013, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -64,6 +64,9 @@ struct st_row_applier_config {
   enum_rpl_yes_no requires_row_format;
   Relay_log_info::enum_require_table_primary_key
       require_table_primary_key_check;
+  Assign_gtids_to_anonymous_transactions_info::enum_type
+      assign_gtids_to_anonymous_transactions_type;
+  std::string assign_gtids_to_anonymous_transactions_value{""};
 };
 
 class PFS_index_rpl_applier_config : public PFS_engine_index {
@@ -71,7 +74,7 @@ class PFS_index_rpl_applier_config : public PFS_engine_index {
   PFS_index_rpl_applier_config()
       : PFS_engine_index(&m_key), m_key("CHANNEL_NAME") {}
 
-  ~PFS_index_rpl_applier_config() {}
+  ~PFS_index_rpl_applier_config() override {}
 
   virtual bool match(Master_info *mi);
 
@@ -107,26 +110,26 @@ class table_replication_applier_configuration : public PFS_engine_table {
     @param read_all         true if all columns are read.
   */
 
-  virtual int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
-                              bool read_all);
+  int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
+                      bool read_all) override;
 
   table_replication_applier_configuration();
 
  public:
-  ~table_replication_applier_configuration();
+  ~table_replication_applier_configuration() override;
 
   /** Table share. */
   static PFS_engine_table_share m_share;
   static PFS_engine_table *create(PFS_engine_table_share *);
   static ha_rows get_row_count();
 
-  virtual void reset_position(void);
+  void reset_position(void) override;
 
-  virtual int rnd_next();
-  virtual int rnd_pos(const void *pos);
+  int rnd_next() override;
+  int rnd_pos(const void *pos) override;
 
-  virtual int index_init(uint idx, bool sorted);
-  virtual int index_next();
+  int index_init(uint idx, bool sorted) override;
+  int index_next() override;
 
  private:
   PFS_index_rpl_applier_config *m_opened_index;
