@@ -1,16 +1,15 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2020, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -65,10 +64,9 @@ struct row_mutex_instances {
 
 class PFS_index_mutex_instances : public PFS_engine_index {
  public:
-  explicit PFS_index_mutex_instances(PFS_engine_key *key_1)
-      : PFS_engine_index(key_1) {}
+  PFS_index_mutex_instances(PFS_engine_key *key_1) : PFS_engine_index(key_1) {}
 
-  ~PFS_index_mutex_instances() override = default;
+  ~PFS_index_mutex_instances() override {}
 
   virtual bool match(PFS_mutex *pfs) = 0;
 };
@@ -78,7 +76,7 @@ class PFS_index_mutex_instances_by_instance : public PFS_index_mutex_instances {
   PFS_index_mutex_instances_by_instance()
       : PFS_index_mutex_instances(&m_key), m_key("OBJECT_INSTANCE_BEGIN") {}
 
-  ~PFS_index_mutex_instances_by_instance() override = default;
+  ~PFS_index_mutex_instances_by_instance() override {}
 
   bool match(PFS_mutex *pfs) override;
 
@@ -91,7 +89,7 @@ class PFS_index_mutex_instances_by_name : public PFS_index_mutex_instances {
   PFS_index_mutex_instances_by_name()
       : PFS_index_mutex_instances(&m_key), m_key("NAME") {}
 
-  ~PFS_index_mutex_instances_by_name() override = default;
+  ~PFS_index_mutex_instances_by_name() override {}
 
   bool match(PFS_mutex *pfs) override;
 
@@ -105,7 +103,7 @@ class PFS_index_mutex_instances_by_thread_id
   PFS_index_mutex_instances_by_thread_id()
       : PFS_index_mutex_instances(&m_key), m_key("LOCKED_BY_THREAD_ID") {}
 
-  ~PFS_index_mutex_instances_by_thread_id() override = default;
+  ~PFS_index_mutex_instances_by_thread_id() override {}
 
   bool match(PFS_mutex *pfs) override;
 
@@ -121,13 +119,13 @@ class table_mutex_instances : public PFS_engine_table {
   static PFS_engine_table *create(PFS_engine_table_share *);
   static ha_rows get_row_count();
 
-  void reset_position() override;
+  void reset_position(void) override;
 
   int rnd_next() override;
   int rnd_pos(const void *pos) override;
 
   int index_init(uint idx, bool sorted) override;
-  int index_next() override;
+  int index_next(void) override;
 
  private:
   int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
@@ -135,7 +133,7 @@ class table_mutex_instances : public PFS_engine_table {
   table_mutex_instances();
 
  public:
-  ~table_mutex_instances() override = default;
+  ~table_mutex_instances() override {}
 
  protected:
   int make_row(PFS_mutex *pfs);
@@ -173,10 +171,9 @@ struct row_rwlock_instances {
 
 class PFS_index_rwlock_instances : public PFS_engine_index {
  public:
-  explicit PFS_index_rwlock_instances(PFS_engine_key *key_1)
-      : PFS_engine_index(key_1) {}
+  PFS_index_rwlock_instances(PFS_engine_key *key_1) : PFS_engine_index(key_1) {}
 
-  ~PFS_index_rwlock_instances() override = default;
+  ~PFS_index_rwlock_instances() override {}
 
   virtual bool match(PFS_rwlock *pfs) = 0;
 };
@@ -187,7 +184,7 @@ class PFS_index_rwlock_instances_by_instance
   PFS_index_rwlock_instances_by_instance()
       : PFS_index_rwlock_instances(&m_key), m_key("OBJECT_INSTANCE_BEGIN") {}
 
-  ~PFS_index_rwlock_instances_by_instance() override = default;
+  ~PFS_index_rwlock_instances_by_instance() override {}
 
   bool match(PFS_rwlock *pfs) override;
 
@@ -200,7 +197,7 @@ class PFS_index_rwlock_instances_by_name : public PFS_index_rwlock_instances {
   PFS_index_rwlock_instances_by_name()
       : PFS_index_rwlock_instances(&m_key), m_key("NAME") {}
 
-  ~PFS_index_rwlock_instances_by_name() override = default;
+  ~PFS_index_rwlock_instances_by_name() override {}
 
   bool match(PFS_rwlock *pfs) override;
 
@@ -215,7 +212,7 @@ class PFS_index_rwlock_instances_by_thread_id
       : PFS_index_rwlock_instances(&m_key),
         m_key("WRITE_LOCKED_BY_THREAD_ID") {}
 
-  ~PFS_index_rwlock_instances_by_thread_id() override = default;
+  ~PFS_index_rwlock_instances_by_thread_id() override {}
 
   bool match(PFS_rwlock *pfs) override;
 
@@ -231,13 +228,13 @@ class table_rwlock_instances : public PFS_engine_table {
   static PFS_engine_table *create(PFS_engine_table_share *);
   static ha_rows get_row_count();
 
-  void reset_position() override;
+  void reset_position(void) override;
 
   int rnd_next() override;
   int rnd_pos(const void *pos) override;
 
   int index_init(uint idx, bool sorted) override;
-  int index_next() override;
+  int index_next(void) override;
 
  private:
   int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
@@ -245,7 +242,7 @@ class table_rwlock_instances : public PFS_engine_table {
   table_rwlock_instances();
 
  public:
-  ~table_rwlock_instances() override = default;
+  ~table_rwlock_instances() override {}
 
  protected:
   int make_row(PFS_rwlock *pfs);
@@ -277,10 +274,9 @@ struct row_cond_instances {
 
 class PFS_index_cond_instances : public PFS_engine_index {
  public:
-  explicit PFS_index_cond_instances(PFS_engine_key *key_1)
-      : PFS_engine_index(key_1) {}
+  PFS_index_cond_instances(PFS_engine_key *key_1) : PFS_engine_index(key_1) {}
 
-  ~PFS_index_cond_instances() override = default;
+  ~PFS_index_cond_instances() override {}
 
   virtual bool match(PFS_cond *pfs) = 0;
 };
@@ -290,7 +286,7 @@ class PFS_index_cond_instances_by_instance : public PFS_index_cond_instances {
   PFS_index_cond_instances_by_instance()
       : PFS_index_cond_instances(&m_key), m_key("OBJECT_INSTANCE_BEGIN") {}
 
-  ~PFS_index_cond_instances_by_instance() override = default;
+  ~PFS_index_cond_instances_by_instance() override {}
 
   bool match(PFS_cond *pfs) override;
 
@@ -303,7 +299,7 @@ class PFS_index_cond_instances_by_name : public PFS_index_cond_instances {
   PFS_index_cond_instances_by_name()
       : PFS_index_cond_instances(&m_key), m_key("NAME") {}
 
-  ~PFS_index_cond_instances_by_name() override = default;
+  ~PFS_index_cond_instances_by_name() override {}
 
   bool match(PFS_cond *pfs) override;
 
@@ -319,13 +315,13 @@ class table_cond_instances : public PFS_engine_table {
   static PFS_engine_table *create(PFS_engine_table_share *);
   static ha_rows get_row_count();
 
-  void reset_position() override;
+  void reset_position(void) override;
 
   int rnd_next() override;
   int rnd_pos(const void *pos) override;
 
   int index_init(uint idx, bool sorted) override;
-  int index_next() override;
+  int index_next(void) override;
 
  private:
   int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
@@ -333,7 +329,7 @@ class table_cond_instances : public PFS_engine_table {
   table_cond_instances();
 
  public:
-  ~table_cond_instances() override = default;
+  ~table_cond_instances() override {}
 
  protected:
   int make_row(PFS_cond *pfs);
