@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2020, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -58,7 +57,7 @@ struct MemoryMonitor {
      * [in] Number of bytes.
      * @return Heap-memory consumption after increase. */
     static size_t increase(size_t bytes) {
-      assert(ram <= std::numeric_limits<decltype(bytes)>::max() - bytes);
+      DBUG_ASSERT(ram <= std::numeric_limits<decltype(bytes)>::max() - bytes);
       return ram.fetch_add(bytes) + bytes;
     }
     /** Log decrements of heap-memory consumption.
@@ -66,7 +65,7 @@ struct MemoryMonitor {
      * [in] Number of bytes.
      * @return Heap-memory consumption after decrease. */
     static size_t decrease(size_t bytes) {
-      assert(ram >= bytes);
+      DBUG_ASSERT(ram >= bytes);
       return ram.fetch_sub(bytes) - bytes;
     }
     /** Get heap-memory threshold level. Level is defined by this Allocator.
@@ -85,7 +84,7 @@ struct MemoryMonitor {
      * [in] Number of bytes.
      * @return MMAP-memory consumption after increase. */
     static size_t increase(size_t bytes) {
-      assert(mmap <= std::numeric_limits<decltype(bytes)>::max() - bytes);
+      DBUG_ASSERT(mmap <= std::numeric_limits<decltype(bytes)>::max() - bytes);
       return mmap.fetch_add(bytes) + bytes;
     }
     /** Log decrements of MMAP-backed memory consumption.
@@ -93,7 +92,7 @@ struct MemoryMonitor {
      * [in] Number of bytes.
      * @return MMAP-memory consumption after decrease. */
     static size_t decrease(size_t bytes) {
-      assert(mmap >= bytes);
+      DBUG_ASSERT(mmap >= bytes);
       return mmap.fetch_sub(bytes) - bytes;
     }
     /** Get MMAP-backed memory threshold level. Level is defined by this
@@ -119,36 +118,6 @@ struct MemoryMonitor {
   static std::atomic<size_t> mmap;
 };
 
-/* Thin abstraction which enables logging of how much resources have been
- * consumed at the per-table level. Each temptable::Table will be composed
- * of this type so that the temptable::Allocator through its policies can
- * monitor its memory consumption and act appropriately when threshold
- * is reached.
- **/
-class TableResourceMonitor {
- public:
-  TableResourceMonitor(size_t threshold)
-      : m_threshold(threshold), m_total_bytes(0) {}
-
-  size_t increase(size_t bytes) {
-    assert(m_total_bytes <=
-           std::numeric_limits<decltype(bytes)>::max() - bytes);
-    m_total_bytes += bytes;
-    return m_total_bytes;
-  }
-  size_t decrease(size_t bytes) {
-    assert(m_total_bytes >= bytes);
-    m_total_bytes -= bytes;
-    return m_total_bytes;
-  }
-  size_t threshold() { return m_threshold; }
-  size_t consumption() { return m_total_bytes; }
-
- private:
-  size_t m_threshold;
-  size_t m_total_bytes;
-};
-
 /* Allocation scheme, a type which controls allocation patterns in TempTable
  * allocator.
  *
@@ -196,22 +165,17 @@ struct Allocation_scheme {
   static size_t block_size(size_t number_of_blocks, size_t n_bytes_requested) {
     return Block_size_policy::block_size(number_of_blocks, n_bytes_requested);
   }
-  static void block_freed(uint32_t block_size, Source block_source) {
-    Block_source_policy::block_freed(block_size, block_source);
-  }
 };
 
 /* Concrete implementation of Block_source_policy, a type which controls where
  * TempTable allocator is going to be allocating next Block of memory from.
  *
- * In particular, this policy will make TempTable allocator:
+ * In particular, this policy will make TempTable allocator to:
  *  1. Use RAM as long as temptable_max_ram threshold is not reached.
  *  2. Start using MMAP when temptable_max_ram threshold is reached.
  *  3. Go back using RAM as soon as RAM consumption drops below the
- *     temptable_max_ram threshold and there is enough space to accommodate the
+ *     temptable_max_ram threshold and there is enough space to accomodate the
  *     new block given the size.
- *  4. Not take into account per-table memory limits defined through
- *     tmp_table_size SYSVAR.
  * */
 struct Prefer_RAM_over_MMAP_policy {
   static Source block_source(uint32_t block_size) {
@@ -233,14 +197,6 @@ struct Prefer_RAM_over_MMAP_policy {
     }
     throw Result::RECORD_FILE_FULL;
   }
-
-  static void block_freed(uint32_t block_size, Source block_source) {
-    if (block_source == Source::RAM) {
-      MemoryMonitor::RAM::decrease(block_size);
-    } else {
-      MemoryMonitor::MMAP::decrease(block_size);
-    }
-  }
 };
 
 /* Concrete implementation of Block_size_policy, a type which controls how big
@@ -311,77 +267,7 @@ using Exponential_growth_preferring_RAM_over_MMAP =
   guide's recommendation to have clear ownership of objects, but at least it
   avoids the use-after-free.
  */
-template <class AllocationScheme>
-class AllocatorState {
- public:
-  /**
-   * Destroys the state, deallocate the current_block if it was left empty.
-   */
-  ~AllocatorState() noexcept {
-    if (!current_block.is_empty()) {
-      free_block(current_block);
-    }
-    /* User must deallocate all data from all blocks, otherwise the memory will
-     * be leaked.
-     */
-    assert(number_of_blocks == 0);
-  }
-
-  /**
-   * Gets a Block from which a new allocation of the specified size should be
-   * performed. It will use the current Block or create a new one if it is too
-   * small.
-   * [in] Number of bytes that will be allocated from the returned block.
-   */
-  Block *get_block_for_new_allocation(size_t n_bytes_requested) {
-    if (current_block.is_empty() ||
-        !current_block.can_accommodate(n_bytes_requested)) {
-      /* The current_block may have been left empty during some deallocate()
-       * call. It is the last opportunity to free it before we lose reference to
-       * it.
-       */
-      if (!current_block.is_empty() &&
-          current_block.number_of_used_chunks() == 0) {
-        free_block(current_block);
-      }
-
-      const size_t block_size =
-          AllocationScheme::block_size(number_of_blocks, n_bytes_requested);
-      current_block =
-          Block(block_size, AllocationScheme::block_source(block_size));
-      ++number_of_blocks;
-    }
-    return &current_block;
-  }
-
-  /**
-   * Informs the state object of a block that has no data allocated inside of it
-   * anymore for garbage collection.
-   * [in] The empty block to manage and possibly free.
-   */
-  void block_is_not_used_anymore(Block block) noexcept {
-    if (block == current_block) {
-      /* Do nothing. Keep the last block alive. Some queries are repeatedly
-       * allocating one Row and freeing it, leading to constant allocation and
-       * deallocation of 1MB of memory for the current_block. Let's keep this
-       * block empty ready for a future use.
-       */
-    } else {
-      free_block(block);
-    }
-  }
-
- private:
-  /**
-   * Frees the specified block and takes care of all accounting.
-   * [in] The empty block to free.
-   */
-  void free_block(Block &block) noexcept {
-    AllocationScheme::block_freed(block.size(), block.type());
-    block.destroy();
-    --number_of_blocks;
-  }
-
+struct AllocatorState {
   /** Current not-yet-full block to feed allocations from. */
   Block current_block;
 
@@ -453,7 +339,7 @@ class Allocator {
   };
 
   /** Constructor. */
-  Allocator(Block *shared_block, TableResourceMonitor &table_resource_monitor);
+  explicit Allocator(Block *shared_block);
 
   /** Constructor from allocator of another type. The state is copied into the
    * new object. */
@@ -531,7 +417,7 @@ class Allocator {
     Shared state between all the copies and rebinds of this allocator.
     See AllocatorState for details.
    */
-  std::shared_ptr<AllocatorState<AllocationScheme>> m_state;
+  std::shared_ptr<AllocatorState> m_state;
 
   /** A block of memory which is a state external to this allocator and can be
    * shared among different instances of the allocator (not simultaneously). In
@@ -539,37 +425,27 @@ class Allocator {
    * memory of this shared block.
    */
   Block *m_shared_block;
-  /** Table resource monitor control mechanism that limits the amount of
-   *  resources that can be consumed at the per-table level.
-   */
-  TableResourceMonitor &m_table_resource_monitor;
 };
 
 /* Implementation of inlined methods. */
 
 template <class T, class AllocationScheme>
-inline Allocator<T, AllocationScheme>::Allocator(
-    Block *shared_block, TableResourceMonitor &table_resource_monitor)
-    : m_state(std::make_shared<AllocatorState<AllocationScheme>>()),
-      m_shared_block(shared_block),
-      m_table_resource_monitor(table_resource_monitor) {}
+inline Allocator<T, AllocationScheme>::Allocator(Block *shared_block)
+    : m_state(std::make_shared<AllocatorState>()),
+      m_shared_block(shared_block) {}
 
 template <class T, class AllocationScheme>
 template <class U>
 inline Allocator<T, AllocationScheme>::Allocator(const Allocator<U> &other)
-    : m_state(other.m_state),
-      m_shared_block(other.m_shared_block),
-      m_table_resource_monitor(other.m_table_resource_monitor) {}
+    : m_state(other.m_state), m_shared_block(other.m_shared_block) {}
 
 template <class T, class AllocationScheme>
 template <class U>
 inline Allocator<T, AllocationScheme>::Allocator(Allocator<U> &&other) noexcept
-    : m_state(std::move(other.m_state)),
-      m_shared_block(other.m_shared_block),
-      m_table_resource_monitor(other.m_table_resource_monitor) {}
+    : m_state(std::move(other.m_state)), m_shared_block(other.m_shared_block) {}
 
 template <class T, class AllocationScheme>
-inline Allocator<T, AllocationScheme>::~Allocator() = default;
+inline Allocator<T, AllocationScheme>::~Allocator() {}
 
 template <class T, class AllocationScheme>
 template <class U>
@@ -587,7 +463,7 @@ inline bool Allocator<T, AllocationScheme>::operator!=(
 
 template <class T, class AllocationScheme>
 inline T *Allocator<T, AllocationScheme>::allocate(size_t n_elements) {
-  assert(n_elements <= std::numeric_limits<size_type>::max() / sizeof(T));
+  DBUG_ASSERT(n_elements <= std::numeric_limits<size_type>::max() / sizeof(T));
   DBUG_EXECUTE_IF("temptable_allocator_oom", throw Result::OUT_OF_MEM;);
   DBUG_EXECUTE_IF("temptable_allocator_record_file_full",
                   throw Result::RECORD_FILE_FULL;);
@@ -608,38 +484,28 @@ inline T *Allocator<T, AllocationScheme>::allocate(size_t n_elements) {
   } else if (m_shared_block &&
              m_shared_block->can_accommodate(n_bytes_requested)) {
     block = m_shared_block;
+  } else if (m_state->current_block.is_empty() ||
+             !m_state->current_block.can_accommodate(n_bytes_requested)) {
+    const size_t block_size = AllocationScheme::block_size(
+        m_state->number_of_blocks, n_bytes_requested);
+    m_state->current_block =
+        Block(block_size, AllocationScheme::block_source(block_size));
+    block = &m_state->current_block;
+    ++m_state->number_of_blocks;
   } else {
-    block = m_state->get_block_for_new_allocation(n_bytes_requested);
+    block = &m_state->current_block;
   }
 
-  /* temptable::Table is allowed to fit no more data than the given threshold
-   * controlled through TableResourceMonitor abstraction. TableResourceMonitor
-   * is a simple abstraction which is in its part an alias for tmp_table_size, a
-   * system variable that end MySQL users will be using to control this
-   * threshold.
-   *
-   * Updating the tmp_table_size threshold can only be done through the separate
-   * SET statement which implies that the tmp_table_size threshold cannot be
-   * updated during the duration of some query which is running within the same
-   * session. Separate sessions can still of course change this value to their
-   * liking.
-   */
-  if (m_table_resource_monitor.consumption() + n_bytes_requested >
-      m_table_resource_monitor.threshold()) {
-    throw Result::RECORD_FILE_FULL;
-  }
-  m_table_resource_monitor.increase(n_bytes_requested);
-
   T *chunk_data =
       reinterpret_cast<T *>(block->allocate(n_bytes_requested).data());
-  assert(reinterpret_cast<uintptr_t>(chunk_data) % alignof(T) == 0);
+  DBUG_ASSERT(reinterpret_cast<uintptr_t>(chunk_data) % alignof(T) == 0);
   return chunk_data;
 }
 
 template <class T, class AllocationScheme>
 inline void Allocator<T, AllocationScheme>::deallocate(T *chunk_data,
                                                        size_t n_elements) {
-  assert(reinterpret_cast<uintptr_t>(chunk_data) % alignof(T) == 0);
+  DBUG_ASSERT(reinterpret_cast<uintptr_t>(chunk_data) % alignof(T) == 0);
 
   if (chunk_data == nullptr) {
     return;
@@ -654,10 +520,20 @@ inline void Allocator<T, AllocationScheme>::deallocate(T *chunk_data,
     if (m_shared_block && (block == *m_shared_block)) {
       // Do nothing. Keep the last block alive.
     } else {
-      m_state->block_is_not_used_anymore(block);
+      DBUG_ASSERT(m_state->number_of_blocks > 0);
+      if (block.type() == Source::RAM) {
+        MemoryMonitor::RAM::decrease(block.size());
+      } else {
+        MemoryMonitor::MMAP::decrease(block.size());
+      }
+      if (block == m_state->current_block) {
+        m_state->current_block.destroy();
+      } else {
+        block.destroy();
+      }
+      --m_state->number_of_blocks;
     }
   }
-  m_table_resource_monitor.decrease(n_bytes_requested);
 }
 
 template <class T, class AllocationScheme>
