@@ -1,16 +1,15 @@
-/* Copyright (c) 2002, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,16 +27,15 @@
 #include <algorithm>
 
 #include "errmsg.h"
+#include "m_ctype.h"
+#include "m_string.h"
 #include "my_alloc.h"
 #include "my_default.h"
 #include "my_getopt.h"
 #include "my_sys.h"
 #include "mysql/service_mysql_alloc.h"
-#include "mysql/strings/m_ctype.h"
-#include "nulls.h"
 #include "print_version.h"
 #include "sql_common.h"
-#include "strxmov.h"
 #include "welcome_copyright_notice.h" /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
 #define MAX_TEST_QUERY_LENGTH 300 /* MAX QUERY BUFFER LENGTH */
@@ -51,7 +49,7 @@ static char *opt_password = nullptr;
 static char *opt_host = nullptr;
 static char *opt_unix_socket = nullptr;
 #if defined(_WIN32)
-static char *shared_memory_base_name = nullptr;
+static char *shared_memory_base_name = 0;
 #endif
 static unsigned int opt_port;
 static bool tty_password = false;
@@ -111,10 +109,11 @@ static void print_error(MYSQL *l_mysql, const char *msg);
 static void print_st_error(MYSQL_STMT *stmt, const char *msg);
 static void client_disconnect(MYSQL *mysql);
 static void get_options(int *argc, char ***argv);
-[[noreturn]] static void die(const char *file, int line, const char *expr);
+static void die(const char *file, int line, const char *expr)
+    MY_ATTRIBUTE((noreturn));
 
 /*
-Abort unless given expression is non-zero.
+Abort unless given experssion is non-zero.
 
 SYNOPSIS
 DIE_UNLESS(expr)
@@ -142,14 +141,14 @@ static void die(const char *file, int line, const char *expr) {
 
 #define myquery(RES)      \
   {                       \
-    const int r = (RES);  \
+    int r = (RES);        \
     if (r) myerror(NULL); \
     DIE_UNLESS(r == 0);   \
   }
 
 #define myquery2(L_MYSQL, RES)      \
   {                                 \
-    const int r = (RES);            \
+    int r = (RES);                  \
     if (r) myerror2(L_MYSQL, NULL); \
     DIE_UNLESS(r == 0);             \
   }
@@ -172,16 +171,16 @@ static void die(const char *file, int line, const char *expr) {
     DIE_UNLESS(r != 0);           \
   }
 
-#define check_stmt(stmt)                   \
-  {                                        \
-    if (stmt == nullptr) myerror(nullptr); \
-    DIE_UNLESS(stmt != nullptr);           \
+#define check_stmt(stmt)          \
+  {                               \
+    if (stmt == 0) myerror(NULL); \
+    DIE_UNLESS(stmt != 0);        \
   }
 
-#define check_stmt_r(stmt)                 \
-  {                                        \
-    if (stmt == nullptr) myerror(nullptr); \
-    DIE_UNLESS(stmt == nullptr);           \
+#define check_stmt_r(stmt)        \
+  {                               \
+    if (stmt == 0) myerror(NULL); \
+    DIE_UNLESS(stmt == 0);        \
   }
 
 #define mytest(x)      \
@@ -195,28 +194,25 @@ static void die(const char *file, int line, const char *expr) {
     DIE_UNLESS(false); \
   }
 
-#define mytest2(lmysql, x)  \
-  if (!(x)) {               \
-    myerror2(lmysql, NULL); \
-    DIE_UNLESS(false);      \
-  }
-
 /* Silence unused function warnings for some of the static functions. */
-[[maybe_unused]] static int cmp_double(double *a, double *b);
-[[maybe_unused]] static void verify_col_data(const char *table, const char *col,
-                                             const char *exp_data);
-[[maybe_unused]] static void do_verify_prepare_field(
-    MYSQL_RES *result, unsigned int no, const char *name, const char *org_name,
-    enum enum_field_types type, const char *table, const char *org_table,
-    const char *db, unsigned long length, const char *file, int line);
-[[maybe_unused]] static void verify_st_affected_rows(MYSQL_STMT *stmt,
-                                                     ulonglong exp_count);
-[[maybe_unused]] static void verify_affected_rows(ulonglong exp_count);
-[[maybe_unused]] static void verify_field_count(MYSQL_RES *result,
-                                                uint exp_count);
-[[maybe_unused]] static void execute_prepare_query(const char *query,
-                                                   ulonglong exp_count);
-[[maybe_unused]] static bool thread_query(const char *query);
+static int cmp_double(double *a, double *b) MY_ATTRIBUTE((unused));
+static void verify_col_data(const char *table, const char *col,
+                            const char *exp_data) MY_ATTRIBUTE((unused));
+static void do_verify_prepare_field(MYSQL_RES *result, unsigned int no,
+                                    const char *name, const char *org_name,
+                                    enum enum_field_types type,
+                                    const char *table, const char *org_table,
+                                    const char *db, unsigned long length,
+                                    const char *def, const char *file, int line)
+    MY_ATTRIBUTE((unused));
+static void verify_st_affected_rows(MYSQL_STMT *stmt, ulonglong exp_count)
+    MY_ATTRIBUTE((unused));
+static void verify_affected_rows(ulonglong exp_count) MY_ATTRIBUTE((unused));
+static void verify_field_count(MYSQL_RES *result, uint exp_count)
+    MY_ATTRIBUTE((unused));
+static void execute_prepare_query(const char *query, ulonglong exp_count)
+    MY_ATTRIBUTE((unused));
+static bool thread_query(const char *query) MY_ATTRIBUTE((unused));
 
 /* A workaround for Sun Forte 5.6 on Solaris x86 */
 
@@ -324,10 +320,11 @@ opt_port and opt_unix_socket.
 
 @param flag           client_flag passed on to mysql_real_connect
 @param protocol       MYSQL_PROTOCOL_* to use for this connection
+@param auto_reconnect set to 1 for auto reconnect
 
 @return pointer to initialized and connected MYSQL object
 */
-static MYSQL *client_connect(ulong flag, uint protocol) {
+static MYSQL *client_connect(ulong flag, uint protocol, bool auto_reconnect) {
   MYSQL *mysql;
   int rc;
   static char query[MAX_TEST_QUERY_LENGTH];
@@ -360,6 +357,8 @@ static MYSQL *client_connect(ulong flag, uint protocol) {
     fprintf(stdout, "\n Check the connection options using --help or -?\n");
     exit(1);
   }
+  mysql->reconnect = auto_reconnect;
+
   if (!opt_silent) fprintf(stdout, "OK");
 
   /* set AUTOCOMMIT to ON*/
@@ -686,16 +685,18 @@ static void verify_col_data(const char *table, const char *col,
 /* Utility function to verify the field members */
 
 #define verify_prepare_field(result, no, name, org_name, type, table,          \
-                             org_table, db, length)                            \
+                             org_table, db, length, def)                       \
   do_verify_prepare_field((result), (no), (name), (org_name), (type), (table), \
-                          (org_table), (db), (length), __FILE__, __LINE__)
+                          (org_table), (db), (length), (def), __FILE__,        \
+                          __LINE__)
 
 static void do_verify_prepare_field(MYSQL_RES *result, unsigned int no,
                                     const char *name, const char *org_name,
                                     enum enum_field_types type,
                                     const char *table, const char *org_table,
                                     const char *db, unsigned long length,
-                                    const char *file, int line) {
+                                    const char *def, const char *file,
+                                    int line) {
   MYSQL_FIELD *field;
   CHARSET_INFO *cs;
   ulonglong expected_field_length;
@@ -727,6 +728,8 @@ static void do_verify_prepare_field(MYSQL_RES *result, unsigned int no,
             expected_field_length);
     fprintf(stdout, "\n    maxlength:`%ld`", field->max_length);
     fprintf(stdout, "\n    charsetnr:`%d`", field->charsetnr);
+    fprintf(stdout, "\n    default  :`%s`\t(expected: `%s`)",
+            field->def ? field->def : "(null)", def ? def : "(null)");
     fprintf(stdout, "\n");
   }
   DIE_UNLESS(strcmp(field->name, name) == 0);
@@ -751,7 +754,7 @@ static void do_verify_prepare_field(MYSQL_RES *result, unsigned int no,
   DIE_UNLESS(strcmp(field->db, db) == 0);
   /*
   Character set should be taken into account for multibyte encodings, such
-  as utf8mb4. Field length is calculated as number of characters * maximum
+  as utf8. Field length is calculated as number of characters * maximum
   number of bytes a character can occupy.
   */
   if (length && (field->length != expected_field_length)) {
@@ -759,6 +762,7 @@ static void do_verify_prepare_field(MYSQL_RES *result, unsigned int no,
             expected_field_length, field->length);
     DIE_UNLESS(field->length == expected_field_length);
   }
+  if (def) DIE_UNLESS(strcmp(field->def, def) == 0);
 }
 
 /* Utility function to verify the parameter count */
@@ -967,7 +971,8 @@ bool fetch_n(const char **query_list, unsigned query_count,
              enum fetch_type fetch_type) {
   unsigned open_statements = query_count;
   int rc, error_count = 0;
-  auto *fetch_array = (Stmt_fetch *)calloc(1, sizeof(Stmt_fetch) * query_count);
+  Stmt_fetch *fetch_array =
+      (Stmt_fetch *)calloc(1, sizeof(Stmt_fetch) * query_count);
   Stmt_fetch *fetch;
   DBUG_TRACE;
 
@@ -1037,6 +1042,7 @@ static bool thread_query(const char *query) {
     error = true;
     goto end;
   }
+  l_mysql->reconnect = true;
   if (mysql_query(l_mysql, query)) {
     fprintf(stderr, "Query failed (%s)\n", mysql_error(l_mysql));
     error = true;
@@ -1085,8 +1091,8 @@ static struct my_option client_test_long_options[] = {
      NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
 #if defined(_WIN32)
     {"shared-memory-base-name", 'm', "Base name of shared memory.",
-     &shared_memory_base_name, (uchar **)&shared_memory_base_name, nullptr,
-     GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &shared_memory_base_name, (uchar **)&shared_memory_base_name, 0, GET_STR,
+     REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 #endif
     {"socket", 'S', "Socket file to use for connection", &opt_unix_socket,
      &opt_unix_socket, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0,
@@ -1110,7 +1116,7 @@ static struct my_option client_test_long_options[] = {
     {nullptr, 0, nullptr, nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0,
      0, nullptr, 0, nullptr}};
 
-static void usage() {
+static void usage(void) {
   /* show the usage string when the user asks for this */
   print_version();
   puts(ORACLE_WELCOME_COPYRIGHT_NOTICE("2002"));
@@ -1125,7 +1131,7 @@ static struct my_tests_st *get_my_tests(); /* To be defined in main .c file */
 static struct my_tests_st *my_testlist = nullptr;
 
 static bool get_one_option(int optid,
-                           const struct my_option *opt [[maybe_unused]],
+                           const struct my_option *opt MY_ATTRIBUTE((unused)),
                            char *argument) {
   switch (optid) {
     case '#':
@@ -1181,6 +1187,7 @@ static void get_options(int *argc, char ***argv) {
     exit(ho_error);
 
   if (tty_password) opt_password = get_tty_password(NullS);
+  return;
 }
 
 /*
@@ -1241,8 +1248,8 @@ int main(int argc, char **argv) {
   if (mysql_server_init(0, nullptr, nullptr))
     DIE("Can't initialize MySQL server");
 
-  /* connect to server with no flags, default protocol */
-  mysql = client_connect(0, MYSQL_PROTOCOL_DEFAULT);
+  /* connect to server with no flags, default protocol, auto reconnect true */
+  mysql = client_connect(0, MYSQL_PROTOCOL_DEFAULT, true);
 
   total_time = 0;
   for (iter_count = 1; iter_count <= opt_count; iter_count++) {
