@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2009, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -42,28 +42,6 @@
 // We choose non-zero to avoid it working by coincidence.
 int Fake_TABLE::highest_table_id = 5;
 
-namespace {
-
-bool opt_use_tap = false;
-bool opt_unit_help = false;
-
-struct my_option unittest_options[] = {
-    {"tap-output", 1, "TAP (default) or gunit output.", &opt_use_tap,
-     &opt_use_tap, nullptr, GET_BOOL, OPT_ARG, opt_use_tap, 0, 1, nullptr, 0,
-     nullptr},
-    {"help", 2, "Help.", &opt_unit_help, &opt_unit_help, nullptr, GET_BOOL,
-     NO_ARG, opt_unit_help, 0, 1, nullptr, 0, nullptr},
-    {nullptr, 0, nullptr, nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0,
-     0, nullptr, 0, nullptr}};
-
-extern "C" bool get_one_option(int, const struct my_option *, char *) {
-  return false;
-}
-
-}  // namespace
-
-extern void install_tap_listener();
-
 int main(int argc, char **argv) {
   ::testing::InitGoogleTest(&argc, argv);
 
@@ -81,14 +59,6 @@ int main(int argc, char **argv) {
   ::testing::InitGoogleMock(&argc, argv);
   MY_INIT(argv[0]);
 
-  if (handle_options(&argc, &argv, unittest_options, get_one_option))
-    return EXIT_FAILURE;
-  if (opt_use_tap) install_tap_listener();
-  if (opt_unit_help)
-    printf(
-        "\n\nTest options: [--[enable-]tap-output] output TAP "
-        "rather than googletest format\n");
-
   my_testing::setup_server_for_unit_tests();
   int ret = RUN_ALL_TESTS();
   my_testing::teardown_server_for_unit_tests();
