@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,7 +36,7 @@ class GcsParametersTest : public GcsBaseTest {
  protected:
   GcsParametersTest() : m_gcs(nullptr) {}
 
-  virtual void SetUp() {
+  void SetUp() override {
     m_gcs = Gcs_xcom_interface::get_interface();
 
     // convenience alias to specialized version of Gcs_interface.
@@ -50,7 +50,7 @@ class GcsParametersTest : public GcsBaseTest {
     m_params.add_parameter("poll_spin_loops", "100");
     m_params.add_parameter("compression", "on");
     m_params.add_parameter("compression_threshold", "1024");
-    m_params.add_parameter("ip_whitelist", "127.0.0.1,192.168.1.0/24");
+    m_params.add_parameter("ip_allowlist", "127.0.0.1,192.168.1.0/24");
     m_params.add_parameter("non_member_expel_timeout", "5");
     m_params.add_parameter("suspicions_processing_period", "25");
     m_params.add_parameter("member_expel_timeout", "120");
@@ -60,7 +60,7 @@ class GcsParametersTest : public GcsBaseTest {
     m_params.add_parameter("fragmentation_threshold", "1024");
   }
 
-  virtual void TearDown() {
+  void TearDown() override {
     // fake factory cleanup member function
     static_cast<Gcs_xcom_interface *>(m_gcs)->cleanup();
   }
@@ -361,9 +361,9 @@ TEST_F(GcsParametersTest, InvalidLocalNodeAddress) {
   *p = save;
 }
 
-TEST_F(GcsParametersTest, InvalidWhitelistIPMask) {
+TEST_F(GcsParametersTest, InvalidAllowlistIPMask) {
   std::string *p =
-      const_cast<std::string *>(m_params.get_parameter("ip_whitelist"));
+      const_cast<std::string *>(m_params.get_parameter("ip_allowlist"));
   std::string save = *p;
 
   *p = "192.168.1.1/33";
@@ -371,9 +371,9 @@ TEST_F(GcsParametersTest, InvalidWhitelistIPMask) {
   *p = save;
 }
 
-TEST_F(GcsParametersTest, InvalidWhitelistIP) {
+TEST_F(GcsParametersTest, InvalidAllowlistIP) {
   std::string *p =
-      const_cast<std::string *>(m_params.get_parameter("ip_whitelist"));
+      const_cast<std::string *>(m_params.get_parameter("ip_allowlist"));
   std::string save = *p;
 
   *p = "192.168.1.256/24";
@@ -381,9 +381,9 @@ TEST_F(GcsParametersTest, InvalidWhitelistIP) {
   *p = save;
 }
 
-TEST_F(GcsParametersTest, InvalidWhitelistIPs) {
+TEST_F(GcsParametersTest, InvalidAllowlistIPs) {
   std::string *p =
-      const_cast<std::string *>(m_params.get_parameter("ip_whitelist"));
+      const_cast<std::string *>(m_params.get_parameter("ip_allowlist"));
   std::string save = *p;
 
   *p = "192.168.1.222/24,255.257.256.255";
@@ -393,7 +393,7 @@ TEST_F(GcsParametersTest, InvalidWhitelistIPs) {
 
 TEST_F(GcsParametersTest, HalfBakedIP) {
   std::string *p =
-      const_cast<std::string *>(m_params.get_parameter("ip_whitelist"));
+      const_cast<std::string *>(m_params.get_parameter("ip_allowlist"));
   std::string save = *p;
 
   *p = "192.168.";
