@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,12 +40,12 @@ class MockGcsMpscQueue : public Gcs_mpsc_queue<xcom_input_request,
   MOCK_METHOD1(push, bool(xcom_input_request *payload));
 
   MockGcsMpscQueue() { ON_CALL(*this, push(_)).WillByDefault(Return(false)); }
-  ~MockGcsMpscQueue() = default;
+  ~MockGcsMpscQueue() {}
 };
 
 class GcsXcomInputQueueTest : public GcsBaseTest {
  protected:
-  GcsXcomInputQueueTest() = default;
+  GcsXcomInputQueueTest() {}
 
   void SetUp() override {}
 
