@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,15 +23,12 @@
 #ifndef MOCK_CREATE_FIELD_H
 #define MOCK_CREATE_FIELD_H
 
-#include <assert.h>
-
-#include <optional>
-
+#include "my_dbug.h"
 #include "sql/create_field.h"
 #include "sql/dd/types/column.h"
 
 class Mock_create_field : public Create_field {
-  LEX_CSTRING m_lex_string{nullptr, 0};
+  LEX_CSTRING m_lex_string;
 
  public:
   Mock_create_field(enum_field_types field_type, Item *insert_default,
@@ -40,8 +37,8 @@ class Mock_create_field : public Create_field {
       Only TIMESTAMP is implemented for now.
       Other types would need different parameters (fld_length, etc).
     */
-    assert(field_type == MYSQL_TYPE_TIMESTAMP ||
-           field_type == MYSQL_TYPE_TIMESTAMP2);
+    DBUG_ASSERT(field_type == MYSQL_TYPE_TIMESTAMP ||
+                field_type == MYSQL_TYPE_TIMESTAMP2);
     init(nullptr,  // THD *thd
          nullptr,  // char *fld_name
          field_type,
@@ -63,7 +60,7 @@ class Mock_create_field : public Create_field {
          0,              // uint fld_geom_type
          nullptr,        // gcol info
          nullptr,        // gen default val
-         {},             // std::optional<gis::srid_t> srid
+         {},             // Nullable<gis::srid_t> srid
          dd::Column::enum_hidden_type::HT_VISIBLE);  // Visible
   }
 };
