@@ -1,16 +1,15 @@
-/* Copyright (c) 2006, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2020, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,15 +51,15 @@ class Remembering_visitor : public Select_lex_visitor {
   vector<const char *> field_names;
 
   Remembering_visitor()
-      : m_saw_query_block(false), m_saw_query_block_query_expression(false) {}
+      : m_saw_select_lex(false), m_saw_select_lex_unit(false) {}
 
-  bool visit_union(Query_expression *) override {
-    m_saw_query_block_query_expression = true;
+  bool visit_union(SELECT_LEX_UNIT *) override {
+    m_saw_select_lex_unit = true;
     return false;
   }
 
-  bool visit_query_block(Query_block *) override {
-    m_saw_query_block = true;
+  bool visit_query_block(SELECT_LEX *) override {
+    m_saw_select_lex = true;
     return false;
   }
 
@@ -73,15 +72,13 @@ class Remembering_visitor : public Select_lex_visitor {
     return false;
   }
 
-  bool saw_query_block() { return m_saw_query_block; }
-  bool saw_query_block_query_expression() {
-    return m_saw_query_block_query_expression;
-  }
+  bool saw_select_lex() { return m_saw_select_lex; }
+  bool saw_select_lex_unit() { return m_saw_select_lex_unit; }
 
-  ~Remembering_visitor() override = default;
+  ~Remembering_visitor() override {}
 
  private:
-  bool m_saw_query_block, m_saw_query_block_query_expression;
+  bool m_saw_select_lex, m_saw_select_lex_unit;
 };
 
 /**
@@ -116,13 +113,12 @@ TEST_F(SelectLexVisitorTest, SelectLex) {
   EXPECT_CALL(where, walk(_, _, _)).Times(1);
   EXPECT_CALL(having, walk(_, _, _)).Times(1);
 
-  Query_block query_block(thd()->mem_root, &where, &having);
+  SELECT_LEX query_block(thd()->mem_root, &where, &having);
 
-  Query_expression unit(CTX_NONE);
+  SELECT_LEX_UNIT unit(CTX_NONE);
 
   LEX lex;
   query_block.include_down(&lex, &unit);
-  unit.set_query_term(&query_block);
   List<Item> items;
   JOIN join(thd(), &query_block);
   join.where_cond = &where;
@@ -133,13 +129,13 @@ TEST_F(SelectLexVisitorTest, SelectLex) {
 
   Remembering_visitor visitor;
   unit.accept(&visitor);
-  EXPECT_TRUE(visitor.saw_query_block());
-  EXPECT_TRUE(visitor.saw_query_block_query_expression());
+  EXPECT_TRUE(visitor.saw_select_lex());
+  EXPECT_TRUE(visitor.saw_select_lex_unit());
 }
 
 TEST_F(SelectLexVisitorTest, InsertList) {
-  Query_block *query_block = parse("INSERT INTO t VALUES (1, 2, 3)", 0);
-  ASSERT_FALSE(query_block == nullptr);
+  SELECT_LEX *select_lex = parse("INSERT INTO t VALUES (1, 2, 3)", 0);
+  ASSERT_FALSE(select_lex == nullptr);
 
   Remembering_visitor visitor;
   thd()->lex->accept(&visitor);
@@ -150,8 +146,8 @@ TEST_F(SelectLexVisitorTest, InsertList) {
 }
 
 TEST_F(SelectLexVisitorTest, InsertList2) {
-  Query_block *query_block = parse("INSERT INTO t VALUES (1, 2), (3, 4)", 0);
-  ASSERT_FALSE(query_block == nullptr);
+  SELECT_LEX *select_lex = parse("INSERT INTO t VALUES (1, 2), (3, 4)", 0);
+  ASSERT_FALSE(select_lex == nullptr);
 
   Remembering_visitor visitor;
   thd()->lex->accept(&visitor);
@@ -163,8 +159,8 @@ TEST_F(SelectLexVisitorTest, InsertList2) {
 }
 
 TEST_F(SelectLexVisitorTest, InsertSet) {
-  Query_block *query_block = parse("INSERT INTO t SET a=1, b=2, c=3", 0);
-  ASSERT_FALSE(query_block == nullptr);
+  SELECT_LEX *select_lex = parse("INSERT INTO t SET a=1, b=2, c=3", 0);
+  ASSERT_FALSE(select_lex == nullptr);
 
   Remembering_visitor visitor;
   thd()->lex->accept(&visitor);
@@ -180,9 +176,9 @@ TEST_F(SelectLexVisitorTest, InsertSet) {
 }
 
 TEST_F(SelectLexVisitorTest, ReplaceList) {
-  Query_block *query_block =
+  SELECT_LEX *select_lex =
       parse("REPLACE INTO t(a, b, c) VALUES (1,2,3), (4,5,6)", 0);
-  ASSERT_FALSE(query_block == nullptr);
+  ASSERT_FALSE(select_lex == nullptr);
 
   Remembering_visitor visitor;
   thd()->lex->accept(&visitor);
@@ -198,9 +194,9 @@ TEST_F(SelectLexVisitorTest, ReplaceList) {
 }
 
 TEST_F(SelectLexVisitorTest, InsertOnDuplicateKey) {
-  Query_block *query_block = parse(
+  SELECT_LEX *select_lex = parse(
       "INSERT INTO t VALUES (1,2) ON DUPLICATE KEY UPDATE c= 44, a= 55", 0);
-  ASSERT_FALSE(query_block == nullptr);
+  ASSERT_FALSE(select_lex == nullptr);
 
   Remembering_visitor visitor;
   thd()->lex->accept(&visitor);
@@ -215,8 +211,8 @@ TEST_F(SelectLexVisitorTest, InsertOnDuplicateKey) {
 }
 
 TEST_F(SelectLexVisitorTest, Update) {
-  Query_block *query_block = parse("UPDATE t SET a= 0, c= 25", 0);
-  ASSERT_FALSE(query_block == nullptr);
+  SELECT_LEX *select_lex = parse("UPDATE t SET a= 0, c= 25", 0);
+  ASSERT_FALSE(select_lex == nullptr);
 
   Remembering_visitor visitor;
   thd()->lex->accept(&visitor);
