@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2020, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,9 +38,6 @@ namespace mock {
 
 class Protocol_encoder : public iface::Protocol_encoder {
  public:
-  Protocol_encoder();
-  virtual ~Protocol_encoder() override;
-
   MOCK_METHOD(bool, is_building_row, (), (const, override));
   MOCK_METHOD(bool, send_result, (const ngs::Error_code &), (override));
   MOCK_METHOD(bool, send_ok, (), (override));
