@@ -1,16 +1,15 @@
-/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -79,16 +78,7 @@ TEST_F(Socket_events_task_suite, break_loop_from_thread) {
   break_thread.join();
 }
 
-// "Socket_events::break_loop" calls "net::io_context::stop".
-// The "stop" method doesn't work when its called from active thread.
-// This behavior is not consitent with "asio::io_context::stop".
-// still the test-case is optional because X Plugin doesn't
-// calls the stop from active thread.
-//
-// The test is going to be disabled until proper behavior
-// of io_context::stop is implemnted.
-TEST_F(Socket_events_task_suite,
-       DISABLED_break_loop_from_thread_always_active) {
+TEST_F(Socket_events_task_suite, break_loop_from_thread_always_active) {
   std::atomic<uint64_t> execution_count{0};
 
   std::thread break_thread{[this, &execution_count]() {
